
namespace eval BuildingsTerrain {
    variable info_plugin ;#information read from .xml plugin file
    variable percent_done
    variable show_advance_bar 1
    variable _raster_cache

    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

proc BuildingsTerrain::ReadRaster { filename_raster show_advance_bar } {
    variable _raster_cache
    if { [info exists _raster_cache($filename_raster)] } {
        set raster $_raster_cache($filename_raster)
    } else {
        set raster [GDAL::ReadRaster $filename_raster $show_advance_bar]
        set _raster_cache($filename_raster) $raster
    }
    return $raster
}

#########################################################################################
######################################### CREATION ######################################
#########################################################################################

#also creates the group if it doesn't exists
#gid_entity_type: point line surface volume ...
proc BuildingsTerrain::AssignToGroup { group gid_entity_type entity_ids } {
    if { ![GiD_Groups exists $group] } {
        GiD_Groups create $group
    }
    GiD_EntitiesGroups assign $group ${gid_entity_type}s $entity_ids
}

#assume whole entity in zame z, use its center
proc BuildingsTerrain::GetEntityIncrementToZAbsolute { gid_entity_type entity_id z } {
    set z_current [lindex [GidUtils::GetEntityCenter $gid_entity_type $entity_id] 2]
    set z_increment [expr $z-$z_current]
    return $z_increment
}


proc BuildingsTerrain::MoveEntitiesZRelative { gid_entity_type entity_ids z_increment } {
    if { $z_increment != 0 } {
        GiD_Process Mescape Utilities Move ${gid_entity_type}s Duplicate MaintainLayers Translation FNoJoin 0,0,0 FNoJoin 0,0,$z_increment {*}$entity_ids escape escape
    }
    return 0
}

#assume all entities in same z, use z of first
proc BuildingsTerrain::MoveEntitiesZAbsolute { gid_entity_type entity_ids z } {
    set z_increment [BuildingsTerrain::GetEntityIncrementToZAbsolute $gid_entity_type [lindex $entity_ids 0] $z]
    return [BuildingsTerrain::MoveEntitiesZRelative $gid_entity_type $entity_ids $z_increment]
}

proc BuildingsTerrain::ExtrudeEntitiesZRelative { gid_entity_type entity_ids z_increment } {
    if { $gid_entity_type == "point" } {
        set extruded_entity_type Lines
    } elseif { $gid_entity_type == "line" } {
        set extruded_entity_type Surfaces
    } elseif { $gid_entity_type == "surface" } {
        set extruded_entity_type Volumes
    } else {
        error "uildingsTerrain::ExtrudeEntitiesZRelative: unexpected gid_entity_type=$gid_entity_type"
    }
    GiD_Process Mescape Utilities Copy ${gid_entity_type}s Duplicate DoExtrude $extruded_entity_type MaintainLayers Translation FNoJoin 0,0,0 FNoJoin 0,0,$z_increment {*}$entity_ids escape
    return 0
}

#copy scale
proc BuildingsTerrain::CopyEntitiesToZZero { gid_entity_type entity_ids } {
    GiD_Process MEscape Utilities Copy ${gid_entity_type}s Duplicate MaintainLayers Scale FNoJoin 0,0,0 FNoJoin 1,1,0 {*}$entity_ids escape
    return 0
}

#move scale x 0
proc BuildingsTerrain::MoveEntitiesToZZero { gid_entity_type entity_ids } {
    GiD_Process Mescape Utilities Move ${gid_entity_type}s Duplicate MaintainLayers Scale FNoJoin 0,0,0 FNoJoin 1,1,0 {*}$entity_ids escape
}

#multiple
proc BuildingsTerrain::GetZTerrainAtXYs { xys raster } {
    if { $raster == "" } {
        set z [objarray new int [objarray length $xys] 0.0]
    } else {
        set z [GiD_Raster interpolate $raster [list nodes $xys]]
    }
    return $z
}

proc BuildingsTerrain::GetZTerrainAtXY { xy raster } {
    if { $raster == "" } {
        set z 0.0
    } else {
        return [objarray get [GiD_Raster interpolate $raster [list nodes $xy]] 0]
    }
}

proc BuildingsTerrain::GetZTerrainAtBoundingBox2DCenter { bounding_box_2d raster } {
    lassign $bounding_box_2d corner_min(0) corner_min(1) corner_max(0) corner_max(1)
    set center_xy [list [expr 0.5*($corner_max(0)+$corner_min(0))] [expr 0.5*($corner_max(1)+$corner_min(1))]]
    return [BuildingsTerrain::GetZTerrainAtXY $center_xy $raster]
}

proc BuildingsTerrain::GetVolumesSurfaces { volume_ids } {
    set surface_ids [objarray new intarray 0]
    foreach volume_id $volume_ids {
        if { [GiD_Geometry exists volume $volume_id] } {
            set surface_ids [objarray concat $surface_ids [GidUtils::GetVolumeSurfaces $volume_id]]
        }
    }
    objarray sort -unique $surface_ids
    return $surface_ids
}

proc BuildingsTerrain::GetSurfacesLines { surface_ids } {
    set line_ids [objarray new intarray 0]
    foreach surface_id $surface_ids {
        if { [GiD_Geometry exists surface $surface_id] } {
            set line_ids [objarray concat $line_ids [GidUtils::GetSurfaceLines $surface_id]]
        }
    }
    objarray sort -unique $line_ids
    return $line_ids
}

proc BuildingsTerrain::GetLinesPoints { line_ids } {
    set point_ids [objarray new intarray 0]
    foreach line_id $line_ids {
        set point_ids [objarray concat $point_ids [GidUtils::GetLinePoints $line_id]]
    }
    objarray sort -unique $point_ids
    return $point_ids
}

proc BuildingsTerrain::GetEntitiesOfGroups { groups gid_entity_type } {
    set entity_ids [objarray new intarray 0]
    foreach group $groups {
        if { [GiD_Groups exists $group] } {
            set entity_ids [objarray concat $entity_ids [GiD_EntitiesGroups get $group $gid_entity_type]]
        }
    }
    return $entity_ids
}

proc BuildingsTerrain::GetFilenameRaster { } {
    variable filename_raster
    GidUtils::SetWarnLine  [_ "Enter name of the raster file that define the topography"]
    set types [list [list "Raster" {.asc .txt .tif}] [list [_ "All files"] ".*"]]
    set default_extension .asc
    set filename_raster [MessageBoxGetFilename file read "Raster topography" "" $types $default_extension 0 [list]]
    return $filename_raster
}


#return 1 if bounding_box_2d_test is completelly inside bounding_box_2d (with a tolernace)
proc BuildingsTerrain::IsBoundinbox2DInside { bounding_box_2d bounding_box_2d_test {tolerance 0.0} } {
    lassign $bounding_box_2d corner_min(0) corner_min(1) corner_max(0) corner_max(1)
    lassign $bounding_box_2d_test corner_min_test(0) corner_min_test(1) corner_max_test(0) corner_max_test(1)
    foreach i_dim { 0 1 } {
        if { $corner_min($i_dim)>[expr $corner_min_test($i_dim)+$tolerance] } {
            return 0
        }
        if { $corner_max($i_dim)<[expr $corner_max_test($i_dim)-$tolerance] } {
            return 0
        }
    }
    return 1
}

#set bbox_2d [list $xmin $ymin $zmin $xmax $ymax $zmax]
proc BuildingsTerrain::ListSurfacesInsideBBox2D { bounding_box_2d {surface_ids ""} } {
    set fail 0
    if { [llength $surface_ids] == 0 } {
        #select all
        set surface_ids [GiD_Geometry list surface]
    }
    set inside_surface_ids [list]
    foreach surface_id $surface_ids {
        lassign [GiD_Info bounding_box surface $surface_id] x_min x_max y_min y_max z_min z_max
        set bounding_box_2d_test [list $x_min $y_min $x_max $y_max] ;#ordered lower-left corner -> upper-right corner
        if { [BuildingsTerrain::IsBoundinbox2DInside $bounding_box_2d $bounding_box_2d_test] } {
            lappend inside_surface_ids $surface_id
        }
    }
    return [objarray new intarray -values $inside_surface_ids]
}

proc BuildingsTerrain::GetSurfaceHeightsFromCondition { condition_name condition_question_height } {
    set surface_heights [list]
    #do not raise error if the condition doesn't exists, return empty list
    if { [lsearch -exact [GiD_Info conditions ovsurf] $condition_name] != -1 } {
        set condition_fields_defaults [lrange [GiD_Info conditions $condition_name] 2 end]
        set pos [lsearch $condition_fields_defaults $condition_question_height ]
        if { $pos == -1 } {
            error "question $condition_question_height not found in condition $condition_name"
        }
        set z_item_pos [expr 3+$pos/2]
        foreach item [GiD_Info conditions $condition_name geometry] {
            lappend surface_heights [lindex $item 1] [lindex $item $z_item_pos]
        }
        BuildingsTerrain::DeleteCondition $condition_name
        GiDMenu::UpdateMenus
    }
    return $surface_heights
}

proc BuildingsTerrain::DeleteCondition { condition_name } {
    GiD_CreateData delete condition $condition_name
}

proc BuildingsTerrain::GetSurfaceHeightsFromZ { surface_ids } {
    set surface_heights [list]
    foreach surface_id $surface_ids {
        set surface_center [GidUtils::GetEntityCenter surface $surface_id]
        lappend surface_heights $surface_id [lindex $surface_center 2]
    }
    return $surface_heights
}

proc BuildingsTerrain::MoveSurfacesZAbsolute { surface_ids surface_heights raster } {
    set fail 0
    array set building_height $surface_heights
    foreach surface_id $surface_ids {
        set height $building_height($surface_id)
        if { $raster == "" } {
            set z_terrain 0.0
        } else {
            set surface_center [GidUtils::GetEntityCenter surface $surface_id]
            set z_terrain [BuildingsTerrain::GetZTerrainAtXY [lrange $surface_center 0 1] $raster]
        }
        set z_absolute [expr $z_terrain+$height]
        BuildingsTerrain::MoveEntitiesZAbsolute surface [list $surface_id] $z_absolute
    }
    return $fail
}

proc BuildingsTerrain::RemoveHolesSurface { surface_id delete_lines } {
    if { [GiD_Geometry -v2 get surface $surface_id has_holes] } {
        set index_end_outer_lines [expr [objarray get [GiD_Geometry -v2 get surface $surface_id index_boundaries] 1]-1]
        set surface_data [GiD_Geometry -v2 get surface $surface_id]
        #remove inner lines
        if { $delete_lines } {
            set inner_line_ids [objarray range [lindex $surface_data 3] [expr $index_end_outer_lines+1] end]
        }
        lset surface_data 3 [objarray range [lindex $surface_data 3] 0 $index_end_outer_lines]
        lset surface_data 4 [objarray range [lindex $surface_data 4] 0 $index_end_outer_lines]
        #the surface must not have higherentities
        GiD_Geometry -v2 delete surface [list $surface_id]
        GiD_Geometry -v2 create surface $surface_id {*}$surface_data
        if { $delete_lines } {
            GiD_Geometry -v2  delete -also_lower_entities line $inner_line_ids
        }
    }
}

proc BuildingsTerrain::RemoveHolesSurfaces { surface_ids delete_lines } {
    foreach surface_id $surface_ids {
        BuildingsTerrain::RemoveHolesSurface $surface_id $delete_lines
    }
}

proc BuildingsTerrain::DoRemoveHolesSurfaces { } {
    variable llx
    variable lly
    variable urx
    variable ury
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        error [_ "must set the lower-left and upper-right 2D corners of the region of interest"]
    }
    set bounding_box_2d [list $llx $lly $urx $ury]
    set fail 0
    set delete_lines 1
    BuildingsTerrain::SetMessage [_ "Removing surface's holes"]...
    GidUtils::DisableGraphics
    set surface_ids [BuildingsTerrain::ListSurfacesInsideBBox2D $bounding_box_2d]
    BuildingsTerrain::RemoveHolesSurfaces $surface_ids $delete_lines
    GidUtils::EnableGraphics
    BuildingsTerrain::ZoomFrame
    BuildingsTerrain::SetMessage [_ "Done"]
    return $fail
}


proc BuildingsTerrain::AddHolesSurface { surface_id holing_surfaces } {
    GiD_Process Mescape Geometry Edit HoleNurb WithSurfaces $surface_id {*}$holing_surfaces escape escape
    return 0
}

#holes_flag:
#  outer_boundary :to take into account only the lines of the outer loop
#  inner_boundaries :to take into account only the lines of the inner loops (holes)
#  all_boundaries : to take into account both cases
proc BuildingsTerrain::ExtrudeLinesOfSurfacesZRelative { surface_ids surface_heights holes_flag {callback_percent ""} } {
    set fail 0
    array set building_height $surface_heights
    set n [llength $surface_ids]
    set nrefresh [expr {int($n/5.0)}]
    if { $nrefresh<1 } {
        set nrefresh 1
    }
    set i 0
    foreach surface_id $surface_ids {
        if { $callback_percent != "" && ![expr $i%$nrefresh] } {
            set percent [expr int($i/($n-1.0)*100.0)]
            set stop [$callback_percent $percent]
            if { $stop } {
                set fail -1
                break
            }
        }
        set surface_lines [GidUtils::GetSurfaceLines $surface_id]
        if { $holes_flag == "outer_boundary" } {
            if { [GiD_Geometry -v2 get surface $surface_id has_holes] } {
                set surface_index_boundaries [GiD_Geometry -v2 get surface $surface_id index_boundaries]
                set index_hole [objarray get $surface_index_boundaries 1]
                set surface_lines [objarray range $surface_lines 0 [expr $index_hole-1]]
            }
        } elseif { $holes_flag == "inner_boundaries" } {
            if { [GiD_Geometry -v2 get surface $surface_id has_holes] } {
                set surface_index_boundaries [GiD_Geometry -v2 get surface $surface_id index_boundaries]
                set index_hole [objarray get $surface_index_boundaries 1]
                set surface_lines [objarray range $surface_lines $index_hole end]
            }
        } else {
            #do nothing special with outer and inner boundaries, consider all
        }

        BuildingsTerrain::ExtrudeEntitiesZRelative line $surface_lines -$building_height($surface_id)
        incr i
    }
    return $fail
}

proc BuildingsTerrain::CreateSurfaceFromBoundingBox { bounding_box_2d z } {
    lassign $bounding_box_2d corner_min(0) corner_min(1) corner_max(0) corner_max(1)
    set layer [GiD_Info Project LayerToUse]
    set max_id_point_before [GiD_Info geometry MaxNumPoints]
    set max_id_line_before [GiD_Info geometry MaxNumLines]
    set max_id_surface_before [GiD_Info geometry MaxNumSurfaces]
    set ipoint $max_id_point_before
    GiD_Geometry create point [incr ipoint] $layer $corner_min(0) $corner_min(1) $z
    GiD_Geometry create point [incr ipoint] $layer $corner_max(0) $corner_min(1) $z
    GiD_Geometry create point [incr ipoint] $layer $corner_max(0) $corner_max(1) $z
    GiD_Geometry create point [incr ipoint] $layer $corner_min(0) $corner_max(1) $z
    set iline $max_id_line_before
    GiD_Geometry create line [incr iline] stline $layer [expr $max_id_point_before+1] [expr $max_id_point_before+2]
    GiD_Geometry create line [incr iline] stline $layer [expr $max_id_point_before+2] [expr $max_id_point_before+3]
    GiD_Geometry create line [incr iline] stline $layer [expr $max_id_point_before+3] [expr $max_id_point_before+4]
    GiD_Geometry create line [incr iline] stline $layer [expr $max_id_point_before+4] [expr $max_id_point_before+1]
    set isurface $max_id_surface_before
    #set lines_and_versos [list [list [expr $max_id_line_before+1] 0] [list [expr $max_id_line_before+2] 0] [list [expr $max_id_line_before+3] 0] [list [expr $max_id_line_before+4] 0]]
    #GiD_Geometry create surface [incr isurface] coonsurface $layer [llength $lines_and_versos] {*}$lines_and_versos
    set lines [list [expr $max_id_line_before+1] [expr $max_id_line_before+2] [expr $max_id_line_before+3] [expr $max_id_line_before+4]]
    set versos [list 0 0 0 0]
    set degree_u 1
    set degree_v 1
    set num_points_u 2
    set num_points_v 2
    set is_trimmed 0
    set control_points [list $corner_min(0) $corner_min(1) $z $corner_max(0) $corner_min(1) $z $corner_min(0) $corner_max(1) $z $corner_max(0) $corner_max(1) $z]
    set weights [list]
    set knots_u [list 0.0 0.0 1.0 1.0]
    set knots_v [list 0.0 0.0 1.0 1.0]
    set geometrical_data [list $degree_u $degree_v $num_points_u $num_points_v $control_points $weights $knots_u $knots_v]
    GiD_Geometry -v2 create surface [incr isurface] nurbssurface $layer $is_trimmed $lines $versos $geometrical_data
    return $isurface
}

#it must be better if can mesh without GiD_Process, without modify any meshing variable and the current model,
#with some gid_mesh_library function wrapped from Tcl scripting ...
#surface_mesher: 0==rfast, 6==minelem
#mesh_variables: list of valid GiD variable name and value
#e.g. { SurfaceMesher 0 AutomaticCorrectSizes 0 SizeTransitionsFactor 1.0 }
# boundary_one_division: 0 or 1
proc BuildingsTerrain::GenerateMeshFromSurface { surface_id size mesh_variables boundary_one_division } {
    set fail 0
    if { ![string is integer -strict $surface_id] } {
        error "BuildingsTerrain::GenerateMeshFromSurface.wong surface id=$surface_id"
        set fail 1
        return $fail
    }
    set max_element_id_before [GiD_Info Mesh MaxNumElements]
    foreach {key value} $mesh_variables {
        set old($key) [GiD_Set $key]
        GiD_Set $key $value
    }
    GiD_Process Mescape Meshing MeshCriteria NoMesh Surfaces 1:end escape
    GiD_Process Mescape Meshing MeshCriteria Mesh Surfaces $surface_id escape
    if { $boundary_one_division } {
        set line_ids [GidUtils::GetSurfaceLines $surface_id]
        GiD_Process Mescape Meshing Structured Lines 1 {*}$line_ids escape escape escape
    }
    GiD_Process Mescape Meshing Generate Yes $size escape
    GiD_Process Mescape Meshing Reset Yes escape
    foreach {key value} $mesh_variables {
        GiD_Set $key $old($key)
    }
    if { $max_element_id_before == [GiD_Info Mesh MaxNumElements] } {
        set fail 1
    }
    return $fail
}

proc BuildingsTerrain::GenerateMeshFromSurfaceTrySeveralMethods { surface_id } {
    #get an automatic mesh size
    lassign [GiD_Info bounding_box surface $surface_id] x0 x1 y0 y1 z0 z1
    set diagonal [MathUtils::VectorDistance [list $x0 $y0] [list $x1 $y1]]
    set size [expr $diagonal/20.0]

    package require triangle
    set fail [Triangle::GenerateMeshFromSurface $surface_id $size {constrained 0 quality 5.0}]
    if { $fail } {
        set fail 1
        set boundary_one_division 1
        #SurfaceMesher 0== rfast, 6==minelem
        foreach mesh_variables {{SurfaceMesher 0 AutomaticCorrectSizes 0 SizeTransitionsFactor 1.0} {SurfaceMesher 6 AutomaticCorrectSizes 0 SizeTransitionsFactor 1.0}} {
            #BuildingsTerrain::SetMessage [_ "Meshing soil (%s)" $mesh_variables]...
            set fail [BuildingsTerrain::GenerateMeshFromSurface $surface_id $size $mesh_variables $boundary_one_division]
            if { !$fail} {
                break
            }
        }
    }
    return $fail
}

proc BuildingsTerrain::CreateGeometryFromMesh { element_from_id element_to_id } {
    GiD_Process Mescape Geometry Edit ReConstruction OneSurfForEachElement selection $element_from_id:$element_to_id escape
    #GiD_Process Mescape Geometry Create SurfMesh SelectMesh selection $element_from_id:$element_to_id escape
    return 0
}

proc BuildingsTerrain::MoveGeometryPointsToZTerrain { raster point_ids } {
    set old(CreateAlwaysNewPoint) [GiD_Set CreateAlwaysNewPoint]
    GiD_Set CreateAlwaysNewPoint 1
    foreach point_id $point_ids {
        lassign [GiD_Geometry -v2 get point $point_id] layer x y z
        set z [BuildingsTerrain::GetZTerrainAtXY [list $x $y] $raster]
        GiD_Process Mescape Geometry Edit MovePoint $point_id FNoJoin $x,$y,$z escape
    }
    GiD_Set CreateAlwaysNewPoint $old(CreateAlwaysNewPoint)
}

proc BuildingsTerrain::CollapseEntities { gid_entity_type entity_ids  } {
    set join_tangent_lines 0
    set old(CollapseAllowMoreTasks) [GiD_Set CollapseAllowMoreTasks]
    if { $join_tangent_lines } {
        GiD_Set CollapseAllowMoreTasks 1
    } else {
        GiD_Set CollapseAllowMoreTasks 0
    }
    set old(AutoImportTolerance) [GiD_Set AutoImportTolerance]
    GiD_Process Mescape Utilities Collapse ${gid_entity_type}s {*}$entity_ids escape
    GiD_Set AutoImportTolerance $old(AutoImportTolerance)
    GiD_Set CollapseAllowMoreTasks $old(CollapseAllowMoreTasks)
}

proc BuildingsTerrain::CreateVolume { volume_surfaces } {
    GiD_Process Mescape Geometry Create Volume {*}$volume_surfaces escape escape
}

proc BuildingsTerrain::ZoomFrame { } {
    GiD_Process 'Zoom Frame
}

proc BuildingsTerrain::CreateRasterZConstant { bounding_box_2d z_constant } {
    lassign $bounding_box_2d corner_min(0) corner_min(1) corner_max(0) corner_max(1)
    set xcellsize [expr $corner_max(0)-$corner_min(0)]
    set ycellsize [expr $corner_max(1)-$corner_min(1)]
    set raster [list 2 2 $corner_min(0) $corner_min(1) $xcellsize $$ycellsize -9999 [objarray new doublearray 4 $z_constant]]
    return $raster
}

############################################################################################################################
# BuildingsTerrain::DoVolumeBodyFitted
############################################################################################################################
# start from 2D surfaces located with z terrain (or heigths provided as argument), and a raster file, else assume z terrain=0.0
# a selection 2D rectangle (else the bounding box of surfaces and/or raster could be used), and a box height (relative to floor)
# objective: body-fitted mesh of tetrahedra for CFD classical simulation
# select only the surfaces completely inside a rectangle and create a single volume of the air around the surfaces
# extruding lateral surfaces until z terrain, and fitting the terrain with unstructured triangles
# limitation: the surfaces of the buildings must not be overlapped or too closed between
# them (te be able to create the 2D floor surface or to mesh it or the volume)
#############################################################################################################################

#surface_heights: list of surface_id surface_height ... (surface_id could be implicit 1...n?)
#if is empty it is expected that the surface z implicitly means the building height
proc BuildingsTerrain::VolumeBodyFitted { bounding_box_2d box_height filename_raster surface_heights } {
    variable show_advance_bar
    set remove_holes 1 ;#to avoid more complications and have less entities
    set fail 0
    BuildingsTerrain::SetMessage [_ "Reading raster"]...
    if { $filename_raster == "" } {
        set raster [BuildingsTerrain::CreateRasterZConstant $bounding_box_2d 0.0]
    } else {
        set raster [BuildingsTerrain::ReadRaster $filename_raster $show_advance_bar]
    }

    BuildingsTerrain::SetMessage [_ "Creating geometry"]...
    GidUtils::DisableGraphics
    set inside_surface_ids [BuildingsTerrain::ListSurfacesInsideBBox2D $bounding_box_2d]
    BuildingsTerrain::DeleteAllButSurfaces $inside_surface_ids
    if { $remove_holes } {
        BuildingsTerrain::RemoveHolesSurfaces $inside_surface_ids 1
    }
    BuildingsTerrain::AssignToGroup "buildings_top" surface $inside_surface_ids
    BuildingsTerrain::SetMessage [_ "Placing top surfaces and building laterals"]...
    if { ![llength $surface_heights] } {
        #if surface_heights is empty it is assumed that the surfaces are already located with z=building height (relative to ground)
        set surface_heights [BuildingsTerrain::GetSurfaceHeightsFromZ $inside_surface_ids]
    }
    BuildingsTerrain::MoveSurfacesZAbsolute $inside_surface_ids $surface_heights $raster
    set max_id_line_before_extrude_building_tops [GiD_Info geometry MaxNumLines]
    set max_id_surface_before_extrude_building_tops [GiD_Info geometry MaxNumSurfaces]
    BuildingsTerrain::ExtrudeLinesOfSurfacesZRelative $inside_surface_ids $surface_heights outer_boundary ""
    if { !$remove_holes } {
        set max_id_line_before_extrude_building_top_holes [GiD_Info geometry MaxNumLines]
        set max_id_surface_before_extrude_building_top_holes [GiD_Info geometry MaxNumSurfaces]
        BuildingsTerrain::ExtrudeLinesOfSurfacesZRelative $inside_surface_ids $surface_heights inner_boundaries ""
        set soil_hole_line_ids [GiD_Geometry list -higherentity 1 line [expr $max_id_line_before_extrude_building_top_holes+1]:end]
        #but it is necessary to create at soil heigth one surface by each hole, to be able to close a volume...
    }
    BuildingsTerrain::AssignToGroup "buildings_lateral" surface [GiD_Geometry list surface [expr $max_id_surface_before_extrude_building_tops+1]:end]
    set z_terrain [BuildingsTerrain::GetZTerrainAtBoundingBox2DCenter $bounding_box_2d $raster]
    set top_surface_id [BuildingsTerrain::CreateSurfaceFromBoundingBox $bounding_box_2d [expr $z_terrain+$box_height]]
    BuildingsTerrain::AssignToGroup "box_top" surface [list $top_surface_id]
    set max_id_surface_before_extrude_top [GiD_Info geometry MaxNumSurfaces]
    BuildingsTerrain::ExtrudeEntitiesZRelative line [GidUtils::GetSurfaceLines $top_surface_id] -$box_height
    set surfaces_lateral [GiD_Geometry list surface [expr $max_id_surface_before_extrude_top+1]:end]
    BuildingsTerrain::AssignToGroup "box_lateral_x_min" surface [GidUtils::GetSurfacesFilterNormal $surfaces_lateral {1.0 0.0 0.0} 5.0]
    BuildingsTerrain::AssignToGroup "box_lateral_x_max" surface [GidUtils::GetSurfacesFilterNormal $surfaces_lateral {-1.0 0.0 0.0} 5.0]
    BuildingsTerrain::AssignToGroup "box_lateral_y_min" surface [GidUtils::GetSurfacesFilterNormal $surfaces_lateral {0.0 1.0 0.0} 5.0]
    BuildingsTerrain::AssignToGroup "box_lateral_y_max" surface [GidUtils::GetSurfacesFilterNormal $surfaces_lateral {0.0 -1.0 0.0} 5.0]
    BuildingsTerrain::SetMessage [_ "Creating soil"]...
    set soil_boundary_line_ids [GiD_Geometry list -higherentity 1 line [expr $max_id_line_before_extrude_building_tops+1]:end]
    if { $filename_raster == "" } {
        set bottom_surface_id [GidUtils::CreateSurfaceFromBoundaryLines $soil_boundary_line_ids]
        BuildingsTerrain::AssignToGroup "soil" surface [list $bottom_surface_id]
    } else {
        set max_id_line_before_copy_lines [GiD_Info geometry MaxNumLines]
        BuildingsTerrain::CopyEntitiesToZZero line $soil_boundary_line_ids
        set lines_copied [GiD_Geometry list line [expr $max_id_line_before_copy_lines+1]:end]
        set bottom_surface_id [GidUtils::CreateSurfaceFromBoundaryLines $lines_copied]
        if { $bottom_surface_id == "" } {
            set fail 1
            BuildingsTerrain::SetMessage [_ "Error: soil surface not created"]
        } else {
            set max_node_id_before [GiD_Info Mesh MaxNumNodes]
            set max_element_id_before [GiD_Info Mesh MaxNumElements]
            set fail_mesh [BuildingsTerrain::GenerateMeshFromSurfaceTrySeveralMethods $bottom_surface_id]
            if { $fail_mesh } {
                set fail 1
                BuildingsTerrain::SetMessage [_ "Error: auxiliary mesh not created"]
            } else {
                GiD_Geometry -v2 delete -also_lower_entities surface [list $bottom_surface_id]
                GIS::MoveNodesToZRaster $raster [expr $max_node_id_before+1]:[GiD_Info Mesh MaxNumNodes]
                set max_id_surface_before_create_soil [GiD_Info geometry MaxNumSurfaces]
                BuildingsTerrain::CreateGeometryFromMesh [expr $max_element_id_before+1] end
                GiD_Mesh delete element [GiD_Mesh list element [expr $max_node_id_before+1]:end]
                BuildingsTerrain::AssignToGroup "soil" surface [GiD_Geometry list surface [expr $max_id_surface_before_create_soil+1]:end]
                set soil_boundary_point_ids [BuildingsTerrain::GetLinesPoints $soil_boundary_line_ids]
                BuildingsTerrain::MoveGeometryPointsToZTerrain $raster $soil_boundary_point_ids
                set border_line_ids [GiD_Geometry list -higherentity 1 line [expr $max_id_line_before_extrude_building_tops+1]:end]
                BuildingsTerrain::CollapseEntities line $border_line_ids
            }
        }
    }
    set num_surfaces_soil [llength [BuildingsTerrain::GetEntitiesOfGroups [list soil] surface]]
    if { $num_surfaces_soil == 0 } {
        BuildingsTerrain::SetMessage [_ "Error: soil surface not created"]
    } else {
        BuildingsTerrain::SetMessage [_ "Creating volume"]...
        set max_id_volume_before [GiD_Info geometry MaxNumVolumes]
        set volume_surfaces [BuildingsTerrain::GetEntitiesOfGroups [list buildings_top buildings_lateral box_top box_lateral_x_min box_lateral_x_max box_lateral_y_min box_lateral_y_max soil] surface]
        BuildingsTerrain::CreateVolume $volume_surfaces
        set max_id_volume_after [GiD_Info geometry MaxNumVolumes]
        set num_volumes [expr $max_id_volume_after-$max_id_volume_before]
        if { $num_volumes > 0 } {
            BuildingsTerrain::AssignToGroup "air" volume [list $max_id_volume_after]
            BuildingsTerrain::SetMessage [_ "Done"]
        } else {
            BuildingsTerrain::SetMessage [_ "Error: air volume not created"]
        }
    }
    GidUtils::EnableGraphics
    GeometryView
    BuildingsTerrain::ZoomFrame
    Groups::GroupsToLayers
    GidUtils::UpdateWindow GROUPS
    BuildingsTerrain::AssignProblemtypeData
    return $fail
}

proc BuildingsTerrain::GetKratosActiveApplication { } {
    return [apps::getActiveAppId]
}

proc BuildingsTerrain::AssignProblemtypeData { } {
    if { [GiD_Info project Problemtype] == "kratos" } {
        set kratos_active_application [BuildingsTerrain::GetKratosActiveApplication]
        if { ($kratos_active_application == "Fluid" || $kratos_active_application == "EmbeddedFluid") && $::Model::SpatialDimension == "3D" } {
            BuildingsTerrain::AssignProblemtypeDataKratosFluid3D
        }
    } elseif { [GiD_Info project Problemtype] == "IBER" } {
        BuildingsTerrain::AssignProblemtypeDataIBER
    }
}

#set a wind of 1 m/s in x direction
proc BuildingsTerrain::AssignProblemtypeDataKratosFluid3D { } {
    #assign data for Fluid 3D application
    set kratos_active_application [BuildingsTerrain::GetKratosActiveApplication]
    if { ($kratos_active_application != "Fluid" && $kratos_active_application != "EmbeddedFluid") || $::Model::SpatialDimension != "3D" } {
        W "BuildingsTerrain::AssignProblemtypeDataKratosFluid3D. Unexpected kratos application $kratos_active_application, or dimension $::Model::SpatialDimension"
        return 1
    }


    set root [customlib::GetBaseRoot]
    set condtype surface
    # Monolithic solution strategy set
    spdAux::SetValueOnTreeItem v "Monolithic" FLSolStrat

    # Fluid Parts
    set group_name air
    set fluidParts [spdAux::getRoute "FLParts"]
    set fluidNode [spdAux::AddConditionGroupOnXPath $fluidParts $group_name]
    #Material Air

    if { [BuildingsTerrain::GetKratosActiveApplication] == "Fluid" } {
        set props [list Element Monolithic3D ConstitutiveLaw Newtonian DENSITY 1.0 DYNAMIC_VISCOSITY 1e-6]
    } elseif { [BuildingsTerrain::GetKratosActiveApplication] == "EmbeddedFluid" } {
        set props [list Element Monolithic3D ConstitutiveLaw Newtonian DENSITY 1.0 DYNAMIC_VISCOSITY 1e-6]
    } else {
        #unexpected case
        set props [list]
    }
    foreach {prop val} $props {
        set propnode [$fluidNode selectNodes "./value\[@n = '$prop'\]"]
        if {$propnode ne "" } {
            $propnode setAttribute v $val
        } else {
            W "Warning - Couldn't find property Fluid $prop"
        }
    }
    set fluidConditions [spdAux::getRoute "FLBC"]
    # Fluid Inlet
    set group_name box_lateral_x_min
    set fluidInlet "$fluidConditions/condition\[@n='AutomaticInlet3D'\]"
    set inlets [list Total 0 End 1.0]
    #Erase Previous Intervals (excepts some specials)
    set interval_base [spdAux::getRoute "Intervals"]
    foreach int [$root selectNodes "$interval_base/blockdata\[@n='Interval'\]"] {
        if {[$int @name] ni [list Initial Total Custom1]} {$int delete}
    }
    foreach {inlet_name ini end modulus} $inlets {
        if { [lsearch {Initial Total Custom1} $inlet_name] == -1 } {
            spdAux::CreateInterval $inlet_name $ini $end
        }
        if { ![GiD_Groups exists "$group_name//$inlet_name"] } {
            GiD_Groups create "$group_name//$inlet_name"
        }
        spdAux::AddIntervalGroup $group_name "$group_name//$inlet_name"
        set inletNode [spdAux::AddConditionGroupOnXPath $fluidInlet "$group_name//$inlet_name"]
        $inletNode setAttribute ov $condtype
        set props [list ByFunction No modulus $modulus direction automatic_inwards_normal Interval $inlet_name]
        foreach {prop val} $props {
            set propnode [$inletNode selectNodes "./value\[@n = '$prop'\]"]
            if {$propnode ne "" } {
                $propnode setAttribute v $val
            } else {
                W "Warning - Couldn't find property Inlet $prop"
            }
        }
    }

    # Fluid Outlet
    set group_name box_lateral_x_max
    set fluidOutlet "$fluidConditions/condition\[@n='Outlet3D'\]"
    set outletNode [spdAux::AddConditionGroupOnXPath $fluidOutlet $group_name]
    $outletNode setAttribute ov $condtype
    set props [list value 0.0]
    foreach {prop val} $props {
        set propnode [$outletNode selectNodes "./value\[@n = '$prop'\]"]
        if {$propnode ne "" } {
            $propnode setAttribute v $val
        } else {
            W "Warning - Couldn't find property Outlet $prop"
        }
    }
    # Fluid Conditions
    #No Slip
    if { [BuildingsTerrain::GetKratosActiveApplication] == "Fluid" } {
        set group_names [list buildings_lateral buildings_top soil]
    } elseif { [BuildingsTerrain::GetKratosActiveApplication] == "EmbeddedFluid" } {
        set group_names [list soil]
    } else {
        #unexpected case
        set group_names [list]
    }

    foreach group_name $group_names {
        [spdAux::AddConditionGroupOnXPath "$fluidConditions/condition\[@n='NoSlip3D'\]" $group_name] setAttribute ov $condtype
    }
    #Slip
    set group_name box_lateral_and_top
    if { ![GiD_Groups exists $group_name] } {
        #seems that for a limitation of current Kratos (smoothed normal on edges) is necessary set Slip3D in a single group, instead of several times
        #then create an auxilary group
        BuildingsTerrain::AssignToGroup $group_name surface [BuildingsTerrain::GetEntitiesOfGroups [list box_lateral_y_min box_lateral_y_max box_top] surface]
    }
    [spdAux::AddConditionGroupOnXPath "$fluidConditions/condition\[@n='Slip3D'\]" $group_name] setAttribute ov $condtype

    # Time parameters
    set time_parameters [list EndTime 50 AutomaticDeltaTime No DeltaTime 1.0]
    set time_params_path [spdAux::getRoute "FLTimeParameters"]
    foreach {n v} $time_parameters {
        [$root selectNodes "$time_params_path/value\[@n='$n'\]"] setAttribute v $v
    }
    # Output
    set time_parameters [list OutputControlType step OutputDeltaStep 1]
    set time_params_path [spdAux::getRoute "Results"]
    foreach {n v} $time_parameters {
        [$root selectNodes "$time_params_path/value\[@n='$n'\]"] setAttribute v $v
    }
    # Parallelism
    set time_parameters [list ParallelSolutionType OpenMP OpenMPNumberOfThreads 8]
    set time_params_path [spdAux::getRoute "Parallelization"]
    foreach {n v} $time_parameters {
        [$root selectNodes "$time_params_path/value\[@n='$n'\]"] setAttribute v $v
    }
    spdAux::RequestRefresh
}

proc BuildingsTerrain::SetBoxCorners { min_x min_y max_x max_y } {
    variable llx
    variable lly
    variable urx
    variable ury
    if { $min_x > $max_x } {
        set tmp $min_x
        set min_x $max_x
        set max_x $tmp
    }
    if { $min_y > $max_y } {
        set tmp $min_y
        set min_y $max_y
        set max_y $tmp
    }
    lassign [list $min_x $min_y $max_x $max_y] llx lly urx ury
}

proc BuildingsTerrain::SelectBox { } {
    BuildingsTerrain::UsurpateCursorAndGiD_Event_DynamicBox
    GidUtils::SetWarnLine [_ "Enter first corner of the box to be selected"]
    set ::zoom_box_state(in) 1 ;#variable to indicate the action to be done on mouse events (open a selection rectangle)  set ::zoom_box_state(in) 1 ;#variable to indicate the action to be done on mouse events (open a selection rectangle)              set ::(in) 1
}

proc BuildingsTerrain::UsurpateCursorAndGiD_Event_DynamicBox { } {
    variable gid_event_dynamicbox_prev
    if { [info procs ::GiD_Event_DynamicBox] != "" } {
        set gid_event_dynamicbox_prev [info body ::GiD_Event_DynamicBox]
    }
    proc ::GiD_Event_DynamicBox { w x y type } {
        BuildingsTerrain::GiD_Event_DynamicBox $w $x $y $type
    }
    variable cursor_prev
    set cursor_prev [gid_client_get_cursor]
    gid_client_set_cursor CREATE_POINT
}

proc BuildingsTerrain::RestoreCursorAndGiD_Event_DynamicBox { } {
    variable gid_event_dynamicbox_prev
    if { [info exists gid_event_dynamicbox_prev] } {
        proc ::GiD_Event_DynamicBox { w x y type } $gid_event_dynamicbox_prev
        unset gid_event_dynamicbox_prev
    }
    variable cursor_prev
    if { [info exists cursor_prev] } {
        gid_client_set_cursor $cursor_prev
        unset cursor_prev
    }
}

proc BuildingsTerrain::GiD_Event_DynamicBox { w x y type } {
    variable dynamibox_begin ;#to store temporary the first corner until the second corner is clicked
    if { $type == "BEGIN" } {
        set BuildingsTerrain::dynamibox_begin($w) [list $x $y]
        GidUtils::SetWarnLine [_ "Enter second corner of the box to be selected"]
    } elseif { $type == "END" } {
        if { [info exists BuildingsTerrain::dynamibox_begin($w)] } {
            BuildingsTerrain::RestoreCursorAndGiD_Event_DynamicBox
            lassign $BuildingsTerrain::dynamibox_begin($w) screen_min_x screen_min_y
            unset BuildingsTerrain::dynamibox_begin($w)
            lassign [list $x $y] screen_max_x screen_max_y
            lassign [lrange [GiD_GetWorldCoord $screen_min_x $screen_min_y] 0 2] min_x min_y
            lassign [lrange [GiD_GetWorldCoord $screen_max_x $screen_max_y] 0 2] max_x max_y
            BuildingsTerrain::SetBoxCorners $min_x $min_y $max_x $max_y
            GidUtils::SetWarnLine [_ "Selection box set"]
        }
    }
}

proc BuildingsTerrain::SetMessage { message } {
    set w .gid.buildings
    set fmessages $w.fmessages
    if { [winfo exists $fmessages] } {
        $fmessages.message configure -text $message
        update
    }
}

proc BuildingsTerrain::EnableStop { } {
    variable stop
    set stop 0
    .gid.buildings.fmessages.stop configure -state enable
}

proc BuildingsTerrain::PressStop { } {
    variable stop
    set stop 1
    .gid.buildings.fmessages.stop configure -state disable
}

proc BuildingsTerrain::CreateWindow { } {
    variable llx
    variable lly
    variable urx
    variable ury
    variable z_zero
    variable filename_raster
    variable box_height
    set w .gid.buildings
    InitWindow $w [_ "Building and terrain"] BuildingsTerrainWindowGeom BuildingsTerrain::TestWin
    set fbox [ttk::labelframe $w.fbox -text [_ "Box to be considered"]]
    ttk::label $fbox.ll -text [_ "Lower-left corner"]
    ttk::entry $fbox.llx -textvariable BuildingsTerrain::llx
    ttk::entry $fbox.lly -textvariable BuildingsTerrain::lly
    ttk::button $fbox.getbox -image [gid_themes::GetImage point.png] -command [list BuildingsTerrain::SelectBox]
    ttk::label $fbox.ur -text [_ "Upper-right corner"]
    ttk::entry $fbox.urx -textvariable BuildingsTerrain::urx
    ttk::entry $fbox.ury -textvariable BuildingsTerrain::ury
    grid $fbox.ll $fbox.llx $fbox.lly $fbox.getbox -sticky ew
    grid configure $fbox.ll $fbox.getbox -sticky w
    grid $fbox.ur $fbox.urx $fbox.ury -sticky ew
    grid configure $fbox.ur -sticky w

    set fboxheight [ttk::frame $fbox.fboxheight]
    ttk::label $fboxheight.bh -text [_ "Box height"]
    ttk::entry $fboxheight.eh -textvariable BuildingsTerrain::box_height
    grid $fboxheight.bh $fboxheight.eh -sticky ew
    grid configure $fboxheight.bh -sticky w
    grid $fboxheight -sticky new -columnspan 4
    grid columnconfigure $fboxheight 2 -weight 1

    grid $fbox -sticky new
    grid columnconfigure $fbox {1 2} -weight 1


    set ftopograpy [ttk::labelframe $w.ftopograpy -text [_ "Topography from raster"]]
    ttk::checkbutton $ftopograpy.zero -text "z=0" -variable BuildingsTerrain::z_zero
    ttk::entry $ftopograpy.filename -textvariable BuildingsTerrain::filename_raster
    ttk::button $ftopograpy.getfilename -image [gid_themes::GetImage folder.png] -command BuildingsTerrain::GetFilenameRaster
    grid $ftopograpy.zero -sticky w -columnspan 2
    grid $ftopograpy.filename $ftopograpy.getfilename -sticky ew
    grid configure $ftopograpy.getfilename -sticky w
    grid $ftopograpy -sticky new
    grid columnconfigure $ftopograpy 0 -weight 1

    set fmessages [ttk::frame $w.fmessages]
    ttk::label $fmessages.message -text "" -foreground red
    ttk::button $fmessages.stop -text [_ "Stop"] -command [list BuildingsTerrain::PressStop]
    grid $fmessages.message $fmessages.stop -sticky ew -padx 2 -pady 2
    grid configure $fmessages.stop -sticky e
    grid $fmessages -sticky new
    grid columnconfigure $fmessages 0 -weight 1

    set fbuttons [ttk::frame $w.buts -style BottomFrame.TFrame]
    ttk::button $w.buts.apply1 -text [_ "Surfaces at height from volumes"] -command [list BuildingsTerrain::DoSurfacesAtHeightFromVolumes] -width 24
    ttk::button $w.buts.apply2 -text [_ "Extrude surfaces"] -command [list BuildingsTerrain::DoMoveAndExtrudeSurfaces] -width 24
    ttk::button $w.buts.apply3 -text [_ "Average surfaces"] -command [list BuildingsTerrain::DoJoinAndAverageSurfaces] -width 24
    ttk::button $w.buts.apply4 -text [_ "Remove holes"] -command [list BuildingsTerrain::DoRemoveHolesSurfaces] -width 24

    ttk::button $w.buts.apply5 -text [_ "Volume body fitted"] -command [list BuildingsTerrain::DoVolumeBodyFitted] -width 24
    ttk::button $w.buts.apply6 -text [_ "Volume body fitted 2"] -command [list BuildingsTerrain::DoVolumeBodyFitted2] -width 24
    ttk::button $w.buts.apply7 -text [_ "Volume embedded"] -command [list BuildingsTerrain::DoVolumeEmbedded] -width 24
    ttk::button $w.buts.apply8 -text [_ "Nodes to z"] -command [list BuildingsTerrain::DoMoveNodesToZTerrain] -width 24

    ttk::button $w.buts.apply9 -text [_ "Surface body fitted"] -command [list BuildingsTerrain::DoSurfaceBodyFitted] -width 24



    ttk::button $w.buts.close -text [_ "Close"] -command [list destroy $w] -style BottomFrame.TButton
    grid $w.buts.apply1 $w.buts.apply2 $w.buts.apply3 $w.buts.apply4
    grid $w.buts.apply5 $w.buts.apply6 $w.buts.apply7 $w.buts.apply8
    grid $w.buts.apply9
    grid $w.buts.close
    grid $w.buts -row 5 -padx 2 -pady 2 -sticky sew
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }

    grid rowconfigure $w 4 -weight 1
    grid columnconfigure $w 0 -weight 1

    trace add variable BuildingsTerrain::z_zero write [list BuildingsTerrain::OnChangeZZeero $ftopograpy]
    bind $w <Destroy> [list +BuildingsTerrain::DestroyCreateWindow %W $w $ftopograpy] ;# + to add to previous script
    if { ![info exists z_zero] } {
        set z_zero 1
    }
    set z_zero $z_zero ;#to raise trace
}

proc BuildingsTerrain::DestroyCreateWindow { W w ftopograpy } {
    if { $W != $w } return
    trace remove variable BuildingsTerrain::z_zero write [list BuildingsTerrain::OnChangeZZeero $ftopograpy]
}

proc BuildingsTerrain::OnChangeZZeero { ftopograpy args } {
    variable z_zero
    if { [winfo exists $ftopograpy] } {
        if { $z_zero } {
            set state disabled
        } else {
            set state normal
        }
        $ftopograpy.filename configure -state $state
        $ftopograpy.getfilename configure -state $state
    }
}

proc BuildingsTerrain::DoVolumeBodyFitted { } {
    variable llx
    variable lly
    variable urx
    variable ury
    variable box_height
    variable z_zero
    variable filename_raster
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        error [_ "must set the lower-left and upper-right 2D corners of the region of interest"]
    }
    if { ![info exists box_height] } {
        error [_ "must set the height of the box to be created"]
    }
    if { ![info exists z_zero] } {
        error [_ "must set if z=0 or z(x,y) must be get from a raster file"]
    }
    if { !$z_zero && ![info exists filename_raster] } {
        error [_ "must set the filename of the raster file to get z(x,y)"]
    }
    if { $z_zero } {
        set filename ""
    } else {
        set filename $filename_raster
    }
    set bounding_box_2d [list $llx $lly $urx $ury]
    #set surface_heights [BuildingsTerrain::GetSurfaceHeightsFromCondition surface_imported-1 ZValue]
    set surface_heights [list] ;#now it is unneded, surfaces are located at its height
    BuildingsTerrain::VolumeBodyFitted $bounding_box_2d $box_height $filename $surface_heights
}

#initialize to my test values
proc BuildingsTerrain::TestInitializeValues { } {
    variable llx
    variable lly
    variable urx
    variable ury
    variable box_height
    variable z_zero
    variable filename_raster
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        lassign [4.5432e+05 4.6004e+06 4.5459e+05 4.6006e+06] llx lly urx ury
    }
    if { ![info exists box_height] } {
        set box_height 50.0
    }
    if { ![info exists z_zero] } {
        set z_zero 0
    }
    if { ![info exists filename_raster] } {
        set filename_raster {C:\gid project more\gidusers\CIMNE\Claudio Zinggerling\Shapefile_and_raster_terrain\Riera Mataro\Riera_Mataro.asc}
    }

}


############################################################################################################################
# BuildingsTerrain::DoSurfacesAtHeightFromVolumes
############################################################################################################################
# start from all building volumes located with z terrain or not
# objective: have only the top surfaces with z= building height (without take into account any z terrain)
# e.g. for the case of the step example with volumes, to be in the standard case of surfaces with z=height and apply other procs
############################################################################################################################

proc BuildingsTerrain::GetVolumeHeightFromBoundingBox { volume_id } {
    lassign [GiD_Info bounding_box volume $volume_id] x_min x_max y_min y_max z_min z_max
    set height [expr $z_max-$z_min]
    return $height
}

proc BuildingsTerrain::GetVolumeIdsOfSurfaceId { surface_id } {
    return [GidUtils::GetEntityHigherEntities surface $surface_id]
}

proc BuildingsTerrain::GetSurfaceHeightFromVolume { surface_id } {
    set volume_ids [BuildingsTerrain::GetVolumeIdsOfSurfaceId $surface_id]
    if { [llength $volume_ids] == 1 } {
        set height [BuildingsTerrain::GetVolumeHeightFromBoundingBox [lindex $volume_ids 0]]
    } else {
        set height ""
        #error "BuildingsTerrain::GetSurfaceHeightFromVolume. surface $surface_id doesn't below to one volume"
    }
    return $height
}

proc BuildingsTerrain::GetSurfaceHeightsFromVolumes { top_surface_ids } {
    set fail 0
    set surface_heights [list]
    foreach surface_id $top_surface_ids {
        set surface_height [BuildingsTerrain::GetSurfaceHeightFromVolume $surface_id]
        if { $surface_height == "" } {
            set fail 1
            #error "BuildingsTerrain::GetSurfaceHeightsFromVolume. surface $surface_id doesn't below to one volume"
            break
        } else {
            lappend surface_heights $surface_id $surface_height
        }
    }
    if { $fail } {
        set surface_heights [list]
    }
    return $surface_heights
}

proc BuildingsTerrain::DeleteAllButSurfaces { surface_ids } {
    GiD_Process Mescape Geometry Delete Volumes 1:end escape
    GiD_Process Mescape Geometry Delete Surfaces {*}$surface_ids InvertSelection escape
    GiD_Process Mescape Geometry Delete Lines 1:end escape
    GiD_Process Mescape Geometry Delete Points 1:end escape
}

proc BuildingsTerrain::MoveSurfacesToHeights { surface_heights } {
    foreach {surface_id height} $surface_heights {
        BuildingsTerrain::MoveEntitiesZAbsolute surface [list $surface_id] $height
    }
}

proc BuildingsTerrain::DoSurfacesAtHeightFromVolumes { } {
    variable llx
    variable lly
    variable urx
    variable ury
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        error [_ "must set the lower-left and upper-right 2D corners of the region of interest"]
    }
    set bounding_box_2d [list $llx $lly $urx $ury]
    set fail 0
    BuildingsTerrain::SetMessage [_ "Creating geometry"]...
    GidUtils::DisableGraphics
    set surface_ids [BuildingsTerrain::ListSurfacesInsideBBox2D $bounding_box_2d]
    set top_surface_ids [GidUtils::GetSurfacesFilterNormal $surface_ids {0.0 0.0 1.0} 5.0]
    set surface_heights [BuildingsTerrain::GetSurfaceHeightsFromVolumes $top_surface_ids]
    if { [expr [llength $surface_heights]/2] != [llength $top_surface_ids] } {
        set fail 1
    } else {
        BuildingsTerrain::DeleteAllButSurfaces $top_surface_ids
        #BuildingsTerrain::MoveEntitiesToZZero surface $top_surface_ids
        BuildingsTerrain::MoveSurfacesToHeights $surface_heights
    }
    BuildingsTerrain::SetMessage [_ "Done"]
    GidUtils::EnableGraphics
    BuildingsTerrain::ZoomFrame
    return $fail
}

############################################################################################################################
# BuildingsTerrain::DoVolumeBodyFitted2
############################################################################################################################
# difference with DoVolumeBodyFitted: allow adjacent buildings (surfaces virtually sharing lines in the 2D projection, although in 3D have different z and then line is not shared)
#
# start from all building surfaces located with z=heigth (and a raster file, else assume z terrain=0.0)
# objective: have the boundary surfaces (without the floor) of the volume of the surfaces extruded until the terrain
#     must not be any internal overlapped or inner surface,
#                 any line short than a tolerance or unconnected entities close that the tolerance (e.g. surfaces of two close, but not joined, buildings)
#                 if possible with the minimal amount of points, lines and surfaces
# try to:
# -proyect surfaces in 2D
# -colapse to join points and line (that could be at different heights in 3D to be collapsed)
# -find groups of connected blocks (sharing lines in 2D projection)
# -move surfaces to 3D (height+z terrain)
#    -for each block
#       -for each surface of the block
#          -collect all its zs and collapse them with a big tolerance to have a small list of ordered heigths
#          -snap the surfaces moving them in z to the closest value of the list
#       -for each increment of heights
#          -for each surface of the block
#             -extrude volumes from the surfaces that have current height equal to the source heigth
#             -identify the opposite surface and use it to replace the one of the block, for the next iteration
############################################################################################################################

proc BuildingsTerrain::CreateLayerIfNotExists { layer } {
    if { ![GiD_Layers exists $layer] } {
        GiD_Layers create $layer
    }
    return 0
}

proc BuildingsTerrain::AssignToLayer { layer gid_entity_type entity_ids  also_lower_entities } {
    BuildingsTerrain::CreateLayerIfNotExists $layer
    if { $also_lower_entities } {
        GiD_EntitiesLayers assign $layer -also_lower_entities ${gid_entity_type}s $entity_ids
    } else {
        GiD_EntitiesLayers assign $layer -also_lower_entities ${gid_entity_type}s $entity_ids
    }
    return 0
}

#tolerance to do a 1D collapse avoiding too near values
#return a sorted array of unrepeated (with tolerance) double values of building heigths
#the extra 0.0 value is added as ending value
proc BuildingsTerrain::GetIncreasingVectorHeights { surface_ids surface_heights tolerance } {
    set n [objarray length $surface_ids]
    set heights [objarray new doublearray [expr $n+1]] ;#+1 to store always the 0
    array set building_height $surface_heights
    objarray set $heights 0 0.0
    for {set i 0} {$i < $n} {incr i} {
        objarray set $heights [expr $i+1] $building_height([objarray get $surface_ids $i])
    }
    objarray sort -unique $heights
    set n [objarray length $heights] ;#sort -unique could reduce the array size
    if { $tolerance > 0.0 } {
        set height_prev [objarray get $heights 0]
        set heights_collapsed [list $height_prev]
        for {set i 1} {$i < $n} {incr i} {
            set height_i [objarray get $heights $i]
            if { [expr $height_i-$height_prev] >= $tolerance } {
                lappend heights_collapsed $height_i
                set height_prev $height_i
            }
        }
        if { [lindex $heights_collapsed end] != [objarray get $heights end] } {
            #force to preserve exact start and end
            if { [llength $heights_collapsed] == 1} {
                #in this case start and end could be small that epsilon
                lappend heights_collapsed [objarray get $heights end]
            } else {
                lset heights_collapsed end [objarray get $heights end]
            }
        }
        set heights [objarray new doublearray -values $heights_collapsed]
    }
    #objarray reverse $heights ;#to have -decreasing sort
    return $heights
}

proc BuildingsTerrain::GetExtrudedVolumeOppositeSurface { volume_id surface_id } {
    set opposite_surface_id -1
    set surface_points_sorted [BuildingsTerrain::GetLinesPoints [GidUtils::GetSurfaceLines $surface_id]]
    objarray sort $surface_points_sorted
    set volume_surface_ids [GidUtils::GetVolumeSurfaces $volume_id]
    foreach volume_surface_id $volume_surface_ids {
        if { $volume_surface_id == $surface_id } {
            continue
        }
        set reject_surface 0
        set volume_surface_lines [GidUtils::GetSurfaceLines $volume_surface_id]
        set n_lines [objarray length $volume_surface_lines]
        for {set i_line 0} {$i_line<$n_lines} {incr i_line} {
            set point_ids [GidUtils::GetLinePoints [objarray get $volume_surface_lines $i_line]]
            set n_points [objarray length $point_ids]
            for {set i_point 0} {$i_point<$n_points} {incr i_point} {
                set point_id [objarray get $point_ids $i_point]
                if { [objarray search -sorted $surface_points_sorted $point_id] != -1 } {
                    set reject_surface 1
                    break
                }
            }
            if { $reject_surface } {
                break
            }
        }
        if { !$reject_surface } {
            set opposite_surface_id $volume_surface_id
            break
        }
    }
    return $opposite_surface_id
}


#values are expected increasing
proc BuildingsTerrain::GetNearValue { values value } {
    set i_interval -1
    set num_values [objarray length $values]
    for {set i_value 0} {$i_value < $num_values} {incr i_value} {
        if { [objarray get $values $i_value]>= $value } {
            set i_interval $i_value
            break
        }
    }
    #W "i_interval=$i_interval"
    if { $i_interval ==-1 ||  $i_interval >= [expr $num_values-1] } {
        set value [objarray get $values end]
    } else {
        set v1 [objarray get $values $i_interval]
        set v2 [objarray get $values [expr $i_interval+1]]
        set d1 [expr $value-$v1]
        set d2 [expr $v2-$value]
        if { $d1 <= $d2 } {
            set value $v1
        } else {
            set value $v2
        }
    }
    return $value
}

proc BuildingsTerrain::GetAveragedHeight { surface_ids surface_heights } {
    set averaged_height 0.0
    set n [objarray length $surface_ids]
    array set building_height $surface_heights

    for {set i 0} {$i < $n} {incr i} {
        set averaged_height [expr $averaged_height+$building_height([objarray get $surface_ids $i])]
    }
    set averaged_height [expr $averaged_height/$n]
    return $averaged_height
}

proc BuildingsTerrain::JoinVolumes { volume_ids } {
    GiD_Process Mescape Geometry Edit JoinVolumes {*}$volume_ids escape
}

proc BuildingsTerrain::JoinSurfaces { surface_ids } {
    #GiD_Process Mescape Geometry Edit JoinSurfaces NoEachOneAlone JoinCoplanary {*}$surface_ids escape
    set string_ids [concat $surface_ids] ;#trick to avoid lost type intarray doing {*}$surface_ids
    GiD_Process Mescape Geometry Edit JoinSurfaces JoinCoplanary {*}$string_ids escape
}

proc BuildingsTerrain::JoinLinesTangent { line_ids } {
    #BuildingsTerrain::CollapseEntities line $line_ids ;#trick to join tangent lines
    GiD_Process Mescape Geometry Edit JoinLines {*}$line_ids escape
}

proc BuildingsTerrain::DoJoinAndAverageSurfaces { } {
    variable llx
    variable lly
    variable urx
    variable ury
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        error [_ "must set the lower-left and upper-right 2D corners of the region of interest"]
    }
    set join_tangent_lines 0
    set bounding_box_2d [list $llx $lly $urx $ury]
    set fail 0
    set callback_percent BuildingsTerrain::ShowPercent
    BuildingsTerrain::SetMessage [_ "Joining and averaging surfaces"]...
    GidUtils::DisableGraphics
    set surface_ids [BuildingsTerrain::ListSurfacesInsideBBox2D $bounding_box_2d]
    BuildingsTerrain::DeleteAllButSurfaces $surface_ids
    BuildingsTerrain::JoinAndAverageSurfaces $surface_ids $join_tangent_lines $callback_percent
    GidUtils::EnableGraphics
    BuildingsTerrain::ZoomFrame
    BuildingsTerrain::SetMessage [_ "Done"]
}

proc BuildingsTerrain::JoinAndAverageSurfaces { surface_ids join_tangent_lines {callback_percent ""} } {
    #store initial surface heights in a list, to be able to recover after move them to z zero and do other operations
    set surface_heights [BuildingsTerrain::GetSurfaceHeightsFromZ $surface_ids]
    BuildingsTerrain::MoveEntitiesToZZero surface $surface_ids
    BuildingsTerrain::CollapseEntities surface $surface_ids
    #update surfaces_moved, some of them could have been deleted by the collapse
    set surfaces_moved [GiD_Geometry list surface [lindex $surface_ids 0]:[lindex $surface_ids end]]
    set parts [GiD_Tools geometry classify_connected_parts surface $surfaces_moved]
    if { $callback_percent != "" } {
        BuildingsTerrain::EnableStop
    }
    set n [llength $parts]
    set nrefresh [expr {int($n/100.0)}]
    if { $nrefresh<1 } {
        set nrefresh 1
    }
    set i 0
    foreach part_surface_ids $parts {
        if { $callback_percent != "" && ![expr $i%$nrefresh] } {
            set percent [expr int($i/($n-1.0)*100.0)]
            set stop [$callback_percent $percent]
            if { $stop } {
                set fail -1
                break
            }
        }
        if { [objarray length $part_surface_ids] > 1 } {
            set averaged_height [BuildingsTerrain::GetAveragedHeight $part_surface_ids $surface_heights]
            set max_id_surface_before_join [GiD_Info geometry MaxNumSurfaces]
            BuildingsTerrain::JoinSurfaces $part_surface_ids
            set max_id_surface_after_join [GiD_Info geometry MaxNumSurfaces]
            if { [expr $max_id_surface_after_join-$max_id_surface_before_join] != 1 } {
                W "problem joining surfaces max_id_surface_before_join=$max_id_surface_before_join max_id_surface_after_join=$max_id_surface_after_join"
                W "part_surface_ids=$part_surface_ids"
            } else {
                set joined_surface_id $max_id_surface_after_join
                lappend surface_heights $joined_surface_id $averaged_height ;#could remove values of deleted entities...
                lset parts $i [objarray new intarray -values [list $joined_surface_id]]
                if { $join_tangent_lines } {
                    #comment this JoinLinesTangent that call collapse because is too expensive do it once by part, it expends more and more memory and finally crash!!
                    BuildingsTerrain::JoinLinesTangent [BuildingsTerrain::GetSurfacesLines [list $joined_surface_id]]
                }
            }
        } elseif { [objarray length $part_surface_ids] == 1 } {
            if { $join_tangent_lines } {
                #comment this JoinLinesTangent that call collapse because is too expensive do it once by part, it expends more and more memory and finally crash!!
                BuildingsTerrain::JoinLinesTangent [BuildingsTerrain::GetSurfacesLines [list [objarray get $part_surface_ids 0]]]
            }
        } else {
            W "Error: part without any surface"
        }
        incr i
    }
    #update surfaces_moved, some of them could have been deleted by the collapse
    set surfaces_moved [GiD_Geometry list surface [lindex $surface_ids 0]:end]

    set raster ""
    BuildingsTerrain::MoveSurfacesZAbsolute $surfaces_moved $surface_heights $raster
    return 0
}

proc BuildingsTerrain::DoVolumeBodyFitted2 { } {
    variable llx
    variable lly
    variable urx
    variable ury
    variable box_height
    variable z_zero
    variable filename_raster
    variable average_surfaces
    set average_surfaces 0
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        error [_ "must set the lower-left and upper-right 2D corners of the region of interest"]
    }
    if { ![info exists box_height] } {
        error [_ "must set the height of the box to be created"]
    }
    if { ![info exists z_zero] } {
        error [_ "must set if z=0 or z(x,y) must be get from a raster file"]
    }
    if { !$z_zero && ![info exists filename_raster] } {
        error [_ "must set the filename of the raster file to get z(x,y)"]
    }
    if { $z_zero } {
        set filename ""
    } else {
        set filename $filename_raster
    }
    set bounding_box_2d [list $llx $lly $urx $ury]

    set callback_percent BuildingsTerrain::ShowPercent

    set fail 0
    set tolerance 1.0 ;#assumed in meters (consider same build heigth with this difference)
    BuildingsTerrain::SetMessage [_ "Creating geometry"]...
    GidUtils::DisableGraphics
    set raster "" ;#in case of have filename_raster must be the true raster...
    set surface_ids [BuildingsTerrain::ListSurfacesInsideBBox2D $bounding_box_2d]
    BuildingsTerrain::DeleteAllButSurfaces $surface_ids
    #store initial surface heights in a list, to be able to recover after move them to z zero and do other operations
    set surface_heights [BuildingsTerrain::GetSurfaceHeightsFromZ $surface_ids]
    BuildingsTerrain::MoveEntitiesToZZero surface $surface_ids
    BuildingsTerrain::CollapseEntities surface $surface_ids
    #update surfaces_moved, some of them could have been deleted by the collapse
    set surfaces_moved [GiD_Geometry list surface [lindex $surface_ids 0]:[lindex $surface_ids end]]
    set parts [GiD_Tools geometry classify_connected_parts surface $surfaces_moved]
    if { $average_surfaces } {
        if { $callback_percent != "" } {
            BuildingsTerrain::EnableStop
        }
        set n [llength $parts]
        set nrefresh [expr {int($n/100.0)}]
        if { $nrefresh<1 } {
            set nrefresh 1
        }
        set i 0
        foreach part_surface_ids $parts {
            if { $callback_percent != "" && ![expr $i%$nrefresh] } {
                set percent [expr int($i/($n-1.0)*100.0)]
                set stop [$callback_percent $percent]
                if { $stop } {
                    set fail -1
                    break
                }
            }
            if { [objarray length $part_surface_ids] > 1 } {
                set averaged_height [BuildingsTerrain::GetAveragedHeight $part_surface_ids $surface_heights]
                set max_id_surface_before_join [GiD_Info geometry MaxNumSurfaces]
                BuildingsTerrain::JoinSurfaces $part_surface_ids
                set max_id_surface_after_join [GiD_Info geometry MaxNumSurfaces]
                if { [expr $max_id_surface_after_join-$max_id_surface_before_join] != 1 } {
                    W "problem joining surfaces max_id_surface_before_join=$max_id_surface_before_join max_id_surface_after_join=$max_id_surface_after_join"
                    W "part_surface_ids=$part_surface_ids"
                } else {
                    set joined_surface_id $max_id_surface_after_join
                    lappend surface_heights $joined_surface_id $averaged_height ;#could remove values of deleted entities...
                    lset parts $i [objarray new intarray -values [list $joined_surface_id]]
                    #comment this JoinLinesTangent that call collapse because is too expensive do it once by part, it expends more and more memory and finally crash!!
                    BuildingsTerrain::JoinLinesTangent [BuildingsTerrain::GetSurfacesLines [list $joined_surface_id]]
                }
            } elseif { [objarray length $part_surface_ids] == 1 } {
                #comment this JoinLinesTangent that call collapse because is too expensive do it once by part, it expends more and more memory and finally crash!!
                BuildingsTerrain::JoinLinesTangent [BuildingsTerrain::GetSurfacesLines [list [objarray get $part_surface_ids 0]]]
            } else {
                W "Error: part without any surface"
            }
            incr i
        }
        #update surfaces_moved, some of them could have been deleted by the collapse
        set surfaces_moved [GiD_Geometry list surface [lindex $surface_ids 0]:end]
    }
    BuildingsTerrain::SetMessage [_ "Restoring surface's z"]
    BuildingsTerrain::MoveSurfacesZAbsolute $surfaces_moved $surface_heights $raster

    #GidUtils::EnableGraphics
    #BuildingsTerrain::ZoomFrame
    #BuildingsTerrain::SetMessage [_ "Done"]
    #return 0

    #W "num parts=[llength $parts]"
    foreach part_surface_ids $parts {
        set num_surfaces_part [objarray length $part_surface_ids]
        #W "num_surfaces_part=$num_surfaces_part"
        #W "part_surface_ids: $part_surface_ids"
        set part_increasing_heights [BuildingsTerrain::GetIncreasingVectorHeights $part_surface_ids $surface_heights $tolerance]
        set num_part_heights [objarray length $part_increasing_heights]
        #W "num_part_heights=$num_part_heights: $part_increasing_heights"
        #adjust surfaces to the discrete collection on values of part_increasing_heights
        for {set i_surface_part 0} {$i_surface_part < $num_surfaces_part} {incr i_surface_part} {
            set surface_id [objarray get $part_surface_ids $i_surface_part]
            set surface_center [GidUtils::GetEntityCenter surface $surface_id]
            set z_terrain [BuildingsTerrain::GetZTerrainAtXY [lrange $surface_center 0 1] $raster]
            set height [lindex $surface_center 2]
            set z_absolute [expr $z_terrain+$height]
            set z_absolute_round [BuildingsTerrain::GetNearValue $part_increasing_heights $z_absolute]
            #W "surface $surface_id z_absolute=$z_absolute z_absolute_round=$z_absolute_round"
            BuildingsTerrain::MoveEntitiesZAbsolute surface $surface_id $z_absolute_round
        }
        set max_id_volume_before_extrude [GiD_Info geometry MaxNumVolumes]
        set volume_ids [list]
        for {set i_copy 1} {$i_copy < $num_part_heights} {incr i_copy} {
            #W "i_copy=$i_copy"
            set i_origin [expr $num_part_heights-$i_copy]
            set i_destination [expr $i_origin-1]
            set heigth_copy_origin [objarray get $part_increasing_heights $i_origin]
            set heigth_copy_destination [objarray get $part_increasing_heights $i_destination]
            set height_copy_increment [expr $heigth_copy_destination-$heigth_copy_origin]
            #W "heigth_copy_origin=$heigth_copy_origin heigth_copy_destination=$heigth_copy_destination height_copy_increment=$height_copy_increment"
            for {set i_surface_part 0} {$i_surface_part < $num_surfaces_part} {incr i_surface_part} {
                set surface_id [objarray get $part_surface_ids $i_surface_part]
                set surface_center [GidUtils::GetEntityCenter surface $surface_id]
                set height [lindex $surface_center 2]
                set distance_z [expr abs($heigth_copy_origin-$height)]
                if { $distance_z < $tolerance } {
                    set max_id_volume_before [GiD_Info geometry MaxNumVolumes]
                    BuildingsTerrain::ExtrudeEntitiesZRelative surface $surface_id $height_copy_increment
                    set max_id_volume_after [GiD_Info geometry MaxNumVolumes]
                    set num_volumes [expr $max_id_volume_after-$max_id_volume_before]
                    if { $num_volumes != 1 } {
                        error "Extruded volume not found for surface_id=$surface_id height_copy_increment=$height_copy_increment"
                    } else {
                        set volume_id $max_id_volume_after
                        set surface_copy_id [BuildingsTerrain::GetExtrudedVolumeOppositeSurface $volume_id $surface_id]
                        #replace the part volume for the new copy, for next copies
                        objarray set $part_surface_ids $i_surface_part $surface_copy_id
                        lappend volume_ids $volume_id
                    }
                }
            }
        }
        BuildingsTerrain::CollapseEntities volume $volume_ids
        BuildingsTerrain::JoinVolumes $volume_ids
        set joined_volume_ids [GiD_Geometry list volume [expr $max_id_volume_before_extrude+1]:end]
        if { [llength $joined_volume_ids] != 1 } {
            W "volumes $volume_ids joined to $joined_volume_ids"
        }
        BuildingsTerrain::JoinSurfaces [BuildingsTerrain::GetVolumesSurfaces $joined_volume_ids]
        BuildingsTerrain::JoinLinesTangent [BuildingsTerrain::GetSurfacesLines [BuildingsTerrain::GetVolumesSurfaces $joined_volume_ids]]
        GiD_Geometry -v2 delete volume $joined_volume_ids
        GiD_Geometry -v2 delete -also_lower_entities surface $part_surface_ids
    }
    BuildingsTerrain::SetMessage [_ "Done"]
    GidUtils::EnableGraphics
    #GidUtils::UpdateWindow LAYER
    GeometryView
    BuildingsTerrain::ZoomFrame
    return $fail
}

########################################################################################################################################
# BuildingsTerrain::DoVolumeEmbedded
########################################################################################################################################
#start from a raster file (and any building surface), and a 2D rectangle (else bounding box of raster could be used), and box height
#objective: have a volume of a 2D rectangle, with the floor with the z of the terrain and the top witht the box heitht relative to floor
#           to be used as domain for embedded solver (after the soup or surfaces, without foors, of the building must be added)
########################################################################################################################################

#bounding_box_2d: list of corner_min_x corner_min_y corner_max_x corner_max_y
proc BuildingsTerrain::Resample { raster bounding_box_2d ncols nrows } {
    lassign $bounding_box_2d corner_min(0) corner_min(1) corner_max(0) corner_max(1)
    set xcellsize [expr ($corner_max(0)-$corner_min(0))/double($ncols)]
    set ycellsize [expr ($corner_max(1)-$corner_min(1))/double($nrows)]
    set raster_resampled [list $ncols $nrows $corner_min(0) $corner_min(1) $xcellsize $ycellsize -9999 [objarray new doublearray 0]]
    set zs [GiD_Raster interpolate $raster [list raster $raster_resampled]]
    set raster_resampled [list $ncols $nrows $corner_min(0) $corner_min(1) $xcellsize $ycellsize -9999 $zs]
    return $raster_resampled
}

proc BuildingsTerrain::DivideLines { line_ids num_divisions } {
    GiD_Process Mescape Geometry Edit DivideLine Multiple NumDivisions $num_divisions {*}$line_ids escape escape
}

proc BuildingsTerrain::SurfacesRebuildByBoundary { surfaces_ids } {
    GiD_Process Mescape Geometry Edit JoinSurfaces EachOneAlone QuickJoin {*}$surfaces_ids escape escape
}

proc BuildingsTerrain::DoVolumeEmbedded { } {
    set fail 0
    variable llx
    variable lly
    variable urx
    variable ury
    variable box_height
    variable z_zero
    variable filename_raster

    set num_divisions 20

    set show_advance_bar 1
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        error [_ "must set the lower-left and upper-right 2D corners of the region of interest"]
    }
    if { ![info exists box_height] } {
        error [_ "must set the height of the box to be created"]
    }
    if { ![info exists z_zero] } {
        error [_ "must set if z=0 or z(x,y) must be get from a raster file"]
    }
    if { !$z_zero && ![info exists filename_raster] } {
        error [_ "must set the filename of the raster file to get z(x,y)"]
    }
    if { $z_zero } {
        set filename ""
    } else {
        set filename $filename_raster
    }
    set bounding_box_2d [list $llx $lly $urx $ury]

    if { $filename == "" } {
        set raster [BuildingsTerrain::CreateRasterZConstant $bounding_box_2d 0.0]
    } else {
        set raster [BuildingsTerrain::ReadRaster $filename $show_advance_bar]
    }
    BuildingsTerrain::SetMessage [_ "Creating geometry"]...
    GidUtils::DisableGraphics
    set max_id_line_begin [GiD_Info geometry MaxNumLines]
    set z_terrain [BuildingsTerrain::GetZTerrainAtBoundingBox2DCenter $bounding_box_2d $raster]
    set top_surface_id [BuildingsTerrain::CreateSurfaceFromBoundingBox $bounding_box_2d [expr $z_terrain+$box_height]]
    BuildingsTerrain::AssignToGroup "box_top" surface [list $top_surface_id]
    set max_id_surface_before_extrude_top [GiD_Info geometry MaxNumSurfaces]
    BuildingsTerrain::ExtrudeEntitiesZRelative line [GidUtils::GetSurfaceLines $top_surface_id] -$box_height
    set surfaces_lateral [GiD_Geometry list surface [expr $max_id_surface_before_extrude_top+1]:end]
    BuildingsTerrain::AssignToGroup "box_lateral_x_min" surface [GidUtils::GetSurfacesFilterNormal $surfaces_lateral {1.0 0.0 0.0} 5.0]
    BuildingsTerrain::AssignToGroup "box_lateral_x_max" surface [GidUtils::GetSurfacesFilterNormal $surfaces_lateral {-1.0 0.0 0.0} 5.0]
    BuildingsTerrain::AssignToGroup "box_lateral_y_min" surface [GidUtils::GetSurfacesFilterNormal $surfaces_lateral {0.0 1.0 0.0} 5.0]
    BuildingsTerrain::AssignToGroup "box_lateral_y_max" surface [GidUtils::GetSurfacesFilterNormal $surfaces_lateral {0.0 -1.0 0.0} 5.0]
    set soil_boundary_line_ids [GiD_Geometry list -higherentity 1 line [expr $max_id_line_begin+1]:end]
    BuildingsTerrain::DivideLines $soil_boundary_line_ids $num_divisions
    #to move only the points of the box, but better move also the points of the soil because are calculated smoothed to nodes, not in elements
    #set soil_boundary_line_ids [GiD_Geometry list -higherentity 1 line [expr $max_id_line_begin+1]:end]
    #set soil_boundary_point_ids [BuildingsTerrain::GetLinesPoints $soil_boundary_line_ids]

    set max_id_surface_before_create_soil [GiD_Info geometry MaxNumSurfaces]
    set ncols $num_divisions
    set nrows $ncols
    set raster_resampled [BuildingsTerrain::Resample $raster $bounding_box_2d $ncols $nrows]
    set value_smoothed_to_nodes 1
    GIS::ImportRaster_Geometry $raster_resampled $show_advance_bar $value_smoothed_to_nodes
    BuildingsTerrain::AssignToGroup "soil" surface [GiD_Geometry list surface [expr $max_id_surface_before_create_soil+1]:end]

    set soil_boundary_line_ids [GiD_Geometry list -higherentity 1 line [expr $max_id_line_begin+1]:end]
    set soil_boundary_point_ids [BuildingsTerrain::GetLinesPoints $soil_boundary_line_ids]
    BuildingsTerrain::MoveGeometryPointsToZTerrain $raster $soil_boundary_point_ids

    set soil_boundary_line_ids [GiD_Geometry list -higherentity 1 line [expr $max_id_line_begin+1]:end]
    BuildingsTerrain::CollapseEntities line $soil_boundary_line_ids
    #rebuild lateral surfaces that are distorted by the movement
    set lateral_surfaces [BuildingsTerrain::GetEntitiesOfGroups [list box_lateral_x_min box_lateral_x_max box_lateral_y_min box_lateral_y_max] surface]
    BuildingsTerrain::SurfacesRebuildByBoundary $lateral_surfaces
    set max_id_volume_before [GiD_Info geometry MaxNumVolumes]
    set volume_surfaces [BuildingsTerrain::GetEntitiesOfGroups [list box_top box_lateral_x_min box_lateral_x_max box_lateral_y_min box_lateral_y_max soil] surface]
    BuildingsTerrain::CreateVolume $volume_surfaces
    set max_id_volume_after [GiD_Info geometry MaxNumVolumes]
    set num_volumes [expr $max_id_volume_after-$max_id_volume_before]
    if { $num_volumes > 0 } {
        BuildingsTerrain::AssignToGroup "air" volume [list $max_id_volume_after]
        BuildingsTerrain::SetMessage [_ "Done"]
    } else {
        BuildingsTerrain::SetMessage [_ "Error: air volume not created"]
    }

    GidUtils::EnableGraphics
    GeometryView
    BuildingsTerrain::ZoomFrame
    GidUtils::UpdateWindow GROUPS
    return $fail
}

############################################################################################################################
# BuildingsTerrain::DoMoveAndExtrudeSurfaces
############################################################################################################################
#start from all building surfaces located with z=heigth (and a raster file, else assume z terrain=0.0)
#objective: have the boundary surfaces (without the floor) of the volume of the surfaces located with z terreain
#and extruded until the floor. It doesn't matter overlapped or intersecting surfaces (for embedded solver)
############################################################################################################################
proc BuildingsTerrain::ShowPercent { percent } {
    variable stop
    BuildingsTerrain::SetMessage [_ "%s percent" $percent]
    return $stop
}

proc BuildingsTerrain::DoMoveAndExtrudeSurfaces { } {
    set fail 0
    variable llx
    variable lly
    variable urx
    variable ury
    variable box_height
    variable z_zero
    variable filename_raster


    set show_advance_bar 1
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        error [_ "must set the lower-left and upper-right 2D corners of the region of interest"]
    }
    if { ![info exists z_zero] } {
        error [_ "must set if z=0 or z(x,y) must be get from a raster file"]
    }
    if { !$z_zero && ![info exists filename_raster] } {
        error [_ "must set the filename of the raster file to get z(x,y)"]
    }
    if { $z_zero } {
        set filename ""
    } else {
        set filename $filename_raster
    }

    set bounding_box_2d [list $llx $lly $urx $ury]

    if { $filename == "" } {
        set raster [BuildingsTerrain::CreateRasterZConstant $bounding_box_2d 0.0]
    } else {
        set raster [BuildingsTerrain::ReadRaster $filename $show_advance_bar]
    }


    BuildingsTerrain::SetMessage [_ "Creating geometry"]...
    set callback_percent BuildingsTerrain::ShowPercent
    if { $callback_percent != "" } {
        BuildingsTerrain::EnableStop
    }
    GidUtils::DisableGraphics
    set surface_ids [BuildingsTerrain::ListSurfacesInsideBBox2D $bounding_box_2d]
    set surface_heights [BuildingsTerrain::GetSurfaceHeightsFromZ $surface_ids]
    BuildingsTerrain::MoveSurfacesZAbsolute $surface_ids $surface_heights $raster
    BuildingsTerrain::ExtrudeLinesOfSurfacesZRelative $surface_ids $surface_heights all_boundaries $callback_percent
    BuildingsTerrain::SetMessage [_ "Done"]
    GidUtils::EnableGraphics
    BuildingsTerrain::ZoomFrame
    return $fail
}

proc BuildingsTerrain::DoMoveNodesToZTerrain {  } {
    variable filename_raster
    set fail 0
    set show_advance_bar 1
    BuildingsTerrain::SetMessage [_ "Reading raster"]...
    set raster [BuildingsTerrain::ReadRaster $filename_raster $show_advance_bar]
    GIS::MoveNodesToZRaster $raster 1:[GiD_Info Mesh MaxNumNodes]
    BuildingsTerrain::ZoomFrame
    BuildingsTerrain::SetMessage [_ "Done"]
}


############################################################################################################################
# BuildingsTerrain::SurfaceBodyFitted
############################################################################################################################
# start from 2D surfaces located with z terrain (or heigths provided as argument), and a raster file, else assume z terrain=0.0
# a selection 2D rectangle (else the bounding box of surfaces and/or raster could be used)
# objective: body-fitted mesh of triangle for CFD classical simulation
# select only the surfaces completely inside a rectangle and create a single surface of the water holed by the initial surfaces
# mesh it and set the nodes as z terrain, and reconvert to geometry?
# limitation: the surfaces of the buildings must not be overlapped or too closed between
# them (te be able to create the 2D floor surface or to mesh it)
#############################################################################################################################

proc BuildingsTerrain::SurfaceBodyFitted { bounding_box_2d filename_raster } {
    variable show_advance_bar
    set join_surfaces 1 ;#join surfaces sharing some line, to avoid repeated line (without space between them) when create a whole in the main surface
    set remove_holes 1 ;#to avoid surfaces inside holes when create a whole in the main surface
    #to join surfaces sharing lines (they will be a topologic problem to create a hole in the
    #the complementary surface, with repeated line for the surface and no space to fit elements)
    set create_surfaces_from_mesh_at_z 1
    set fail 0
    BuildingsTerrain::SetMessage [_ "Reading raster"]...
    if { $filename_raster == "" } {
        set raster [BuildingsTerrain::CreateRasterZConstant $bounding_box_2d 0.0]
    } else {
        set raster [BuildingsTerrain::ReadRaster $filename_raster $show_advance_bar]
    }

    BuildingsTerrain::SetMessage [_ "Creating geometry"]...
    GidUtils::DisableGraphics
    set inside_surface_ids [BuildingsTerrain::ListSurfacesInsideBBox2D $bounding_box_2d]
    #BuildingsTerrain::DeleteAllButSurfaces $inside_surface_ids
    if { $join_surfaces } {
        BuildingsTerrain::JoinSurfaces $inside_surface_ids
        set inside_surface_ids [BuildingsTerrain::ListSurfacesInsideBBox2D $bounding_box_2d]
    }
    if { $remove_holes } {
        BuildingsTerrain::RemoveHolesSurfaces $inside_surface_ids 1
    }
    BuildingsTerrain::SetMessage [_ "Creating soil"]...
    BuildingsTerrain::MoveEntitiesToZZero surface $inside_surface_ids
    set bottom_surface_id [BuildingsTerrain::CreateSurfaceFromBoundingBox $bounding_box_2d 0.0]
    #set num_divisions 20
    #BuildingsTerrain::DivideLines [GidUtils::GetSurfaceLines $bottom_surface_id] $num_divisions
    BuildingsTerrain::AddHolesSurface $bottom_surface_id $inside_surface_ids

    if { $create_surfaces_from_mesh_at_z } {
        set max_node_id_before [GiD_Info Mesh MaxNumNodes]
        set max_element_id_before [GiD_Info Mesh MaxNumElements]
        set fail_mesh [BuildingsTerrain::GenerateMeshFromSurfaceTrySeveralMethods $bottom_surface_id]
        if { $fail_mesh } {
            set fail 1
            BuildingsTerrain::SetMessage [_ "Error: auxiliary mesh not created"]
        } else {
            #maybe can not delete the main holed surface, to be re-meshed by GiD (but by now this meshing is problematic)
            GiD_Geometry -v2 delete -also_lower_entities surface [list $bottom_surface_id]
            #maybe can not delete initial building surfaces, for visualization pourposes
            #BuildingsTerrain::AssignToGroup "buildings" surface $inside_surface_ids
            GiD_Geometry -v2 delete -also_lower_entities surface $inside_surface_ids
            GIS::MoveNodesToZRaster $raster [expr $max_node_id_before+1]:[GiD_Info Mesh MaxNumNodes]
            set max_id_surface_before_create_soil [GiD_Info geometry MaxNumSurfaces]
            BuildingsTerrain::CreateGeometryFromMesh [expr $max_element_id_before+1] end
            GiD_Mesh delete element [GiD_Mesh list element [expr $max_node_id_before+1]:end]
            BuildingsTerrain::AssignToGroup "soil" surface [GiD_Geometry list surface [expr $max_id_surface_before_create_soil+1]:end]
            lassign $bounding_box_2d llx lly urx ury
            BuildingsTerrain::AssignToGroup "box_lateral_x_min" line [GiD_Geometry list -higherentity 1 -plane [list 1 0 0 [expr -$llx] 1e-5] line]
            BuildingsTerrain::AssignToGroup "box_lateral_x_max" line [GiD_Geometry list -higherentity 1 -plane [list 1 0 0 [expr -$urx] 1e-5] line]
            BuildingsTerrain::AssignToGroup "box_lateral_y_min" line [GiD_Geometry list -higherentity 1 -plane [list 0 1 0 [expr -$lly] 1e-5] line]
            BuildingsTerrain::AssignToGroup "box_lateral_y_max" line [GiD_Geometry list -higherentity 1 -plane [list 0 1 0 [expr -$ury] 1e-5] line]
        }
    }

    BuildingsTerrain::SetMessage [_ "Done"]

    GidUtils::EnableGraphics
    GeometryView
    BuildingsTerrain::ZoomFrame
    Groups::GroupsToLayers
    GidUtils::UpdateWindow GROUPS
    BuildingsTerrain::AssignProblemtypeData
    return $fail
}

proc BuildingsTerrain::DoSurfaceBodyFitted { } {
    variable llx
    variable lly
    variable urx
    variable ury
    variable z_zero
    variable filename_raster
    if { ![info exists llx] || ![info exists lly] || ![info exists urx] || ![info exists ury] } {
        error [_ "must set the lower-left and upper-right 2D corners of the region of interest"]
    }
    if { ![info exists z_zero] } {
        error [_ "must set if z=0 or z(x,y) must be get from a raster file"]
    }
    if { !$z_zero && ![info exists filename_raster] } {
        error [_ "must set the filename of the raster file to get z(x,y)"]
    }
    if { $z_zero } {
        set filename ""
    } else {
        set filename $filename_raster
    }
    set bounding_box_2d [list $llx $lly $urx $ury]
    BuildingsTerrain::SurfaceBodyFitted $bounding_box_2d $filename
}

#set an initial water height of 0.2 m in the whole domain, a manning and a simulation time
proc BuildingsTerrain::AssignProblemtypeDataIBER { } {
    #general data (use questions trimmed removing the last 'units part', that maybe could de changed in the future)
    foreach {prop value} {Max_simulation_time 200 Results_time_interval 10 Number_of_Threads 8 Force_Results_to_Vertex ON} {
        GiD_AccessValue set gendata $prop $value
    }
    set condition 2D_Outlet
    set over line
    set values [list SUBCRITICAL Given_level {#N# 2 0.0 0.0} 0.0 Height 0.0 0.0 1 {#N# 0} 1 {#N# 0} 1]
    set group_names [list box_lateral_x_min box_lateral_x_max box_lateral_y_min box_lateral_y_max]
    GiD_AssignData condition $condition ${over}s $values [BuildingsTerrain::GetEntitiesOfGroups $group_names $over]

    set condition 2D_Initial_Condition
    set over surface
    set values [list Depth 0 1.0]
    set group_names [list soil]
    GiD_AssignData condition $condition ${over}s $values [BuildingsTerrain::GetEntitiesOfGroups $group_names $over]

    #material->manning
    set material residential
    set over surface
    set group_names [list soil]
    GiD_AssignData material $material ${over}s [BuildingsTerrain::GetEntitiesOfGroups $group_names $over]
    return 0
}
