
namespace eval Carat {
    variable _merge 1
    variable info_plugin ;#information read from .xml plugin file
    variable percent_done
    variable stop
    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}


#########################################################################################
######################################### READ PRE ######################################
#########################################################################################

proc Carat::NormalizeParameter { a b value } {
    set result [expr {double($value-$a)/double($b-$a)}]
    if { $result < 0.0 } {
        if { $result < -0.001 } {
            W "Error, wrong parameter=$result"
        }
        set result 0.0
    } elseif { $result > 1.0 } {
        if { $result > 1.001 } {
            W "Error, wrong parameter=$result"
        }
        set result 1.0
    }
    return $result
}

proc Carat::NormalizeKnots { knots } {
    set a [lindex $knots 0]
    set b [lindex $knots end]
    set normalized_knots [list]
    foreach knot $knots {
        lappend normalized_knots [Carat::NormalizeParameter $a $b $knot]
    }
    return $normalized_knots
}

#remove close points to allow a interpolation
proc Carat::FilterInterpolationPoints { interpolation_points tolerance } {
    set p_prev [lindex $interpolation_points 0]
    lappend filtered_interpolation_points $p_prev
    foreach p_i [lrange $interpolation_points 1 end] {
        set d [MathUtils::VectorDistance $p_prev $p_i]
        if { $d > $tolerance } {
            lappend filtered_interpolation_points $p_i
        }
    }
    #force the last point to be exactly like the original
    lset filtered_interpolation_points end [lindex $interpolation_points end]
    return $filtered_interpolation_points
}

#jump commented lines and empty lines
proc Carat::GetNextLine { fp } {
    set line ""
    while { ![eof $fp] } {
        gets $fp line
        set line [string trim $line]
        if { [string index $line 0] == "!" } {
            continue
        } else {
            break
        }
    }
    return $line
}
#read .georhino.txt files
proc Carat::ReadPreGeometry { filename } {
    variable master_control_point ;#to be used for results, to get non existent results of a master node with results if it exists
    variable num_control_points_surface ;#used to convert .post.res,it is necessary to assign values to each surface
    array unset master_control_point
    array unset num_control_points_surface

    set gid_verso(TRUE) 0
    set gid_verso(FALSE) 1
    set fail 0
    if { ![file exists $filename] } {
        WarnWin [_ "File %s not found" $filename]
        return 1
    }
    set fp [open $filename r]
    if { $fp == "" } {
        WarnWin [_ "Cannot open file '%s'" $filename]
        return 1
    }

    set control_point_count 0


    set layer [GiD_Info Project LayerToUse]
    while { ![eof $fp] } {
        set line [Carat::GetNextLine $fp]
        if { $line != "ND-COOR" } {
            #error
            break
        }
        #read NODE coordinates (knot positions used for calculation, and referenced in results)
        while { ![eof $fp] } {
            set line [Carat::GetNextLine $fp]
            if { [lindex $line 0] == "NODE" } {
                lassign $line dummy node_id dummy x dummy y dummy z
                set node($node_id) [list $x $y $z]
            } else {
                break
            }
        }
        #read CTRLPT_PAR
        set line [Carat::GetNextLine $fp]
        if { $line == "CTRL_PTS_PAR_DEF" } {
            #read NODE coordinates (knot positions used for calculation, and referenced in results)
            while { ![eof $fp] } {
                set line [Carat::GetNextLine $fp]
                if { [lindex $line 0] == "CTRLPT_PAR" } {
                    lassign $line dummy ctrlpt_par_id x y z
                    set ctrlpt_par($ctrlpt_par_id) [list $x $y $z]
                } else {
                    set line [Carat::GetNextLine $fp]
                    break
                }
            }
        }
        #read information of each surface  NURBS-BLOCK
        while { ![eof $fp] } {
            if { [lindex $line 0] != "NURBS_PATCH" } {
                #error
                break
            }
            lassign $line dummy surface_id dummy key
            if { $key != "NURBS_2D" } {
                #error
                break
            }
            set line [Carat::GetNextLine $fp]
            set id_ctrl_pts_ctrl_pts_nodes [lindex $line 3] ;#CTRL_PTS = CTRL_PTS_NODES 1
            set line [Carat::GetNextLine $fp]
            set num_control_points_u [lindex $line 2] ;#NCTRL =  44
            incr num_control_points_u ;#seems that was the num of spans
            set line [Carat::GetNextLine $fp]
            set num_control_points_v [lindex $line 2] ;#MCTRL =  44
            incr num_control_points_v ;#seems that was the num of spans
            set line [Carat::GetNextLine $fp]
            set degree_u [lindex $line 2];#PDEG =   3
            set line [Carat::GetNextLine $fp]
            set degree_v [lindex $line 2];#QDEG =   3
            set line [Carat::GetNextLine $fp]
            set knots_u [lrange $line 2 end];#UKNOT =  0.00000000000000e+000, 0.00000000000000e+000, ... , 1.00000000000000e+001
            set knots_u [string map {, {}} $knots_u] ;#replace unwanted commas by spaces
            set line [Carat::GetNextLine $fp]
            set knots_v [lrange $line 2 end];#VKNOT =  0.00000000000000e+000,..., 1.00000000000000e+001
            set knots_v [string map {, {}} $knots_v] ;#replace unwanted commas by spaces
            set line [Carat::GetNextLine $fp]
            if { [lindex $line 0] ==  "TRIMMING"} {
                set id_trimming_b_rep [lindex $line 3];#TRIMMING = B_REP 1
                set istrimmed 1 ;# Set surface as trimmed
                set line [Carat::GetNextLine $fp]
            } else {
                set istrimmed 0 ;# Set surface as untrimmed
            }
            if { [lindex $line 0] == "ADD_PAR" } {
                #this seems optional
                set add_par [lrange $line 2 end] ;#ADD_PAR =  2007,   2008,   2009,  2010
                set add_par [string map {, {}} $add_par] ;#replace unwanted commas by spaces
                set line [Carat::GetNextLine $fp]
            } else {
                set add_par [list]
            }
            #ctrl_pts_nodes
            if { [lindex $line 0] != "CTRL_PTS_NODES" } {
                #error
                break
            }
            set ctrl_pts_nodes_id [lindex $line 1]
            set ctrl_points_surface [list]
            set weights_surface [list]
            while { ![eof $fp] } {
                set line [Carat::GetNextLine $fp]
                if { [lindex $line 0] == "NODE_ID" } {
                    incr control_point_count
                    lassign $line dummy node_id dummy x dummy y dummy z dummy weight
                    if { $node_id } {
                        set master_control_point($control_point_count) $node_id
                        lappend ctrl_points_surface $node($node_id)
                        lappend weights_surface $x
                    } else {
                        lappend ctrl_points_surface [list $x $y $z]
                        lappend weights_surface $weight
                    }
                } else {
                    break
                }
            }

            set is_surface_rational 0
            foreach weight $weights_surface {
                if { [expr {abs($weight-1.0)}] > 1e-10 } {
                    set is_surface_rational 1
                    break
                }
            }
            if { $is_surface_rational } {
                set control_points [list]
                foreach item $ctrl_points_surface weight $weights_surface {
                    lappend control_points [list {*}$item $weight]
                }
            } else {
                set control_points $ctrl_points_surface
            }
            #if the surface was untrimmed, create the surface and break the loop:
            if {$istrimmed==0} {
                set cp_line_u_0 [list]
                set cp_line_u_1 [list]
                set cp_line_v_0 [list]
                set cp_line_v_1 [list]
                set total_number_cp [expr $num_control_points_u * $num_control_points_v]
                # for loops will create lists for the control points involved in the boundary lines (u=0, u=1, v=0 and v=1)
                for {set i 0} {$i<$num_control_points_u} {incr i} {
                    lappend cp_line_v_0 [lindex $ctrl_points_surface $i]
                    lappend cp_line_v_1 [lindex $ctrl_points_surface [expr $total_number_cp - $num_control_points_u + $i]]
                }
                for {set i 0} {$i<$total_number_cp} { incr i $num_control_points_u } {
                    lappend cp_line_u_0 [lindex $ctrl_points_surface $i]
                    lappend cp_line_u_1 [lindex $ctrl_points_surface [expr $num_control_points_u + $i -1]]
                }
                #Creation of the corners that will be the beggining and end of the boundaries
                set corner_1 [GiD_Geometry create point append $layer {*}[lindex $cp_line_v_0 0]]
                set corner_2 [GiD_Geometry create point append $layer {*}[lindex $cp_line_v_0 end]]
                set corner_3 [GiD_Geometry create point append $layer {*}[lindex $cp_line_v_1 end]]
                set corner_4 [GiD_Geometry create point append $layer {*}[lindex $cp_line_v_1 0]]
                #Creation of the boundary lines
                set line_v_0 [GiD_Geometry create line append nurbsline $layer $corner_1 $corner_2 $degree_u \
                    [llength $cp_line_v_0] 0 {*}$cp_line_v_0 {*}[Carat::NormalizeKnots $knots_u]]
                set line_u_1 [GiD_Geometry create line append nurbsline $layer $corner_2 $corner_3 $degree_v \
                    [llength $cp_line_u_1] 0 {*}$cp_line_u_1 {*}[Carat::NormalizeKnots $knots_v]]
                set line_v_1 [GiD_Geometry create line append nurbsline $layer $corner_4 $corner_3 $degree_u \
                    [llength $cp_line_v_1] 0 {*}$cp_line_v_1 {*}[Carat::NormalizeKnots $knots_u]]
                set line_u_0 [GiD_Geometry create line append nurbsline $layer $corner_1 $corner_4 $degree_v \
                    [llength $cp_line_u_0] 0 {*}$cp_line_u_0 {*}[Carat::NormalizeKnots $knots_v]]
                #Creation of the surface
                set geometrical_data [list $degree_u $degree_v $num_control_points_u $num_control_points_v $istrimmed $is_surface_rational \
                    {*}$control_points {*}[Carat::NormalizeKnots $knots_u] {*}[Carat::NormalizeKnots $knots_v]]
                set boundary_curves [list [list $line_v_0 0] [list $line_u_1 0] [list $line_v_1 1] [list $line_u_0 1]]
                set gid_surface($surface_id) [GiD_Geometry create surface $surface_id nurbssurface $layer 4\
                    {*}[lrange $geometrical_data 0 5] {*}$boundary_curves {*}[lrange $geometrical_data 6 end]]
                set num_control_points_surface($surface_id) [list $num_control_points_u $num_control_points_v]
                break
            }
            #trimming
            if { [lindex $line 0] != "B_REP" } {
                #error
                break
            }
            set brep_index [lindex $line 1]
            set boundary_loops [list]
            while { ![eof $fp] } {
                set line [Carat::GetNextLine $fp]
                if { [lindex $line 0] == "B_REP_LOOP" } {
                    set outer [lindex $line 2]
                    set loop_curves [list]
                    set loop_versos [list]
                    foreach {curve_id verso} [lrange $line 3 end] {
                        lappend loop_curves $curve_id
                        lappend loop_versos $verso
                    }
                    lappend boundary_loops $loop_curves $loop_versos
                } else {
                    break
                }
            }
            #add_par unused ... ?
            if { [llength $add_par] } {
                #append add_par to boundary_loops to read them
                set add_par_pseudo_versos [lrepeat [llength $add_par] NONE]
                lappend boundary_loops $add_par $add_par_pseudo_versos
            }

            set is_trimmed 1
            set geometrical_data [list $degree_u $degree_v $num_control_points_u $num_control_points_v $is_trimmed $is_surface_rational \
                {*}$control_points {*}[Carat::NormalizeKnots $knots_u] {*}[Carat::NormalizeKnots $knots_v]]
            set gid_surface_untrimmed_id [GiD_Geometry -no_model create surface append nurbssurface "" 0 \
                {*}[lrange $geometrical_data 0 5] {*}[lrange $geometrical_data 6 end]]
            #lines are defined in uv space, not in 3D
            set a_u [lindex $knots_u 0]
            set b_u [lindex $knots_u end]
            set a_v [lindex $knots_v 0]
            set b_v [lindex $knots_v end]

            set boundary_curves [list]
            foreach {loop_curves loop_versos} $boundary_loops {
                set gid_id_point_start 0
                set gid_id_point_connect 0
                set first_curve [lindex $loop_curves 0]
                set last_curve [lindex $loop_curves end]
                foreach curve_id $loop_curves verso $loop_versos {
                    ###################################################################
                    ####                  NURBS-BLOCK-PARAMETER                    ####
                    ###################################################################
                    if { [lindex $line 0] != "NURBS_PATCH_PAR" } {
                        #error
                        break
                    }
                    lassign $line dummy curve_id dummy key
                    if { $key != "NURBS_1D" } {
                        #error
                        break
                    }
                    set line [Carat::GetNextLine $fp]
                    set id_ctrl_pts_ctrl_pts_par [lindex $line 3] ;#CTRL_PTS = CTRL_PTS_PAR 1
                    set line [Carat::GetNextLine $fp]
                    set num_control_points [lindex $line 2] ;#NCTRL =  44
                    incr num_control_points ;#seems that was the num of spans
                    set line [Carat::GetNextLine $fp]
                    set degree [lindex $line 2];#PDEG =   3
                    set line [Carat::GetNextLine $fp]
                    set knots [lrange $line 2 end];#UKNOT =  0.00000000000000e+000, 0.00000000000000e+000, ... , 1.00000000000000e+001
                    set knots [string map {, {}} $knots];#replace unwanted commas by spaces
                    #ctrl_pts_param
                    set line [Carat::GetNextLine $fp]
                    if { [lindex $line 0] != "CTRL_PTS_PAR" } {
                        #error
                        break
                    }
                    set ctrl_pts_par_id [lindex $line 1]
                    set ctrl_points_curve [list]
                    while { ![eof $fp] } {
                        set line [Carat::GetNextLine $fp]
                        if { [lindex $line 0] == "CTRLPT_PAR_ID" } {
                            set ctrlpt_par_id [lindex $line 1]
                            lappend ctrl_points_curve $ctrlpt_par($ctrlpt_par_id)
                        } else {
                            break
                        }
                    }
                    #create the curve? (only boundary_curves not add_par ??)
                    if { $verso == "TRUE" } {
                        #verso true == same1st
                        if { $curve_id == $first_curve } {
                            lassign [lindex $ctrl_points_curve 0] u v dummy
                            set point_3d [GiD_Info parametric -no_model surface $gid_surface_untrimmed_id coord [Carat::NormalizeParameter $a_u $b_u $u] [Carat::NormalizeParameter $a_v $b_v $v]]
                            set gid_point_1 [GiD_Geometry create point append $layer {*}$point_3d]
                            set gid_id_point_start $gid_point_1
                        } else {
                            set gid_point_1 $gid_id_point_connect
                        }
                        if { $curve_id == $last_curve } {
                            set gid_point_2 $gid_id_point_start
                        } else {
                            lassign [lindex $ctrl_points_curve end] u v dummy
                            set point_3d [GiD_Info parametric -no_model surface $gid_surface_untrimmed_id coord [Carat::NormalizeParameter $a_u $b_u $u] [Carat::NormalizeParameter $a_v $b_v $v]]
                            set gid_point_2 [GiD_Geometry create point append $layer {*}$point_3d]
                        }
                        set gid_id_point_connect $gid_point_2
                    } elseif { $verso == "FALSE" } {
                        #verso false == diff1st
                        if { $curve_id == $first_curve } {
                            lassign [lindex $ctrl_points_curve end] u v dummy
                            set point_3d [GiD_Info parametric -no_model surface $gid_surface_untrimmed_id coord [Carat::NormalizeParameter $a_u $b_u $u] [Carat::NormalizeParameter $a_v $b_v $v]]
                            set gid_point_2 [GiD_Geometry create point append $layer {*}$point_3d]
                            set gid_id_point_start $gid_point_2
                        } else {
                            set gid_point_2 $gid_id_point_connect
                        }
                        if { $curve_id == $last_curve } {
                            set gid_point_1 $gid_id_point_start
                        } else {
                            lassign [lindex $ctrl_points_curve 0] u v dummy
                            set point_3d [GiD_Info parametric -no_model surface $gid_surface_untrimmed_id coord [Carat::NormalizeParameter $a_u $b_u $u] [Carat::NormalizeParameter $a_v $b_v $v]]
                            set gid_point_1 [GiD_Geometry create point append $layer {*}$point_3d]
                        }
                        set gid_id_point_connect $gid_point_1
                    } elseif { $verso == "NONE" } {
                        # add_par curves, unused??
                        continue
                    } else {
                        W "Error, unexpected orientation=$verso"
                        break
                    }
                    set is_curve_rational 0 ;#now seems that curves are non rationals...
                    #create temporary the curve in uv
                    set normalized_knots [Carat::NormalizeKnots $knots]
                    set gid_line_uv_id [GiD_Geometry -no_model create line append nurbsline "" 0 0 \
                        $degree $num_control_points $is_curve_rational {*}$ctrl_points_curve {*}$normalized_knots]
                    #create and approximated curve in 3d with num_t interpolation points, could calculate tangents for better shape..
                    set num_t 51 ;#enough points to have o good approximation
                    if { $num_t < $num_control_points } {
                        set num_t $num_control_points
                    }
                    set ts [list]
                    for {set i_t 0} {$i_t <= $num_t} {incr i_t} {
                        set t [expr {double($i_t)/$num_t}]
                        lappend ts $t
                    }
                    #force insertion of knots ts
                    set ts [lsort -real -unique [concat $ts [lrange [lsort -real -unique $normalized_knots] 1 end-1]]]

                    set interpolation_points [list]
                    foreach t $ts {
                        lassign [GiD_Info parametric -no_model line $gid_line_uv_id coord $t] u v dummy
                        set u [Carat::NormalizeParameter $a_u $b_u $u]
                        set v [Carat::NormalizeParameter $a_v $b_v $v]
                        lappend interpolation_points [GiD_Info parametric -no_model surface $gid_surface_untrimmed_id coord $u $v]
                    }
                    GiD_Geometry -no_model delete line $gid_line_uv_id

                    set interpolation_points [Carat::FilterInterpolationPoints $interpolation_points 1e-5]
                    if { [llength $interpolation_points] < 2 } {
                        #degenerated line
                        set gid_line($curve_id) [GiD_Geometry create line append stline $layer $gid_point_1 $gid_point_2]
                    } else {
                        set gid_line($curve_id) [GiD_Geometry create line append nurbsline $layer $gid_point_1 $gid_point_2 -interpolate [llength $interpolation_points] {*}$interpolation_points]
                        # ?-tangents {t0_x t0_y t0_z} {t1_x t1_y t1_z}?
                    }
                    lappend boundary_curves [list $gid_line($curve_id) $gid_verso($verso)]
                }
            }
            #now can create the trimmed surface with the 3D curves
            GiD_Geometry -no_model delete surface $gid_surface_untrimmed_id
            set gid_surface($surface_id) [GiD_Geometry create surface $surface_id nurbssurface $layer [llength $boundary_curves] \
                {*}[lrange $geometrical_data 0 5] {*}$boundary_curves {*}[lrange $geometrical_data 6 end]]
            set num_control_points_surface($surface_id) [list $num_control_points_u $num_control_points_v]
        }
        #uvs of surface 1, 2 and shared?
        array unset coup_point_1
        array unset coup_point_2
        array unset coup_point
        while { ![eof $fp] } {
            set line [Carat::GetNextLine $fp]
            if { [lindex $line 0] == "COUP_POINT_1_GEO" } {
                lassign $line dummy surface_id u v
                lappend coup_point_1($surface_id) [list $u $v]
            } elseif { [lindex $line 0] == "COUP_POINT_2_GEO" } {
                lassign $line dummy surface_id u v
                lappend coup_point_2($surface_id) [list $u $v]
            } elseif { [lindex $line 0] == "COUP_POINT" } {
                lassign $line dummy surface_id u v
                lappend coup_point($surface_id) [list $u $v]
            } else {
                break
            }
        }
    }
    close $fp

    #try to convert out.post.res to our model
    set file_rootname [file rootname [file rootname $filename]] ;#could have double extension (out.georhino.txt)
    set filename_results_in ${file_rootname}.post.res
    if { [file exists $filename_results_in] } {
        set model_name [GiD_Info project modelname]
        if { $model_name != "UNNAMED" } {
            set filename_results_out [file join ${model_name}.gid [file tail $model_name].post.res]
        } else {
            set filename_results_out ${file_rootname}_gid.post.res
        }
        if { ![file exists $filename_results_out] } {
            Carat::ReadResults $filename_results_in $filename_results_out
            GidUtils::SetWarnLine [_ "Results converted to %s" $filename_results_out]
        }
    }
    GiD_RaiseEvent GiD_Event_AfterOpenFile $filename CARAT_GEOMETRY_FORMAT $fail
    return $fail
}


proc Carat::ReadPre0 { filenames } {
    variable percent_done
    variable stop
    set fail 0
    set ::Carat::stop 0
    set ::Carat::percent_done 0
    set percent_max [llength $filenames]
    ::GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::Carat::percent_done $percent_max {set ::Carat::stop 1}
    foreach filename $filenames {
        if { $::Carat::stop } {
            break
        }
        if { [file exists $filename] } {
            set res ""
            if { [catch { set local_fail [Carat::ReadPreGeometry $filename] } err options] } {
                #set fail 1
                #W $err
                GidUtils::DeleteAdvanceBar
                return -options $options $err
            } else {
                if { $local_fail } {
                    set fail 1
                }
            }
        }
        incr ::Carat::percent_done
    }
    GidUtils::DeleteAdvanceBar
    return $fail
}

#read and finally redraw showing what is read
proc Carat::ReadPre { filenames } {
    GidUtils::WaitState .gid
    GidUtils::DisableGraphics
    if { [catch { set fail [Carat::ReadPre0 $filenames] } msg] } {
        set fail 1
        WarnWin "$msg $::errorInfo"
    } else {
        if { [GiD_Set AutoCollapseAfterImport] } {
            if { [GiD_Set AutoImportTolerance] } {
                #in our tests is necessary to increase x2 the tolerance to collapse lines
                set prev_tolerance [GiD_Set ImportTolerance]
                set bounding_box [lindex [GiD_Info layers -bbox -use geometry] 0]
                set d [MathUtils::VectorDistance [lrange $bounding_box 0 2] [lrange $bounding_box 3 5]]
                set tolerance [expr {$d*1e-3}]
                GiD_Set AutoImportTolerance 0
                GiD_Set ImportTolerance $tolerance
                set changed_auto_tolerance 1
            } else {
                set changed_auto_tolerance 0
            }
            GiD_Process Mescape Utilities Collapse Model Yes escape
            if { $changed_auto_tolerance } {
                #restore values
                GiD_Set AutoImportTolerance 1
                GiD_Set ImportTolerance $prev_tolerance
            }
        }
    }
    GidUtils::EnableGraphics
    GidUtils::EndWaitState .gid
    if { $fail } {
        GidUtils::SetWarnLine [_ "Error reading geometry"]
    } else {
        GiD_Process 'Zoom Frame escape
        GidUtils::UpdateWindow GROUPS
        GidUtils::SetWarnLine [_ "Geometry read"]
    }
    return
}

#to drag and drop a file
proc Carat::ReadPreSingleFile { filename } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch Carat::ReadPre [list $filename]
}

#to add more options to the standar file browser
proc Carat::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkmerge -text [_ "Merge"] -variable Carat::_merge
    grid $f.chkmerge -sticky w -columnspan 2
    grid columnconfigure $f 1 -weight 1

    variable import_dir
    if { ![info exists import_dir] } {
        #trick to show the more options
        set import_dir open
    }
    return Carat::import_dir
}

proc Carat::ReadPreW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read Carat geometry"] \
        {} [list [list {Carat} {.georhino.txt}] [list [_ "All files"] {.*}]] {} $multiple \
        [list [_ "Import options"] Carat::MoreImportOptions]]
    if {$filenames == ""} {
        return
    }
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch Carat::ReadPre $filenames
}

#########################################################################################
######################################### READ POST #####################################
#########################################################################################


proc Carat::ReadResults { filename_in filename_out } {
    variable num_control_points_surface ;#filled by Carat::ReadPreGeometry, it is necessary to assign values to each surface
    variable master_control_point  ;#filled by Carat::ReadPreGeometry, used to copy missing results of other master if it exists
    set fail 0
    if { ![file exists $filename_in] } {
        set fail 1
        return $fail
    }
    set fp [open $filename_in r]
    set line [string trim [gets $fp]]
    if { $line != "Rhino Post Results File 1.0" } {
        #wrong input file
        close $fp
        set fail 1
        return $fail
    }

    set outfile [open $filename_out w]
    puts $outfile "GiD Post Results File 1.2"
    puts $outfile ""

    set result_header ""
    set result_dimension 0
    while {![eof $fp]} {
        set line [string trim [gets $fp]]
        if { $line == "" || [string index $line 0] == "#" } {
            continue
        }
        if { [lindex $line 0] == "Result" } {
            if {[lsearch $line "OnNodes"] != -1 } {
                #Replace word OnNodes by OnNurbsSurface
                regsub OnNodes $line OnNurbsSurface result_header
                if { [lsearch $line "Scalar"] != -1 } {
                    set result_dimension 1
                } elseif { [lsearch $line "Vector"] != -1 } {
                    set result_dimension 3
                } elseif { [lsearch $line "Matrix"] != -1 } {
                    set result_dimension 6
                } else {
                    set result_dimension 0
                }
            }
        } elseif { $line == "Values" } {
            while { ![eof $fp] } {
                set line [string trim [gets $fp]]
                if { $line == "End Values" } {
                    puts $outfile $result_header
                    puts $outfile "Values"
                    set control_point_id 0
                    foreach surface_id [lsort -integer [array names num_control_points_surface]] {
                        lassign $num_control_points_surface($surface_id) num_u num_v
                        set num_control_points_local [expr {$num_u*$num_v}]
                        set num_values [expr {$num_control_points_local*$result_dimension}]
                        puts $outfile $surface_id
                        for {set i_u 0} {$i_u < $num_u} {incr i_u} {
                            for {set i_v 0} {$i_v < $num_v} { incr i_v } {
                                incr control_point_id
                                if { [info exists values($control_point_id)] } {
                                    puts $outfile $values($control_point_id)
                                } else {
                                    if { [info exists master_control_point($control_point_id)] } {
                                        #get the result of the master control point
                                        set control_point_master_id $master_control_point($control_point_id)
                                        puts $outfile $values($control_point_master_id)
                                    } else {
                                        puts $outfile [lrepeat $result_dimension NR]
                                    }
                                }
                            }
                        }
                    }
                    puts $outfile "End Values"
                    array unset values
                    set result_header ""
                    set result_dimension 0
                    break
                } else {
                    set control_point_id [lindex $line 0]
                    set values($control_point_id) [lrange $line 1 end]
                }
            }
        } else {

        }
    }

    close $outfile
    close $fp
    array unset num_control_points_surface
    array unset master_control_point
    return $fail
}

#########################################################################################
######################################### OTHER #########################################
#########################################################################################

proc Carat::AddToMenuPRE { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "Carat..."] PRE] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Import->XYZ ascii points"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ points"] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import "Carat..."] $position PRE Carat::ReadPreW "" "" insertafter _
}


proc Carat::AddToMenu { } {
    Carat::AddToMenuPRE
}

#invoke this menus changes
Carat::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc Carat::AddToMenu

#register the proc to be automatically called when dropping a file
GiD_RegisterExtensionProc ".georhino.txt" PRE Carat::ReadPreSingleFile

#Carat::ReadPre0 {{C:\gid project more\Proyectos\IGA (Iso Geometrical Analysis)\NURBS results\R1\out.georhino.txt}}
