#Collada read plugin

#########################################################################################
########################################### READ ########################################
#########################################################################################


namespace eval Collada {
    variable info_plugin ;#information read from .xml plugin file
    variable callback_mesh_triangles ""
    variable stop
    variable percent_done
    variable _as_mesh 1 ;#0 to create a visual opengl draw with textures
    variable _merge  1 ;#import options: merge to current model or delete old model
    variable _show_advance_bar 1
    variable opengl_lists
    variable bounding_box_model

    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

#callback prototype
proc Collada::SetCallbackMeshTriangles { callback } {
    variable callback_mesh_triangles
    set callback_mesh_triangles $callback
}

proc Collada::GetFilenameTexture { dom_root filename } {
    #must find the filename in a more elegant way, by the material,...
    #now i am assuming to use the last image
    set filename_texture ""
    foreach dom_library_images [$dom_root getElementsByTagName "library_images"] {
        foreach dom_image [$dom_library_images getElementsByTagName "image"] {
            set dom_init_from [$dom_image getElementsByTagName "init_from"]
            set init_from [$dom_init_from text]
            if { [string range $init_from 0 6] == "file://" } {
                set filename_texture [string range $init_from 7 end]
            } else {
                set filename_texture [file join [file dirname $filename] $init_from]
            }

        }
    }
    return $filename_texture
}

#use [list {*} ] because objarray only expect values separed by a single space, without spaces at start or end
#and also without endlines as separator
proc Collada::GetFloatArrayFromDomNode { dom_node } {
    return [objarray new floatarray -values [list {*}[$dom_node text]]]
}

proc Collada::GetIntArrayFromDomNode { dom_node } {
    return [objarray new intarray -values [list {*}[$dom_node text]]]
}

proc Collada::ApplyMatrix { matrix node_coordinates } {
    #apply matrix to node_coordinates and rotation to normals...
    set rotation [list {*}[lrange $matrix 0 2] {*}[lrange $matrix 4 6] {*}[lrange $matrix 8 10]]
    set translation [list [lindex $matrix 3] [lindex $matrix 7] [lindex $matrix 11]]
    set scale [lindex $matrix 15]
    objarray rotate $node_coordinates $rotation
    objarray translate $node_coordinates $translation
    objarray scale $node_coordinates $scale
    return 0
}

proc Collada::ReadGeometry_MeshTriangles { dom_root filename geometry_id dom_mesh matrix node_ids } {
    variable callback_mesh_triangles
    foreach dom_triangles [$dom_mesh getElementsByTagName "triangles"] {
        set node_coordinates ""
        set elements ""
        set normals ""
        set texcoords ""
        set colors ""
        set count_triangles [$dom_triangles getAttribute "count"]
        #set material [$dom_triangles getAttribute "material"]
        set num_components 0
        foreach dom_input [$dom_triangles getElementsByTagName "input"] {
            set semantic_triangles_input [$dom_input getAttribute "semantic"]
            set source_id [string range [$dom_input getAttribute "source"] 1 end]
            set offset($semantic_triangles_input) [$dom_input getAttribute "offset"]
            if { $semantic_triangles_input == "VERTEX" } {
                set dom_vertices [$dom_mesh selectNodes "x:vertices\[@id='$source_id'\]"]
                foreach dom_input [$dom_vertices getElementsByTagName "input"] {
                    set semantic_vertex_input [$dom_input getAttribute "semantic"]
                    set source_id [string range [$dom_input getAttribute "source"] 1 end]
                    set dom_source [$dom_mesh selectNodes "x:source\[@id='$source_id'\]"]
                    set dom_float_array [$dom_source getElementsByTagName "float_array"]
                    set count_floats [$dom_float_array getAttribute "count"]
                    if { $semantic_vertex_input == "POSITION" } {
                        set node_coordinates [Collada::GetFloatArrayFromDomNode $dom_float_array]
                    } elseif { $semantic_vertex_input == "NORMAL" } {
                        set normals [Collada::GetFloatArrayFromDomNode $dom_float_array]
                    } elseif { $semantic_vertex_input == "COLOR" } {
                        set colors [Collada::GetFloatArrayFromDomNode $dom_float_array]
                    } else {
                        W "Collada::ReadGeometry_MeshTriangles. unexpected semantic_vertex_input=$semantic_vertex_input"
                    }
                }
            } elseif { $semantic_triangles_input == "TEXCOORD" } {
                set dom_source [$dom_mesh selectNodes "x:source\[@id='$source_id'\]"]
                set dom_float_array [$dom_source getElementsByTagName "float_array"]
                set count_texcoord [$dom_float_array getAttribute "count"]
                set texcoords [Collada::GetFloatArrayFromDomNode $dom_float_array]
            } elseif { $semantic_triangles_input == "NORMAL" } {
                set dom_source [$dom_mesh selectNodes "x:source\[@id='$source_id'\]"]
                set dom_float_array [$dom_source getElementsByTagName "float_array"]
                set count_normals [$dom_float_array getAttribute "count"]
                set normals [Collada::GetFloatArrayFromDomNode $dom_float_array]
            } elseif { $semantic_triangles_input == "COLOR" } {
                set dom_source [$dom_mesh selectNodes "x:source\[@id='$source_id'\]"]
                set dom_float_array [$dom_source getElementsByTagName "float_array"]
                set count_colors [$dom_float_array getAttribute "count"]
                set colors [Collada::GetFloatArrayFromDomNode $dom_float_array]
            } else {
                W "Collada::ReadGeometry_MeshTriangles. unexpected semantic_triangles_input=$semantic_triangles_input"
            }
            incr num_components
        }

        set dom_p [$dom_triangles getElementsByTagName "p"]
        set data [Collada::GetIntArrayFromDomNode $dom_p]
        set num_data [objarray length $data]
        set num_data_block [expr $num_data/$num_components]
        objarray deinterleave $data $num_components
        for {set i 0} {$i<$num_components} {incr i} {
            set begin [expr $i*$num_data_block]
            set end [expr ($i+1)*$num_data_block-1]
            set block($i) [objarray range $data $begin $end]
        }
        set element_connectivities $block($offset(VERTEX))
        if { [info exists offset(TEXCOORD)] && [info exists block($offset(TEXCOORD))] } {
            set element_textures_indices $block($offset(TEXCOORD))
        } else {
            set element_textures_indices [objarray new intarray 0]
        }
        if { [info exists offset(COLOR)] && [info exists block($offset(COLOR))] } {
            set element_color_indices $block($offset(COLOR))
        } else {
            set element_color_indices [objarray new intarray 0]
        }
        #must find the filename in a more elegant way, by the material, now i am assuming to use the last image
        set filename_texture [Collada::GetFilenameTexture $dom_root $filename]
        if { $matrix != "" } {
            Collada::ApplyMatrix $matrix $node_coordinates
        }
        set name $geometry_id
        #set name [join $node_ids _]
        #W "name=$name node_ids=$node_ids"
        $callback_mesh_triangles $name $node_coordinates $element_connectivities $texcoords $element_textures_indices $filename_texture $colors $element_color_indices
    }
    return 0
}

proc Collada::ReadGeometry_MeshPolylist { dom_root filename geometry_id dom_mesh matrix node_ids } {
    variable callback_mesh_triangles
    foreach dom_polylist [$dom_mesh getElementsByTagName "polylist"] {
        set node_coordinates ""
        set elements ""
        set normals ""
        set texcoords ""
        set colors ""
        set count_polylist [$dom_polylist getAttribute "count"]
        #set material [$dom_polylist getAttribute "material"]
        set num_components 0
        foreach dom_input [$dom_polylist getElementsByTagName "input"] {
            set semantic_polylist_input [$dom_input getAttribute "semantic"]
            set source_id [string range [$dom_input getAttribute "source"] 1 end]
            set offset($semantic_polylist_input) [$dom_input getAttribute "offset"]
            if { $semantic_polylist_input == "VERTEX" } {
                set dom_vertices [$dom_mesh selectNodes "x:vertices\[@id='$source_id'\]"]
                foreach dom_input [$dom_vertices getElementsByTagName "input"] {
                    set semantic_vertex_input [$dom_input getAttribute "semantic"]
                    set source_id [string range [$dom_input getAttribute "source"] 1 end]
                    set dom_source [$dom_mesh selectNodes "x:source\[@id='$source_id'\]"]
                    set dom_float_array [$dom_source getElementsByTagName "float_array"]
                    set count_floats [$dom_float_array getAttribute "count"]
                    if { $semantic_vertex_input == "POSITION" } {
                        set node_coordinates [Collada::GetFloatArrayFromDomNode $dom_float_array]
                    } elseif { $semantic_vertex_input == "NORMAL" } {
                        set normals [Collada::GetFloatArrayFromDomNode $dom_float_array]
                    } elseif { $semantic_vertex_input == "COLOR" } {
                        set colors [Collada::GetFloatArrayFromDomNode $dom_float_array]
                    } else {
                        W "Collada::ReadGeometry_MeshPolylist. unexpected semantic_vertex_input=$semantic_vertex_input"
                    }
                }
            } elseif { $semantic_polylist_input == "TEXCOORD" } {
                set dom_source [$dom_mesh selectNodes "x:source\[@id='$source_id'\]"]
                set dom_float_array [$dom_source getElementsByTagName "float_array"]
                set count_texcoord [$dom_float_array getAttribute "count"]
                set texcoords [Collada::GetFloatArrayFromDomNode $dom_float_array]
            } elseif { $semantic_polylist_input == "NORMAL" } {
                set dom_source [$dom_mesh selectNodes "x:source\[@id='$source_id'\]"]
                set dom_float_array [$dom_source getElementsByTagName "float_array"]
                set count_normals [$dom_float_array getAttribute "count"]
                set normals [Collada::GetFloatArrayFromDomNode $dom_float_array]
            } elseif { $semantic_triangles_input == "COLOR" } {
                set dom_source [$dom_mesh selectNodes "x:source\[@id='$source_id'\]"]
                set dom_float_array [$dom_source getElementsByTagName "float_array"]
                set count_colors [$dom_float_array getAttribute "count"]
                set colors [Collada::GetFloatArrayFromDomNode $dom_float_array]
            } else {
                W "Collada::ReadGeometry_MeshPolylist. unexpected semantic_polylist_input=$semantic_polylist_input"
            }
            incr num_components
        }

        set dom_p [$dom_polylist getElementsByTagName "p"]
        set data [Collada::GetIntArrayFromDomNode $dom_p]
        set num_data [objarray length $data]
        set num_data_block [expr $num_data/$num_components]
        objarray deinterleave $data $num_components
        for {set i 0} {$i<$num_components} {incr i} {
            set begin [expr $i*$num_data_block]
            set end [expr ($i+1)*$num_data_block-1]
            set block($i) [objarray range $data $begin $end]
        }
        set element_connectivities $block($offset(VERTEX))
        if { [info exists offset(TEXCOORD)] && [info exists block($offset(TEXCOORD))] } {
            set element_textures_indices $block($offset(TEXCOORD))
        } else {
            set element_textures_indices [objarray new intarray 0]
        }
        if { [info exists offset(COLOR)] && [info exists block($offset(COLOR))] } {
            set element_color_indices $block($offset(COLOR))
        } else {
            set element_color_indices [objarray new intarray 0]
        }

        #must find the filename in a more elegant way, by the material, now i am assuming to use the last image
        set filename_texture [Collada::GetFilenameTexture $dom_root $filename]

        set dom_vcount [$dom_polylist getElementsByTagName "vcount"]
        set polygons_nnodes [Collada::GetIntArrayFromDomNode $dom_vcount]
        #now only valid if all polygons are really triangles (3 nodes), else must be triangulated...
        if { $matrix != "" } {
            Collada::ApplyMatrix $matrix $node_coordinates
        }
        if { [objarray minimum $polygons_nnodes] == 3 &&  [objarray maximum $polygons_nnodes] == 3 } {
            $callback_mesh_triangles $geometry_id $node_coordinates $element_connectivities $texcoords $element_textures_indices $filename_texture $colors $element_color_indices
        }
    }
    return 0
}

proc Collada::ReadGeometry_Node { dom_root filename dom_node matrix node_ids } {
    set fail 0
    set geometry_ids [list]
    foreach child_node [$dom_node childNodes] {
        set name [$child_node nodeName]
        if { $name == "node" } {
            if { [$child_node hasAttribute "id"] } {
                set node_id [$child_node getAttribute "id"]
                lappend node_ids $node_id
            }
            Collada::ReadGeometry_Node $dom_root $filename $child_node $matrix $node_ids
        } elseif { $name == "translate" } {
            set to_stop 1
        } elseif { $name == "rotate" } {
            set to_stop 1
        } elseif { $name == "scale" } {
            set to_stop 1
        } elseif { $name == "matrix" } {
            set matrix [Collada::GetFloatArrayFromDomNode $child_node]
        } elseif { $name == "lookat" } {
            set to_stop 1
        } elseif { $name == "instance_geometry" } {
            lappend geometry_ids [string range [$child_node getAttribute "url"] 1 end]
        } elseif { $name == "instance_camera" } {
            set to_stop 1
        } elseif { $name == "instance_light" } {
            set to_stop 1
        } elseif { $name == "instance_controller" } {
            set controller_id [string range [$child_node getAttribute "url"] 1 end]
            set dom_controller [$dom_root selectNodes "x:library_controllers/x:controller\[@id='$controller_id'\]"]
            foreach dom_skin [$dom_controller getElementsByTagName "skin"] {
                lappend geometry_ids [string range [$dom_skin getAttribute "source"] 1 end]
            }
        } elseif { $name == "instance_node" } {
            set node_id [string range [$child_node getAttribute "url"] 1 end]
            set dom_node [$dom_root selectNodes "x:library_nodes/x:node\[@id='$node_id'\]"]
            lappend node_ids $node_id
            Collada::ReadGeometry_Node $dom_root $filename $dom_node $matrix $node_ids
        } elseif { $name == "extra" } {
            set to_stop 1
        } elseif { $name == "asset" } {
            set to_stop 1
        } else {
            W "Unknown scene node $name"
        }
    }
    foreach geometry_id $geometry_ids {
        set dom_geometry [$dom_root selectNodes "x:library_geometries/x:geometry\[@id='$geometry_id'\]"]
        set dom_mesh [$dom_geometry getElementsByTagName "mesh"]
        Collada::ReadGeometry_MeshTriangles $dom_root $filename $geometry_id $dom_mesh $matrix $node_ids
        Collada::ReadGeometry_MeshPolylist $dom_root $filename $geometry_id $dom_mesh $matrix $node_ids
    }
    return $fail
}

proc Collada::ReadGeometry { dom_root filename } {
    set fail 0
    set dom_instance_visual_scene [$dom_root selectNodes "x:scene/x:instance_visual_scene"]
    if { $dom_instance_visual_scene != "" } {
        set visual_scene_id [string range [$dom_instance_visual_scene getAttribute "url"] 1 end]
        set dom_visual_scene [$dom_root selectNodes "x:library_visual_scenes/x:visual_scene\[@id='$visual_scene_id'\]"]
        set dom_nodes [$dom_visual_scene selectNodes "x:node"]
        foreach dom_node $dom_nodes {
            set matrix ""
            set node_ids [list]
            set fail [Collada::ReadGeometry_Node $dom_root $filename $dom_node $matrix $node_ids]
        }

    }
    return $fail
}

#to drag and drop a file
proc Collada::ReadPreSingleFile { filename } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch Collada::ReadPre [list $filename]
}

proc Collada::ReadPre { filenames } {
    set fail 0
    variable _as_mesh
    GidUtils::WaitState
    #GidUtils::DisableGraphics
    if { [catch { set fail [Collada::ReadPre0 $filenames] } err options] } {
        set fail 1
    }
    #GidUtils::EnableGraphics
    GidUtils::EndWaitState
    if { $fail } {
        return -options $options $err
    } else {
        if { $_as_mesh } {
            set flag_pre_post -pre
            if { [GetCurrentPrePostMode] == "PRE" } {
                set flag_pre_post -pre
                MeshView
                GidUtils::UpdateWindow LAYER
            } else {
                set flag_pre_post -post
                GidUtils::UpdateWindow SETS
            }
            GiD_RaiseEvent GiD_Event_AfterChangeMesh [GiD_Info Mesh $flag_pre_post NumNodes] [GiD_Info Mesh $flag_pre_post NumElements]
        } else {
            Collada::BeginMyDraw
        }
        GiD_Process 'Zoom Frame
        GidUtils::SetWarnLine [_ "Collada read"]
    }
    return $fail
}

#read and finally redraw showing what is read
proc Collada::ReadPre0 { filenames } {
    variable percent_done
    variable stop
    variable _show_advance_bar
    variable _as_mesh

    package require tdom
    package require objarray

    if { $_as_mesh } {
        if { [GetCurrentPrePostMode] == "PRE" } {
            Collada::SetCallbackMeshTriangles Collada::CallbackCreateGiDMeshTrianglesPre
        } else {
            Collada::SetCallbackMeshTriangles Collada::CallbackCreateGiDMeshTrianglesPost
        }
    } else {
        Collada::EndMyDraw
        Collada::SetCallbackMeshTriangles Collada::CallbackCreateMyDraw
    }

    set fail 0
    set ::Collada::stop 0
    set ::Collada::percent_done 0
    set percent_max [llength $filenames]
    if { $_show_advance_bar } {
        GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::Collada::percent_done $percent_max {set ::Collada::stop 1}
    }
    set create_layer_filename 0
    if { 0 } {
        #now Collada is sending parts to layers, probably is better assign to groups, and use layer for filenames...
        if { [llength $filenames]>1 } {
            set create_layer_filename 1
        }
    }
    foreach filename $filenames {
        if { $::Collada::stop } {
            break
        }
        if { $create_layer_filename } {
            set layer [file rootname [file tail $filename]]
            if { ![GiD_Layers exists $layer] } {
                GiD_Layers create $layer
            }
            GiD_Layers edit to_use $layer
        }
        set xml [tDOM::xmlReadFile $filename]
        set doc [dom parse $xml]
        set dom_root [$doc documentElement]
        if { [$dom_root nodeName] != "COLLADA" } {
            W [_ "'%s' Is not a Collada file" $filename]
            set fail 1
            continue
        } else {
            #set ns {"http://www.collada.org/2005/11/COLLADASchema"}
            set xmlns [$dom_root getAttribute xmlns]
            #to avoid specify -namespaces in selectNodes
            #but in any case must prefix in selectNodes nodenames with x:
            $doc selectNodesNamespaces [list x $xmlns]
            set stop 0
            incr fail [Collada::ReadGeometry $dom_root $filename]
        }
        $doc delete
        GiD_RaiseEvent GiD_Event_AfterOpenFile $filename Collada_FORMAT 0
        incr ::Collada::percent_done
    }
    if { $_show_advance_bar } {
        GidUtils::DeleteAdvanceBar
    }
    return $fail
}

#########################################################################################
#################################### DIALOG WINDOW ######################################
#########################################################################################

#to add more options to the standar file browser
proc Collada::MoreImportOptions { f } {
    variable _as_mesh
    variable _merge
    ttk::checkbutton $f.chkGeometry -text [_ "As mesh"] -variable Collada::_as_mesh
    GidHelp $f.chkGeometry [_ "If 'As mesh' then is imported as a normal mesh, otherwise is only visualized and with textures"]
    grid $f.chkGeometry -sticky w -columnspan 2
    ttk::checkbutton $f.chkmerge -text [_ "Merge"] -variable Collada::_merge
    grid $f.chkmerge -sticky w -columnspan 2
    grid columnconfigure $f 1 -weight 1

    variable import_dir
    if { ![info exists import_dir] } {
        #trick to show the more options
        set import_dir open
    }
    return Collada::import_dir
}

proc Collada::ReadPreW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read Collada file"] \
        {} {{{Collada} {.dae}} {{All files} {.*}}} {} $multiple \
        [list [_ "Import options"] Collada::MoreImportOptions]]
    if { [llength $filenames] } {
        if { !$_merge } {
            GiD_Process Mescape Files New No escape ;#to avoid previous geometry
        }
        GidUtils::EvalAndEnterInBatch Collada::ReadPre $filenames
    }
}

#########################################################################################
###################################### CALLBACKS ########################################
#########################################################################################
#callback to be called when a mesh is readed from collada file
proc Collada::CallbackCreateGiDMeshTrianglesPre { geometry_id node_coordinates element_connectivities texcoords element_textures_indices filename_texture colors element_color_indices } {
    set element_type triangle
    set element_num_nodes 3
    set node_ids ""
    set element_ids ""
    set zero_based_array 1
    #set layer_name [GiD_Layers get to_use]
    set layer_name $geometry_id
    if { ![GiD_Layers exists $layer_name] } {
        GiD_Layers create $layer_name
    }
    #GiD_Layers edit to_use $layer_name
    GiD_MeshPre create $layer_name $element_type $element_num_nodes -zero_based_array $node_ids $node_coordinates $element_ids $element_connectivities
}

#callback to be called when a mesh is readed from collada file
proc Collada::CallbackCreateGiDMeshTrianglesPost { geometry_id node_coordinates element_connectivities texcoords element_textures_indices filename_texture colors element_color_indices } {
    set element_type triangle
    set element_num_nodes 3
    set node_ids ""
    set element_ids ""
    set mesh_name $geometry_id
    set offset_nodes [GiD_Info Mesh -post MaxNumNodes]
    if { $offset_nodes } {
        set num_nodes_local [expr [objarray length $node_coordinates]/3]
        set node_ids [objarray new_from_to intarray $offset_nodes [expr $offset_nodes+$num_nodes_local-1]]
        objarray incr $element_connectivities $offset_nodes
    }
    GiD_MeshPost create $mesh_name $element_type $element_num_nodes -zero_based_array $node_ids $node_coordinates $element_ids $element_connectivities
}

#callback to be called when a mesh is readed from collada file
proc Collada::CallbackCreateMyDraw { geometry_id node_coordinates element_connectivities texcoords element_textures_indices filename_texture colors element_color_indices } {
    variable opengl_lists
    variable bounding_box_model
    package require gid_draw_opengl
    if { $filename_texture != "" } {
        set drawing_procedure [list gid_draw_opengl::draw_mesh_textures triangles $node_coordinates $element_connectivities $texcoords $element_textures_indices $filename_texture]
    } elseif { [objarray length $colors] } {
        if { [objarray length $element_color_indices] } {
            set drawing_procedure [list gid_draw_opengl::draw_mesh_colors_by_elements_vertex triangles $node_coordinates $element_connectivities $colors $element_color_indices]
        } else {
            set drawing_procedure [list gid_draw_opengl::draw_mesh_colors_by_vertex triangles $node_coordinates $element_connectivities $colors]
        }
    } else {
        W "Collada::CallbackCreateMyDraw. how to draw: draw_mesh_textures, draw_mesh_colors_by_vertex, draw_mesh_colors_by_elements_vertex?"
    }
    set transform_matrix ""
    lappend opengl_lists [gid_draw_opengl::create_opengl_list_drawing $drawing_procedure $transform_matrix]
    set bounding_box [GidUtils::BoundingBoxCoordinates $node_coordinates]
    if { [info exists bounding_box_model] } {
        set bounding_box_model [GidUtils::BoundingBoxJoinBoundingBox $bounding_box $bounding_box_model]
    } else {
        set bounding_box_model $bounding_box
    }

}

proc Collada::EndMyDraw { } {
    variable opengl_lists
    variable bounding_box_model
    GiD_UnRegisterEvents PLUGIN Collada
    if { [info exist opengl_lists] } {
        foreach opengl_list $opengl_lists {
            gid_draw_opengl::delete_opengl_list_drawing $opengl_list
        }
        unset opengl_lists
    }
    unset -nocomplain bounding_box_model
}

proc Collada::GiD_Implement_Event_AfterDrawModel {} {
    variable opengl_lists
    if { [info exist opengl_lists] } {
        foreach opengl_list $opengl_lists {
            GiD_OpenGL draw -call $opengl_list
        }
    }
}

proc Collada::GiD_Implement_Event_GetBoundingBox { useof } {
    variable bounding_box_model
    if { [info exists bounding_box_model] } {
        return $bounding_box_model
    }
    return ""
}

proc Collada::GiD_Implement_Event_AfterNewGIDProject { } {
    Collada::EndMyDraw
}

proc Collada::BeginMyDraw { } {
    GiD_RegisterEvent GiD_Event_AfterDrawModel Collada::GiD_Implement_Event_AfterDrawModel PLUGIN Collada
    GiD_RegisterEvent GiD_Event_GetBoundingBox Collada::GiD_Implement_Event_GetBoundingBox PLUGIN Collada
    GiD_RegisterEvent GiD_Event_AfterNewGIDProject Collada::GiD_Implement_Event_AfterNewGIDProject PLUGIN Collada
}



#####################################################################################
##################################### OTHER #########################################
#####################################################################################

proc Collada::AddToMenuPRE { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "Collada..."] PRE] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import "Collada..."] $position PRE Collada::ReadPreW "" "" insertafter _
}

proc Collada::AddToMenuPOST { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "Collada..."] POST] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] POST]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import "Collada..."] $position POST Collada::ReadPreW "" "" insertafter _
}

proc Collada::AddToMenu { } {
    Collada::AddToMenuPRE
    Collada::AddToMenuPOST
}

#invoke this menus changes
Collada::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc Collada::AddToMenu

#register the proc to be automatically called when dropping a file
foreach extension {.dae} {
    GiD_RegisterExtensionProc $extension PRE Collada::ReadPreSingleFile
}


