#Gcode read plugin

#########################################################################################
########################################### READ ########################################
#########################################################################################


namespace eval Gcode {
    variable info_plugin ;#information read from .xml plugin file
    variable percent_done
    variable stop
    variable _merge  1 ;#import options: merge to current model or delete old model
    variable _show_advance_bar 1

    variable layer_by_z 1 ;#1 to create layers based on z
    variable current
    variable units mm ;#default mm

    variable relative_position 0;#bool
    variable relative_extrusion 0 ;#bool

    variable parameters ;#possible parameters from 1 to 5400

    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

proc Gcode::Init { } {
    variable current
    variable parameters
    variable units

    variable relative_position
    variable relative_extrusion
    set relative_extrusion 0 ;#bool
    set relative_position 0 ;#bool

    set units mm ;#default mm

    array set current {
        X 0.0
        Y 0.0
        Z 0.0
        E 0.0
        F 0.0
    }

    #possible parameters from 1 to 5400
    array set parameters {
        5161 0
        5162 0
        5163 0
        5164 0
        5165 0
        5166 0
        5181 0
        5182 0
        5183 0
        5184 0
        5185 0
        5186 0
        5211 0
        5212 0
        5213 0
        5214 0
        5215 0
        5216 0
        5220 1
        5221 0
        5222 0
        5223 0
        5224 0
        5225 0
        5226 0
        5241 0
        5242 0
        5243 0
        5244 0
        5245 0
        5246 0
        5261 0
        5262 0
        5263 0
        5264 0
        5265 0
        5266 0
        5281 0
        5282 0
        5283 0
        5284 0
        5285 0
        5286 0
        5301 0
        5302 0
        5303 0
        5304 0
        5305 0
        5306 0
        5321 0
        5322 0
        5323 0
        5324 0
        5325 0
        5326 0
        5341 0
        5342 0
        5343 0
        5344 0
        5345 0
        5346 0
        5361 0
        5362 0
        5363 0
        5364 0
        5365 0
        5366 0
        5381 0
        5382 0
        5383 0
        5384 0
        5385 0
        5386 0
    }
}

#to drag and drop a file
proc Gcode::ReadPreSingleFile { filename } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
}

proc Gcode::ReadPre { filenames } {
    set fail 0
    GidUtils::WaitState
    GidUtils::DisableGraphics
    if { [catch { set fail [Gcode::ReadPre0 $filenames] } err options] } {
        set fail 1
    }
    GidUtils::EnableGraphics
    GidUtils::EndWaitState
    if { $fail } {
        return -options $options $err
    } else {
        GidUtils::UpdateWindow LAYER
        GiD_Process 'Zoom Frame
        GidUtils::SetWarnLine [_ "Gcode read"]
    }
    return $fail
}

#read and finally redraw showing what is read
proc Gcode::ReadPre0 { filenames } {
    variable _current_line
    variable percent_done
    variable stop
    variable _show_advance_bar

    set fail 0
    set ::Gcode::stop 0
    set ::Gcode::percent_done 0
    if { 0 } {
        #by now use the existent advancebar of each file
        set percent_max [llength $filenames]
        if { $_show_advance_bar } {
            GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::Gcode::percent_done $percent_max {set ::Gcode::stop 1}
        }
    }
    set create_layer_filename 0
    if { 0 } {
        #by now use the layers of the file
        if { [llength $filenames]>1 } {
            set create_layer_filename 1
        }
    }
    foreach filename $filenames {
        if { $::Gcode::stop } {
            break
        }
        if { $create_layer_filename } {
            set layer [file rootname [file tail $filename]]
            if { ![GiD_Layers exists $layer] } {
                GiD_Layers create $layer
            }
            GiD_Layers edit to_use $layer
        }
        set Gcode::_current_line [list 0 ""]
        set err [catch { set fail [Gcode::ReadGeometry $filename] } err_txt]
        if { $err } {
            if { $_show_advance_bar } {
                #on error must delete bar!!
                GidUtils::DeleteAdvanceBar
            }
            W "$err_txt , line $Gcode::_current_line"
            return -code error $err_txt
        } else {
            if { $fail } {
                WarnWin [_ "User stopped"]
            }
        }
        GiD_RaiseEvent GiD_Event_AfterOpenFile $filename Gcode_FORMAT 0
        if { 0 } {
            incr ::Gcode::percent_done
        }
    }
    if { 0 } {
        #by now use the existent advancebar of each file
        if { $_show_advance_bar } {
            GidUtils::DeleteAdvanceBar
        }
    }
    return $fail
}

proc Gcode::RemoveParenthesisBlocks { line } {
    set pos_1 [string first "(" $line 0]
    while { $pos_1 != -1 } {
        set pos_2 [string first ")" $line $pos_1]
        if { $pos_2 == -1 } {
            error "unbalanced () in $line"
        }
        set line [string trim [concat [string range $line 0 $pos_1-1] [string range $line $pos_2+1 end]]]
        set pos_1 [string first "(" $line 0]
    }
    return $line
}

#replace parameters, and other features...
proc Gcode::EvaluateValue { value } {
    variable parameters
    if { [string index $value 0] == "#" } {
        set parameter_number [string range $value 1 end]
        if { [info exists parameters($parameter_number)] } {
            set value $parameters($parameter_number)
        } else {
            W "parameter $parameter_number doesn't exists"
        }
    }
    if { $value == 0 } {
        set value 0 ;#a single 0, e.g. 00 -> 0
    } else {
        set value [string trimleft $value 0] ;#avoid left zeros (e.g 09 is a problem because is considered by Tcl as octal, not decimal 9)
    }
    return $value
}

#letter
#return 0 if not, 1 , 2, 3 otherwise, depending of its cathegory
proc Gcode::IsMidLineLetter { letter } {
    set is_mid_line_letter 0
    set letter_uppercase [string toupper $letter]
    switch $letter_uppercase {
        A - B - C - X - Y - Z {
            #axis letters, argument of a main letter
            set is_mid_line_letter 2
        }
        D - F - H - I - J - K - L - P - Q - R {
            #letters argument of a main letter
            set is_mid_line_letter 3
        }
        G - M - S - T {
            #main letters
            set is_mid_line_letter 1
        }
    }
    return $is_mid_line_letter
}

#mid_line_word could be joined without spaces between them, it is not usual, but legal
proc Gcode::EnforceSpaceBeforeMainLetter { line } {
    set line_out ""
    set c_prev " "
    set n [string length $line]
    for { set i 0 } { $i<$n } { incr i } {
        set c [string index $line $i]
        if { [Gcode::IsMidLineLetter $c] && ![string is space $c_prev] } {
            append line_out " "
        }
        append line_out $c
        set c_prev $c
    }
    return $line_out
}

#fill array variable line_code_values
proc Gcode::ParseLine { line last_line_letter_and_code } {
    variable line_code_values
    variable parameters
    array unset line_code_values
    set line [Gcode::RemoveParenthesisBlocks $line]
    if { [string index $line 0] == "/" } {
        #optional block delete character
        set block_delete 1
        set line [string range $line 1 end]
    }
    set line_number ""
    set line_letter [string toupper [string index $line 0]]
    if { $line_letter == "N" } {
        #optional line number, jump it
        set line_number [string range [lindex $line 0] 1 end] ;#but line maybe has no list syntax!!
        set line [lrange $line 1 end] ;#but line maybe has no list syntax!!
        set line_letter [string toupper [string index $line 0]]
    }
    set line [Gcode::EnforceSpaceBeforeMainLetter $line]
    set code ""
    set next_line ""
    if { $line_letter == ";" } {
        #comment
    } else {
        set code [string range [lindex $line 0] 1 end] ;#but line maybe has no list syntax!!
        set code [Gcode::EvaluateValue $code]
        if { [Gcode::IsMidLineLetter $line_letter] == 2 } {
            #axis words without G. G move commands are modal, assumed the last retained
            lassign $last_line_letter_and_code line_letter code
            #set the line as words to be re-parsed as key-value
            set words $line
        } else {
            set words [lrange $line 1 end] ;#but line maybe has no list syntax!!
        }
        set num_words [llength $words]
        for {set i_word 0} {$i_word<$num_words} {incr i_word} {
            set word [lindex $words $i_word]
            set word_letter [string toupper [string index $word 0]]
            if { $word_letter == ";" } {
                #comment until end
                break
            }
            if { $word_letter == "(" } {
                #must comment until ")"
                #by now assume as until end...
                break
            }
            if { [Gcode::IsMidLineLetter $word_letter] == 1 } {
                #the line has multiple commands, return them as next_line to be parsed in next loop !!
                set next_line [lrange $words $i_word end]
                break
            }
            set value [string range $word 1 end]
            if { $value=="" } {
                #e.g. "G1 Z 10 F1000;"
                #I expected do not have any space between Z and 10, must be "G1 Z10 F1000;"!!
                set next_word [lindex $words $i_word+1]
                if { ![Gcode::IsMidLineLetter [string toupper [string index $next_word 0]]] } {
                    set value $next_word
                    incr i_word
                }
            }
            set value [Gcode::EvaluateValue $value]
            if { $value != "" } {
                set line_code_values($word_letter) $value
            } else {
                W "code $word_letter without expected value in $words"
            }
        }
    }
    return [list $line_letter $code $next_line]
}

#name X Y Z E (E extrusion)
proc Gcode::CalculateNewValue { name } {
    variable line_code_values
    variable current
    set new_value $line_code_values($name)
    if { $name == "E" } {
        variable relative_extrusion
        set relative $relative_extrusion
    } elseif { $name == "X" || $name == "Y" || $name == "Z"} {
        variable relative_position
        set relative $relative_position
    } else {
        W "Gcode::CalculateNewValue. unexpected name=$name"
    }
    if { $relative } {
        set new_value [expr {$current($name)+$new_value}]
    }
    return $new_value
}

proc Gcode::SetLayer { layer } {
    if { ![GiD_Layers exists $layer] } {
        GiD_Layers create $layer
    }
    GiD_Layers edit to_use $layer
    return $layer
}

proc Gcode::CreateStLine { layer id_start id_end } {
    return [GiD_Geometry create line append stline $layer $id_start $id_end]
}

proc Gcode::CreateArcStartCenterAngle { layer id_start center angle } {
    set angle_degrees [MathUtils::FromRadToDegree $angle]
    set last_line_id [GiD_Info geometry MaxNumLines]
    set layer_to_use_prev [GiD_Layers get to_use]
    if { $layer_to_use_prev != $layer } {
        GiD_Layers edit to_use $layer
    }
    GiD_Process Mescape Utilities Copy Points Duplicate DoExtrude Lines MaintainLayers Rotation FNoJoin [join $center ,] TwoDim $angle_degrees $id_start escape
    if { $layer_to_use_prev != $layer } {
        GiD_Layers edit to_use $layer_to_use_prev
    }
    set line_id [GiD_Info geometry MaxNumLines]
    if { $line_id == $last_line_id } {
        set line_id ""
        W "arc line not created: start $id_start center $center angle $angle"
    }
    return $line_id
}

proc Gcode::ReadGeometry { filename } {
    set fail 0

    variable _show_advance_bar
    variable percent_done
    variable stop

    variable _current_line


    variable layer_by_z

    variable parameters
    variable current
    variable line_code_values

    variable units
    variable relative_position
    variable relative_extrusion

    Gcode::Init


    #set coordinate_sytems_current $parameters(5220)
    # A value of 1 means the first coordinate system (the one G54 activates),
    # a value of 2 means the second coordinate system (the one G55 activates), and so on.

    set fp [open $filename r]
    if { [file extension $filename] == ".gz" } {
        #to allow read also gzip compressed files
        fconfigure $fp -translation binary
        zlib push gunzip $fp
    }

    set layerNum 0
    set layerZ -9999999

    set layer [GiD_Info Project LayerToUse]
    set next_line ""
    set last_line_letter_and_code [list "" ""]
    set ::Gcode::stop 0
    set ::Gcode::percent_done 0
    set num_lines 1
    if { $_show_advance_bar } {
        set num_lines [GidUtils::LineCount $filename]
        set num_lines_refresh [expr int($num_lines/100.0)]
        GidUtils::CreateAdvanceBar [_ "Reading file"] [_ "Process"] ::Gcode::percent_done 100 {set ::Gcode::stop 1}
    }
    set i_percents 0
    set i_line 0
    while { ![eof $fp] } {
        if { $::Gcode::stop } {
            break
        }
        if { $next_line != "" } {
            set line $next_line
            set next_line ""
            set Gcode::_current_line [list -1 $line]
        } else {
            gets $fp line
            set line [string trim $line]
            incr i_line
            set Gcode::_current_line [list $i_line $line]
            if { ![expr {$i_line%$num_lines_refresh}] } {
                set ::Gcode::percent_done [expr {int(($i_line*100.0)/$num_lines)}]
            }
        }
        set pos_comment [string first ";" $line]
        if { $pos_comment != -1 } {
            set line [string trim [string range $line 0 $pos_comment-1]]
        }
        if { $line == "" } {
            continue
        }
        if { $line == "%" } {
            #start or end of the file in some dialects
            incr i_percents
            if { $i_percents == 2 } {
                #The useful contents of a file demarcated by percents stop after the second percent line. Anything after that is ignored
                break
            } else {
                continue
            }
        }
        if { [string index $line 0] == "#" } {
            lassign [split [string range $line 1 end] =] parameter_number parameter_value
            set parameters($parameter_number) $parameter_value
            continue
        }
        if { $line == "" } {
            continue
        }
        lassign [Gcode::ParseLine $line $last_line_letter_and_code] line_letter code next_line
        if { $line_letter == ";" } {
            #comment
        } elseif { $line_letter == "" } {
            #empty line after remove comments, line numbers,...
        } elseif { $line_letter == "G" }  {
            #General commands
            if { $code == 0 } {
                #Rapid Move
                lassign [list $current(X) $current(Y) $current(Z) $current(E) $current(F)] prev(X) prev(Y) prev(Z) prev(E) prev(F)
                if { [info exists line_code_values(X)] } {
                    set current(X) $line_code_values(X)
                }
                if { [info exists line_code_values(Y)] } {
                    set current(Y) $line_code_values(Y)
                }
                if { [info exists line_code_values(Z)] } {
                    set current(Z) $line_code_values(Z)
                }
                if { [info exists line_code_values(E)] } {
                    set current(E) $line_code_values(E)
                }
                if { [info exists line_code_values(F)] } {
                    set current(F) $line_code_values(F)
                }
                if { $current(X) != $prev(X) || $current(Y) != $prev(Y) || $current(Z) != $prev(Z) } {
                    if { $layer_by_z } {
                        #set layer [Gcode::SetLayer Z_$current(Z)]
                        set layer [Gcode::SetLayer Rapid]
                    } else {
                        set layer [Gcode::SetLayer Rapid]
                    }
                    set point_id [GiD_Geometry create point append $layer $current(X) $current(Y) $current(Z)]
                    if { $point_id>1 } {
                        set line_id [Gcode::CreateStLine $layer [expr $point_id-1] $point_id]
                    }
                }
            } elseif { $code == 1 } {
                #Linear Move
                #e.g. G1 Z15.0 F6000 ;Move the platform down 15mm
                lassign [list $current(X) $current(Y) $current(Z) $current(E) $current(F)] prev(X) prev(Y) prev(Z) prev(E) prev(F)
                if { [info exists line_code_values(X)] } {
                    set current(X) [Gcode::CalculateNewValue X]
                }
                if { [info exists line_code_values(Y)] } {
                    set current(Y) [Gcode::CalculateNewValue Y]
                }
                if { [info exists line_code_values(Z)] } {
                    set current(Z) [Gcode::CalculateNewValue Z]
                }
                if { [info exists line_code_values(E)] } {
                    set current(E) [Gcode::CalculateNewValue E]
                }
                if { [info exists line_code_values(F)] } {
                    set current(F) $line_code_values(F)
                }
                if { $current(E) != $prev(E) } {
                    #fill
                } else {
                    #move
                }
                if { $current(F) != $prev(F) } {
                    set change_velocity 1
                }
                if { $current(X) != $prev(X) || $current(Y) != $prev(Y) || $current(Z) != $prev(Z) } {
                    if { $layer_by_z } {
                        set layer [Gcode::SetLayer Z_[format %.2f $current(Z)]]
                    } else {
                        set layer [Gcode::SetLayer Cutting]
                    }
                    set point_id [GiD_Geometry create point append $layer $current(X) $current(Y) $current(Z)]
                    set line_id [Gcode::CreateStLine $layer [expr $point_id-1] $point_id]
                }
            } elseif { $code == 2 || $code == 3 } {
                #G2 Arc (or helix) Move (clockwise arc)
                #G3 Arc (or helix) Move (counterclockwise arc)
                #e.g. G3 F300.0 X12.9975 Y1.714 I0.4346 J1.0696
                set can_create 1
                if { $current(AXIS) == "Z" } {
                    if { [info exists line_code_values(X)] && ![info exists line_code_values(Y)] } {
                        W "G$code wrong data, expected X or Y"
                    }
                    lassign [list $current(X) $current(Y) $current(Z) $current(E) $current(F)] prev(X) prev(Y) prev(Z) prev(E) prev(F)
                    if { [info exists line_code_values(X)] } {
                        set current(X) $line_code_values(X)
                    }
                    if { [info exists line_code_values(Y)] } {
                        set current(Y) $line_code_values(Y)
                    }
                    if { [info exists line_code_values(Z)] } {
                        set current(Z) $line_code_values(Z)
                    }
                    if { $current(Z) != $prev(Z) } {
                        #helix
                        W "G$code helix to be implemented... line $line"
                    }
                    if { [info exists line_code_values(R)] } {
                        #Radius Format Arc
                        # X Y Z is the endpoint
                        set radius $line_code_values(R)
                        if { $radius < 0.0 } {
                            #a negative radius indicates a turn of 180 degrees to 359.999 degrees
                        }
                        set p_start [list $prev(X) $prev(Y) $prev(Z)]
                        set p_end [list $current(X) $current(Y) $current(Z)]
                        set d_start_end [MathUtils::VectorDistance $p_start $p_end]
                        if { $d_start_end < 1e-7 } {
                            #direction is not defined, set arbitrarily +x
                            set direction_start_end [list 1.0 0.0 0.0]
                        } else {
                            set direction_start_end [MathUtils::VectorDirectionNormalized $p_start $p_end]
                        }
                        lassign $direction_start_end dx dy dz
                        if { $code == 2 } {
                            #(clockwise arc)
                            set direction_rotated_90_degrees [list $dy [expr -1.0*$dx] $dz]
                        } else {
                            #$code == 3 (counterclockwise arc)
                            set direction_rotated_90_degrees [list [expr -1.0*$dy] $dx $dz]
                        }
                        set p_mean [MathUtils::VectorMean $p_start $p_end]
                        set c_1 [MathUtils::VectorDistance $p_start $p_mean]
                        set c_2 [expr sqrt($radius*$radius-$c_1*$c_1)]
                        set p_center [MathUtils::VectorSum $p_mean [MathUtils::ScalarByVectorProd $c_2 $direction_rotated_90_degrees]]
                    } elseif { [info exists line_code_values(I)] || [info exists line_code_values(J)] || [info exists line_code_values(K)] } {
                        lassign [list $prev(X) $prev(Y) $prev(Z)] center(X) center(Y) center(Z)
                        if { [info exists line_code_values(I)] } {
                            set center(X) [expr $center(X)+$line_code_values(I)]
                        }
                        if { [info exists line_code_values(J)] } {
                            set center(Y) [expr $center(Y)+$line_code_values(J)]
                        }
                        if { [info exists line_code_values(K)] } {
                            set center(Z) [expr $center(Z)+$line_code_values(K)]
                        }
                        set p_start [list $prev(X) $prev(Y) $prev(Z)]
                        set p_end [list $current(X) $current(Y) $current(Z)]
                        set p_center [list $center(X) $center(Y) $center(Z)]
                        set radius_1 [MathUtils::VectorDistance $p_start $p_center]
                        set radius_2 [MathUtils::VectorDistance $p_end $p_center]
                        if { [expr abs($radius_1-$radius_2)] >0.002 } {
                            set can_create 0
                            W "G$code wrong data r_start $radius_1 != r_end $radius_2. line $line"
                        }
                    } else {
                        set can_create 0
                        W "G$code wrong data, expected R or (I or J). line $line"
                    }
                } else {
                    W "G$code implemented only for Z AXIS, not $current(AXIS). line $line"
                }
                if { $can_create } {
                    foreach i {start end} {
                        lassign [MathUtils::VectorDiff $p_center [set p_$i]] x_local y_local z_local
                        set angle($i) [expr atan2($y_local,$x_local)]
                    }
                    if { $layer_by_z } {
                        set layer [Gcode::SetLayer Z_$current(Z)]
                    } else {
                        set layer [Gcode::SetLayer Cutting]
                    }
                    set point_id_start [GiD_Info geometry MaxNumPoints]
                    #angle space is periodic (atan2 in range [-PI,PI]) and the increment must take into account the 'hyperspace jump'
                    set angle_increment [expr $angle(end)-$angle(start)]
                    if { $code == 2 } {
                        #( clockwise arc)
                        if { $angle_increment > 0.0 } {
                            # (start to -PI) + (PI to end)
                            set angle_increment [expr $angle_increment-$MathUtils::2PI]
                        }
                        if { $angle_increment > 0.0 } {
                            W "expected angle_increment negative. angle_increment=$angle_increment line $line"
                        }
                        if { $angle_increment <-$MathUtils::2PI } {
                            W "$angle_increment <-$MathUtils::2PI line $line"
                        }
                    } else {
                        #$code == 3 (counterclockwise arc)
                        if { $angle_increment < 0.0 } {
                            # (start to PI) + (-PI to end)
                            set angle_increment [expr $angle_increment+$MathUtils::2PI]
                        }
                        if { $angle_increment < 0.0 } {
                            W "expected angle_increment positive. angle_increment=$angle_increment line $line"
                        }
                        if { $angle_increment >$MathUtils::2PI } {
                            W "$angle_increment >$MathUtils::2PI line $line"
                        }
                    }
                    if { [expr abs($angle_increment)] < 1e-7 } {
                        #if points are the same it is a full circumference (and must be defined by center, not radius)
                        set angle_increment $MathUtils::2PI
                    }
                    set line_id [Gcode::CreateArcStartCenterAngle $layer $point_id_start $p_center $angle_increment]
                }
            } elseif { $code == 4 } {
                #For a dwell, program G4 P� . This will keep the axes unmoving for the period of time in seconds
                if { [info exists line_code_values(P)] } {
                    set period $line_code_values(P)
                }
            } elseif { $code == 7 } {
                #Lathe Diameter Mode
            } elseif { $code == 8 } {
                #Lathe Radius Mode

            } elseif { $code == 10 } {
                #L1 Set Tool Table
                #L2 Set Coordinate System. global or 1 to 9 locals, set by P$i
                #L10 Set Tool Table
                #L11 Set Tool Table
                #L20 Set Coordinate System
                #e.g. G10
                if { [info exists line_code_values(L)] } {
                    if { $line_code_values(L) == 2 } {
                        if { [info exists line_code_values(P)] } {
                            set coordinate_system $line_code_values(P)
                            #set the offsets of the nine program coordinate systems ...
                        } else {
                        }

                    } else {
                    }
                }
            } elseif { $code == 11 } {
                #undocummented
                #retract recover filament according to settings of M208

            } elseif { $code == 17 } {
                #set the current plane to XY (default)
                set current(AXIS) Z
            } elseif { $code == 18 } {
                #set the current plane to ZX
                set current(AXIS) Y
                W "G18 AXIS $ current(AXIS) case not implemented"
            } elseif { $code == 19 } {
                #set the current plane to YZ
                set current(AXIS) X
                W "G18 AXIS $ current(AXIS) case not implemented"
            } elseif { $code == 20 } {
                #set units to inches
                set units in
            } elseif { $code == 21 } {
                #set units to mm
                set units mm
            } elseif { $code == 28 } {
                #Go to Predefined Position
                #5161-5169 - "G28" Home for X, Y, Z, A, B, C, U, V & W. Persistent.
                #e.g. G28 ;Home
                set current(X) $parameters(5161)
                set current(Y) $parameters(5162)
                set current(Z) $parameters(5163)
                set point_id [GiD_Geometry create point append $layer $current(X) $current(Y) $current(Z)]
            } elseif { $code == 40 } {
                #cancel cutter radius compensation
            } elseif { $code == 41 } {
                # start cutter radius compensation left
            } elseif { $code == 42 } {
                # start cutter radius compensation right
            } elseif { $code == 43 } {
                # tool length offset (plus)
                if { [info exists line_code_values(H)] } {
                    set tool_index $line_code_values(H)
                }
            } elseif { $code == 49 } {
                #cancel tool length offset
            } elseif { $code == 50 } {
                #unknown
            } elseif { $code == 53 } {
                #You can make straight moves in the absolute machine coordinate system by using G53 with either G0 or G1
                set to_stop 1
            } elseif { $code == 54 || $code == 55 || $code == 56 || $code == 57 || $code == 58 || $code == 59 } {
                #select the current coordinate system 1 to 6
                set coordinate_sytems_current [expr $code-53]
            } elseif { $code == "59.1" || $code == "59.2" || $code == "59.3" } {
                #select the current coordinate system 7 to 9
                set coordinate_sytems_current [expr [string index $code 3]+6]
            } elseif { $code == 61 } {
                #follows the programmed path as exactly as possible, slowing or stopping if necessary at sharp corners
                set path_contro_mode exact
            } elseif { $code == 64 } {
                #sharp corners of the path may be rounded slightly so that the feed rate may be kept up
                set path_contro_mode continuous
            } elseif { $code == 80 } {
                # cancel motion mode (including any canned cycle)
            } elseif { $code >= 81 && $code <= 89 } {
                # canned cycles
            } elseif { $code == 90 } {
                #set absolute positioning
                set relative_position 0
            } elseif { $code == 91 } {
                #set relative positioning
                set relative_position 1
            } elseif { $code == 92 } {
                #Coordinate System Offset
                #e.g. G92 E0
                #use relative extrusion
                if { [info exists line_code_values(E)] } {
                    set relative_extrusion $line_code_values(E)
                } else {
                    set relative_extrusion 0 ;#??
                }
            } elseif { $code == 94 } {
                # units per minute feed rate mode
            } elseif { $code == 98 || $code == 99 } {
                #Canned Cycle Return Level
            } elseif { $code == 108 } {
                #unknown, ignore it
                #e.g. G108 P O
            } elseif { $code == 112 } {
                #unknown, ignore it
                #e.g. G112 V5000 C1
            } elseif { $code == 113 } {
                #unknown, ignore it
                #e.g. G113 S0
            } elseif { $code == 114 } {
                #unknown, ignore it
                #e.g. G114
            } elseif { $code == 119 } {
                #unknown, ignore it
                #e.g. G119 Z0
            } elseif { $code == 122 } {
                #unknown, ignore it
                #e.g. G122 S4000
            } elseif { $code == 215 } {
                #unknown, ignore it
                #e.g. G215 W1
            } elseif { $code == 280 } {
                #unknown, ignore it
                #e.g. G280 S1
            } else {
                incr unexpected_g_code($code)
                W "unexpected $line_letter code $code line $line"
            }
        } elseif { $line_letter == "M" }  {
            #M=Miscellaneous commands
            if { $code == 0 } {
                #program stop
            } elseif { $code == 1 } {
                #optional program stop
            } elseif { $code == 2 } {
                #program end
                break
            } elseif { $code == 3 } {
                #turn spindle clockwise
            } elseif { $code == 4 } {
                #turn spindle counterclockwise
            } elseif { $code == 5 } {
                #stop spindle turning
            } elseif { $code == 6 } {
                #tool change
            } elseif { $code == 7 } {
                #mist coolant on
            } elseif { $code == 8 } {
                #flood coolant on
            } elseif { $code == 9 } {
                #mist and flood coolant off
            } elseif { $code == 30 } {
                #program end, pallet shuttle, and reset
                break
            } elseif { $code == 48 } {
                #enable speed and feed overrides
            } elseif { $code == 49 } {
                #disable speed and feed overrides
            } elseif { $code == 60 } {
                # pallet shuttle and program stop
            } elseif { $code == 80 } {
                # Turn on Power Supply
            } elseif { $code == 81 } {
                # Turn off Power Supply
            } elseif { $code == 82 } {
                # Set E codes absolute (default)
                #e.g. M82 ;absolute extrusion mode
            } elseif { $code == 83 } {
                # Set E codes relative while in Absolute Coordinates (G90) mode
            } elseif { $code == 84 } {
                # Disable steppers until next move, or use S to specify an inactivity timeout, after which the steppers will be disabled. S0 to disable the timeout.
            } elseif { $code == 101 } {
                #unknown
            } elseif { $code == 102 } {
                #unknown
            } elseif { $code == 103 } {
                #unknown

            } elseif { $code == 104 } {
                #Set extruder target temp
            } elseif { $code == 105 } {
                #Read current temp

            } elseif { $code == 106 } {
                #Fan on

            } elseif { $code == 107 } {
                #Fan off

            } elseif { $code == 109 } {
                # Wait for extruder current temp to reach target temp.

            } elseif { $code == 114 } {
                # Display current position

            } elseif { $code == 140 } {

            } elseif { $code == 204 } {
                #  Set default acceleration: S normal moves T filament only moves (M204 S3000 T7000) im mm/sec^2
                #  also sets minimum segment time in ms (B20000) to prevent buffer underruns and M20 minimum feedrate
            } elseif { $code == 205 } {
                # advanced settings: minimum travel speed S=while printing T=travel only, B=minimum segment time X= maximum xy jerk, Z=maximum Z jerk, E=maximum E jerk

            } elseif { $code == 400 } {
                #unknown

            } else {
                #W "unexpected $line_letter code $code line $line"
                incr unexpected_m_code($code)
            }
        } elseif { $line_letter == "S" }  {
            #Start ?
            if { $code == 0 } {
                #surface?
            } elseif { $code == 255 || $code == 1600 } {
                # ?
            } else {
                #W "unexpected $line_letter code $code line $line"
            }
        } elseif { $line_letter == "T" }  {
            #Terminate?
            if { $code == 0 } {
                #use extruder 0?
            } elseif { $code == 1 } {
                # ?
            } else {
                #W "unexpected $line_letter code $code line $line"
            }
        } elseif { $line_letter == "O" }  {
            #?
            if { $code == 0 } {
                #?
            } else {
                W "unexpected $line_letter code $code line $line"
            }
        } else {
            W "unexpected $line"
        }
        set last_line_letter_and_code [list $line_letter $code]
    }
    close $fp
    if { $_show_advance_bar } {
        GidUtils::DeleteAdvanceBar
    }
    set unexpected_codes [array names unexpected_g_code]
    if { [catch { set unexpected_codes [lsort -integer $unexpected_codes] } msg] } {
        W "some code is not integer"
    }
    foreach code $unexpected_codes {
        W "G$code $unexpected_g_code($code) times"
    }
    set unexpected_codes [array names unexpected_m_code]
    if { [catch { set unexpected_codes [lsort -integer $unexpected_codes] } msg] } {
        W "some code is not integer"
    }
    foreach code $unexpected_codes {
        W "M$code $unexpected_m_code($code) times"
    }

    if { [GiD_Layers exists Layer0] && [llength [GiD_Layers list]] > 1 } {
        #delete Layer0 if possible
        if { ![GiD_Layers get num_entities Layer0] && ![GiD_Layers get num_conditions Layer0] } {
            GiD_Layers delete Layer0
        }
    }
    if { [GiD_Layers exists Rapid] } {
        #set this layer to yellow
        GiD_Layers edit color Rapid #ffff00
    }
    if { [GiD_Units get system] != "" } {
        GiD_Units edit model_unit_length $units
    }

    return $fail
}

proc Gcode::DrawCurrentPosition { } {
    variable _current_draw_position
    gid_draw_opengl::draw_symbol_point_load $_current_draw_position [list 0 0 -1]
}

#Animate: test to try to show animation with a chain of after calls
proc Gcode::AnimateLoop { i n increment delay } {
    variable stop
    variable percent_done
    variable _line_ids
    if { !$::Gcode::stop } {
        if { $i < $n } {
            #set num_update [expr int($n/100)]
            set num_update 100
            if { ![expr $i%$num_update] } {
                set Gcode::percent_done [expr int(100.0*$i/$n)]
            }
            set index_end [expr $i+$increment-1]
            if { $index_end>=$n} {
                #this is unneeded since objarray 1.17
                set index_end [expr $n-1]
            }
            set selection_line_ids [objarray range $_line_ids $i $index_end]
            GiD_EntitiesLayers assign_front_layer -also_lower_entities geometry lines $selection_line_ids
            #set point_id [lindex [GiD_Geometry get line [objarray get $selection_line_ids end]] 3]
            #set _current_draw_position [lrange [GiD_Geometry get point $point_id] 1 3]
            GiD_Redraw
            after $delay [list Gcode::AnimateLoop [expr $i+$increment] $n $increment $delay]
        } else {
            #GiD_OpenGL unregister $_opengl_register_id
            GiD_EntitiesLayers assign_front_layer -also_lower_entities geometry lines $_line_ids
            unset _line_ids
            GiD_Redraw
            GidUtils::DeleteAdvanceBar
        }
    } else {
        #GiD_OpenGL unregister $_opengl_register_id
        GiD_EntitiesLayers assign_front_layer -also_lower_entities geometry lines $_line_ids
        unset _line_ids
        GiD_Redraw
    }
}

#delay: integer value in microseconds to delay each frame, set to 1 to minimum delay (0 has problems to allow stop it)
#increment: integer step increment, each step will show a new a line, set >1 to draw a frame each $increment new lines, e.g. 25
proc Gcode::Animate { {delay 1} {increment 1}} {
    variable _line_ids
    #variable _opengl_register_id
    #variable _current_draw_position
    variable stop
    variable percent_done
    if { $delay<=0 } {
        #with delay 0 it is not possible to press stop button!!
        set delay 1
    }
    #package require gid_draw_opengl
    set stop 0
    set _line_ids [GiD_Geometry list line]
    set num_lines [objarray length $_line_ids]
    GiD_EntitiesLayers assign_back_layer -also_lower_entities lines $_line_ids
    GidUtils::CreateAdvanceBar [_ "G-Code"] [= "Percentage"] Gcode::percent_done 100 [list set ::Gcode::stop 1]
    #set _opengl_register_id [GiD_OpenGL register Gcode::DrawCurrentPosition]
    Gcode::AnimateLoop 0 $num_lines $increment $delay
}

#########################################################################################
#################################### DIALOG WINDOW ######################################
#########################################################################################

#to add more options to the standar file browser
proc Gcode::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkmerge -text [_ "Merge"] -variable Gcode::_merge
    grid $f.chkmerge -sticky w -columnspan 2
    ttk::checkbutton $f.chklayerbyz -text [_ "Layer by z"] -variable Gcode::layer_by_z
    grid $f.chklayerbyz -sticky w -columnspan 2
    grid columnconfigure $f 1 -weight 1

    variable import_dir
    if { ![info exists import_dir] } {
        #trick to show the more options
        set import_dir open
    }
    return Gcode::import_dir
}

proc Gcode::ReadPreW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read Gcode file"] \
        {} {{{Gcode} {.gcode .ncc .ngc .tap .gcode.gz}} {{All files} {.*}}} {} $multiple \
        [list [_ "Import options"] Gcode::MoreImportOptions]]
    if { [llength $filenames] } {
        if { !$_merge } {
            GiD_Process Mescape Files New No escape ;#to avoid previous geometry
        }
        GidUtils::EvalAndEnterInBatch Gcode::ReadPre $filenames
    }
    return 0
}

#####################################################################################
##################################### OTHER #########################################
#####################################################################################

proc Gcode::AddToMenuPRE { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "Gcode..."] PRE] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import "Gcode..."] $position PRE Gcode::ReadPreW "" "" insertafter _
}

proc Gcode::AddToMenu { } {
    Gcode::AddToMenuPRE
}

#invoke this menus changes
Gcode::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc Gcode::AddToMenu

#register the proc to be automatically called when dropping a file
foreach extension {.gcode .ncc .ngc .tap} {
    GiD_RegisterExtensionProc $extension PRE Gcode::ReadPreSingleFile
}

Gcode::Init
