#HDF5 Vibez medical images read plugin

namespace eval Vibez {
    variable info_plugin ;#information read from .xml plugin file
    variable _merge 1
    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

proc Vibez::GetAttribute { h path name {default ""}} {
    set value [string trim [$h get_attribute -default $default $path $name]]
    return $value
}

proc Vibez::IsVibez { h } {
    set element_size_um [Vibez::GetAttribute $h /t0/channel0 element_size_um ""]
    if { $element_size_um != "" } {
        return 1
    } else {
        return 0
    }
}

proc Vibez::ReadAndConvertVtk { filename } {
    set filename_out ""
    if { ![file exists $filename] } {
        WarnWin [_ "File %s not found" $filename]
        return ""
    }
    package require hdf5
    package require gid_cross_platform
    set h hd1
    if { [catch {hdf5 -readonly $h $filename} err] } {
        WarnWin [_ "File %s is not an HDF5 file" $filename]
        return ""
    }

    if { ![Vibez::IsVibez $h] } {
        WarnWin [_ "File %s is not a Vibez HDF5 file" $filename]
        $h close
        return ""
    }
    if { [$h is_group /t0] && [$h is_dataset /t0/channel0] } {
        set origin {0.0 0.0 0.0}
        set spacing [Vibez::GetAttribute $h /t0/channel0 element_size_um ""]
        set dimensions [lindex [$h get -dimensions /t0/channel0] 0]
        #set ndimensions [llength $dimensions]
        set data [lindex [$h get /t0/channel0] 0]
        #set ndata [llength $data]
        $h close

        #
        set _data_mode "Binary"
        set vtk_version [package require vtk]
        package require objarray
        package require vtk_objarray

        set h [vtkXMLImageDataWriter New]
        #set filename_out [file rootname $filename].vti
        set filename_out [gid_cross_platform::get_unused_tmp_filename gid_vibez .vti]
        #set h [vtkDataSetWriter New]
        #set $filename_out [file rootname $filename].vtk
        $h SetFileName $filename_out
        if { $_data_mode == "Ascii" } {
            $h SetDataModeToAscii
        } elseif { $_data_mode == "Binary" } {
            $h SetDataModeToBinary
            #$h SetCompressorTypeToNone
            $h SetCompressorTypeToZLib ;#to compress Binary and Appended Data
        } elseif { $_data_mode == "Compressed" } {
            $h SetDataModeToAppended
            #$h SetCompressorTypeToNone
            $h SetCompressorTypeToZLib ;#to compress Binary and Appended Data
            #AppendedData encoding="base64" en los ejemplos KwiViewer es "raw"
            #$h EncodeAppendedDataOn ;#AppendedData encoding="base64". XML compliant but slower
            $h EncodeAppendedDataOff ;#AppendedData encoding="raw". NON XML compliant but faster
        } else {
            $h Delete
            W "Unexpected data mode $_data_mode"
            return ""
        }

        set vtkStructuredPoints [vtkStructuredPoints New]
        #set vtkStructuredPoints [vtkImageData New]

        #it seems that the hf5 Vibez and vtk ordering of x and z are swapped
        #$vtkStructuredPoints SetExtent 0 [expr [lindex $dimensions 2] -1] 0 [expr [lindex $dimensions 1] -1] 0 [expr [lindex $dimensions 0] -1]
        #use SetDimensions but the documentation said that is deprecated and SetExtent must be used insted
        $vtkStructuredPoints SetDimensions [lindex $dimensions 2] [lindex $dimensions 1] [lindex $dimensions 0]
        $vtkStructuredPoints SetOrigin [lindex $origin 2] [lindex $origin 1] [lindex $origin 0]
        $vtkStructuredPoints SetSpacing [lindex $spacing 2] [lindex $spacing 1] [lindex $spacing 0]
        set vtkDataSet $vtkStructuredPoints
        set vtkPointData [$vtkDataSet GetPointData]
        set vtkDataSetAttributes $vtkPointData

        set vtkDataArray [vtk_objarray vtk_data_array_from_objarray $data vtkUnsignedShortArray]

        #$vtkDataSetAttributes SetScalars $vtkDataArray
        $vtkDataSetAttributes AddArray $vtkDataArray
        $vtkDataArray SetName volume_scalars

        #vtk > 5 must use $h SetInputData $vtkStructuredPoints
        $h SetInputConnection [$vtkStructuredPoints GetProducerPort]
        $h Write
        $h Delete
        GidUtils::SetWarnLine [_ "File '%s' written" $filename_out]
        GiD_RaiseEvent GiD_Event_AfterSaveFile $filename_out VTK_FORMAT 0
    }
    return $filename_out
}

proc Vibez::ReadPost { filename } {
    set filename_out [Vibez::ReadAndConvertVtk $filename]
    if { $filename_out != "" } {
        Vtk::ReadPostSingleFile $filename_out
        GiD_RaiseEvent GiD_Event_AfterOpenFile $filename HDF5_VIBEZ_FORMAT 0
    }
}

#to add more options to the standar file browser
proc Vibez::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkmerge -text [_ "Merge"] -variable Vibez::_merge
    grid $f.chkmerge -sticky w -columnspan 2
    grid columnconfigure $f 1 -weight 1

    variable import_dir
    if { ![info exists import_dir] } {
        #trick to show the more options
        set import_dir open
    }
    return Vibez::import_dir
}

proc Vibez::ReadPostW { } {
    set multiple 0
    set filename [MessageBoxGetFilename file read [_ "Read HDF5 Vibez file"] \
        {} {{{HDF5 Vibez} {.h5 }} {{All files} {.*}}} {} $multiple \
        [list [_ "Import options"] Vibez::MoreImportOptions]]
    if {$filename == ""} {
        return
    }
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New Yes escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch Vibez::ReadPost $filename
    GidUtils::SetWarnLine [_ "File %s read." $filename]
}

proc Vibez::AddToMenuPOST { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import [_ "HDF5 Vibez"]...] POST] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] POST]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import [_ "HDF5 Vibez"]...] $position POST Vibez::ReadPostW "" "" insertafter _
}


proc Vibez::AddToMenu { } {
    Vibez::AddToMenuPOST
}


#invoke this menus changes
Vibez::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc Vibez::AddToMenu


#register the proc to be automatically called when dropping a file
GiD_RegisterExtensionProc ".h5" POST Vibez::ReadPost
