#KML read plugin

#########################################################################################
######################################### READ PRE ######################################
#########################################################################################

namespace eval KML {
    variable info_plugin ;#information read from .xml plugin file
    variable projections {None UTM Mercator PlateCarree}
    variable import_to_projection None
    variable utm_zone ""
    variable utm_letter ""

    variable layer_to_use ""

    #temporary use, extra selection in import/export KML window
    variable import_extrude 0; #0= do not extrude, 1=extrude
    variable export_from_projection None ;#to declare the current coordinates system to transform from it to geodetic: longitude-latitude-heigh
    variable export_from_utm_zone ""
    variable export_from_utm_letter ""
    variable export_type screen ;#screen : image capture, vectorial: polygons (mesh only)
    variable export_animate 1 ;#1 images of all time steps, 0 only current step

    variable _merge 1
    variable stop
    variable percent_done
    variable i_polygons
    variable num_polygons

    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}


proc KML::MyXmlSelectNodes { xml_node nodeName } {
    return [$xml_node getElementsByTagName $nodeName]
}

proc KML::MyXmlSelectNodesString { xml_node nodeName } {
    set txt ""
    set xml_child [$xml_node getElementsByTagName $nodeName]
    if { $xml_child != "" } {
        set txt [$xml_child text]
    }
    return $txt
}

proc KML::MyXmlCreateTextNode { xml_parent txt} {
    set newNode [[$xml_parent ownerDocument] createTextNode $txt]
    $xml_parent appendChild $newNode
    return $newNode
}

proc KML::MyXmlCreateElement { xml_parent name {attributes ""} } {
    set newNode [[$xml_parent ownerDocument] createElement $name]
    if { $attributes != "" } {
        $newNode setAttribute {*}$attributes
    }
    $xml_parent appendChild $newNode
    return $newNode
}

proc KML::ErrorMessage { msg {xml_node ""}} {
    if { $xml_node != "" } {
        set txt "nodeName: [$xml_node nodeName] nodeType: [$xml_node nodeType]"
        foreach attribute [$xml_node attributes] {
            catch { append txt "$attribute: [$xml_node getAttribute $attribute]" } ;#with namespaces fail
        }
        append msg $txt
    }
    WarnWinText $msg
}

proc KML::IsAbstractGeometryGroup { nodeName } {
    if { [lsearch -sorted {LineString LinearRing Model MultiGeometry Point Polygon} $nodeName] == -1 } {
        return 0
    } else {
        return 1
    }
}

proc KML::ExtrudeSurfacesZIncrement { surface_ids z_increment } {
    GiD_Process Mescape Utilities Copy Surfaces Duplicate DoExtrude Volumes MaintainLayers Translation FNoJoin 0,0,0 FNoJoin 0,0,$z_increment {*}$surface_ids escape
    return 0
}

proc KML::ExtrudeLinesZIncrement { line_ids z_increment } {
    GiD_Process Mescape Utilities Copy Lines Duplicate DoExtrude Surfaces MaintainLayers Translation FNoJoin 0,0,0 FNoJoin 0,0,$z_increment {*}$line_ids escape
    return 0
}

proc KML::ExtrudePointsZIncrement { point_ids z_increment } {
    GiD_Process Mescape Utilities Copy Points Duplicate DoExtrude Lines MaintainLayers Translation FNoJoin 0,0,0 FNoJoin 0,0,$z_increment {*}$point_ids escape
}

proc KML::LineString { coordinates extrude } {
    variable layer_to_use
    variable import_extrude
    set point_ids [KML::coordinates $coordinates]
    set line_ids [list ]
    set p0 [lindex $point_ids 0]
    foreach p1 [lrange $point_ids 1 end] {
        lappend line_ids [GiD_Geometry create line append stline $layer_to_use $p0 $p1]
        set p0 $p1
    }

    if { $extrude } {
        if { $import_extrude } {
            set z_increment -[lindex [split [lindex $coordinates 0] ,] 2]
            KML::ExtrudeLinesZIncrement $line_ids $z_increment
        }
    }
    return $line_ids
}

proc KML::LinearRing { coordinates extrude } {
    variable layer_to_use
    variable import_extrude
    set point_ids [KML::coordinates [lrange $coordinates 0 end-1]]

    set line_ids [list ]
    set p0 [lindex $point_ids 0]
    foreach p1 [lrange $point_ids 1 end] {
        lappend line_ids [GiD_Geometry create line append stline $layer_to_use $p0 $p1]
        set p0 $p1
    }
    #create a closing line joining end and 0
    set p1 [lindex $point_ids 0]
    lappend line_ids [GiD_Geometry create line append stline $layer_to_use $p0 $p1]

    if { $extrude } {
        if { $import_extrude } {
            set z_increment -[lindex [split [lindex $coordinates 0] ,] 2]
            KML::ExtrudeLinesZIncrement $line_ids $z_increment
        }
    }
    return [list $line_ids]
}

proc KML::Model { extrude } {

}

proc KML::coordinates { coordinates } {
    variable layer_to_use
    variable import_to_projection
    set h 0
    set point_ids [list ]
    foreach p $coordinates {
        set c [split $p ","]
        if { [llength $c] == 3 } {
            lassign [split $p ","] lon lat h
        } elseif { [llength $c] == 2 } {
            lassign [split $p ","] lon lat
        } else {
            KML::ErrorMessage "Point: wrong number of coordinates=[llength $c]"
            return 1
        }
        if { $import_to_projection == "None" } {
            #doesn't do any conversion of coordinates
            lassign [list $lon $lat $h] x y z
        } else {
            lassign [KML::ConvertFromLonLatHToCurrentProjection $lon $lat $h] x y z
        }
        lappend point_ids [GiD_Geometry create point append $layer_to_use $x $y $z]
    }
    return $point_ids
}

proc KML::Point { coordinates extrude } {
    set point_id [KML::coordinates $coordinates]
    if { $extrude } {
        if { $import_extrude } {
            set z_increment -[lindex [split [lindex $coordinates 0] ,] 2]
            KML::ExtrudePointsZIncrement [list $point_id] $z_increment
        }
    }
    return $point_id
}

proc KML::Polygon { coordinates inner_coordinates_list_list extrude } {
    variable import_to_projection
    variable layer_to_use
    variable import_extrude ; #0= do not extrude, 1=extrude, 2=store the extrude data in condition surface_import-1, question ZValue

    lassign [KML::LinearRing [lreverse $coordinates] 0] line_ids
    set hole_lines [list ]
    foreach inner_coordinates_list $inner_coordinates_list_list {
        foreach inner_coordinates $inner_coordinates_list {
            lassign [KML::LinearRing [lreverse $inner_coordinates] 0] hole_line_ids
            lappend hole_lines {*}$hole_line_ids
        }
    }
    if { $layer_to_use != [GiD_Layers get to_use] } {
        GiD_Layers edit to_use $layer_to_use
    }
    set surface_id [GidUtils::CreateSurfaceFromBoundaryLines [concat $line_ids $hole_lines]]
    if { $extrude } {
        if { $import_extrude } {
            set z_increment -[lindex [split [lindex $coordinates 0] ,] 2]
            KML::ExtrudeSurfacesZIncrement [list $surface_id] $z_increment
        }
    }

    return 0
}

#get information from  xml node and invoke the proc to process it
proc KML::XML_LineString { xml_node } {
    set coordinates [KML::MyXmlSelectNodesString $xml_node coordinates]
    set extrude [KML::MyXmlSelectNodesString $xml_node extrude]
    if { $extrude == "" } {
        set extrude 0
    }
    KML::LineString $coordinates $extrude
}

proc KML::XML_LinearRing { xml_node } {
    set coordinates [KML::MyXmlSelectNodesString $xml_node coordinates]
    set extrude [KML::MyXmlSelectNodesString $xml_node extrude]
    if { $extrude == "" } {
        set extrude 0
    }
    KML::LinearRing $coordinates $extrude
}

proc KML::XML_Model { xml_node } {

}

proc KML::XML_MultiGeometry { xml_node } {
    foreach xml_child [$xml_node childNodes] {
        set nodeName [$xml_child nodeName]
        if { [KML::IsAbstractGeometryGroup $nodeName] } {
            KML::XML_$nodeName $xml_child
        }
    }
    return 0
}

proc KML::XML_ReadNodesRecursive { xml_node } {
    variable stop
    set fail 0
    foreach xml_child [$xml_node childNodes] {
        if { $KML::stop } {
            set fail 1
            break
        }
        set nodeName [$xml_child nodeName]
        if { $nodeName == "Placemark" } {
            KML::XML_Placemark $xml_child
        } elseif { [$xml_child hasChildNodes] } {
            set fail [XML_ReadNodesRecursive $xml_child]
        }
    }
    return $fail
}

proc KML::XML_Placemark { xml_node } {
    variable layer_to_use
    foreach xml_child [$xml_node childNodes] {
        set nodeName [$xml_child nodeName]
        if { $nodeName == "name" } {
            set layer [$xml_child text]
            if { $layer != "" } {
                set layer_to_use $layer
                if { ![GiD_Layers exists $layer] } {
                    GiD_Layers create $layer
                }
            }
        } elseif { [KML::IsAbstractGeometryGroup $nodeName] } {
            KML::XML_$nodeName $xml_child
        }
    }
}

proc KML::XML_Point { xml_node } {
    set coordinates [KML::MyXmlSelectNodesString $xml_node coordinates]
    if { $coordinates == "" } {
        KML::ErrorMessage "coordinates child not found" $xml_node
        return 1
    }
    set extrude [KML::MyXmlSelectNodesString $xml_node extrude]
    if { $extrude == "" } {
        set extrude 0
    }
    KML::Point $coordinates $extrude
}

#get information from  xml node and invoke the proc to process it
proc KML::XML_Polygon { xml_node } {
    variable percent_done
    variable i_polygons
    variable num_polygons
    if { $num_polygons >= 100 } {
        incr i_polygons
        if { ![expr {$i_polygons%int($num_polygons/100)}] } {
            set percent_done [expr {int(($i_polygons*100.0)/$num_polygons)}]
        }
    }

    set outerBoundaryIs [KML::MyXmlSelectNodes $xml_node outerBoundaryIs]
    if { $outerBoundaryIs == "" } {
        KML::ErrorMessage "outerBoundaryIs child not found" $xml_node
        return 1
    }
    set LinearRing [KML::MyXmlSelectNodes $outerBoundaryIs LinearRing]

    set coordinates [KML::MyXmlSelectNodesString $LinearRing coordinates]
    if { [lindex $coordinates end] == [lindex $coordinates end-1] } {
        #SMS triangle mesh examples have wrong repeated coordinate, maybe it is a degenerated quadrilateral
        set coordinates [lrange $coordinates 0 end-1]
    }

    set inner_coordinates_list [list]
    foreach innerBoundaryIs [KML::MyXmlSelectNodes $xml_node innerBoundaryIs] {
        set LinearRings [KML::MyXmlSelectNodes $innerBoundaryIs LinearRing]
        foreach LinearRing $LinearRings {
            set inner_coordinates ""
            lappend inner_coordinates [KML::MyXmlSelectNodesString $LinearRing coordinates]
            lappend inner_coordinates_list $inner_coordinates
        }
    }

    set extrude [KML::MyXmlSelectNodesString $xml_node extrude]
    if { $extrude == "" } {
        set extrude 0
    }
    set altitudeMode [KML::MyXmlSelectNodesString $xml_node altitudeMode]
    if { $altitudeMode == "relativeToGround" } {

    } elseif { $altitudeMode == "absolute" } {
        #must substract the earth radius or the z terrain relative to sea to h?
    } else {
    }

    KML::Polygon $coordinates $inner_coordinates_list $extrude
    return 0
}

#to drag and drop a file
proc KML::ReadPreSingleFile { filename } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch KML::ReadPre [list $filename]
}

proc KML::ReadPre { filenames } {
    variable import_to_projection
    set fail 0
    GidUtils::WaitState
    GidUtils::DisableGraphics
    set save_CreateAlwaysNewPoint [GiD_Set CreateAlwaysNewPoint]
    GiD_Set CreateAlwaysNewPoint -2
    if { [catch { set fail [KML::ReadPre0 $filenames] } err options] } {
        set fail 1
    }
    GiD_Set CreateAlwaysNewPoint $save_CreateAlwaysNewPoint
    GidUtils::EnableGraphics
    GidUtils::EndWaitState
    if { $fail } {
        return -options $options $err
    } else {
        GidUtils::UpdateWindow LAYER
        GiD_Process 'Zoom Frame
        set msg [_ "KML read. Geodetic coordinates transformed to %s" $import_to_projection]
        if { $import_to_projection == "UTM" } {
            variable utm_zone
            variable utm_letter
            append msg " " [_ "(zone %s letter %s)" $utm_zone $utm_letter]
        }
        GidUtils::SetWarnLine $msg
    }
    return $fail
}

#read and finally redraw showing what is read
proc KML::ReadPre0 { filenames } {
    variable layer_to_use
    variable import_to_projection ;#combobox variable
    variable stop
    variable percent_done
    variable i_polygons
    variable num_polygons

    package require mapproj
    package require gid_map
    package require tdom


    set fail 0
    set ::Gcode::stop 0
    set ::Gcode::percent_done 0
    if { 0 } {
        #by now use the existent advancebar of each file
        set percent_max [llength $filenames]
        if { $_show_advance_bar } {
            GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::Gcode::percent_done $percent_max {set ::Gcode::stop 1}
        }
    }
    set create_layer_filename 0
    if { 0 } {
        #by now use the layers of the file
        if { [llength $filenames]>1 } {
            set create_layer_filename 1
        }
    }
    foreach filename $filenames {
        if { $::Gcode::stop } {
            break
        }
        if { $create_layer_filename } {
            set layer [file rootname [file tail $filename]]
            if { ![GiD_Layers exists $layer] } {
                GiD_Layers create $layer
            }
            GiD_Layers edit to_use $layer
        }
        if { [string tolower [file extension $filename]] == ".kmz" } {
            set zipfile_decode_present 0
            if { [catch { package require zipfile::decode } err] } {
                #KML::ErrorMessage $err
            } else {
                set zipfile_decode_present 1
            }
            if { $zipfile_decode_present } {
                zipfile::decode::open $filename
                set zdict [zipfile::decode::archive]
                set filename_inside doc.kml
                if { [zipfile::decode::hasfile $zdict $filename_inside] } {
                    set xml [zipfile::decode::getfile $zdict $filename_inside]
                } else {
                    KML::ErrorMessage [_ "File %s inside %s not found" $filename_inside $filename]
                }
                zipfile::decode::close
            }
        } else {
            set xml [tDOM::xmlReadFile $filename]
        }
        set doc [dom parse $xml]
        set root [$doc documentElement]
        if { [$root nodeName] != "kml" } {
            KML::ErrorMessage "Is not a kml file" $root
            return 1
        } else {
            #set ns {kml "http://www.opengis.net/kml/2.2"}
            #set ns {kml "http://earth.google.com/kml/2.2"}
            set ns [list kml [$root getAttribute xmlns]]
            set layer_to_use [GiD_Info project layertouse]
            if { $import_to_projection == "UTM" } {
                variable utm_zone
                variable utm_letter
                set utm_zone ""
                set utm_letter ""
            }
            #count to provide an advance bar...
            set num_polygons [llength [$root getElementsByTagName Polygon]]

            set stop 0
            if { $num_polygons >= 100 } {
                set percent_done 0
                set i_polygons 0
                GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::KML::percent_done 100 {set ::KML::stop 1}
            }
            set fail [KML::XML_ReadNodesRecursive $root]
            if { $num_polygons >= 100 } {
                GidUtils::DeleteAdvanceBar
            }
            if { $fail } {
                WarnWin [_ "User stopped"]
            }
        }
        $doc delete
        GiD_RaiseEvent GiD_Event_AfterOpenFile $filename KML_FORMAT 0
        if { 0 } {
            incr ::KML::percent_done
        }
    }
    if { 0 } {
        #by now use the existent advancebar of each file
        if { $_show_advance_bar } {
            GidUtils::DeleteAdvanceBar
        }
    }
    return $fail
}

#to add more options to the standar file browser
proc KML::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkmerge -text [_ "Merge"] -variable UNV::_merge
    grid $f.chkmerge -sticky w -columnspan 2
    variable import_to_projection
    variable projections
    variable import_extrude
    ttk::label $f.lprojection -text [_ "To projection"]
    ttk::combobox $f.projection -values $projections -textvariable KML::import_to_projection
    GidHelp [list $f.lprojection $f.projection] [_ "Declare projection to be converted (from longitude,latitude)"]
    ttk::checkbutton $f.extrude -text [_ "Extrude"] -variable KML::import_extrude
    GidHelp [list $f.extrude] [_ "True to extrude walls of the buildings"]
    grid $f.lprojection $f.projection $f.extrude -sticky w
    grid columnconfigure $f 3 -weight 1
    variable import_dir
    if { ![info exists import_dir] } {
        #trick to show the more options
        set import_dir open
    }
    return KML::import_dir
}

proc KML::ReadPreW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read KML Google earth file"] \
        {} {{{KML} {.kml .kmz}} {{All files} {.*}}} {} $multiple \
        [list [_ "Import options"] KML::MoreImportOptions]]
    if { [llength $filenames] } {
        if { !$_merge } {
            GiD_Process Mescape Files New No escape ;#to avoid previous geometry
        }
        GidUtils::EvalAndEnterInBatch KML::ReadPre $filenames
    }
}


proc KML::ConvertFromLonLatHToCurrentProjection { lon lat h } {
    variable import_to_projection
    variable utm_zone
    variable utm_letter

    if { $import_to_projection == "Mercator" || $import_to_projection == "PlateCarree"} {
        set xy [::mapproj::to$import_to_projection 0 0 $lon $lat]
        lassign $xy x y
        #mapproj works with a sphere of radius 1
        set x [expr {$x*[gid_map::get_radius_earth]}]
        set y [expr {$y*[gid_map::get_radius_earth]}]
        set z $h
    } elseif { $import_to_projection == "UTM" } {
        set utm [gid_map::ll2utm $lon $lat]
        if { [lindex $utm 2] != $utm_zone && $utm_zone != "" } {
            KML::ErrorMessage "region span more than one utm zone: [lindex $utm 2] != $utm_zone, utm coordinates discontinous"
        }
        if { [lindex $utm 3] != $utm_letter && $utm_letter != "" } {
            KML::ErrorMessage "region span both hemispheres, utm coordinates discontinous"
        }
        lassign $utm x y utm_zone utm_letter
        set z $h
    } elseif { $import_to_projection == "None" } {
        set xyz [gid_map::ll2xyz $lon $lat $h]
        lassign $xyz x y z
    } else {
        set x $lon
        set y $lat
        set z $h
    }
    return [list $x $y $z]
}

proc KML::ConvertFromCurrentProjectionToLonLatH { x y z } {
    variable export_from_projection
    variable export_from_utm_zone
    variable export_from_utm_letter
    if { $export_from_projection == "Mercator" || $export_from_projection == "PlateCarree"} {
        set x [expr {$x/[gid_map::get_radius_earth]}]
        set y [expr {$y/[gid_map::get_radius_earth]}]
        #mapproj works with a sphere of radius 1
        set lonlat [::mapproj::from$export_from_projection 0 0 $x $y]
        lassign $lonlat lon lat
        set h $z
    } elseif { $export_from_projection == "UTM" } {
        set lonlat [gid_map::utm2ll $x $y $export_from_utm_zone $export_from_utm_letter]
        lassign $lonlat lon lat
        set h $z
    } elseif { $export_from_projection == "None" } {
        set lonlath [gid_map::xyz2ll $x $y $z]
        lassign $lonlath lon lat h
    }
    return [list $lon $lat $h]
}

proc KML::WriteElementValue { xml_node_mesh i_element_type element_id value } {
    set xml_node_Placemark [KML::MyXmlCreateElement $xml_node_mesh Placemark]
    set xml_node_styleUrl [KML::MyXmlCreateElement $xml_node_Placemark styleUrl]
    KML::MyXmlCreateTextNode $xml_node_styleUrl #set$i_element_type
    set xml_node_Polygon [KML::MyXmlCreateElement $xml_node_Placemark Polygon [list id $element_id]]
    set xml_node_outerBoundaryIs [KML::MyXmlCreateElement $xml_node_Polygon outerBoundaryIs]
    set xml_node_LinearRing [KML::MyXmlCreateElement $xml_node_outerBoundaryIs LinearRing]
    set xml_node_coordinates [KML::MyXmlCreateElement $xml_node_LinearRing coordinates]
    KML::MyXmlCreateTextNode $xml_node_coordinates $value
}

proc KML::WritePostMeshes { xml_node } {
    variable import_to_projection
    variable export_from_projection
    variable export_from_utm_zone
    variable export_from_utm_letter
    if { $export_from_projection == "UTM" && ($export_from_utm_zone == "" || $export_from_utm_letter == "") } {
        KML::ErrorMessage [_ "Error: a UTM zone and letter must be set"]
        return 1
    }

    set xml_node_Document [KML::MyXmlCreateElement $xml_node Document]

    # coordinates
    set nodes [GiD_Info mesh -post nodes -array]
    lassign $nodes node_ids coordinates
    lassign $coordinates xs ys zs
    if { $export_from_projection == "None" } {
        #doesn't do any conversion of coordinates
        foreach node_id $node_ids x $xs y $ys z $zs {
            set nodes_xyz($node_id) $x,$y,$z
        }
    } else {
        foreach node_id $node_ids x $xs y $ys z $zs {
            lassign [KML::ConvertFromCurrentProjectionToLonLatH $x $y $z] lon lat h
            set nodes_xyz($node_id) $lon,$lat,$h
        }
    }
    unset nodes node_ids coordinates xs ys zs

    set elements [GiD_Info mesh -post elements Any -array]
    set num_element_types [llength $elements]

    #is not easy to get the information of postprocess sets: name, color, type_elements, <elements>
    #then use only the list of set colors, but sets won't match the used color
    set i_element_type 1
    foreach item [concat [GiD_Info post get all_surfaces_colors] [GiD_Info post get all_volumes_colors]] {
        set color [lindex $item 1] ;# #RRGGBB
        set alpha [lindex $item 3] ;# 0.0 to 1.1
        set kml_color($i_element_type) [string range $color 5 6][string range $color 3 4][string range $color 1 2] ;#BBGGRR reversed !!
        incr i_element_type
    }
    for {set i_element_type 1} { $i_element_type<=$num_element_types } {incr i_element_type} {
        set xml_node_Style [KML::MyXmlCreateElement $xml_node_Document Style [list id set$i_element_type]]
        set xml_node_LineStyle [KML::MyXmlCreateElement $xml_node_Style LineStyle]
        set xml_node_color [KML::MyXmlCreateElement $xml_node_LineStyle color]
        KML::MyXmlCreateTextNode $xml_node_color FF$kml_color($i_element_type)
        set xml_node_width [KML::MyXmlCreateElement $xml_node_LineStyle width]
        KML::MyXmlCreateTextNode $xml_node_width 1

        set xml_node_PolyStyle [KML::MyXmlCreateElement $xml_node_Style PolyStyle]
        set xml_node_color [KML::MyXmlCreateElement $xml_node_PolyStyle color]
        KML::MyXmlCreateTextNode $xml_node_color BF$kml_color($i_element_type)
        set xml_node_fill [KML::MyXmlCreateElement $xml_node_PolyStyle fill]
        KML::MyXmlCreateTextNode $xml_node_fill 1
        set xml_node_outline [KML::MyXmlCreateElement $xml_node_PolyStyle outline]
        KML::MyXmlCreateTextNode $xml_node_outline 1
    }

    set xml_node_meshes [KML::MyXmlCreateElement $xml_node_Document Folder {name meshes}]

    set i_element_type 1
    foreach item $elements {
        lassign $item elemtype element_ids connectivities materials
        set xml_node_mesh [KML::MyXmlCreateElement $xml_node_meshes Folder [list id $elemtype]]
        set nnodes [llength $connectivities]
        if { $elemtype == "Linear" } {
            if { $nnodes == 2 } {
                lassign $connectivities n1s n2s
                foreach element_id $element_ids n1 $n1s n2 $n2s {
                    set value "$nodes_xyz($n1)\n$nodes_xyz($n2)\n$nodes_xyz($n1)"
                    KML::WriteElementValue $xml_node_mesh $i_element_type $element_id $value
                }
            } elseif { $nnodes == 3 } {
                lassign $connectivities n1s n2s n3s
                foreach element_id $element_ids n1 $n1s n2 $n2s n3 $n3s {
                    set value "$nodes_xyz($n1)\n$nodes_xyz($n3)\n$nodes_xyz($n2)\n$nodes_xyz($n1)"
                    KML::WriteElementValue $xml_node_mesh $i_element_type $element_id $value
                }
            }
            set xml_node_Placemark [KML::MyXmlCreateElement $xml_node_mesh Placemark]
            set xml_node_styleUrl [KML::MyXmlCreateElement $xml_node_Placemark styleUrl]
            KML::MyXmlCreateTextNode $xml_node_styleUrl #set$i_element_type
            set xml_node_LineString [KML::MyXmlCreateElement $xml_node_Placemark LineString [list id $element_id]]
            set xml_node_coordinates [KML::MyXmlCreateElement $xml_node_LineString coordinates]
            KML::MyXmlCreateTextNode $xml_node_coordinates $value
            incr i_element_type
            continue
        } elseif { $elemtype == "Triangle" } {
            if { $nnodes == 3 } {
                lassign $connectivities n1s n2s n3s
                foreach element_id $element_ids n1 $n1s n2 $n2s n3 $n3s {
                    set value "$nodes_xyz($n1)\n$nodes_xyz($n2)\n$nodes_xyz($n3)\n$nodes_xyz($n1)"
                    KML::WriteElementValue $xml_node_mesh $i_element_type $element_id $value
                }
            } elseif { $nnodes == 6 } {
                lassign $connectivities n1s n2s n3s n4s n5s n6s
                foreach element_id $element_ids n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s {
                    set value "$nodes_xyz($n1)\n$nodes_xyz($n4)\n$nodes_xyz($n2)\n$nodes_xyz($n5)\n$nodes_xyz($n3)\n$nodes_xyz($n6)$nodes_xyz($n1)\n"
                    KML::WriteElementValue $xml_node_mesh $i_element_type $element_id $value
                }
            }
        } elseif { $elemtype == "Quadrilateral" } {
            if { $nnodes == 4 } {
                lassign $connectivities n1s n2s n3s n4s
                foreach element_id $element_ids n1 $n1s n2 $n2s n3 $n3s n4 $n4s {
                    set value "$nodes_xyz($n1)\n$nodes_xyz($n2)\n$nodes_xyz($n3)\n$nodes_xyz($n4)\n$nodes_xyz($n1)"
                    KML::WriteElementValue $xml_node_mesh $i_element_type $element_id $value
                }
            } elseif { $nnodes == 8 || $nnodes == 9 } {
                if { nnodes == 8 } {
                    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s
                } else {
                    #9 nodes ignore the last center node
                    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s
                }
                foreach element_id $element_ids n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s {
                    set value "$nodes_xyz($n1)\n$nodes_xyz($n5)\n$nodes_xyz($n2)\n$nodes_xyz($n6)\n$nodes_xyz($n3)\n$nodes_xyz($n7)\n$nodes_xyz($n4)\n$nodes_xyz($n8)\n$nodes_xyz($n1)"
                    KML::WriteElementValue $xml_node_mesh $i_element_type $element_id $value
                }
            }
        } else {
            incr i_element_type
            continue
        }
        incr i_element_type
    }

    return 0
}

proc KML::WriteGroundOverlay { xml_node image_filename {time_span ""}} {
    variable export_from_projection
    lassign [GiD_Info ortholimits] L R B T N F E
    if { $export_from_projection == "None" } {
        #must use vectorial kml, because is 3D, not a 2D projection.!!
        #really get a 2D image is not a good idea with this projection, because the geometry is a 3D sphere
        #and the screen is a 2D image in an arbitrary view direction. at least must to set an XY view
        if { [expr ($N+$F)] > 0 } {
            set hemisphere north
        } else {
            set hemisphere north
        }
        lassign [gid_map::xyhemisphere2ll $L $B $hemisphere] west south h
        lassign [gid_map::xyhemisphere2ll $R $T $hemisphere] east north h
    } else {
        lassign [KML::ConvertFromCurrentProjectionToLonLatH $L $B 0.0] west south h
        lassign [KML::ConvertFromCurrentProjectionToLonLatH $R $T 0.0] east north h
    }
    set xml_node_GroundOverlay [KML::MyXmlCreateElement $xml_node GroundOverlay]
    KML::MyXmlCreateTextNode [KML::MyXmlCreateElement $xml_node_GroundOverlay name] $image_filename
    set xml_node_Icon [KML::MyXmlCreateElement $xml_node_GroundOverlay Icon]
    KML::MyXmlCreateTextNode [KML::MyXmlCreateElement $xml_node_Icon href] $image_filename
    KML::MyXmlCreateTextNode [KML::MyXmlCreateElement $xml_node_Icon viewBoundScale] 0.75
    set xml_node_LatLonBox [KML::MyXmlCreateElement $xml_node_GroundOverlay LatLonBox]
    foreach item {north south east west} {
        KML::MyXmlCreateTextNode [KML::MyXmlCreateElement $xml_node_LatLonBox $item] [set $item]
    }
    KML::MyXmlCreateTextNode [KML::MyXmlCreateElement $xml_node_LatLonBox rotation] 0.0
    if { [llength $time_span]== 2 } {
        lassign $time_span begin end
        set xml_node_TimeSpan [KML::MyXmlCreateElement $xml_node_GroundOverlay TimeSpan]
        KML::MyXmlCreateTextNode [KML::MyXmlCreateElement $xml_node_TimeSpan begin] $begin
        KML::MyXmlCreateTextNode [KML::MyXmlCreateElement $xml_node_TimeSpan end] $end
    }
}

proc KML::WritePost { filename } {
    variable export_from_projection
    variable export_type
    variable export_animate
    set fail 0
    if { $export_from_projection == "" } {
        KML::ErrorMessage [_ "Error: must declare the current projection we are exporting from"]
        return 1
    }

    package require mapproj
    package require gid_map
    package require tdom
    package require gid_cross_platform
    set desired_extension [string tolower [file extension $filename]]
    if { $desired_extension == ".kmz" } {
        set zip_package_present 0
        #package require zipfile::encode ;#->set encoder [zipfile::encode] ; $encoder file: $file_to_zip 0 $file_to_zip
        #package require zipfile::mkzip ;#->zipfile::mkzip::mkzip $zip_filename *
        if { [catch { package require customLib_utils::c } err] } {
            #KML::ErrorMessage $err
        } else {
            package require customLib_utils ;#to source also the proc cu::zipfile
            if { [info procs ::cu::zipfile] != "" } {
                set zip_package_present 1
            }
        }
        if { !$zip_package_present } {
            set compressed 0
            set dst_folder [file dirname $filename]
        } else {
            set compressed 1
            #zip in a temporary folder
            set dst_folder [gid_cross_platform::create_folder_tmp]
            file mkdir $dst_folder
        }
    } else {
        set compressed 0
        set dst_folder [file dirname $filename]
    }

    set doc [dom parse "<kml xmlns='http://www.opengis.net/kml/2.2'/>"]
    set root [$doc documentElement]
    if { $export_type== "screen" } {
        #screen capture
        GidUtils::DisableWarnLine
        GidUtils::EnableGraphics
        set xml_node_Document [KML::MyXmlCreateElement $root Document]
        set analysis [GiD_Info postprocess get cur_analysis]
        set all_steps [GiD_Info postprocess get all_steps $analysis]
        set current_TranspBGOnImages [lindex [GiD_Info postprocess get HardcopyOptions] 4]
        if { !$current_TranspBGOnImages } {
            GiD_Process 'Hardcopy Options TranspBGOnImages yes escape
        }
        if { $export_animate == 0 || [llength $all_steps] == 0} {
            set image_filename screen.png
            set full_image_filename [file join $dst_folder $image_filename]
            GiD_Process 'Hardcopy PNG $full_image_filename escape
            KML::WriteGroundOverlay $xml_node_Document $image_filename
        } else {
            set current_step [GiD_Info postprocess get cur_step]
            set current_result [GiD_Info postprocess get cur_result]
            set current_component [GiD_Info postprocess get cur_component]
            set current_result_view [GiD_Info postprocess get cur_results_view]
            set current_result_view [string map {_ ""} $current_result_view]
            set i 0
            foreach step $all_steps {
                incr i
                GiD_Process Mescape Results AnalysisSel $analysis $step \
                    $current_result_view $current_result $current_component escape
                set image_filename screen_${i}.png
                set full_image_filename [file join $dst_folder $image_filename]
                GiD_Process 'Hardcopy PNG $full_image_filename escape
                file delete [file rootname $full_image_filename].pgw
                set time_span [list [expr {$i-1}] $i]
                KML::WriteGroundOverlay $xml_node_Document $image_filename $time_span
            }
            GiD_Process Mescape Results AnalysisSel $analysis $current_step \
                $current_result_view $current_result $current_component escape
        }
        if { !$current_TranspBGOnImages } {
            GiD_Process 'Hardcopy Options TranspBGOnImages no escape
        }
        GidUtils::DisableGraphics
        GidUtils::EnableWarnLine
    } else {
        #vectorial (mesh only)
        KML::WritePostMeshes $root
    }
    if { $compressed } {
        set filename_inside [file join $dst_folder doc.kml]
    } else {
        if { $desired_extension == ".kmz" && !$compressed } {
            set filename_inside [file rootname $filename].kml
            KML::ErrorMessage [_ "zip package disabled, will be saved uncompressed as .kml"]
        } else {
            set filename_inside $filename
        }
    }
    set fout [open $filename_inside w]
    if { $fout != "" } {
        fconfigure $fout -encoding utf-8
        puts -nonewline $fout {<?xml version="1.0" encoding="utf-8"?>}
        puts $fout [$doc asXML -indent 2]
        close $fout
    } else {
        set fail 1
    }

    if { !$fail } {
        if { $compressed } {
            set zip_filename [file rootname $filename].kmz
            set previous_folder [pwd]
            cd $dst_folder
            set files_to_zip [glob *]
            ::cu::zipfile -recursive $zip_filename {*}$files_to_zip
            #test tcllib package zipfile::encode
            #set encoder [zipfile::encode]
            #foreach file_to_zip $files_to_zip {
            #    $encoder file: $file_to_zip 0 $file_to_zip
            #}
            #$encoder write $zip_filename
            #
            #test tcllib package zipfile::mkzip
            #zipfile::mkzip::mkzip $zip_filename *
            cd $previous_folder
            file delete -force $dst_folder
        }
    }

    GiD_RaiseEvent GiD_Event_AfterSaveFile $filename KML_FORMAT $fail
    return $fail
}

proc KML::OnChangeExportFromProjection { frame_utm args } {
    variable export_from_projection
    if { [winfo exists $frame_utm] } {
        if { $export_from_projection == "UTM" } {
            grid $frame_utm
        } else {
            grid remove $frame_utm
        }
    }
}

#to add more options to the standar file browser
proc KML::MoreExportOptions { f } {
    variable export_from_projection ;#to declare the current coordinates system to transform from it to geodetic: longitude-latitude-heigh
    variable export_from_utm_zone
    variable export_from_utm_letter
    variable export_type ;#screen : image capture, vectorial: mesh only
    variable export_animate

    variable projections
    variable import_to_projection
    variable utm_zone
    variable utm_letter

    ttk::label $f.lprojection -text [_ "From projection"]
    ttk::combobox $f.projection -values $projections -textvariable KML::export_from_projection
    GidHelp [list $f.lprojection $f.projection] [_ "Declare the current projection (converted to longitude,latitude to export)"]

    ttk::frame $f.utm
    ttk::label $f.utm.lutm_zone -text [_ "UTM Zone"]
    ttk::combobox $f.utm.utm_zone -values [objarray new_from_to intarray 1 60] \
        -textvariable KML::export_from_utm_zone -width 6
    ttk::label $f.utm.lutm_letter -text [_ "UTM Hemisphere"]
    TTKComboBox $f.utm.utm_letter -textvariable KML::export_from_utm_letter \
        -labels [list [_ "North"] [_ "South"]] -values {N M} -state readonly  -width 6

    ttk::label $f.ltype -text [_ "Type"]
    TTKComboBox $f.type -textvariable KML::export_type \
        -labels [list [_ "Screen"] [_ "Vectorial"]] -values {screen vectorial} -state readonly
    GidHelp [list $f.ltype $f.type] [_ "Screen: image capture, Vectorial: mesh polygons only"]
    ttk::checkbutton $f.animate -text [_ "Animation"] -variable KML::export_animate
    GidHelp [list $f.animate] [_ "Animation: to save images of all time steps"]

    grid $f.lprojection $f.projection $f.utm -sticky w
    grid $f.utm.lutm_zone $f.utm.utm_zone $f.utm.lutm_letter $f.utm.utm_letter -sticky w

    grid $f.ltype $f.type $f.animate -sticky w
    grid columnconfigure $f 4 -weight 1

    if { [trace info variable KML::export_from_projection] == "" } {
        trace add variable KML::export_from_projection write [list KML::OnChangeExportFromProjection $f.utm]
    }

    if { $import_to_projection != "" } {
        set export_from_projection $import_to_projection
    }
    if { $utm_zone != "" } {
        set export_from_utm_zone $utm_zone
    }
    if { $utm_letter != "" } {
        set export_from_utm_letter $utm_letter
    }

    variable export_dir
    if { ![info exists export_dir] } {
        #trick to show the more options
        set export_dir open
    }
    return KML::export_dir
}

proc KML::WritePostW { } {
    variable export_type
    set multiple 0
    set filename [MessageBoxGetFilename file write [_ "Write KML Google earth file"] \
        {} {{{KML} {.kmz .kml}}} {.kmz} $multiple \
        [list [_ "Export options"] KML::MoreExportOptions]]
    if {$filename != ""} {
        if { $export_type == "screen" } {
            #get the visual: must be 0.0 0.0 -1.0, else rotate to XY view
            #really for export_from_projection == "None" points not close to xy but on a 3D sphere
            #and the best visual will be pointing to the earth center on the mid of our entities, but...
            lassign [lrange [GiD_GetWorldCoord 0 0] 3 end] vx vy vz
            if { $vx != 0.0 || $vy != 0.0 || $vz != -1.0 } {
                GiD_Process 'Rotate Angle 270 90 escape
                GidUtils::SetWarnLine [_ "View rotated to XY to get appropiated image for kml file"]
            }
        }
        GidUtils::WaitState .gid
        GidUtils::DisableGraphics
        set fail 0
        if { [catch { set res [GidUtils::EvalAndEnterInBatch KML::WritePost $filename] } err options] } {
            set fail 1
        }
        GidUtils::EnableGraphics
        GidUtils::EndWaitState .gid
        if { $fail } {
            return -options $options $err
        } else {
            GidUtils::SetWarnLine [_ "KML file '%s' written" $filename]
        }
    }
}


#####################################################################################
##################################### OTHER #########################################
#####################################################################################

proc KML::AddToMenuPRE { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "KML..."] PRE] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Import->XYZ points"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ points"] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import "KML..."] $position PRE KML::ReadPreW "" "" insertafter _
}

proc KML::AddToMenuPOST { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Export "KML..."] POST] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Export->Visible surfaces to PLY"
    set position [GiDMenu::GetOptionIndex Files [list Export "Visible surfaces to PLY"] POST]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Export "KML..."] $position POST KML::WritePostW "" "" insertafter _
}

proc KML::AddToMenu { } {
    KML::AddToMenuPRE
    KML::AddToMenuPOST
}

#invoke this menus changes
KML::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc KML::AddToMenu

#register the proc to be automatically called when dropping a file
foreach extension {.kml .kmz } {
    GiD_RegisterExtensionProc $extension PRE KML::ReadPreSingleFile
}
