#test of python in tcl: first test import stl files
#import several mesh formats using the library meshio (https://github.com/nschloe/meshio)
#that is written in python and using the tohil package (https://github.com/flightaware/tohil) to add a python interpreter and allow communication tcl<->python
#included python (<GiD>\scripts\tohil\python ) with more extra python packages
# - meshio 5.3.4 (<GiD>\scripts\tohil\python\lib\site-packages\meshio)
# - numpy 1.2.3.1 (<GiD>\scripts\tohil\python\lib\site-packages\numpy)

#GiD_Python_StartDebuggerServer ;#to allow a remote debug from VSCode, at localhost, port 5678 (can attach to a process also, that use the same values


namespace eval MeshIO {
    variable info_plugin ;#information read from .xml plugin file
    #internal use variables
    variable percent_done
    variable stop
    variable _merge  1 ;#to set in file browser options if want to merge or delete old model+results
    variable meshio_num_nodes_per_cell
    variable meshio_gid_element
    variable _python_code_sourced 0
    #
    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

proc MeshIO::Init { } {
    variable meshio_num_nodes_per_cell
    variable meshio_gid_element
    variable meshio_element_name

    package require objarray 1.18 ;#for objarray set $objarray $index $values with value a list of more than one number

    #linear and serendipit
    set meshio_num_nodes_per_cell(vertex) 1
    set meshio_num_nodes_per_cell(line) 2
    set meshio_num_nodes_per_cell(triangle) 3
    set meshio_num_nodes_per_cell(quad) 4
    set meshio_num_nodes_per_cell(quad8) 8
    set meshio_num_nodes_per_cell(tetra) 4
    set meshio_num_nodes_per_cell(hexahedron) 8
    set meshio_num_nodes_per_cell(hexahedron20) 20
    set meshio_num_nodes_per_cell(hexahedron24) 24
    set meshio_num_nodes_per_cell(wedge) 6
    set meshio_num_nodes_per_cell(pyramid) 5
    #quadratic
    set meshio_num_nodes_per_cell(line3) 3
    set meshio_num_nodes_per_cell(triangle6) 6
    set meshio_num_nodes_per_cell(quad9) 9
    set meshio_num_nodes_per_cell(tetra10) 10
    set meshio_num_nodes_per_cell(hexahedron27) 27
    set meshio_num_nodes_per_cell(wedge15) 15
    set meshio_num_nodes_per_cell(wedge18) 18
    set meshio_num_nodes_per_cell(pyramid13) 13
    set meshio_num_nodes_per_cell(pyramid14) 14
    #degree 3
    set meshio_num_nodes_per_cell(line4) 4
    set meshio_num_nodes_per_cell(triangle10) 10
    set meshio_num_nodes_per_cell(quad16) 16
    set meshio_num_nodes_per_cell(tetra20) 20
    set meshio_num_nodes_per_cell(wedge40) 40
    set meshio_num_nodes_per_cell(hexahedron64) 64
    #degree 4
    set meshio_num_nodes_per_cell(line5) 5
    set meshio_num_nodes_per_cell(triangle15) 15
    set meshio_num_nodes_per_cell(quad25) 25
    set meshio_num_nodes_per_cell(tetra35) 35
    set meshio_num_nodes_per_cell(wedge75) 75
    set meshio_num_nodes_per_cell(hexahedron125) 125
    #degree 5
    set meshio_num_nodes_per_cell(line6) 6
    set meshio_num_nodes_per_cell(triangle21) 21
    set meshio_num_nodes_per_cell(quad36) 36
    set meshio_num_nodes_per_cell(tetra56) 56
    set meshio_num_nodes_per_cell(wedge126) 126
    set meshio_num_nodes_per_cell(hexahedron216) 216
    #degree 6
    set meshio_num_nodes_per_cell(line7) 7
    set meshio_num_nodes_per_cell(triangle28) 28
    set meshio_num_nodes_per_cell(quad49) 49
    set meshio_num_nodes_per_cell(tetra84) 84
    set meshio_num_nodes_per_cell(wedge196) 196
    set meshio_num_nodes_per_cell(hexahedron343) 343
    #degree 7
    set meshio_num_nodes_per_cell(line8) 8
    set meshio_num_nodes_per_cell(triangle36) 36
    set meshio_num_nodes_per_cell(quad64) 64
    set meshio_num_nodes_per_cell(tetra120) 120
    set meshio_num_nodes_per_cell(wedge288) 288
    set meshio_num_nodes_per_cell(hexahedron512) 512
    #degree 8
    set meshio_num_nodes_per_cell(line9) 9
    set meshio_num_nodes_per_cell(triangle45) 45
    set meshio_num_nodes_per_cell(quad81) 81
    set meshio_num_nodes_per_cell(tetra165) 165
    set meshio_num_nodes_per_cell(wedge405) 405
    set meshio_num_nodes_per_cell(hexahedron729) 729
    #degree 9
    set meshio_num_nodes_per_cell(line10) 10
    set meshio_num_nodes_per_cell(triangle55) 55
    set meshio_num_nodes_per_cell(quad100) 100
    set meshio_num_nodes_per_cell(tetra220) 220
    set meshio_num_nodes_per_cell(wedge550) 550
    set meshio_num_nodes_per_cell(hexahedron1000) 1000
    set meshio_num_nodes_per_cell(hexahedron1331) 1331
    #degree 10
    set meshio_num_nodes_per_cell(line11) 11
    set meshio_num_nodes_per_cell(triangle66) 66
    set meshio_num_nodes_per_cell(quad121) 121
    set meshio_num_nodes_per_cell(tetra286) 286

    #linear and serendipit
    set meshio_gid_element(vertex) point
    set meshio_gid_element(line) line
    set meshio_gid_element(triangle) triangle
    set meshio_gid_element(quad) quadrilateral
    set meshio_gid_element(tetra) tetrahedra
    set meshio_gid_element(hexahedron) hexahedra
    set meshio_gid_element(wedge) prism
    set meshio_gid_element(pyramid) pyramid
    #quadratic
    set meshio_gid_element(line3) line
    set meshio_gid_element(triangle6) triangle
    set meshio_gid_element(quad8) quadrilateral
    set meshio_gid_element(quad9) quadrilateral
    set meshio_gid_element(tetra10) tetrahedra
    set meshio_gid_element(hexahedron20) hexahedra
    set meshio_gid_element(hexahedron27) hexahedra
    set meshio_gid_element(wedge15) prism
    set meshio_gid_element(pyramid13) pyramid


    set meshio_element_name(point,1) vertex
    set meshio_element_name(line,2) line
    set meshio_element_name(linear,2) line
    set meshio_element_name(triangle,3) triangle
    set meshio_element_name(quadrilateral,4) quad
    set meshio_element_name(tetrahedra,4) tetra
    set meshio_element_name(hexahedra,8) hexahedron
    set meshio_element_name(prism,6) wedge
    set meshio_element_name(pyramid,5) pyramid
    #quadratic
    set meshio_element_name(line,3) line3
    set meshio_element_name(linear,3) line3
    set meshio_element_name(triangle,6) triangle6
    set meshio_element_name(quadrilateral,8) quad8
    set meshio_element_name(quadrilateral,9) quad9
    set meshio_element_name(tetrahedra,10) tetra10
    set meshio_element_name(hexahedra,20) hexahedron20
    set meshio_element_name(hexahedra,27) hexahedron27
    set meshio_element_name(prism,15) wedge15
    set meshio_element_name(pyramid,13) pyramid13
}

#to load in python the file gid_meshio.py to define its python functions before be called
#it is not easy in current python to source a file, can import as a module but must be in search path
proc MeshIO::InitPython { } {
    variable info_plugin
    set fail 0
    variable _python_code_sourced
    if { !$_python_code_sourced} {
        # do not add meshio to this list because we have some fixed bugs compared with 5.3.5 version that can be get with pip
        set packages_fail [GiD_Python_PipInstallMissingPackages [list numpy netCDF4 h5py rich]]
        if { [llength $packages_fail] } {
            set fail 1
        } else {
            set plugin_path $info_plugin(path)
            GiD_Python_PathAppend [file join $plugin_path site-packages src];#to find meshio in the plugin path, instead site-packages
            set filename_python [file join $plugin_path gid_meshio.py]
            GiD_Python_Import_File $filename_python
            #GiD_Python_Source $filename_python
            set _python_code_sourced 1
        }
    }
    return $fail
}


#########################################################################################
######################################### READ PRE ######################################
#########################################################################################

#to add more options to the standar file browser
proc MeshIO::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkGeometry -text [_ "Merge"] -variable MeshIO::_merge
    grid columnconfigure $f 1 -weight 1
    grid $f.chkGeometry -sticky w -columnspan 2

    variable export_dir
    if { ![info exists export_dir] } {
        #trick to show the more options
        set export_dir open
    }
    return MeshIO::export_dir
}

proc MeshIO::ReadPreW { } {
    variable _merge
    set multiple 1
    set types [list]
    lappend types {{All meshio files} {.inp .msh .avs .cgns .xml .e .exo \
        .f3grid .h5m .mdpa .mesh .meshb .med .bdf .fem .nas .vol .vol.gz \
        .obj .off .post .post.gz .dato .dato.gz .ply .stl .dat .node .ele .svg .su2 \
        .ugrid .vtk .vtu .wkt .xdmf .xmf}}
    foreach {name extensions} {Abaqus .inp "ANSYS msh" .msh AVS-UCD .avs CGNS .cgns "DOLFIN XML" .xml Exodus {.e .exo} \
        FLAC3D .f3grid H5M .h5m Kratos/MDPA .mdpa Medit {.mesh .meshb} MED/Salome .med Nastran {.bdf .fem .nas} Netgen {.vol .vol.gz} \
        OBJ .obj OFF .off PERMAS {.post .post.gz .dato .dato.gz} PLY .ply STL .stl Tecplot .tec TetGen {.node .ele} SVG .svg SU2 .su2 \
        UGRID .ugrid VTK .vtk VTU .vtu "WKT (TIN)" .wkt XDMF {.xdmf .xmf}} {
            lappend types [list $name $extensions]
    }
    lappend types {{All files} {.*}}
    set filenames [MessageBoxGetFilename file read [_ "Read meshio file"] \
        {} $types {} $multiple [list [_ "Import options"] MeshIO::MoreImportOptions]]
    if {$filenames != ""} {
        GidUtils::EvalAndEnterInBatch MeshIO::ReadPre $filenames
    }
}

#to drag and drop a file
proc MeshIO::ReadPreSingleFile { filename  } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch MeshIO::ReadPre [list $filename]
}

proc MeshIO::ReadPre { filenames { lst_options {}} } {
    variable _merge
    set fail [MeshIO::InitPython]
    if { $fail } {
        return 1
    }

    # process lst_options
    if { [llength $lst_options] > 0} {
        array set my_options $lst_options
        if { [info exists my_options(merge)]} {
            set MeshIO::_merge $my_options(merge)
        }
    }
    if { !$MeshIO::_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }

    set num_nodes_before [GiD_Info Mesh NumNodes]
    set num_elements_before [GiD_Info Mesh NumElements]
    GidUtils::WaitState
    GidUtils::DisableGraphics
    set tcl_error [catch { set fail [MeshIO::ReadPre0 $filenames] } err options]
    GidUtils::EnableGraphics
    GidUtils::EndWaitState
    set num_nodes [GiD_Info Mesh NumNodes]
    set num_elements [GiD_Info Mesh NumElements]
    if { $num_nodes != $num_nodes_before || $num_elements != $num_elements_before } {
        GiD_RaiseEvent GiD_Event_AfterChangeMesh $num_nodes $num_elements
    }
    MeshView
    GiD_Process 'Zoom Frame
    GidUtils::UpdateWindow LAYER
    GidUtils::UpdateWindow GROUPS
    GidUtils::SetWarnLine [_ "Meshio read." ]
    if { $tcl_error } {
        return -options $options $err
    }
    return $fail
}

proc MeshIO::ReadPre0 { filenames } {
    set fail 0
    set tcl_error 0
    foreach filename $filenames {
        if { ![file exists $filename] } {
            W [_ "File %s not found" $filename]
            continue
        }
        if { [catch { set fail [MeshIO::ReadPreUnstructuredMesh $filename] } err options] } {
            set tcl_error 1
        }
        GiD_RaiseEvent GiD_Event_AfterOpenFile $filename MESHIO_FORMAT 0
    }
    if { [info procs ::DeleteLayerEmpty] != "" } {
        #this proc exists from GiD 14.1.9d
        DeleteLayerEmpty Layer0
    }

    if { $tcl_error } {
        return -options $options $err
    }
    return $fail
}

proc MeshIO::ReadPreUnstructuredMesh { filename } {
    set fail 0
    variable meshio_gid_element
    variable meshio_num_nodes_per_cell
    set file_type ""
    if { [file extension $filename] == ".f3grid" } {
        set file_type "flac3d"
    }

    set m [GiD_Python_Call gid_meshio.my_meshio_read_mesh $filename]
    set nodes_coordinates [lindex [GiD_Python_PythonArrayToTclList [lindex $m 0]] 0]
    set meshio_element_types_and_connectivities [GiD_Python_PythonArrayToTclList [lindex $m 1]]
    set layer_name [GiD_Layers get to_use]
    set offset_nodes [GiD_Info mesh MaxNumNodes]
    set offset_elements [GiD_Info mesh MaxNumElements]
    set last_element_id $offset_elements

    set num_nodes [llength $nodes_coordinates]
    set node_ids [objarray new_from_to intarray [expr $offset_nodes+1] [expr $offset_nodes+$num_nodes]]
    set node_coordinates [objarray new doublearray [expr $num_nodes*3] 0.0] 
    # initialize to 0.0 because for 2d coordinates xyz has only xy the z is missing
    set i 0
    foreach xyz $nodes_coordinates {
        objarray set $node_coordinates $i [string trim $xyz]
        incr i 3
    }
    set i_mesh 0
    foreach {meshio_element_type meshio_connectivities} $meshio_element_types_and_connectivities {
        set element_type ""
        set element_num_nodes 0
        if { [info exists meshio_gid_element($meshio_element_type)] } {
            set element_type $meshio_gid_element($meshio_element_type)
            set element_num_nodes $meshio_num_nodes_per_cell($meshio_element_type)
        } else {
            W "element $meshio_element_type not supported"
            continue
        }
        set elements [lindex $meshio_connectivities 0]
        set num_elements [llength $elements]
        set element_ids [objarray new_from_to intarray [expr $last_element_id+1] [expr $last_element_id+$num_elements]]
        set element_connectivities [objarray new intarray [expr $num_elements*$element_num_nodes]]
        set i 0
        foreach element $elements {
            objarray set $element_connectivities $i $element
            incr i [llength $element]
        }
        objarray incr $element_connectivities [expr {$offset_nodes+1}]
        GiD_MeshPre create $layer_name $element_type $element_num_nodes $node_ids $node_coordinates $element_ids $element_connectivities
        set mesh_offset_elements($i_mesh) $last_element_id
        incr last_element_id $num_elements
        incr i_mesh
    }
    if { ![llength $meshio_element_types_and_connectivities] && $num_nodes } {
        #the mesh has only nodes without any element, create these isolated nodes
        set element_type ""
        set element_num_nodes 0
        set element_ids ""
        set element_connectivities ""
        GiD_MeshPre create $layer_name $element_type $element_num_nodes $node_ids $node_coordinates $element_ids $element_connectivities
    }
    # cell/node data of special cases of Gmsh integer ids create groups
    # and cell/node sets create groups
    if { 1 } {

        foreach {key value} [lindex $m 4] {
            if { $key == "med:nom" } {
                #values are names of results
                continue
            }
            set group_id [lindex [lindex [GiD_Python_PythonArrayToTclList $value] 0] 0]
            set field_name_by_id($group_id) $key
            if { ![GiD_Groups exists $key] } {
                GiD_Groups create $key
            }
        }

        foreach {key value} [lindex $m 2] {
            if { $key == "point_tags" || $key == "medit:ref" || $key == "tetgen:ref" || $key == "ugrid:ref" } {
                #e.g. .met
                set node_id $offset_nodes
                set tag_ids [lindex [GiD_Python_PythonArrayToTclList $value] 0]
                foreach tag_id $tag_ids {
                    incr node_id
                    set selection [list $node_id]
                    set group "_point_tags//$tag_id"
                    if { ![GiD_Groups exists $group] } {
                        GiD_Groups create $group
                    }
                    GiD_EntitiesGroups assign $group nodes $selection
                }
            } else {
                #do nothing: probably is a result on nodes with float values, not ids of elements
                #in postprocess can create a nodal result...
                set to_stop 1
            }
        }


        foreach {key value} [lindex $m 3] {
            if { $key == "gmsh:physical" || $key == "netgen:index" } {
                #try to use previously created groups with name instead generic name based on index
                set element_id $offset_elements
                foreach mesh_group_ids $value {
                    set group_ids [lindex [GiD_Python_PythonArrayToTclList $mesh_group_ids] 0]
                    foreach group_id $group_ids {
                        incr element_id
                        set selection [list $element_id]
                        if { [info exists field_name_by_id($group_id)] } {
                            #created previously and use its name instead generic name index
                            set group $field_name_by_id($group_id)
                        } else {
                            set group "_cell_tags//$group_id"
                            if { ![GiD_Groups exists $group] } {
                                GiD_Groups create $group
                            }
                        }
                        GiD_EntitiesGroups assign $group elements $selection
                    }
                }
            } elseif { $key == "gmsh:geometrical" ||$key == "cell_tags" || $key == "medit:ref" || $key == "su2:tag" || $key == "tetgen:ref" || $key == "ugrid:ref" } {
                #element's group or layer of gmsh file
                set element_id $offset_elements
                foreach mesh_group_ids $value {
                    set group_ids [lindex [GiD_Python_PythonArrayToTclList $mesh_group_ids] 0]
                    foreach group_id $group_ids {
                        incr element_id
                        set selection [list $element_id]
                        set group "_cell_tags//$group_id"
                        if { ![GiD_Groups exists $group] } {
                            GiD_Groups create $group
                        }
                        GiD_EntitiesGroups assign $group elements $selection
                    }
                }
            } elseif { $key == "gmsh:dim_tags" ||  $key == "cell_ids" } {
                #do nothing
                set to_stop 1
            } else {
                #do nothing: probably is a result on elements with float values, not ids of elements
                #in postprocess can create a elemental (1-gauss point) result...
                set to_stop 1
            }
        }

        #nodes set -> group, there is a single list of nodes
        foreach {key value} [lindex $m 5] {
            set group $key
            if { $group == "" } {
                set group unnamed_nodes
            }
            if { ![GiD_Groups exists $group] } {
                GiD_Groups create $group
            }
            set selection [list]
            set local_entity_ids [lindex [GiD_Python_PythonArrayToTclList $value] 0]
            foreach local_entity_id $local_entity_ids {
                set node_id [expr $local_entity_id+$offset_nodes+1]
                lappend selection $node_id
            }
            if { [llength $selection] } {
                GiD_EntitiesGroups assign $group nodes $selection
            }
        }

        #elements set -> group, there is a list of elements for each mesh
        foreach {key value} [lindex $m 6] {
            if { $key == "gmsh:bounding_entities" } {
                # this information is related to ids of boundary of geometry, negative indices mean opposite orientation
                # ignore it by now..
                continue
            }
            set group $key
            if { $group == "" } {
                set group unnamed_elements
            }
            if { ![GiD_Groups exists $group] } {
                GiD_Groups create $group
            }
            set negative_indices 0
            set selection [list]
            set i_mesh 0
            foreach meshentity_ids $value {
                set local_entity_ids [lindex [GiD_Python_PythonArrayToTclList $meshentity_ids] 0]
                foreach local_entity_id $local_entity_ids {
                    if { $local_entity_id >=0 } {
                        if { $file_type == "flac3d" } {
                            #it seems that flac3d .f3grid files have absolute ids, not relative in each mesh
                            set element_id [expr $local_entity_id+1]
                        } else {
                            set element_id [expr $local_entity_id+$mesh_offset_elements($i_mesh)+1]
                        }
                        lappend selection $element_id
                    } else {
                        incr negative_indices
                    }
                }
                incr i_mesh
            }
            if { $negative_indices } {
                W "MeshIO::ReadPreUnstructuredMesh element set '$key' with $negative_indices negative indices"
            }
            if { [llength $selection] } {
                GiD_EntitiesGroups assign $group elements $selection
            }
        }
    }
    return $fail
}


#########################################################################################
######################################### WRITE PRE #####################################
#########################################################################################

proc MeshIO::WritePreW { } {
    set types [list]
    foreach {name extensions} {Abaqus .inp "ANSYS msh" .msh AVS-UCD .avs CGNS .cgns "DOLFIN XML" .xml Exodus {.e .exo} \
        FLAC3D .f3grid H5M .h5m Kratos/MDPA .mdpa Medit {.mesh .meshb} MED/Salome .med Nastran {.bdf .fem .nas} Netgen {.vol .vol.gz} \
        OBJ .obj OFF .off PERMAS {.post .post.gz .dato .dato.gz} PLY .ply STL .stl Tecplot .dat TetGen {.node .ele} SVG .svg SU2 .su2 \
        UGRID .ugrid VTK .vtk VTU .vtu "WKT (TIN)" .wkt XDMF {.xdmf .xmf}} {
            lappend types [list $name $extensions]
    }
    lappend types {{All files} {.*}}
    set filename [MessageBoxGetFilename file write [_ "Write MeshIO file"] {} $types]
    if {$filename != ""} {
        set fail [MeshIO::InitPython]
        if { $fail } {
            return 1
        }
        GidUtils::WaitState
        set tcl_error [catch { set fail [GidUtils::EvalAndEnterInBatch MeshIO::WritePre $filename] } err options]
        GidUtils::EndWaitState
        GiD_RaiseEvent GiD_Event_AfterSaveFile $filename MESHIO_FORMAT 0
        GidUtils::SetWarnLine [_ "File %s written." $filename]
        if { $tcl_error } {
            return -options $options $err
        }
    }
}

proc MeshIO::WritePre { filename } {
    set fail 0
    set fail [MeshIO::InitPython]
    if { $fail } {
        return 1
    }
    if { [file exists $filename] } {
        if { [catch { file delete $filename } err options] } {
            WarnWin $err
            return 1
        }
    }
    if { ![lindex [GiD_Info Mesh] 0] } {
        WarnWin [_ "There is no mesh"]
        return 1
    }
    set tcl_error [catch { set fail [MeshIO::WritePreUnstructuredMesh $filename] } err options]
    if { $tcl_error } {
        return -options $options $err
    }
    return $fail
}

proc MeshIO::TclObjarrayToPythonArrayPoints { node_xyzs } {
    set points ""
    lassign $node_xyzs xs ys zs
    set num_nodes [objarray length $xs]
    for {set i_node 0} {$i_node<$num_nodes} {incr i_node} {
        set x [objarray get $xs $i_node]
        set y [objarray get $ys $i_node]
        set z [objarray get $zs $i_node]
        append points "\[$x,$y,$z\],"
    }
    return "\[$points\]"
}

proc MeshIO::TclObjarrayToPythonArrayConnectivities { element_num_nodes connectivities } {
    set cells ""
    set num_elements [expr [objarray length $connectivities]/$element_num_nodes]
    set i 0
    for {set i_element  0} {$i_element<$num_elements} {incr i_element} {
        set node_ids [list]
        for {set i_node  0} {$i_node<$element_num_nodes} {incr i_node} {
            lappend node_ids [objarray get $connectivities $i]
            incr i
        }
        append cells "\[[join $node_ids ,]\],"
    }
    return "\[$cells\]"
}

proc MeshIO::WritePreUnstructuredMesh { filename } {
    set fail 0
    variable meshio_element_name
    # coordinates
    lassign [GiD_Info mesh nodes -array] node_ids node_xyzs

    set max_id_nodes [objarray get $node_ids end]
    set num_nodes [objarray length [lindex $node_xyzs 0]]
    set new_node_ids ""
    set nodes_renumbered_for_meshio 0
    if { $max_id_nodes != $num_nodes } {
        set nodes_renumbered_for_meshio 1
        set new_node_ids [objarray new_from_to intarray 0 [expr $num_nodes-1]]
    }
    set points [MeshIO::TclObjarrayToPythonArrayPoints $node_xyzs]
    set cells "\["
    foreach element_type {linear triangle quadrilateral tetrahedra pyramid prism hexahedra } {
        set elements_data [lindex [GiD_Info mesh elements $element_type -array2] 0]
        if { [llength $elements_data] } {
            lassign $elements_data element_type_ret element_ids connectivities materials
            set num_elements_block [objarray length $element_ids]
            if { $num_elements_block } {
                if { $nodes_renumbered_for_meshio } {
                    objarray renumber $connectivities $node_ids $new_node_ids
                } else {
                    objarray incr $connectivities -1 ;#meshio is zero based
                }
                set some_element_id [objarray get $element_ids 0]
                set element_num_nodes [llength [GiD_Mesh get element $some_element_id connectivities]]
                set meshio_etype $meshio_element_name($element_type,$element_num_nodes)
                append cells "(\"$meshio_etype\",[MeshIO::TclObjarrayToPythonArrayConnectivities $element_num_nodes $connectivities]),"
            }
        }
    }
    append cells "\]"
    set result [GiD_Python_Call gid_meshio.my_meshio_write_mesh $points $cells $filename]
    return $fail
}

#similar but asking GiD data from Python and processing there the information
proc MeshIO::WritePreUnstructuredMesh2 { filename } {
    set fail 0
    set result [GiD_Python_Call gid_meshio.my_meshio_write_mesh2 $filename]
    return $fail
}

#########################################################################################
######################################### OTHER #########################################
#########################################################################################

proc MeshIO::AddToMenuPRE { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "Meshio..."] PRE] == -1 } {
        #try to insert this menu after the word "Files->Import->XYZ nodes"
        set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
        if { $position == -1 } {
            set position end
        }
        GiDMenu::InsertOption Files [list Import "Meshio..."] $position PRE MeshIO::ReadPreW "" "" insertafter _
    }

    if { [GiDMenu::GetOptionIndex Files [list Export "Meshio..."] PRE] == -1 } {
        #try to insert this menu after the word "Files->Export->GiD mesh"
        set position [GiDMenu::GetOptionIndex Files [list Export "GiD mesh"] PRE]
        if { $position == -1 } {
            set position end
        }
        GiDMenu::InsertOption Files [list Export "Meshio..."] $position PRE MeshIO::WritePreW "" "" insertafter _
    }
}

proc MeshIO::AddToMenu { } {
    MeshIO::AddToMenuPRE
}

MeshIO::Init
#invoke this menus changes
MeshIO::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc MeshIO::AddToMenu

#register the proc to be automatically called when dropping a file
#foreach extension { .stl } {
#    GiD_RegisterExtensionProc $extension PRE MeshIO::ReadPreSingleFile
#}

foreach extension { .off .vol } {
    GiD_RegisterExtensionProc $extension PRE MeshIO::ReadPreSingleFile
}