# Abaqus read plugin .inp mesh

namespace eval Abaqus {
    variable info_plugin ;#information read from .xml plugin file
    #internal use variables
    variable percent_done
    variable stop
    variable _merge  1 ;#to set in file browser options if want to merge or delete old model+results
    variable _show_advance_bar 1
    variable debug 0

    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

#to add more options to the standar file browser
proc Abaqus::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkGeometry -text [_ "Merge"] -variable Abaqus::_merge
    grid columnconfigure $f 1 -weight 1
    grid $f.chkGeometry -sticky w -columnspan 2

    variable export_dir
    if { ![info exists export_dir] } {
        #trick to show the more options
        set export_dir open
    }
    return Abaqus::export_dir
}

proc Abaqus::ReadPreW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read Abaqus mesh"] \
        {} [list [list {Abaqus} {.inp }] [list [_ "All files"] {.*}]] {} $multiple \
        [list [_ "Import options"] Abaqus::MoreImportOptions]]
    if { [llength $filenames] } {
        if { !$_merge } {
            GiD_Process Mescape Files New No escape ;#to avoid previous geometry
        }
        GidUtils::EvalAndEnterInBatch Abaqus::ReadPre $filenames
    }
}


#to drag and drop a file
proc Abaqus::ReadPreSingleFile { filename } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch Abaqus::ReadPre [list $filename]
}

proc Abaqus::ReadPre { filenames } {
    variable info_plugin
    set fail 0
    GidUtils::WaitState
    GidUtils::DisableGraphics
    if { [catch { set fail [Abaqus::ReadPre0 $filenames] } err options] } {
        set fail 1
    }
    GidUtils::EnableGraphics
    GidUtils::EndWaitState
    if { $fail } {
        return -options $options $err
    } else {
        GiD_RaiseEvent GiD_Event_AfterChangeMesh [GiD_Info Mesh NumNodes] [GiD_Info Mesh NumElements]
        MeshView 0
        GiD_Process 'Zoom Frame escape
        GidUtils::UpdateWindow LAYER
        GidUtils::UpdateWindow GROUPS        
        GidUtils::SetWarnLine [_ "%s file read" $info_plugin(Name)]
    }
    return $fail
}

proc Abaqus::GetGiDElementTypeAndNnodes { abaqus_element_type } {
    set linear_2 {B21 B21H B23 B23H B31 B31H GK3D2 GK3D2N PIPE21 PIPE21H SAX1 T2D2 T2D2H T2D3 T2D3H T3D2 T3D3H}
    set linear_3 {B22 B22H B32 B32H B32R B33 B33H B34 D PIPE31 PIPE31H}
    set triangles_3 {CPS3 DS3 M3D3 R3D3 S3 S3R STRI3 STRI35}
    set triangles_6 {CAX6 CPE6 DS6 M3D6 S6 STRI65 U6-SOLID}
    set quadrilateral_4 {CAX4P CPS4 CPS4R DS4 M3D4 M3D4R R3D4 S4 S4R S4RS S4RSW S4R5}
    set quadrilateral_8 {CAX8 CAX8R CPS8 CPS8R DS8 M3D8 M3D8R S8 S8R S8R5 S8RT}
    set quadrilateral_9 {S9 S9R5}
    set tetrahedra_4 {C3D4 C3D4E C3D4H DC3D4 DC3D4E}
    set tetrahedra_10 {C3D10 C3D10E C3D10H C3D10I C3D10M C3D10MH DC3D10 DC3D10E}
    set hexahedra_8 {AC3D8 C3D8 C3D8E C3D8H C3D8HT C3D8I C3D8IH C3D8R C3D8RH C3D8T DC3D8 DC3D8E DCC3D8D DK3D8N F3D8 GK3D8}
    set hexahedra_20 {AC3D20 C3D20 C3D20E C3D20H C3D20HT C3D20R C3D20RH C3D20RHT C3D20RT C3D20T CDD20RE DC3D20 DC3D20E}
    set hexahedra_27 {C3D27}
    set prism_6 {C3D6 C3D6E C3D6H DC3D6 DC3D6E F3D6 GK3D6 GK3D6N}
    set prism_12 {GK3D12M}
    set prism_15 {C3D15 C3D15E C3D15H DC3D15 DC3D15E}
    set pyramid_4 {C3D5}
    if { [lsearch $linear_2 $abaqus_element_type] != -1  } {
        set element_type Linear
        set element_num_nodes 2
    } elseif { [lsearch $linear_3 $abaqus_element_type] != -1  } {
        set element_type Linear
        set element_num_nodes 3
    } elseif { [lsearch $triangles_3 $abaqus_element_type] != -1  } {
        set element_type Triangle
        set element_num_nodes 3
    } elseif { [lsearch $triangles_6 $abaqus_element_type] != -1  } {
        set element_type Triangle
        set element_num_nodes 6
    } elseif { [lsearch $quadrilateral_4 $abaqus_element_type] != -1  } {
        set element_type Quadrilateral
        set element_num_nodes 4
    } elseif { [lsearch $quadrilateral_8 $abaqus_element_type] != -1  } {
        set element_type Quadrilateral
        set element_num_nodes 8
    } elseif { [lsearch $quadrilateral_9 $abaqus_element_type] != -1  } {
        set element_type Quadrilateral
        set element_num_nodes 9
    } elseif { [lsearch $tetrahedra_4 $abaqus_element_type] != -1  } {
        set element_type Tetrahedra
        set element_num_nodes 4
    } elseif { [lsearch $tetrahedra_10 $abaqus_element_type] != -1  } {
        set element_type Tetrahedra
        set element_num_nodes 10
    } elseif { [lsearch $hexahedra_8 $abaqus_element_type] != -1  } {
        set element_type Hexahedra
        set element_num_nodes 8
    } elseif { [lsearch $hexahedra_20 $abaqus_element_type] != -1  } {
        set element_type Hexahedra
        set element_num_nodes 20
    } elseif { [lsearch $hexahedra_27 $abaqus_element_type] != -1  } {
        set element_type Hexahedra
        set element_num_nodes 27
    } elseif { [lsearch $prism_6 $abaqus_element_type] != -1  } {
        set element_type Prism
        set element_num_nodes 6
    } elseif { [lsearch $prism_12 $abaqus_element_type] != -1  } {
        set element_type Prism
        set element_num_nodes 12
    } elseif { [lsearch $prism_15 $abaqus_element_type] != -1  } {
        set element_type Prism
        set element_num_nodes 15
    } elseif { [lsearch $pyramid_4 $abaqus_element_type] != -1  } {
        set element_type Pyramid
        set element_num_nodes 5
    } else {
        W "Abaqus::GetGiDElementTypeAndNnodes. Unexpected element type $abaqus_element_type"
        set element_type ""
        set element_num_nodes 0
    }
    return [list $element_type $element_num_nodes]
}

proc Abaqus::CloneSubgroupInGroup { group_name sub_group_name } {
    if { [GiD_Groups exists $sub_group_name] } {
        set tmp_group [Groups::GetAutomaticName ""]
        GiD_Groups clone $sub_group_name $tmp_group
        GiD_Groups edit parent $tmp_group $group_name
        set full_tmp_group [Groups::NameJoin $group_name $tmp_group]
        set full_sub_group_name [Groups::NameJoin $group_name $sub_group_name]
        GiD_Groups edit name $full_tmp_group $full_sub_group_name
    }
    return 0
}

proc Abaqus::GetListFromLine { line } {
    set list_items [list]
    foreach item [split $line ,] {
        lappend list_items [string trim $item]
    }
    return $list_items
}

#pre_post must be pre or post
#if offset_nodes or offset_elements some input arrays can be
proc Abaqus::CreateGiD_Mesh { pre_post meshname gid_element_type gid_element_num_nodes zero_based_array \
    gid_node_ids gid_node_coordinates gid_element_ids gid_element_connectivities {offset_nodes 0} {offset_elements 0}} {
    set max_node_id_before [GiD_Info Mesh MaxNumNodes]
    set max_element_id_before [GiD_Info Mesh MaxNumElements]
    if { $offset_nodes } {
        # check if gipost_node_ids exists, if not --> create it
        # may be gid_node_coordinates is empty, i.e. not an objarray
        if { [GidUtils::IsObjarray $gid_node_coordinates] } {
            if { ![GidUtils::IsObjarray $gid_node_ids] } {
                set num_nodes [expr [objarray length $gid_node_coordinates]/3]
                set gid_node_ids [objarray new_from_to intarray 0 [expr $num_nodes-1]]
            }
        }
        objarray incr $gid_node_ids $offset_nodes
        objarray incr $gid_element_connectivities $offset_nodes
    }
    if { $offset_elements } {
        # check if gipost_node_ids exists, if not --> create it
        if { ![GidUtils::IsObjarray $gid_element_ids] } {
            set num_elements [expr [llength $gid_element_connectivities]/$gid_element_num_nodes]
            set gid_element_ids [objarray new_from_to intarray 0 [expr $num_elements-1]]
        }
        objarray incr $gid_element_ids $offset_elements
    }
    if { $pre_post == "pre" } {
        set gid_mesh_command GiD_MeshPre
    } else {
        set gid_mesh_command GiD_MeshPost
    }
    if { $zero_based_array == "-zero_based_array" } {
        $gid_mesh_command create $meshname $gid_element_type $gid_element_num_nodes -zero_based_array $gid_node_ids $gid_node_coordinates $gid_element_ids $gid_element_connectivities
    } else {
        $gid_mesh_command create $meshname $gid_element_type $gid_element_num_nodes $gid_node_ids $gid_node_coordinates $gid_element_ids $gid_element_connectivities
    }
    set max_node_id_after [GiD_Info Mesh MaxNumNodes]
    set max_element_id_after [GiD_Info Mesh MaxNumElements]
    return [list [expr $max_node_id_before+1] $max_node_id_after [expr $max_element_id_before+1] $max_element_id_after]
}

#type: nodes, elements
proc Abaqus::AssignEntitiesToGroup { type group_items group_name } {
    if { $group_name != "" && [llength $group_items] } {
        if { ![GiD_Groups exists $group_name] } {
            GiD_Groups create $group_name
        }
        GiD_EntitiesGroups assign $group_name $type $group_items
    }
    return 0
}

#return line_next because need to read one line more, othe option is to set back the $fp pos before gets
proc Abaqus::ReadKey_NODE { fp } {
    #or to avoid two copies as list and as objarray can do a loop to count and allocate objarray and another to fill (or a loop and resize+copy several times)
    set node_ids_list [list]
    set node_coordinates_list [list]
    while { ![eof $fp] } {
        set position_before_gets [tell $fp]
        gets $fp line
        if { $line == "" || [string range $line 0 1] == "**" } {
            #comment
            continue
        } elseif { [string index $line 0] == "*" } {
            seek $fp $position_before_gets
            break
        } else {
            lassign [string map {, { }} $line] node_id x y z
            if { $z == "" } {
                set z 0
            }
            lappend node_ids_list $node_id
            lappend node_coordinates_list $x $y $z
        }
    }
    set gid_node_ids [objarray new intarray -values $node_ids_list]
    set gid_node_coordinates [objarray new floatarray -values $node_coordinates_list]
    return [list $gid_node_ids $gid_node_coordinates]
}

#check if use trick of encode pyramid C3D5 as C3D8R repeating 4 times at the ent the same last node
proc Abaqus::ElementHexaIsReallyPyramid { element_connectivities } {
    set is_pyramid 1
    set node_check [lindex $element_connectivities 4]
    foreach node_i [lrange $element_connectivities 5 end] {
        if { $node_i != $node_check} {
            set is_pyramid 0
            break
        }
    }
    return $is_pyramid
}

proc Abaqus::ReadKey_ELEMENT { fp gid_element_type gid_element_num_nodes} {
    set gid_element_ids_list [list]
    set gid_element_connectivities_list [list]
    set is_really_pyramid ""
    while { ![eof $fp] } {
        set position_before_gets [tell $fp]
        gets $fp line
        if { $line == "" || [string range $line 0 1] == "**" } {
            #comment
            continue
        } elseif { [string index $line 0] == "*" } {
            seek $fp $position_before_gets
            break
        } else {
            while { [string index $line end] == "," && ![eof $fp] } {
                gets $fp line_extra
                set line $line$line_extra
            }
            set element_items [string map {, { }} $line]
            set element_id [lindex $element_items 0]
            set element_connectivities [lrange $element_items 1 end]
            if { $gid_element_type == "Hexahedra" } {
                if { $gid_element_num_nodes == 20 } {
                    #Abaqus has different connectivities that GiD
                    set block_1 [lrange $element_connectivities 0 11]
                    set block_2 [lrange $element_connectivities 16 19]
                    set block_3 [lrange $element_connectivities 12 15]
                    set element_connectivities [concat $block_1 $block_2 $block_3]
                } elseif { $gid_element_num_nodes == 8 } {
                    if { $is_really_pyramid == "" } {
                        set is_really_pyramid [Abaqus::ElementHexaIsReallyPyramid $element_connectivities]
                    }
                    if { $is_really_pyramid } {
                        #C3D5
                        set element_connectivities [lrange $element_connectivities 0 4]
                    }
                }
            } elseif { $gid_element_type == "Prism" } {
                if { $gid_element_num_nodes == 15 } {
                    #Abaqus has different connectivities that GiD
                    set block_1 [lrange $element_connectivities 0 8]
                    set block_2 [lrange $element_connectivities 12 14]
                    set block_3 [lrange $element_connectivities 9 11]
                    set element_connectivities [concat $block_1 $block_2 $block_3]
                }
            }
            lappend gid_element_ids_list $element_id
            lappend gid_element_connectivities_list {*}$element_connectivities
        }
    }
    if { $is_really_pyramid == 1 } {
        set gid_element_type Pyramid
        set gid_element_num_nodes 5
    }
    set gid_element_ids [objarray new intarray -values $gid_element_ids_list]
    set gid_element_connectivities [objarray new intarray -values $gid_element_connectivities_list]
    return [list $gid_element_type $gid_element_num_nodes $gid_element_ids $gid_element_connectivities]
}

proc Abaqus::ReadKey_NSET { fp group_name generate } {
    set group_items_list [list]
    while { ![eof $fp] } {
        set position_before_gets [tell $fp]
        gets $fp line
        if { $line == "" || [string range $line 0 1] == "**" } {
            #comment
            continue
        } elseif { [string index $line 0] == "*" } {
            seek $fp $position_before_gets
            break
        } else {
            set items [Abaqus::GetListFromLine $line]
            if { $generate } {
                lassign $items from to increment
                for {set i $from} {$i<=$to} {incr i $increment} {
                    lappend group_items_list $i
                }
            } else {
                foreach item $items {
                    if { [string is integer -strict $item] } {
                        lappend group_items_list $item
                    } else {
                        Abaqus::CloneSubgroupInGroup $group_name $item
                    }
                }
            }
        }
    }
    set group_items [objarray new intarray -values $group_items_list]
    return $group_items
}

proc Abaqus::ReadKey_ELSET { fp group_name generate } {
    set group_items_list [list]
    while { ![eof $fp] } {
        set position_before_gets [tell $fp]
        gets $fp line
        if { $line == "" || [string range $line 0 1] == "**" } {
            #comment
            continue
        } elseif { [string index $line 0] == "*" } {
            seek $fp $position_before_gets
            break
        } else {
            #expects a list of existing elset (converted to groups)
            set items [Abaqus::GetListFromLine $line]
            if { $generate } {
                lassign $items from to increment
                for {set i $from} {$i<=$to} {incr i $increment} {
                    lappend group_items_list $i
                }
            } else {
                foreach item $items {
                    if { [string is integer -strict $item] } {
                        lappend group_items_list $item
                    } else {
                        Abaqus::CloneSubgroupInGroup $group_name $item
                    }
                }
            }
        }
    }
    set group_items [objarray new intarray -values $group_items_list]
    return $group_items
}

proc Abaqus::ReadKey_to_ignore { fp } {
    while { ![eof $fp] } {
        set position_before_gets [tell $fp]
        gets $fp line
        if { $line == "" || [string range $line 0 1] == "**" } {
            #comment
            continue
        } elseif { [string index $line 0] == "*" } {
            seek $fp $position_before_gets
            break
        } else {
            #do something...
        }
    }
    return 0
}

#to get an unused name, based on "prefix label"
proc Abaqus::GetPostMeshNameAutomatic { filename label } {
    set prefix [file rootname [file tail $filename]]
    set new_name "${prefix} ${label}"
    set names [concat [GiD_Info postprocess get all_surfacesets] [GiD_Info postprocess get all_volumesets]]
    if { [lsearch $names $new_name] == -1 } {
        set name $new_name
    } else {
        set i 1
        while { 1 } {
            set name ${new_name}_$i
            if { [lsearch $names $name] == -1 } {
                break
            }
            incr i
        }
    }
    return $name
}

#tricky proc to allow be called recursivelly to allow to split a .inp file in several include files
proc Abaqus::ReadPreMeshFilePartial { filename } {
    variable debug
    set fail 0
    set keys_to_ignore [lsort -dictionary {AMPLITUDE {BEAM SECTION} BOUNDARY BOUNDARYF BUCKLE CFLUX
    {CHANGE FRICTION} {CHANGE MATERIAL} {CHANGE PLASTIC} CLOAD CONDUCTIVITY
    {CONTACT FILE} {CONTACT PAIR} CONTROLS {COUPLED TEMPERATURE-DISPLACEMENT} CREEP {CYCLIC HARDENING}
    DAMPING {DEFORMATION PLASTICITY} DENSITY DEPVAR DFLUX DLOAD DSLOAD DYNAMIC
    {EL FILE} {EL PRINT} ELASTIC {END STEP} ENDSTEP EQUATION EXPANSION {FACE PRINT} FILM {FLUID CONSTANTS} {FLUID SECTION}
    FREQUENCY FRICTION HEADING {HEAT TRANSFER} HYPERELASTIC HYPERFOAM
    {INITIAL CONDITIONS}
    MATERIAL MEMBRANE {MODAL DAMPING} {MODAL DYNAMIC} {MODEL CHANGE} {MOHR COULOMB}  {MOHR COULOMB HARDENING} MPC
    {NO ANALYSIS} {NODE FILE} {NODE OUTPUT} {NODE PRINT} ORIENTATION
    PERMEABILITY {PHYSICAL CONSTANTS} PLASTIC RADIATE RESTART {RIGID BODY}
    {SHELL SECTION} SOILS {SOLID SECTION} {SPECIFIC GAS CONSTANT} {SPECIFIC HEAT} STATIC {STEADY STATE DYNAMICS} STEP
    SURFACE {SURFACE BEHAVIOR} {SURFACE INTERACTION} TEMPERATURE
    TIE {TIME POINTS} TRANSFORM TRANSFORMF {USER MATERIAL} {VALUES AT INFINITY} VIEWFACTOR VISCO}]

    #it is required to know and modify the variables that exists at calling level, to split a .inp file in several include files
    foreach caller_var_name {pre_post zero_based_array offset_nodes offset_elements max_num_nodes_before part_current assembly_current instance_current gid_meshet_nodes_and_coordinates} {
        upvar 1 $caller_var_name $caller_var_name

    }

    set fp [open $filename r]
    if { $fp == "" } {
        WarnWin [_ "Cannot open file '%s'" $filename]
        return 1
    }

    while { ![eof $fp] } {
        gets $fp line
        if { $line == "" } {
            continue
        } elseif { [string range $line 0 1] == "**" } {
            #comment
            continue
        } elseif { [string index $line 0] == "*" } {
            set items [split $line ,]
            set key [string toupper [string trim [string range [lindex $items 0] 1 end]]]
            #extra attributes
            array unset abaqus_key_attributes
            foreach item [lrange $items 1 end] {
                lassign [split $item =] subkey value
                set abaqus_key_attributes([string toupper [string trim $subkey]]) [string trim $value]
            }
            if { $key == "ASSEMBLY" } {
                set assembly_name $abaqus_key_attributes(NAME)
                #create a layer to be parent of the instances inside (each instance points to a part, that is implemented as a GiD Layer)
                set layer $assembly_name
                if { ![GiD_Layers exists $layer] } {
                    GiD_Layers create $layer
                }
                set assembly_current $assembly_name
            } elseif { $key == "END ASSEMBLY" } {
                set assembly_current ""
            } elseif { $key == "END INSTANCE" } {
                set instance_current ""
            } elseif { $key == "END PART" } {
                #can for example assign a group to store information of parts
                set part_current ""
            } elseif { $key == "ELEMENT" } {
                #attributes: type elset
                lassign [Abaqus::GetGiDElementTypeAndNnodes [string toupper $abaqus_key_attributes(TYPE)]] gid_element_type gid_element_num_nodes
                set group_name ""
                if { [info exists abaqus_key_attributes(ELSET)] } {
                    set group_name $abaqus_key_attributes(ELSET)
                }
                #return also id_element_type gid_element_num_nodes because can change from Hexahedra 8 to Pyramid 5 depending on Abaqus encoding tricks
                lassign [Abaqus::ReadKey_ELEMENT $fp $gid_element_type $gid_element_num_nodes] gid_element_type gid_element_num_nodes gid_element_ids gid_element_connectivities
                if { $pre_post == "pre" } {
                    set meshname ""
                } else {
                    set meshname [Abaqus::GetPostMeshNameAutomatic $filename $gid_element_type] ;#??
                }
                lassign $gid_meshet_nodes_and_coordinates gid_node_ids gid_node_coordinates nodes_group_name
                #it is allowed to repeat several times for each mesh the same all nodes
                set res [Abaqus::CreateGiD_Mesh $pre_post $meshname $gid_element_type $gid_element_num_nodes $zero_based_array \
                    $gid_node_ids $gid_node_coordinates $gid_element_ids $gid_element_connectivities $offset_nodes $offset_elements]
                lassign $res node_id_from node_id_to element_id_from element_id_to
                if { $nodes_group_name  != "" } {
                    #do it only once...
                    #set group_items [objarray new_from_to intarray $node_id_from $node_id_to]
                    set group_items [GiD_Mesh list node ${node_id_from}:${node_id_to}]
                    Abaqus::AssignEntitiesToGroup nodes $group_items $nodes_group_name
                }
                if { $group_name != "" } {
                    #set group_items [objarray new_from_to intarray $element_id_from $element_id_to]
                    set group_items [GiD_Mesh list element ${element_id_from}:${element_id_to}]
                    Abaqus::AssignEntitiesToGroup elements $group_items $group_name
                }
            } elseif { $key == "ELSET" } {
                #attributes: elset
                set group_name ""
                if { [info exists abaqus_key_attributes(ELSET)] } {
                    set group_name $abaqus_key_attributes(ELSET)
                }
                set generate 0
                if { [info exists abaqus_key_attributes(GENERATE)] } {
                    set generate 1
                }
                set group_items [Abaqus::ReadKey_ELSET $fp $group_name $generate]
                if { $offset_elements } {
                    objarray incr $group_items $offset_elements
                }
                Abaqus::AssignEntitiesToGroup elements $group_items $group_name
            } elseif { $key == "INCLUDE" } {
                set include_input $abaqus_key_attributes(INPUT)
                if { [file pathtype $include_input] == "relative" } {
                    set include_input [file join [file dirname $filename] $include_input]
                }
                incr fail [Abaqus::ReadPreMeshFilePartial $include_input]
            } elseif { $key == "INSTANCE" } {
                set instance_name $abaqus_key_attributes(NAME)
                set part_name $abaqus_key_attributes(PART)
                if { $assembly_current != "" } {
                    set layer_parent $assembly_current
                    if { [GiD_Layers exists $layer_parent] } {
                        #really each instance must have a copy of the entities of part_name, this is wrong if a part belong to multples instances!!
                        set layer_child $part_name
                        if { [GiD_Layers exists $layer_child] } {
                            GiD_Layers edit parent $layer_child $layer_parent
                        } else {
                            #error
                        }
                    } else {
                        #error
                    }
                }
                set instance_current $instance_name
            } elseif { $key == "NODE" } {
                #attributes: nset
                set group_name ""
                if { [info exists abaqus_key_attributes(NSET)] } {
                    set group_name $abaqus_key_attributes(NSET)
                }
                lassign [Abaqus::ReadKey_NODE $fp] gid_node_ids gid_node_coordinates
                set gid_meshet_nodes_and_coordinates [list $gid_node_ids $gid_node_coordinates $group_name]
            } elseif { $key == "NSET" } {
                #attributes: nset generate instance
                set group_name ""
                if { [info exists abaqus_key_attributes(NSET)] } {
                    set group_name $abaqus_key_attributes(NSET)
                }
                set generate 0
                if { [info exists abaqus_key_attributes(GENERATE)] } {
                    set generate 1
                }
                set instance ""
                if { [info exists abaqus_key_attributes(INSTANCE)] } {
                    set instance $abaqus_key_attributes(INSTANCE)
                }
                set group_items [Abaqus::ReadKey_NSET $fp $group_name $generate]
                #if fact the ids depends on each instance, locally start from 1 in each part...
                if { $offset_nodes } {
                    objarray incr $group_items $offset_nodes
                }
                Abaqus::AssignEntitiesToGroup nodes $group_items $group_name
            } elseif { $key == "PART" } {
                set part_name $abaqus_key_attributes(NAME)
                #each part has local numeration, must use offset to not repeat ids
                set offset_nodes [GiD_Info Mesh MaxNumNodes]
                set offset_elements [GiD_Info Mesh MaxNumElements]
                set layer $part_name
                if { ![GiD_Layers exists $layer] } {
                    GiD_Layers create $layer
                }
                GiD_Layers edit to_use $layer
                set part_current $part_name
            } elseif { [lsearch -sorted $keys_to_ignore $key] != -1 } {
                #attributes:
                Abaqus::ReadKey_to_ignore $fp
            } elseif { $key == "*" } {
                #comment
            } elseif { $key == "" } {
                #???
            } else {
                set unimplemented_types($key) 1
            }
        } else {
            W "Abaqus::ReadPreMesh. Ignored line $line"
        }
    }
    close $fp
    return $fail
}

proc Abaqus::ReadPreMesh { filename } {
    variable debug
    set fail 0

    #offset_nodes to be added to the number if a previous model exists
    set pre_post pre
    set zero_based_array 0
    set offset_nodes [GiD_Info Mesh MaxNumNodes]
    set offset_elements [GiD_Info Mesh MaxNumElements]
    set max_num_nodes_before $offset_nodes
    set part_current ""
    set assembly_current ""
    set instance_current ""
    set gid_meshet_nodes_and_coordinates [list ]

    set fail [Abaqus::ReadPreMeshFilePartial $filename]

    if { [GiD_Layers exists Layer0] && [llength [GiD_Layers list]] > 1 } {
        #delete Layer0 if possible
        if { ![GiD_Layers get num_entities Layer0] && ![GiD_Layers get num_conditions Layer0] } {
            GiD_Layers delete Layer0
        }
    }

    if { $max_num_nodes_before != [GiD_Info Mesh MaxNumNodes] } {
        GiD_ModifiedFileFlag set 1
    }
    if { $debug } {
        if { [llength [array names unimplemented_types]] } {
            W "Unimplemented types: [lsort -dictionary [array names unimplemented_types]]"
        }
    }
    return $fail
}

proc Abaqus::ReadPre0 { filenames } {
    variable percent_done
    variable stop
    variable _show_advance_bar

    set fail 0
    set ::Abaqus::stop 0
    set ::Abaqus::percent_done 0
    set percent_max [llength $filenames]
    if { $_show_advance_bar } {
        GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::Abaqus::percent_done $percent_max {set ::Abaqus::stop 1}
    }
    set create_layer_filename 0
    if { [llength $filenames]>1 } {
        set create_layer_filename 1
    }
    foreach filename $filenames {
        if { $::Abaqus::stop } {
            break
        }
        if { [file exists $filename] } {

            if { $create_layer_filename } {
                set layer [file rootname [file tail $filename]]
                if { ![GiD_Layers exists $layer] } {
                    GiD_Layers create $layer
                }
                GiD_Layers edit to_use $layer
            }
            set res ""
            if { [catch { set local_fail [Abaqus::ReadPreMesh $filename] } err options] } {
                #set fail 1
                if { $_show_advance_bar } {
                    GidUtils::DeleteAdvanceBar
                }
                return -options $options $err
            } else {
                if { $local_fail } {
                    set fail 1
                }
                GiD_RaiseEvent GiD_Event_AfterOpenFile $filename ABAQUS_FORMAT $local_fail
            }

        }
        incr ::Abaqus::percent_done
    }
    if { $_show_advance_bar } {
        GidUtils::DeleteAdvanceBar
    }
    return $fail
}

proc Abaqus::AddToMenu { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "Abaqus..."] PRE] != -1 } {
        return
    }

    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import "Abaqus..."] $position PRE Abaqus::ReadPreW "" "" insertafter _
}


#invoke this menus changes
Abaqus::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc Abaqus::AddToMenu

#register the proc to be automatically called when dropping a file
GiD_RegisterExtensionProc ".Abaqus .inp" PRE Abaqus::ReadPreSingleFile
