#Amelet read/write plugin

namespace eval Amelet {
    variable info_plugin ;#information read from .xml plugin file
    variable compress_level 9

    variable hdf5_required_version 1.6 ;#minimum required version of hdf5 package (better use >=1.7)
    variable hdf5_current_version 0.0 ;#really loaded version

    #auxiliary arrays to map gid - amelet codes
    variable EdgeOrFaceCode ;#amelet face from gid face
    variable GiDFaceId ;#inverse: gid face from amelet face
    variable AmeletLevelType ;#level from amelet code
    variable GidPostElementType ;#gid pos type from amelet code
    variable GidPostElementNNode ;#gid pos number of nodes by element from amelet code
    variable AmeletElementNNode ;#number of nodes used in the Amelet definition of the element

    variable stop
    variable _merge 1;#when importing if merge then mesh is added to old mesh, else previous mesh is deleted

    #internal variables for cartesian grid, with values n0 and n0*n1
    variable _grid_n0
    variable _grid_n01

    variable max_warnings 50 ;#a lot of messages could be too slow
    variable _num_warnings 0 ;#internal counter

    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }

}

proc Amelet::Warning { msg } {
    variable _num_warnings
    variable max_warnings
    incr _num_warnings
    if { $_num_warnings == $max_warnings } {
        WarnWinText ...
    } elseif { $_num_warnings < $max_warnings } {
        WarnWinText $msg
    }
}

proc Amelet::Init { } {
    variable EdgeOrFaceCode ;#amelet face from gid face
    variable GiDFaceId ;#inverse: gid face from amelet face
    variable AmeletLevelType ;#level from amelet code
    variable GidPostElementType ;#gid pos type from amelet code
    variable GidPostElementNNode ;#gid pos number of nodes by element from amelet code
    variable AmeletElementNNode ;#number of nodes used in the Amelet definition of the element

    #amelet face from gid face
    set EdgeOrFaceCode(Triangle,3,1) 1
    set EdgeOrFaceCode(Triangle,3,2) 2
    set EdgeOrFaceCode(Triangle,3,3) 3
    set EdgeOrFaceCode(Triangle,6,1) {1 2}
    set EdgeOrFaceCode(Triangle,6,2) {3 4}
    set EdgeOrFaceCode(Triangle,6,3) {5 6}
    set EdgeOrFaceCode(Quadrilateral,4,1) 1
    set EdgeOrFaceCode(Quadrilateral,4,2) 2
    set EdgeOrFaceCode(Quadrilateral,4,3) 3
    set EdgeOrFaceCode(Quadrilateral,4,4) 4
    set EdgeOrFaceCode(Quadrilateral,8,1) {1 2}
    set EdgeOrFaceCode(Quadrilateral,8,2) {3 4}
    set EdgeOrFaceCode(Quadrilateral,8,3) {5 6}
    set EdgeOrFaceCode(Quadrilateral,8,4) {7 8}
    set EdgeOrFaceCode(Tetrahedra,4,1) 4
    set EdgeOrFaceCode(Tetrahedra,4,2) 2
    set EdgeOrFaceCode(Tetrahedra,4,3) 3
    set EdgeOrFaceCode(Tetrahedra,4,4) 1
    for {set i 1} {$i <=4} {incr i} {
        set EdgeOrFaceCode(Tetrahedra,10,$i) $EdgeOrFaceCode(Tetrahedra,4,$i)
    }
    set EdgeOrFaceCode(Pyramid,5,1) 1
    set EdgeOrFaceCode(Pyramid,5,2) 2
    set EdgeOrFaceCode(Pyramid,5,3) 3
    set EdgeOrFaceCode(Pyramid,5,4) 4
    set EdgeOrFaceCode(Pyramid,5,5) 5
    set EdgeOrFaceCode(Prism,6,1) 2
    set EdgeOrFaceCode(Prism,6,2) 1
    set EdgeOrFaceCode(Prism,6,3) 4
    set EdgeOrFaceCode(Prism,6,4) 5
    set EdgeOrFaceCode(Prism,6,5) 3
    set EdgeOrFaceCode(Hexahedra,8,1) 1
    set EdgeOrFaceCode(Hexahedra,8,2) 5
    set EdgeOrFaceCode(Hexahedra,8,3) 2
    set EdgeOrFaceCode(Hexahedra,8,4) 3
    set EdgeOrFaceCode(Hexahedra,8,5) 4
    set EdgeOrFaceCode(Hexahedra,8,6) 6
    for {set i 1} {$i <=6} {incr i} {
        set EdgeOrFaceCode(Hexahedra,20,$i) $EdgeOrFaceCode(Hexahedra,8,$i)
    }

    #inverse: gid face from amelet face
    foreach item [array names EdgeOrFaceCode] {
        lassign [split $item ,] elemtype nnodes face_id
        set GiDFaceId([Amelet::ElementCode $elemtype $nnodes],$EdgeOrFaceCode($item)) $face_id
    }

    #level from amelet code
    foreach element_code {101 102 103 104 106 107 108 109} {
        set AmeletLevelType($element_code) volume
    }
    foreach element_code {11 12 13 14} {
        set AmeletLevelType($element_code) face
    }
    foreach element_code {1 2} {
        set AmeletLevelType($element_code) edge
    }

    set GidPostElementType(1) Linear
    set GidPostElementType(2) Linear
    set GidPostElementType(11) Triangle
    set GidPostElementType(12) Triangle
    set GidPostElementType(13) Quadrilateral
    set GidPostElementType(14) Quadrilateral
    set GidPostElementType(101) Tetrahedra
    set GidPostElementType(102) Pyramid
    set GidPostElementType(103) Prism
    set GidPostElementType(104) Hexahedra
    set GidPostElementType(108) Tetrahedra
    set GidPostElementType(109) Hexahedra
    #canonical geometrical elements
    #set GidPostElementType(16) Circle
    #set GidPostElementType(107) Sphere

    set GidPostElementNNode(1) 2
    set GidPostElementNNode(2) 3
    set GidPostElementNNode(11) 3
    set GidPostElementNNode(12) 6
    set GidPostElementNNode(13) 4
    set GidPostElementNNode(14) 8
    set GidPostElementNNode(101) 4
    set GidPostElementNNode(102) 5
    set GidPostElementNNode(103) 6
    set GidPostElementNNode(104) 8
    set GidPostElementNNode(108) 10
    set GidPostElementType(109) 20
    #canonical geometrical elements
    #set GidPostElementNNode(16) 1
    #set GidPostElementNNode(107) 1

    #number of nodes used in the Amelet definition of the element
    #must define all possible types to jump unimplemented ones!!
    set AmeletElementNNode(1) 2
    set AmeletElementNNode(2) 3
    set AmeletElementNNode(11) 3
    set AmeletElementNNode(12) 6
    set AmeletElementNNode(13) 4
    set AmeletElementNNode(14) 8
    set AmeletElementNNode(101) 4
    set AmeletElementNNode(102) 5
    set AmeletElementNNode(103) 6
    set AmeletElementNNode(104) 8
    set AmeletElementNNode(108) 10
    set AmeletElementNNode(109) 20
    #canonical geometrical elements
    set AmeletElementNNode(15) 3 ;#plane
    set AmeletElementNNode(16) 3 ;#circle
    set AmeletElementNNode(17) 3 ;#ellipse
    set AmeletElementNNode(105) 3 ;#cylinder
    set AmeletElementNNode(106) 4 ;#cone
    set AmeletElementNNode(107) 2 ;#sphere
}

proc Amelet::LoadHdf5Package { } {
    variable hdf5_required_version
    variable hdf5_current_version
    if { [catch { package require hdf5 $hdf5_required_version } msg] } {
        Amelet::Warning "Error loading hdf5 package: $msg"
    } else {
        set hdf5_current_version $msg
    }
}

#it return the edge or face number of Amelet from GiD face data
proc Amelet::EdgeOrFaceCode { elemtype nnodes face_id } {
    variable EdgeOrFaceCode
    if { [info exists EdgeOrFaceCode($elemtype,$nnodes,$face_id)] } {
        return $EdgeOrFaceCode($elemtype,$nnodes,$face_id)
    } else {
        return 0
    }
}

#it return GiD face number from the Amelet data
proc Amelet::GiDFaceCode { element_code amelet_face } {
    variable GiDFaceId
    if { [info exists GiDFaceId($element_code,$amelet_face)] } {
        return $GiDFaceId($element_code,$amelet_face)
    } else {
        return 0
    }
}

proc Amelet::ElementEntityType { elemtype } {
    variable AmeletLevelType
    if { [info exists AmeletLevelType($elemtype)] } {
        return $AmeletLevelType($elemtype)
    } else {
        return unknown
    }
}

proc Amelet::GetGidPostElementType { element_code } {
    variable GidPostElementType
    if { [info exists GidPostElementType($element_code)] } {
        return $GidPostElementType($element_code)
    } else {
        return ""
    }
}

proc Amelet::GetGidPostElementNNode { element_code } {
    variable GidPostElementNNode
    if { [info exists GidPostElementNNode($element_code)] } {
        return $GidPostElementNNode($element_code)
    } else {
        return ""
    }
}

proc Amelet::GetAttribute { h path name {default ""}} {
    set value [string trim [$h get_attribute -default $default $path $name]]
    if { $value == "" && [lsearch [$h get_attribute $path] $name] == -1 } {
        set pseudo_name [string tolower $name]
        set value [string trim [$h get_attribute -default $default $path $pseudo_name]]
        if { $value != "" } {
            Amelet::Warning [_ "Warning, expected attribute '%s' but found '%s' for '%s'" $name $pseudo_name $path]
        }
    }
    return $value
}

proc Amelet::GetAttributes { h path } {
    return [$h get_attribute $path]
}

proc Amelet::IsAmelet { h } {
    set FORMAT [Amelet::GetAttribute $h / FORMAT]
    if { $FORMAT == "AMELETHDF" } {
        return 1
    } else {
        return 0
    }
}

#AMELET_bar2=1,AMELET_bar3=2,AMELET_tri3=11,AMELET_tri6=12,
#AMELET_quad4=13,AMELET_quad8=14,AMELET_tetra4=101,AMELET_pyra5=102,AMELET_penta6=103,AMELET_hexa8=104,
#AMELET_plane=15,AMELET_circle=16,AMELET_cylinder=105,AMELET_cone=106,AMELET_sphere=107,
#AMELET_tetra10=108,AMELET_hexa20=109
proc Amelet::ElementCode { elemtype nnodes } {
    if { ( $elemtype == "Linear") || ( $elemtype == "Line")} {
        if { $nnodes == 2 } {
            return 1
        } else {
            return 2
        }
    } elseif { $elemtype == "Triangle" } {
        if { $nnodes == 3 } {
            return 11
        } else {
            return 12
        }
    } elseif { $elemtype == "Quadrilateral" } {
        if { $nnodes == 4 } {
            return 13
        } elseif { $nnodes == 8} {
            return 14
        } elseif { $nnodes == 9} {
            return 14 ;#not exists in Amelet 1.5.3, write as 8 nodes case
        } else {
            return 0 ;#unexpected case
        }
    } elseif { $elemtype == "Tetrahedra" } {
        if { $nnodes == 4 } {
            return 101
        } elseif { $nnodes == 10 } {
            return 108
        } else {
            return 0 ;#unexpected case
        }
    } elseif { $elemtype == "Pyramid" } {
        if { $nnodes == 5 } {
            return 102
        } elseif { $nnodes == 13 } {
            return 102 ;#not exists in Amelet 1.5.3, write as 5 nodes case
        } else {
            return 0 ;#unexpected case
        }
    } elseif { $elemtype == "Prism" } {
        if { $nnodes == 4 } {
            return 103
        } elseif { $nnodes == 15 } {
            return 103 ;#not exists in Amelet 1.5.3, write as 4 nodes case
        } else {
            return 0 ;#unexpected case
        }
    } elseif { $elemtype == "Hexahedra" } {
        if { $nnodes == 8 }  {
            return 104
        } elseif { $nnodes == 20 }  {
            return 109
        } elseif { $nnodes == 27 }  {
            return 109 ;#not exists in Amelet 1.5.3, write as 20 nodes case
        } else {
            return 0 ;#unexpected case
        }
    } elseif { $elemtype == "Circle" } {
        return 16
    } elseif { $elemtype == "Sphere" } {
        return 107
    }
    return 0
}

proc Amelet::IsValidGroupName { name } {
    if { [string first / $name] != -1 } {
        return 0
    }
    if { [string first . $name] != -1 } {
        return 0
    }
    return [string is ascii $name]
}

proc Amelet::ForceValidGroupName { name } {
    #avoid forbidden characters
    set name [string map {. ?} $name]
    set name [string map {/ ?} $name]
    if { ![string is ascii $name] } {
        set newname ""
        foreach letter [split $name {}] {
            if { [string is ascii $letter] } {
                append newname $letter
            } else {
                append newname *
            }
        }
        set name $newname
    }
    return $name
}

proc Amelet::IsValidAttributeName { name } {
    return [string is ascii $name]
}

#GiD layer names could be invalid group names, because contain invalid characters, more than 20 chars
#or multiple sub-layers could have the same tail name repeating the group name
#of a layer can have mixed element types: edge, face, volume and must be separed in several groups.
#when writtin GiD add to the group a 'label' attribute storing the true layer name, to recover it.
proc Amelet::GetNewGroupName { h parent name {num_types 1} {entityType ""}} {
    set item [string map {// _} $name] ;#change // layer separator by _ to avoid lost this information
    set groupname [Amelet::ForceValidGroupName $item] ;#avoid non valid characters
    if { $num_types > 1 } {
        #to have a different name for each element type: edge, face, volume
        set groupname ${groupname}_[string index $entityType 0]
    }
    if { [string length $groupname] > 20 } {
        set groupname [string range ${groupname} 0 18]0
    }
    #if dataset exists create a new name adding a number
    set base_groupname $groupname
    set cont 1
    while { [$h is_dataset ${parent}/$groupname] } {
        set groupname ${base_groupname}${cont}
        while { [string length $groupname] > 20 } {
            set base_groupname [string range ${base_groupname} 0 end-1]
            set groupname ${base_groupname}${cont}
        }
        incr cont
    }
    return $groupname
}


#########################################################################################
######################################### WRITE PRE #####################################
#########################################################################################

#return a list with the names used for groups to be used in groupGroup
proc Amelet::WritePreLayerUnstructuredMesh { h meshpath layername } {
    variable compress_level
    variable NewElementNum
    variable NewNodeNum

    set created_groups ""

    #write isolated nodes as a group with attribute==node
    set ids [GiD_Info layers -entities nodes -higherentity 0 $layername]
    if { $ids != "" } {
        if { [GiD_Info layers -entities elements $layername] == "" } {
            set num_types 1
            set entityType ""
        } else {
            #to force a different group name that the obtained later from the elements, adding a suffix
            set num_types 2
            set entityType node
        }
        #node id's must start from 0, not from 1
        set newids [list]
        foreach id $ids {
            lappend newids $NewNodeNum($id)
        }
        set groupname [Amelet::GetNewGroupName $h ${meshpath}/group $layername $num_types $entityType]
        $h set -vtype int -compress $compress_level ${meshpath}/group/$groupname $newids
        $h set_attribute -type string ${meshpath}/group/$groupname type node
        if { $layername != $groupname } {
            $h set_attribute -type string ${meshpath}/group/$groupname label $layername
        }
        lappend created_groups $groupname
    }
    #write elements of layers as groups with attribute==edge, face or volume
    set entityTypes {edge face volume}
    set elemtypesoflevel(edge) {Linear}
    set elemtypesoflevel(face) {Triangle Quadrilateral Circle}
    set elemtypesoflevel(volume) {Tetrahedra Hexahedra Pyramid Prism Sphere}
    array set some_element {edge 0 face 0 volume 0}
    foreach entityType $entityTypes {
        foreach elemtype $elemtypesoflevel($entityType) {
            if {[GiD_Info layers -entities elements -elementtype $elemtype $layername] != "" } {
                set some_element($entityType) 1
                break
            }
        }
    }
    set num_types [expr {$some_element(edge)+$some_element(face)+$some_element(volume)}]
    if { $num_types } {
        foreach entityType $entityTypes {
            set ids [list]
            foreach elemtype $elemtypesoflevel($entityType) {
                lappend ids {*}[GiD_Info layers -entities elements -elementtype $elemtype $layername]
            }
            if { $ids != "" } {
                set elements [list]
                foreach id $ids {
                    lappend elements $NewElementNum($id)
                }
                set groupname [Amelet::GetNewGroupName $h ${meshpath}/group $layername $num_types $entityType]
                $h set -vtype int -compress $compress_level ${meshpath}/group/$groupname $elements
                $h set_attribute -type string ${meshpath}/group/$groupname type element entityType $entityType
                if { $layername != $groupname } {
                    $h set_attribute -type string ${meshpath}/group/$groupname label $layername
                }
                lappend created_groups $groupname
            }
        }
    }
    return $created_groups
}

#set parent "" as root
#limitations:
#-prefix name with folder_ to avoid leaves repeating name
#-Amelet names must have max 20 characters (and maybe must be ASCII?)
#-only a level of folders in gid is valid, althought Amelet allow create groupGroups of groupGroups
proc Amelet::WriteRecursivePreLayerUnstructuredMesh { h meshpath parent } {
    variable compress_level
    set LayerNames [GetChildLayerNames $parent]
    set childs ""
    foreach item $LayerNames {
        if { $parent != "" } {
            set layername $parent//$item
        } else {
            set layername $item
        }
        set created_groups [Amelet::WritePreLayerUnstructuredMesh $h $meshpath $layername]
        set has_childs [HasChilds $layername]
        if { $has_childs} {
            Amelet::WriteRecursivePreLayerUnstructuredMesh $h $meshpath $layername
        } else {
            lappend childs {*}$created_groups
        }
    }
    if { $parent !="" && [llength $childs] } {
        #set groupname $parent
        set groupname [Amelet::GetNewGroupName $h ${meshpath}/groupGroup $parent]
        $h set -vtype string -compress $compress_level ${meshpath}/groupGroup/$groupname $childs
    }
    return
}

#set parent "" as root
#limitations:
#-prefix name with folder_ to avoid leaves repeating name
#-Amelet names must have max 20 characters (and maybe must be ASCII?)
#-only a level of folders in gid is valid, althought Amelet allow create groupGroups of groupGroups
proc Amelet::WriteRecursivePreLayerUnstructuredMesh_Old { h meshpath parent } {
    variable compress_level
    set LayerNames [GetChildLayerNames $parent]
    set childs ""
    foreach item $LayerNames {
        if { $parent != "" } {
            set layername $parent//$item
        } else {
            set layername $item
        }
        if { [IsFolderOfLayers $item] } {
            #only implemented one level (but it is possible to create groupGroups of groupGroups)
            Amelet::WriteRecursivePreLayerUnstructuredMesh_Old $h $meshpath $layername
        } else {
            set created_groups [Amelet::WritePreLayerUnstructuredMesh $h $meshpath $layername]
            lappend childs {*}$created_groups
        }
    }

    if { $parent !="" && [llength $childs] } {
        set groupname [Amelet::GetNewGroupName $h ${meshpath}/groupGroup [string range $parent 0 end-2]]
        $h set -vtype string -compress $compress_level ${meshpath}/groupGroup/$groupname $childs
    }

}



proc Amelet::WritePreUnstructuredMesh { h } {
    variable compress_level
    variable NewElementNum
    variable NewNodeNum

    array unset NewElementNum
    array unset NewNodeNum

    set meshgroup /mesh/gid
    set meshpath $meshgroup/unstructuredmesh
    $h create_group /mesh $meshgroup $meshpath
    $h set_attribute -type string $meshpath type unstructured
    $h create_group $meshpath/group $meshpath/groupGroup $meshpath/selectorOnMesh

    # coordinates
    set nodes [GiD_Info mesh nodes -array]
    lassign $nodes ids coordinates
    lassign $coordinates xs ys zs
    set data [list]
    foreach x $xs y $ys z $zs {
        lappend data $x $y $z
    }
    $h set -vtype float -compress $compress_level -dimensions {0 3} ${meshpath}/nodes $data

    # connectivity
    #Amelet nodes must be stored with implicit number 0,..,n-1
    set inode -1
    foreach id $ids {
        set NewNodeNum($id) [incr inode]
    }

    set elementTypes [list]
    set elementNodes [list]
    set elements [GiD_Info mesh elements Any -array]
    set ielement -1
    foreach item $elements {
        lassign $item elemtype ids connectivities materials
        #test only for triangles
        set nnodes [llength $connectivities]
        set nodes [list]
        if { $elemtype == "Linear" } {
            if { $nnodes == 2 } {
                lassign $connectivities n1s n2s
                foreach n1 $n1s n2 $n2s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2)
                }
            } elseif { $nnodes == 3 } {
                lassign $connectivities n1s n2s n3s
                foreach n1 $n1s n2 $n2s n3 $n3s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3)
                }
            }
        } elseif { $elemtype == "Triangle" } {
            if { $nnodes == 3 } {
                lassign $connectivities n1s n2s n3s
                foreach n1 $n1s n2 $n2s n3 $n3s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3)
                }
            } elseif { $nnodes == 6 } {
                lassign $connectivities n1s n2s n3s n4s n5s n6s
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) \
                        $NewNodeNum($n4) $NewNodeNum($n5) $NewNodeNum($n6)
                }
            }
        } elseif { $elemtype == "Quadrilateral" } {
            if { $nnodes == 4 } {
                lassign $connectivities n1s n2s n3s n4s
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4)
                }
            } elseif { $nnodes == 8 || $nnodes == 9 } {
                if { nnodes == 8 } {
                    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s
                } else {
                    #9 nodes ignore the last center node
                    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s
                }
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) \
                        $NewNodeNum($n5) $NewNodeNum($n6) $NewNodeNum($n7) $NewNodeNum($n8)
                }
            }
        } elseif { $elemtype == "Tetrahedra" } {
            if { $nnodes == 4 } {
                lassign $connectivities n1s n2s n3s n4s
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4)
                }
            } else {
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s n9 $n9s n10 $n10s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) \
                        $NewNodeNum($n5) $NewNodeNum($n6) $NewNodeNum($n7) $NewNodeNum($n8) \
                        $NewNodeNum($n9) $NewNodeNum($n10)
                }
            }

        } elseif { $elemtype == "Pyramid" } {
            if { $nnodes == 5 } {
                lassign $connectivities n1s n2s n3s n4s n5s
            } else {
                #13 nodes ignore all quadratic ones
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s n11s n12s n13s
            }
            foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s {
                lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) $NewNodeNum($n5)
            }
        } elseif { $elemtype == "Prism" } {
            if { $nnodes == 6 } {
                lassign $connectivities n1s n2s n3s n4s n5s n6s
            } else {
                #15 nodes ignore all quadratic ones
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s n11s n12s n13s n14s n15s
            }
            foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s {
                lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) \
                    $NewNodeNum($n4) $NewNodeNum($n5) $NewNodeNum($n6)
            }
        } elseif { $elemtype == "Hexahedra" } {
            if { $nnodes == 8 } {
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) \
                        $NewNodeNum($n5) $NewNodeNum($n6) $NewNodeNum($n7) $NewNodeNum($n8)
                }
            } else {
                if { $nnodes == 20 } {
                    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s \
                        n11s n12s n13s n14s n15s n16s n17s n18s n19s n20s
                } else {
                    #27 nodes ignore the last 7 ones
                    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s \
                        n11s n12s n13s n14s n15s n16s n17s n18s n19s n20s \
                        n21s n22s n23s n24s n25s n26s n27s
                }

                #the order of the quadratic hexahedra nodes is different in GiD and Amelet
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s n9 $n9s n10 $n10s n11 $n11s n12 $n12s \
                    n17 $n17s n18 $n18s n19 $n19s n20 $n20s n13 $n13s n14 $n14s n15 $n15s n16 $n16s {
                        lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) \
                        $NewNodeNum($n5) $NewNodeNum($n6) $NewNodeNum($n7) $NewNodeNum($n8) \
                        $NewNodeNum($n9) $NewNodeNum($n10) $NewNodeNum($n11) $NewNodeNum($n12) \
                        $NewNodeNum($n13) $NewNodeNum($n14) $NewNodeNum($n15) $NewNodeNum($n16) \
                        $NewNodeNum($n17) $NewNodeNum($n18) $NewNodeNum($n19) $NewNodeNum($n20)
                }
            }
        } elseif { $elemtype == "Circle" } {
            continue
        } elseif { $elemtype == "Sphere" } {
            continue
        } elseif { $elemtype == "Point" } {
            continue
        } else {
            continue
        }
        set elementTypes [concat $elementTypes [lrepeat [llength $ids] [Amelet::ElementCode $elemtype $nnodes]]]
        set elementNodes [concat $elementNodes $nodes]
        #Amelet elements must stored with implicit number 0,..,n-1
        foreach id $ids {
            set NewElementNum($id) [incr ielement]
        }
    }
    $h set -vtype char -compress $compress_level ${meshpath}/elementTypes $elementTypes
    $h set -vtype int -compress $compress_level ${meshpath}/elementNodes $elementNodes

    if { [GidUtils::VersionCmp 10.1.9d] >= 0} {
        #GetChildLayerNames changed and IsFolderOfLayers not exists now
        Amelet::WriteRecursivePreLayerUnstructuredMesh $h $meshpath ""
    } else {
        #layers tree without explicit parents
        Amelet::WriteRecursivePreLayerUnstructuredMesh_Old $h $meshpath ""
    }

    if { [info procs ::Amelet::WritePreSelectorOnmeshUnstructuredMesh] != "" } {
        if { [catch { ::Amelet::WritePreSelectorOnmeshUnstructuredMesh $h $meshpath } err] } {
            Amelet::Warning [concat [_ "Error"]: $err]
        }
    }

    return 0
}


#Amelet::WritePreUnstructuredMesh {C:\gid project\gid\hdf5tcl\examples\test_3.h5}

#num starting from 1
#i,j,k starting from 0
proc Amelet::GetNumFromIjk { i j k } {
    variable _grid_n0
    variable _grid_n01
    return [expr {$k*$_grid_n01+$j*$_grid_n0+$i+1}]
}

#n0 num grid points in i direction
#n01=n0*n1 num grid points by ij plane
#num starting from 1
#i,j,k starting from 0 since Amelet 1.3.0 (previous versions started from 1)
proc Amelet::GetIjkFromNum { num } {
    variable _grid_n0
    variable _grid_n01
    incr num -1
    set k [expr $num/$_grid_n01]
    set rest [expr $num%$_grid_n01]
    set j [expr $rest/$_grid_n0]
    set i [expr $rest%$_grid_n0]
    return "$i $j $k"
}


proc Amelet::CheckIncreasingCorners { element } {
    lassign $element x0 y0 z0 x1 y1 z1
    if { $x0 > $x1 } {
        set aux $x0
        set x0 $x1
        set x1 $aux
    }
    if { $y0 > $y1 } {
        set aux $y0
        set y0 $y1
        set y1 $aux
    }
    if { $z0 > $z1 } {
        set aux $z0
        set z0 $z1
        set z1 $aux
    }
    if { [info exists aux] } {
        Amelet::Warning "Decreasing corner $element"
    }
    return [list $x0 $y0 $z0 $x1 $y1 $z1]
}
#limitations:
#-prefix name with folder_ to avoid leaves repeating name
#-Amelet names must have max 20 characters (and maybe must be ASCII?)
proc Amelet::WritePreLayerStructuredMesh { h meshpath layername } {
    variable compress_level
    set created_groups ""
    set num_types 0
    foreach elemtype {Point Linear Quadrilateral Hexahedra} {
        if { $elemtype == "Point" } {
            #consider Point elements and isolated nodes in the same way -> nodes
            #and ids is then the list of nodes, not elements
            set ids {}
            foreach id [GiD_Info layers -entities elements -elementtype $elemtype $layername] {
                lappend ids [lindex [lrange [GiD_Mesh get element $id] 3 end] 0]
            }
            lappend ids {*}[GiD_Info layers -entities nodes -higherentity 0 $layername]
        } else {
            set ids [GiD_Info layers -entities elements -elementtype $elemtype $layername]
        }
        if { $ids != "" } {
            incr num_types
            if { $num_types > 1 } {
                break
            }
        }
    }
    foreach elemtype {Point Linear Quadrilateral Hexahedra} {
        if { $elemtype == "Point" } {
            #consider Point elements and isolated nodes in the same way -> nodes
            #and ids is then the list of nodes, not elements
            set ids {}
            foreach id [GiD_Info layers -entities elements -elementtype $elemtype $layername] {
                lappend ids [lindex [lrange [GiD_Mesh get element $id] 3 end] 0]
            }
            lappend ids {*}[GiD_Info layers -entities nodes -higherentity 0 $layername]
        } else {
            set ids [GiD_Info layers -entities elements -elementtype $elemtype $layername]
        }
        if { $ids != "" } {

            if { $elemtype == "Point" } {
                set nodes [list]
            } else {
                set elements [list]
                set normals [list] ;#for faces or edges only
                set firsteleminfo [GiD_Mesh get element [lindex $ids 0]]
                set elemtype [lindex $firsteleminfo 1]
            }
            #warning: must do more checks to see that the element is really cartesian,
            #and all elements of the layer are of the same type !!! ...
            if { $elemtype == "Hexahedra" } {
                set entityType volume
                foreach id $ids {
                    set idnodes [lrange [GiD_Mesh get element $id] 3 end]
                    set ijk_0 [Amelet::GetIjkFromNum [lindex $idnodes 0]]
                    #lassign $ijk0 i j k
                    #lappend elements $i $j $k [expr $i+1] [expr $j+1] [expr $k+1] ;#assumed first node as lower left corner!!
                    set ijk_6 [Amelet::GetIjkFromNum [lindex $idnodes 6]]
                    set element [concat $ijk_0 $ijk_6]
                    #set element [Amelet::CheckIncreasingCorners $element]
                    lappend elements {*}$element
                }
            } elseif { $elemtype == "Quadrilateral" } {
                set entityType face
                foreach id $ids {
                    set idnodes [lrange [GiD_Mesh get element $id] 3 end]
                    set ijk_0 [Amelet::GetIjkFromNum [lindex $idnodes 0]]
                    set ijk_2 [Amelet::GetIjkFromNum [lindex $idnodes 2]]
                    if { [lindex $ijk_0 0] == [lindex $ijk_2 0] } {
                        set ijk_1 [Amelet::GetIjkFromNum [lindex $idnodes 1]]
                        if { [lindex $ijk_0 2] == [lindex $ijk_1 2] } {
                            set normal x+
                        } else {
                            set normal x-
                        }
                        set element [concat $ijk_0 $ijk_2]
                    } elseif { [lindex $ijk_0 1] == [lindex $ijk_2 1] } {
                        set ijk_1 [Amelet::GetIjkFromNum [lindex $idnodes 1]]
                        if { [lindex $ijk_0 0] == [lindex $ijk_1 0] } {
                            set normal y+
                        } else {
                            set normal y-
                        }
                        set element [concat $ijk_0 $ijk_2]
                    } elseif { [lindex $ijk_0 2] == [lindex $ijk_2 2] } {
                        set ijk_1 [Amelet::GetIjkFromNum [lindex $idnodes 1]]
                        if { [lindex $ijk_0 1] == [lindex $ijk_1 1] } {
                            set normal z+
                        } else {
                            set normal z-
                        }
                        set element [concat $ijk_0 $ijk_2]
                    } else {
                        Amelet::Warning [_ "Wrong element '%s'" $id]
                        continue
                    }
                    #set element [Amelet::CheckIncreasingCorners $element]
                    lappend normals $normal
                    lappend elements {*}$element
                }
            } elseif { $elemtype == "Linear" } {
                set entityType edge
                foreach id $ids {
                    set idnodes [lrange [GiD_Mesh get element $id] 3 end]
                    set ijk_0 [Amelet::GetIjkFromNum [lindex $idnodes 0]]
                    set ijk_1 [Amelet::GetIjkFromNum [lindex $idnodes 1]]
                    if { [lindex $ijk_0 0] != [lindex $ijk_1 0] } {
                        if { [lindex $ijk_0 0] < [lindex $ijk_1 0] } {
                            set normal x+
                        } else {
                            set normal x-
                        }
                        set element [concat $ijk_0 $ijk_1]
                    } elseif { [lindex $ijk_0 1] != [lindex $ijk_1 1] } {
                        if { [lindex $ijk_0 1] < [lindex $ijk_1 1] } {
                            set normal y+
                        } else {
                            set normal y-
                        }
                        set element [concat $ijk_0 $ijk_1]
                    } elseif { [lindex $ijk_0 2] != [lindex $ijk_1 2] } {
                        if { [lindex $ijk_0 2] < [lindex $ijk_1 2] } {
                            set normal z+
                        } else {
                            set normal z-
                        }
                        set element [concat $ijk_0 $ijk_1]
                    } else {
                        Amelet::Warning [_ "Wrong element '%s'" $id]
                    }
                    #set element [Amelet::CheckIncreasingCorners $element]
                    lappend normals $normal
                    lappend elements {*}$element
                }
            } elseif { $elemtype == "Point" } {
                set entityType node
                foreach id $ids {
                    set ijk_0 [Amelet::GetIjkFromNum $id]
                    lappend nodes {*}$ijk_0
                }
            } else {
                #entity not valid for Amelet structured (cartesian) mesh
                continue
            }

            set groupname [Amelet::GetNewGroupName $h ${meshpath}/group $layername $num_types $entityType]
            if { $entityType == "node" } {
                $h set -vtype int -compress $compress_level -dimensions {0 3} ${meshpath}/group/$groupname $nodes
                $h set_attribute -type string ${meshpath}/group/$groupname type node
            } else {
                $h set -vtype int -compress $compress_level -dimensions {0 6} ${meshpath}/group/$groupname $elements
                $h set_attribute -type string ${meshpath}/group/$groupname type element entityType $entityType
                if { $elemtype == "Quadrilateral" || $elemtype == "Linear" } {
                    $h set -vtype string -compress $compress_level ${meshpath}/normal/$groupname $normals
                }
            }
            if { $layername != $groupname } {
                $h set_attribute -type string ${meshpath}/group/$groupname label $layername
            }
            lappend created_groups $groupname
        }
    }
    return $created_groups
}

#set parent "" as root
#limitations:
#-prefix name with folder_ to avoid leaves repeating name
#-Amelet names must have max 20 characters (and maybe must be ASCII?)
#-only a level of folders in gid is valid, althought Amelet allow create groupGroups of groupGroups
proc Amelet::WriteRecursivePreLayerStructuredMesh { h meshpath parent } {
    variable compress_level
    set LayerNames [GetChildLayerNames $parent]
    set childs ""
    foreach item $LayerNames {
        if { $parent != "" } {
            set layername $parent//$item
        } else {
            set layername $item
        }
        set created_groups [Amelet::WritePreLayerStructuredMesh $h $meshpath $layername]
        set has_childs [HasChilds $layername]
        if { $has_childs } {
            Amelet::WriteRecursivePreLayerStructuredMesh $h $meshpath $layername
        } else {
            lappend childs {*}$created_groups
        }
    }
    if { $parent !="" && [llength $childs] } {
        #set groupname $parent
        set groupname [Amelet::GetNewGroupName $h ${meshpath}/groupGroup $parent]
        $h set -vtype string -compress $compress_level ${meshpath}/groupGroup/$groupname $childs
    }

}

#set parent "" as root
#limitations:
#-prefix name with folder_ to avoid leaves repeating name
#-Amelet names must have max 20 characters (and maybe must be ASCII?)
#-only a level of folders in gid is valid, althought Amelet allow create groupGroups of groupGroups
proc Amelet::WriteRecursivePreLayerStructuredMesh_Old { h meshpath parent } {
    variable compress_level
    set LayerNames [GetChildLayerNames $parent]
    foreach item $LayerNames {
        if { $parent != "" } {
            set layername $parent//$item
        } else {
            set layername $item
        }
        if { [IsFolderOfLayers $item] } {
            #only implemented one level (but it is possible to create groupGroups of groupGroups)
            set childs [list]
            foreach child [GetChildLayerNames $layername] {
                #lappend childs $layername$child
                lappend childs $child
            }
            set groupname [string range $item 0 end-2]
            $h set -vtype string -compress $compress_level ${meshpath}/groupGroup/$groupname $childs
            Amelet::WriteRecursivePreLayerStructuredMesh_Old $h $meshpath $layername
        } else {
            Amelet::WritePreLayerStructuredMesh $h $meshpath $layername
        }
    }
}

proc Amelet::WritePreStructuredMesh { h } {
    variable compress_level
    variable _grid_n0
    variable _grid_n01

    set meshgroup /mesh/gid
    set meshpath $meshgroup/structuredmesh
    $h create_group /mesh $meshgroup $meshpath
    $h set_attribute -type string $meshpath type structured
    $h create_group $meshpath/cartesianGrid $meshpath/group $meshpath/groupGroup $meshpath/normal $meshpath/selectorOnMesh

    set pmin [GiD_Cartesian get corner]
    set boxsize [GiD_Cartesian get boxsize]
    set ngridpoints [GiD_Cartesian get ngridpoints]
    set dimension [GiD_Cartesian get dimension]

    lassign [GiD_Cartesian get coordinates] c(x) c(y) c(z)

    foreach i {0 1 2} axe {x y z} {
        if { $i>=$dimension } break
        set n [lindex $ngridpoints $i]
        if { $c($axe) == "" } {
            set v [lindex $pmin $i]
            set dv [expr {[lindex $boxsize $i]/($n-1)}]
            set data [list]
            for {set j 0} {$j<$n} {incr j} {
                lappend data $v
                set v [expr $v+$dv]
            }
        } else {
            set data $c($axe)
        }
        $h set -vtype float -compress $compress_level ${meshpath}/cartesianGrid/$axe $data
    }

    set _grid_n0 [lindex $ngridpoints 0]
    set _grid_n01 [expr {$_grid_n0*[lindex $ngridpoints 1]}]

    if { [GidUtils::VersionCmp 10.1.9d] >= 0} {
        #GetChildLayerNames changed and IsFolderOfLayers not exists now
        Amelet::WriteRecursivePreLayerStructuredMesh $h $meshpath ""
    } else {
        #layers tree without explicit parents
        Amelet::WriteRecursivePreLayerStructuredMesh_Old $h $meshpath ""
    }
    if { [info procs ::Amelet::WritePreSelectorOnmeshStructuredMesh] != "" } {
        if { [catch { ::Amelet::WritePreSelectorOnmeshStructuredMesh $h $meshpath } err] } {
            Amelet::Warning [concat [_ "Error"]: $err]
        }
    }

    return 0
}

proc Amelet::WritePreMesh { h } {
    if { [GiD_Cartesian get iscartesian] } {
        Amelet::WritePreStructuredMesh $h
    } else {
        Amelet::WritePreUnstructuredMesh $h
    }
}

proc Amelet::GetFreeHDF5Id { } {
    set prefix hd
    for {set i 1} {$i<1000} {incr i} {
        set id ${prefix}${i}
        if { [info commands $id] == "" } {
            return $id
        }
    }
    return ""
}

proc Amelet::WritePre { filename } {
    variable _num_warnings
    set _num_warnings 0
    Amelet::LoadHdf5Package

    if { [file exists $filename] } {
        if { [catch { file delete $filename } err] } {
            WarnWin $err
            return 1
        }
    }
    if { ![lindex [GiD_Info Mesh] 0] } {
        WarnWin [_ "There is no mesh"]
        return 1
    }
    set h [Amelet::GetFreeHDF5Id]
    hdf5 $h $filename

    $h set_attribute -type string / FORMAT AMELETHDF AMELETHDF_FORMAT_VERSION 1.5.1

    Amelet::WritePreMesh $h
    #     if { [catch { Amelet::WritePreMesh $h } err] } {
    #         Amelet::Warning [concat [_ "Error"]: $err]
    #     }
    $h close
}

proc Amelet::WritePreW { } {
    #trick to not ask a filename from HIRF framework
    set filename [Amelet::GeAutomaticNameFromFramework]
    if { $filename == "" } {
        set filename [MessageBoxGetFilename file write [_ "Write Amelet file"] \
            {} {{{Amelet} {.h5 }} {{All files} {.*}}} ]
    }
    if {$filename == ""} {
        return
    }
    GidUtils::WaitState .gid
    GidUtils::EvalAndEnterInBatch Amelet::WritePre $filename
    GidUtils::EndWaitState .gid
    GiD_RaiseEvent GiD_Event_AfterSaveFile $filename AMELET_FORMAT 0
    GidUtils::SetWarnLine [_ "File %s written." $filename]
}

proc Amelet::IsHirfProblemtype { } {
    set problemtype [GiD_Info Project ProblemType]
    if { $problemtype == "HIRF" } {
        #new naming style from version HIRF 1.3
        return 1
    } else {
        #old naming style inside a folder HIRFx.x\HIRF, from version HIRF 1.3 to automatically offer transform
        #this folder is avoided and a xml file is storing with the model the problemtype version
        #HIRF1.0\HIRF on Windows but HIRF1.0/HIRF on Linux
        set problemtype [string map {\\ /} $problemtype]
        if { [lindex [split $problemtype /] 1] == "HIRF" } {
            #set problemtype_version [string range [lindex [split $problemtype /] 0] 4 end]
            return 1
        } else {
            return 0
        }
    }
}

proc Amelet::GetHirfProblemtypeVersion { } {
    set problemtype [GiD_Info Project ProblemType]
    if { $problemtype == "HIRF" } {
        #new naming style from version HIRF 1.3
        if { ![info exists ::HIRF::VersionNumber] } {
            return 0 ;#unexpected case
        }
        return $::HIRF::VersionNumber
    } else {
        #old naming style inside a folder HIRFx.x\HIRF, from version HIRF 1.3 to automatically offer transform
        #this folder is avoided and a xml file is storing with the model the problemtype version
        #HIRF1.0\HIRF on Windows but HIRF1.0/HIRF on Linux
        set problemtype [string map {\\ /} $problemtype]
        if { [lindex [split $problemtype /] 1] == "HIRF" } {
            set problemtype_version [string range [lindex [split $problemtype /] 0] 4 end]
            return $problemtype_version
        } else {
            return 0
        }
    }
}

#########################################################################################
######################################### READ PRE ######################################
#########################################################################################


proc Amelet::IsMesh { h meshpath } {
    set type [Amelet::GetAttribute $h $meshpath type]
    if { $type == "unstructured" || $type == "structured" } {
        return $type
    } else {
        return ""
    }
}

#to drag and drop a file
proc Amelet::ReadPreSingleFile { filename } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch Amelet::ReadPre [list $filename]
}


proc Amelet::ReadPre { filenames } {
    set fail 0
    GidUtils::WaitState
    GidUtils::DisableGraphics
    if { [catch { set fail [Amelet::ReadPre0 $filenames] } err options] } {
        set fail 1
    }
    GidUtils::EnableGraphics
    GidUtils::EndWaitState
    if { $fail } {
        return -options $options $err
    } else {
        GiD_RaiseEvent GiD_Event_AfterChangeMesh [GiD_Info Mesh NumNodes] [GiD_Info Mesh NumElements]
        GidUtils::UpdateWindow LAYER
        GiD_Process 'Zoom Frame
        GidUtils::SetWarnLine [_ "Amelet read"]
    }
    return $fail
}

proc Amelet::ReadPre0 { filenames } {
    variable _num_warnings
    variable percent_done
    variable stop
    variable _show_advance_bar

    set _num_warnings 0
    Amelet::LoadHdf5Package

    set fail 0
    set ::Gcode::stop 0
    set ::Gcode::percent_done 0
    if { 0 } {
        #by now use the existent advancebar of each file
        set percent_max [llength $filenames]
        if { $_show_advance_bar } {
            GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::Gcode::percent_done $percent_max {set ::Gcode::stop 1}
        }
    }
    set create_layer_filename 0
    if { 0 } {
        #by now use the layers of the file
        if { [llength $filenames]>1 } {
            set create_layer_filename 1
        }
    }
    foreach filename $filenames {
        if { $::Gcode::stop } {
            break
        }
        if { $create_layer_filename } {
            set layer [file rootname [file tail $filename]]
            if { ![GiD_Layers exists $layer] } {
                GiD_Layers create $layer
            }
            GiD_Layers edit to_use $layer
        }
        if { ![file exists $filename] } {
            W [_ "File %s not found" $filename]
            continue
        }
        GiD_Process Mescape Files HDF5Read $filename escape
        if { [info procs ::Amelet::ReadPreSelectorOnmeshRecursivelly] != ""} {
            set h [Amelet::GetFreeHDF5Id]
            if { [catch {hdf5 -readonly $h $filename} err] } {
                WarnWin [_ "File %s is not an HDF5 file" $filename]
                return 1
            }
            if { [catch { ::Amelet::ReadPreSelectorOnmeshRecursivelly $h /mesh } err] } {
                Amelet::Warning [concat [_ "Error"]: $err]
            }
            $h close
            GidUtils::SetWarnLine [_ "Applied conditions read"]
        }
        GiD_RaiseEvent GiD_Event_AfterOpenFile $filename AMELET_FORMAT 0
        if { 0 } {
            incr ::Amelet::percent_done
        }
    }
    if { 0 } {
        #by now use the existent advancebar of each file
        if { $_show_advance_bar } {
            GidUtils::DeleteAdvanceBar
        }
    }
    return $fail
}

proc Amelet::ReadPreW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read Amelet file"] \
        {} {{{Amelet} {.h5 }} {{All files} {.*}}} {} $multiple \
        [list [_ "Import options"] Amelet::MoreImportOptions]]
    if { [llength $filenames] } {
        if { !$_merge } {
            GiD_Process Mescape Files New No escape ;#to avoid previous geometry
        }
        GidUtils::EvalAndEnterInBatch Amelet::ReadPre $filenames
    }
}


#########################################################################################
######################################### READ POST #####################################
#########################################################################################

#we can't use this method to read mesh+results, because results are defined on groups
#and this procedure read all mesh entities, also if groups doesn't exists
proc Amelet::ReadPostUnstructuredMesh { h meshpath } {
    variable AmeletElementNNode
    variable _offsets_mesh_post ;#to associate results to the mesh ids

    set offset_nodes [GiD_Info Mesh -post MaxNumNodes]
    set offset_elements [GiD_Info Mesh -post MaxNumElements]
    set _offsets_mesh_post($meshpath) [list $offset_nodes $offset_elements]

    set num_meshes 0

    set gidpost_node_ids {} ;#empty for implicit increasing from the last
    set gidpost_element_ids {} ;#empty for implicit increasing from the last

    set nodes [lindex [$h get $meshpath/nodes] 0]
    set elements [lindex [$h get $meshpath/elementNodes] 0]

    set element_types [lindex [$h get $meshpath/elementTypes] 0]
    set different_element_types [lsort -integer -unique $element_types]
    #now groups information is avoided, better create for each group a mesh, if the  element type is the same
    #else can separe in multiple meshes, one by type.
    #GiD now write the true layer name related to each group in a 'label' attribute
    if { [llength $different_element_types] == 1 } {
        #do faster this simple case
        set meshname $meshpath
        set element_type [lindex $different_element_types 0]
        set gidpost_element_type [GetGidPostElementType $element_type]
        if { $gidpost_element_type != "" } {
            set gidpost_element_nnode [GetGidPostElementNNode $element_type]
            Amelet::CreateGiD_MeshPost $meshname \
                $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids $nodes \
                $gidpost_element_ids $elements $offset_nodes $offset_elements
            incr num_meshes
        }
    } else {
        foreach element_type $different_element_types {
            set meshname $meshpath/$element_type
            set gidpost_element_type [GetGidPostElementType $element_type]
            if { $gidpost_element_type != "" } {
                set gidpost_element_nnode [GetGidPostElementNNode $element_type]
                set some_elements {}
                set pos 0
                foreach type $element_types {
                    if { $type == $element_type } {
                        #if [GetGidPostElementNNode $element_type] != $AmeletElementNNode($element_type) must do something...
                        #e.g. sphere defined by 2 amelet nodes must be converted in node+radius
                        lappend some_elements {*}[lrange $elements $pos [expr {$pos+$gidpost_element_nnode-1}]]
                    }
                    incr pos $AmeletElementNNode($element_type)
                }
                #could be a problem send several times all nodes?
                Amelet::CreateGiD_MeshPost $meshname \
                    $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids $nodes \
                    $gidpost_element_ids $some_elements $offset_nodes $offset_elements
                incr num_meshes
            }
        }
    }
    return $num_meshes
}


proc Amelet::IncrListItems { list increment } {
    set xs {}
    foreach item $list {
        lappend xs [incr item $increment]
    }
    return $xs
}

proc Amelet::CreateGiD_MeshPost { meshname gidpost_element_type gidpost_element_nnode zero_based_array \
    gidpost_node_ids gidpost_nodes gidpost_element_ids gidpost_elements {offset_nodes 0} {offset_elements 0}} {

    if { $offset_nodes } {
        set gidpost_node_ids [Amelet::IncrListItems $gidpost_node_ids $offset_nodes]
        set gidpost_elements [Amelet::IncrListItems $gidpost_elements $offset_nodes]
    }
    if { $offset_elements } {
        set gidpost_element_ids [Amelet::IncrListItems $gidpost_element_ids $offset_elements]
    }

    if { $zero_based_array == "-zero_based_array" } {
        GiD_MeshPost create $meshname $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids $gidpost_nodes \
            $gidpost_element_ids $gidpost_elements
    } else {
        GiD_MeshPost create $meshname $gidpost_element_type $gidpost_element_nnode $gidpost_node_ids $gidpost_nodes \
            $gidpost_element_ids $gidpost_elements
    }
}

#create the elements defined in all groups
proc Amelet::ReadPostUnstructuredMeshByGroups { h meshpath } {
    variable AmeletElementNNode
    variable _offsets_mesh_post ;#to associate results to the mesh ids

    set num_meshes 0
    if { ![$h is_group $meshpath/group] } {
        return 0
    }

    if { ![$h is_dataset $meshpath/nodes] } {
        return 0
    }

    #some case like MaxEfieldplanxz_fixed.h5 starts nodes and elements of each mesh again from 0
    set offset_nodes [GiD_Info Mesh -post MaxNumNodes]
    set offset_elements [GiD_Info Mesh -post MaxNumElements]
    set _offsets_mesh_post($meshpath) [list $offset_nodes $offset_elements]

    set nodes [lindex [$h get $meshpath/nodes] 0]

    if { [$h is_dataset $meshpath/elementNodes] } {
        set elements [lindex [$h get $meshpath/elementNodes] 0]
    } else {
        set elements ""
    }
    if { [$h is_dataset $meshpath/elementTypes] } {
        set element_types [lindex [$h get $meshpath/elementTypes] 0]
    } else {
        set element_types ""
    }
    set different_element_types [lsort -integer -unique $element_types]

    set groups [$h glob -directory $meshpath/group -types dataset *]
    foreach group $groups {
        set type [Amelet::GetAttribute $h $group type]
        #GiD now write the true layer name related to each group in a 'label' attribute
        set meshname [Amelet::GetAttribute $h $group label [lindex [split $group /] end]]
        if { $type == "node" } {
            set gidpost_node_ids [lindex [$h get $group] 0]
            set gidpost_element_type Point
            set gidpost_element_nnode 1
            set gidpost_nodes {}
            foreach node_id $gidpost_node_ids {
                set pos [expr {$node_id*3}]
                lappend gidpost_nodes {*}[lrange $nodes $pos [expr {$pos+2}]]
            }

            #empty for implicit increasing from the last.
            #But can repeat numbers  if there are other groups with explicit element numbers!!
            #and we are creating 'onlypoint' elements but really only the nodes are defined.
            set gidpost_element_ids {} ;#empty for implicit increasing from the last.
            set gidpost_elements $gidpost_node_ids
            Amelet::CreateGiD_MeshPost $meshname \
                $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids \
                $gidpost_nodes $gidpost_element_ids $gidpost_elements $offset_nodes $offset_elements
            incr num_meshes
        } elseif { $type == "element" } {
            set gidpost_element_ids [lindex [$h get $group] 0]
            set gidpost_element_ids [lsort -integer $gidpost_element_ids] ;#sort increasing to find faster
            set entityType [Amelet::GetAttribute $h $group entityType]
            set global_nodes_used {}
            if { $entityType == "edge" ||  $entityType == "face" || $entityType == "volume" } {
                set group_element_types {}
                foreach element_id $gidpost_element_ids {
                    lappend group_element_types [lindex $element_types $element_id]
                }
                set group_different_element_types [lsort -integer -unique $group_element_types]
                if { [llength $different_element_types] == 1 } {
                    #faster if all elements are of the same type
                    #only do this if there is type of element in different_element_types
                    #not if group_different_element_types, because could be other element types and
                    #then is not possible to get its nodes because nnode is not constant for all elements
                    set element_type [lindex $group_different_element_types 0]
                    set gidpost_element_type [GetGidPostElementType $element_type]
                    set gidpost_element_nnode [GetGidPostElementNNode $element_type]
                    set gidpost_elements {}
                    foreach element_id $gidpost_element_ids {
                        set pos [expr {$element_id*$gidpost_element_nnode}]
                        lappend gidpost_elements {*}[lrange $elements $pos [expr {$pos+$gidpost_element_nnode-1}]]
                    }
                    set gidpost_node_ids [lsort -integer -unique $gidpost_elements] ;#remove repeateds
                    set gidpost_nodes {}
                    foreach node_id $gidpost_node_ids {
                        set pos [expr {$node_id*3}]
                        lappend gidpost_nodes {*}[lrange $nodes $pos [expr {$pos+2}]]
                    }
                    Amelet::CreateGiD_MeshPost $meshname \
                        $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids \
                        $gidpost_nodes $gidpost_element_ids $gidpost_elements $offset_nodes $offset_elements
                    incr num_meshes
                } else {
                    foreach element_type $group_different_element_types {
                        set gidpost_element_type [GetGidPostElementType $element_type]
                        set gidpost_element_nnode [GetGidPostElementNNode $element_type]
                        set gidpost_elements {}
                        #must follow the list of all elements, because the number of nodes is variable
                        #and we don't know where starts their nodes in the list
                        set pos 0
                        set element_id 0
                        foreach element_type_i $element_types {
                            if { [lsearch -integer -sorted $gidpost_element_ids $element_id] != -1 } {
                                if { $element_type_i == $element_type } {
                                    lappend gidpost_elements {*}[lrange $elements $pos [expr {$pos+$gidpost_element_nnode-1}]]
                                } else {
                                    Amelet::Warning [_ "Element %s: wrong element type %s, expected %s" $element_id $element_type_i $element_type]
                                }
                            }
                            incr pos $AmeletElementNNode($element_type_i)
                            incr element_id
                        }
                        set gidpost_node_ids [lsort -integer -unique $gidpost_elements] ;#remove repeateds
                        set gidpost_nodes {}
                        foreach node_id $gidpost_node_ids {
                            set pos [expr {$node_id*3}]
                            lappend gidpost_nodes {*}[lrange $nodes $pos [expr {$pos+2}]]
                        }
                        Amelet::CreateGiD_MeshPost $meshname \
                            $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids \
                            $gidpost_nodes $gidpost_element_ids $gidpost_elements $offset_nodes $offset_elements
                        incr num_meshes
                    }
                }
            } else {
                Amelet::Warning [_ "Unexpected value '%s' of attribute 'entityType' for group '%s'. Must be 'edge', 'face' or 'volume'" $entityType $group]
                continue
            }
        } else {
            Amelet::Warning [_ "Unexpected value '%s' of  attribute 'type' for group '%s'. Must be 'node' or 'element'" $type $group]
            continue
        }
    }
    return $num_meshes
}


proc Amelet::GetSingleEdge {i0 j0 k0 i1 j1 k1} {
    set n1 [Amelet::GetNumFromIjk $i0 $j0 $k0]
    set n2 [Amelet::GetNumFromIjk $i1 $j1 $k1]
    return [list $n1 $n2]
}

proc Amelet::GetSingleFace {i0 j0 k0 i1 j1 k1 } {
    if { $i0 == $i1 } {
        set n1 [Amelet::GetNumFromIjk $i0 $j0 $k0]
        set n2 [Amelet::GetNumFromIjk $i0 $j1 $k0]
        set n3 [Amelet::GetNumFromIjk $i0 $j1 $k1]
        set n4 [Amelet::GetNumFromIjk $i0 $j0 $k1]
    } elseif { $j0 == $j1 } {
        set n1 [Amelet::GetNumFromIjk $i0 $j0 $k0]
        set n2 [Amelet::GetNumFromIjk $i0 $j0 $k1]
        set n3 [Amelet::GetNumFromIjk $i1 $j0 $k1]
        set n4 [Amelet::GetNumFromIjk $i1 $j0 $k0]
    } else {
        set n1 [Amelet::GetNumFromIjk $i0 $j0 $k0]
        set n2 [Amelet::GetNumFromIjk $i1 $j0 $k0]
        set n3 [Amelet::GetNumFromIjk $i1 $j1 $k0]
        set n4 [Amelet::GetNumFromIjk $i0 $j1 $k0]
    }
    return [list $n1 $n2 $n3 $n4]
}

proc Amelet::GetSingleVolume {i0 j0 k0 i1 j1 k1} {
    set n1 [Amelet::GetNumFromIjk $i0 $j0 $k0]
    set n2 [Amelet::GetNumFromIjk $i0 $j1 $k0]
    set n3 [Amelet::GetNumFromIjk $i0 $j1 $k1]
    set n4 [Amelet::GetNumFromIjk $i0 $j0 $k1]
    set n5 [Amelet::GetNumFromIjk $i1 $j0 $k0]
    set n6 [Amelet::GetNumFromIjk $i1 $j1 $k0]
    set n7 [Amelet::GetNumFromIjk $i1 $j1 $k1]
    set n8 [Amelet::GetNumFromIjk $i1 $j0 $k1]
    return [list $n1 $n2 $n2 $n4 $n5 $n6 $n7 $n8]
}

#create the elements defined in all groups
proc Amelet::ReadPostStructuredMeshByGroups { h meshpath } {
    variable _grid_n0
    variable _grid_n01
    variable _node_ids

    variable _structured_elements_post_range_by_group ;#save the range of elements of each group to apply results
    variable _offsets_mesh_post ;#to associate results to the mesh ids

    set offset_nodes [GiD_Info Mesh -post MaxNumNodes]
    set offset_elements [GiD_Info Mesh -post MaxNumElements]
    set _offsets_mesh_post($meshpath) [list $offset_nodes $offset_elements]

    set num_meshes 0

    set xs [lindex [$h get $meshpath/cartesianGrid/x] 0]
    set ys [lindex [$h get $meshpath/cartesianGrid/y] 0]
    set zs [lindex [$h get $meshpath/cartesianGrid/z] 0]
    set _grid_n0 [llength $xs]
    set _grid_n01 [expr {$_grid_n0*[llength $ys]}]
    set groups [$h glob -directory $meshpath/group -types dataset *]

    set global_nodes_elements_unrepeated {}
    set gidpost_nodes {}
    foreach step {CREATE_NODES CREATE_ELEMENTS} {
        foreach group $groups {
            set type [Amelet::GetAttribute $h $group type]
            #GiD now write the true layer name related to each group in a 'label' attribute
            set meshname [Amelet::GetAttribute $h $group label [lindex [split $group /] end]]
            if { $type == "node" } {
                if { $step == "CREATE_NODES" } {
                    set cartesian_nodes [lindex [$h get $group] 0]
                    foreach {i j k} $cartesian_nodes {
                        lappend global_nodes_elements {*}[Amelet::GetNumFromIjk $i $j $k]
                    }
                    #create also a mesh with only nodes (or elements of a node)??
                    set gidpost_element_type Point
                    set gidpost_element_nnode 1
                }
            } elseif { $type == "element" } {
                if { $step == "CREATE_NODES" } {
                    set cartesian_elements [lindex [$h get $group] 0]
                    set entityType [Amelet::GetAttribute $h $group entityType]
                    set global_nodes_elements {}
                    if { $entityType == "edge" } {
                        set gidpost_element_type Linear
                        set gidpost_element_nnode 2
                        foreach {i0 j0 k0 i1 j1 k1} $cartesian_elements {
                            lappend global_nodes_elements {*}[Amelet::GetSingleEdge $i0 $j0 $k0 $i1 $j1 $k1]
                        }
                    } elseif { $entityType == "face" } {
                        set gidpost_element_type Quadrilateral
                        set gidpost_element_nnode 4
                        foreach {i0 j0 k0 i1 j1 k1} $cartesian_elements {
                            lappend global_nodes_elements {*}[Amelet::GetSingleFace $i0 $j0 $k0 $i1 $j1 $k1]
                        }
                    } elseif { $entityType == "volume" } {
                        set gidpost_element_type Hexahedra
                        set gidpost_element_nnode 8
                        foreach {i0 j0 k0 i1 j1 k1} $cartesian_elements {
                            lappend global_nodes_elements {*}[Amelet::GetSingleVolume $i0 $j0 $k0 $i1 $j1 $k1]
                        }
                    } else {
                        Amelet::Warning [_ "Unexpected value '%s' of attribute 'entityType' for group '%s'. Must be 'edge', 'face' or 'volume'" $entityType $group]
                        continue
                    }
                }
            } else {
                Amelet::Warning [_ "Unexpected value '%s' of  attribute 'type' for group '%s'. Must be 'node' or 'element'" $type $group]
                continue
            }
            if { $step == "CREATE_ELEMENTS" } {
                set gidpost_node_ids {} ;#empty for implicit increasing from the last
                set gidpost_element_ids {} ;#empty for implicit increasing from the last
                set gidpost_elements {}
                foreach num $global_nodes_elements {
                    lappend gidpost_elements $_node_ids($num)
                }
                GiD_MeshPost create $meshname $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids $gidpost_nodes \
                    $gidpost_element_ids $gidpost_elements
                incr num_meshes
                #save the range of elements of each group to apply results
                set num_new_elements [expr {[llength $gidpost_elements]/$gidpost_element_nnode}]
                set _structured_elements_post_range_by_group($group) [list [expr {$offset_elements+1}] [expr {$offset_elements+$num_new_elements}]]
                incr offset_elements $num_new_elements
            } else {
                #sort locally removing repeateds
                lappend global_nodes_elements_unrepeated {*}[lsort -integer -unique $global_nodes_elements]
            }
        }
        if { $step == "CREATE_NODES" } {
            #sort globally removing repeateds
            set global_nodes_elements_unrepeated [lsort -integer -unique $global_nodes_elements_unrepeated]
            set cont 0
            foreach num $global_nodes_elements_unrepeated {
                lassign [Amelet::GetIjkFromNum $num] i j k
                lappend gidpost_nodes [lindex $xs $i] [lindex $ys $j] [lindex $zs $k]
                set _node_ids($num) $cont
                incr cont
            }
        }
    }
    return $num_meshes
}

#recursive read
proc Amelet::ReadPostGroupsMesh { h meshpath } {
    set num_meshes 0
    set groups [$h glob -directory $meshpath -types group *]
    foreach group $groups {
        set type [Amelet::IsMesh $h $group]
        if { $type == "unstructured" } {
            if { [$h is_group $group/group] } {
                if { [$h is_dataset $group/group/all_nodes] } {
                    #GiD like special case: exists elements but results are on nodes instead of gauss points
                    incr num_meshes [Amelet::ReadPostUnstructuredMesh $h $group]
                } else {
                    incr num_meshes [Amelet::ReadPostUnstructuredMeshByGroups $h $group]
                }
            } else {
                #there is no group, read all the mesh
                incr num_meshes [Amelet::ReadPostUnstructuredMesh $h $group]
            }
        } elseif { $type == "structured" } {
            incr num_meshes [Amelet::ReadPostStructuredMeshByGroups $h $group]
        } else {
            incr num_meshes [Amelet::ReadPostGroupsMesh $h $group]
        }
    }
    return $num_meshes
}

proc Amelet::ReadPostMeshes { filename } {
    Amelet::LoadHdf5Package

    if { ![file exists $filename] } {
        WarnWin [_ "File %s not found" $filename]
        return 0
    }

    set h [Amelet::GetFreeHDF5Id]
    if { [catch {hdf5 -readonly $h $filename} err] } {
        WarnWin [_ "File %s is not an HDF5 file" $filename]
        return 0
    }

    if { ![IsAmelet $h] } {
        WarnWin [_ "File %s is not an Amelet HDF5 file" $filename]
        $h close
        return 0
    }
    if { [ $h is_group /mesh] } {
        set num_meshes [Amelet::ReadPostGroupsMesh $h /mesh]
    } else {
        #WarnWin [_ "File %s doesn't has any mesh" $filename]
        set num_meshes 0
    }
    $h close
    return $num_meshes
}


proc Amelet::IsResult { h meshpath } {
    set type [Amelet::GetAttribute $h $meshpath floatingType]
    if { $type == "singleInteger" } {
        return $type
    } elseif { $type == "singleReal" } {
        return $type
    } elseif { $type == "singleComplex" } {
        return $type
    } elseif { $type == "singleString" } {
        return $type
    } elseif { $type == "vector" } {
        return $type
    } elseif { $type == "linearListOfReal1" } {
        return $type
    } elseif { $type == "linearListOfReal2" } {
        return $type
    } elseif { $type == "logarithmListOfReal" } {
        return $type
    } elseif { $type == "perDecadeListOfReal" } {
        return $type
    } elseif { $type == "linearListOfInteger2" } {
        return $type
    } elseif { $type == "rationalFunction" } {
        return $type
    } elseif { $type == "generalRationalFunction" } {
        return $type
    } elseif { $type == "generalRationalFraction" } {
        return $type
    } elseif { $type == "rational" } {
        return $type
    } elseif { $type == "dataSet" } {
        return $type
    } elseif { $type == "arraySet" } {
        return $type
    } else {
        return ""
    }
}

#return the parent mesh of this tree
proc Amelet::GetParentMesh { h path } {
    set path_splitted [split $path /]
    if { [llength $path_splitted] < 2 } {
        return ""
    }
    set parent_path [join [lrange $path_splitted 0 end-1] /]
    if { [Amelet::IsMesh $h $parent_path] != "" } {
        return $parent_path
    }
    return [Amelet::GetParentMesh $h $parent_path]
}

proc Amelet::AreAllIncreasingNumbers { xs } {
    set x_prev -1.0e300
    foreach x $xs {
        if { ![string is double $x] } {
            return 0
        }
        if { $x < $x_prev } {
            return 0
        }
        set x_prev $x
    }
    return 1
}

#get list from 0 to n-1
proc Amelet::GetListIncreasingNumbers { n } {
    set xs {}
    for {set i 0} {$i<$n} {incr i} {
        lappend xs $i
    }
    return $xs
}

#to get an unused name, based on label
#type must be surface (also for line elemens) or volume
proc Amelet::GetPostMeshNameAutomatic { label type } {
    if { $type == "surface" } {
        set names [GiD_Info postprocess get all_surfacesets]
    } elseif { $type == "volume" } {
        set names [GiD_Info postprocess get all_volumesets]
    } else {
        Amelet::Warning "Amelet::GetPostMeshNameAutomatic, wrong type $type"
        set names ""
    }
    if { [lsearch $names $label] == -1 } {
        set name $label
    } else {
        set i 1
        while { 1 } {
            set name $label_$i
            if { [lsearch $names $label] == -1 } {
                break
            }
        }
    }
    return $name
}

proc Amelet::CreatePostMeshGrid2D { xs ys label } {
    set n_x [llength $xs]
    if { 1 || ![Amelet::AreAllIncreasingNumbers $xs] } {
        #replace wrong non numeric scale values by increasing integers
        set xs [Amelet::GetListIncreasingNumbers $n_x]
    }
    set n_y [llength $ys]
    if { 1 || ![Amelet::AreAllIncreasingNumbers $ys] } {
        #replace wrong non numeric scale values by increasing integers
        set ys [Amelet::GetListIncreasingNumbers $n_y]
    }
    if { $n_x<2 || $n_y<2 } {
        Amelet::Warning [_ "CreatePostMeshGrid2D: xs and ys must have at least two items"]
        return 0
    }

    set meshname $label
    set gidpost_element_type Quadrilateral
    set gidpost_element_nnode 4
    set gidpost_element_ids {} ;#empty for implicit increasing from the last
    set gidpost_nodes {}
    set gidpost_elements {}
    set offset_nodes [GiD_Info Mesh -post MaxNumNodes]
    if { [GidUtils::VersionCmp 10.1.9d] >= 0} {
        set gidpost_node_ids {} ;#empty for implicit increasing from the last
    } else {
        #error in previous versions, can't use without node ids because every mesh start by 0 !!
        set gidpost_node_ids [objarray new_from_to intarray [expr {$offset_nodes}] [expr {[llength $xs]*[llength $ys]-1+$offset_nodes}]]
    }

    set j 1
    foreach y $ys {
        set i 1
        foreach x $xs {
            lappend gidpost_nodes $x $y 0.0
            if { $i<$n_x && $j<$n_y } {
                set n0 [expr {($j-1)*$n_x+$i-1+$offset_nodes}]
                set n1 [expr {$n0+1}]
                set n3 [expr {$n0+$n_x}]
                set n2 [expr {$n3+1}]
                lappend gidpost_elements $n0 $n1 $n2 $n3
            }
            incr i
        }
        incr j
    }


    GiD_MeshPost create $meshname $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids $gidpost_nodes \
        $gidpost_element_ids $gidpost_elements
    return 1
}

#gidpost_type must be Scalar, ComplexScalar, Vector, ComplexVector
proc Amelet::CreatePostGraph2D { xs ys label_x label_y unit_x unit_y label unit gidpost_values gidpost_type } {
    variable _grids
    if { $gidpost_type != "Scalar" && $gidpost_type != "ComplexScalar" && $gidpost_type != "Vector" && $gidpost_type != "ComplexVector"} {
        Amelet::Warning [_ "CreatePostGraph2D type '$gidpost_type' not implemented yet"]
        return 0
    }

    set checksum [encrypter MD5 encode [list $xs $ys]]
    if { [info exists _grids($checksum)] } {
        #not create again the mesh was created with the same xs yz
        set offset_nodes $_grids($checksum)
    } else {
        set offset_nodes [GiD_Info Mesh -post MaxNumNodes]
        set num_meshes [Amelet::CreatePostMeshGrid2D $xs $ys [Amelet::GetPostMeshNameAutomatic $label surface]]
        if { $num_meshes != 1 } {
            return 0
        }
        set _grids($checksum) $offset_nodes
    }

    set gidpost_ids [objarray new_from_to intarray [expr {1+$offset_nodes}] [expr {[llength $xs]*[llength $ys]+$offset_nodes}]]

    set gidpost_resultname $label
    set gidpost_analysisname none
    set gidpost_step 1
    set gidpost_over OnNodes
    set gidpost_result [list Result $gidpost_resultname $gidpost_analysisname $gidpost_step $gidpost_type $gidpost_over]
    #set gidpost_component_names [list $label_x $label_y]
    set unit_name $unit


    if { $gidpost_type == "ComplexScalar" && [GidUtils::VersionCmp "10.1.9d"] < 0 } {
        #trick to work also with versions prior to 10.1.9d that have a bug with -array for ComplexScalar
        set gidpost_some_values ""
        foreach id $gidpost_ids re [lindex $gidpost_values 0] im [lindex $gidpost_values 1] {
            lappend gidpost_some_values [list $id $re $im]
        }
        GiD_Result create $gidpost_result [list Unit $unit_name] {*}$gidpost_some_values
        return 1
    }

    set gidpost_some_values [list $gidpost_ids $gidpost_values]
    #GiD_Result create -array $gidpost_result [list ComponentNames {*}$gidpost_component_names] [list Unit $unit_name] $gidpost_some_values
    GiD_Result create -array $gidpost_result [list Unit $unit_name] $gidpost_some_values
    return 1
}


proc Amelet::CreatePostMeshGrid3D { xs ys zs label } {

    if { [GidUtils::VersionCmp "10.1.9d"] < 0 } {
        Amelet::Warning [_ "This version %s of GiD is not able to create a hexahedra grid, update GiD" [GiD_Info GiDVersion] ]
        return 0
    }

    set n_x [llength $xs]
    if { 1 || ![Amelet::AreAllIncreasingNumbers $xs] } {
        #replace wrong non numeric scale values by increasing integers
        set xs [Amelet::GetListIncreasingNumbers $n_x]
    }
    set n_y [llength $ys]
    if { 1 || ![Amelet::AreAllIncreasingNumbers $ys] } {
        #replace wrong non numeric scale values by increasing integers
        set ys [Amelet::GetListIncreasingNumbers $n_y]
    }

    set n_z [llength $zs]
    if { 1 || ![Amelet::AreAllIncreasingNumbers $zs] } {
        #replace wrong non numeric scale values by increasing integers
        set zs [Amelet::GetListIncreasingNumbers $n_z]
    }

    if { $n_x<2 || $n_y<2 || $n_z<2 } {
        Amelet::Warning [_ "CreatePostMeshGrid3D: xs, ys and zs must have at least two items"]
        return 0
    }

    set meshname $label
    set gidpost_element_type Hexahedra
    set gidpost_element_nnode 8
    set gidpost_node_ids {} ;#empty for implicit increasing from the last
    set gidpost_element_ids {} ;#empty for implicit increasing from the last
    set gidpost_nodes {}
    set gidpost_elements {}
    set offset_nodes [GiD_Info Mesh -post MaxNumNodes]

    set n_xy [expr {$n_x*$n_y}]

    set k 1
    foreach z $zs {
        set j 1
        foreach y $ys {
            set i 1
            foreach x $xs {
                lappend gidpost_nodes $x $y $z
                if { $i<$n_x && $j<$n_y && $k<$n_z } {
                    set n0 [expr {($k-1)*$n_xy+($j-1)*$n_x+$i-1+$offset_nodes}]
                    set n1 [expr {$n0+1}]
                    set n2 [expr {$n1+$n_x}]
                    set n3 [expr {$n2-1}]
                    set n4 [expr {$n0+$n_xy}]
                    set n5 [expr {$n4+1}]
                    set n6 [expr {$n5+$n_x}]
                    set n7 [expr {$n6-1}]
                    lappend gidpost_elements $n0 $n1 $n2 $n3 $n4 $n5 $n6 $n7
                }
                incr i
            }
            incr j
        }
        incr k
    }

    GiD_MeshPost create $meshname $gidpost_element_type $gidpost_element_nnode -zero_based_array $gidpost_node_ids $gidpost_nodes \
        $gidpost_element_ids $gidpost_elements

    return 1
}

proc Amelet::CreatePostGraph3D { xs ys zs label_x label_y label_z unit_x unit_y unit_z label unit gidpost_values gidpost_type } {
    variable _grids
    if { $gidpost_type != "Scalar" && $gidpost_type != "ComplexScalar" && $gidpost_type != "Vector" && $gidpost_type != "ComplexVector"} {
        Amelet::Warning [_ "CreatePostGraph3D type '%s' not implemented yet" $gidpost_type]
        return 0
    }

    set checksum [encrypter MD5 encode [list $xs $ys $zs]]
    if { [info exists _grids($checksum)] } {
        #not create again the mesh was created with the same xs yz zs
        set offset_nodes $_grids($checksum)
    } else {
        set offset_nodes [GiD_Info Mesh -post MaxNumNodes]
        set num_meshes [Amelet::CreatePostMeshGrid3D $xs $ys $zs [Amelet::GetPostMeshNameAutomatic $label volume]]
        if { $num_meshes != 1 } {
            return 0
        }
        set _grids($checksum) $offset_nodes
    }

    set gidpost_ids [objarray new_from_to intarray [expr {1+$offset_nodes}] [expr {[llength $xs]*[llength $ys]*[llength $zs]+$offset_nodes}]]

    set gidpost_resultname $label
    set gidpost_analysisname none
    set gidpost_step 1
    set gidpost_over OnNodes
    set gidpost_result [list Result $gidpost_resultname $gidpost_analysisname $gidpost_step $gidpost_type $gidpost_over]
    #set gidpost_component_names [list $label_x $label_y $label_z]
    set unit_name $unit

    if { $gidpost_type == "ComplexScalar" && [GidUtils::VersionCmp "10.1.9d"] < 0 } {
        #trick to work also with versions prior to 10.1.9d that have a bug with -array for ComplexScalar
        set gidpost_some_values ""
        foreach id $gidpost_ids re [lindex $gidpost_values 0] im [lindex $gidpost_values 1] {
            lappend gidpost_some_values [list $id $re $im]
        }
        GiD_Result create $gidpost_result [list Unit $unit_name] {*}$gidpost_some_values
        return 1
    }

    set gidpost_some_values [list $gidpost_ids $gidpost_values]
    #GiD_Result create -array $gidpost_result [list ComponentNames {*}$gidpost_component_names] [list Unit $unit_name] $gidpost_some_values
    GiD_Result create -array $gidpost_result [list Unit $unit_name] $gidpost_some_values
    return 1
}

#zs is considered as a results along the time instead another grid dimension
proc Amelet::CreatePostGrid2DTimeResult { xs ys time_steps label_x label_y label_t unit_x unit_y unit_t label unit gidpost_values gidpost_type } {
    variable _grids
    if { $gidpost_type != "Scalar" && $gidpost_type != "ComplexScalar" && $gidpost_type != "Vector" && $gidpost_type != "ComplexVector"} {
        Amelet::Warning [_ "CreatePostGrid2DTimeResult type '%s' not implemented yet" $gidpost_type]
        return 0
    }

    set checksum [encrypter MD5 encode [list $xs $ys $time_steps]]
    if { [info exists _grids($checksum)] } {
        #not create again the mesh was created with the same xs yz and same time steps
        set offset_nodes $_grids($checksum)
    } else {
        set offset_nodes [GiD_Info Mesh -post MaxNumNodes]
        set num_meshes [Amelet::CreatePostMeshGrid2D $xs $ys [Amelet::GetPostMeshNameAutomatic $label surface]]
        if { $num_meshes != 1 } {
            return 0
        }
        set _grids($checksum) $offset_nodes
    }


    set gidpost_ids [objarray new_from_to intarray [expr {1+$offset_nodes}] [expr {[llength $xs]*[llength $ys]+$offset_nodes}]]

    set gidpost_resultname $label
    set gidpost_analysisname $label_t
    set gidpost_over OnNodes

    set gidpost_component_names ""
    set unit_name $unit

    set num_results 0

    set n_steps [llength $time_steps]

    set n_gidpost_values [llength [lindex $gidpost_values 0]]
    set n_ids [expr {[llength $xs]*[llength $ys]}]
    if { [expr {$n_gidpost_values/$n_steps}] != $n_ids } {
        Amelet::Warning [_ "number of results %s divided by %d steps not match with number of ids %s" $n_gidpost_values $n_steps $n_ids]
        return [list $num_results $num_graphs]
    }
    set i_step 0
    foreach gidpost_step $time_steps {
        set v {}
        foreach item $gidpost_values {
            lappend v [lrange $item [expr {$n_ids*$i_step}] [expr {$n_ids*($i_step+1)-1}]]
        }
        set gidpost_result [list Result $gidpost_resultname $gidpost_analysisname $gidpost_step $gidpost_type $gidpost_over]
        if { $gidpost_over == "OnGaussPoints" } {
            lappend gidpost_result $gidpost_gauss_point_name
        }
        if { $gidpost_type == "ComplexScalar" && [GidUtils::VersionCmp "10.1.9d"] < 0 } {
            #trick to work also with versions prior to 10.1.9d that have a bug with -array for ComplexScalar
            set gidpost_some_values ""
            foreach id $gidpost_ids re [lindex $v 0] im [lindex $v 1] {
                lappend gidpost_some_values [list $id $re $im]
            }
            GiD_Result create $gidpost_result [list ComponentNames {*}$gidpost_component_names] [list Unit $unit_name] {*}$gidpost_some_values
        } else {
            set gidpost_some_values [list $gidpost_ids $v]
            if { $gidpost_component_names != "" } {
                if { $unit_name != "" } {
                    GiD_Result create -array $gidpost_result [list ComponentNames {*}$gidpost_component_names] [list Unit $unit_name] $gidpost_some_values
                } else {
                    GiD_Result create -array $gidpost_result [list ComponentNames {*}$gidpost_component_names] $gidpost_some_values
                }
            } else {
                if { $unit_name != "" } {
                    GiD_Result create -array $gidpost_result [list Unit $unit_name] $gidpost_some_values
                } else {
                    GiD_Result create -array $gidpost_result $gidpost_some_values
                }
            }
        }
        incr i_step
    }
    incr num_results

    return $num_results
}



proc Amelet::ReadPostResultArraySet { h resultpath } {
    variable _offsets_mesh_post ;#to associate results to the mesh ids
    variable _structured_elements_post_range_by_group ;#save the range of elements of each group to apply results
    variable _node_ids ;#to know how structured nodes are renumbered

    set num_results 0
    set num_graphs 0

    set meshEntity ""
    set component_names ""
    set time_steps ""
    set idim_meshEntity ""
    set idim_component_names ""
    set idim_time_steps ""
    set location "barycenter"
    set unit_name ""

    set data_dimensions [lindex [$h get -dimensions $resultpath/data] 0]
    set ndimensions [llength $data_dimensions]

    foreach dim [$h glob -directory $resultpath/ds -types dataset dim*] {
        #optional attributes: physicalNature label unit comment
        set physicalNature [Amelet::GetAttribute $h $dim physicalNature]
        set unit_name [Amelet::GetAttribute $h $dim unit]
        if { $unit_name != "" } {
            set unit_name_physical_nature($physicalNature) $unit_name
        }
        if { $physicalNature == "meshEntity" } {
            #is the mesh group path of entities where the data resuls are applied
            set meshEntity [string trim [lindex [lindex [$h get $dim] 0] 0]]
            set location [Amelet::GetAttribute $h $dim location barycenter]
            set idim_meshEntity [expr {$ndimensions-[string index [lindex [split $dim /] end] end]}]
        } elseif { $physicalNature == "component" } {
            #are the component names of the data rows for vectors
            #could be: x y z
            #          r theta z
            #          r theta phi
            set component_names ""
            foreach component_name [lindex [$h get $dim] 0] {
                lappend component_names [string trim $component_name]
            }
            set idim_component_names [expr {$ndimensions-[string index [lindex [split $dim /] end] end]}]
        } elseif { $physicalNature == "freePhysicalNature" } {
            #it's like component
            #could be for example: Ex Ey Ez mE
            set component_names ""
            foreach component_name [lindex [$h get $dim] 0] {
                lappend component_names [string trim $component_name]
            }
            set idim_component_names [expr {$ndimensions-[string index [lindex [split $dim /] end] end]}]
        } elseif { $physicalNature == "EnvironnementNumber" } {
            #is this a time step??
            set time_steps [lindex [$h get $dim] 0]
            set idim_time_steps [expr {$ndimensions-[string index [lindex [split $dim /] end] end]}]
        } elseif { $physicalNature == "time" } {
            set time_steps [lindex [$h get $dim] 0]
            set idim_time_steps [expr {$ndimensions-[string index [lindex [split $dim /] end] end]}]
        } elseif { $physicalNature == "frequency" } {
            set time_steps [lindex [$h get $dim] 0]
            set idim_time_steps [expr {$ndimensions-[string index [lindex [split $dim /] end] end]}]
        } else {
            #Amelet::Warning [_ "unexpected physicalNature %s, assumed like time" $physicalNature]
            #not implemented
            set time_steps [lindex [$h get $dim] 0]
            set idim_time_steps [expr {$ndimensions-[string index [lindex [split $dim /] end] end]}]
        }
    }

    if { $meshEntity != "" } {
        set parent_mesh [Amelet::GetParentMesh $h $meshEntity]
        set mesh_type [Amelet::IsMesh $h $parent_mesh]
        if { $mesh_type == "" } {
            Amelet::Warning [_ "parent mesh of %s not found" $meshEntity]
            return [list 0 0]
        }
        if { [info exists _offsets_mesh_post($parent_mesh)] } {
            lassign $_offsets_mesh_post($parent_mesh) offset_nodes offset_elements
        } else {
            set offset_nodes 0
            set offset_elements 0
        }
        set gidpost_resultname ""
        set gidpost_analysisname ""
        set gidpost_step ""
        set gidpost_type ""
        set gidpost_over ""
        set gidpost_ids ""
        set gidpost_values ""
        set nodes ""
        set elements ""
        set type [Amelet::GetAttribute $h $meshEntity type]
        if { $type == "node" } {
            set gidpost_over OnNodes
            set nodes [lindex [$h get $meshEntity] 0]
            if { $mesh_type == "unstructured" } {
                #node numbers start by 1 in GiD
                foreach id $nodes {
                    lappend gidpost_ids [expr {$id+$offset_nodes+1}]
                }
            } elseif { $mesh_type == "structured" } {
                #instead of node numbers, there are grid i j k index
                if { [expr {[llength $nodes]%3}] == 0 } {
                    foreach {i j k} $nodes {
                        set num [Amelet::GetNumFromIjk $i $j $k]
                        if { [info exists _node_ids($num)] } {
                            #node numbers start by 1 in GiD
                            lappend gidpost_ids [expr {$_node_ids($num)+$offset_nodes+1}]
                        } else {
                            Amelet::Warning [_ "node number %d with index %s %s %s was not previously created" $num $i $j $k]
                            return [list $num_results $num_graphs]
                        }
                    }
                } else {
                    Amelet::Warning [_ "number of node index %s of %s is expected to be multiple of 3" [llength $nodes] $meshEntity]
                    return [list $num_results $num_graphs]
                }
            } else {
                Amelet::Warning [_ "Unexpected mesh type '%s' for mesh '%s'" $mesh_type $meshEntity]
                return [list $num_results $num_graphs]
            }
        } elseif { $type == "element" } {
            set gidpost_over OnGaussPoints
            set entityType [Amelet::GetAttribute $h $meshEntity entityType]
            if { $entityType == "edge" } {
                set gidpost_gauss_point_name "GP_LINEAR_1"
            } elseif { $entityType == "face" } {
                if { $mesh_type == "unstructured" } {
                    set firstElement [lindex [lindex [$h get $meshEntity] 0] 0]
                    set element_type [lindex [lindex [$h get $parent_mesh/elementTypes] 0] $firstElement]
                    set gidpost_element_type [GetGidPostElementType $element_type]
                    if { $gidpost_element_type == "Triangle" } {
                        set gidpost_gauss_point_name "GP_TRIANGLE_1"
                    } elseif { $gidpost_element_type == "Quadrilateral" } {
                        set gidpost_gauss_point_name "GP_QUADRILATERAL_1"
                    } else {
                        Amelet::Warning [_ "Unexpected element type '%s' for mesh '%s'" $element_type $parent_mesh]
                        return [list $num_results $num_graphs]
                    }
                } else {
                    set gidpost_gauss_point_name "GP_QUADRILATERAL_1"
                }
            } elseif { $entityType == "volume" } {
                if { $mesh_type == "unstructured" } {
                    set firstElement [lindex [lindex [$h get $meshEntity] 0] 0]
                    set element_type [lindex [lindex [$h get $parent_mesh/elementTypes] 0] $firstElement]
                    set gidpost_element_type [GetGidPostElementType $element_type]
                    if { $gidpost_element_type == "Tetrahedra" } {
                        set gidpost_gauss_point_name "GP_TETRAHEDRA_1"
                    } elseif { $gidpost_element_type == "HexahedRa" } {
                        set gidpost_gauss_point_name "GP_HEXAHEDRA_1"
                    } elseif { $gidpost_element_type == "Prism" } {
                        set gidpost_gauss_point_name "GP_PRISM_1"
                    } elseif { $gidpost_element_type == "Pyramid" } {
                        set gidpost_gauss_point_name "GP_PYRAMID_1"
                    } else {
                        Amelet::Warning [_ "Unexpected element type '%s' for mesh '%s'" $element_type $parent_mesh]
                        return [list $num_results $num_graphs]
                    }
                } else {
                    set gidpost_gauss_point_name "GP_HEXAHEDRA_1"
                }
            } else {
                Amelet::Warning [_ "Unexpected value '%s' of attribute 'entityType' for mesh '%s'. Must be 'edge', 'face' or 'volume'" $entityType $meshEntity]
                return [list $num_results $num_graphs]
            }
            set location [Amelet::GetAttribute $h $meshEntity location]
            if { $location == "barycenter" } {
            } elseif { $location == "node" } {
            } elseif { $location == "middleEdge" } {
            } elseif { $location == "centerFace" } {
            } else {
                if { $location != "" } {
                    Amelet::Warning [_ "Unexpected value '%s' of attribute 'location' for mesh %s, Must be 'barycenter', 'node', 'middleEdge' or 'centerFace'" $location $meshEntity]
                }
            }

            if { $mesh_type == "unstructured" } {
                set elements [lindex [$h get $meshEntity] 0]
                foreach id $elements {
                    lappend gidpost_ids [expr {$id+$offset_elements+1}]
                }
            } elseif { $mesh_type == "structured" } {
                #set elements [lindex [$h get $meshEntity] 0]
                #
                #elements are defined by its six corner integer coordinates
                #previously must have saved _structured_elements_post_range_by_group($meshEntity) of each group
                if { [info exists _structured_elements_post_range_by_group($meshEntity)] } {
                    set gidpost_ids [objarray new_from_to intarray [lindex $_structured_elements_post_range_by_group($meshEntity) 0] [lindex $_structured_elements_post_range_by_group($meshEntity) 1]]
                } else {
                    Amelet::Warning [_ "structured elements of group %s not found" $meshEntity]
                    return [list $num_results $num_graphs]
                }
            } else {
                Amelet::Warning [_ "Unexpected value '%s' of attribute 'type' for mesh %s, Must be 'unstructured' or 'structured'" $mesh_type $parent_mesh]
                return [list $num_results $num_graphs]
            }
        } else {
            Amelet::Warning [_ "Unexpected value '%s' of attribute 'type' for mesh %s, Must be 'node' or 'element'" $type $meshEntity]
            return [list $num_results $num_graphs]
        }
        if { $type == "node" || $type == "element"  } {
            set data_dimensions [lindex [$h get -dimensions $resultpath/data] 0]
            set data [lindex [$h get $resultpath/data] 0]
            set ndata [llength $data]
            set ndimensions [llength $data_dimensions]

            if { $ndimensions == 3 || $ndimensions == 2 } {
                set total_dimensions [lindex $data_dimensions 0]
                for {set i 1} {$i<$ndimensions} {incr i} {
                    set total_dimensions [expr {$total_dimensions*[lindex $data_dimensions $i]}]
                }
                set ncompound [expr {$ndata/$total_dimensions}]
                if { $idim_component_names != "" && $idim_component_names >=0 } {
                    set num_components [lindex $data_dimensions $idim_component_names]
                } else {
                    set num_components 1
                }
                if { $num_components == 1 } {
                    #scalar
                    if { $ncompound == 1 } {
                        set gidpost_type Scalar
                        set gidpost_values [list $data]
                        if { [llength $component_names] == 1 } {
                            set gidpost_component_names [list [lindex $component_names 0]]
                        } else {
                            set gidpost_component_names "";#[list x]
                        }
                    } elseif { $ncompound == 2 } {
                        set gidpost_type ComplexScalar
                        foreach axe {0} {
                            set reals($axe) ""
                            set imaginaries($axe) ""
                            foreach {r i} $data {
                                lappend reals($axe) $r
                                lappend imaginaries($axe) $i
                            }
                        }
                        set gidpost_values [list $reals(0) $imaginaries(0)]
                        if { [llength $component_names] == 1 } {
                            set gidpost_component_names [list \
                                Re-[lindex $component_names 0] Im-[lindex $component_names 0]]
                        } else {
                            set gidpost_component_names "";#[list Re-x Im-x]
                        }
                    } else {
                    }
                } elseif { $num_components == 3 } {
                    if { [expr {$ndata%3}] != 0 } {
                        Amelet::Warning [_ "wrong number of results in %s" $resultpath]
                        return [list $num_results $num_graphs]
                    }
                    #vector
                    set d(0) [lrange $data 0 [expr {$ndata/3-1}]]
                    set d(1) [lrange $data [expr {$ndata/3}] [expr {$ndata/3*2-1}]]
                    set d(2) [lrange $data [expr {$ndata/3*2}] end]
                    if { $ncompound == 1 } {
                        set gidpost_type Vector
                        set gidpost_values [list $d(0) $d(1) $d(2)]
                        if { [llength $component_names] == 3 } {
                            set gidpost_component_names [list \
                                [lindex $component_names 0] [lindex $component_names 1] [lindex $component_names 2]]
                        } else {
                            set gidpost_component_names [list x y z]
                        }
                    } elseif { $ncompound == 2 } {
                        set gidpost_type ComplexVector
                        foreach axe {0 1 2} {
                            set reals($axe) ""
                            set imaginaries($axe) ""
                            foreach {r i} $d($axe) {
                                lappend reals($axe) $r
                                lappend imaginaries($axe) $i
                            }
                        }
                        set gidpost_values [list $reals(0) $imaginaries(0) \
                            $reals(1) $imaginaries(1) $reals(2) $imaginaries(2)]
                        if { [llength $component_names] == 3 } {
                            set gidpost_component_names [list \
                                Re-[lindex $component_names 0] Im-[lindex $component_names 0] \
                                Re-[lindex $component_names 1] Im-[lindex $component_names 1] \
                                Re-[lindex $component_names 2] Im-[lindex $component_names 2]]
                        } else {
                            set gidpost_component_names "";#[list Re-x Im-x Re-y Im-y Re-z Im-z]
                        }
                    } else {
                        Amelet::Warning [_ "Unexpected compound number of items %s in %s" $ncompound $resultpath]
                        return [list $num_results $num_graphs]
                    }
                } elseif { $num_components == 4 } {
                    if { [expr {$ndata%4}] != 0 } {
                        Amelet::Warning [_ "wrong number of results in %s" $resultpath]
                        return [list $num_results $num_graphs]
                    }
                    #vector and modulus
                    set d(0) [lrange $data 0 [expr {$ndata/4-1}]]
                    set d(1) [lrange $data [expr {$ndata/4}] [expr {$ndata/4*2-1}]]
                    set d(2) [lrange $data [expr {$ndata/4*2}] [expr {$ndata/4*3-1}]]
                    set d(3) [lrange $data [expr {$ndata/4*3}] end]
                    if { $ncompound == 1 } {
                        set gidpost_type Vector
                        set gidpost_values [list $d(0) $d(1) $d(2) $d(3)]
                        if { [llength $component_names] == 4 } {
                            set gidpost_component_names [list \
                                [lindex $component_names 0] [lindex $component_names 1] [lindex $component_names 2]]
                        } else {
                            set gidpost_component_names "";#[list x y z |.|]
                        }
                    } elseif { $ncompound == 2 } {
                        set gidpost_type ComplexVector
                        foreach axe {0 1 2 3} {
                            set reals($axe) ""
                            set imaginaries($axe) ""
                            foreach {r i} $d($axe) {
                                lappend reals($axe) $r
                                lappend imaginaries($axe) $i
                            }
                        }
                        set gidpost_values [list $reals(0) $imaginaries(0) \
                            $reals(1) $imaginaries(1) $reals(2) $imaginaries(2) \
                            $reals(3) $imaginaries(3)]
                        if { [llength $component_names] == 4 } {
                            set gidpost_component_names [list \
                                Re-[lindex $component_names 0] Im-[lindex $component_names 0] \
                                Re-[lindex $component_names 1] Im-[lindex $component_names 1] \
                                Re-[lindex $component_names 2] Im-[lindex $component_names 2] \
                                Re-[lindex $component_names 3] Im-[lindex $component_names 3]]
                        } else {
                            set gidpost_component_names "";#[list Re-x Im-x Re-y Im-y Re-z Im-z |Re| |Im|]
                        }
                    } else {
                        Amelet::Warning [_ "Unexpected compound number of items %s in %s" $ncompound $resultpath]
                        return [list $num_results $num_graphs]
                    }
                } else {
                    Amelet::Warning [_ "Unexpected num components %s in %s" $num_components $resultpath]
                    return [list $num_results $num_graphs]
                }
            } else {
                Amelet::Warning [_ "Unexpected ndimensions %s in %s" $ndimensions $resultpath]
                return [list $num_results $num_graphs]
            }


            #optional attributes: physicalNature label unit comment
            array set data_attributes [Amelet::GetAttributes $h $resultpath/data]
            if { [info exists data_attributes(label)] } {
                set gidpost_resultname [string trim $data_attributes(label)]
            } else {
                if { [info exists data_attributes(physicalNature)] } {
                    set gidpost_resultname [string trim $data_attributes(physicalNature)]
                } else {
                    set gidpost_resultname [lindex [split $resultpath /] end]
                }
            }

            if { [info exists data_attributes(unit)] } {
                set unit_name [string trim $data_attributes(unit)]
            } else {
                set unit_name ""
            }
            if { [info exists data_attributes(gidAnalysis)] } { #gidAnalysis is not Amelet standar
            set gidpost_analysisname [string trim $data_attributes(gidAnalysis)]
        } else {
            set gidpost_analysisname none
        }

        if { $time_steps == "" } {
            set time_steps [list 1]
        }

        set n_steps [llength $time_steps]
        if { $n_steps == 1 }  {
            set gidpost_step [lindex $time_steps 0]
            set gidpost_result [list Result $gidpost_resultname $gidpost_analysisname $gidpost_step $gidpost_type $gidpost_over]
            if { $gidpost_over == "OnGaussPoints" } {
                lappend gidpost_result $gidpost_gauss_point_name
            }
            if { $gidpost_type == "ComplexScalar" && [GidUtils::VersionCmp "10.1.9d"] < 0 } {
                #trick to work also with versions prior to 10.1.9d that have a bug with -array for ComplexScalar
                set gidpost_some_values ""
                foreach id $gidpost_ids re [lindex $gidpost_values 0] im [lindex $gidpost_values 1] {
                    lappend gidpost_some_values [list $id $re $im]
                }
                GiD_Result create $gidpost_result [list ComponentNames {*}$gidpost_component_names] [list Unit $unit_name] {*}$gidpost_some_values
            } else {
                set gidpost_some_values [list $gidpost_ids $gidpost_values]
                if { $gidpost_component_names != "" } {
                    if { $unit_name != "" } {
                        GiD_Result create -array $gidpost_result [list ComponentNames {*}$gidpost_component_names] [list Unit $unit_name] $gidpost_some_values
                    } else {
                        GiD_Result create -array $gidpost_result [list ComponentNames {*}$gidpost_component_names] $gidpost_some_values
                    }
                } else {
                    if { $unit_name != "" } {
                        GiD_Result create -array $gidpost_result [list Unit $unit_name] $gidpost_some_values
                    } else {
                        GiD_Result create -array $gidpost_result $gidpost_some_values
                    }
                }
            }
            incr num_results
        } else {
            set n_gidpost_values [llength [lindex $gidpost_values 0]]
            set n_ids [llength $gidpost_ids]
            if { [expr {$n_gidpost_values/$n_steps}] != $n_ids } {
                Amelet::Warning [_ "number of results %s divided by %d steps not match with number of ids %s in %s" $n_gidpost_values $n_steps $n_ids $resultpath]
                return [list $num_results $num_graphs]
            }
            set i_step 0
            foreach gidpost_step $time_steps {
                set v {}
                foreach item $gidpost_values {
                    lappend v [lrange $item [expr {$n_ids*$i_step}] [expr {$n_ids*($i_step+1)-1}]]
                }
                set gidpost_result [list Result $gidpost_resultname $gidpost_analysisname $gidpost_step $gidpost_type $gidpost_over]
                if { $gidpost_over == "OnGaussPoints" } {
                    lappend gidpost_result $gidpost_gauss_point_name
                }
                if { $gidpost_type == "ComplexScalar" && [GidUtils::VersionCmp "10.1.9d"] < 0 } {
                    #trick to work also with versions prior to 10.1.9d that have a bug with -array for ComplexScalar
                    set gidpost_some_values ""
                    foreach id $gidpost_ids re [lindex $v 0] im [lindex $v 1] {
                        lappend gidpost_some_values [list $id $re $im]
                    }
                    GiD_Result create $gidpost_result [list ComponentNames {*}$gidpost_component_names] [list Unit $unit_name] {*}$gidpost_some_values
                } else {
                    set gidpost_some_values [list $gidpost_ids $v]
                    if { $gidpost_component_names != "" } {
                        if { $unit_name != "" } {
                            GiD_Result create -array $gidpost_result [list ComponentNames {*}$gidpost_component_names] [list Unit $unit_name] $gidpost_some_values
                        } else {
                            GiD_Result create -array $gidpost_result [list ComponentNames {*}$gidpost_component_names] $gidpost_some_values
                        }
                    } else {
                        if { $unit_name != "" } {
                            GiD_Result create -array $gidpost_result [list Unit $unit_name] $gidpost_some_values
                        } else {
                            GiD_Result create -array $gidpost_result $gidpost_some_values
                        }
                    }
                }
                incr i_step
            }
            incr num_results
        }
    }
} else {
    #there is no mesh, could be a 1D graph or 2D/3D graph
    if { $time_steps != "" } {
        set data_dimensions [lindex [$h get -dimensions $resultpath/data] 0]
        set data [lindex [$h get $resultpath/data] 0]
        set ndata [llength $data]
        set ndimensions [llength $data_dimensions]
        if { $ndimensions == 1 || $ndimensions == 2 || $ndimensions == 3 } {
            #graph 1D
            #or graph 2D/3D -> could create an auxiliary mesh grid to represent it
            set total_dimensions [lindex $data_dimensions 0]
            for {set i 1} {$i<$ndimensions} {incr i} {
                set total_dimensions [expr {$total_dimensions*[lindex $data_dimensions $i]}]
            }
            set ncompound [expr {$ndata/$total_dimensions}]
            if { $idim_component_names != "" && $idim_component_names >=0 } {
                set num_components [lindex $data_dimensions $idim_component_names]
            } else {
                set num_components 1
            }
            if { $num_components == 1 } {
                #scalar
                if { $ncompound == 1 } {
                    set gidpost_type Scalar
                    set gidpost_values [list $data]
                    if { [llength $component_names] == 1 } {
                        set gidpost_component_names [list [lindex $component_names 0]]
                    } else {

                        set gidpost_component_names "";#[list x]
                    }
                } elseif { $ncompound == 2 } {
                    set gidpost_type ComplexScalar
                    foreach axe {0} {
                        set reals($axe) ""
                        set imaginaries($axe) ""
                        foreach {r i} $data {
                            lappend reals($axe) $r
                            lappend imaginaries($axe) $i
                        }
                    }
                    set gidpost_values [list $reals(0) $imaginaries(0)]
                    if { [llength $component_names] == 1 } {
                        set gidpost_component_names [list Re-[lindex $component_names 0] Im-[lindex $component_names 0]]
                    } else {
                        set gidpost_component_names "";#[list Re-x Im-x]
                    }
                } else {
                }
            } elseif { $num_components == 3 } {
                if { [expr {$ndata%3}] != 0 } {
                    Amelet::Warning [_ "wrong number of results in %s" $resultpath]
                    return [list $num_results $num_graphs]
                }
                #vector
                set d(0) [lrange $data 0 [expr {$ndata/3-1}]]
                set d(1) [lrange $data [expr {$ndata/3}] [expr {$ndata/3*2-1}]]
                set d(2) [lrange $data [expr {$ndata/3*2}] end]
                if { $ncompound == 1 } {
                    set gidpost_type Vector
                    set gidpost_values [list $d(0) $d(1) $d(2)]
                    if { [llength $component_names] == 3 } {
                        set gidpost_component_names [list \
                            [lindex $component_names 0] [lindex $component_names 1] [lindex $component_names 2]]
                    } else {
                        set gidpost_component_names "";#[list x y z]
                    }
                } elseif { $ncompound == 2 } {
                    set gidpost_type ComplexVector
                    foreach axe {0 1 2} {
                        set reals($axe) ""
                        set imaginaries($axe) ""
                        foreach {r i} $d($axe) {
                            lappend reals($axe) $r
                            lappend imaginaries($axe) $i
                        }
                    }
                    set gidpost_values [list $reals(0) $imaginaries(0) \
                        $reals(1) $imaginaries(1) $reals(2) $imaginaries(2)]
                    if { [llength $component_names] == 3 } {
                        set gidpost_component_names [list \
                            Re-[lindex $component_names 0] Im-[lindex $component_names 0] \
                            Re-[lindex $component_names 1] Im-[lindex $component_names 1] \
                            Re-[lindex $component_names 2] Im-[lindex $component_names 2]]
                    } else {
                        set gidpost_component_names "";#[list Re-x Im-x Re-y Im-y Re-z Im-z]
                    }
                } else {
                    Amelet::Warning [_ "Unexpected compound number of items %s in %s" $ncompound $resultpath]
                    return [list $num_results $num_graphs]
                }
            } elseif { $num_components == 4 } {
                if { [expr {$ndata%4}] != 0 } {
                    Amelet::Warning [_ "wrong number of results in %s" $resultpath]
                    return [list $num_results $num_graphs]
                }
                #vector and modulus
                set d(0) [lrange $data 0 [expr {$ndata/4-1}]]
                set d(1) [lrange $data [expr {$ndata/4}] [expr {$ndata/4*2-1}]]
                set d(2) [lrange $data [expr {$ndata/4*2}] [expr {$ndata/4*3-1}]]
                set d(3) [lrange $data [expr {$ndata/4*3}] end]
                if { $ncompound == 1 } {
                    set gidpost_type Vector
                    set gidpost_values [list $d(0) $d(1) $d(2) $d(3)]
                    if { [llength $component_names] == 4 } {
                        set gidpost_component_names [list \
                            [lindex $component_names 0] [lindex $component_names 1] [lindex $component_names 2]]
                    } else {
                        set gidpost_component_names "";#[list x y z |.|]
                    }
                } elseif { $ncompound == 2 } {
                    set gidpost_type ComplexVector
                    foreach axe {0 1 2 3} {
                        set reals($axe) ""
                        set imaginaries($axe) ""
                        foreach {r i} $d($axe) {
                            lappend reals($axe) $r
                            lappend imaginaries($axe) $i
                        }
                    }
                    set gidpost_values [list $reals(0) $imaginaries(0) \
                        $reals(1) $imaginaries(1) $reals(2) $imaginaries(2) \
                        $reals(3) $imaginaries(3)]
                    if { [llength $component_names] == 4 } {
                        set gidpost_component_names [list \
                            Re-[lindex $component_names 0] Im-[lindex $component_names 0] \
                            Re-[lindex $component_names 1] Im-[lindex $component_names 1] \
                            Re-[lindex $component_names 2] Im-[lindex $component_names 2] \
                            Re-[lindex $component_names 3] Im-[lindex $component_names 3]]
                    } else {
                        set gidpost_component_names "";#[list Re-x Im-x Re-y Im-y Re-z Im-z |Re| |Im|]
                    }
                } else {
                    Amelet::Warning [_ "Unexpected compound number of items %s in %s" $ncompound $resultpath]
                    return [list $num_results $num_graphs]
                }
            } else {
                Amelet::Warning [_ "Unexpected num components %s in %s" $num_components $resultpath]
                return [list $num_results $num_graphs]
            }


            if { $ndimensions == 1 } {
                #graph 1D
                set label [Amelet::GetAttribute $h $resultpath/data label]
                set unit [Amelet::GetAttribute $h $resultpath/data unit]
                if { $gidpost_type == "Scalar" } {
                    if { [llength $gidpost_values] != 1 } {
                        Amelet::Warning [_ "wrong dimension of graph data in %s" $resultpath]
                        return [list $num_results $num_graphs]
                    }
                    if { [llength $time_steps] != [llength [lindex $gidpost_values 0]] } {
                        Amelet::Warning [_ "wrong number of x and y graph data in %s" $resultpath]
                        return [list $num_results $num_graphs]
                    }
                    set gidpost_result [lindex [split $resultpath /] end]
                    set label_x [lindex $gidpost_component_names 0]
                    set unit_x ""
                    if { [$h is_dataset $resultpath/ds/dim1] } {
                        set label_x [Amelet::GetAttribute $h $resultpath/ds/dim1 label $label_x]
                        set unit_x [Amelet::GetAttribute $h $resultpath/ds/dim1 unit]
                    }
                    set label_y $label
                    set unit_y $unit
                    GiD_Graph create $gidpost_result $label_x $label_y $time_steps [lindex $gidpost_values 0] $unit_x $unit_y
                    incr num_graphs
                } elseif { $gidpost_type == "ComplexScalar"} {
                    set gidpost_result [lindex [split $resultpath /] end]
                    set label_x [lindex $gidpost_component_names 0]
                    set unit_x ""
                    if { [$h is_dataset $resultpath/ds/dim1] } {
                        set label_x [Amelet::GetAttribute $h $resultpath/ds/dim1 label $label_x]
                        set unit_x [Amelet::GetAttribute $h $resultpath/ds/dim1 unit]
                    }

                    if { [llength $time_steps] == [llength [lindex $gidpost_values 0]] } {
                        #now create two graphs of x vs Re and x vs Im , with x could represent time, frequency,
                        #maybe in the future GiD_Graph can implement a single graph of x vs (Re,Im)
                        #joining parts real and imaginary in a single complex... (to allow FFT to tranform between time and frequency domains)
                        set label_y $label
                        set unit_y $unit
                        foreach ys $gidpost_values part {Re Im} {
                            if { [llength $time_steps] != [llength $ys] } {
                                Amelet::Warning [_ "wrong number of x and y graph data in %s" $resultpath]
                                continue
                            }
                            GiD_Graph create $gidpost_result-${part} $label_x $label_y $time_steps $ys $unit_x $unit_y
                            incr num_graphs
                        }
                    } else {
                        #now create a x-y graph Re vs Im
                        set label_x Re
                        set label_y Im
                        set unit_x $unit
                        set unit_y $unit
                        GiD_Graph create ${gidpost_result}-Complex $label_x $label_y [lindex $gidpost_values 0] [lindex $gidpost_values 1] $unit_x $unit_y
                        incr num_graphs
                    }
                }
            } elseif { $ndimensions == 2 } {
                #graph 2D -> create an auxiliary mesh grid to represent it...
                set dim1 $resultpath/ds/dim1
                set dim2 $resultpath/ds/dim2
                if { [$h is_dataset $dim1] && [$h is_dataset $dim2] } {
                    set label [Amelet::GetAttribute $h $resultpath/data label]
                    set unit [Amelet::GetAttribute $h $resultpath/data unit]
                    set label_x [Amelet::GetAttribute $h $dim1 label]
                    set label_y [Amelet::GetAttribute $h $dim2 label]
                    set unit_x [Amelet::GetAttribute $h $dim1 unit]
                    set unit_y [Amelet::GetAttribute $h $dim2 unit]
                    set xs [lindex [$h get $dim1] 0]
                    set ys [lindex [$h get $dim2] 0]
                    if { $gidpost_type == "Scalar" || $gidpost_type == "ComplexScalar" || $gidpost_type == "Vector" || $gidpost_type == "ComplexVector"} {
                        Amelet::CreatePostGraph2D $xs $ys $label_x $label_y $unit_x $unit_y $label $unit $gidpost_values $gidpost_type
                        incr num_results
                    } else {
                        Amelet::Warning [_ "Graph2D, can't create 2D graphs of type %s in %s" $gidpost_type $resultpath]
                    }
                } else {
                    Amelet::Warning [_ "Graph2D expected dim1 and dim2 in %s" $resultpath]
                    return [list $num_results $num_graphs]
                }
            } elseif { $ndimensions == 3 } {
                #graph 3D -> create an auxiliary mesh grid to represent it...
                set dim1 $resultpath/ds/dim1
                set dim2 $resultpath/ds/dim2
                set dim3 $resultpath/ds/dim3
                if { [$h is_dataset $dim1] && [$h is_dataset $dim2] && [$h is_dataset $dim3] } {

                    if { $gidpost_type == "Scalar" || $gidpost_type == "ComplexScalar" || $gidpost_type == "Vector" || $gidpost_type == "ComplexVector"} {
                        set label [Amelet::GetAttribute $h $resultpath/data label]
                        set unit [Amelet::GetAttribute $h $resultpath/data unit]
                        if { [llength $time_steps] > 0 } {
                            #create a square mesh with results along the time/frequency
                            if { $idim_time_steps == 0 } {
                                #do nothing
                            } elseif { $idim_time_steps == 1 } {
                                set dim1 $resultpath/ds/dim1
                                set dim2 $resultpath/ds/dim3
                                set dim3 $resultpath/ds/dim2
                            } elseif { $idim_time_steps == 2 } {
                                set dim1 $resultpath/ds/dim2
                                set dim2 $resultpath/ds/dim3
                                set dim3 $resultpath/ds/dim1
                            } else {
                                Amelet::Warning [_ "Graph3D time/frequency unexpected idim_time_steps=%s in %s" $idim_time_steps $resultpath]
                                return [list $num_results $num_graphs]
                            }
                        }
                        set label_x [Amelet::GetAttribute $h $dim1 label]
                        set label_y [Amelet::GetAttribute $h $dim2 label]
                        set label_z [Amelet::GetAttribute $h $dim3 label]
                        set unit_x [Amelet::GetAttribute $h $dim1 unit]
                        set unit_y [Amelet::GetAttribute $h $dim2 unit]
                        set unit_z [Amelet::GetAttribute $h $dim3 unit]
                        set xs [lindex [$h get $dim1] 0]
                        set ys [lindex [$h get $dim2] 0]
                        set zs [lindex [$h get $dim3] 0]
                        if { [llength $time_steps] > 0 } {
                            #create a square mesh with results along the time/frequency
                            incr num_results [Amelet::CreatePostGrid2DTimeResult $xs $ys $zs $label_x $label_y $label_z $unit_x $unit_y $unit_z $label $unit $gidpost_values $gidpost_type]
                        } else {
                            #create a cubic mesh with results with one time step
                            incr num_results [Amelet::CreatePostGraph3D $xs $ys $zs $label_x $label_y $label_z $unit_x $unit_y $unit_z $label $unit $gidpost_values $gidpost_type]
                        }
                    } else {
                        Amelet::Warning [_ "Graph3D, can't create 3D graphs of type %s in %s" $gidpost_type $resultpath]
                    }
                } else {
                    Amelet::Warning [_ "Graph3D expected dim1, dim2 and dim3 in %s" $resultpath]
                    return [list $num_results $num_graphs]
                }
            }
        } else {
            Amelet::Warning [_ "Unexpected ndimensions %s in %s" $ndimensions $resultpath]
            return [list $num_results $num_graphs]
        }
    } else {
        Amelet::Warning [_ "Unexpected graph case in %s" $resultpath]
        return [list $num_results $num_graphs]
    }

}

return [list $num_results $num_graphs]
}

proc Amelet::ReadPostGroupsResults { h resultpath } {
    set num_results 0
    set num_graphs 0
    set groups [$h glob -directory $resultpath -types group *]
    foreach group $groups {
        set type [Amelet::IsResult $h $group]
        if { $type == "arraySet" } {
            lassign [Amelet::ReadPostResultArraySet $h $group] num_results_local num_graphs_local
            incr num_results $num_results_local
            incr num_graphs $num_graphs_local
        } else {
            if { $type != "" } {
                Amelet::Warning [_ "Result or type %s not implemented" $type]
            } else {
                lassign [Amelet::ReadPostGroupsResults $h $group] num_results_local num_graphs_local
                incr num_results $num_results_local
                incr num_graphs $num_graphs_local
            }
        }
    }
    return [list $num_results $num_graphs]
}

proc Amelet::ReadPostResults { filename } {
    variable _grids ;#to avoid create several times the same auxiliary grid
    Amelet::LoadHdf5Package
    if { ![file exists $filename] } {
        WarnWin [_ "File %s not found" $filename]
        return [list 0 0]
    }

    set h [Amelet::GetFreeHDF5Id]
    if { [catch {hdf5 -readonly $h $filename} err] } {
        WarnWin [_ "File %s is not an HDF5 file" $filename]
        return [list 0 0]
    }

    if { ![IsAmelet $h] } {
        WarnWin [_ "File %s is not an Amelet HDF5 file" $filename]
        $h close
        return [list 0 0]
    }

    if { [ $h is_group /floatingType] } {
        unset -nocomplain _grids
        lassign [Amelet::ReadPostGroupsResults $h /floatingType] num_results num_graphs
        unset -nocomplain _grids
    } else {
        set num_results 0
        set num_graphs 0
    }
    $h close
    return [list $num_results $num_graphs]
}


proc Amelet::GetFilesRecursive { dirname } {
    set names [glob -nocomplain -directory $dirname -types f *.h5]
    foreach name [glob -nocomplain -directory $dirname -types d *] {
        lappend names {*}[Amelet::GetFilesRecursive $name]
    }
    return $names
}

proc Amelet::ReadPost0 { filenames } {
    variable _offsets_mesh_post ;#to associate results to the mesh ids
    variable _structured_elements_post_range_by_group
    variable _num_warnings
    variable percent_done
    variable stop
    variable _show_advance_bar
    set _num_warnings 0

    lassign {0 0 0} num_meshes num_results num_graphs

    if { [llength $filenames] == 1 && [file isdirectory $filenames] } {
        #now the window only select files, not directories, but it is possible to invoke with folder
        #from Tcl command
        set filenames [Amelet::GetFilesRecursive $filenames]
    }
    set ::Amelet::stop 0
    set tasks_max [llength $filenames]
    set tasks_done 0
    set ::Vtk::percent_done 0
    if { $_show_advance_bar } {
        ::GidUtils::CreateAdvanceBar [_ "Reading file"] [_ "Process"] ::Amelet::percent_done 100 {set ::Amelet::stop 1}
    }

    foreach filename $filenames {
        if { $::Amelet::stop } {
            break
        }
        if { [file exists $filename] } {
            unset -nocomplain _offsets_mesh_post
            unset -nocomplain _structured_elements_post_range_by_group
            set num_meshes [Amelet::ReadPostMeshes $filename]
            lassign [Amelet::ReadPostResults $filename] num_results_local num_graphs_local
            incr num_results $num_results_local
            incr num_graphs $num_graphs_local
            incr tasks_done
            set ::Amelet::percent_done [expr {int(($tasks_done*100.0)/$tasks_max)}]
            GiD_RaiseEvent GiD_Event_AfterOpenFile $filename AMELET_FORMAT 0
        }
    }

    if { $_show_advance_bar } {
        #on error must delete bar!!
        GidUtils::DeleteAdvanceBar
    }
    unset -nocomplain _offsets_mesh_post
    unset -nocomplain _structured_elements_post_range_by_group

    return [list $num_meshes $num_results $num_graphs]
}

#to drag and drop a file
proc Amelet::ReadPostSingleFile { filename } {
    variable _merge
    GidUtils::EvalAndEnterInBatch Amelet::ReadPost [list $filename] [list merge $Amelet::_merge]
}

#read and finally redraw showing what is read
proc Amelet::ReadPost { filenames { lst_options {}} } {
    set fail 0
    GidUtils::WaitState .gid
    GidUtils::DisableGraphics

    # process lst_options
    if { [ llength $lst_options] > 0} {
        array set my_options $lst_options
        if { [ info exists my_options(merge)]} {
            set Amelet::_merge $my_options(merge)
        }
    }

    if { !$Amelet::_merge } {
        GiD_Process Mescape Files New Yes escape ;#to avoid previous geometry
    }

    if { [catch { set res [Amelet::ReadPost0 $filenames] } msg] } {
        set fail 1
        set num_meshes 0
        set num_results 0
        set num_graphs 0
        W $msg
    } else {
        lassign $res num_meshes num_results num_graphs
    }
    GidUtils::EnableGraphics
    GidUtils::EndWaitState .gid
    GiD_RaiseEvent GiD_Event_BeforeUpdateWindow SETS
    GiD_RaiseEvent GiD_Event_AfterChangeMesh [GiD_Info Mesh NumNodes] [GiD_Info Mesh NumElements]
    if { $num_meshes || $num_results > 0 } {
        GiD_Process Mescape View Zoom Frame escape
    }
    if { $num_results > 0 } {
        #set the first analysis and step to use
        set analysis_to_use [lindex [GiD_Info postprocess get all_analysis] 0]
        set step_to_use [lindex [GiD_Info postprocess get all_steps $analysis_to_use] 0]
        GiD_Process Mescape results analysissel $analysis_to_use $step_to_use escape
    } elseif { $num_graphs > 0 } {
        GiD_Graph show
    }
    GidUtils::SetWarnLine [_ "Read %d meshes and %d results %d graphs" $num_meshes $num_results $num_graphs]
    return
}

#to add more options to the standar file browser
proc Amelet::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkGeometry -text [_ "Merge"] -variable Amelet::_merge
    grid columnconfigure $f 1 -weight 1
    grid $f.chkGeometry -sticky w -columnspan 2

    variable export_dir
    if { ![info exists export_dir] } {
        #trick to show the more options
        set export_dir open
    }
    return Amelet::export_dir
}

proc Amelet::ReadPostW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read Amelet results file"] \
        {} {{{Amelet} {.h5 }} {{All files} {.*}}} {} $multiple \
        [list [_ "Import options"] Amelet::MoreImportOptions]]

    if {$filenames == ""} {
        return
    }
    GidUtils::EvalAndEnterInBatch Amelet::ReadPost $filenames [list merge $Amelet::_merge]
}


#########################################################################################
######################################### WRITE POST ####################################
#########################################################################################
proc Amelet::WritePostSets { h meshpath } {
    variable compress_level

    if { ![$h is_group $meshpath/group] } {
        $h create_group $meshpath/group
    }
    #     if { ![$h is_group $meshpath/groupGroup] } {
    #         $h create_group $meshpath/groupGroup
    #     }

    foreach item [GiD_Info mesh -post elements Any -array] {
        lassign $item elemtype ids connectivities materials

    }

    set entityType face
    set surface_sets [GiD_Info postprocess get all_surfacesets]
    set i 0
    foreach surface_set $surface_sets {
        if { [Amelet::IsValidGroupName $surface_set] } {
            set group $resultpath/$surface_set
        } else {
            set group $resultpath/[Amelet::ForceValidGroupName $surface_set]
        }
        $h set -vtype int -compress $compress_level ${meshpath}/group/$group $elements
        $h set_attribute -type string ${meshpath}/group/$group type element entityType $entityType
        $h set_attribute -type string ${meshpath}/group/$group label $surface_set
        incr i
    }
}

proc Amelet::WritePostGroupsAllEntities { h meshpath elementTypes nnodes } {
    variable compress_level
    variable AmeletLevelType

    set some_result_on_nodes 0
    set some_result_on_elements 0
    set some_result_on_unexpected 0

    foreach analysis_name [GiD_Info postprocess get all_analysis] {
        set time_step [lindex [GiD_Info postprocess get all_steps $analysis_name] 0]
        #assumed that the results of the first step are the same for other steps !!
        set result_names [GiD_Info postprocess get results_list CONTOUR_FILL $analysis_name $time_step]
        foreach result_name $result_names {
            set item [lindex [GiD_Result get -info [list $result_name $analysis_name $time_step]] 0]
            set gidpost_over [lindex $item 5]
            if { $gidpost_over == "OnNodes" } {
                incr some_result_on_nodes
            } elseif { $gidpost_over == "OnGaussPoints" } {
                incr some_result_on_elements
            } else {
                incr some_result_on_unexpected
            }
        }
    }

    if { $some_result_on_unexpected } {
        Amelet::Warning "WritePostGroupsAllEntities: $some_result_on_unexpected results on unexpected location"
    }
    if { !$some_result_on_nodes && !$some_result_on_elements } {
        return
    }
    #write groups allnodes and allelements
    if { ![$h is_group $meshpath/group] } {
        $h create_group $meshpath/group
    }

    if { $some_result_on_nodes } {
        set nodes []
        for {set inode 0} {$inode<$nnodes} {incr inode } {
            lappend nodes $inode
        }
        $h set -vtype int -compress $compress_level ${meshpath}/group/all_nodes $nodes
        $h set_attribute -type string ${meshpath}/group/all_nodes type node
    }

    if { $some_result_on_elements } {
        set all_volumes ""
        set all_faces ""
        set all_edges ""
        set all_unexpected ""
        set ielement 0
        foreach element_code $elementTypes {
            if { $AmeletLevelType($element_code) == "volume" } {
                lappend all_volumes $ielement
            } elseif { $AmeletLevelType($element_code) == "face" } {
                lappend all_faces $ielement
            } elseif { $AmeletLevelType($element_code) == "edge" } {
                lappend all_edges $ielement
            } else {
                lappend all_unexpected $ielement
            }
            incr ielement
        }
        if { [llength $all_unexpected] } {
            Amelet::Warning "WritePostGroupsAllEntities: [llength $all_unexpected] elements of unexpected type"
        }
        set all_elements ""
        foreach entityType {volume face edge} elements [list $all_volumes $all_faces $all_edges] {
            if { [llength $elements] } {
                lappend all_elements all_${entityType}s
                $h set -vtype int -compress $compress_level ${meshpath}/group/all_${entityType}s $elements
                $h set_attribute -type string ${meshpath}/group/all_${entityType}s type element entityType $entityType
            }
        }
        if { [llength $all_elements] } {
            if { ![$h is_group $meshpath/groupGroup] } {
                $h create_group $meshpath/groupGroup
            }
            $h set -vtype string -compress $compress_level ${meshpath}/groupGroup/all_elements $all_elements
            $h set_attribute -type string ${meshpath}/groupGroup/all_elements type element
        }
    }
}

proc Amelet::WritePostUnstructuredMesh { h } {
    variable compress_level
    variable NewElementNum

    array unset NewElementNum

    set meshgroup /mesh/gidpost
    set meshpath $meshgroup/unstructuredmesh
    $h create_group $meshgroup
    $h create_group $meshpath
    $h set_attribute -type string $meshpath type unstructured

    # coordinates
    set nodes [GiD_Info mesh -post nodes -array]
    lassign $nodes ids coordinates
    set num_nodes [llength $ids]
    lassign $coordinates xs ys zs
    set data [list]
    foreach x $xs y $ys z $zs {
        lappend data $x $y $z
    }
    $h set -vtype float -compress $compress_level -dimensions {0 3} ${meshpath}/nodes $data

    # connectivity
    #Amelet nodes must be stored with implicit number 0,..,n-1
    set inode -1
    foreach id $ids {
        set NewNodeNum($id) [incr inode]
    }

    set elementTypes [list]
    set elementNodes [list]
    set elements [GiD_Info mesh -post elements Any -array]
    set ielement -1
    foreach item $elements {
        lassign $item elemtype ids connectivities materials
        set nnodes [llength $connectivities]
        set nodes [list]
        if { $elemtype == "Linear" } {
            if { $nnodes == 2 } {
                lassign $connectivities n1s n2s
                foreach n1 $n1s n2 $n2s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2)
                }
            } elseif { $nnodes == 3 } {
                lassign $connectivities n1s n2s n3s
                foreach n1 $n1s n2 $n2s n3 $n3s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3)
                }
            }
        } elseif { $elemtype == "Triangle" } {
            if { $nnodes == 3 } {
                lassign $connectivities n1s n2s n3s
                foreach n1 $n1s n2 $n2s n3 $n3s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3)
                }
            } elseif { $nnodes == 6 } {
                lassign $connectivities n1s n2s n3s n4s n5s n6s
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) \
                        $NewNodeNum($n4) $NewNodeNum($n5) $NewNodeNum($n6)
                }
            }
        } elseif { $elemtype == "Quadrilateral" } {
            if { $nnodes == 4 } {
                lassign $connectivities n1s n2s n3s n4s
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4)
                }
            } elseif { $nnodes == 8 || $nnodes == 9 } {
                if { nnodes == 8 } {
                    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s
                } else {
                    #9 nodes ignore the last center node
                    lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s
                }
                foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s {
                    lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) \
                        $NewNodeNum($n5) $NewNodeNum($n6) $NewNodeNum($n7) $NewNodeNum($n8)
                }
            }
        } elseif { $elemtype == "Tetrahedra" } {
            if { $nnodes == 4 } {
                lassign $connectivities n1s n2s n3s n4s
            } else {
                #10 nodes ignore all quadratic ones
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s
            }
            foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s {
                lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4)
            }
        } elseif { $elemtype == "Pyramid" } {
            if { $nnodes == 5 } {
                lassign $connectivities n1s n2s n3s n4s n5s
            } else {
                #13 nodes ignore all quadratic ones
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s n11s n12s n13s
            }
            foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s {
                lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) $NewNodeNum($n5)
            }
        } elseif { $elemtype == "Prism" } {
            if { $nnodes == 6 } {
                lassign $connectivities n1s n2s n3s n4s n5s n6s
            } else {
                #15 nodes ignore all quadratic ones
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s n11s n12s n13s n14s n15s
            }
            foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s {
                lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) \
                    $NewNodeNum($n4) $NewNodeNum($n5) $NewNodeNum($n6)
            }
        } elseif { $elemtype == "Hexahedra" } {
            if { $nnodes == 8 } {
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s
            } elseif { $nnodes == 20 } {
                #20 nodes ignore all quadratic ones
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s \
                    n11s n12s n13s n14s n15s n16s n17s n18s n19s n20s
            } else {
                #27 nodes ignore all quadratic ones
                lassign $connectivities n1s n2s n3s n4s n5s n6s n7s n8s n9s n10s \
                    n11s n12s n13s n14s n15s n16s n17s n18s n19s n20s \
                    n21s n22s n23s n24s n25s n26s n27s
            }
            foreach n1 $n1s n2 $n2s n3 $n3s n4 $n4s n5 $n5s n6 $n6s n7 $n7s n8 $n8s {
                lappend nodes $NewNodeNum($n1) $NewNodeNum($n2) $NewNodeNum($n3) $NewNodeNum($n4) \
                    $NewNodeNum($n5) $NewNodeNum($n6) $NewNodeNum($n7) $NewNodeNum($n8)
            }
        } elseif { $elemtype == "Circle" } {
            continue
        } elseif { $elemtype == "Sphere" } {
            continue
        } elseif { $elemtype == "Point" } {
            continue
        } else {
            continue
        }
        set elementTypes [concat $elementTypes [lrepeat [llength $ids] [Amelet::ElementCode $elemtype $nnodes]]]
        set elementNodes [concat $elementNodes $nodes]
        #Amelet elements must stored with implicit number 0,..,n-1
        foreach id $ids {
            set NewElementNum($id) [incr ielement]
        }
    }
    $h set -vtype char -compress $compress_level ${meshpath}/elementTypes $elementTypes
    $h set -vtype int -compress $compress_level ${meshpath}/elementNodes $elementNodes


    if { 1 } {
        #to have groups with all elements and nodes to be pointed by results dim1
        Amelet::WritePostGroupsAllEntities $h $meshpath $elementTypes $num_nodes
    } else {
        #to have the sets as groups information
        #Amelet::WritePostSets $h $meshpath
    }

    #$h create_group $meshpath/selectorOnMesh
    #Amelet::WritePostSelectorOnmeshUnstructuredMesh $h $meshpath
    return 1
}


proc Amelet::WritePostMeshes { filename } {
    Amelet::LoadHdf5Package
    set h [Amelet::GetFreeHDF5Id]
    if { [file exists $filename] } {
        #WarnWin [_ "File %s already exists" $filename]
        #return 0
        #open to append
        hdf5 $h $filename
    } else {
        hdf5 $h $filename
        $h set_attribute / FORMAT AMELETHDF AMELETHDF_FORMAT_VERSION 1.5.1
    }
    if { ![$h is_group /mesh] } {
        $h create_group /mesh
    }
    set num_meshes [Amelet::WritePostUnstructuredMesh $h]
    $h close
    return $num_meshes
}

proc Amelet::WritePostGraph { h resultpath graph_name } {
    variable compress_level

    if { [lsearch [GiD_Graph list] $graph_name] == -1 } {
        return 0
    }

    set num_graphs 0
    lassign [GiD_Graph get $graph_name] label_x label_y xs ys unit_x unit_y

    if { [Amelet::IsValidGroupName $graph_name] } {
        set group $resultpath/$graph_name
    } else {
        set group $resultpath/[Amelet::ForceValidGroupName $graph_name]
    }
    if { ![$h is_group $group] } {
        $h create_group $group
    }
    $h set_attribute $group floatingType arraySet
    $h set_attribute $group label $graph_name

    $h set -vtype float -compress $compress_level $group/data $ys
    $h set_attribute $group/data label $label_y
    $h set_attribute $group/data unit $unit_y
    $h set_attribute $group/data physicalNature freePhysicalNature

    if { ![$h is_group $group/ds] } {
        $h create_group $group/ds
    }
    $h set -vtype float -compress $compress_level $group/ds/dim1 $xs
    $h set_attribute $group/ds/dim1 label $label_x
    $h set_attribute $group/ds/dim1 unit $unit_x
    $h set_attribute $group/ds/dim1 physicalNature time

    incr num_graphs
    return $num_graphs
}

proc Amelet::WritePostGraphSet { h resultpath graphset_name graph_name } {
    variable compress_level

    if { [lsearch [GiD_Graph list $graphset_name $graphset_name] $graph_name] == -1 } {
        return 0
    }

    set num_graphs 0
    lassign [GiD_Graph get $graph_name $graphset_name] label_x label_y xs ys unit_x unit_y

    if { [Amelet::IsValidGroupName $graph_name] } {
        set group $resultpath/$graph_name
    } else {
        set group $resultpath/[Amelet::ForceValidGroupName $graph_name]
    }
    if { ![$h is_group $group] } {
        $h create_group $group
    }
    $h set_attribute $group floatingType arraySet
    $h set_attribute $group label $graph_name

    $h set -vtype float -compress $compress_level $group/data $ys
    $h set_attribute $group/data label $label_y
    $h set_attribute $group/data unit $unit_y
    $h set_attribute $group/data physicalNature freePhysicalNature

    if { ![$h is_group $group/ds] } {
        $h create_group $group/ds
    }
    $h set -vtype float -compress $compress_level $group/ds/dim1 $xs
    $h set_attribute $group/ds/dim1 label $label_x
    $h set_attribute $group/ds/dim1 unit $unit_x
    $h set_attribute $group/ds/dim1 physicalNature time

    incr num_graphs
    return $num_graphs
}

#this procedure is used to save in a new file the specified graph,
#or all graphs if graph_name == "all" (this is a limitation of the graph names, this special word 'all' can't be used)
proc Amelet::WritePostGraphToFile { filename graph_name } {
    Amelet::LoadHdf5Package
    if { [ file exists $filename] } {
        if { [ catch { file delete $filename } err] } {
            WarnWin $err
            return 1
        }
    }
    set h [Amelet::GetFreeHDF5Id]
    hdf5 $h $filename
    $h set_attribute / FORMAT AMELETHDF AMELETHDF_FORMAT_VERSION 1.5.1
    set group /floatingType
    if { ![$h is_group $group] } {
        $h create_group $group
    }
    if { $graph_name != "all" } {
        if { [lsearch [GiD_Graph list] $graph_name] == -1 } {
            #ExportGraph has replaced before underline by space, try the inverse...
            regsub -all { } $graph_name {_} graph_name2
            if { [lsearch [GiD_Graph list] $graph_name2] == -1 } {
                WarnWin [_ "There is no graph with name %s" $graph_name]
                return 1
            } else {
                set graph_name $graph_name2
            }
        }
        Amelet::WritePostGraph $h $group $graph_name
    } else {
        foreach graph_name [GiD_Graph list] {
            Amelet::WritePostGraph $h $group $graph_name
        }
    }
    $h close
}

#what should be: AllGraphSets OneGraphSet OneGraph
proc Amelet::WritePostGraphSetToFile { filename what graphset_name graph_name } {
    Amelet::LoadHdf5Package
    if { [ file exists $filename] } {
        if { [ catch { file delete $filename } err] } {
            WarnWin $err
            return 1
        }
    }
    set h [Amelet::GetFreeHDF5Id]
    hdf5 $h $filename
    $h set_attribute / FORMAT AMELETHDF AMELETHDF_FORMAT_VERSION 1.5.1
    set group /floatingType
    if { ![$h is_group $group] } {
        $h create_group $group
    }
    if { $what == "AllGraphSets" } {
        foreach graphset_name [GiD_GraphSet list] {
            foreach graph_name [GiD_Graph list $graphset_name] {
                Amelet::WritePostGraphSet $h $group $graphset_name $graph_name
            }
        }
    } elseif { $what == "OneGraphSet" } {
        foreach graph_name [GiD_Graph list $graphset_name] {
            Amelet::WritePostGraphSet $h $group $graphset_name $graph_name
        }
    } elseif { $what == "OneGraph" } {
        Amelet::WritePostGraphSet $h $group $graphset_name $graph_name
    } else {
        WarnWin [_ "Unexpected case %s" $what]
    }
    $h close
}


proc Amelet::WritePostResult { h resultpath analysis_name result_name } {
    variable compress_level

    set num_results 0

    set time_steps_anaysis [GiD_Info postprocess get all_steps $analysis_name]
    set time_steps "" ;#maybe some times are not defined for this result
    foreach time_step $time_steps_anaysis {
        if { [catch {GiD_Result get -info [list $result_name $analysis_name $time_step]} err] } {
            set result_not_exists_for_this_step 1
        } else {
            lappend time_steps $time_step
        }
    }
    set gidpost_type ""
    set gidpost_over ""
    set gidpost_unit ""
    set gidpost_component_names ""
    if { $time_steps  == "" } {
        return 0
    }
    set gidpost_ids ""
    set result_info [GiD_Result get -array [list $result_name $analysis_name [lindex $time_steps 0]]]
    foreach item $result_info {
        if { [lindex $item 0] == "Result"} {
            #Result "electricFieldPhase" "Efieldphase0planxz" 3e+007 Vector OnGaussPoints "GP QUADRILATERAL 1"
            set gidpost_type [lindex $item 4]
            set gidpost_over [lindex $item 5]
        } elseif { [lindex $item 0] == "Unit"} {
            set gidpost_unit [lindex $item 1]
        } elseif { [lindex $item 0] == "ComponentNames"} {
            set gidpost_component_names [lrange $item 1 end]
        } else {
            set gidpost_ids [lindex $item 0]
        }
    }

    if { [Amelet::IsValidGroupName $result_name] } {
        set group $resultpath/$result_name
    } else {
        set group $resultpath/[Amelet::ForceValidGroupName $result_name]
    }
    if { ![$h is_group $group] } {
        $h create_group $group
    }
    $h set_attribute $group floatingType arraySet
    $h set_attribute $group label $result_name
    if { $gidpost_over == "OnNodes" } {
        set meshEntity /mesh/gidpost/unstructuredmesh/group/all_nodes
    } else {
        #if there are mixed volumes,faces, and edges is necessary a groupGroup to join all in a single name
        #set meshEntity /mesh/gidpost/unstructuredmesh/group/groupGroup/all_elements
        set meshEntity /mesh/gidpost/unstructuredmesh/group/all_faces
    }
    $h create_group $group/ds
    $h set -vtype string -compress $compress_level $group/ds/dim1 $meshEntity
    $h set_attribute $group/ds/dim1 physicalNature meshEntity
    $h set -vtype float -compress $compress_level $group/ds/dim2 $time_steps
    $h set_attribute $group/ds/dim2 physicalNature time ;#could be frequency or other...
    #$h set_attribute $group/ds/dim2 unit second

    if { $gidpost_component_names != "" } {
        set component_names $gidpost_component_names
    } else {
        if { $gidpost_type == "Scalar" } {
            set component_names [list x]
        } elseif { $gidpost_type == "Vector" } {
            set component_names [list x y z]
        } else {
            set component_names ""
        }
    }

    foreach time_step $time_steps {
        set result_data [lindex [GiD_Result get -array [list $result_name $analysis_name $time_step]] end]
        if { $gidpost_ids != [lindex $result_data 0] } {
            must be the same for all time steps!!
        }
        set i_component 0
        foreach component_values [lindex $result_data 1] {
            foreach value $component_values {
                lappend data($i_component) $value
            }
            incr i_component
        }
    }

    set num_columns [llength $gidpost_ids]
    if { $gidpost_type == "Scalar" } {
        $h set -vtype float -compress $compress_level -dimensions [list 0 $num_columns] $group/data $data(0)
    } else {
        set num_components [llength [array names data]]
        if { [llength $component_names] != $num_components } {
            Amelet::Warning "WritePostResult: component_names='$component_names' but num_components=$num_components"
        }
        set data_joined {}
        for {set i_component 0} {$i_component<$num_components} {incr i_component} {
            foreach value $data($i_component) {
                lappend data_joined $value
            }
        }
        set num_rows [llength $time_steps]
        $h set -vtype float -compress $compress_level -dimensions [list $num_rows $num_columns $num_components] $group/data $data_joined
        $h set -vtype string -compress $compress_level $group/ds/dim3 $component_names
        $h set_attribute $group/ds/dim3 physicalNature component
    }
    $h set_attribute $group/data label $result_name
    $h set_attribute $group/data unit $gidpost_unit
    $h set_attribute $group/data physicalNature freePhysicalNature
    $h set_attribute $group gidAnalysis $analysis_name ;#non Amelet standard

    return $num_results
}

proc Amelet::WritePostGroupsResults { h resultpath } {
    set num_results 0
    set num_graphs 0

    if { [info commands GiD_GraphSet] != "" } {
        foreach graphset_name [GiD_GraphSet list] {
            foreach graph_name [GiD_Graph list $graphset_name] {
                Amelet::WritePostGraphSet $h $resultpath $graphset_name $graph_name
                incr num_graphs
            }
        }
    } else {
        foreach graph_name [GiD_Graph list] {
            Amelet::WritePostGraph $h $resultpath $graph_name
            incr num_graphs
        }
    }

    foreach analysis_name [GiD_Info postprocess get all_analysis] {
        #set result_names [GiD_Result list $analysis_name] ;#this command doesn't exists
        set steps [GiD_Info postprocess get all_steps $analysis_name]
        foreach step $steps {
            lappend result_names {*}[GiD_Info postprocess get results_list CONTOUR_FILL $analysis_name $step]
        }
        set result_names [lsort -unique $result_names]
        foreach result_name $result_names {
            Amelet::WritePostResult $h $resultpath $analysis_name $result_name
            incr num_results
        }
    }

    return [list $num_results $num_graphs]
}

proc Amelet::WritePostResults { filename } {
    Amelet::LoadHdf5Package
    set h [Amelet::GetFreeHDF5Id]
    if { [file exists $filename] } {
        #WarnWin [_ "File %s already exists" $filename]
        #return [list 0 0 0]
        #open to append
        hdf5 $h $filename
    } else {
        hdf5 $h $filename
        $h set_attribute / FORMAT AMELETHDF AMELETHDF_FORMAT_VERSION 1.5.1
    }
    set group /floatingType
    if { ![$h is_group $group] } {
        $h create_group $group
    }
    lassign [Amelet::WritePostGroupsResults $h $group] num_results num_graphs
    $h close
    return [list $num_results $num_graphs]
}

proc Amelet::WritePost { filename } {
    variable _num_warnings
    set _num_warnings 0
    if { [file exists $filename] } {
        file delete $filename
    }
    set num_meshes [Amelet::WritePostMeshes $filename]
    lassign [Amelet::WritePostResults $filename] num_results num_graphs
    GiD_RaiseEvent GiD_Event_AfterSaveFile $filename AMELET_FORMAT 0
    return [list $num_meshes $num_results $num_graphs]
}

proc Amelet::WritePostW { } {
    set filename [MessageBoxGetFilename file write [_ "Write Amelet results file"] \
        {} {{{Amelet} {.h5 }} {{All files} {.*}}} ]
    if {$filename == ""} {
        return
    }
    set res [GidUtils::EvalAndEnterInBatch Amelet::WritePost $filename]
    lassign $res num_meshes num_results num_graphs
    GidUtils::SetWarnLine [_ "Wrote %d meshes and %d results %d graphs" $num_meshes $num_results $num_graphs]
}


#########################################################################################
######################################### OTHER #########################################
#########################################################################################

#trick to not ask a filename from HIRF framework
proc Amelet::GeAutomaticNameFromFramework { } {
    set curdir [pwd]
    if {  [file tail $curdir] == "workingDir" && [file tail [file dirname $curdir]] == "simulation" } {
        #set filename [file join $curdir output.h5]

        #new automatic name as Didier suggested:
        #a 'outputs' subfolder with multiple and non constant 'output.h5' name
        file mkdir [file join $curdir outputs]
        set basename [file tail [GiD_Info Project ModelName]]
        set filename ""
        set num 1
        while { 1 } {
            set filename ${basename}-${num}.h5
            set fullfilename [file join $curdir outputs $filename]
            if { ![file exists $fullfilename] } {
                break
            }
            incr num
        }
        #let the user set a filename, but output folder is forced
        set filename [tk_dialogEntryRAM .gid.ameletfilename [_ "HIRF Amelet"] [_ "Enter file name"] question any $filename]
        if { $filename == "--CANCEL--" } {
            set filename ""
        } else {
            if { [string tolower [file extension $filename]] != ".h5" } {
                #force to have .h5 extension
                set filename [file rootname $filename].h5
            }
            set fullfilename [file join $curdir outputs $filename]
            if { [file exists $fullfilename] } {
                WarnWin [_ "File %s already exists, select another file name" $filename]
                return [Amelet::GeAutomaticNameFromFramework]
            }
            set filename $fullfilename
        }
        return $filename
    } else {
        return ""
    }
}

proc Amelet::AddToMenuPRE { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import [_ "Amelet"]...] PRE] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import [_ "Amelet"]...] $position PRE Amelet::ReadPreW "" "" insertafter _
    #set position [GiDMenu::GetOptionIndex Files [list Export "GiD mesh"] PRE]
    set position [GiDMenu::GetOptionIndex Files [list Export [_ "%s mesh#C#menu" $::GidPriv(ProgName)]] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [_ "Export Amelet"]... $position PRE Amelet::WritePreW "" "" insertafter _
}

proc Amelet::AddToMenuPOST { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import [_ "Amelet"]...] POST] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Import->XYZ ascii points"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ ascii points"] POST]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import [_ "Amelet"]...] $position POST Amelet::ReadPostW "" "" insertafter _

    if { [GiDMenu::GetOptionIndex Files [list Export [_ "Amelet"]...] POST] != -1 } {
        return
    }
    #try to insert this menu after the word "Files->Export->Visible surfaces to PLY"
    set position [GiDMenu::GetOptionIndex Files [list Export "Visible surfaces to PLY"] POST]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Export [_ "Amelet"]...] $position POST Amelet::WritePostW "" "" insertafter _
}
proc Amelet::AddToMenu { } {
    Amelet::AddToMenuPRE
    Amelet::AddToMenuPOST
}

Amelet::Init

#invoke this menus changes
Amelet::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc Amelet::AddToMenu


#register the proc to be automatically called when dropping a file
GiD_RegisterExtensionProc ".h5" PRE Amelet::ReadPreSingleFile
GiD_RegisterExtensionProc ".h5" POST Amelet::ReadPostSingleFile
