namespace eval EMA3D {
    variable _merge 1
    variable compressed 0 ;#1 to allow elements spanning multiple cells
    #internal use variables:
    variable nodeexists ;#to know if a node has been created
    variable grid ;#to store grid variables

    variable info_plugin ;#information read from .xml plugin file

    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

#to add more options to the standar file browser
proc EMA3D::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkmerge -text [_ "Merge"] -variable EMA3D::_merge
    grid $f.chkmerge -sticky w -columnspan 2
    variable compressed
    ttk::checkbutton $f.chkCompressed -text [_ "Compressed"] -variable EMA3D::compressed
    grid columnconfigure $f 1 -weight 1
    grid $f.chkCompressed -sticky w -columnspan 2

    variable export_dir
    if { ![info exists export_dir] } {
        #trick to show the more options
        set export_dir open
    }
    return EMA3D::export_dir
}

proc EMA3D::ReadPreW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read EMA3D file"] \
        {} {{{EMA3D} {.emin }} {{All files} {.*}}} {} $multiple \
        [list [_ "Import options"] EMA3D::MoreImportOptions]]
    if { [llength $filenames] } {
        if { !$_merge } {
            GiD_Process Mescape Files New No escape ;#to avoid previous geometry
        }
        GidUtils::EvalAndEnterInBatch EMA3D::ReadPre $filenames
    }
}

proc EMA3D::CreateNode { i j k } {
    variable nodeexists
    incr i -1
    incr j -1
    incr k -1
    set num [EMA3D::GetNumFromIjk $i $j $k]
    if { ![info exists nodeexists($num)] } {
        set x [EMA3D::GetCoordinate x $i]
        set y [EMA3D::GetCoordinate y $j]
        set z [EMA3D::GetCoordinate z $k]
        GiD_Mesh create node $num "$x $y $z"
        set nodeexists($num) 1
    }
    return $num
}

proc EMA3D::ProcessBodies {i0 j0 k0 i1 j1 k1} {
    EMA3D::CreateBodies $i0 $j0 $k0 $i1 $j1 $k1
}

proc EMA3D::CreateSingleBody {i0 j0 k0 i1 j1 k1} {
    incr i1
    incr j1
    incr k1
    set n1 [EMA3D::CreateNode $i0 $j0 $k0]
    set n2 [EMA3D::CreateNode $i1 $j0 $k0]
    set n3 [EMA3D::CreateNode $i1 $j1 $k0]
    set n4 [EMA3D::CreateNode $i0 $j1 $k0]
    set n5 [EMA3D::CreateNode $i0 $j0 $k1]
    set n6 [EMA3D::CreateNode $i1 $j0 $k1]
    set n7 [EMA3D::CreateNode $i1 $j1 $k1]
    set n8 [EMA3D::CreateNode $i0 $j1 $k1]
    GiD_Mesh create element append hexahedra 8 "$n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8"
}

proc EMA3D::CreateMultipleBody {i0 j0 k0 i1 j1 k1} {
    incr i1
    incr j1
    incr k1
    for {set k $k0} {$k<$k1} {incr k} {
        set knext [expr $k+1]
        for {set j $j0} {$j<$j1} {incr j} {
            set jnext [expr $j+1]
            for {set i $i0} {$i<$i1} {incr i} {
                set inext [expr $i+1]
                set n1 [EMA3D::CreateNode $i $j $k]
                set n2 [EMA3D::CreateNode $inext $j $k]
                set n3 [EMA3D::CreateNode $inext $jnext $k]
                set n4 [EMA3D::CreateNode $i $jnext $k]
                set n5 [EMA3D::CreateNode $i $j $knext]
                set n6 [EMA3D::CreateNode $inext $j $knext]
                set n7 [EMA3D::CreateNode $inext $jnext $knext]
                set n8 [EMA3D::CreateNode $i $jnext $knext]
                GiD_Mesh create element append hexahedra 8 "$n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8"
            }
        }
    }
}

proc EMA3D::ProcessFaces {i0 j0 k0 i1 j1 k1 axe} {
    #foreach because has a kind of compressed notation representing multiple faces normal to axe
    if { $axe == "X" } {
        if { $i0 == $i1 } {
            set is $i0
        } else {
            set is [list $i0 $i1]
        }
        foreach i $is {
            EMA3D::CreateFaces $i $j0 $k0 $i $j1 $k1 $axe
        }
    } elseif { $axe == "Y" } {
        if { $j0 == $j1 } {
            set js $j0
        } else {
            set js [list $j0 $j1]
        }
        foreach j $js {
            EMA3D::CreateFaces $i0 $j $k0 $i1 $j $k1 $axe
        }
    } else {
        foreach k [list $k0 $k1] {
            EMA3D::CreateFaces $i0 $j0 $k $i1 $j1 $k $axe
        }
    }
}

proc EMA3D::CreateSingleFace {i0 j0 k0 i1 j1 k1 axe} {
    if { $axe == "X" } {
        incr j1
        incr k1
        set n1 [EMA3D::CreateNode $i0 $j0 $k0]
        set n2 [EMA3D::CreateNode $i0 $j1 $k0]
        set n3 [EMA3D::CreateNode $i0 $j1 $k1]
        set n4 [EMA3D::CreateNode $i0 $j0 $k1]
    } elseif { $axe == "Y" } {
        incr i1
        incr k1
        set n1 [EMA3D::CreateNode $i0 $j0 $k0]
        set n2 [EMA3D::CreateNode $i0 $j0 $k1]
        set n3 [EMA3D::CreateNode $i1 $j0 $k1]
        set n4 [EMA3D::CreateNode $i1 $j0 $k0]
    } else {
        incr i1
        incr j1
        set n1 [EMA3D::CreateNode $i0 $j0 $k0]
        set n2 [EMA3D::CreateNode $i1 $j0 $k0]
        set n3 [EMA3D::CreateNode $i1 $j1 $k0]
        set n4 [EMA3D::CreateNode $i0 $j1 $k0]
    }
    GiD_Mesh create element append quadrilateral 4 "$n1 $n2 $n3 $n4"
}

proc EMA3D::CreateMultipleFaces {i0 j0 k0 i1 j1 k1 axe} {
    if { $axe == "X" } {
        incr j1
        incr k1
        for {set j $j0} {$j<$j1} {incr j} {
            set jnext [expr $j+1]
            for {set k $k0} {$k<$k1} {incr k} {
                set knext [expr $k+1]
                set n1 [EMA3D::CreateNode $i0 $j $k]
                set n2 [EMA3D::CreateNode $i0 $jnext $k]
                set n3 [EMA3D::CreateNode $i0 $jnext $knext]
                set n4 [EMA3D::CreateNode $i0 $j $knext]
                GiD_Mesh create element append quadrilateral 4 "$n1 $n2 $n3 $n4"
            }
        }
    } elseif { $axe == "Y" } {
        incr i1
        incr k1
        for {set i $i0} {$i<$i1} {incr i} {
            set inext [expr $i+1]
            for {set k $k0} {$k<$k1} {incr k} {
                set knext [expr $k+1]
                set n1 [EMA3D::CreateNode $i $j0 $k]
                set n2 [EMA3D::CreateNode $i $j0 $knext]
                set n3 [EMA3D::CreateNode $inext $j0 $knext]
                set n4 [EMA3D::CreateNode $inext $j0 $k]
                GiD_Mesh create element append quadrilateral 4 "$n1 $n2 $n3 $n4"
            }
        }
    } else {
        incr i1
        incr j1
        for {set j $j0} {$j<$j1} {incr j} {
            set jnext [expr $j+1]
            for {set i $i0} {$i<$i1} {incr i} {
                set inext [expr $i+1]
                set n1 [EMA3D::CreateNode $i $j $k0]
                set n2 [EMA3D::CreateNode $inext $j $k0]
                set n3 [EMA3D::CreateNode $inext $jnext $k0]
                set n4 [EMA3D::CreateNode $i $jnext $k0]
                GiD_Mesh create element append quadrilateral 4 "$n1 $n2 $n3 $n4"
            }
        }
    }
}

proc EMA3D::ProcessEdges {i0 j0 k0 i1 j1 k1 axe} {
    #foreach because has a kind of compressed notation representing multiple edges parallel to axe
    if { $axe == "X" } {
        if { $j0 == $j1 } {
            set js $j0
        } else {
            set js [list $j0 $j1]
        }
        if { $k0 == $k1 } {
            set ks $k0
        } else {
            set ks [list $k0 $k1]
        }
        foreach j $js {
            foreach k $ks {
                EMA3D::CreateEdges $i0 $j $k $i1 $j $k $axe
            }
        }
    } elseif { $axe == "Y" } {
        if { $i0 == $i1 } {
            set is $i0
        } else {
            set is [list $i0 $i1]
        }
        if { $k0 == $k1 } {
            set ks $k0
        } else {
            set ks [list $k0 $k1]
        }
        foreach i $is {
            foreach k $ks {
                EMA3D::CreateEdges $i $j0 $k $i $j1 $k $axe
            }
        }
    } else {
        if { $i0 == $i1 } {
            set is $i0
        } else {
            set is [list $i0 $i1]
        }
        if { $j0 == $j1 } {
            set js $j0
        } else {
            set js [list $j0 $j1]
        }
        foreach i $is {
            foreach j $js {
                EMA3D::CreateEdges $i $j $k0 $i $j $k1 $axe
            }
        }
    }
}

proc EMA3D::CreateSingleEdge {i0 j0 k0 i1 j1 k1 axe} {
    if { $axe == "X" } {
        incr i1
    } elseif { $axe == "Y" } {
        incr j1
    } else {
        incr k1
    }
    set n1 [EMA3D::CreateNode $i0 $j0 $k0]
    set n2 [EMA3D::CreateNode $i1 $j1 $k1]
    GiD_Mesh create element append linear 2 "$n1 $n2"
}

proc EMA3D::CreateMultipleEdges {i0 j0 k0 i1 j1 k1 axe} {
    if { $axe == "X" } {
        incr i1
        for {set i $i0} {$i<$i1} {incr i} {
            set n1 [EMA3D::CreateNode $i $j0 $k0]
            set n2 [EMA3D::CreateNode [expr $i+1] $j0 $k0]
            GiD_Mesh create element append linear 2 "$n1 $n2"
        }
    } elseif { $axe == "Y" } {
        incr j1
        for {set j $j0} {$j<$j1} {incr j} {
            set n1 [EMA3D::CreateNode $i0 $j $k0]
            set n2 [EMA3D::CreateNode $i0 [expr $j+1] $k0]
            GiD_Mesh create element append linear 2 "$n1 $n2"
        }
    } else {
        incr k1
        for {set k $k0} {$k<$k1} {incr k} {
            set n1 [EMA3D::CreateNode $i0 $j0 $k]
            set n2 [EMA3D::CreateNode $i0 $j0 [expr $k+1]]
            GiD_Mesh create element append linear 2 "$n1 $n2"
        }
    }
}

#index starting from 0
proc EMA3D::GetCoordinate { axe index } {
    variable grid
    #$index must be between $grid($axe,lowerindex)-1 and $grid($axe,upperindex)
    if { $grid(uniform) } {
        return [expr {$grid($axe,origin)+$index*$grid($axe,increment)}]
    } else {
        return $grid($axe,position,$index)
    }
}

#num starting from 1
#i,j,k starting from 0
proc EMA3D::GetNumFromIjk { i j k } {
    variable grid
    return [expr {$k*$grid(n01)+$j*$grid(n0)+$i+1}]
}

#n0 num grid points in i direction
#n01=n0*n1 num grid points by ij plane
#num starting from 1
#i,j,k starting from 0
proc EMA3D::GetIjkFromNum { num } {
    variable grid
    incr num -1
    set k [expr $num/$grid(n01)]
    set rest [expr $num%$grid(n01)]
    set j [expr $rest/$grid(n0)]
    set i [expr $rest%$grid(n0)]
    return "$i $j $k"
}

proc EMA3D::SetLayer { layername } {
    variable layers
    variable currentlayer
    if { ![info exists layer($layername)] } {
        GiD_Layers create $layername
        set layers($layername) 1
        set currentlayer $layername
    } elseif { $currentlayer!=$layername } {
        GiD_Layers edit to_use $layer
        set currentlayer $layername
    }
}

#to drag and drop a file
proc EMA3D::ReadPreSingleFile { filename } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch EMA3D::ReadPre [list $filename]
}

proc EMA3D::ReadPre { filenames } {
    variable info_plugin
    set fail 0
    GidUtils::WaitState
    GidUtils::DisableGraphics
    if { [catch { set fail [EMA3D::ReadPre0 $filenames] } err options] } {
        set fail 1
    }
    GidUtils::EnableGraphics
    GidUtils::EndWaitState
    if { $fail } {
        return -options $options $err
    } else {
        GiD_RaiseEvent GiD_Event_AfterChangeMesh [GiD_Info Mesh NumNodes] [GiD_Info Mesh NumElements]
        GidUtils::UpdateWindow LAYER
        MeshView 0
        #GiD_Redraw
        GiD_Process 'Zoom Frame
        GidUtils::SetWarnLine [_ "%s file read" $info_plugin(Name)]
    }
    return $fail
}

proc EMA3D::ReadPre0 { filenames } {
    variable compressed
    variable nodeexists
    variable grid
    variable layers
    variable currentlayer

    variable percent_done
    variable stop
    variable _show_advance_bar

    set fail 0
    set ::EMA3D::stop 0
    set ::EMA3D::percent_done 0
    if { 0 } {
        #by now use the existent advancebar of each file
        set percent_max [llength $filenames]
        if { $_show_advance_bar } {
            GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::EMA3D::percent_done $percent_max {set ::EMA3D::stop 1}
        }
    }
    set create_layer_filename 0
    if { 0 } {
        #by now use the layers of the file
        if { [llength $filenames]>1 } {
            set create_layer_filename 1
        }
    }
    foreach filename $filenames {
        if { $::EMA3D::stop } {
            break
        }
        if { $create_layer_filename } {
            set layer [file rootname [file tail $filename]]
            if { ![GiD_Layers exists $layer] } {
                GiD_Layers create $layer
            }
            GiD_Layers edit to_use $layer
        }


        if { ![file exists $filename] } {
            W [_ "file '%s' not exists" $filename]
            continue
        }
        set fp [open $filename r]
        if { $fp == "" } {
            W [_ "Cannot open file '%s'" $filename]
            continue
        }

        if { $compressed } {
            interp alias "" EMA3D::CreateBodies "" EMA3D::CreateSingleBody
            interp alias "" EMA3D::CreateFaces "" EMA3D::CreateSingleFace
            interp alias "" EMA3D::CreateEdges "" EMA3D::CreateSingleEdge
        } else {
            interp alias "" EMA3D::CreateBodies "" EMA3D::CreateMultipleBodies
            interp alias "" EMA3D::CreateFaces "" EMA3D::CreateMultipleFaces
            interp alias "" EMA3D::CreateEdges "" EMA3D::CreateMultipleEdges
        }

        array unset nodeexists
        array unset grid

        array unset layers
        foreach layername [GiD_Info layers] {
            set layers($layername) 1
        }
        set currentlayer [GiD_Info Project LayerToUse]

        set last_comment ""
        set must_read 1

        while { ![eof $fp] } {
            if { $must_read } {
                gets $fp line
                set line [string trim $line]
            } else {
                set must_read 1
            }
            set firstchar [string index $line 0]
            if { $firstchar == "*" } {
                #comment
                set last_comment [string trim [string range $line 1 end]]
            } elseif { $firstchar == "!" } {
                #command
                if { $line == "!TIMESTEPS" } {
                    gets $fp line
                    lassign $line timestep nsteps
                } elseif { $line == "!SUMMARY" } {
                    #nothing
                } elseif { $line == "!NEWSPACESTEPS" } {
                    gets $fp line
                    if { $line == "!!XCONS" } {
                        set grid(uniform) 1
                        set axe x
                        gets $fp line
                        lassign $line grid($axe,origin)
                        gets $fp line
                        lassign $line grid($axe,lowerindex) grid($axe,upperindex)
                        gets $fp line
                        lassign $line grid($axe,increment)
                        gets $fp line
                        if { $line != "!!YCONS" } {
                            break
                        }
                        set axe y
                        gets $fp line
                        lassign $line grid($axe,origin)
                        gets $fp line
                        lassign $line grid($axe,lowerindex) grid($axe,upperindex)
                        gets $fp line
                        lassign $line grid($axe,increment)
                        gets $fp line
                        if { $line != "!!ZCONS" } {
                            break
                        }
                        set axe z
                        gets $fp line
                        lassign $line grid($axe,origin)
                        gets $fp line
                        lassign $line grid($axe,lowerindex) grid($axe,upperindex)
                        gets $fp line
                        lassign $line grid($axe,increment)

                        set grid(n0) [expr $grid(x,upperindex)-$grid(x,lowerindex)+1]
                        set grid(n01) [expr {$grid(n0)*($grid(y,upperindex)-$grid(y,lowerindex)+1)}]
                    } elseif { $line == "!!XVARI" } {
                        set grid(uniform) 0
                        set axe x
                        gets $fp line
                        lassign $line grid($axe,origin)
                        gets $fp line
                        lassign $line grid($axe,lowerindex) grid($axe,upperindex)
                        set i 0
                        while { ![eof $fp] } {
                            gets $fp line
                            if { [scan $line "%f" x] == 1 } {
                                set grid($axe,increment,$i) $x
                                incr i
                            } else {
                                break
                            }
                        }
                        if { $line == "!!YVARI" } {
                            set axe y
                            gets $fp line
                            lassign $line grid($axe,origin)
                            gets $fp line
                            lassign $line grid($axe,lowerindex) grid($axe,upperindex)
                            set i 0
                            while { ![eof $fp] } {
                                gets $fp line
                                if { [scan $line "%f" x] == 1 } {
                                    set grid($axe,increment,$i) $x
                                    incr i
                                } else {
                                    break
                                }
                            }
                        }
                        if { $line == "!!ZVARI" } {
                            set axe z
                            gets $fp line
                            lassign $line grid($axe,origin)
                            gets $fp line
                            lassign $line grid($axe,lowerindex) grid($axe,upperindex)
                            set i 0
                            while { ![eof $fp] } {
                                gets $fp line
                                if { [scan $line "%f" x] == 1 } {
                                    set grid($axe,increment,$i) $x
                                    incr i
                                } else {
                                    break
                                }
                            }
                        }
                        set grid(n0) [expr $grid(x,upperindex)-$grid(x,lowerindex)+1]
                        set grid(n01) [expr {$grid(n0)*($grid(y,upperindex)-$grid(y,lowerindex)+1)}]
                        foreach axe "x y z" {
                            set grid($axe,location,0) $grid($axe,origin)
                            set location $grid($axe,location,0)
                            set i 0
                            while { [info exists grid($axe,increment,$i)] } {
                                set location [expr {$location+$grid($axe,increment,$i)}]
                                incr i
                                set grid($axe,location,$i) $location
                            }
                        }
                    } else {
                        break
                    }
                } elseif { $line == "!BACKGROUND" } {
                    #background physical properties
                    gets $fp line
                    set firstchar [string index $line 0]
                    if { $line == "!!LINEAR" } {
                        gets $fp line
                        lassign $line bg_conductivity bg_permittivity bg_permeability bg_magnetic_conductivity
                    } elseif { $line == "!!NONLIN" } {
                        #...
                    } else {

                    }
                } elseif { $line == "!BOUNDARY CONDITION" } {
                    gets $fp line ;#PML ALL
                    gets $fp line ;#8.0    2.0    0.0010000000475
                } elseif { $line == "!PLANE WAVE SOURCE" } {
                    gets $fp line ;#dg700m.dat
                    gets $fp line ;#locked
                    gets $fp line ;#3   3   3
                    gets $fp line ;#954   301   1080
                    gets $fp line ;#  1.5708     0.7854     1.5708     2.3562
                } elseif { $line == "!SCREEN PROBE" } {
                    gets $fp line ;#tstart  tstop  tstep
                    gets $fp line ;#type1  i1  j1  k1
                    #how many lines n??
                    #gets $fp line ;#typen  in jn  kn
                } elseif { $line == "!NEW PROBE" } {
                    gets $fp line ;#!!NUMER
                    gets $fp line ;#!!!TIME
                    gets $fp line ;#0.0   1.0000e-004   2.5000e-011
                    gets $fp line ;#HX  286   23   542
                    gets $fp line ;#HY  286   23   542
                    gets $fp line ;#HZ  286   23   542
                } elseif { $line == "!BULK CURRENT PROBE" } {
                    gets $fp line ;#!!ELECT
                    gets $fp line ;#0.0   1.0000e-004   2.5000e-011
                    gets $fp line ;#143   90   553   X   10000
                    gets $fp line ;#143   94   557
                } elseif { $line == "!ISOTROPIC LINE" } {
                    EMA3D::SetLayer $last_comment
                    gets $fp line
                    set firstchar [string index $line 0]
                    if { $firstchar == "*" || $line==""} {
                        #comment
                    } elseif { $line == "!!METAL" || $line == "!!NONME" || $line == "!!MGMET"} {
                        set material $line
                        while { ![eof $fp] } {
                            if { $must_read } {
                                gets $fp line
                            } else {
                                set must_read 1
                            }
                            if { $line == "!!!2PNT" } {
                                if { $material == "!!NONME" } {
                                    gets $fp line
                                    lassign $line sigma epsilon mu sigmam
                                }
                                while { ![eof $fp] } {
                                    gets $fp line
                                    set line [string trim $line]
                                    set firstchar [string index $line 0]
                                    if { $firstchar == "*" } continue
                                    if { $firstchar == "!" || $line==""} {
                                        set must_read 0
                                        break
                                    }
                                    lassign $line i0 j0 k0 axe
                                    gets $fp line
                                    lassign $line i1 j1 k1
                                    EMA3D::ProcessEdges $i0 $j0 $k0 $i1 $j1 $k1 $axe
                                }
                            } elseif { $line == "!!!1PNT" } {
                                if { $material == "!!NONME" } {
                                    gets $fp line
                                    lassign $line sigma epsilon mu sigmam
                                }
                                while { ![eof $fp] } {
                                    gets $fp line
                                    set line [string trim $line]
                                    set firstchar [string index $line 0]
                                    if { $firstchar == "*" } continue
                                    if { $firstchar == "!" || $line==""} {
                                        set must_read 0
                                        break
                                    }
                                    lassign $line i0 j0 k0 axe
                                    EMA3D::CreateSingleEdge $i0 $j0 $k0 $i0 $j0 $k0 $axe
                                }
                            } else {
                                set must_read 0
                                break
                            }
                        }
                    } else {
                    }
                } elseif { $line == "!ISOTROPIC SURFACE" } {
                    EMA3D::SetLayer $last_comment
                    gets $fp line
                    set firstchar [string index $line 0]
                    if { $firstchar == "*" || $line==""} {
                        #comment
                    } elseif { $line == "!!METAL" || $line == "!!NONME" || $line == "!!MGMET" || $line == "!!COMPO"} {
                        set material $line
                        while { ![eof $fp] } {
                            if { $must_read } {
                                gets $fp line
                            } else {
                                set must_read 1
                            }
                            if { $line == "!!!2PNT" } {
                                if { $material == "!!NONME" || $material == "!!COMPO" } {
                                    gets $fp line
                                    lassign $line sigma epsilon mu sigmam
                                }
                                while { ![eof $fp] } {
                                    gets $fp line
                                    set line [string trim $line]
                                    set firstchar [string index $line 0]
                                    if { $firstchar == "*" } continue
                                    if { $firstchar == "!" || $line== "" } {
                                        set must_read 0
                                        break
                                    }
                                    lassign $line i0 j0 k0 axe
                                    gets $fp line
                                    lassign $line i1 j1 k1
                                    EMA3D::ProcessFaces $i0 $j0 $k0 $i1 $j1 $k1 $axe
                                }
                            } elseif { $line == "!!!1PNT" } {
                                if { $material == "!!NONME" || $material == "!!COMPO" } {
                                    gets $fp line
                                    lassign $line sigma epsilon mu sigmam
                                }
                                while { ![eof $fp] } {
                                    gets $fp line
                                    set line [string trim $line]
                                    set firstchar [string index $line 0]
                                    if { $firstchar == "*" } continue
                                    if { $firstchar == "!" || $line== "" } {
                                        set must_read 0
                                        break
                                    }
                                    lassign $line i0 j0 k0 axe
                                    EMA3D::CreateSingleFace $i0 $j0 $k0 $i0 $j0 $k0 $axe
                                }
                            } else {
                                set must_read 0
                                break
                            }
                        }
                    } else {
                    }

                } elseif { $line == "!ISOTROPIC BODY" } {
                    EMA3D::SetLayer $last_comment
                    gets $fp line
                    set firstchar [string index $line 0]
                    if { $firstchar == "*" || $line==""} {
                        #comment
                    } elseif { $line == "!!METAL" || $line == "!!NONME" || $line == "!!MGMET" } {
                        set material $line
                        while { ![eof $fp] } {
                            if { $must_read } {
                                gets $fp line
                            } else {
                                set must_read 1
                            }
                            if { $line == "!!!2PNT" } {
                                if { $material == "!!NONME" } {
                                    gets $fp line
                                    lassign $line sigma epsilon mu sigmam
                                }
                                while { ![eof $fp] } {
                                    gets $fp line
                                    set line [string trim $line]
                                    set firstchar [string index $line 0]
                                    if { $firstchar == "*" } continue
                                    if { $firstchar == "!" || $line== "" } {
                                        set must_read 0
                                        break
                                    }
                                    lassign $line i0 j0 k0
                                    gets $fp line
                                    lassign $line i1 j1 k1
                                    EMA3D::ProcessBodies $i0 $j0 $k0 $i1 $j1 $k1
                                }
                            } elseif { $line == "!!!1PNT" } {
                                if { $material == "!!NONME" } {
                                    gets $fp line
                                    lassign $line sigma epsilon mu sigmam
                                }
                                while { ![eof $fp] } {
                                    gets $fp line
                                    set line [string trim $line]
                                    set firstchar [string index $line 0]
                                    if { $firstchar == "*" } continue
                                    if { $firstchar == "!" || $line== "" } {
                                        set must_read 0
                                        break
                                    }
                                    lassign $line i0 j0 k0
                                    EMA3D::CreateSingleBody $i0 $j0 $k0 $i0 $j0 $k0
                                }
                            } else {
                                set must_read 0
                                break
                            }
                        }
                    } else {
                    }

                } elseif { $line == "!NEW THIN WIRE" } {
                    EMA3D::SetLayer $last_comment
                    gets $fp line
                    lassign $line rad res ind tl tr
                    if { $tl == "Parallel" } {
                        gets $fp line
                        lassign $line rtl itl ctl idl
                    }
                    if { $tr == "Parallel" } {
                        gets $fp line
                        lassign $line rtr itr ctr idr
                    }
                    gets $fp line
                    lassign $line enl enr
                    while { ![eof $fp] } {
                        gets $fp line
                        if { $line == "" } {
                            break
                        }
                        lassign $line i0 j0 k0 axe mi ndi
                        if { [expr {abs($mi)}] > 0.0 } {
                            gets $fp line
                            lassign $line srctypei srcfilei
                        }
                        EMA3D::CreateSingleEdge $i0 $j0 $k0 $i0 $j0 $k0 $axe
                    }
                } elseif { $line == "!END" } {
                    break
                } else {
                    WarnWinText "'$line' not implemented"
                }
            }
        }
        close $fp

        set fail 0
        if { !$fail } {
            GiD_Cartesian set dimension 3
            GiD_Cartesian set corner [list $grid(x,origin) $grid(y,origin) $grid(z,origin)]
            foreach axe { x y z } {
                set n($axe) [expr $grid($axe,upperindex)-$grid($axe,lowerindex)+1]
            }
            GiD_Cartesian set ngridpoints [list $n(x) $n(y) $n(z)]
            if { $grid(uniform) } {
                foreach axe { x y z } {
                    set bs($axe) [expr {$grid($axe,increment)*($n($axe)-1)}]
                }
            } else {
                foreach axe { x y z } {
                    set v($axe) {}
                    for {set i 0} {$i<$n($axe)} {incr i} {
                        lappend v($axe) $grid($axe,location,$i)
                    }
                }
                GiD_Cartesian set coordinates [list $v(x) $v(y) $v(z)]
                foreach axe { x y z } {
                    set bs($axe) [expr {[lindex $v($axe) $n($axe)-1]-[lindex $v($axe) 0]}]
                }
            }
            GiD_Cartesian set boxsize [list $bs(x) $bs(y) $bs(z)]
        }
        GiD_RaiseEvent GiD_Event_AfterOpenFile $filename EMA3D_FORMAT 0
        if { 0 } {
            incr ::EMA3D::percent_done
        }
    }
    if { 0 } {
        #by now use the existent advancebar of each file
        if { $_show_advance_bar } {
            GidUtils::DeleteAdvanceBar
        }
    }
    return $fail
}

proc EMA3D::AddToMenu { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "EMA3D..."] PRE] != -1 } {
        return
    }

    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import "EMA3D..."] $position PRE EMA3D::ReadPreW "" "" insertafter _
}


#invoke this menus changes
EMA3D::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc EMA3D::AddToMenu


#register the proc to be automatically called when dropping a file
GiD_RegisterExtensionProc ".emin" PRE EMA3D::ReadPreSingleFile

