namespace eval UNV {
    variable _merge 1
    variable debug 0
    variable percent_done
    variable stop
    variable _show_advance_bar 1

    variable info_plugin ;#information read from .xml plugin file

    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        WarnWinText [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        WarnWinText [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

#trick to convert FORTRAN-like real numbers like 1.0D+00 to 1.0e+00, valid for %e
proc UNV::DScan {string format args} {
    return [uplevel 1 [list scan [string map {D e} $string] $format {*}$args]]
}

#to add more options to the standar file browser
proc UNV::MoreImportOptions { f } {
    variable _merge
    ttk::checkbutton $f.chkmerge -text [_ "Merge"] -variable UNV::_merge
    grid $f.chkmerge -sticky w -columnspan 2
    grid columnconfigure $f 1 -weight 1

    variable import_dir
    if { ![info exists import_dir] } {
        #trick to show the more options
        set import_dir open
    }
    return UNV::import_dir
}

proc UNV::ReadPreW { } {
    variable _merge
    set multiple 1
    set filenames [MessageBoxGetFilename file read [_ "Read UNV file"] \
        {} {{{Ideas} {.unv }} {{Ideas} {.uff}} {{All files} {.*}}} {} $multiple \
        [list [_ "Import options"] UNV::MoreImportOptions]]
    if { [llength $filenames] } {
        if { !$_merge } {
            GiD_Process Mescape Files New No escape ;#to avoid previous geometry
        }
        GidUtils::EvalAndEnterInBatch UNV::ReadPre $filenames
    }
}

proc UNV::OpenResultFile { filename } {
    variable fp_res
    if { [info exists fp_res] } {
        UNV::CloseResultFile
    }
    set fp_res [open $filename w]
    if { $fp_res != "" } {
        puts $fp_res "GiD Post Results File 1.0"
    } else {
        unset fp_res
    }
}

proc UNV::CloseResultFile { } {
    variable fp_res
    if { [info exists fp_res] } {
        close $fp_res
        unset fp_res
    }
}

proc UNV::PutResult { name analysis step info } {
    variable fp_res
    if { ![info exists fp_res] } {
        set modelname [GiD_Info Project ModelName]
        if { $modelname != "UNNAMED" } {
            set filename [file join $modelname.gid [file tail $modelname].post.res]
        } else {
            set filename [file join $::env(TMP) kk.post.res]
        }
        OpenResultFile $filename
    }

    if { [info exists fp_res] } {
        puts $fp_res "Result \"$name\" \"$analysis\" $step Vector OnNodes"
        puts $fp_res "Values"
        foreach {nodenum values} $info {
            puts $fp_res "$nodenum $values"
        }
        puts $fp_res "End Values"
    }
}

#to drag and drop a file
proc UNV::ReadPreSingleFile { filename } {
    variable _merge
    if { !$_merge } {
        GiD_Process Mescape Files New No escape ;#to avoid previous geometry
    }
    GidUtils::EvalAndEnterInBatch UNV::ReadPre [list $filename]
}

#in GiD 14.1.4d moved to GidUtils::LineCount
proc UNV::LineCount { filename {eofchar "\n"} } {
    set i 0
    set fid [open $filename]
    # Use a 512K buffer.
    fconfigure $fid -buffersize 524288 -translation binary
    while {![eof $fid]} {
        incr i [expr {[llength [split [read $fid 524288] $eofchar]]-1}]
    }
    close $fid
    return $i
}

proc UNV::ReadPre { filenames } {
    variable info_plugin
    set fail 0
    GidUtils::WaitState
    GidUtils::DisableGraphics
    if { [catch { set fail [UNV::ReadPre0 $filenames] } err options] } {
        set fail 1
    }
    GidUtils::EnableGraphics
    GidUtils::EndWaitState
    if { $fail } {
        return -options $options $err
    } else {
        GiD_RaiseEvent GiD_Event_AfterChangeMesh [GiD_Info Mesh NumNodes] [GiD_Info Mesh NumElements]        
        MeshView 0
        #GiD_Redraw
        GiD_Process 'Zoom Frame escape
        GidUtils::UpdateWindow LAYER
        GidUtils::UpdateWindow GROUPS
        GidUtils::SetWarnLine [_ "%s file read" $info_plugin(Name)]
    }
    return $fail
}

# 164
proc UNV::ReadKey_UNITS { fp } {
    variable i_line
    set fail 0
    #units
    gets $fp line
    incr i_line
    set units_code [string trim [string range $line 0 9]]
    set units_description [string trim [string range $line 10 29]]
    set temperature_mode [string trim [string range $line 30 39]]
    #factors to convert to SI (to convert mus divide by the factor)
    gets $fp line
    incr i_line
    if { [UNV::DScan $line {%e%e%e} factor_length factor_force factor_temperature] != 3 } {
        set fail 1
    } else {
        gets $fp line
        incr i_line
        if { [UNV::DScan $line {%e} temperature_offset] != 1 } {
            set fail 1
        }
    }        
    set unit_length  ""
    if { $units_code == 1 } {
        #SI: Meter (newton)
        set unit_length m
    } elseif { $units_code == 2 } {
        #BG: Foot (pound f)
        set unit_length  "ft"
    } elseif { $units_code == 3 } {
        #MG: Meter (kilogram f)
        set unit_length m
    } elseif { $units_code == 4 } {
        #BA: Foot (poundal)
        set unit_length  "ft"
    } elseif { $units_code == 5 } {
        #MM: mm (milli newton)
        set unit_length mm
    } elseif { $units_code == 6 } {
        #CM: cm (centi newton)
        set unit_length cm
    } elseif { $units_code == 7 } {
        #IN: Inch (pound f)
        set unit_length  "in"
    } elseif { $units_code == 8 } {
        #GM: mm (kilogram f)
        set unit_length  mm
    } elseif { $units_code == 9 } {
        #US: USER_DEFINED
        set unit_length  ""
    } else {
        #unexpected
    }    
    #-1
    gets $fp line
    incr i_line    
    return [list $fail $unit_length]
}


# 151
proc UNV::ReadKey_HEADER { fp } {
    variable i_line
    set fail 0
    #header
    gets $fp line
    incr i_line
    set model_file_name $line
    gets $fp line
    incr i_line
    set model_file_description $line    
    gets $fp line
    incr i_line
    set program_creator_db $line
    gets $fp line ;#FORMAT(10A1,10A1,3I10) -> date time version subversion file_type
    incr i_line
    gets $fp line ;# FORMAT(10A1,10A1) -> date time (last saved)
    incr i_line
    gets $fp line
    incr i_line
    set program_creator_file $line
    gets $fp line ;#FORMAT(10A1,10A1,4I5) ->date time version host_id test_id release_counter
    incr i_line
    gets $fp line ;#-1
    incr i_line
    return [list $fail]
}

# 2400
proc UNV::ReadKey_MODEL_HEADER { fp } {
    variable i_line
    set fail 0
    #header
    gets $fp line
    incr i_line    
    #Model Header
    # FORMAT(I12,2I6,I12) -> model_uid entity_type entity_subtype version_number
    gets $fp line ;#entity_name
    incr i_line
    gets $fp line ;#part_number
    incr i_line
    gets $fp line ;#status_mask
    incr i_line
    gets $fp line ;#FORMAT(10A2,3I12) -> Date/time ...
    incr i_line
    gets $fp line ;#FORMAT(I12) -> optimization_switch_bitmask ...                
    incr i_line
    gets $fp line ;#-1
    incr i_line
    return [list $fail]
}
        
# 2411
proc UNV::ReadKey_NODE { fp } {
    variable i_line
    set fail 0
    set node_ids_list [list]
    set node_coordinates_list [list]
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }
        if { [scan $line {%i%i%i%i} node_id export_coordinate_sytem displacement_coordinate_syteme node_color] != 4 } {
            set fail 1
            break
        } else {
            gets $fp line
            incr i_line
            if { [UNV::DScan $line {%e%e%e} x y z] != 3 } {
                set fail 1
                break
            }
        }        
        lappend node_ids_list $node_id
        lappend node_coordinates_list $x $y $z
    }
    set gid_node_ids [objarray new intarray -values $node_ids_list]
    set gid_node_coordinates [objarray new floatarray -values $node_coordinates_list]
    return [list $fail $gid_node_ids $gid_node_coordinates]
}

# 2412 
proc UNV::ReadKey_ELEMENT { fp } {
    variable i_line
    set fail 0
    set elements_multiple_types [list]
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }
        if { [scan $line {%i%i%i%i%i%i} element_id element_unv_type physical_table material_table color gid_element_num_nodes] != 6 } {
            set fail 1
        }
        if { $element_unv_type == 11 } {
            # 11  Rod     
            gets $fp line
            incr i_line
            if { [scan $line {%i%i%i} beam_orientation_node_number beam_fore_end_cross_section_number beam_aft_end_cross_section_number] != 3 } {
                set fail 1
            }
        } elseif { $element_unv_type == 22 } {
            # 22  Tapered beam       
            gets $fp line
            incr i_line
            if { [scan $line {%i%i%i} v1 v2 v3] != 3 } {
                set fail 1
            }
        }
        set element_nodes [list]
        while { !$fail && [llength $element_nodes] < $gid_element_num_nodes } {
            gets $fp line
            incr i_line
            set n_scan [scan $line {%i%i%i%i%i%i%i%i} n(1) n(2) n(3) n(4) n(5) n(6) n(7) n(8)]
            if { $n_scan <= 0 } {
                set fail 1
            } else {
                for {set i_scan 1} {$i_scan<=$n_scan} {incr i_scan} {                    
                    lappend element_nodes $n($i_scan)
                }
            }
        }
        if { $fail } {
            break
        }
        if { $element_unv_type == 161 } {
            # Lumped Mass
            set gid_element_type point
            #$gid_element_num_nodes == 1
            set element_connectivities $element_nodes
        } elseif { $element_unv_type == 11 || $element_unv_type == 21 || $element_unv_type == 31 || $element_unv_type == 121 } {
            # 11  Rod
            # 21  Linear beam
            # 31  Straight pipe
            # 121 Rigid Bar
            set gid_element_type linear
            #$gid_element_num_nodes == 2
            set element_connectivities $element_nodes
        } elseif { $element_unv_type == 22 || $element_unv_type == 23 || $element_unv_type == 24 || $element_unv_type == 32 } {
            # 22  Tapered beam
            # 23  Curved beam
            # 24  Parabolic beam
            # 32  Curved pipe
            set gid_element_type linear
            #$gid_element_num_nodes == 3
            lassign $element_nodes n1 n3 n2
            set element_connectivities [list $n1 $n2 $n3]
        } elseif { $element_unv_type == 41 || $element_unv_type == 51 || $element_unv_type == 61 || $element_unv_type == 74 || $element_unv_type == 81 || $element_unv_type == 91 } {
            # 41  Plane Stress Linear Triangle
            # 51  Plane Strain Linear Triangle
            # 61  Plate Linear Triangle
            # 74  Membrane Linear Triangle
            # 81  Axisymetric Solid Linear Triangle
            # 91  Thin Shell Linear Triangle
            set gid_element_type triangle
            #$gid_element_num_nodes == 3
            set element_connectivities $element_nodes
        } elseif { $element_unv_type == 42 || $element_unv_type == 52 || $element_unv_type == 62 || $element_unv_type == 72 || $element_unv_type == 82 || $element_unv_type == 92 } {
            # 42  Plane Stress Parabolic Triangle
            # 52  Plane Strain Parabolic Triangle
            # 62  Plate Parabolic Triangle
            # 72  Membrane Parabolic Triangle
            # 82  Axisymetric Solid Parabolic Triangle
            # 92  Thin Shell Parabolic Triangle
            set gid_element_type triangle
            #$gid_element_num_nodes == 6
            lassign $element_nodes n1 n4 n2 n5 n3 n6
            set element_connectivities [list $n1 $n2 $n3 $n4 $n5 $n6]
        } elseif { $element_unv_type == 44 || $element_unv_type == 54 || $element_unv_type == 64 || $element_unv_type == 71 || $element_unv_type == 84 || $element_unv_type == 94 } {
            # 44  Plane Stress Linear Quadrilateral
            # 54  Plane Strain Linear Quadrilateral
            # 64  Plate Linear Quadrilateral
            # 71  Membrane Linear Quadrilateral
            # 84  Axisymetric Solid Linear Quadrilateral
            # 94  Thin Shell Linear Quadrilateral
            set gid_element_type quadrilateral
            #$gid_element_num_nodes == 4
            set element_connectivities $element_nodes
        } elseif { $element_unv_type == 45 || $element_unv_type == 55 || $element_unv_type == 65 || $element_unv_type == 75 || $element_unv_type == 85 || $element_unv_type == 95 } {
            # 45  Plane Stress Parabolic Quadrilateral
            # 55  Plane Strain Parabolic Quadrilateral
            # 65  Plate Parabolic Quadrilateral
            # 75  Membrane Parabolic Quadrilateral
            # 85  Axisymetric Solid Parabolic Quadrilateral
            # 95  Thin Shell Parabolic Quadrilateral
            set gid_element_type quadrilateral
            #$gid_element_num_nodes == 8
            lassign $element_nodes n1 n5 n2 n6 n3 n7 n4 n8
            set element_connectivities [list $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8]
        } elseif { $element_unv_type == 111 } {
            # 111 Solid Linear Tetrahedron
            set gid_element_type tetrahedra
            #$gid_element_num_nodes == 4
            # this order is ok for case Ihor_Mirzov_Baffle_3D.unv but bad for case 111.unv and Niclas_Stenberg_Empty_set.unv
            set element_connectivities $element_nodes
            #lassign $element_nodes n1 n3 n2 n4
            #set element_connectivities [list $n1 $n2 $n3 $n4]
        } elseif { $element_unv_type == 101 || $element_unv_type == 112 } {
            # 101 Thick Shell Linear Wedge
            # 112 Solid Linear Wedge            
            set gid_element_type prism
            #$gid_element_num_nodes == 6
            set element_connectivities $element_nodes
            #lassign $element_nodes n1 n3 n2 n4 n6 n5
            #set element_connectivities [list $n1 $n2 $n3 $n4 $n5 $n6]
        } elseif { $element_unv_type == 102 || $element_unv_type == 113 } {
            # 102 Thick Shell Parabolic Wedge
            # 113 Solid Parabolic Wedge            
            set gid_element_type prism
            #$gid_element_num_nodes == 15
            #lassign $element_nodes n1 n9 n3 n8 n2 n7 n10 n12 n11 n4 n15 n6 n14 n5 n13       
            lassign $element_nodes n1 n7 n2 n8 n3 n9 n10 n11 n12 n4 n13 n5 n14 n6 n15                        
            set element_connectivities [list $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 $n9 $n10 $n11 $n12 $n13 $n14 $n15]
        } elseif { $element_unv_type == 104 || $element_unv_type == 115 } {
            # 104 Thick Shell Linear Brick
            # 115 Solid Linear Brick
            set gid_element_type hexahedra
            #$gid_element_num_nodes == 8
            set element_connectivities $element_nodes
            #lassign $element_nodes n1 n4 n3 n2 n5 n8 n7 n6
            #set element_connectivities [list $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8]            
        } elseif { $element_unv_type == 105 || $element_unv_type == 116 } {
            # 105 Thick Shell Parabolic Brick
            # 116 Solid Parabolic Brick
            set gid_element_type hexahedra
            #$gid_element_num_nodes == 20
            lassign $element_nodes n1 n9 n2 n10 n3 n11 n4 n12 n13 n14 n15 n16 n5 n17 n6 n18 n7 n19 n8 n20
            set element_connectivities [list $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 $n9 $n10 $n11 $n12 $n13 $n14 $n15 $n16 $n17 $n18 $n19 $n20]
        } elseif { $element_unv_type == 118 } {
            # 118 Solid Parabolic Tetrahedron
            set gid_element_type tetrahedra
            #$gid_element_num_nodes == 10
            lassign $element_nodes n1 n5 n2 n6 n3 n7 n8 n9 n10 n4
            set element_connectivities [list $n1 $n2 $n3 $n4 $n5 $n6 $n7 $n8 $n9 $n10]
        } else {
            set unimplemented_element_unv_types($element_unv_type) 1
            continue
        }
        set key [list $gid_element_type $gid_element_num_nodes]
        lappend gid_element_ids_list($key) $element_id
        lappend gid_element_connectivities_list($key) {*}$element_connectivities
    }
    foreach key [lsort -dictionary [array names gid_element_connectivities_list]] {
        lassign $key gid_element_type gid_element_num_nodes
        set gid_element_ids [objarray new intarray -values $gid_element_ids_list($key)]
        set gid_element_connectivities [objarray new intarray -values $gid_element_connectivities_list($key)]
        lappend elements_multiple_types [list $gid_element_type $gid_element_num_nodes $gid_element_ids $gid_element_connectivities]
    }    
    return [list $fail $elements_multiple_types]
}


# 2420
proc UNV::ReadKey_COORDINATE_SYSTEMS { fp } {
    variable i_line
    set fail 0
    gets $fp line
    incr i_line
    if { [scan $line {%i} part_uid] != 1 } {
        set fail 1
    } else {
        #part_name
        gets $fp line
        incr i_line
        gets $fp line
        incr i_line
        if { [scan $line {%i%i%i} coordinate_system_label coordinate_system_type coordinate_system_color] != 3 } {
            set fail 1
        } else {
            #coordinate_system_type 0=Cartesian, 1=Cylindrical, 2=Spherical
            #coordinate_system_name
            gets $fp line
            incr i_line
            #Transformation Matrix
            set matrix [list]
            foreach i {1 2 3 4} {
                gets $fp line
                incr i_line
                lassign $line x y z
                lappend matrix [list $x $y $z]
            }
        }
    }
    #-1
    gets $fp line
    incr i_line
    return [list $fail]
}

# 2470
proc UNV::ReadKey_PHYSICAL_PROPERTIES { fp } {
    variable i_line
    set fail 0
    set elements_multiple_types [list]
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }                        
        if { [scan $line {%i%i%i%i} table_number table_descriptor num_properties v4] < 3 } {
            set fail 1
            break
        }
        gets $fp line ;#property table name
        incr i_line
        for { set i_property 0 } { $i_property<$num_properties}  {incr i_property} {
            gets $fp line ;#FORMAT(3I10)
            incr i_line
            if { [scan $line {%i%i%i} property_index property_type number_values] != 3 } {
                set fail 1
            }
            if { $property_type == 1 } {
                set max_items_record 8
            } elseif { $property_type == 2 } {
                set max_items_record 5
            } elseif { $property_type == 3 } {
                set max_items_record 80
            } else {
                set fail 1
                set max_items_record 1
            }
            set n_parts [expr int(ceil(double($number_values)/$max_items_record))]
            for {set i_part 0} {$i_part<$n_parts} {incr i_part} {
                gets $fp line ;#values with format and amount depending on property_type number_values
                incr i_line
            }
        }
    }
    return [list $fail]
}

# 2428 Permanent Groups
# 2429 Permanent Groups
# 2435 Permanent Groups
# 2467 Permanent Groups
# 2477 
proc UNV::ReadKey_GROUPS_ELEMENTS { fp } {
    variable i_line
    set fail 0 
    set groups_and_items [list]           
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }    
        if { [scan $line {%i%i%i%i%i%i%i%i} group_num v2 v3 v4 v5 v6 v7 num_items] != 8 } {
            set fail 1
        } 
        gets $fp line
        incr i_line
        set group_name [string trim $line]
        set group_items_list [list]
        set entity_type_code -1
        if { $num_items == 0 } {
            #there is an empty line in this case (e.g. Niclas_Stenberg_Empty_set.unv)
            gets $fp line
            incr i_line
        } else {
            while { !$fail && [llength $group_items_list] < $num_items } {
                gets $fp line
                incr i_line
                set n_scan [scan $line {%i%i%i%i%i%i%i%i} entity_type_code n(2) n(3) n(4) entity_type_code2 n(6) n(7) n(8)]
                #n(3) n(7) node_leaf_id
                #n(4) n(8) entity_component_id
                if { $n_scan == 4 } {
                    lappend group_items_list $n(2)
                } elseif { $n_scan ==8 } {
                    lappend group_items_list $n(2) $n(6)
                } else {
                    set fail 1
                }
            }
        }
        set over unexpected
        if { $entity_type_code == 7 } {
            set over nodes
        } elseif { $entity_type_code == 8 } {
            set over elements
        } else {
            set over $entity_type_code
        }
        lappend groups_and_items [list $group_name $over [objarray new intarray -values $group_items_list]]
    }
    return [list $fail $groups_and_items]
}

# 2438 
proc UNV::ReadKey_ASSOCIATED_ELEMENT_DATA { fp } {
    variable i_line
    set fail 0
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }
    }
    return [list $fail]
}


proc UNV::ReadKey_UNIMPLEMENTED { fp } {
    variable i_line
    set fail 0
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }
    }
    return [list $fail]
}

###############
# OBSOLETE KEYS
###############

# 15
proc UNV::ReadKey_OBSOLETE_NODE { fp } {
    variable i_line
    set fail 0
    set node_ids_list [list]
    set node_coordinates_list [list]
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }
        if { [scan $line {%i%i%i%i%e%e%e} node_id definition_coordinate_sytem displacement_coordinate_syteme node_color x y z] != 7 } {        
            set fail 1
            break
        }       
        lappend node_ids_list $node_id
        lappend node_coordinates_list $x $y $z
    }
    set gid_node_ids [objarray new intarray -values $node_ids_list]
    set gid_node_coordinates [objarray new floatarray -values $node_coordinates_list]
    return [list $fail $gid_node_ids $gid_node_coordinates]
}

# 55 
proc UNV::ReadKey_OBSOLETE_DATA_AT_NODES { fp } {
    variable i_line
    set fail 0
    set model_type analysis_type ""
    set data_characteristic ""
    set specific_data_type ""
    set data_type ""
    set ndv ""
    set load_case_number ""
    set mode_number ""
    set re_eigenvalue ""
    set im_eigenvalue ""
    set re_modal_a ""
    set im_modal_a ""
    set re_modal_b ""
    set im_modal_b ""
    set info ""
    set record 1
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }
        switch $record {
            1 - 2 - 3 - 4 - 5 {
                #headers, ignore them a this moment
            }
            6 {
                if { [scan $line {%i%i%i%i%i%i} model_type analysis_type data_characteristic specific_data_type data_type ndv] != 6 } {
                    set fail 1
                }
            }
            7 {
                if { $analysis_type == 3} {
                    #Complex Eigenvalue
                    if { [scan $line {%i%i%i%i} dummy2 dummy6 load_case_number mode_number] != 4 } {
                        set fail 1
                    }                    
                } else {
                    
                }
            }
            8 {
                if { [UNV::DScan $line {%e%e%e%e%e%e} re_eigenvalue im_eigenvalue re_modal_a im_modal_a re_modal_b im_modal_b] != 6 } {
                    set fail 1
                }
            }
            9 {
                if { [scan $line {%i} node_number] != 1 } {
                    set fail 1
                }
                lappend info $node_number
            }
            default {
                if { [UNV::DScan $line {%e%e%e%e%e%e} re_x im_x re_y im_y re_z im_z] != 6 } {
                    set fail 1
                }
                lappend info [list $re_x $re_y $re_z]
                incr record -2 ;#to set next record as 9
            }
        }
        if { $fail } {
            break
        }
        incr record        
    }
    if { $mode_number != "" } {
        UNV::PutResult  "Structural" "Complex Eigenvalue $mode_number" 1 $info
    }
    return [list $fail]
}


# 82 
proc UNV::ReadKey_OBSOLETE_TRACELINES { fp offset_nodes } {
    variable i_line
    set fail 0
    set info ""
    set record 1
    while { ![eof $fp] } {
        gets $fp line
        incr i_line
        if { [string range $line 0 5] == "    -1"} {
            break
        }
        switch $record {
            1 {
                if { [scan $line {%i%i%i} line_num number_of_nodes line_color] != 3 } {
                    set fail 1
                }
            }
            2 {
                set name $line
            }
            default {
                set info [concat $info $line]
            }
        }
        incr record
        if { $fail } {
            break
        }
    }        
    set n1 0
    foreach n $info {
        if { $n>0 } {
            if { $n1>0 } {
                set n2 $n
                incr n2 $offset_nodes
                GiD_Mesh create element append linear 2 [list $n1 $n2]
                set n1 $n2
            } else {
                set n1 $n
                incr n1 $offset_nodes
            }
        } else {
            set n1 0
        }
    }
    return [list $fail]
}
    
proc UNV::ReadPreMesh { filename } {
    variable debug
    variable i_line          
    set fail 0
    set fp [open $filename r]
    if { $fp == "" } {
        W [_ "Cannot open file '%s'" $filename]
        return 1
    }
    #offset_nodes to be added to the number if a previous model exists
    set offset_nodes [GiD_Info Mesh MaxNumNodes]
    set offset_elements [GiD_Info Mesh MaxNumElements]    
    
    set gid_node_ids [objarray new intarray -values {}]
    set gid_node_coordinates [objarray new floatarray -values {}]

    #set num_lines [UNV::LineCount $filename]
    set i_line 0
    while { ![eof $fp] } {
        if { $fail } {
            break
        }
        gets $fp line
        incr i_line                
        if { [string range $line 0 5] != "    -1"} {                        
            # e.g. ignore first line "GLOBAL/LOCAL COORDINATE SYSTEMS" of symetrie_cyclique_hexa.unv
            continue
        }
        gets $fp line
        incr i_line        
        set type [string trim $line]
        if { ![string is integer $type] } {
            set fail 1
            break
        }                                                             
        switch $type {
            15 {
                lassign [UNV::ReadKey_OBSOLETE_NODE $fp] fail gid_node_ids gid_node_coordinates
            }
            55 {
                
                lassign [UNV::ReadKey_OBSOLETE_DATA_AT_NODES $fp] fail
            }
            82 {
                lassign [UNV::ReadKey_OBSOLETE_TRACELINES $fp $offset_nodes] fail
            }
            151 {
                lassign [UNV::ReadKey_HEADER $fp] fail                
            }
            164 {                
                lassign [UNV::ReadKey_UNITS $fp] fail unit_length
                #units are only available in some problemtypes
                if { 0 && [GiD_Units get model_unit_length] != "" && $unit_length != "" } {
                    GiD_Units edit model_unit_length $unit_length
                }
            }
            2400 {
                lassign [UNV::ReadKey_MODEL_HEADER $fp] fail                             
            }
            2411 {
                lassign [UNV::ReadKey_NODE $fp] fail gid_node_ids gid_node_coordinates
            }
            2412 {
                lassign [UNV::ReadKey_ELEMENT $fp] fail elements_multiple_types                
                foreach item $elements_multiple_types {
                    lassign $item gid_element_type gid_element_num_nodes gid_element_ids gid_element_connectivities
                    set name ""
                    GiD_MeshPre create $name $gid_element_type $gid_element_num_nodes $gid_node_ids $gid_node_coordinates $gid_element_ids $gid_element_connectivities
                }
            }
            2420 {
                lassign [UNV::ReadKey_COORDINATE_SYSTEMS $fp] fail
            }
            2428 -
            2429 -
            2435 -
            2467 -
            2477 {
                lassign [UNV::ReadKey_GROUPS_ELEMENTS $fp] fail groups_and_items
                foreach item $groups_and_items {
                    lassign $item group_name over group_items
                    if { ![GiD_Groups exists $group_name] } {
                        GiD_Groups create $group_name
                    }
                    if { [objarray length $group_items] > 0 } {
                        if { $over == "nodes" } {
                            if { $offset_nodes } {
                                objarray incr $group_items $offset_nodes
                            }
                            GiD_EntitiesGroups assign $group_name $over $group_items
                        } elseif { $over == "elements" } {
                            if { $offset_elements } {
                                objarray incr $group_items $offset_elements
                            }
                            GiD_EntitiesGroups assign $group_name $over $group_items
                        } else {                       
                            set unimplemented_group_entity_type_code($over) 1                    
                        }
                    }
                }
            }
            2438 {
                lassign [UNV::ReadKey_ASSOCIATED_ELEMENT_DATA $fp] fail
            }
            2470 {
                lassign [UNV::ReadKey_PHYSICAL_PROPERTIES $fp] fail
            }
            default {
                lassign [UNV::ReadKey_UNIMPLEMENTED $fp] fail
                set unimplemented_unv_types($type) 1
            }
        }
    }
    close $fp
    

    UNV::CloseResultFile

    if { $debug } {
        if { [llength [array names unimplemented_unv_types]] } {
            WarnWinText "Unimplemented unv types: [lsort -integer [array names unimplemented_unv_types]]"
        }
        if { [llength [array names unimplemented_element_unv_types]] } {
            WarnWinText "Unimplemented unv element type: [lsort -integer [array names unimplemented_element_unv_types]]"
        }
        if { [llength [array names unimplemented_group_entity_type_code]] } {
            WarnWinText "Unimplemented unv group entity type: [lsort -integer [array names unimplemented_group_entity_type_code]]"
        }
    }
    return $fail
}

proc UNV::ReadPre0 { filenames } {
    variable debug
    variable percent_done
    variable stop
    variable _show_advance_bar


    set fail 0
    set ::UNV::stop 0
    set ::UNV::percent_done 0
    set percent_max [llength $filenames]
    if { $_show_advance_bar } {
        GidUtils::CreateAdvanceBar [_ "Loading file"] [_ "Process"] ::UNV::percent_done $percent_max {set ::UNV::stop 1}
    }
    set create_layer_filename 0
    if { [llength $filenames]>1 } {
        set create_layer_filename 1
    }
    foreach filename $filenames {
        if { $::UNV::stop } {
            break
        }
        if { [file exists $filename] } {
            if { $create_layer_filename } {
                set layer [file rootname [file tail $filename]]
                if { ![GiD_Layers exists $layer] } {
                    GiD_Layers create $layer
                }
                GiD_Layers edit to_use $layer
            }
            set res ""
            if { [catch { set local_fail [UNV::ReadPreMesh $filename] } err options] } {
                #set fail 1
                if { $_show_advance_bar } {
                    GidUtils::DeleteAdvanceBar
                }
                return -options $options $err
            } else {
                if { $local_fail } {
                    set fail 1
                }
                GiD_RaiseEvent GiD_Event_AfterOpenFile $filename UNV_FORMAT $local_fail
            }
        }
        incr ::Abaqus::percent_done
    }
    if { $_show_advance_bar } {
        GidUtils::DeleteAdvanceBar
    }
    return $fail
}

proc UNV::AddToMenu { } {
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "UNV..."] PRE] != -1 } {
        return
    }

    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
        set position end
    }
    GiDMenu::InsertOption Files [list Import "UNV..."] $position PRE UNV::ReadPreW "" "" insertafter _
}


#invoke this menus changes
UNV::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc UNV::AddToMenu


#register the proc to be automatically called when dropping a file
GiD_RegisterExtensionProc ".unv .uff" PRE UNV::ReadPreSingleFile
