##+#################################################################################
# based on slippy.tcl - demo of the slippy tcllib package by Keith Vetter, July 2010
# Slippy is a partial front end to the OpenStreet Map project
# http://wiki.openstreetmap.org/wiki/Slippy_map_tilenames#Tiles
##+#################################################################################

namespace eval gid_slippy {
    variable S
    variable V
    variable D
    
    variable background_image_3d
    set background_image_3d(create) 0
    
    #to know the name of the main canvas without send by parameter to all
    variable CANVAS_MAIN
    variable WINDOW_MAIN
    #to get z or image from server and import in GiD
    variable my_gid_map_data_z
    variable fetcher_topography
    variable my_gid_map_data_streets
    variable fetcher_background_image

    variable info_plugin ;#information read from .xml plugin file
    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
    if { [info exists info_plugin(Version)] } {
        set other_plugin_version $info_plugin(Version)
    }
    if { [info procs ReadProblemtypeXml] != "" } {
        array set info_plugin [ReadProblemtypeXml [file join $info_plugin(path) [file tail $info_plugin(path)].xml] InfoPlugin {Name Version MinimumGiDVersion}]
    } else {
        #to allow load it in old GiD 11
        array set info_plugin [list Name gid_slippy Version 1.0 MinimumGiDVersion 11.0.9]
    }
    if { [info exists other_plugin_version] && $other_plugin_version != $info_plugin(Version)} {
        W [_ "There are two versions of %s: the %s and %s inside the plugins folder. One must be deleted" $info_plugin(Name) $other_plugin_version $info_plugin(Version)]
    }
    if { [GidUtils::VersionCmp $info_plugin(MinimumGiDVersion)] < 0 } {
        W [_ "This plugin requires GiD %s or later" $info_plugin(MinimumGiDVersion)]
    }
}

proc gid_slippy::DoSearch { } {
    variable S
    variable V
    $V(nominatim) search $S(search)
}

proc gid_slippy::SearchDone { result } {
    variable S
    set best_item [lindex $result 0]
    array set data $best_item
    if { [info exists data(lat)] } {
        set S(lat) $data(lat)
        set S(lon) $data(lon)
        gid_slippy::ReMapIt
    }
}

proc gid_slippy::Init { } {
    variable S
    variable V
    variable D
    package require Tk
    package require Img
    #package require img::tiff
    package require http
    package require tls
    package require map::slippy
    package require map::slippy::fetcher
    package require map::slippy::cache
    package require map::geocode::nominatim
    package require gid_map
    package require gid_cross_platform

    #set V(map,schemes) {OpenStreetMap IGN Amazon Arcgis CycleMaps }
    set V(map,schemes) {OpenStreetMap IGN}
    set V(cacheDir) [file join [GidUtils::GetTmp] slippy.cache]

    set S(lat) 48.37
    set S(lon) 11.25
    set S(zoom) 4
    set S(lastZoom) -1
    set S(scheme) [lindex $V(map,schemes) 0]
    set S(lastScheme) ""
    set S(loaded) 0
    set S(pending) 0
    set S(cached) 0
    set S(grid) 1
    #to create entities in GiD
    set S(create_background_image) 0
    set S(projection) UTM
    set S(action) mesh
    set S(increment) 1

    # Urls and valid zoom ranges for various map types
    #set V(url,Amazon) https://s3.amazonaws.com/elevation-tiles-prod/normal
    set V(url,Amazon) https://s3.amazonaws.com/elevation-tiles-prod/terrarium
    set V(zoom,Amazon) {0 15}
    set V(url,OpenStreetMap) https://tile.openstreetmap.org   
    #set V(zoom,OpenStreetMap) {0 18}    
    set V(zoom,OpenStreetMap) [list 0 [lindex $V(zoom,Amazon) 1]] ;#really allow until 18 but limite it to be the same as Amazon to get z    
    set V(url,IGN) https://www.ign.es/wmts/pnoa-ma
    #set V(zoom,IGN) {0 19}
    set V(zoom,IGN) [list 0 [lindex $V(zoom,Amazon) 1]] ;#really allow until 19 but limite it to be the same as Amazon to get z
    #with Arcgis seems that tiles are transposed and in other coordinates, must be studied...
    set V(url,Arcgis) https://server.arcgisonline.com/ArcGIS/rest/services/World_Imagery/MapServer/tile
    set V(zoom,Arcgis) {0 23}
    set V(url,CycleMaps) https://tile.opencyclemap.org/cycle
    set V(zoom,CycleMaps) {1 12}
    #set V(url,Stamen) https://maps.stamen.com/terrain/?service=WMTS&request=GetCapabilities&version=1.0.0#
    #set V(zoom,Stamen) {0 16}
    

    set V(nominatim) [::map::geocode::nominatim Searcher -callback gid_slippy::SearchDone]

    set D(upleft,center) {-1 -1}
    set D(upleft,shift) {22 11 11 00 12 01 21 10}
    set D(upleft,blank) {20 10 00 01 02}
    set D(up,center) {-1 0}
    set D(up,shift) {20 10 21 11 22 12 10 00 11 01 12 02}
    set D(up,blank) {00 01 02}
    set D(upright,center) {-1 1}
    set D(upright,shift) {20 11 11 02 10 01 21 12}
    set D(upright,blank) {00 01 02 12 22}
    set D(right,center) {0 1}
    set D(right,shift) {00 01 10 11 20 21 01 02 11 12 21 22}
    set D(right,blank) {02 12 22}
    set D(downright,center) {1 1}
    set D(downright,shift) {00 11 11 22 10 21 01 12 }
    set D(downright,blank) {02 12 22 21 20}
    set D(down,center) {1 0}
    set D(down,shift) {00 10 01 11 02 12 10 20 11 21 12 22}
    set D(down,blank) {20 21 22}
    set D(downleft,center) {1 -1}
    set D(downleft,shift) {02 11 11 20 01 10 12 21}
    set D(downleft,blank) {00 10 20 21 22}
    set D(left,center) {0 -1}
    set D(left,shift) {02 01 12 11 22 21 01 00 11 10 21 20}
    set D(left,blank) {00 10 20}

    image create bitmap ::bit::up -data {
        #define up_width 11
        #define up_height 11
        static char up_bits = {
            0x00, 0x00, 0x20, 0x00, 0x70, 0x00, 0xf8, 0x00, 0xfc, 0x01, 0xfe,
            0x03, 0x70, 0x00, 0x70, 0x00, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00
        }
    }
    image create bitmap ::bit::down -data {
        #define down_width 11
        #define down_height 11
        static char down_bits = {
            0x00, 0x00, 0x00, 0x00, 0x70, 0x00, 0x70, 0x00, 0x70, 0x00, 0xfe,
            0x03, 0xfc, 0x01, 0xf8, 0x00, 0x70, 0x00, 0x20, 0x00, 0x00, 0x00
        }
    }
    image create bitmap ::bit::left -data {
        #define left_width 11
        #define left_height 11
        static char left_bits = {
            0x00, 0x00, 0x20, 0x00, 0x30, 0x00, 0x38, 0x00, 0xfc, 0x01, 0xfe,
            0x01, 0xfc, 0x01, 0x38, 0x00, 0x30, 0x00, 0x20, 0x00, 0x00, 0x00
        }
    }
    image create bitmap ::bit::right -data {
        #define right_width 11
        #define right_height 11
        static char right_bits = {
            0x00, 0x00, 0x20, 0x00, 0x60, 0x00, 0xe0, 0x00, 0xfc, 0x01, 0xfc,
            0x03, 0xfc, 0x01, 0xe0, 0x00, 0x60, 0x00, 0x20, 0x00, 0x00, 0x00
        }
    }
    image create bitmap ::bit::upleft -data {
        #define upleft_width 11
        #define upleft_height 11
        static char upleft_bits = {
            0x00, 0x00, 0x7e, 0x00, 0x3e, 0x00, 0x3e, 0x00, 0x7e, 0x00, 0xfe,
            0x00, 0xf2, 0x01, 0xe0, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00
        }
    }
    image create bitmap ::bit::upright -data {
        #define upright_width 11
        #define upright_height 11
        static char upright_bits = {
            0x00, 0x00, 0xf0, 0x03, 0xe0, 0x03, 0xe0, 0x03, 0xf0, 0x03, 0xf8,
            0x03, 0x7c, 0x02, 0x38, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00
        }
    }
    image create bitmap ::bit::downleft -data {
        #define downleft_width 11
        #define downleft_height 11
        static char downleft_bits = {
            0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0xe0, 0x00, 0xf2, 0x01, 0xfe,
            0x00, 0x7e, 0x00, 0x3e, 0x00, 0x3e, 0x00, 0x7e, 0x00, 0x00, 0x00
        }
    }
    image create bitmap ::bit::downright -data {
        #define downright_width 11
        #define downright_height 11
        static char downright_bits = {
            0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x38, 0x00, 0x7c, 0x02, 0xf8,
            0x03, 0xf0, 0x03, 0xe0, 0x03, 0xe0, 0x03, 0xf0, 0x03, 0x00, 0x00
        }
    }
    image create bitmap ::bmp::plus -data {
        #define bullet_width 8
        #define bullet_height 8
        static char bullet_bits = {
            0x18, 0x18, 0x18, 0xff, 0xff, 0x18, 0x18, 0x18
        }
    }
    image create bitmap ::bmp::minus -data {
        #define bullet_width 8
        #define bullet_height 8
        static char bullet_bits = {
            0x00, 0x00, 0x00, 0xff, 0xff, 0x00, 0x00, 0x00
        }
    }
}

proc gid_slippy::DoDisplay {} {
    variable S
    variable CANVAS_MAIN
    variable WINDOW_MAIN
    set WINDOW_MAIN .gid.slippy
    set w $WINDOW_MAIN
    InitWindow $w [_ "Get map topography"] TopographyWindowGeom gid_slippy::DoDisplay
    #toplevel $w
    if { ![winfo exists $w] } return ;# windows disabled || usemorewindows == 0
    #wm title $w [_ "Get map topography"]
    

    ttk::frame $w.ctrl

    set c_to_scroll [CreateScrolledCanvas $w.center]
    set fcenter [tk::frame $c_to_scroll.fcenter]
    set sz [expr {3*256}]
    set CANVAS_MAIN [canvas $fcenter.c]
    $CANVAS_MAIN configure -highlightthickness 0 -bd 2 -relief solid -width $sz -height $sz -bg #bbeeff
    $CANVAS_MAIN yview moveto 0
    $CANVAS_MAIN xview moveto 0
    grid $CANVAS_MAIN -sticky nsew

    AddToScrolledCanvas $w.center $fcenter

    gid_slippy::DoCanvas

    ttk::frame $w.bottom -borderwidth 2 -relief ridge
    grid $w.ctrl -sticky new
    grid $w.center -sticky nsew
    grid $w.bottom -sticky sew


    ttk::label $w.ctrl.lsearch -text [_ "Find"]:
    ttk::entry $w.ctrl.search -textvariable gid_slippy::S(search)
    ttk::label $w.ctrl.llat -text [_ "Latitude"]:
    ttk::entry $w.ctrl.elat -textvariable gid_slippy::S(lat) -width 15
    ttk::label $w.ctrl.llon -text [_ "Longitude"]: -justify right
    ttk::entry $w.ctrl.elon -textvariable gid_slippy::S(lon) -width 15
    ttk::button $w.ctrl.go -text [_ "Go"] -command gid_slippy::ReMapIt

    ttk::checkbutton $w.ctrl.grid -text [_ "Show grid"] -variable gid_slippy::S(grid) -command gid_slippy::ToggleGrid
    ttk::checkbutton $w.ctrl.cache -text [_ "Use caching"] -variable gid_slippy::S(caching)


    ttk::label $w.bottom.msg -borderwidth 2 -relief solid -textvariable gid_slippy::S(msg) -anchor c
    ttk::frame $w.bottom.load -borderwidth 2 -relief solid
    ttk::label $w.bottom.load.loaded -text [_ "Loaded"]:
    ttk::label $w.bottom.load.vloaded -textvariable gid_slippy::S(loaded) -width 4
    ttk::frame $w.bottom.pend -borderwidth 2 -relief solid
    ttk::label $w.bottom.pend.pending -text [_ "Pending"]:
    ttk::label $w.bottom.pend.vpending -textvariable gid_slippy::S(pending) -width 4
    ttk::frame $w.bottom.cache -borderwidth 2 -relief solid
    ttk::label $w.bottom.cache.cached -text [_ "Cached"]:
    ttk::label $w.bottom.cache.vcached -textvariable gid_slippy::S(cached) -width 4

    #grid $w.bottom.load.loaded $w.bottom.load.vloaded -sticky w
    #grid $w.bottom.pend.pending $w.bottom.pend.vpending  -sticky w
    #grid $w.bottom.cache.cached $w.bottom.cache.vcached -sticky w
    #grid $w.bottom.msg $w.bottom.load $w.bottom.pend $w.bottom.cache -sticky news
    #grid columnconfig $w.bottom 0 -weight 1

    grid $w.ctrl.lsearch $w.ctrl.search $w.ctrl.llat $w.ctrl.elat $w.ctrl.llon $w.ctrl.elon $w.ctrl.go x $w.ctrl.grid $w.ctrl.cache -sticky w
    grid configure $w.ctrl.grid $w.ctrl.cache -sticky e
    grid configure $w.ctrl.search -sticky ew
    grid columnconfigure $w.ctrl 1 -weight 1

    ttk::frame $w.createoptions
    ttk::checkbutton $w.createoptions.create_background_image -text [_ "Create background image"] -variable gid_slippy::S(create_background_image)
    set labels [gid_map::get_available_projections]
    set values $labels
    TTKComboBox $w.createoptions.projection -textvariable gid_slippy::S(projection) -labels $labels -values $values -state readonly
    set labels [list [= "Create geometry"]  [= "Create mesh"] [= "Convert Arc/Info ASCII file"]]
    set values {geometry mesh convert_file}
    TTKComboBox $w.createoptions.action -textvariable gid_slippy::S(action) -labels $labels -values $values -state readonly
    ttk::label $w.createoptions.lIncrement -text [= "Increment"]
    ttk::entry $w.createoptions.eIncrement -textvariable gid_slippy::S(increment)
    grid $w.createoptions.create_background_image $w.createoptions.projection $w.createoptions.action $w.createoptions.lIncrement $w.createoptions.eIncrement -sticky w
    grid $w.createoptions -sticky ew
    ttk::frame $w.buts -style BottomFrame.TFrame
    ttk::button $w.buts.createmap -text [_ "Create map"] -command [list gid_slippy::DoCreateMap] -style BottomFrame.TButton
    ttk::button $w.buts.cancel -text [_ "Close"] -command [list destroy $w] -style BottomFrame.TButton
    grid $w.buts -sticky ew
    if { $::tcl_version >= 8.5 } { grid anchor $w.buts center }
    grid $w.buts.createmap $w.buts.cancel -pady 12 -padx 8
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 1 -weight 1

    bind $w <Alt-c> [list $w.buts.cancel invoke]
    bind $w <Escape> [list $w.buts.cancel invoke]
    bind $w.ctrl.search <Return> [list gid_slippy::DoSearch]

    set size_scrolls 25
    set max_width [expr $sz+$size_scrolls]
    set max_height [expr $sz+[winfo height $w.ctrl]+[winfo height $w.bottom]+[winfo height $w.createoptions]+$size_scrolls+85]
    wm maxsize $w $max_width $max_height
    wm geometry $w ${max_width}x${max_height}
    update
}

proc gid_slippy::DoCreateMap {} {
    variable S
    gid_map::set_current_projection $S(projection)
    GidUtils::EvalAndEnterInBatch gid_slippy::get_z_tile_by_longitude_latitude $S(zoom) $S(lon) $S(lat)
    if { $S(create_background_image) } {
        GidUtils::EvalAndEnterInBatch gid_slippy::get_streets_tile_by_longitude_latitude $S(zoom) $S(lon) $S(lat)
    }
    GidUtils::SetWarnLine [_ "Get map zoom=%s latitude=%s longitude=%s. Projection %s" $S(zoom) $S(lat) $S(lon) $S(projection)]
}

proc gid_slippy::DoCanvas {} {
    variable S
    variable V
    variable CANVAS_MAIN
    variable WINDOW_MAIN
    foreach d {upleft up upright right downright down downleft left} {
        ttk::button $CANVAS_MAIN.$d -image ::bit::$d -command [list gid_slippy::Shift $d]
    }
    foreach d {up right down left} {
        bind $WINDOW_MAIN <Key-[string totitle $d]> [list gid_slippy::Shift $d]
    }

    place $CANVAS_MAIN.upleft -x .1i -y .1i -anchor nw
    place $CANVAS_MAIN.up -relx .5 -y .1i -anchor n
    place $CANVAS_MAIN.upright -relx 1 -x -.1i -y .1i -anchor ne
    place $CANVAS_MAIN.right -relx 1 -x -.1i -rely .5 -anchor e
    place $CANVAS_MAIN.downright -relx 1 -x -.1i -rely 1 -y -.1i -anchor se
    place $CANVAS_MAIN.down -relx .5 -rely 1 -y -.1i -anchor s
    place $CANVAS_MAIN.downleft -x .1i -rely 1 -y -.1i -anchor sw
    place $CANVAS_MAIN.left -x .1i -rely .5 -anchor w

    ttk::frame $CANVAS_MAIN.zoom -relief solid -borderwidth 2
    ttk::button $CANVAS_MAIN.zoom.plus -image ::bmp::plus -command {gid_slippy::Zoom 1}
    ttk::label $CANVAS_MAIN.zoom.zoom -textvariable gid_slippy::S(zoom) -justify c -anchor c
    ttk::button $CANVAS_MAIN.zoom.minus -image ::bmp::minus -command {gid_slippy::Zoom -1}
    pack {*}[winfo child $CANVAS_MAIN.zoom] -side top -fill x
    place $CANVAS_MAIN.zoom -x .1i -y .5i -anchor nw
    if { 1 } {
        ttk::frame $CANVAS_MAIN.scheme -relief solid -borderwidth 2
        ttk::menubutton $CANVAS_MAIN.scheme.mb -menu $CANVAS_MAIN.scheme.mb.menu -textvariable gid_slippy::S(scheme)
        menu $CANVAS_MAIN.scheme.mb.menu -tearoff 0
        foreach v $V(map,schemes) {
            $CANVAS_MAIN.scheme.mb.menu add radiobutton -label $v -variable gid_slippy::S(scheme) -command gid_slippy::NewScheme
        }
        pack $CANVAS_MAIN.scheme.mb -fill both
        place $CANVAS_MAIN.scheme -relx 1 -x -.5i -y .1i -anchor ne
    }

    set sz [expr {3*256}]
    foreach ab {00 01 02 10 11 12 20 21 22} {
        set iname "::map::$ab"
        image create photo $iname -width 256 -height 256
    }

    set sz [expr {3*256}]
    $CANVAS_MAIN create line 256 0 256 $sz -dash 1 -tag grid
    $CANVAS_MAIN create line 512 0 512 $sz -dash 1 -tag grid
    $CANVAS_MAIN create line 0 256 $sz 256 -dash 1 -tag grid
    $CANVAS_MAIN create line 0 512 $sz 512 -dash 1 -tag grid
    $CANVAS_MAIN create rectangle 256 256 512 512 -outline red -tag grid


    $CANVAS_MAIN create image   0 0   -image ::map::00 -anchor nw
    $CANVAS_MAIN create image 256 0   -image ::map::01 -anchor nw
    $CANVAS_MAIN create image 512 0   -image ::map::02 -anchor nw
    $CANVAS_MAIN create image   0 256 -image ::map::10 -anchor nw
    $CANVAS_MAIN create image 256 256 -image ::map::11 -anchor nw
    $CANVAS_MAIN create image 512 256 -image ::map::12 -anchor nw
    $CANVAS_MAIN create image   0 512 -image ::map::20 -anchor nw
    $CANVAS_MAIN create image 256 512 -image ::map::21 -anchor nw
    $CANVAS_MAIN create image 512 512 -image ::map::22 -anchor nw

    $CANVAS_MAIN create text 128 128 -tag {txt ::map::00} -anchor c -font {Times 32 bold}
    $CANVAS_MAIN create text 384 128 -tag {txt ::map::01} -anchor c -font {Times 32 bold}
    $CANVAS_MAIN create text 640 128 -tag {txt ::map::02} -anchor c -font {Times 32 bold}
    $CANVAS_MAIN create text 128 384 -tag {txt ::map::10} -anchor c -font {Times 32 bold}
    $CANVAS_MAIN create text 384 384 -tag {txt ::map::11} -anchor c -font {Times 32 bold}
    $CANVAS_MAIN create text 640 384 -tag {txt ::map::12} -anchor c -font {Times 32 bold}
    $CANVAS_MAIN create text 128 640 -tag {txt ::map::20} -anchor c -font {Times 32 bold}
    $CANVAS_MAIN create text 384 640 -tag {txt ::map::21} -anchor c -font {Times 32 bold}
    $CANVAS_MAIN create text 640 640 -tag {txt ::map::22} -anchor c -font {Times 32 bold}
}

proc gid_slippy::RegisterFetchers {} {
    variable V
    variable S
    variable my_gid_map_data_z
    variable fetcher_topography
    variable fetcher_background_image
    variable my_gid_map_data_streets

    foreach urlScheme [lsort -dictionary [array names V url,*]] {
        lassign [split $urlScheme ,] . scheme
        set level_max [expr [lindex $V(zoom,$scheme) 1]+1]
        #set level_max 99 ;# We'll validate ourselves instead
        set V(fetcher,$scheme) [::map::slippy::fetcher $scheme $level_max $V(url,$scheme) .png ""]
        if {$V(cacheDir) ne ""} {
            set cacherName "c$scheme"
            catch {rename $cacherName {}}
            set cacheDir [file join $V(cacheDir) $scheme]
            file mkdir $cacheDir
            set V(cacher,$scheme) [::map::slippy::cache $cacherName $cacheDir $V(fetcher,$scheme)]
        }
    }
    # register the fetchers for z and streets image for GiD (yes, they are registered twice, before must be resistered without image variable)
    # now getting the image content as variable with a modified map::slippy::fetcher package   
    if { ![info exists fetcher_topography] } {
        set scheme Amazon        
        set level_max [expr [lindex $V(zoom,$scheme) 1]+1]
        set fetcher_topography [::map::slippy::fetcher topography $level_max $V(url,$scheme) .png ::gid_slippy::my_gid_map_data_z]
    }
    if { ![info exists fetcher_background_image] } {
        set scheme $S(scheme)
        set level_max [expr [lindex $V(zoom,$scheme) 1]+1]
        set fetcher_background_image [::map::slippy::fetcher streets $level_max $V(url,$scheme) .png ::gid_slippy::my_gid_map_data_streets]
    }
}

proc gid_slippy::GetCenterTile {} {
    variable S
    set geo [list $S(zoom) $S(lat) $S(lon)]
    set tile [::map::slippy geo 2tile $geo]
    gid_slippy::PrettyLat
    return $tile
}

proc gid_slippy::ReMapIt {} {
    variable S
    set S(lastZoom) -1;
    gid_slippy::MapIt
}

proc gid_slippy::MapIt {} {
    variable S
    variable CANVAS_MAIN
    variable WINDOW_MAIN

    if {$S(lastZoom) == $S(zoom) && $S(lastScheme) eq $S(scheme)} return
    if {! [IsLegal]} return

    set center [gid_slippy::GetCenterTile]
    lassign $center zoom crow ccol

    gid_slippy::EraseMaps

    set S(center) [list $crow $ccol]
    foreach {drow dcol} {-1 -1 -1 0 -1 1 0 -1 0 0 0 1 1 -1 1 0 1 1} {
        set row [expr {$crow + $drow}]
        set col [expr {$ccol + $dcol}]
        set iname "::map::[expr {$drow+1}][expr {$dcol+1}]"
        set n [catch {gid_slippy::FetchMap $zoom $row $col $iname} emsg]
        if {$n} {
            $CANVAS_MAIN itemconfigure $iname -text ""
            $iname put [$CANVAS_MAIN cget -bg] -to 0 0 256 256
            continue
        }
    }
    set S(lastZoom) $S(zoom)
    set S(lastScheme) $S(scheme)
    gid_slippy::ToggleGrid
    wm title $WINDOW_MAIN [_ "Get map topography %s" $S(msg)]
}

proc gid_slippy::Shift {dir} {
    variable S
    variable D
    variable CANVAS_MAIN

    if {! [gid_slippy::IsLegal]} return

    lassign $S(center) crow ccol
    lassign $D($dir,center) drow dcol
    incr crow $drow
    incr ccol $dcol
    set S(center) [list $crow $ccol]
    lassign [gid_slippy::GetCenterLatLon] S(lat) S(lon)

    foreach {dst src} $D($dir,shift) {
        ::map::$dst copy ::map::$src
    }
    foreach src $D($dir,blank) {
        ::map::$src put [$CANVAS_MAIN cget -bg] -to 0 0 256 256
    }
    foreach src $D($dir,blank) {
        lassign [split $src ""] a b
        set row [expr {$crow + $a - 1}]
        set col [expr {$ccol + $b - 1}]
        set iname "::map::$src"
        set n [catch {gid_slippy::FetchMap $S(zoom) $row $col $iname} emsg]
        if {$n} {
            $CANVAS_MAIN itemconfigure $iname -text ""
            $iname put [$CANVAS_MAIN cget -bg] -to 0 0 256 256
            continue
        }
    }
}

proc gid_slippy::FetchMap {zoom row col iname} {
    variable S
    variable V
    variable CANVAS_MAIN
    #$iname put yellow -to 0 0 256 256
    #$CANVAS_MAIN itemconfig $iname -text [_ "Loading"]...
    #update

    set tile [list $zoom $row $col]
    set ::tile $tile
    set ::cmd [list gid_slippy::FetchDone $iname]
    if {$S(caching) == 0 || $V(cacheDir) eq ""} {
        $V(fetcher,$S(scheme)) get $tile [list gid_slippy::FetchDone $iname]
    } else {
        if {[$V(cacher,$S(scheme)) exists $tile]} {
            incr S(cached)
        }
        $V(cacher,$S(scheme)) get $tile [list gid_slippy::FetchDone $iname]
    }
    incr S(pending)
}

proc gid_slippy::FetchDone {iname cmd tile {newImage ""}} {
    variable S
    variable CANVAS_MAIN
    incr S(loaded)
    incr S(pending) -1
    if {$cmd eq "set"} {
        if {$newImage in [image names]} {
            $iname copy $newImage
            image delete $newImage
        }
        $CANVAS_MAIN itemconfig $iname -text ""
    } else {
        $CANVAS_MAIN itemconfig $iname -text [_ "error"]
    }
}

proc gid_slippy::EraseMaps {} {
    variable CANVAS_MAIN
    foreach ab {00 01 02 10 11 12 20 21 22} {
        ::map::$ab put [$CANVAS_MAIN cget -bg] -to 0 0 256 256
    }
    $CANVAS_MAIN itemconfig txt -text ""
}

proc gid_slippy::IsLegal {} {
    variable S
    variable V
    lassign $V(zoom,$S(scheme)) lo hi
    if {$S(zoom) >= $lo && $S(zoom) <= $hi} { return 1 }
    W "$S(scheme) maps must have zoom between $lo and $hi"
    return 0
}

proc gid_slippy::GetCenterLatLon {} {
    variable S
    lassign $S(center) crow ccol
    set tile1 [list $S(zoom) $crow $ccol]
    set tile2 [list $S(zoom) [expr {$crow+1}] [expr {$ccol+1}]]
    set geo1 [::map::slippy tile 2geo $tile1]
    set geo2 [::map::slippy tile 2geo $tile2]
    lassign $geo1 . lat1 lon1
    lassign $geo2 . lat2 lon2
    set lat [expr {($lat1+$lat2)/2}]
    set lon [expr {($lon1+$lon2)/2}]
    return [list $lat $lon]
}

proc gid_slippy::ToggleGrid {} {
    variable S
    variable CANVAS_MAIN
    $CANVAS_MAIN [expr {$S(grid) ? "raise" : "lower"}] grid
}

proc gid_slippy::int2lat {lat} {
    set deg [expr {int($lat)}]
    set lat [expr {($lat - $deg)*60}]
    set min [expr {int($lat)}]
    set sec [format "%.2f" [expr {($lat - $min)*60}]]
    regsub {(.)\.?0*$} $sec {\1} sec
    return [list $deg $min $sec]
}
proc gid_slippy::PrettyLat {} {
    variable S
    lassign [int2lat [expr {abs($S(lat))}]] lat1 lat2 lat3
    lassign [int2lat [expr {abs($S(lon))}]] lon1 lon2 lon3
    set N [expr {$S(lat) >= 0 ? "N" : "S"}]
    set W [expr {$S(lon) >= 0 ? "E" : "W"}]
    set lat "$N $lat1\xB0 $lat2' $lat3\x22"
    set lon "$W $lon1\xB0 $lon2' $lon3\x22"
    set S(msg) "$lat   $lon"
}

proc gid_slippy::Zoom {dir} {
    variable S
    variable V
    set newZoom [expr {$S(zoom) + $dir}]
    lassign $V(zoom,$S(scheme)) lo hi
    if {$newZoom >= $lo && $newZoom <= $hi} {
        set S(zoom) $newZoom
        gid_slippy::MapIt
    }
}

proc gid_slippy::NewScheme {} {
    variable S
    variable V
    variable fetcher_background_image
    variable my_gid_map_data_streets
    lassign $V(zoom,$S(scheme)) lo hi
    if {$S(zoom) > $hi} { set S(zoom) $hi}
    if {$S(zoom) < $lo} { set S(zoom) $lo}
    set scheme $S(scheme)
    set level_max [expr [lindex $V(zoom,$scheme) 1]+1]
    if { [info exists fetcher_background_image] } {
        rename $fetcher_background_image {}
    }
    set fetcher_background_image [::map::slippy::fetcher streets $level_max $V(url,$scheme) .png ::gid_slippy::my_gid_map_data_streets]
    gid_slippy::MapIt
}

proc gid_slippy::Show { } {
    variable S
    variable V
    if { ![info exists S(lat)] } {
        gid_slippy::Init
    }
    set S(caching) 0
    if {$V(cacheDir) eq ""} {
        set S(caching) 0
    }
    # by now ignore error reopening window 
    catch {  gid_slippy::RegisterFetchers } msg
    gid_slippy::DoDisplay
    gid_slippy::ReMapIt
    return
}

###############################################################
# utilities to get tiles from z and street image public servers
###############################################################

#varname must be the name of a global variable
proc gid_slippy::write_variable_data_to_file { varname filename } {
    set fp [open $filename wb]
    puts -nonewline $fp [set $varname]
    close $fp    
}

proc gid_slippy::get_tile_corners_geodetical { tile } {
    set level [lindex $tile 0]
    lassign [::map::slippy tile 2geo $tile] z latitude_left_top longitude_left_top
    set tile_next [list $level [expr [lindex $tile 1]+1] [expr [lindex $tile 2]+1]]
    lassign [::map::slippy tile 2geo $tile_next] z latitude_right_bottom longitude_right_bottom
    return [list $longitude_left_top $latitude_left_top $longitude_right_bottom $latitude_right_bottom]
}


proc gid_slippy::write_georeference_to_file { tile filename } {
    lassign [gid_slippy::get_tile_corners_geodetical $tile] longitude_left_top latitude_left_top longitude_right_bottom latitude_right_bottom
    lassign [gid_map::project_with_current_projection $longitude_left_top $latitude_left_top] L T
    lassign [gid_map::project_with_current_projection $longitude_right_bottom $latitude_right_bottom] R B
    set Mxx [expr {double($R-$L)/256.0}]
    set Myx 0
    set Mxy 0
    set Myy [expr {double($B-$T)/256.0}]
    set Dx $L
    set Dy $T
    set fp [open $filename w]
    puts $fp $Mxx
    puts $fp $Myx
    puts $fp $Mxy
    puts $fp $Myy
    puts $fp $Dx
    puts $fp $Dy
    close $fp
}

#it doesn't require tile position because GDAL read the location from the associated georeference file if any
proc gid_slippy::gid_import_z_image { filename } {
    variable S
    #import it using the GDAL plugin
    set gdal_variables {show_advance_bar action increment}
    foreach gdal_variable $gdal_variables {
        set prev($gdal_variable) ""
        if { [info exists GDAL::$gdal_variable] } {
            set prev($gdal_variable) [set GDAL::$gdal_variable]
        }
    }
    set GDAL::show_advance_bar 0
    set GDAL::action $S(action)
    set GDAL::increment $S(increment)
    set raster_z [GDAL::ReadPreRaster $filename]
    foreach gdal_variable $gdal_variables {
        if { $prev($gdal_variable) != "" } {
            set GDAL::$gdal_variable $prev($gdal_variable)
        } else {
            unset GDAL::$gdal_variable
        }
    }
    return $raster_z
}

proc gid_slippy::get_z_tile_cmd { action tile {image ""} } {
    variable my_gid_map_data_z
    variable background_image_3d
    if { $image != "" } {
        #image delete $image ;#original package map::slippy::fetcher return a image, not its content in variable
        #set filename [file join {C:\gid project more\Graficos\GIS\tests_web_services} z_[join $tile _].tif]
        #set filename [file join {C:\gid project more\Graficos\GIS\tests_web_services} z_[join $tile _].png]
        set filename [gid_cross_platform::get_unused_tmp_filename gid_slippy .png]
        gid_slippy::write_variable_data_to_file ::gid_slippy::my_gid_map_data_z $filename
        unset ::gid_slippy::my_gid_map_data_z
        set georeference_filename ${filename}w
        gid_slippy::write_georeference_to_file $tile $georeference_filename
        set raster_z [gid_slippy::gid_import_z_image $filename]
        if { $gid_slippy::background_image_3d(create) } {
            set gid_slippy::background_image_3d(raster_z) $raster_z
        }
        file delete $filename
        file delete $georeference_filename
    } else {
        W "cannot get tile=$tile"
    }
}

proc gid_slippy::get_z_tile_by_zxy { zoom x y } {
    variable fetcher_topography
    $fetcher_topography get [list $zoom $x $y] gid_slippy::get_z_tile_cmd
}

proc gid_slippy::get_z_tile_by_longitude_latitude { zoom longitude latitude } {
    lassign [map::slippy geo 2tile [list $zoom $latitude $longitude]] zoom x y
    gid_slippy::get_z_tile_by_zxy $zoom $x $y
}

#tile used to georeference the image (GiD_BackgroundImage is not using the  associated georeference file if any, GiD_Process use it)
proc gid_slippy::gid_import_background_image { filename tile } {
    #it doesn't require position because it read the location from the associated georeference file if any
    #GiD_Process Mescape Utilities BackgroundImg RealSizeImg $filename escape
    GiD_BackgroundImage set filename $filename
    lassign [gid_slippy::get_tile_corners_geodetical $tile] longitude_left_top latitude_left_top longitude_right_bottom latitude_right_bottom
    lassign [gid_map::project_with_current_projection $longitude_left_top $latitude_left_top] L T
    lassign [gid_map::project_with_current_projection $longitude_right_bottom $latitude_right_bottom] R B
    GiD_BackgroundImage set location [list $L $B $R $B $L $T]
}

proc gid_slippy::get_streets_tile_cmd { action tile {image_data ""} } {
    variable my_gid_map_data_streets
    variable background_image_3d
    if { $image_data != "" } {
        if { $gid_slippy::background_image_3d(create) } {
            # test create terrain image as texture with z
            if { [info exists gid_slippy::background_image_3d(raster_z)] } {
                #this is bad, really both images can return in any async order, can use promises...
                set raster_z $gid_slippy::background_image_3d(raster_z)
                package require gid_draw_opengl
                set img [image create photo -data $::gid_slippy::my_gid_map_data_streets]
                set opengl_lists [gid_draw_opengl::create_opengl_lists_from_background_image_mesh_rectangle $raster_z $img]
                gid_draw_opengl::draw_lists_append $opengl_lists
            }
        } else {
            #image delete $image #original package map::slippy::fetcher return a image, not its content in variable
            #set filename [file join {C:\gid project more\Graficos\GIS\tests_web_services} streets_[join $tile _].png]
            set filename [gid_cross_platform::get_unused_tmp_filename gid_slpippy .png]
            gid_slippy::write_variable_data_to_file ::gid_slippy::my_gid_map_data_streets $filename
            unset ::gid_slippy::my_gid_map_data_streets
            #set georeference_filename [file rootname $filename].pngw
            #gid_slippy::write_georeference_to_file $tile $georeference_filename
            gid_slippy::gid_import_background_image $filename $tile
            file delete $filename
            #file delete $georeference_filename
        }
    } else {
        W "cannot get tile=$tile"
    }
}

proc gid_slippy::get_streets_tile_by_zxy { zoom x y } {
    variable fetcher_background_image
    $fetcher_background_image get [list $zoom $x $y] gid_slippy::get_streets_tile_cmd
}

proc gid_slippy::get_streets_tile_by_longitude_latitude { zoom longitude latitude } {
    lassign [map::slippy geo 2tile [list $zoom $latitude $longitude]] zoom x y
    gid_slippy::get_streets_tile_by_zxy $zoom $x $y
}


######################################################################


proc gid_slippy::AddToMenu { } {
    variable info_plugin
    if { [GidUtils::IsTkDisabled] } {
        return
    }
    if { [GiDMenu::GetOptionIndex Files [list Import "Topography..."] PRE] != -1 } {
        return
    }

    #try to insert this menu after the word "Files->Import->XYZ nodes"
    set position [GiDMenu::GetOptionIndex Files [list Import "XYZ nodes"] PRE]
    if { $position == -1 } {
        set position end
    }
    set image [file join $info_plugin(path) topography.png]
    GiDMenu::InsertOption Files [list Import "Topography..."] $position PRE gid_slippy::Show "" $image insertafter _
}


#invoke this menus changes
gid_slippy::AddToMenu
GiDMenu::UpdateMenus

#register the proc to be automatically called when re-creating all menus (e.g. when doing files new)
GiD_RegisterPluginAddedMenuProc gid_slippy::AddToMenu

#e.g. use:
# package require gid_slippy
# gid_slippy::Show
# gid_map::set_current_projection UTM ;#only valid for small regions!!
# gid_slippy::get_streets_tile_by_longitude_latitude 0 2.175 41.360 ;#level 0: full map -> set backgroung image from png
# gid_slippy::get_z_tile_by_longitude_latitude 8 2.175 41.360 ;#level 8 near Barcelona -> create mesh of quadrilaterals from png or geotiff
