# GiD plugin on develep to call automatically a proc when meshing to replace the line elements 
# of lines set to duplicate nodes and forced to be meshed with quadrilaterals of 0 size to represent contact elements
# this code is temporary, is only a test and must be deleted soon, the definitive code will be moved to GiD scripts or C++ code
#
#
# to duplicate and create contact elements must assign to the line to be duplicated and also forced to be meshed 
# Mesh->Mesh criteria->Duplicate->Lines
# Mesh->Mesh criteria->Mesh->Lines
# (is the line is not forced to be meshed won't generate the line elements that are replacec by this proc by quadrilateral flat contact elements)
#
#
#  Algorithm used:
# 1- obtener lista de ids de lineas marcadas para mallar duplicate
# 2- obtener lista de ids de elementos de las lineas anteriores, 
#     para eso recorro todos los elements de tipo linea y me quedo con los que provienen de alguna de esas lineas
# 3- obtengo la lista de los nodos (sin repetición) de esos elementos
# 4- para estos nodos primero relleno un array 'nodes_xyz' de su id por su posicion espacial "x y z" como key, para busqueda posterior
#    para todos los nodos si coinciden en "x y z" con alguno de los de las lineas los pongo en un array, en dicha key pongo una lista de nodos con estas coordenadas exactas
# 
# 5- para cada los elementos de esas líneas obtengo sus nodos y los considero como una cara
#    5.1- para el signo de la cara master busco entre toda la malla los elementos que tengan en alguna cara estos mismos nodos (con cualquier orientacion)
#    si hay uno y solo un elemento con esta cara, miro si tiene la misma orientacion u opuesta, para orientar bien el nuevo elemento de contacto
#    
#    5.2- si no he encontrado ningun elemento con esta cara, 
#    (quizá los nodos no coinciden como puntero pero si por posicion "x y z", por ser algun nodo copiado)
#    busco como medida desesperada de fuerza bruta entre todas las combinaciones cartesianas de caras probando todos los posibles nodos en las mismas x y z y buscando en toda la malla si hay un elemento con esta para candidata
#    si solo queda una cara candidata la escojo para mi futuro elemento de contacto, si hay varias tomo la primera y que sea lo que dios quiera, sino falla
# 
#    5.3- partiendo de esta primera cara master, busco los nodos slave en sus coordenadas 
#    (sin que el slave sea el propio master)
#    puede haber varios nodos slave candidatos, por ejemplo si en un punto convergen mas de dos 'lineas duplicate', hay que intentar escoger el apropiado
#    de nuevo pruebo todas las combinaciones cartesianas 
#      y busco en toda la malla si hay alguna cara de elemento que coincida con la probada
#      (si hay mas de un elemento con esta cara doy warnig de caso inesperado)   
#    5-4  si solo tengo una cara slave candidata construyo el nuevo contact element con los nodos de la cara master y la cara slave
#      segun el signo de 5.1 creo el elemento con una orientacion o la opuesta
     
namespace eval GidUtils {
}

proc GidUtils::LinesMeshCriteriaDuplicateAndToBeMeshed_CreateContactSurfaceElements { } {
    set lines_marked_duplicate [GiD_Geometry list -mesh_data meshing duplicate line]
    set num_duplicated_lines 0
    catch { set num_duplicated_lines [objarray length $lines_marked_duplicate] }
    # catch to avoid a problem loading ggcLib_64.dll (package gid_groups_conds of compassFEM problemtype that redefines our objarray command)
    if { $num_duplicated_lines} {
        #filter the line elements that are created from these lines
        set line_element_ids [list]
        objarray foreach element_id [GiD_Mesh list -element_type [list Line] element] {
            set line_id [lindex [GiD_Mesh get element $element_id geometry_source] 1]
            if { [objarray search -sorted $lines_marked_duplicate $line_id] != -1 } {
                lappend line_element_ids $element_id
            }
        }

        if { 0 } {
            #the nodes duplicated are expected to be on the same layer as the lines marked duplicated, fill a map of the nodes of these layers to find only in this subset
            objarray foreach line_id $lines_marked_duplicate {
                set layer [lindex [GiD_Geometry -v2 get line $line_id] 1]
                set layer_lines($layer) 1
                # it seems that the possible nodes could be also in the layer of the lines' points
                foreach point_id [lrange [GiD_Geometry -v2 get line $line_id] 2 3] {
                    set layer [lindex [GiD_Geometry -v2 get point $point_id] 0]
                    set layer_lines($layer) 1
                }
            }
            set layernames [lsort -dictionary [array names layer_lines]]
            #fill a map nodes_xyz by the nodes exactly at xyz (considered as a exact string, without any tolerance)
            foreach layername $layernames {
                set layer_nodes [GiD_Mesh list -layer $layername node]
                objarray foreach node_id $layer_nodes {
                    set xyz [GiD_Mesh get node $node_id coordinates]
                    lappend nodes_xyz($xyz) $node_id
                }
            }
        } else {
            # the previous is bad, the nodes of points are on the layer of the point, not the layer of the line
            # must find the nodes of the line elements in other way
            foreach element_id $line_element_ids {
                set element_node_ids [GiD_Mesh get element $element_id connectivities]
                foreach node_id $element_node_ids {
                    set node_of_line($node_id) 1
                }
            }
            set lines_nodes_ids [lsort -integer [array names node_of_line]]
            foreach node_id $lines_nodes_ids {
                set xyz [GiD_Mesh get node $node_id coordinates]
                lappend nodes_of_lines_xyz($xyz) $node_id
            }
            # fill the nodes of the whole mesh that match the xyz of some node of the lines duplicated            
            objarray foreach node_id [GiD_Mesh list node] {
                set xyz [GiD_Mesh get node $node_id coordinates]
                if { [info exists nodes_of_lines_xyz($xyz)] } {
                    lappend nodes_xyz($xyz) $node_id
                }
            }
            array unset nodes_of_lines_xyz
        }
        
        foreach element_id $line_element_ids {
            set element_node_ids [GiD_Mesh get element $element_id connectivities]
            set elements_with_face [GiD_Mesh get element from_face $element_node_ids]
            set sign ""
            if { [llength $elements_with_face] == 1 } {
                set element_neighbor [lindex $elements_with_face 0]
                # compare faces to try to set the orientation same or diff of the contact element to be created
                foreach element_face [GiD_Mesh get element $element_neighbor face] {
                    if { [lsort -integer $element_node_ids] != [lsort -integer $element_face] } {
                        set sign 0
                    } else {
                        # are equal or reverted
                        if { $element_node_ids == $element_face } {
                            # consider that start node is the same in both faces 
                            # but this can be false with faces that are not line, e.g triangles ABC and BCA will not be considered equal with same sign
                            set sign 1
                        } else {
                            set sign -1
                        }
                        break
                    }
                }
            }
            if { [llength $elements_with_face] == 0 } {
                # it seems that is a small defect, the line nodes maybe are not face of any current element, 
                # because some node of current elements is original and some new duplicateds
                # can find a valid face of all possible permutations
                #W "$element_node_ids is not any face, try to find some..." 
                set node_candidates_list [list]
                foreach node_id $element_node_ids {
                    set xyz [GiD_Mesh get node $node_id coordinates]
                    lappend node_candidates_list $nodes_xyz($xyz)
                }
                set face_candidates [list]
                foreach face_candidate [GidUtils::CartesianProduct {*}$node_candidates_list] {
                    set elements_with_face_candidate [GiD_Mesh get element from_face $face_candidate]
                    if { [llength $elements_with_face_candidate] == 1 } {
                        lappend face_candidates $face_candidate
                    } elseif { [llength $elements_with_face_candidate] > 1 } {
                        W "wrong master face $element_node_ids multiple elements with this face: $elements_with_face_candidate"
                    }
                }
                if { [llength $face_candidates] == 1} {
                    #W "... face $element_node_ids replaced by $face_candidates"
                    set element_node_ids [lindex $face_candidates 0]
                } elseif { [llength $face_candidates] >= 1} {
                    #W "... face $element_node_ids has multiple candidates: $face_candidates (by now get the fist one)"
                    #select one that preserve some node if any?
                    #by now get the fist one
                    set element_node_ids [lindex $face_candidates 0]
                } else {
                    W "element $element_node_ids seems that is wrongly connected, not match any element face"
                }
            }
            #find slave nodes to form the contact element
            
            array unset slave_node_candidates
            foreach node_id $element_node_ids {
                set xyz [GiD_Mesh get node $node_id coordinates]
                set slave_node_candidates($node_id) [list]
                foreach node_xyz_id $nodes_xyz($xyz) {
                    if { $node_xyz_id != $node_id } {
                        lappend slave_node_candidates($node_id) $node_xyz_id
                    }
                }
            }
            
            #try all combinations of possible faces with slave_node_candidates 
            #e.g. faults crossing create more than 2 node duplications, musk pick the correct one...
            
            set slave_node_candidates_list [list]
            foreach node_id $element_node_ids {
                lappend slave_node_candidates_list $slave_node_candidates($node_id)
            }
            set slave_face_candidates [list]
            foreach face_candidate [GidUtils::CartesianProduct {*}$slave_node_candidates_list] {
                set elements_with_face_candidate [GiD_Mesh get element from_face $face_candidate]
                if { [llength $elements_with_face_candidate] == 1 } {
                    lappend slave_face_candidates $face_candidate
                } elseif { [llength $elements_with_face_candidate] > 1 } {
                    W "master face $element_node_ids multiple elements with this face: $elements_with_face_candidate"
                }
            }
            if { [llength $slave_face_candidates] == 1} {
                set slave_node_ids [lindex $slave_face_candidates 0]
                set layername [lindex [GiD_Mesh get element $element_id] 0]
                GiD_Layers edit to_use $layername
                #set material [GiD_Mesh get element $element_id> material]
                set connectivities [concat $element_node_ids [lreverse $slave_node_ids]]
                if { $sign == 1 } {
                    # revert connectivities to have normal coherent with element_neighbor 
                    set connectivities [lreverse $connectivities]
                }
                #W "GiD_Mesh create element append quadrilateral 4 $connectivities"
                #GiD_Mesh create element append quadrilateral 4 $connectivities]
                GiD_Mesh edit element $element_id quadrilateral 4 $connectivities
            } else {
                W "master face $element_node_ids slave face candidates $slave_face_candidates"
            }
        }
        #GiD_Mesh delete element $line_element_ids
    }
    return 0
}

#register the proc to be automatically called when meshing
proc DuplicateNodesCreateContacts_Implement_Event_AfterMeshGeneration { fail } {
    if { !$fail } {
        GidUtils::LinesMeshCriteriaDuplicateAndToBeMeshed_CreateContactSurfaceElements
    }
}

GiD_RegisterEvent GiD_Event_AfterMeshGeneration DuplicateNodesCreateContacts_Implement_Event_AfterMeshGeneration
