import numpy as np
import tohil
import pymeshfix

#to create functions and variables for all tcl available ones
tcl=tohil.import_tcl()

def gid_points_to_meshfix_points(node_xyzs):
    xs,ys,zs=node_xyzs
    num_nodes=len(xs)
    points=np.empty((num_nodes,3))
    for i_node in range(num_nodes):
        points[i_node]=(float(xs[i_node]),float(ys[i_node]),float(zs[i_node]))
    #points[:, 0] = xs[:]
    #points[:, 1] = ys[:]
    #points[:, 2] = zs[:]
    return points

def tohil_obj_array_int_to_numpy(items):
    num_items=len(items)
    numpy_array=np.empty(num_items,np.int32)
    for i in range(num_items) :
        numpy_array[i]=int(items[i])
    return numpy_array

def numpy_renumber(connectivities,old_node_ids,new_node_ids):
    fail=0
    length_connectivities=len(connectivities)
    length_old_ids=len(old_node_ids)
    length_new_ids=len(new_node_ids)
    if(length_old_ids==length_new_ids):
        max_old_id=np.max(old_node_ids)
        new_number=np.empty(max_old_id,np.int32)
        for i in range(length_old_ids) :
            new_number[old_node_ids[i]]=new_node_ids[i]
        for i in range(length_connectivities) :
            connectivities[i]=new_number[connectivities[i]]
    else:
        fail=1
    return connectivities

def numpy_incr(connectivities,increment):
    connectivities=connectivities+increment
    return connectivities

def my_meshfix_clean_mesh_all():
    result=0
    info_nodes=tuple(tcl.GiD_Info('mesh','nodes','-array'))
    node_ids,node_xyzs=info_nodes
    #tcl.W(node_ids)
    #tcl.W(node_xyzs)
    max_id_nodes=int(node_ids[-1])
    num_nodes=len(node_ids)
    nodes_renumbered_for_meshfix=False
    if (max_id_nodes != num_nodes):
        nodes_renumbered_for_meshfix=True
        new_node_ids=np.arange(num_nodes)
    points=gid_points_to_meshfix_points(node_xyzs)
    info_elements=tuple(tcl.GiD_Info('mesh','elements','triangle','-array2'))
    if (len(info_elements)):
        #tcl.W(info_elements)
        elements_data=info_elements[0]
        element_type_ret,element_ids_original,connectivities_original,materials=elements_data
        #element_ids=tohil_obj_array_int_to_numpy(tuple(element_ids_original))
        connectivities=tohil_obj_array_int_to_numpy(tuple(connectivities_original))
        if (nodes_renumbered_for_meshfix):
            connectivities=numpy_renumber(connectivities,node_ids,new_node_ids)
        else:
            connectivities=numpy_incr(connectivities,-1)
        num_faces=int(len(connectivities)/3)
        faces=np.empty((num_faces,3),np.int32)
        for i_face in range(num_faces):
            pos=i_face*3
            faces[i_face]=(connectivities[pos],connectivities[pos+1],connectivities[pos+2])
        #tcl.W(points)
        #tcl.W(faces)
        #to avoid that numpy truncate the printed representation of its arrays
        np.set_printoptions(threshold=np.inf)
        points_clean,faces_clean=pymeshfix.clean_from_arrays(points,faces)
    return [points_clean,faces_clean]

