#source [file join $::GIDDEFAULT scripts gid_plugin_mesh_class.tcl]

namespace eval Spheres {
    variable info_plugin ;#information read from .xml plugin file
    set info_plugin(path) [file dirname [info script]] ;#to know where to find the files
}

#-----------------------------------------------------------------------

# class that store the mesher variables derived from GiDMeshVariables
package require TclOO

if { [info commands SphereMeshVariables] != "" } {
    SphereMeshVariables destroy ;#to protect of multiple source
}

oo::class create SphereMeshVariables {
    superclass GiDMeshVariables
    #variables showed in preferences:
    variable MinRadiusFactor MaxRadiusFactor PartSizeDist
    variable DistributionParam,StandarDeviation DistributionParam,Scale DistributionParam,Width
    variable Random BoundaryTolerance
    #variables hidden in preferences:
    variable ParticleType
    variable Seed Advanced Priority
    variable HighPorosity Porosity

    constructor { owner } {
        next $owner ;#to invoke the parent constructor
    }

    method GetDefault { key } {
        if { $key == "ParticleType" } {
            set value 1
        } elseif { $key == "MaxRadiusFactor" } {
            set value 1.2
        } elseif { $key == "MinRadiusFactor" } {
            set value 0.8
        } elseif { $key == "PartSizeDist" } {
            set value 0
        } elseif { $key == "DistributionParam,StandarDeviation" } {
            set value 0.1
        } elseif { $key == "DistributionParam,Scale" } {
            set value 1.0
        } elseif { $key == "DistributionParam,Width" } {
            set value 1.0
        } elseif { $key == "Random" } {
            set value 0
        } elseif { $key == "Seed" } {
            set value 1
        } elseif { $key == "Priority" } {
            set value 0
        } elseif { $key == "BoundaryTolerance" } {
            set value 1.0
        } elseif { $key == "Advanced" } {
            set value 0
        } elseif { $key == "HighPorosity" } {
            set value 0
        } elseif { $key == "Porosity" } {
            set value 0.95
        } else {
            set value 0
            error "SphereMeshVariables::GetDefault. Unexpected key $key"
        }
        return $value
    }

    method IsValid {} {
        set valid 1
        if { $ParticleType != 1 } {
            set valid 0
        } elseif { $MinRadius <= 0 } {
            set valid 0
        }
        return $valid
    }
    export GetDefault IsValid
}

#-----------------------------------------------------------------------


proc Spheres::GetExe { } {
    variable info_plugin
    if { $::tcl_platform(pointerSize) == 8 } {
        set bits 64
    } else {
        set bits 32
    }
    if { $::tcl_platform(platform) == "windows" } {
        set exefile [file join $info_plugin(path) bin win-x${bits} sphere_mesher.exe]
    } elseif { $::tcl_platform(os) == "Darwin" } {
        #Mac OS X
        set exefile [file join $info_plugin(path) bin mac-x${bits} sphere_mesher.exe]
        #set exefile ""
        #error "sphere_mesher not available for Mac OS X"
    } else {
        #Linux/Unix
        set exefile [file join $info_plugin(path) bin linux-x${bits} sphere_mesher.exe]
    }
    return $exefile
}

proc Spheres::WriteParameters { dimension size fp } {
    set obj [PluginMeshVariablesClass_GetClassObject SphereMeshVariables MESHING]
    puts $fp PARAMETERS
    #size is the mean diameter, calculate the values min_radius max_radius expected by the mesher based on an adimensional max_radius_factor >=1
    set mean_radius [expr {$size*0.5}] ;#size is the mean diameter, not the mean radius
    if { [$obj Get PartSizeDist] == 6 } {
        #Constant -> like Uniform (Flat), but min_radius==max_radius
        set max_radius $mean_radius
        set min_radius $mean_radius
    } else {
        set min_radius_factor [$obj Get MinRadiusFactor]
        set max_radius_factor [$obj Get MaxRadiusFactor]
        if { $max_radius_factor < 1.0 } {
            set min_radius_factor 1.0
        }
        if { $min_radius_factor <= 0.0 } {
            set max_radius_factor 0.8
        }
        set max_radius [expr {$mean_radius*$max_radius_factor}]
        set min_radius [expr {$mean_radius*$min_radius_factor}]
        #set difference [expr {$max_radius-$mean_radius}]
        #set min_radius [expr {$mean_radius-$difference}]
        #if { $min_radius < 0.0 } {
        #    set min_radius 0.0
        #}
    }

    #other statistic distributions
    foreach item {BoundaryTolerance ParticleType MinRadius MaxRadius PartSizeDist DistributionParam Random Priority Porosity} {
        if { $item == "DistributionParam" } {
            if { [$obj Get PartSizeDist] == 0 } {
                #Gauss
                #instead of a global value DistributionParam,Mean use as mean the size assigned to volumes or the general mesh size
                set standard_deviation [expr {$mean_radius*[$obj Get $item,StandarDeviation]}]
                puts $fp "DistributionParam $mean_radius $standard_deviation"
                #puts $fp "DistributionParam $mean_radius [$obj Get $item,StandarDeviation]"
            } elseif { [$obj Get PartSizeDist] == 1 } {
                #Exponential
                puts $fp "DistributionParam $mean_radius [$obj Get $item,Scale]"
            } elseif { [$obj Get PartSizeDist] == 2 } {
                #Laplace
                puts $fp "DistributionParam $mean_radius [$obj Get $item,Width]"
            } elseif { [$obj Get PartSizeDist] == 3 } {
                #Cauchy
                puts $fp "DistributionParam $mean_radius [$obj Get $item,Scale]"
            } elseif { [$obj Get PartSizeDist] == 4 } {
                #Rayleigh
                puts $fp "DistributionParam $mean_radius [$obj Get $item,Scale]"
            } elseif { [$obj Get PartSizeDist] == 5 } {
                #Uniform (Flat)
                #instead of a global value DistributionParam Min and Max use as range the size assigned to volumes or the general mesh size
                puts $fp "DistributionParam $min_radius $max_radius" ;#it seems that these parameters are not used, what it's used is MinRadius MaxRadius
            } elseif { [$obj Get PartSizeDist] == 6 } {
                #Constant -> like Uniform (Flat), but min_radius==max_radius
                puts $fp "DistributionParam $mean_radius $mean_radius" ;#it seems that these parameters are not used, what it's used is MinRadius MaxRadius
            } else {
                error "Unexpected distribution type [$obj Get PartSizeDist]"
            }
        } elseif { $item == "ParticleType" } {
            #by now not use the value of [$obj Get $item], bur maintain the variable hidden: maybe in the future to use elipsoids or others...
            if { $dimension == 2 } {
                puts $fp "$item 0" ;#0=circle
            } elseif { $dimension == 3 } {
                puts $fp "$item 1" ;#1=sphere
            } else {
                error "Unexpected dimension $dimension"
            }
        } elseif { $item == "MinRadius" } {
            puts $fp "MinRadius $min_radius"
        } elseif { $item == "MaxRadius" } {
            puts $fp "MaxRadius $max_radius"
        } elseif { $item == "Porosity" } {
            if { [$obj Get HighPorosity] == 1 } {
                puts $fp "$item [$obj Get $item]"
            } else {
                puts $fp "$item 0"
            }
        } elseif { $item == "BoundaryTolerance" } {
            set value [$obj Get $item]
            set tolerance [expr {2.0*$max_radius*$value}]
            puts $fp "BoundaryTolerance $tolerance"
        } elseif  { $item == "Random" } {
            #it seems that it is never random, and only change the seed value provided to randomize ??
            if { [$obj Get Random] } {
                #the special value 0 mean randomize with srand(time()), do not mean srand(0), then the series of rand() will be random!!
                set seed [clock seconds]
                Spheres::VariableManager SetValue Seed $seed
                puts $fp "Seed $seed"
            } else {
                #srand(1) provide a deterministic, non random, series of rand() values
                set seed [$obj Get Seed]
                puts $fp "Seed $seed"
            }
        } else {
            puts $fp "$item [$obj Get $item]"
        }
    }

    puts $fp END_PARAMETERS
    return ""
}

proc Spheres::WriteInputMesh { dimension mesh_data fp } {
    if { $dimension != 3 && $dimension != 2 } {
        error "Spheres::WriteInputMesh. Error dimension must be 2 for circles and 3 for spheres"
        return 1
    }
    lassign $mesh_data coordinates faces elements
    #faces and elements are provided with node ids from 0 but must be written from 1 in the file
    package require objarray
    objarray incr $faces
    objarray incr $elements
    puts $fp MESH
    if { $dimension == 3 } {
        puts $fp "$dimension [expr {[llength $coordinates]/3}] [expr {[llength $faces]/3}]"
    } else {
        #$dimension == 2
        puts $fp "$dimension [expr {[llength $coordinates]/2}] [expr {[llength $faces]/2}]"
    }
    puts $fp NODES
    if { $dimension == 3 } {
        foreach {x y z} $coordinates {
            puts $fp "$x $y $z"
        }
    } else {
        #$dimension == 2
        foreach {x y} $coordinates {
            puts $fp "$x $y"
        }
    }
    puts $fp END_NODES
    if { 0 } {
        #this mesher doesn't require the volume mesh, only the boundary
        puts $fp ELEMENTS
        foreach {n0 n1 n2 n3} $elements {
            puts $fp "$n0 $n1 $n2 $n3"
        }
        puts $fp END_ELEMENTS
    }
    if { [llength $faces] } {
        puts $fp FACES
        if { $dimension == 3 } {
            foreach {n0 n1 n2} $faces {
                puts $fp "$n0 $n1 $n2"
            }
        } else {
            #$dimension == 2
            foreach {n0 n1} $faces {
                puts $fp "$n0 $n1"
            }
        }
        puts $fp END_FACES
    }
    puts $fp END_MESH
    return 0
}

proc Spheres::WriteInputBoundaryParts { boundary_parts_data fp } {
    puts $fp BOUNDARY_CONDITIONS
    puts $fp "CONDITION Surface_BC"

    set count 0
    set bc_id 0
    foreach item [lindex $boundary_parts_data 0] {
        append str $bc_id " "
        for {set i $count} {$i < $item} {incr i 1} {
            append str $i " "
        }
        incr bc_id 1
        puts $fp $str
        set str ""
        set count $item
    }

    puts $fp "END_CONDITION Surface_BC"
    puts $fp END_BOUNDARY_CONDITIONS

    return 0
}

proc Spheres::ReadOutResultFile { fp } {
    set fail 0
    set element_radius [list]
    set boundary_list [list]
    set boundary_particles [list]
    gets $fp line
    if { $line != "GiD Post Results File 1.0" } {
        set fail 1
    } else {
        gets $fp line ;#Result "Radius" "(null)" 0 Scalar OnNodes
        set line [string trim $line]
        if { [lindex $line 0] == "Result" && [lindex $line 1] == "Radius" } {
            gets $fp line ;#ComponentNames Radius
            gets $fp line ;#Values
            while { ![eof $fp] } {
                gets $fp line
                if { $line == "End Values" } {
                    break
                } else {
                    #lappend element_radius [lindex $line 2] ;#ignore it, was not used
                }
            }
        }
        gets $fp line ;#Result "surface_bc" "CONDITION" 0 Scalar OnNodes
        set line [string trim $line]
        if { [lindex $line 0] == "Result" && [lindex $line 1] == "surface_bc" } {
            gets $fp line ;#Values
            gets $fp line ;#List of indexes
            set boundary_list [string trim $line]
            gets $fp line ;#End Values
        }
    }

    return [list $element_radius $boundary_list]
}

proc Spheres::ReadOutMeshFile { fp } {
    #assumed that nodes are from 1 to n without holes and consecutives
    #and elements also from 1 to n with node_id==element_id
    set nodes_coordinates [list]
    set element_radius [list]
    gets $fp line ;#MESH BALL dimension 3 ElemType Sphere Nnode 1
    set dim [lindex $line 3]
    if { [lindex $line 0] == "MESH" } {
        while { ![eof $fp] } {
            gets $fp line
            if { $line == "Coordinates" } {
                while { ![eof $fp] } {
                    gets $fp line
                    if { $line == "End Coordinates" } {
                        break
                    } else {
                        lappend nodes_coordinates {*}[lrange $line 1 end]
                    }
                }
            } elseif { $line == "Elements" } {
                while { ![eof $fp] } {
                    gets $fp line
                    if { $line == "End Elements" } {
                        break
                    } else {
                        lappend element_radius [lindex $line 2]
                    }
                }
            } else {

            }
        }
    } else {
        set fail 1
    }
    return [list $nodes_coordinates $element_radius]
}

proc Spheres::ReadOut { base_name } {
    set nodes_coordinates [list]
    set element_radius [list]
    set output_boundary_parts [list]
    set outfile ${base_name}.post.res
    if { [file exists $outfile] } {
        #really must read from .res the boundary regions only, radius could be read from .msh
        set fout [open $outfile r]
        lassign [Spheres::ReadOutResultFile $fout] element_radius output_boundary_parts
        close $fout
        file delete $outfile
    }
    set outfile ${base_name}.post.msh
    if { [file exists $outfile] } {
        set fout [open $outfile r]
        lassign [Spheres::ReadOutMeshFile $fout] nodes_coordinates element_radius
        close $fout
        file delete $outfile
    }
    return [list $nodes_coordinates $element_radius $output_boundary_parts]
}

# define these procs to avoid require the full package gid_cross_platform and allow specialized simple versions
if { $::tcl_platform(platform) == "windows" } {
    package require twapi_process

    proc Spheres::process_exists { pid } {
        return [twapi::process_exists $pid]
    }

    proc Spheres::end_process { pid } {
        return [twapi::end_process $pid -force]
    }
} else {
    proc Spheres::end_process { pid } {
        return [exec kill $pid]
    }

    proc Spheres::process_exists { pid } { 
        set exists 1
        set err [catch {exec ps $pid} err_txt]
        if { $err } {
            set exists 0
            if { [string first $pid $err_txt] != -1} {
                set exists 1
            }
        }
        return $exists
    }
}

proc Spheres::track_process { pid ms t0 finish_procedure progress_procedure } {
    if { [Spheres::process_exists $pid] } {
        if { $progress_procedure != "" } {
            set user_stop [eval $progress_procedure]
        } else {
            set user_stop 0
        }
        if { $user_stop } {
            if { [catch {Spheres::end_process $pid} err] } {
                #W "error '$err'"
            }
            [lindex $finish_procedure 0] $pid user_stop {*}[lrange $finish_procedure 1 end]
        } else {
            after $ms [list Spheres::track_process $pid $ms $t0 $finish_procedure $progress_procedure]
            update ;#to allow after to be processed, else the finish_procedure could be invoked later than the true end
        }
    } else {
        [lindex $finish_procedure 0] $pid ok {*}[lrange $finish_procedure 1 end]
    }
}

#dimension==3 to generate spheres, =2 to generate circles
proc Spheres::Generate { dimension id_entity input_mesh input_boundary_parts size } {
    variable process_info
    set result [list "" "" ""]
    set exefile [Spheres::GetExe]
    set fp [file tempfile tmp_file spheres_[pid]_${id_entity}_]
    set percent_filename [file rootname $tmp_file].percent
    file delete $percent_filename
    set error_filename [file rootname $tmp_file].err
    file delete $error_filename
    if { $fp != ""} {
        Spheres::WriteParameters $dimension $size $fp
        Spheres::WriteInputMesh $dimension $input_mesh $fp
        Spheres::WriteInputBoundaryParts $input_boundary_parts $fp
        close $fp

        set exefile_for_cmd $exefile
        if { [string first { } $exefile_for_cmd] != -1 } {
            set exefile_for_cmd "\"$exefile_for_cmd\""
        }
        set tmp_file_for_cmd $tmp_file
        if { [string first { } $tmp_file_for_cmd] != -1 } {
            set tmp_file "\"$tmp_file_for_cmd\""
        }
        set cmd "$exefile_for_cmd -f $tmp_file_for_cmd"
        set pid [eval exec $cmd &]
        set process_info($pid) 1
        if { $dimension == 3 } {
            set entity_type volume
        } else {
            set entity_type surface
        }
        Spheres::track_process $pid 1000 [clock seconds] Spheres::Finished [list Spheres::ProgressInMeshing $entity_type $id_entity $percent_filename]
        if { $Spheres::process_info($pid) != "0" } {
            vwait Spheres::process_info($pid)
        }
        if { $process_info($pid,status) == "ok" } {
            set error_message ""
            if { [file exists $error_filename] && [file size $error_filename] } {
                set fp [open $error_filename r]
                if { $fp != "" } {
                    set error_message [read $fp]
                    close $fp
                }
            }
            if { $error_message == "" } {
                set result [Spheres::ReadOut [file rootname $tmp_file]]
            } else {
                set result [list $error_message]
            }
        } elseif { $process_info($pid,status) == "user_stop" } {
            set result [list "user_stop"]
        } elseif { $process_info($pid,status) == "timeout" } {
            set result [list "timeout"]
        } else {
            set result [list $process_info($pid,status)]
        }
        file delete $tmp_file
        file delete $percent_filename
        file delete $error_filename
    }
    return $result
}

#this is a procedure with input and output that can be invoked from a GiD-Tcl event to generate internally the mesh of a volume
#input_mesh == [list [GiD_Info Mesh Nodes 1 end -array] [GiD_Info Mesh Elements Triangle 1 end -array]]
proc Spheres::GenerateSpheres { id_entity input_mesh input_boundary_parts size } {
    set dimension 3
    return [Spheres::Generate $dimension $id_entity $input_mesh $input_boundary_parts $size]
}

proc Spheres::GenerateCircles { id_entity input_mesh input_boundary_parts size } {
    set dimension 2
    return [Spheres::Generate $dimension $id_entity $input_mesh $input_boundary_parts $size]
}

proc Spheres::ProgressInMeshing { entity_type id_entity percent_filename } {
    set user_stop 0
    if { [file exists $percent_filename] && [file size $percent_filename]} {
        set fp [open $percent_filename r]
        if { $fp != "" } {
            set content [read $fp]
            close $fp
            if { [llength $content] } {
                set last_item [lindex $content end]
                if { [llength $last_item] == 2 } {
                    lassign $last_item percent num_elements
                    set factor [expr {$percent/100.0}]
                    set user_stop [GiD_ProgressInMeshing $entity_type $id_entity $factor $num_elements $num_elements]
                }
            }
        }
    }
    return $user_stop
}

proc Spheres::Finished { pid status } {
    variable process_info
    set process_info($pid,status) $status
    set process_info($pid) 0
}

#a callback procedure with a fixed prototype to be invoked by the preferences window to handle the custom mesher variables
proc Spheres::VariableManager { operation var {value ""} } {
    return [PluginMeshVariablesClass_VariableManager SphereMeshVariables $operation $var $value]
}
