package require dialogwin

if { [catch { package require gid_themes } err] } {
    set exists_gid_themes 0
} else {
    set exists_gid_themes 1
}

namespace eval gid_groups_conds {
    variable variables_idleid
}

proc gid_groups_conds::function_editor { args } {

    set optional {
        { -units_var varname "" }
        { -ov_var varname "" }
        { -funcs funcsList "" }
        { -options_func options_func "" }
    }
    set compulsory "parent title valueNode funcNode_var"
    parse_args $optional $compulsory $args

    upvar #0 $funcNode_var funcNode

    if { $funcs eq "" } {
        set funcs [cu::commalist_to_list [get_domnode_attribute $valueNode function]]
    }
    regsub -all {\W+} $title {_} t
    set w $parent.fe$t
    destroy $w

    set w [dialogwin_snit $w -title $title -grab 0 -transient 1 -callback \
            [list gid_groups_conds::function_editor_do -options_func $options_func \
                $valueNode $funcNode_var $funcs $units_var $ov_var]]
    ## Apply and Delete buttons unnecessary
#     set w [dialogwin_snit $w -title $title -grab 0 -transient 1 -callback \
#             [list gid_groups_conds::function_editor_do $valueNode \
#                 $funcNode_var $funcs $units_var $ov_var] \
#             -morebuttons [list [_ "Apply"] [_ "Delete"]]]

    set f [$w giveframe]

    set vars_dict ""
    foreach i $funcs {
        lassign $i function var pvar
        if { $function eq "scalar" } { continue }
        if { $pvar eq "" } { set pvar $var }
        dict set vars_dict $var $pvar
    }

    set f1 [ttk::labelframe $f.f1 -text [_ "Function variables"]]
    ttk::checkbutton $f1.ck1 -text [_ "Function on geometry"] \
        -variable [$w give_uservar func_on_xyz]
    cu::combobox $f1.cb1 -textvariable [$w give_uservar func_on_xyz_value] \
        -valuesvariable [$w give_uservar func_on_xyz_valueList ""] \
        -dictvariable [$w give_uservar func_on_xyz_dict ""] -state readonly \
        -width 25
    set xyz_button [ttk::button $f1.b1 -text [_ "View"]]
    set d [dict create 1 "$f1.cb1 $f1.b1"]
    $w enable_disable_on_key func_on_xyz $d

    ttk::checkbutton $f1.ck2 -text [_ "Function on time"] \
        -variable [$w give_uservar func_on_t]
    cu::combobox $f1.cb2 -textvariable [$w give_uservar func_on_t_value] \
        -valuesvariable [$w give_uservar func_on_t_valueList ""] \
        -dictvariable [$w give_uservar func_on_t_dict ""] -state readonly \
        -width 25

    set t_button [ttk::button $f1.b2 -text [_ "View"]]

    set d [dict create 1 "$f1.cb2 $f1.b2"]
    $w enable_disable_on_key func_on_t $d

    set idx 3
    dict for "n v" $vars_dict {
        if { $n eq "xyz" || $n eq "t" } { continue }
        ttk::checkbutton $f1.ck$idx -text [_ "Function on %s" $v] \
            -variable [$w give_uservar func_on_$n]
        cu::combobox $f1.cb$idx -textvariable [$w give_uservar func_on_${n}_value] \
            -valuesvariable [$w give_uservar func_on_${n}_valueList ""] \
            -dictvariable [$w give_uservar func_on_${n}_dict ""] -state readonly \
            -width 25

        set ${n}_button [ttk::button $f1.b$idx -text [_ "View"]]

        set d [dict create 1 "$f1.cb$idx $f1.b$idx $f.n1"]
        $w enable_disable_on_key func_on_$n $d
        incr idx
    }

    ttk::label $f1.ldesc -textvariable [$w give_uservar desc] \
        -relief solid -padding 3 -width 20 \
        -anchor w

    if { [dict exists $vars_dict xyz] } {
        grid $f1.ck1 $f1.cb1 $f1.b1 -sticky w -padx 2 -pady 2
        if { [llength [dict keys $vars_dict]] == 1 } { grid remove $f1.b1 }
    }
    if { [dict exists $vars_dict t] } {
        grid $f1.ck2 $f1.cb2 $f1.b2 -sticky w -padx 2 -pady 2
        if { [llength [dict keys $vars_dict]] == 1 } { grid remove $f1.b2 }
    }

    set idx 3
    dict for "n v" $vars_dict {
        if { $n eq "xyz" || $n eq "t" } { continue }
        if { [llength [$w give_uservar_value func_on_${n}_valueList ""]] > 1 } {
            grid $f1.ck$idx $f1.cb$idx $f1.b$idx -sticky w -padx 2 -pady 2
        } else {
            grid $f1.ck$idx $f1.cb$idx $f1.b$idx -sticky w -padx 2 -pady 2
            grid remove $f1.cb$idx
        }
        if { [llength [dict keys $vars_dict]] == 1 } { grid remove $f1.b$idx }
        incr idx
    }
    #grid $f1.ldesc  -       -   -sticky w -padx "60 2" -pady 6
    grid rowconfigure $f1 0 -weight 1
    grid columnconfigure $f1 0 -weight 1

    ttk::labelframe $f.n1 -text [_ "Function definition"]

    set function_func [get_domnode_attribute $valueNode function_func ""]
    $w set_uservar_value function_func $function_func
    set idx 0

    foreach "func_on_xyz func_on_xyz_value func_on_xyz_valueList func_on_xyz_dict" \
        [list 0 "" "" ""] break
    foreach "func_on_t func_on_t_value func_on_t_valueList func_on_t_dict" \
        [list 0 "" "" ""] break
    set idx 3
    dict for "n v" $vars_dict {
        if { $n eq "xyz" || $n eq "t" } { continue }
        foreach "func_on_$n func_on_${n}_value func_on_${n}_valueList func_on_${n}_dict" \
            [list 0 "" "" ""] break
        incr idx
    }

    set idx 3
    foreach "width height" [list 0 0] break
    set frames ""
    foreach i $funcs {
        lassign $i function var pvar magnitude
        if { $function eq "scalar" } { continue }
        if { [lsearch $funcs "matrix_func"] ne -1 && $function ne "matrix_func" } { continue }
        if { $pvar eq "" } { set pvar $var }
        set idxm [expr {$idx+1}]
        set frame [ttk::frame $f.n1.f$idxm]

        set tooltip ""
        if { $function eq "interpolator_func" } {
            set name [interpolator_func $w create $frame $var $pvar $magnitude \
                    $valueNode $funcNode $units_var $ov_var]
            $w set_uservar_value checkmodel 0
        } elseif { $function eq "matrix_func" } {
            set name [matrix_func $w create $frame $var $pvar $magnitude \
                    $valueNode $funcNode $units_var $ov_var]
            $w set_uservar_value checkmodel 0
        } else {
            if { $options_func != "" } {
                set title [$valueNode @title ""]
                $f1 configure -text [_ "%s" $title]
                set xpf [format_xpath {//value[@n=%s]} "Solve_Temperature_fluid"]
                set node [$gid_groups_conds::doc selectNodes $xpf]
                set Solve_Temperature_fluid [get_domnode_attribute $node v]
                set dict_options_func ""
                set modelList ""
                foreach "ni vi" [split [get_domnode_attribute $valueNode dict_options_func] ,] {
                    dict set dict_options_func $ni [= $vi]
                    lappend modelList $vi
                }

                set icount 0
                foreach ivalueNode [$valueNode childNodes] {
                    if {[$ivalueNode nodeName] != "value"} { continue }
                    set ipn [get_domnode_attribute $ivalueNode pn]
                    set entry [$ivalueNode @n ""]
                    set values [$ivalueNode @values ""]
                    if { $values eq "0,1" } {
                        ttk::checkbutton $f1.cbv$icount -text [_ "%s" $ipn] \
                            -variable [$w give_uservar $entry]
                        $w set_uservar_value $entry [$ivalueNode @v ""]
                        if { $entry eq "checkmodel" } {
                            ttk::label $f1.l$icount -text [_ "%s" $ipn]:
                            cu::combobox $f1.cbvnn1 -textvariable [$w give_uservar model] \
                                -valuesvariable [$w give_uservar modelList ""] \
                                -dictvariable [$w give_uservar dict_options_func ""] -state readonly
                            $w set_uservar_value dict_options_func [$valueNode @dict_options_func ""]
                            $w set_uservar_value modelList $modelList
                            trace add variable [$w give_uservar model] write \
                                "[namespace code [list gid_groups_conds::check_variables_idleid $w $frame $valueNode]];#"
                            bind $w <Destroy> +[list trace remove variable [$w give_uservar model] write \
                                    "[namespace code [list gid_groups_conds::check_variables_idleid $w $frame $valueNode]];#"]
                        }
                    } else {
                        ttk::label $f1.l$icount -text [_ "%s" $ipn]:
                        if {[$ivalueNode @help ""] != ""} {
                            set help [$ivalueNode @help ""]
                            cu::tooltip register $f1.l$icount "$help"
                        }
                        set entry [$ivalueNode @n ""]
                        if { [$ivalueNode @units ""] != "" } {
                            set unit_magnitude [$ivalueNode @unit_magnitude ""]
                            set state_combobox_tree [$ivalueNode @state_combobox_tree "normal"]
                            set state_entry [$ivalueNode @state_entry "normal"]
                            $f1.l$icount configure -state $state_entry
                            gid_groups_conds::entry_units $f1.units$icount -unit_magnitude $unit_magnitude \
                                -units_width 8 -value_variable [$w give_uservar $entry] \
                                -units_variable [$w give_uservar units$entry] \
                                -state_combobox_tree $state_combobox_tree \
                                -state_entry $state_entry
                            $w set_uservar_value units$entry [$ivalueNode @units ""]
                            $w set_uservar_value $entry [$ivalueNode @v ""]
                        } elseif {[$ivalueNode @unit_fixed ""] != "" } {
                            set state_combobox_tree [$ivalueNode @state_combobox_tree "normal"]
                            set state_entry [$ivalueNode @state_entry "normal"]
                            gid_groups_conds::entry_units $f1.units$icount -unit_magnitude "" \
                                -units_width 8 -value_variable [$w give_uservar $entry] \
                                -units_variable [$w give_uservar units$entry] \
                                -state_combobox_tree $state_combobox_tree \
                                -state_entry $state_entry
                            $w set_uservar_value units$entry [$ivalueNode @unit_fixed ""]
                            $w set_uservar_value $entry [$ivalueNode @v ""]
                        } else {
                            ttk::entry $f1.entry$icount -textvariable \
                                [$w give_uservar $entry] -width 22
                            $w set_uservar_value $entry [$ivalueNode @v ""]
                        }
                    }
                    incr icount
                }

                trace add variable [$w give_uservar temp_dep] write \
                    "[namespace code [list gid_groups_conds::check_t $w $frame $valueNode $f1 $Solve_Temperature_fluid]];#"
                bind $w <Destroy> +[list trace remove variable [$w give_uservar temp_dep] write \
                        "[namespace code [list gid_groups_conds::check_t $w $frame $valueNode $f1]];#"]
                trace add variable [$w give_uservar checkmodel] write \
                    "[namespace code [list gid_groups_conds::check_model $w $frame $valueNode $f1 $vars_dict]];#"
                bind $w <Destroy> +[list trace remove variable [$w give_uservar checkmodel] write \
                        "[namespace code [list gid_groups_conds::check_model $w $frame $valueNode $f1 $vars_dict]];#"]
                $w add_trace_to_uservar model [list gid_groups_conds::choose_model $f1 $w $valueNode $Solve_Temperature_fluid]
                $w set_uservar_value model [dict get $dict_options_func [$valueNode selectNodes {string(value[@n="model"]/@v)}]]
                $w set_uservar_value checkmodel [$valueNode selectNodes {string(value[@n="checkmodel"]/@v)}]
            } else {
                $w set_uservar_value checkmodel 0
            }

            set name [uplevel #0 $function_func [list $function create \
                        $var $pvar $frame $valueNode $funcNode \
                        $units_var $ov_var]]
            if { $name eq [_ "Function editor"] } {
                #    set tooltip [_ "Activate/deactivate function editor \nto insert functions into your variable"]
                #    tooltip::tooltip $f1.ck$idx $tooltip
                grid remove $f1.ck$idx
                grid remove $f1
                dict for "n v" $vars_dict {
                    if { $n eq "xyz" || $n eq "t" } { continue }
                    foreach "func_on_$n func_on_${n}_value func_on_${n}_valueList func_on_${n}_dict" \
                        [list 1 "" "" ""] break
                    incr idx
                }
            }
        }
        dict set frames $function $frame
        lappend func_on_${var}_valueList $function
        dict set func_on_${var}_dict $function $name
        if {[winfo exists $f1.ck$idx]} {
            $f1.ck$idx configure -text [_ "%s" $name]
        }

        update idletasks
        if { [winfo reqwidth $frame] > $width } {
            set width [winfo reqwidth $frame]
        }
        if { [winfo reqheight $frame] > $height } {
            set height [winfo reqheight $frame]
        }
        incr idx
    }


    $w set_uservar_value frames $frames
    foreach i "func_on_xyz func_on_xyz_value func_on_xyz_valueList func_on_xyz_dict" {
        $w set_uservar_value $i [set $i]
    }
    foreach i "func_on_t func_on_t_value func_on_t_valueList func_on_t_dict" {
        $w set_uservar_value $i [set $i]
    }
    $w set_uservar_value tree ""

    set idx 3
    dict for "n v" $vars_dict {
        if { $n eq "xyz" || $n eq "t" } { continue }
        foreach i "func_on_$n func_on_${n}_value func_on_${n}_valueList func_on_${n}_dict" {
            $w set_uservar_value $i [set $i]
        }
        incr idx
    }

    set c1 gid_groups_conds::_function_editor_change_desc
    set c2 gid_groups_conds::_function_editor_change_frame
    foreach i $funcs {
        lassign $i function var pvar
        if { $function eq "scalar" } { continue }

        $w add_trace_to_uservar func_on_${var} [list $c1 $valueNode $funcNode \
                $vars_dict $w $var]
        $w add_trace_to_uservar func_on_${var}_value [list $c2 $valueNode $funcNode \
                $vars_dict $w $var \
                $f.n1 $frames $options_func]
        [set ${var}_button] configure -command [list $c2 $valueNode $funcNode \
                $vars_dict $w $var \
                $f.n1 $frames $options_func]

        if { [$w give_uservar_value func_on_${var}_value] eq "" } {
            $w set_uservar_value func_on_${var}_value [lindex $function 0]
        }
    }

    $f.n1 configure -width [expr {$width+6}]

    set h 20
    if { $::exists_gid_themes } {
        set theme [gid_groups_conds::GetCurrentTheme]
        if { [regexp "Tdyn" $theme {} curr_theme] } {
            set h 40
        } else {
            set h 30
        }
    }
    $f.n1 configure -height [expr {$height+$h}]

    grid propagate $f.n1 0
    grid $f.f1 -sticky nsew
    grid $f.n1 -sticky nsew
    grid columnconfigure $f 0 -weight 1
    grid rowconfigure $f 1 -weight 1

    if { $funcNode ne "" } {
        foreach fvarNode [$funcNode selectNodes functionVariable] {
            set var [$fvarNode @variable ""]
            $w set_uservar_value func_on_$var 1
            $w set_uservar_value func_on_${var}_value [$fvarNode @n]
        }
    } elseif { [llength [dict keys $vars_dict]] == 1 } {
        set var [lindex $vars_dict 0]
        if { [llength [$w give_uservar_value func_on_${var}_valueList]] == 2 } {
            $w set_uservar_value func_on_$var 1
            $w set_uservar_value func_on_${var}_value [lindex \
                    [$w give_uservar_value func_on_${var}_valueList] 1]
        }
    }
    if { $options_func != "" } {
        grid remove $f.n1
        set idx 3
        grid remove $f1.ck$idx
    }

    set function_items [split [$valueNode @function ""] ","]
    if { ([llength $function_items] == 1 || \
        [llength $function_items] == 2) && "no_function" in $function_items} {
        $w set_uservar_value func_on_ 1
        grid remove $f1
    }

    #bind $w <Return> [list $w invokeok]
    $w createwindow
}

proc gid_groups_conds::check_t { w frame valueNode f1 Solve_Temperature_fluid } {
    if { [$w give_uservar_value temp_dep] && $Solve_Temperature_fluid } {
        set state normal
    } else {
        set state disabled
    }
    set icount 0
    set temp_dep_entry 0
    foreach ivalueNode [$valueNode childNodes] {
        if {[$ivalueNode nodeName] != "value"} { continue }
        if {[$ivalueNode @n ""] eq "temp_dep"} {
            set temp_dep_entry 1
        } elseif { $temp_dep_entry } {
            $f1.l$icount configure -state $state
            $f1.units$icount.e configure -state $state
            if {$state eq "normal"} { set c_state readonly } else { set c_state disabled }
            $f1.units$icount.c configure -state $c_state
        }
        incr icount
    }
}

proc gid_groups_conds::check_model { w frame valueNode f1 vars_dict } {
    if  { [$w give_uservar_value checkmodel] } {
        $w set_uservar_value func_on_ 1
    } else {
        $w set_uservar_value func_on_ 0
    }
}

proc gid_groups_conds::check_variables_idleid  { w frame valueNode } {
    variable variables_idleid

    if {[info exists variables_idleid]} {
        after cancel $variables_idleid
        unset variables_idleid
    }
    set variables_idleid [after 10 [list gid_groups_conds::check_variables $w $frame $valueNode]]

}

proc gid_groups_conds::check_variables { w frame valueNode } {
    foreach "ni vi" [split [$w give_uservar_value dict_options_func] ","] {
        if {$vi ne [$w give_uservar_value model]} { continue }
        break
    }
    switch $ni {
        herschel_bulkley - power_law - carreau {
            $w set_uservar_value power_law_n 0.4
            if { $ni eq "power_law" || $ni eq "herschel_bulkley" } {
                $w set_uservar_value unitsk "Pa*s^n"
                $w set_uservar_value k 10
            }
        }
        cross {
            $w set_uservar_value power_law_n 0.67
            $w set_uservar_value unitsk "s^-n"
            $w set_uservar_value k 1
        }
    }

}

proc gid_groups_conds::choose_model { f1 w valueNode Solve_Temperature_fluid } {
    foreach "ni vi" [split [get_domnode_attribute $valueNode dict_options_func] ,] {
        if {$vi eq [$w give_uservar_value model]} {
            break
        }
    }
    set icount 0
    foreach ivalueNode [$valueNode childNodes] {
        if {[$ivalueNode nodeName] != "value"} { continue }
        foreach igrid [list $f1.l$icount $f1.units$icount $f1.entry$icount $f1.cbv$icount \
                $f1.cbvnn1] {
            if {[winfo exists $igrid]} { grid remove $igrid }
        }
        incr icount
    }
    set icount 0
    set irow 0
    foreach ivalueNode [$valueNode childNodes] {
        if {[$ivalueNode nodeName] != "value"} { continue }
        if { [lsearch [split [$ivalueNode @modelList ""] ","] $ni] eq -1 && [$ivalueNode @n ""] ne "checkmodel"} {
            foreach igrid [list $f1.l$icount $f1.units$icount $f1.entry$icount $f1.cbv$icount] {
                if {[winfo exists $igrid]} {
                    grid remove $igrid
                }
            }
        } else {
            if {[$ivalueNode @values ""] eq "0,1"} {
                if { [$ivalueNode @n ""] eq "checkmodel" } {
                    grid $f1.l$icount $f1.cbvnn1 -sticky w -padx 2 -pady 2 -row $irow
                } elseif { [$ivalueNode @n ""] eq "temp_dep" } {
                    if {$Solve_Temperature_fluid} {
                        grid $f1.cbv$icount -sticky w -padx 2 -pady 2 -row $irow
                    }
                } else {
                    grid $f1.cbv$icount -sticky w -padx 2 -pady 2 -row $irow
                }
            } else {
                if { [$ivalueNode @n ""] eq "act_energy" || [$ivalueNode @n ""] eq "ref_temp" } {
                    if {$Solve_Temperature_fluid} {
                        if {[$ivalueNode @units ""] != "" || [$ivalueNode @unit_fixed ""] != ""} {
                            grid $f1.l$icount $f1.units$icount -sticky w -padx 2 -pady 2 -row $irow
                        } else {
                            grid $f1.l$icount $f1.entry$icount -sticky w -padx 2 -pady 2 -row $irow
                        }
                    }
                } else {
                    if {[$ivalueNode @units ""] != "" || [$ivalueNode @unit_fixed ""] != ""} {
                        grid $f1.l$icount $f1.units$icount -sticky w -padx 2 -pady 2 -row $irow
                    } else {
                        grid $f1.l$icount $f1.entry$icount -sticky w -padx 2 -pady 2 -row $irow
                    }
                }
            }
        }
        incr icount
        incr irow
    }
}

proc gid_groups_conds::function_editor_do { args } {

    set optional {
        { -options_func options_func "" }
    }
    set compulsory "valueNode funcNode_var funcs units_var ov_var w"
    parse_args $optional $compulsory $args


    upvar #0 $funcNode_var funcNode

    set action [$w giveaction]

    if { ![info exists funcNode] } { set funcNode "" }
    if { $action < 1 } {
        destroy $w
        return
    } elseif { $action == 3 } {
        if { $funcNode ne "" } {
            $funcNode delete
            set funcNode ""
        }
        destroy $w
        return
    }
    if { $funcNode eq "" } {
        set cloneNode ""
    } else {
        set cloneNode [$funcNode cloneNode -deep]
    }

    if { $funcNode eq "" } {
        set doc [$valueNode ownerDocument]
        set funcNode [$doc createElement function]
    } else {
        foreach i [$funcNode childNodes] { $i delete }
    }

    set frames [$w give_uservar_value frames]
    set function_func [$w give_uservar_value function_func]
    foreach i $funcs {
        lassign $i function var pvar magnitude

        if { $function eq "scalar" } { continue }

        if { $pvar eq "" } { set pvar $var }
        if { ![$w give_uservar_value func_on_${var}] && ![$w give_uservar_value checkmodel] } {
            continue
        }
        if { [$w give_uservar_value func_on_${var}_value] eq $function } {
            set f [dict get $frames $function]
            if { $function eq "interpolator_func" } {
                set err [catch { interpolator_func $w apply $f $var $pvar \
                            $magnitude $valueNode $funcNode $units_var \
                            $ov_var } errstring]
            } elseif { $function eq "matrix_func" } {
                set err [catch { matrix_func $w apply $f $var $pvar \
                            $magnitude $valueNode $funcNode $units_var \
                            $ov_var } errstring]
            } elseif { $options_func != "" } {
                if  { [$w give_uservar_value checkmodel] } {
                    $w set_uservar_value func_on_ 1
                } else {
                    $w set_uservar_value func_on_ 0
                }
                if {[$w give_uservar_value checkmodel]} {
                    foreach "ni vi" [split [get_domnode_attribute $valueNode dict_options_func] ,] {
                        if {$vi ne [$w give_uservar_value model]} { continue }
                        set modelNode [$valueNode selectNodes {value[@n="model"]}]
                        $modelNode setAttribute v $ni
                        break
                    }
                    set icount 0
                    foreach ivalueNode [$valueNode childNodes] {
                        if { [$ivalueNode nodeName] ne "value" } { continue }
                        if { [$ivalueNode @n ""] eq "model" } { continue }
                        set n_attr [$ivalueNode @n ""]
                        $ivalueNode setAttribute v [$w give_uservar_value $n_attr]
                        if {[$ivalueNode @units ""] != ""} {
                            $ivalueNode setAttribute units [$w give_uservar_value units$n_attr]
                        }
                        incr icount
                    }
                    set temp_dep [$w give_uservar_value temp_dep]
                    if { $temp_dep } {
                        set ref_tempNode [$valueNode selectNodes {value[@n="ref_temp"]}]
                        $ref_tempNode setAttribute state_entry normal
                        $ref_tempNode setAttribute state_combobox_tree readonly
                        set act_energyNode [$valueNode selectNodes {value[@n="act_energy"]}]
                        $act_energyNode setAttribute state_entry normal
                        $act_energyNode setAttribute state_combobox_tree readonly
                    } else {
                        set ref_tempNode [$valueNode selectNodes {value[@n="ref_temp"]}]
                        $ref_tempNode setAttribute state_entry disabled
                        $ref_tempNode setAttribute state_combobox_tree disabled
                        set act_energyNode [$valueNode selectNodes {value[@n="act_energy"]}]
                        $act_energyNode setAttribute state_entry disabled
                        $act_energyNode setAttribute state_combobox_tree disabled
                    }

                    if { [$w give_uservar_value power_law_n] >= 1 && [$modelNode @v] eq "cross"} {
                        set msg [_ "The power law index (n) must be lower than 1 for Cross model."]
                        snit_messageBox -message $msg -parent $w
                        return
                    }
                    set function "non_newtonian_tdyn_editor"
                    set err [catch { uplevel #0 $function_func [list $function \
                                    apply $var $pvar $f $valueNode $funcNode \
                                    $units_var $ov_var] } \
                            errstring]

                } else {
                    destroy $w
                    return
                }
            } else {
                set err [catch { uplevel #0 $function_func [list $function \
                                apply $var $pvar $f $valueNode $funcNode \
                                $units_var $ov_var] } \
                        errstring]
            }
            if { $err } {
                snit_messageBox -message $errstring -parent $w
                set funcNode $cloneNode
                return
            }
        }
    }
    if { $action == 2 } {
        # to restore initial function data
        set funcNode $cloneNode
    } elseif { $action == 1 } {
        # to make the trace work
        if { [$funcNode selectNodes functionVariable] eq "" } {
            $funcNode delete
            set funcNode ""
        } else {
            if { [[$funcNode selectNodes functionVariable] @pn] eq "" } {
                $funcNode delete
                set funcNode ""
            } else {
                set funcNode $funcNode
            }
        }
        destroy $w
    }
}

proc gid_groups_conds::_function_editor_change_desc { domNode funcNode vars_dict w what } {

    set fname [$domNode @pn_function [$domNode @pn]]

    if { $what ne "" } {
        if { [$w give_uservar_value func_on_$what] &&  \
            [$w give_uservar_value func_on_${what}_value] eq "" } {
            $w set_uservar_value func_on_${what}_value [lindex \
                    [$w give_uservar_value func_on_${what}_valueList] 0]
        }
        if { [$w give_uservar_value func_on_$what] == 0 } {
            $w set_uservar_value func_on_${what}_value ""
        }
    }
#     dict set vars_dict xyz x,y,z
#     dict set vars_dict t t

    set descList ""
    set idx 1
    dict for "n v" $vars_dict {
        if { [$w give_uservar_value func_on_$n] && \
            [$w give_uservar_value func_on_${n}_value] ne "" } {
            lappend descList "f${idx}($v)"
            incr idx
        }
    }
    if { [llength $descList] } {
        set desc "$fname=[join $descList \u00b7]"
    } else {
        set desc [format "$fname= #%s#" [_ "Not defined"]]
    }
    $w set_uservar_value desc $desc

#     upvar #0 $funcNode_var funcNode
    if {$funcNode != ""} {
        set fNode [$funcNode selectNodes {functionVariable}]
    } else {
        set fNode [$domNode selectNodes {function/functionVariable}]
    }
    if { $fNode ne "" && $descList != "" } {
        if { [$w give_uservar_value tree] != "" } {
            set tree [$w give_uservar_value tree]
            $tree item delete all
            set xp {value[@n='point']}
            foreach i [$fNode selectNodes $xp] {
                if { [info exists ov_variable] } {
                    if { [$i @ov ""] == "" || [$i @ov ""] eq $ov_variable } {
                        set v [$i @v ""]
                        set l [cu::commalist_to_list $v]
                        $tree insert end $l
                    }
                } else {
                    set v [$i @v ""]
                    set l [cu::commalist_to_list $v]
                    $tree insert end $l
                }
            }
        }
        if { [$fNode @units ""] ne "" } {
            $w set_uservar_value interpolator_unit \
                [gid_groups_conds::units_to_nice_units [$fNode @units ""]]
        }
        if { [$w give_uservar_value tree] != "" } {
            _interpolator_func_actualize_graph $w $tree
        }
    }
}

proc gid_groups_conds::_function_editor_change_frame { domNode funcNode vars_dict w what f0 frames options_func } {

    _function_editor_change_desc $domNode $funcNode $vars_dict $w ""

    set newtype [$w give_uservar_value func_on_${what}_value]

    foreach i [grid slaves $f0] { 
        grid forget $i
    }
    if { $newtype ne "" } {
        grid [dict get $frames $newtype] -row 0 -column 0 -sticky nwse \
            -padx 3 -pady 3

        grid rowconfigure $f0 0 -weight 1
        grid columnconfigure $f0 0 -weight 1

        set name [dict get [$w give_uservar_value func_on_${what}_dict] \
                $newtype]
        $f0 configure -text $name
    } else {
        $f0 configure -text [_ "Function definition"]
    }
}

################################################################################
#    Interpolator func
################################################################################

proc gid_groups_conds::interpolator_func { w what f0 n pn magnitude domNode funcNode units_var ov_var } {

    upvar #0 $units_var units_variable
    upvar #0 $ov_var ov_variable

    switch $what {
        create {
            set fnames [$domNode @pn_function [$domNode @pn]]            
            set f [ttk::panedwindow $f0.f -orient horizontal]

            package require fulltktree            
            set cols [list [list 10 $pn left text 1] ]         
            foreach fname [cu::commalist_to_list $fnames] {
                if { [info exists units_variable] } {
                  append fname " ($units_variable)"
                }
                lappend cols [list 10 $fname left text 1]                
            }
            set num_colums [llength $cols] 
            set table_width [expr 50*$num_colums]
            set tree [fulltktree $f.t -columns $cols -width $table_width -height 280 \
                    -selectmode extended -expand 1 -bd 1 -relief solid \
                    -showlines 0 -sensitive_cols all \
                    -contextualhandler [list gid_groups_conds::_interpolator_func_contextualmenu $w] \
                    -deletehandler [list gid_groups_conds::_interpolator_func_delete $w $f.t selection] \
                    ]
            $tree notify bind [$tree givetreectrl] <Header-invoke> ""
            $w set_uservar_value tree $tree

            $f add $tree -weight 1

            set f1 [ttk::frame $f.f1]
            $f add $f1 -weight 0
            set f11 [ttk::labelframe $f1.f1 -text [_ "data entry"]]

            lassign [list 0 0] row col

            set iList ""
            set jList ""
            set single_value [$domNode @single_value 0]
            if {!$single_value} {
                set iList [list single_value linear_ramp equation]
                set jList [list [_ "Single value"] [_ "Linear ramp"] [_ "Equation"]]
            }

            foreach i $iList j $jList {
                set r [ttk::radiobutton $f11.r${row}_${col} -text $j -variable \
                        [$w give_uservar interpolator_data_type] -value $i]
                if { $col > 1 } {
                    incr row
                    set col 0
                }
                grid $r -row $row -column $col -sticky nw -padx 2 -pady 2
                incr col
            }

            if { [info exists units_variable] } {
                set cmd [list gid_groups_conds::_interpolator_func_actualize_tree_col_label $w $tree $domNode $pn $units_var]
                trace add variable $units_var write "$cmd ;#"
                bind $f1 <Destroy> [list trace remove variable $units_var write "$cmd ;#"]
            }
            if { $magnitude ne "" && $magnitude ne "integer" } {
                ttk::label $f11.lunits -text [_ "%s units" $pn]:
                gid_groups_conds::entry_units $f11.eunits -unit_magnitude $magnitude \
                    -units_variable [$w give_uservar interpolator_unit]
                grid $f11.lunits -row 0 -column 2 -sticky w -padx "20 2"
                grid $f11.eunits -row 0 -column 3 -sticky w

                set cmd [list gid_groups_conds::_interpolator_func_actualize_tree_col_label \
                        $w $tree $domNode $pn $units_var]
                $w add_trace_to_uservar interpolator_unit $cmd
                $w eval_uservar_traces interpolator_unit
            }
            ttk::frame $f11.data

            grid $f11.data - - - -sticky nsew -padx 1
            grid columnconfigure $f11 3 -weight 1
            grid rowconfigure $f11 2 -weight 1

            ttk::button $f1.b1 -text [_ "Add at end"] -command \
                [list gid_groups_conds::_interpolator_func_add $w $tree end $f11.data $pn $fnames $magnitude]
            ttk::button $f1.b2 -text [_ "Add before"] -command \
            [list gid_groups_conds::_interpolator_func_add $w $tree prev $f11.data $pn $fnames $magnitude]
            ttk::button $f1.b3 -text [_ "Clear all"] -command \
                [list gid_groups_conds::_interpolator_func_delete $w $tree all]

            package require customLib_utils::img
            if { $::CUSTOMLIB_USE_GRAPHS_PLOTCHART == 0} {
                cu::draw_graphs $f1.c -height 100 -relief sunken -bd 1 -xlabel $pn -ylabel f -title [_ "Function graph"]
                if { $num_colums > 2 } {
                    set functionnames [list]
                    foreach col [lrange $cols 1 end] {
                        lappend functionnames [lindex $col 1]
                    }
                    $f1.c configure -functionnames $functionnames -showlegend 1
                }
                $w set_uservar_value draw_graphs $f1.c
            } else {
                set fgraph [ttk::frame $f1.fgraph]
                package require gid_graph
                #set points [list {0.0 25.0 189.0} {2.5 20 200.0} {4.3 5.0 250.0} {6.2 6.7 220.3}]
                set points [list]
                set y_text [list]
                foreach col [lrange $cols 1 end] {
                    lappend y_text [lindex $col 1]
                }
                set plot_command [list GidGraph::InFrame2 -points $points -title  [_ "Function graph"] -x_text $pn -y_text $y_text]
                if { [$domNode @function_graph_logx 0] } {
                    lappend plot_command -logx
                }
                if { [$domNode @function_graph_logy 0] } {
                    lappend plot_command -logy
                }
                if { [$domNode @function_graph_type ""] != "" } {
                    lappend plot_command -type [$domNode @function_graph_type]
                }
                if { [$domNode @function_graph_bar_horizontal 0]} {
                    lappend plot_command -bar_horizontal
                }
                if { [$domNode @function_graph_bar_stacked 0] } {
                    lappend plot_command -bar_stacked
                }
                lappend plot_command $fgraph
                $w set_uservar_value draw_graphs $plot_command
            }

            grid $f11 - - -sticky nsew
            grid $f1.b1 $f1.b2 $f1.b3 -sticky w -padx 3 -pady 3
            if { $::CUSTOMLIB_USE_GRAPHS_PLOTCHART == 0} {
                grid $f1.c - - -sticky nsew
            } else {
                grid $fgraph - - -sticky nsew
                grid columnconfigure $fgraph 0 -weight 1
                grid rowconfigure $fgraph 0 -weight 1
            }
            grid columnconfigure $f1 2 -weight 1
            grid rowconfigure $f1 2 -weight 1

            grid $f -sticky nsew
            grid columnconfigure $f0 0 -weight 1
            grid rowconfigure $f0 0 -weight 1

            set cmd [list gid_groups_conds::_interpolator_func_value_type $w $f11.data $pn $fnames $f1.b1 $magnitude]
            $w add_trace_to_uservar interpolator_data_type $cmd
            $w set_uservar_value interpolator_data_type single_value

            if { $funcNode ne "" } {
                set xp [format_xpath {functionVariable[@variable=%s and @n="interpolator_func"]} $n]
                set fvarNode [$funcNode selectNodes $xp]
            } else { set fvarNode "" }
            if { $fvarNode ne "" } {
                upvar #0 $units_var units_variable
                set xp {value[@n='point']}
                foreach i [$fvarNode selectNodes $xp] {
                    if { [info exists ov_variable] } {
                        if { [$i @ov ""] == "" || [$i @ov ""] eq $ov_variable } {
                            set v [$i @v ""]
                            set l [cu::commalist_to_list $v]
                            $tree insert end $l
                        }
                    } else {
                        set v [$i @v ""]
                        set l [cu::commalist_to_list $v]
                        $tree insert end $l
                    }
                }
                if { [$fvarNode @units ""] ne "" } {
                    $w set_uservar_value interpolator_unit [gid_groups_conds::units_to_nice_units [$fvarNode @units]]
                }
                _interpolator_func_actualize_graph $w $tree
            }
            return [_ "Interpolation function at %s" $pn]
        }
        apply {
            set tree $f0.f.t
            if { ![llength [$tree item children 0]] } {
                error [_ "There are no points to define the interpolation function"]
            }
            set min_two_pnts [$domNode @min_two_pnts 0]
            if { $min_two_pnts } {
                if { [llength [$tree item children 0]] < 2 } {
                    error [_ "There are not enough points to define the interpolation function"]
                }
            }
            set xp [format_xpath {functionVariable[@variable=%s]} $n]
            if { [$funcNode selectNodes $xp] eq "" } {
                set fvarNode [$funcNode appendChildTag functionVariable]
            } else {
                set fvarNode [$funcNode selectNodes $xp]
                foreach i [$fvarNode childNodes] { $i delete }
            }
            $fvarNode setAttribute n interpolator_func pn [_ "Interpolation function at %s" $pn] variable $n

            if { $magnitude ne "" && $magnitude ne "integer" } {
                $fvarNode setAttribute units [gid_groups_conds::nice_units_to_units [$w give_uservar_value interpolator_unit]]
            }
            foreach i [$tree item children 0] {
                set pnt [cu::list_to_commalist [$tree item text $i]]
                set v [$fvarNode appendChildTag value [list attributes() n point pn [= Point] v $pnt]]
            }
        }
    }
}

proc gid_groups_conds::_interpolator_func_actualize_tree_col_label { w tree domNode pn units_var } {

    upvar #0 $units_var units_variable

    set name "$pn ([$w give_uservar_value interpolator_unit])"
    $tree column configure 0 -text $name

    set fnames [$domNode @pn_function [$domNode @pn]]
    set i_column 1
    foreach fname [cu::commalist_to_list $fnames] {
        if { [info exists units_variable] } {
            append fname " ($units_variable)"
        }
        $tree column configure $i_column -text $fname
        incr i_column
    }
}

proc gid_groups_conds::_interpolator_func_actualize_graph { w tree } { 
    set tree_item_children [$tree item children 0]
    set num_rows [llength $tree_item_children]
    if { $num_rows } {
        if { $::CUSTOMLIB_USE_GRAPHS_PLOTCHART == 0} {
            set num_graphs [expr [llength [$tree item text [lindex $tree_item_children 0]]]-1]        
            foreach item $tree_item_children {
                set line [$tree item text $item]
                set x [lindex $line 0]
                for {set j 1} {$j<=$num_graphs} {incr j} {
                    lappend graph($j) [list $x [lindex $line $j]]
                }
            }
            set xyvalues [list]
            if { $num_graphs == 1 } {
                #it seems that cu::draw_graphs is inconsistent: with 1 graph is a list,  >1 is a list of lists (one by graph)!!
                set xyvalues $graph(1)
            } else {
                for {set j 1} {$j<=$num_graphs} {incr j} {
                    lappend xyvalues $graph($j)
                }
            }
            set d [$w give_uservar_value draw_graphs]
            $d configure -xyvalues $xyvalues
        } else {
            set points [list]
            foreach item $tree_item_children {
                lappend points [$tree item text $item]                
            }
            set cmd_graph [$w give_uservar_value draw_graphs]
            set pos_points [lsearch $cmd_graph -points]
            if { $pos_points != -1 } {
                incr pos_points
                lset cmd_graph $pos_points $points
            }
            eval $cmd_graph
     
        }
    }
}

proc gid_groups_conds::_interpolator_func_add { w tree where f pn fnames magnitude } {

    set data_type [$w give_uservar_value interpolator_data_type]
    set integer_fields ""
    set max_col_by_line 3
    switch $data_type {
        single_value {
            set fnames [cu::commalist_to_list $fnames]
            set fields [list x]
            set names [list [list $pn:]]
            set i_graph 1
            foreach fname $fnames {
                lappend fields y$i_graph
                lappend names $fname:
                incr i_graph
            }
            set max_col_by_line 5
            if { $magnitude eq "integer" } { 
                set integer_fields x
            }            
        }
        linear_ramp {
            set fields [list x0 y0 xend yend deltax]
            if { $magnitude eq "integer" } {
                set integer_fields [list x0 xend deltax]
            }
            set names [list [list $pn "" 0 subscript :] [list f "" 0 subscript :] \
                    [list $pn "" end subscript :] [list f "" end subscript :] \u0394$pn:]
        }
        equation {
            set fields [list x0 xend deltax]
            if { $magnitude eq "integer" } {
                set integer_fields [list x0 xend deltax]
            }
            set names [list [list $pn "" 0 subscript :] [list $pn "" end subscript :] \
                    \u0394$pn:]
        }
        periodic {

        }
    }
    lassign [list 0 0] row col
    foreach i $fields j $names {
        set v [$w give_uservar_value $i]
        if { [lsearch $integer_fields $i] != -1 && ![string is integer -strict $v] } {
            set field ""
            foreach {txt tag} $j { 
                append field $txt
            }
            set e $f.e${row}_${col}
            tk::TabToWindow $e
            set t [_ "Error in field %s. It must be a integer" $txt]
            snit_messageBox -parent $f -message $t
            return
        }
        if { ![string is double -strict $v] } {
            set field ""
            foreach {txt tag} $j { 
                append field $txt
            }
            set e $f.e${row}_${col}
            tk::TabToWindow $e
            set t [_ "Error in field %s. It must be a number" $txt]
            snit_messageBox -parent $f -message $t
            return
        }
        if { $col > $max_col_by_line } {
            set col 0
            incr row
        }
        incr col 2
    }

    set pnts ""
    foreach i $fields { 
        set $i [$w give_uservar_value $i]
    }
    switch $data_type {
        single_value {
            set pnt [list]
            foreach i $fields { 
                lappend pnt [set $i]
            }            
            lappend pnts $pnt
        }
        linear_ramp - equation {
            set delta_tot [expr {double($xend-$x0)}]
            if { $delta_tot == 0 } {
                snit_messageBox -parent $f -message [_ "Difference of variable values cannot be zero"]
                return
            }
            if { $deltax == 0 } {
                snit_messageBox -parent $f -message [_ "Increment cannot be zero"]
                return
            }
            if { $deltax*($xend-$x0) < 0 } {
                snit_messageBox -parent $f -message [_ "Increment sign is not correct"]
                return
            }
            if { $data_type eq "equation" } {
                set yfunc [$w give_uservar_value y]
                regsub -all {\mx\M} $yfunc {$x} yfunc
            }
            for { set x $x0 } { $x <= [expr {$xend+.1*$deltax}] } { set x [expr {$x+$deltax}] } {
                switch $data_type {
                    linear_ramp {
                        set y [expr {($x-$x0)/$delta_tot*$yend+
                                ($xend-$x)/$delta_tot*$y0}]
                    }
                    equation {
                        set err [catch { expr $yfunc } y]
                        if { $err } {
                            snit_messageBox -parent $f -message [_ "Function is not correct (%s)" $y]
                            return
                        }
                    }
                }
                lappend pnts [list [nice_number $x] [nice_number $y]]
            }
        }
    }
    switch $where {
        prev {
            set sibling [lindex [$tree selection get] 0]
            if { $sibling eq "" } {
                snit_messageBox -parent $tree -message \
                    [_ "It is necessary to select one point"]
                return
            }
        }
        end {
            set sibling 0
        }
    }
    if { $where eq "end" } {
        set pnt0 [$tree item text last]
        if { $pnt0 eq [lindex $pnts 0] } {
            set ret [snit_messageBox -type okcancel -default ok \
                    -message [_ "Point is equal to the previous one. Are you sure to insert point?"] \
                    -parent $tree]
            if { $ret == "cancel" } { return }
        }
    }
    foreach i $pnts {
        $tree insert $where $i $sibling
    }
    _interpolator_func_actualize_graph $w $tree
}

proc gid_groups_conds::_interpolator_func_contextualmenu { w tree items x y } {

    catch { destroy $tree._cmenu }
    set menu [menu $tree._cmenu -tearoff 0]

    $menu add command -label [_ "Select all"] \
        -command [list gid_groups_conds::_interpolator_func_cutcopypaste $w $tree selectall]
    $menu add separator
    $menu add command -label [_ "Copy"] \
        -command [list gid_groups_conds::_interpolator_func_cutcopypaste $w $tree copy]
    $menu add command -label [_ "Paste"] \
        -command [list gid_groups_conds::_interpolator_func_cutcopypaste $w $tree paste]
    $menu add command -label [_ "Paste end"] \
        -command [list gid_groups_conds::_interpolator_func_cutcopypaste $w $tree paste_end]
    $menu add separator

    $menu add command -label [_ "Delete"] \
        -command [list gid_groups_conds::_interpolator_func_delete $w $tree selection]
    tk_popup $menu $x $y
}

proc gid_groups_conds::_interpolator_func_cutcopypaste { w tree what } {

    switch $what {
        selectall {
            $tree selection add all
        }
        cut {
            set data ""
            foreach i [$tree selection get] {
                lappend data [join [$tree item text $i] \t]
                $tree item delete $i
            }
            clipboard clear
            clipboard append [join $data \n]
            _interpolator_func_actualize_graph $w $tree
        }
        copy {
            set tree_selection [lsort -integer [$tree selection get]]
            foreach i $tree_selection {
                lappend data [join [$tree item text $i] \t]
            }
            clipboard clear
            clipboard append [join $data \n]
        }
        paste {
            if { [$tree item count] <= 1 } {
                #if table is empty paste at the end
                return [gid_groups_conds::_interpolator_func_cutcopypaste $w $tree paste_end]
            }
            set err [catch { clipboard get } data]
            if { $err } { 
                return
            }
            set idx [lindex [$tree selection get] 0]
            if { $idx eq "" } {
                return
            }            
            set num_columns [llength [$tree cget -columns]]
            foreach i [split $data \n] {
                set l [lrange [split $i \t] 0 end]
                if { [llength $l] != $num_columns } {
                    continue
                }
                $tree insert prev $l $idx
            }
            _interpolator_func_actualize_graph $w $tree
        }
        paste_end {
            set err [catch { clipboard get } data]
            if { $err } { 
                return
            }
            set num_columns [llength [$tree cget -columns]]
            foreach i [split $data \n] {
                set l [lrange [split $i \t] 0 end]
                if { [llength $l] != $num_columns } {
                    continue
                }
                $tree insert end $l
            }
            _interpolator_func_actualize_graph $w $tree
        }
    }

}

proc gid_groups_conds::_interpolator_func_delete { w tree what args } {

    switch $what {
        all {
            set ret [snit_messageBox -type okcancel -default ok \
                    -message [_ "Are you sure to delete all points?"] \
                    -parent $tree]
            if { $ret == "cancel" } { return }

            $tree item delete all
        }
        selection {
            foreach i [$tree selection get] {
                $tree item delete $i
            }
        }
    }
    _interpolator_func_actualize_graph $w $tree
}

proc gid_groups_conds::_interpolator_func_value_type { w f pn fnames button magnitude } {

    set data_type [$w give_uservar_value interpolator_data_type]

    eval destroy [winfo children $f]

    if { [string index $pn 0] ne "\u0394" } {
        set delta_pn "\u0394$pn"
    } else {
        set delta_pn "\u0394($pn)"
    }

    set max_col_by_line 3
    switch $data_type {
        single_value {
            set fnames [cu::commalist_to_list $fnames]
            set fields [list x]
            set names [list [list $pn:]]
            set i_graph 1
            foreach fname $fnames {
                lappend fields y$i_graph
                lappend names $fname:
                incr i_graph
            }
            set max_col_by_line 5
        }
        linear_ramp {
            set fields [list x0 y0 xend yend deltax]
            set names [list [list $pn "" 0 subscript :] [list $fnames "" 0 subscript :] \
                    [list $pn "" end subscript :] [list $fnames "" end subscript :] $delta_pn:]
        }
        equation {
            set fields [list x0 xend deltax]
            set names [list [list $pn "" 0 subscript :] [list $pn "" end subscript :] $delta_pn:]
        }
    }
    lassign [list 0 0] row col
    foreach i $fields j $names {
        set l [cu::nicelabel $f.l${row}_${col} -width 9]
        foreach {txt tag} $j {
            $l insert end $txt $tag
        }
        switch $magnitude {
            integer { set default 0 }
            default { set default 0.0 }
        }
        set e [ttk::entry $f.e${row}_${col} -textvariable [$w give_uservar $i $default] -width 8]
        bind $e <Return> "$button invoke; tk::TabToWindow $f.e0_0; break"
        if { $col > $max_col_by_line } {
            set col 0
            incr row
        }
        grid $l -row $row -column $col -sticky w -padx 2 -pady 2
        grid $e -row $row -column [expr {$col+1}] -sticky w -padx 2 -pady 2
        incr col 2
    }

    if { $data_type eq "equation" } {
        set values {
            3*x+log(x)
            pow(x,2)*exp(x)
            x*cos(3*x)
            sqrt(x+2)+sinh(pow(x,3))
            atan(x)
            abs(x)
        }
        ttk::label $f.l -text "${fnames}($pn):"
        ttk::combobox $f.cb -textvariable [$w give_uservar y] -width 25 -values $values
        bind $f.cb <Return> "$button invoke; tk::TabToWindow $f.e0_0; break"
        grid $f.l $f.cb - - -sticky w -padx 2 -pady 2
    }

    grid columnconfigure $f 3 -weight 1

    tk::TabToWindow $f.e0_0
}

################################################################################
#    matrix_func
################################################################################


proc gid_groups_conds::matrix_func { w what f0 n pn magnitude domNode funcNode units_var ov_var } {

    upvar #0 $units_var units_variable

    switch $what {
        create {
            set fname [$domNode @pn_function [$domNode @pn]]
            set value [$domNode @v 0.0]
            if { $value == "" } { set value 0.0 }

            set diag [$domNode @has_diag 1]
            set dimension [get_domnode_attribute $domNode dimension_function 3]

            set dimension_rows [get_domnode_attribute $domNode dimension_function_rows $dimension]
            set dimension_cols [get_domnode_attribute $domNode dimension_function_cols $dimension]

            if { $dimension_rows != $dimension_cols } {
                set symmetric 0
            } else {
                set symmetric [$domNode @symmetric_function 1]
            }
            set funcs ""
            set num_func 0
            foreach f [cu::commalist_to_list [$domNode @function ""]] {
                if { $f eq "" } { continue }
                if { $f ne "scalar" && [lsearch [cu::commalist_to_list [$domNode @function ""]] "no_function"] == "-1" } {
                    incr num_func
                }
                lappend funcs $f
            }
            if { [$domNode @components_function ""] ne "" } {
                set cs [cu::commalist_to_list [$domNode @components_function]]
            } elseif { $dimension <= 3 && $dimension == $dimension_rows && $dimension == $dimension_cols } {
                set cs [list x y z]
            } else {
                set max_dimension [lsort -decreasing [list $dimension $dimension_rows $dimension_cols]]
                set cs [rangeF 1 $max_dimension]
            }
            if { $funcNode ne "" } {
                set xp {functionVariable[not(@variable) and @n="matrix_func"]}
                set fvarNode [$funcNode selectNodes $xp]
                set vNodes [$fvarNode selectNodes value]
            } else {
                set vNodes ""
            }
            if { [llength $vNodes] } {
                set idx 0
                if { $symmetric } {
                    set vdimension_rows [expr int(0.5*(sqrt([llength $vNodes]*8+1)-1))]
                    set vdimension_cols $vdimension_rows
                } elseif { $dimension_rows == $dimension_cols } {
                    set vdimension [expr int(sqrt([llength $vNodes]))]
                    set vdimension_rows $vdimension
                    set vdimension_cols $vdimension
                } else {
                    set vdimension_rows $dimension_rows
                    set vdimension_cols $dimension_cols
                }
                for { set irow 0 } { $irow < $vdimension_rows } { incr irow } {
                    for { set icol 0 } { $icol < $vdimension_cols } { incr icol } {
                        if { $symmetric && $icol < $irow } {
                            continue
                        }
                        set mNode($irow,$icol) [lindex $vNodes $idx]
                        incr idx
                    }
                }
            }
            set idx 0
            for { set irow 0 } { $irow < $dimension_rows } { incr irow } {
                for { set icol 0 } { $icol < $dimension_cols } { incr icol } {
                    set l [cu::nicelabel $f0.l$irow-$icol]
                    if {$irow == $icol && !$diag} {
                        $l insert end ""
                    } else {
                        $l insert end $fname
                        $l insert end [lindex $cs $irow],[lindex $cs $icol] subscript
                        $l insert end :
                    }
                    set e [ttk::entry $f0.e$irow-$icol -width 8]
                    set has_func 0
                    if {$irow == $icol && !$diag} {
                        $e insert end ""
                        $e state disabled
                    } else {
                        if { $symmetric && $icol < $irow } {
                            $e state disabled
                        } else {
                            if { [llength $vNodes] } {
                                if { [info exists mNode($irow,$icol)] } {
                                    set vNode [set mNode($irow,$icol)]
                                    $e insert end [$vNode @v]
                                } else {
                                    if {!$diag} {
                                        $e insert end $value
                                    } else {
                                        $e insert end 0.0
                                    }
                                }
                            } else {
                                set vNode ""
                                if { $irow == $icol } {
                                    $e insert end $value
                                } else {
                                    if {!$diag} {
                                        $e insert end $value
                                    } else {
                                        $e insert end 0.0
                                    }
                                }
                            }
                            incr idx
                            if { $num_func } { set has_func 1 }
                        }
                    }
                    set vfuncs $funcs
                    if { $has_func } {
                        set n [$domNode @n]-$irow-$icol
                        set mpos [lsearch $funcs "matrix_func"]
                        set vfuncs [lreplace $funcs $mpos $mpos]
                        set img [gid_groups_conds::giveimage eq-16]

                        set b [ttk::button $f0.b$irow-$icol -style Toolbutton -image \
                                $img \
                                -command [list gid_groups_conds::function_editor \
                                    -funcs $vfuncs \
                                    $w [_ "Create function for %s" $fname] \
                                    $domNode [$w give_uservar n_$n]]]
                        gid_groups_conds::register_popup_help $b \
                            [_ "Create/edit a function for this field"]
                        if { $vNode ne "" } {
                            #set funcNode [$vNode selectNodes function]
                            if { [info exists mNode($irow,$icol)] } {
                                set vNode [set mNode($irow,$icol)]
                                set funcNode [$vNode selectNodes function]
                            } else {
                                set funcNode ""
                            }
                        } else {
                            set funcNode ""
                        }
                        set cmd [list gid_groups_conds::_matrix_func_update \
                                $w $n $funcs $value $b $e]
                        $w add_trace_to_uservar n_$n $cmd

                        if { $funcNode ne "" } {
                            $w set_uservar_value n_$n [$funcNode cloneNode -deep]
                        } else {
                            $w set_uservar_value n_$n ""
                        }
                    }
                    if { $num_func } {
                        set ic [expr {$icol*3}]
                    } else {
                        set ic [expr {$icol*2}]
                    }
                    $l configure -width 15
                    grid $l -row $irow -column $ic -sticky e -padx "2 0" -pady 2
                    grid $e -row $irow -column [expr {$ic+1}] -sticky ew -padx "0 2" -pady 2
                    grid columnconfigure $f0 [expr {$ic+1}] -weight 1
                    if { $has_func } {
                        grid configure $e -padx 0
                        if { $vfuncs ne "" } {
                            grid $b -row $irow -column [expr {$ic+2}] -sticky ew -padx "0 5" -pady 2
                        }
                    }
                    if { $irow == $dimension-1 } {
                        grid configure $e -pady "2 5"
                    }
                    bind $e <3> [list gid_groups_conds::_matrix_func_contextual $w $e \
                            $has_func $vfuncs $n $fname $domNode %X %Y]
                }
            }
            return [_ "Matrix value for %s" $fname]
        }
        apply {
            set fname [$domNode @pn_function [$domNode @pn]]
            set diag [$domNode @has_diag 1]
            set dimension [get_domnode_attribute $domNode dimension_function 3]

            set dimension_rows [get_domnode_attribute $domNode dimension_function_rows $dimension]
            set dimension_cols [get_domnode_attribute $domNode dimension_function_cols $dimension]

            if { $dimension_rows != $dimension_cols } {
                set symmetric 0
            } else {
                set symmetric [$domNode @symmetric_function 1]
            }
            if { [$domNode @components_function ""] ne "" } {
                set cs [cu::commalist_to_list [$domNode @components_function]]
            } elseif { $dimension <= 3 && $dimension == $dimension_rows && $dimension == $dimension_cols } {
                set cs [list x y z]
            } else {
                set max_dimension [lindex [lsort -decreasing [list $dimension $dimension_rows $dimension_cols]] 0]
                set cs [rangeF 1 $max_dimension]
            }
            for { set irow 0 } { $irow < $dimension_rows } { incr irow } {
                for { set icol 0 } { $icol < $dimension_cols } { incr icol } {
                    if {$irow == $icol && !$diag} {
                        set e $f0.e$irow-$icol
                        set v [$e get]
                        if {$v != ""} {
                            set name "$fname[lindex $cs $irow],[lindex $cs $icol]"
                            error [_ "There should not be a value for %s. \n\
                                    It should be left blank" $name]
                        } else {
                            continue
                        }
                    }
                    if { $symmetric && $icol < $irow } { continue }
                    set n [$domNode @n]-$irow-$icol
                    if { [$w exists_uservar n_$n] && [$w give_uservar_value n_$n] ne "" } {
                        continue
                    }
                    set e $f0.e$irow-$icol
                    set v [$e get]
                    if { ![string is double -strict $v] } {
                        tk::TabToWindow $e
                        set name "$fname[lindex $cs $irow],[lindex $cs $icol]"
                        error [_ "Value %s is not a real number" $name]
                    }
                }
            }
            set xp {functionVariable[not(@variable) and @n="matrix_func"]}
            if { [$funcNode selectNodes $xp] eq "" } {
                set fvarNode [$funcNode appendChildTag functionVariable]
            } else {
                set fvarNode [$funcNode selectNodes $xp]
                foreach i [$fvarNode childNodes] { $i delete }
            }
            $fvarNode setAttribute n matrix_func pn \
                [_ "Matrix value for %s" $fname]

            for { set irow 0 } { $irow < $dimension_rows } { incr irow } {
                for { set icol 0 } { $icol < $dimension_cols } { incr icol } {
                    if { $symmetric && $icol < $irow } { continue }
                    set e $f0.e$irow-$icol
                    set v [$e get]
                    set name "$fname[lindex $cs $irow],[lindex $cs $icol]"
                    set n [$domNode @n]-$irow-$icol
                    set vNode [$fvarNode appendChildTag value [list attributes() \
                                n value pn $name v $v]]
                    if { [$w exists_uservar n_$n] && [$w give_uservar_value n_$n] ne "" } {
                        $vNode setAttribute  v ""
                        $vNode appendChild [[$w give_uservar_value n_$n] cloneNode -deep]
                    }
                }
            }
        }
    }
}

proc gid_groups_conds::_matrix_func_destroy_handler { w n } {
    if { [$w exists_uservar n_$n] && [$w give_uservar_value n_$n] ne "" } {
        [$w give_uservar_value n_$n] delete
        $w unset_uservar $n
    }
}

proc gid_groups_conds::_matrix_func_update { w n funcs value button entry } {

    if { [$w give_uservar_value n_$n] ne "" } {
        $w add_destroy_handler [list gid_groups_conds::_matrix_func_destroy_handler $w n_$n]

        lassign "" nameList valueList
        foreach fvarNode [[$w give_uservar_value n_$n] selectNodes functionVariable] {
            lappend nameList [$fvarNode @pn]
            set txt [$fvarNode @pn]
            set vline [list $txt]
            foreach vNode [$fvarNode selectNodes value] {
                lappend vline "[$vNode @pn]: [$vNode @v]"
            }
            if { [llength $vline] > 10 } {
                set vline [lrange $vline 0 9]
                lappend vline "..."
            }
            lappend valueList [join $vline \n]
        }
        if { $nameList ne "" } {
            $entry state !disabled
            $entry delete 0 end
            $entry insert end "f([join $nameList ,])..."
            $entry state disabled
            set h [lrange [split [gid_groups_conds::register_popup_help $entry] \n] 0 0]
            lappend h {*}$valueList
            gid_groups_conds::register_popup_help $entry [join $h \n]
        }
    } else {
        set nfuncs [llength $funcs]
        if { [lsearch $funcs scalar] == -1 } {
            set state disabled
        } else {
            set state !disabled
            incr nfuncs -1
        }
        set h [gid_groups_conds::register_popup_help $button]
        if { $nfuncs == 0 } {
            $button state disabled
            if { [llength [split $h \n]] == 1 } {
                append h "\nThere are no functions to apply in this configuration"
            }
        } else {
            $button state !disabled
            if { [llength [split $h \n]] == 2 } {
                set h [lindex [split $h \n] 0]
            }
        }
        gid_groups_conds::register_popup_help $button $h

        $entry state $state
#         $entry delete 0 end
#         $entry insert end $value
        set h [lindex [split [gid_groups_conds::register_popup_help $entry] \n] 0]
        gid_groups_conds::register_popup_help $entry $h
    }
}

proc gid_groups_conds::_matrix_func_contextual { w widget has_func funcs n fname domNode x y } {

    focus $widget
    destroy $w.vmenu
    set m [menu $w.vmenu -tearoff 0]

    foreach i [list Cut Copy Paste] j [list [_ "Cut"] [_ "Copy"] [_ "Paste"]] {
        $m add command -label $j -command [list event generate $widget <<$i>>]
    }
    $m add separator
    $m add command -label [_ "Select all"] -command [list $widget selection range 0 end]

    if { $has_func } {
        $m add separator
        $m add command -label [_ "Function editor"] -command \
            [list gid_groups_conds::function_editor \
                -funcs $funcs \
                $w [_ "Create function for %s" $fname] \
                $domNode [$w give_uservar n_$n]]
    }
    tk_popup $m $x $y
}


