
package require dialogwin
package require snit
package require autoscroll

if { [catch { package require gid_themes } err] } {
    set exists_gid_themes 0
} else {
    set exists_gid_themes 1
}

# to be inserted in tclIndex
proc groups_theme {} {}

# GiD change
namespace eval groups_theme {
    variable theme  ""
}

proc groups_theme::GetThemedImage { img_name } {
    variable theme
    if { $::exists_gid_themes && ( $theme == "")} {
        set theme [gid_groups_conds::GetCurrentTheme]
    }
    if { [regexp "Tdyn" $theme {} curr_theme] } {
        set img $img_name
    } elseif { [info exists gid_groups_conds::images(${img_name}-16)] } {
        set img $gid_groups_conds::images(${img_name}-16)
    } else {
        set img [gid_themes::GetImage $img_name]
    }
    return $img
}

proc groups_theme::GetAlternativeThemedImage { previous_image new_img_name {sub_dir ""} } {
    variable theme
    if { $::exists_gid_themes && ( $theme == "")} {
        set theme [gid_groups_conds::GetCurrentTheme]
    }
    
    set img $previous_image
    if { $::exists_gid_themes } {
        if { [regexp "Tdyn" $theme {} curr_theme] } {
            set themedir $::GidPriv(CurrentTheme,path,0,generic)
            set file [file join $themedir $sub_dir ${new_img_name}.png]
            image create photo ${new_img_name} -file $file
            set img ${new_img_name}
        }
    }
    return $img
}

# end GiD change

snit::widget gid_groups_theme {
    option -domdoc ""
    # menu or toplevel
    option -winstyle
    option -open_over ""   
    option -layers 0
    option -groups 0

#     variable searchframe_opener
    variable searchframe
    variable searchcombo
    variable last_searchstring
    variable searchstring

    variable viewtypemenubutton
    variable viewtypemenu
    variable viewtypevar viewall

    variable tree
    variable buttonbox    
    
    variable group_types
    variable SortColumn   
    variable searchstring 
    variable is_check_star 0     

    delegate option -frame_bg as -background to hull
    
    constructor args {     
        $self configurelist $args
        if { $options(-groups) } {
            set bbox grbbox
        } elseif { $options(-layers) } {
            set bbox laybbox
        }
        
        set pwin [winfo parent $win] 
        catch { destroy $pwin.$bbox }       
        install buttonbox as ttk::frame $pwin.$bbox ;#-background [$win cget -frame_bg]  
                  
        upvar 0 ::ttk::theme::[gid_themes::GetTtkTheme]::colors colors
        $pwin.$bbox configure -style Mod.ForcedFrame
        
        set theme ""
        if { $::exists_gid_themes } {
            set theme [gid_groups_conds::GetCurrentTheme]
            if { [regexp "Tdyn" $theme {} curr_theme] } {
                set gid_groups_conds::view_entities 1
                set themedir $::GidPriv(CurrentTheme,path,0,generic)                 
                if { $options(-groups) } {             
                    foreach i [list filenew remove up select removecomment \
                            grouplayer colorize colorize_end view_detailed \
                            layertogroup grouptolayer] {
                        set file [file join $themedir $i.png]
                        image create photo $i -file $file
                    }                                             
                } elseif {$options(-layers)} {
                    foreach i [list filenew grouplayer newlayer deletelayer \
                            selectent outfromlayer layertouse \
                            switch_on switch_off lock unlock trans opaque \
                            remove up select removecomment \
                            grouplayer colorize colorize_end view_detailed \
                            backlayer back check_black layertogroup grouptolayer] {
                        set file [file join $themedir $i.png]
                        image create photo $i -file $file
                    }
                }
                foreach i [list layertouse \
                        switch_on switch_off lock unlock trans opaque check check_star \
                        check_star-16 check-16 check_button checkplus_end-16 checkplus_draw-16] {
                    set file [file join $themedir $i.png]
                    set $i [image create photo $i -file $file]
                }
            } else {
                return
            }
        }        
     
        if { $options(-groups) } {
            set cmds [list \
                    [list filenew [_ "New group"] mbb [mymethod add_group] \
                        [list \
                            [list filenew [_ "New group"] "" [mymethod add_group]] \
                            [list filenew [_ "New child group"] "" [mymethod add_group -child]] \
                            ] \
                        ] \
                    [list remove [_ "Delete group"] "" [mymethod delete_group]] \
                    [list up [_ "Move groups position"] mb [mymethod move_menu %W]] \
                    [list select [_ "Send entities to group"] mb [mymethod send_entities in %W]] \
                    [list removecomment [_ "Take entities out from group"] mb [mymethod send_entities out %W]] \
                    [list layertogroup [_ "Layers from groups"] "" [mymethod groups_to_layers]] \
                    [list colorize [_ "Draw groups by color"] mb [mymethod draw_selected_groups %W]] \
                    [list check_button [_ "Show all groups"] mbb [mymethod update_groups_show_hide show] \
                        [list \
                            [list "filenew" [_ "Show all groups"] "" [mymethod update_groups_show_hide show]] \
                            [list "filenew" [_ "Hide all groups"] "" [mymethod update_groups_show_hide hide]] \
                            ] \
                        ] \
                    [list view_detailed [_ "List groups"] "" [mymethod list_entities]] \
                    ]
        } elseif {$options(-layers)} { 
            set gid_version [GiD_Info GiDVersion]
            if { [regexp {(.*)d} $gid_version {} version_num] } {
                set version_num $version_num
            } else {
                set version_num [lindex [split $gid_version .] 0] 
            }
            if { $version_num >= "14.1.9" } { 
                set ::GidPriv(LayersAlsoLower) [GiD_Set LayersAlsoLower]
            } else {
                if { ![info exists ::GidPriv(LayersAlsoLower)] } {
                    set ::GidPriv(LayersAlsoLower) 1
                }     
            }     
           
            if { ![info exists ::GidPriv(LayersBackOpposite)] } {
                set ::GidPriv(LayersBackOpposite) 0
            }
            set cmds [list \
                    [list newlayer [_ "New layer"] mbb [mymethod add_lay ""] \
                        [list \
                            [list newlayer [_ "New layer"] "" [mymethod add_lay ""]] \
                            [list newlayer [_ "New child layer"] "" [mymethod add_lay -child ""]] \
                            ] \
                        ] \
                    [list deletelayer [_ "Delete layer"] "" [mymethod delete_layer]] \
                    [list up [_ "Move layers position"] \
                        mb [mymethod move_menu %W]] \
                    [list selectent [_ "Send entities to layer"] mb [mymethod send_entities in %W]] \
                    [list backlayer [_ "Send entities to the back of its layer"] mb [mymethod send_entities out %W]] \
                    [list layertouse [_ "Select an entity and set its layer as layer to use"] \
                        mb [mymethod layers_to_use %W]] \
                    [list grouptolayer [_ "Groups from layers"] "" [mymethod selected_layers_to_groups]] \
                    [list view_detailed [_ "List layer entities"] \
                        "" [mymethod list_entities_lay]] \
                    ]
        }           
        
        set col 0
        foreach i $cmds {
            lassign $i img0 help type cmd cmdList
            if { $img0 eq "---" } {
                set b [ttk::separator $pwin.$bbox.f$col -orient vertical]
            } else {
                set img [groups_theme::GetThemedImage $img0]
                switch $type {
                    "" - mb {
                        set b [ttk::button $pwin.$bbox.f$col -image $img \
                        -style ToolbuttonMod]
                        if { $type eq "mb" } {
                            cu::add_down_arrow_to_image -color black -w $b $img
                        }
                    }
                    mbb {
                        set b [cu::menubutton_button $pwin.$bbox.f$col -image $img \
                        -menu $pwin.$bbox.f$col.m -style ToolbuttonMod]                        
                    }
                }
                set cmd [string map [list %W [list $b]] $cmd]
                switch $type {
                    "" {
                        $b configure -command $cmd
                    }
                    mb {
                        bind $b <ButtonPress-1> "$cmd ; break"
                        bind $b <Return> "$cmd ; break"
                        bind $b <space> "$cmd ; break"
                    }
                    mbb {
                        $b configure -command $cmd
                        menu $b.m -tearoff 0 
                        foreach i $cmdList {
                            lassign $i img0_i help_i type_i cmd_i
                            $b.m add command -label $help_i -command $cmd_i
                            if { $img0_i ne "" } {                               
                                # GiD change: only next line
                                set img_i [ groups_theme::GetThemedImage $img0_i]
                                $b.m entryconfigure end -image $img_i -compound left
                            }
                        }
                    }
                }
                cu::tooltip register $b "$help"
            }
            grid $b -row 0 -column $col -sticky nsw -padx 5 -pady "0 3"            
            if { $img0 eq "---" } { grid configure $b -sticky nsw -pady "0 3" }                     
            incr col
        }
             
        grid columnconfigure $pwin.$bbox "$col" -weight 1 
        
        grid $buttonbox -sticky nwe

        grid columnconfigure $win 0 -weight 1
        grid rowconfigure $win 3 -weight 1
                
#         gid_groups_conds::update_toolbar_more_options_button $pwin $cmds $options(-layers) \
#             $options(-groups)
#         bind $pwin <Configure> [list gid_groups_conds::update_toolbar_more_options_button \
#                 $pwin $cmds $options(-layers) $options(-groups)]
                       
        set parent [winfo parent $pwin]
                             
        ttk::frame $win.ft -style Mod.ForcedFrame
        
        if {$options(-groups)} {
            set columns [list \
                    [list 10 [_ "Groups"] left text 1] \
                    [list 10 [_ "Visible"] left image 1] \
                    [list 10 [_ "Type"] left text 1] \
                    ]
            set tree_gr_or_lay $win.ft.tg                      
        } elseif {$options(-layers)} {
            # Layers::ChangeLayers_New
            set columns [list \
                    [list 12 [_ "Name"] left text 1] \
                    [list 4 [_ "Layer to use"] left text 1] \
                    [list 4 [_ "Visible"] left text 1] \
                    [list 4 [_ "Freeze/Unfreeze"] left text 1] \
                    [list 4 [_ "Transp./Opaque"] left text 1] \
                    [list 4 [_ "Back"] left text 1]
                    ]           
            set tree_gr_or_lay $win.ft.tl
            set coltxt ""
            foreach i $columns {
                lassign $i col iname -
                lappend coltxt $iname
            }         
        }
        
        # Height depends on the entry combobox size
        if { [info command ::GiD_Process] eq "" } {
            set cbt [ttk::entry $win.size]
        } else {
            set cbt [ttk::entry $win.size -style Padding0.TEntry]
        }       
        set height [expr {[winfo reqheight $cbt]-1}]
        set h [winfo reqheight $parent.gg]
        set h1 [winfo height $parent.gg]

        package require fulltktree 
            
        install tree as fulltktree $tree_gr_or_lay \
            -returnhandler "[mymethod add_group];#" \
            -selecthandler2 [mymethod press2] \
            -contextualhandler [mymethod contextual_menu_gr_lay] \
            -editbeginhandler [mymethod rename_start] \
            -editaccepthandler [mymethod rename_end] \
            -deletehandler "[mymethod delete_gr_lay];#" \
            -draghandler [mymethod drag_and_drop] \
            -columns $columns -expand 1 \
            -selectmode extended -font SystemFont -showheader 1 \
            -indent 2 -showlines 0 -showroot 0 -showbutton 0 -showrootbutton 0 \
            -xscrollincrement 20 \
            -font SystemFont -yscrollincrement 20 \
            -highlightthickness 0 -folder_image "" \
            -scrollmargin 16 -itemheight $height -height [expr {$h1-80}] \
            -xscrolldelay "500 50" -yscrolldelay "500 50" \
            -doublebuffer window -width 300 -borderwidth 0         
                
        
#         install vscrollbar as ttk::scrollbar $tree_gr_or_lay.sv -orient vertical -command [list $tree yview]
#         install hscrollbar as ttk::scrollbar $tree_gr_or_lay.sh -orient horizontal -command [list $tree xview]
#         
#         install tree as treectrl $win.ft.t -highlightthickness 0 \
#             -xscrollincrement 20 -showheader 0 -indent 2 \
#             -font SystemFont -itemheight $height -selectmode extended -showroot no \
#             -showrootbutton no -showbuttons yes -showlines $showlines \
#             -scrollmargin 16 -xscrolldelay "500 50" -yscrolldelay "500 50" \
#             -width 300 -borderwidth 0 -relief groove -doublebuffer window \
#             -yscrollincrement $height
#            
#         -height $h
#         -itemwidth $width -width $wid               
        
        if {$options(-groups)} {
            bind $tree <ButtonPress-1> [mymethod press1_gr %x %y]
        } else {
            bind $tree <ButtonPress-1> [mymethod press1_lay %x %y]
            bind $tree <Double-Button-1> [mymethod double_click_table %x %y]
        }
#         bind $tree <F2> [mymethod rename_gr_lay_start]
        
        bind $tree <Key-Tab> [mymethod focus_buttons]
        bind $tree <Tab> [mymethod focus_buttons]
        
        bind $tree <Configure> [mymethod height_table_idle $parent]
        
        $tree element create e_col_rect rect -fill "" -outline black \
            -width 15 -height 6 -outlinewidth 1
        
        set S [$tree style create s_color_text -orient horizontal]
        $tree style elements $S [list e_rect e_col_rect e_text_sel \
                e_selmarker_up e_selmarker_down e_image]
        $tree style layout $S e_col_rect -padx {4 0} -expand ns
        $tree style layout $S e_text_sel -padx {4 0} -squeeze x -expand ns \
            -iexpand ns
        if { [package vcompare [package present treectrl] 2.1] >= 0 } {
            $tree style layout $S e_text_sel -iexpand nsx -sticky w
        }
        $tree style layout $S e_rect -union [list e_col_rect e_text_sel] -iexpand nswe -ipadx 2
        
        $tree style layout $S e_selmarker_up -detach 1           
        $tree style layout $S e_selmarker_up -width 100 -height 2 -sticky nw
        $tree style layout $S e_selmarker_down -detach 1 -expand n
        $tree style layout $S e_selmarker_down -width 100 -height 2 -sticky sw
        $tree style layout $S e_image -detach 1 -padx {2 0}
        
        set t1 [$tree givetreectrl]          
        set T $t1    
               
        if {$options(-layers)} {      
            set editable [list {0 text e_text_sel} {1 image e_image} {2 image e_image} \
                    {3 image e_image} {4 image e_image} {5 image e_image}]                       
            set dragimage [list {0 text e_text_sel} {1 image e_image} {2 image e_image} \
                    {3 image e_image} {4 image e_image} {5 image e_image}]           
            set sensitive [list {0 text e_text_sel} {1 image e_image} {2 image e_image} \
                    {3 image e_image} {4 image e_image} {5 image e_image}]
            
            lappend editable [list 0 $S e_text_sel]
            lappend sensitive [list 0 $S e_text_sel e_col_rect]
            lappend dragimage [list 0 $S e_text_sel e_col_rect]                      
                                    
            TreeCtrl::SetEditable $t1 $editable
            TreeCtrl::SetDragImage $t1 $dragimage
            TreeCtrl::SetSensitive $t1 $sensitive
            
            
        } else {
            set editable [TreeCtrl::SetEditable $t1]
            set dragimage [TreeCtrl::SetDragImage $t1]
            set sensitive [TreeCtrl::SetSensitive $t1]                        
            
            lappend editable [list 0 $S e_text_sel]
            lappend sensitive [list 0 $S e_text_sel e_col_rect]
            lappend dragimage [list 0 $S e_text_sel e_col_rect]
                        
            TreeCtrl::SetEditable $t1 $editable
            TreeCtrl::SetDragImage $t1 $dragimage
            TreeCtrl::SetSensitive $t1 $sensitive
        }                              
        if { $::exists_gid_themes } {
            set theme [gid_groups_conds::GetCurrentTheme]
            if { [regexp "Tdyn" $theme {} curr_theme] } {              
                set style header5
                
                upvar ::ttk::theme::[gid_themes::GetTtkTheme]::colors colors
                upvar ::ttk::theme::[gid_themes::GetTtkTheme]::fonts fonts
                
                $T element create header.text text -statedomain header \
                    -lines 1 -fill black -font $fonts(-default)
                
                gid_groups_conds::create_customized_theme
                
                $T element create header.sort image -statedomain header \
                    -image [list mac-collapse down mac-up up]
                
                $T gradient create Gnormal -orient vertical \
                    -stops [list [list 0.0 $colors(-toolbars_bg)] [list 0.5 $colors(-toolbars_bg)] [list 1.0 $colors(-toolbars_bg)]] \
                    -steps 6
                
                $T gradient create Gactive -orient vertical \
                    -stops [list [list 0.0 $colors(-highlight)] [list 0.5 $colors(-highlight)] [list 1.0 $colors(-highlight)]] \
                    -steps 6
                
                $T gradient create Gpressed -orient vertical \
                    -stops [list [list 0.0 $colors(-highlight)] [list 0.5 $colors(-highlight)] [list 1.0 $colors(-highlight)]] \
                    -steps 6
                
                $T gradient create Gsorted -orient vertical \
                    -stops [list [list 0.0 $colors(-toolbars_bg)] [list 0.5 $colors(-toolbars_bg)] [list 1.0 $colors(-toolbars_bg)]] \
                    -steps 6
                
                $T gradient create Gactive_sorted -orient vertical \
                    -stops [list [list 0.0 $colors(-highlight)] [list 0.5 $colors(-highlight)] [list 1.0 $colors(-highlight)]] \
                    -steps 6
                
                $T gradient create Gpressed_sorted -orient vertical \
                    -stops [list [list 0.0 $colors(-toolbars_bg)] [list 0.5 $colors(-toolbars_bg)] [list 1.0 $colors(-toolbars_bg)]] \
                    -steps 6
                
                $T element create header.rect5 rect -statedomain header \
                    -fill [list \
                        Gactive_sorted {active up} \
                        Gpressed_sorted {pressed up} \
                        Gactive_sorted {active down} \
                        Gpressed_sorted {pressed down} \
                        Gsorted up \
                        Gsorted down \
                        Gactive active \
                        Gpressed pressed \
                        Gnormal {} ] \
                    -outline [list \
                        $colors(-highlight) up \
                        $colors(-highlight) down \
                        $colors(-frame_bg) {} ] \
                    -outlinewidth 1 -open { nw !pressed }
                
                set S [$T style create $style -orient horizontal -statedomain header]
                $T style elements $S {header.rect5 header.text header.sort}
                $T style layout $S header.rect5 -detach yes -iexpand xy
                $T style layout $S header.text -center xy -padx 6 -pady 2 -squeeze x
                $T style layout $S header.sort -expand nws -padx {0 6} -visible {no {!down !up}}
                
                set textColor $colors(-select_fg)
                $T element configure header.text -fill $textColor
                
                set style header5       
                
                foreach H [$T header id !header4] {
                    $T header style set $H all $style
                    $T header configure all all -textpadx 6
                    foreach C [$T column list] {
                        $T header text $H $C [$T header cget $H $C -text]
                    }        
                }    
            } else {
                gid_themes::ConfigureHeaders $t1
                gid_themes::ApplyHeadersConfig $t1
            }
        }
        if {$options(-groups)} {           
            $tree column configure 1 -width 50
            $tree column configure 2 -width 70            
            bind [$tree givetreectrl] <F1> [mymethod groups_info]
        } 
        grid columnconfigure $win 0 -weight 1
        grid rowconfigure $win 0 -weight 1
        
        grid $win.ft -sticky news
        
        grid columnconfigure $win.ft 0 -weight 1
        grid rowconfigure $win.ft 0 -weight 1
                       
        grid $tree $tree.sv -sticky nsew -padx 1 -pady 1
        grid $tree.sh -sticky ew -row 3 -column 0
        grid configure $tree.sv -row 0 -column 1 
                        
        $tree configure -background #dbdcde               
        grid configure $tree -pady "2 0"
        
        autoscroll::autoscroll $tree.sv
        autoscroll::autoscroll $tree.sh
       
        $self configurelist $args            
        
        set gr_lay $options(-groups)$options(-layers)        
      
        set wgid .gid.bitmapsStdBarMod
        if { [winfo exists $wgid] } { 
            catch { destroy .gid.bitmapsStdBarMod.search$gr_lay }            
            install searchcombo as ttk::combobox .gid.bitmapsStdBarMod.search$gr_lay \
                -textvariable [varname searchstring] -width 37 -style WNRsearch.entry                            
            set v [_ "Search"]
            set help [_ "Search quickly in data tree"]
            cu::tooltip register .gid.bitmapsStdBarMod.search$gr_lay "<b>$v</b>\n$help"                                         
            set searchstring ""
            set last_searchstring ""        
            update idletasks                
            
            set update_stdbar "[mymethod update_stdbarmod_from_gr_or_lay $wgid $gr_lay]"    
            bind .gid.bitmapsStdBarMod <Configure> "+[mymethod update_stdbarmod_from_gr_or_lay $wgid $gr_lay]"
            eval $update_stdbar
            
            bind .gid.bitmapsStdBarMod.search$gr_lay \
                <Return> "+[mymethod search_tree full] ;break"
            set cmdT "[mymethod search_tree key] ;#"
            trace add variable [varname searchstring] write $cmdT
            bind .gid.bitmapsStdBarMod.search$gr_lay \
                <Destroy> [list trace remove variable \
                    [varname searchstring] write $cmdT]
        }        
    }
    # end of constructor ( <-- GiD change)
    
    onconfigure -domdoc {value} {
        set options(-domdoc) $value      
        if {![winfo exists $win.ft.tg] && ![winfo exists $win.ft.tl]} { return }
        if {[winfo exists $win.ft.tl]} {
            $self actualize_layers
        } 
        if {[winfo exists $win.ft.tg]} {
            $self actualize_groups
        }   
        if { [$tree numitems] > 1 } {
            $tree activate 1
            $tree selection add 1
            $tree selection anchor 1
        }
        focus $tree
    } 
    onconfigure -groups {value} {
        set options(-groups) $value               
    }    
    onconfigure -layers {value} {
        set options(-layers) $value            
    }    
    method update_stdbarmod_from_gr_or_lay { bw gr_lay } {   
        if { [gid_groups_conds::isdrawpost] } { return }        
        if { [info exists gid_groups_conds::menu] } {
            if { $gid_groups_conds::menu eq "data_tree"} { 
                return 
            }       
        }
        # Window resize         
        set ir 0
        set ic 18
        
        grid $bw.search$gr_lay -in $bw.f$ir -row 0 -column $ic -padx 30 -pady {0 3} -sticky ens
        grid columnconfig $bw.f$ir $ic -weight 1           
    }
    variable idleh ""
    method height_table_idle { parent } {
        
        if {[info exists idleh]} {
            after cancel $idleh
            unset idleh
        }
        set idleh [after idle  [list catch [mymethod height_table $parent]]] 

        set wgid .gid.bitmapsStdBarMod
        set gr_lay $options(-groups)$options(-layers)
        $self update_stdbarmod_from_gr_or_lay $wgid $gr_lay
    }
    
    method focus_buttons {} {
#         focus .gid.central.boundaryconds.gg.lt.bbox.f0
#         focus -force [$tree givetreectrl]
        focus $tree
    }
    method height_table { parent } {
        set h [expr {[winfo height $parent] - 70}]
        $tree configure -height $h
        $tree configure -backgroundimage ""
    }
    method close_groups_win {} {
        destroy $win
    }
    method close_layers_win {} {
        destroy $win
    }
#     method toggle_search {} {
#         if { [grid info $searchframe] eq "" } {
#             grid $searchframe
#             focus $searchcombo
# #             grid remove $searchframe_opener
#             $win.f0.search configure -relief sunken
#         } else {
#             grid remove $searchframe
# #             grid $searchframe_opener
#             $win.f0.search configure -relief flat
#         }
#     }
    variable search_tree_after ""
    # what can be: full and key
    method search_tree { what } {      
        after cancel $search_tree_after
        set search_tree_after [after 200 [mymethod search_tree_do $what]]
    }
    method search_tree_do { what } {
        after cancel $search_tree_after
        set search_tree_after ""

        set str [string trim $searchstring]

        set isexpanded 0
        foreach i [$tree item children root] {
            set isex [$self search_expand $i [gid_groups_conds::fcompare_str \
                        $searchstring]]
            if { $isex } { set isexpanded 1 }
        }
        set last_searchstring $str

        if { $str eq "" } { return }
        
        set values [$searchcombo cget -values]

        if { [set ipos [lsearch -exact $values $str]] != -1 } {
            set values [lreplace $values $ipos $ipos]
            set replaced 1
        } else { set replaced 0 }

        if { $what eq "full" || $replaced } {
            set values [linsert $values 0 $str]
        } elseif { $isexpanded && [string first $str [lindex $values 0]] == -1} {
            while { [string first [lindex $values 0] $str] != -1 } {
                set values [lrange $values 1 end]
            }
            set values [linsert $values 0 $str]
        }
        set values [lrange $values 0 14]
        $searchcombo configure -values $values
    }
    method search_expand { id text } {
        set txt [gid_groups_conds::fcompare_str [$tree item text $id 0]]
        
        set visible 0
        foreach id_child [$tree item children $id] {
            if { [$self search_expand $id_child $text] } {
                set visible 1
            }
        }
        if { $visible || $text eq "" || [string match *$text* $txt] } {
            set expand 1
            $tree item configure $id -visible 1
        } else {
            set expand 0
            $tree item configure $id -visible 0
        }
        return $expand
    }
    method give_short_gr_lay_name { item } {
        return [$tree item text $item 0]
    }
    method give_full_group_name { item } {
        set nameList ""
        while { $item != 0 } {
            set nameList [linsert $nameList 0 [$tree item text $item 0]]
            set item [$tree item parent $item]
        }
        set group [join $nameList //]
        set root [$options(-domdoc) documentElement]
        set xp [format_xpath {groups/group[@n=%s]} $group]
        if { [$root selectNodes $xp] eq "" } {
            set group ""
        }
        return $group
    }
    method give_full_layer_name { item } {
        set nameList ""
        while { $item != 0 } {
            set nameList [linsert $nameList 0 [$tree item text $item 0]]
            set item [$tree item parent $item]
        }
        set layer [join $nameList //]       
        return $layer
    }
    method give_full_group_name_list { itemList } {
        set ret ""
        foreach item $itemList {
            lappend ret [list $item [$self give_full_group_name $item]]
        }
        return $ret
    }
    method give_full_layer_name_list { itemList } {
        set ret ""
        foreach item $itemList {
            lappend ret [list $item [$self give_full_layer_name $item]]
        }
        return $ret
    }
    method _give_tree_descendants { item } {
        set ret [$tree item children $item]
        foreach i $ret {
            lappend ret {*}[$self _give_tree_descendants $i]
        }
        return [lsort -integer -unique $ret]
    }
    method give_full_group_name_and_children { itemList } {
        set ret ""
        foreach item $itemList {
            lappend itemList {*}[$self _give_tree_descendants $item]
        }
        set itemList [lsort -integer -unique $itemList]
        foreach item $itemList {
            set name [$self give_full_group_name $item]
            lappend ret [list $item $name]
        }
        return $ret
    }
    method give_full_layer_name_and_children { itemList } {   
        set ret ""
        foreach item $itemList {
            lappend itemList {*}[$self _give_tree_descendants $item]
        }
        set itemList [lsort -integer -unique $itemList]
        foreach item $itemList {
            set name [$self give_full_layer_name $item]
            lappend ret [list $item $name]
        }
        return $ret
    }
    
    method actualize_groups {} {       
        if { $options(-domdoc) ne "" } {
            set root [$options(-domdoc) documentElement]
#             set vs [$root selectNodes string(display_options/@view_groups_search)]
#             if { $vs != 1 } { $self toggle_search }
#             set v [$root selectNodes string(display_options/@groups_search_values)]
#             $searchcombo configure -values [split $v ,]

#             $viewtypemenu delete 0 end
#             $viewtypemenu add radiobutton -label [_ "View all"] -variable \
#                 [varname viewtypevar] -value "viewall" -command \
#                 [mymethod actualize]
#             $viewtypemenu add separator
#             foreach node [$root selectNodes groups_types/group_type] {
#                 $viewtypemenu add radiobutton -label [$node @pn] -variable \
#                     [varname viewtypevar] -value [$node @pn] -command \
#                     [mymethod actualize]
#             }
#             if { $viewtypevar eq "viewall" } {
#                 $viewtypemenubutton state !alternate
#             } else {
#                 $viewtypemenubutton state alternate
#             }
        }

        set xview [lindex [$tree xview] 0]
        set yview [lindex [$tree yview] 0]

        $self store_tree_state all 0

        $tree item delete all

        if { $options(-domdoc) eq "" } { return }
        
        set selected ""
        foreach id [$tree selection get] {
            lappend selected [$self give_full_group_name $id]
        }
        if { [$tree index active] != 0 } {
            set active [$self give_full_group_name active]
        } else {
            set active ""
        }
        set root [$options(-domdoc) documentElement]
        set group_types ""
        foreach node [$root selectNodes groups_types/group_type] {
            lappend group_types [$node @pn]
        }
        set nodes [$root selectNodes groups/group]
#         if { ![llength $nodes] } {
#             set groupsNode [$root selectNodes groups]
#             set id [$groupsNode appendChildTag group]
#             $id setAttribute n [_ group]1 onoff 1 color red type \
#                 [lindex $group_types 0]
#             set nodes [$root selectNodes groups/group]
#         }
        
        lassign [list "" 0 1] ids level has_more_level
        set ids ""
        set level 0
        while { $has_more_level } {
            set has_more_level 0
            foreach domNode $nodes {
                if { $viewtypevar ne "viewall" && $viewtypevar ne [$domNode @type] } {
                    continue
                }
                set nameList [cu::splitx [$domNode @n] {//}]
                set parentL [join [lrange $nameList 0 $level-1] //]
                if { $parentL eq "" } {
                    set parent 0
                } elseif { [dict exists $ids $parentL] } {
                    set parent [dict get $ids $parentL]
                } else {
                    set parentLL [join [lrange $nameList 0 $level-2] //]
                    if { $parentLL eq "" } {
                        set parent 0
                    } else {
                        set parent [dict get $ids $parentLL]
                    }
                    set parent [$self add $parent [lindex $nameList $level-1] [$domNode @onoff 1] \
                            [$domNode @color black] [$domNode @type]]
                    $tree item collapse $parent
                    dict set ids $parentL $parent
                }
                if { [llength $nameList]-1 == $level } {
                    set id [$self add $parent [lindex $nameList end] [$domNode @onoff 1] \
                            [$domNode @color black] [$domNode @type]]
                    if { [lsearch -exact $selected [$domNode @n]] != -1 } {
                        $tree selection add $id
                    }
                    if { [$domNode @n] eq $active } {
                        $tree activate $id
                    }
                    $tree item collapse $id
                    dict set ids [$domNode @n] $id
                } elseif { [llength $nameList] >= $level } {
                    set has_more_level 1
                }
            }
            incr level   
        }
        $tree xview moveto $xview
        $tree yview moveto $yview        
    }
    method actualize_layers { } { 
        set ::GidPriv(updatelayers) 0
        set LayerNames [GiD_Info layers]
        set icount 1
        foreach ilay $LayerNames {                       
            $self add_lay -actualize_lay 1 -icount $icount -exist 1 $ilay            
            incr icount
        }
        set ::GidPriv(updatelayers) 1
        return
    }
    method actualize_layers_old {} { 
        set ::GidPriv(updatelayers) 0
        set LayerNames [GiD_Info layers]
        set icount 0
        foreach ilay $LayerNames {                       
            $self add_lay -actualize_lay 1 -exist 1 $ilay            
            incr icount
        }
        set ::GidPriv(updatelayers) 1
        return
        
#         if { $options(-domdoc) ne "" } {
#             set root [$options(-domdoc) documentElement]
#             set vs [$root selectNodes string(display_options/@view_groups_search)]
#             if { $vs != 1 } { $self toggle_search }
#             set v [$root selectNodes string(display_options/@groups_search_values)]
#             $searchcombo configure -values [split $v ,]

#             $viewtypemenu delete 0 end
#             $viewtypemenu add radiobutton -label [_ "View all"] -variable \
#                 [varname viewtypevar] -value "viewall" -command \
#                 [mymethod actualize]
#             $viewtypemenu add separator
#             foreach node [$root selectNodes groups_types/group_type] {
#                 $viewtypemenu add radiobutton -label [$node @pn] -variable \
#                     [varname viewtypevar] -value [$node @pn] -command \
#                     [mymethod actualize]
#             }
#             if { $viewtypevar eq "viewall" } {
#                 $viewtypemenubutton state !alternate
#             } else {
#                 $viewtypemenubutton state alternate
#             }
#         }            

        set xview [lindex [$tree xview] 0]
        set yview [lindex [$tree yview] 0]

        $self store_tree_state all 0

        $tree item delete all
        
        set selected ""
        foreach id [$tree selection get] {
            lappend selected [$self give_full_layer_name $id]
        }
        if { [$tree index active] != 0 } {
            set active [$self give_full_layer_name active]
        } else {
            set active ""
        }
#         set root [$options(-domdoc) documentElement]
#         set group_types ""
#         foreach node [$root selectNodes groups_types/group_type] {
#             lappend group_types [$node @pn]
#         }
#         set nodes [$root selectNodes groups/group]
#         if { ![llength $nodes] } {
#             set groupsNode [$root selectNodes groups]
#             set id [$groupsNode appendChildTag group]
#             $id setAttribute n [_ group]1 onoff 1 color red type \
#                 [lindex $group_types 0]
#             set nodes [$root selectNodes groups/group]
#         }
        
        lassign [list "" 0 1] ids level has_more_level
        set ids ""
        set level 0
        while { $has_more_level } {
            set has_more_level 0
            set LayerNames [GiD_Info layers]
            foreach ilay $LayerNames {
#                 if { $viewtypevar ne "viewall" && $viewtypevar ne [$domNode @type] } {
#                     continue
#                 }
                set nameList [cu::splitx $ilay {//}]
                set parentL [join [lrange $nameList 0 $level-1] //]
                if { $parentL eq "" } {
                    set parent 0
                } elseif { [dict exists $ids $parentL] } {
                    set parent [dict get $ids $parentL]
                } else {
                    set parentLL [join [lrange $nameList 0 $level-2] //]
                    if { $parentLL eq "" } {
                        set parent 0
                    } else {
                        set parent [dict get $ids $parentLL]
                    }                                                           
                    set LayersProp [lindex [GiD_Info layers $ilay] 0]
                    set color [format "#%02x%02x%02x" [lindex $LayersProp 2] \
                            [lindex $LayersProp 3] [lindex $LayersProp 4]]
                    
                    set lay_to_use 1
                    set on_off on
                    set f_u u
                    set t_o o        
                    set parent [$self add_layer $parent [lindex $nameList $level-1] \
                            $lay_to_use $on_off $f_u $t_o $color]
                    $tree item collapse $parent
                    dict set ids $parentL $parent
                }
                if { [llength $nameList]-1 == $level } {
                    set LayersProp [lindex [GiD_Info layers $ilay] 0]
                    set color [format "#%02x%02x%02x" [lindex $LayersProp 2] \
                            [lindex $LayersProp 3] [lindex $LayersProp 4]]
                    
                    set lay_to_use 1
                    set on_off on
                    set f_u u
                    set t_o o
                    set id [$self add_layer $parent [lindex $nameList end] \
                            $lay_to_use $on_off $f_u $t_o $color]
                    if { [lsearch -exact $selected $ilay] != -1 } {
                        $tree selection add $id
                    }
                    if { $ilay eq $active } {
                        $tree activate $id
                    }
                    $tree item collapse $id
                    dict set ids $ilay $id
                } elseif { [llength $nameList] >= $level } {
                    set has_more_level 1
                }
            }
            incr level   
        }
        $tree xview moveto $xview
        $tree yview moveto $yview
    }
    # what: all display_options
    method store_tree_state { { what all } { id 0 } } {
        if { $options(-domdoc) eq "" } { return }
        
        if { $what eq "all" } {
            # all by now it is not used
        }
        set root [$options(-domdoc) documentElement]
        
        set dispNode [$root selectNodes display_options]
        if { $dispNode eq "" } {
            set dispNode [$root appendChildTag display_options]
        }
        # frame_width is is stored in the conditions tree

#         if { [grid info $searchframe] eq "" } {
#             $dispNode setAttribute view_groups_search 0
#         } else {
#             $dispNode setAttribute view_groups_search 1
#         }
        # if one search value has a comma: problems
#         $dispNode setAttribute groups_search_values \
#             [join [$searchcombo cget -values] ,]
    }

#     method hasfocus {} {
#         if { [focus -lastfor $tree] eq $tree } {
#             return 1
#         } else {
#             return 0
#         }
#     }
#     method focus {} {
#         focus $tree
#     }
    method givetreectrl {} {
        return [$tree givetreectrl]
    }
    proc _sort_by_images { tree id1 id2 } {
        set img1 [$tree item element cget $id1 1 e_image -image]
        set img2 [$tree item element cget $id2 1 e_image -image]

        if { $img1 ne "" } {
            if { $img2 ne "" } { return 0 }
            return 1
        } elseif { $img2 ne "" } {
            return -1
        } else { return 0 }
    }
    method headerinvoke { C } {
        if {$C == $SortColumn} {
            if {[$tree column cget $SortColumn -arrow] eq "down"} {
                set order -increasing
                set arrow up
            } else {
                set order -decreasing
                set arrow down
            }
        } else {
            set order -increasing
            set arrow up
            $self column configure $SortColumn -arrow none
            set SortColumn $C
        }
        $self column configure $C -arrow $arrow
        if { $C == 1 } {
            $self item sort root $order -command \
                [list _sort_by_images $tree]
        } else {
            $self item sort root $order -column $C -dictionary
        }
    }
    method contextual_menu_add_cmds { m cmdsvar } {
        upvar $cmdsvar cmds

        foreach "img txt cmd acc" $cmds {
            if { $img ne "" && ([info command $img] eq "" || $img ni [image names]) } {
                set theme ""
                if { $::exists_gid_themes } {
                    set theme [gid_groups_conds::GetCurrentTheme]                    
                }

                # GiD change: only next line
                set img [ groups_theme::GetThemedImage $img]
            }
            if { $txt eq "-" } {
                $m add separator
            } elseif { [lindex $cmd 0] eq "-" } {
                set i 1
                while { [winfo exists $m.m$i] } { incr i }
                $m add cascade -image $img -label $txt -compound left \
                    -menu $m.m$i
                menu $m.m$i -tearoff 0
                upvar [lindex $cmd 1] [lindex $cmd 1]
                $self contextual_menu_add_cmds $m.m$i [lindex $cmd 1]
            } else {
                $m add command -image $img -label $txt -compound left \
                    -command $cmd
                if { $acc ne "" } {
                    $m entryconfigure end -accelerator $acc
                }
            }
        }
    }
    method contextual_menu_gr_lay { - - x y } {     
        if { [$tree index active] != 0 } {           
            set id_active [$tree index active]
            $tree selection add $id_active
            $tree activate $id_active                        
        }
        
        if {$options(-groups)} {                        
            $self contextual_menu_gr - - $x $y
        } elseif {$options(-layers)} {                       
            $self contextual_menu_lay - - $x $y
        }
    }
    method contextual_menu_lay { - - x y } {       
        
        set infogr "queue"
        set changecol $infogr
        set selall ""
        set rename $infogr      
        # GiD change
        # if another theme is loaded:
        set infogr [ groups_theme::GetAlternativeThemedImage $infogr infogroup]
        set changecol [ groups_theme::GetAlternativeThemedImage $changecol colorgroup]
        set selall [ groups_theme::GetAlternativeThemedImage $selall selectall]
        set rename [ groups_theme::GetAlternativeThemedImage $rename renamegroup]
        set upbut "up"
        set upbut [ groups_theme::GetAlternativeThemedImage $upbut upb]      
        set downbut "down"    
        set downbut [groups_theme::GetAlternativeThemedImage $downbut down]
        set topbut "top"
        set topbut [groups_theme::GetAlternativeThemedImage $topbut top]
        set bottombut "bottom"
        set bottombut [groups_theme::GetAlternativeThemedImage $bottombut bottom]
    
        # end GiD change
        
        set cmds [list newlayer [_ "New"] [mymethod add_lay ""] "" \
                newlayer [_ "New child"] [mymethod add_lay -child ""] "" \
                "" - "" "" \
                deletelayer [_ "Delete"] [mymethod delete_layer] "" \
                $rename [_ "Rename"] [mymethod rename_gr_lay_start] ""]
        
        lappend cmds \
            $changecol [_ "Change color"] [mymethod change_color_layer] "" \
            "" - "" "" \
            up [_ "Move layers position"] "- cmds5" "" \
            "" - "" "" \
            view_detailed [_ "List layer entities"] [mymethod list_entities_lay] "" \
            "" - "" "" \
            selectent [_ "Send entities to layer"] "- cmds2" "" \
            backlayer [_ "Send entities to back"] "- cmds3" "" \
            "" - "" "" \
            grouptolayer [_ "Groups from layers"] [mymethod selected_layers_to_groups] "" \
            "" - "" "" \
            mac-expand [_ "Expand"] [mymethod collapse_expand_lay_gr expand] "" \
            mac-collapse [_ "Collapse"] [mymethod collapse_expand_lay_gr collapse] "" \
            mac-expand2 [_ "Expand all"] [mymethod collapse_expand_lay_gr expandall] "" \
            mac-collapse2 [_ "Collapse all"] [mymethod collapse_expand_lay_gr collapseall]                        
        
        switch [lindex [GiD_Info Project] 4] {
            GEOMETRYUSE {
                set ents [list [_ "Points"] point point [_ "Lines"] line line \
                        [_ "Surfaces"] surface surface [_ "Volumes"] volume volume]
            }
            MESHUSE {
                if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                    set ents [list [_ "Nodes"] node node [_ "Elements"] element element]
                } else {
                    set ents [list [_ "Nodes"] point node [_ "L Elements"] line element \
                            [_ "S Elements"] surface element \
                            [_ "V Elements"] volume element]
                }
            }
        }
        foreach "cmds2 cmds3 cmds4" [list "" "" ""] break               
        
        ### Send entities to layer     
        set gid_version [GiD_Info GiDVersion]
        if { [regexp {(.*)d} $gid_version {} version_num] } {
            set version_num $version_num
        } else {
            set version_num [lindex [split $gid_version .] 0] 
        }
        set img_lay_also ""
        if { $version_num >= "14.1.9" } {
            if { [GiD_Set LayersAlsoLower] } {
                set img_lay_also check_black
            } 
        } else {
            if { $::GidPriv(LayersAlsoLower) } {
                set img_lay_also check_black
            } 
        }
        
        set cmds2 ""
        lappend cmds2 $img_lay_also [_ "Also lower entities"] \
            [mymethod also_lower_entities] ""
        lappend cmds2 "" - "" "" 
        foreach "name n img" $ents {
            lappend cmds2 [gid_themes::GetImage $img small_icons] $name \
                [mymethod send_entities in "" $name $n] ""
        } 
        lappend cmds2 "" - "" ""
        set ids [$tree selection get]
        set gList [$self give_full_layer_name_list $ids]
        lappend cmds2 selectent [_ "Dimensions"] \
            [mymethod send_entities_do_layers in [_ "Dimensions"] Dimensions $gList] ""
               
        lappend cmds2 selectent [_ "All"] \
            [mymethod send_entities_do_layers in [_ "All"] All $gList] ""
                    
        # Send entities to back 
        set cmds3 ""
        lappend cmds3 $img_lay_also [_ "Also lower entities"] \
            [mymethod also_lower_entities] ""
        
        if { $::GidPriv(LayersBackOpposite) } {
            set img_opposite check_black
        } else {
            set img_opposite ""
        }
        lappend cmds3 $img_opposite [_ "Opposite"] \
            [mymethod opposite] "" 
        lappend cmds3 "" - "" ""
        foreach "name n img" $ents {
            lappend cmds3 [gid_themes::GetImage $img small_icons] $name \
                [mymethod send_entities out "" $name $n] ""
        }        
        lappend cmds3 "" - "" ""
        lappend cmds3 backlayer [_ "All"] \
            [mymethod send_entities_do_layers out [_ "All"] All $gList] ""                   
        
        if { [GiD_Info layers -hasbacklayers] } {
            lappend cmds3 "" - "" ""
            lappend cmds3 backlayer [_ "Bring to front"] \
                [mymethod send_layers_to] ""
            lappend cmds3 backlayer [_ "Bring all to front"] \
                [mymethod send_all_layers_to] ""                        
        } 
        
        ###
        foreach "name n img" $ents {
            lappend cmds4 [gid_themes::GetImage $img small_icons] $name \
                [mymethod draw_selected_groups "" $name $n] ""
        }
        set cmds5 [list \
                $upbut [_ "Move up"] [mymethod move_group up] "" \
                $downbut [_ "Move down"] [mymethod move_group down] "" \
                $topbut [_ "Move top"] [mymethod move_group top] "" \
                $bottombut [_ "Move bottom"] [mymethod move_group bottom] "" \
                ]
        set cmds6 [list \
                grouptolayer [_ "Groups from layers"]... [mymethod selected_layers_to_groups] "" \
                layertogroup [_ "Layers from groups"]... [mymethod groups_to_layers] "" \
                ]

        destroy $win.menu
        set m [menu $win.menu -tearoff 0]
        $self contextual_menu_add_cmds $m cmds
        cu::tooltip cancel $win
        tk_popup $m $x $y
    }
    method opposite {} {
        if { $::GidPriv(LayersBackOpposite) } {            
            set ::GidPriv(LayersBackOpposite) 0
        } else {
            set ::GidPriv(LayersBackOpposite) 1
        }                            
    }
    method also_lower_entities {} {
        set gid_version [GiD_Info GiDVersion]
        if { [regexp {(.*)d} $gid_version {} version_num] } {
            set version_num $version_num
        } else {
            set version_num [lindex [split $gid_version .] 0] 
        }
        if { $version_num >= "14.1.9" } {
            if { [GiD_Set LayersAlsoLower] } {
                GiD_Set LayersAlsoLower 0
            } else {
                GiD_Set LayersAlsoLower 1
            }
        } else {
            if { $::GidPriv(LayersAlsoLower) } {
                set ::GidPriv(LayersAlsoLower) 0
            } else {
                set ::GidPriv(LayersAlsoLower) 1
            }
        }
    }
    method list_entities_lay {} {
        set ids [$tree selection get]
        if { [llength $ids] == 0 || [llength $ids] > 1 } {           
            GidUtils::SetWarnLine [_ "It is necessary to select one layer"]           
            return
        }
        Layers::ListEntitiesSelection $tree
    }
    method collapse_expand_lay_gr { collapse_expand } {
        switch $collapse_expand {
            expand {
                if {[$tree selection get] eq ""} {
                    GidUtils::SetWarnLine [_ "Select at least one parent layer to expand"]                     
                    return
                }
                foreach id [$tree selection get] {
                    $tree item expand $id -recurse
                }
            }
            expandall {
                $tree item expand 0 -recurse
            }
            collapse {
                if {[$tree selection get] eq ""} {
                    GidUtils::SetWarnLine [_ "Select at least one parent layer to collapse"]                     
                    return
                }
                foreach id [$tree selection get] {
                    $tree item collapse $id -recurse
                }
            }
            collapseall {
                $tree item collapse 0 -recurse
            }
            collapseother {
                set id [lindex [$tree selection get] 0]
                set domNode [$tree item element cget $id 0 e_text -data]
                set item 0
                foreach node [lrange [$domNode selectNodes ancestor-or-self::*] 1 end] {
                    set state [get_domnode_attribute $node state]
                    if { $state eq "hidden_this" } { continue }
                    foreach i [$tree item children $item] {
                        set d [$tree item element cget $i 0 e_text -data]
                        if { $d ne $node } {
                            $tree item collapse $i
                        } else {
                            set item $i
                        }
                    }
                }
            }
            viewthis {
                foreach id [$tree selection get] {
                    $tree item expand $id
                }
                $self collapse_expand collapseother
            }
        }
    }
    method contextual_menu_gr { - - x y } {     
        
        set infogr "queue"
        set changecol $infogr
        set selall ""
        set rename $infogr
        set upbut "up"
        # GiD change
        # if another theme is loaded:
        set infogr [ groups_theme::GetAlternativeThemedImage $infogr infogroup]
        set changecol [ groups_theme::GetAlternativeThemedImage $changecol colorgroup]
        set selall [ groups_theme::GetAlternativeThemedImage $selall selectall]
        set rename [ groups_theme::GetAlternativeThemedImage $rename renamegroup]
        set upbut [ groups_theme::GetAlternativeThemedImage $upbut upb]
        # additional alternative images
        foreach alt_img [list down top bottom] {
            groups_theme::GetAlternativeThemedImage $alt_img $alt_img
        }    
        # end GiD change
        
        foreach alt_img [list select] {
            set alt_img_i [gid_groups_conds::giveimage ${alt_img}-16]
            set alt_img_i [groups_theme::GetAlternativeThemedImage $alt_img_i $alt_img]
            set $alt_img $alt_img_i
        }
        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {            
            set cmds [list filenew [_ "New"] [mymethod add_group] "" \
                    filenew [_ "New child"] [mymethod add_group -child] "" \
                    "" - "" "" \
                    remove [_ "Delete"] [mymethod delete_group] "" \
                    $select [_ "Copy with assigned entities"] [mymethod copy_group] "" \
                    $infogr [_ "Group info"] [mymethod groups_info] "F1" \
                    $rename [_ "Rename"] [mymethod rename_gr_lay_start] ""]
        } else {
            set cmds [list filenew [_ "New"] [mymethod add_group] "" \
                    filenew [_ "New child"] [mymethod add_group -child] "" \
                    "" - "" "" \
                    remove [_ "Delete"] [mymethod delete_group] "" \
                    $infogr [_ "Group info"] [mymethod groups_info] "F1" \
                    $rename [_ "Rename"] [mymethod rename_gr_lay_start] ""]
        }
        set group_editable 1
        if { $options(-domdoc) ne "" } {
            set root [$options(-domdoc) documentElement]
            set group_editable [$root selectNodes string(groups_types/@editable)]
            if { $group_editable eq "" } { set group_editable 1 }
        }
        if { $group_editable } {
            lappend cmds queue [_ "Change type"] [mymethod change_group_type_start] ""
        }
       
        
        lappend cmds \
            $changecol [_ "Change color"] [mymethod change_color_group] "" \
            "" - "" "" \
            up [_ "Move groups position"] "- cmds5" "" \
            "" - "" "" \
            $selall [_ "Select all"] [mymethod select_all] "" \
            view_detailed [_ "List groups"] [mymethod list_entities] "" \
            "" - "" "" \
            select [_ "Send entities to group"] "- cmds2" "" \
            removecomment [_ "Take entities out"] "- cmds3" "" \
            "" - "" "" \
            layertogroup [_ "Layers from groups"] [mymethod groups_to_layers] "" \
            "" - "" "" \
            colorize [_ "Draw color"] "- cmds4" "" \
            "" - "" "" \
            mac-expand [_ "Expand"] [mymethod collapse_expand_lay_gr expand] "" \
            mac-collapse [_ "Collapse"] [mymethod collapse_expand_lay_gr collapse] "" \
            mac-expand2 [_ "Expand all"] [mymethod collapse_expand_lay_gr expandall] "" \
            mac-collapse2 [_ "Collapse all"] [mymethod collapse_expand_lay_gr collapseall]            
        
        switch [lindex [GiD_Info Project] 4] {
            GEOMETRYUSE {
                set ents [list [_ "Points"] point point [_ "Lines"] line line \
                        [_ "Surfaces"] surface surface [_ "Volumes"] volume volume]
            }
            MESHUSE {
                if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                    set ents [list [_ "Nodes"] node node [_ "Elements"] element element]
                } else {
                    set ents [list [_ "Nodes"] point node [_ "L Elements"] line element \
                            [_ "S Elements"] surface element \
                            [_ "V Elements"] volume element]
                }
            }
        }
        foreach "cmds2 cmds3 cmds4" [list "" "" ""] break               
        
        foreach "name n img" $ents {
            lappend cmds2 [gid_themes::GetImage $img small_icons] $name \
                [mymethod send_entities in "" $name $n] ""
        }
        foreach "name n img" $ents {
            lappend cmds3 [gid_themes::GetImage $img small_icons] $name \
                [mymethod send_entities out "" $name $n] ""
        }
        foreach "name n img" $ents {
            lappend cmds4 [gid_themes::GetImage $img small_icons] $name \
                [mymethod draw_selected_groups "" $name $n] ""
        }
        set cmds5 [list \
                $upbut [_ "Move up"] [mymethod move_group up] "" \
                down [_ "Move down"] [mymethod move_group down] "" \
                top [_ "Move top"] [mymethod move_group top] "" \
                bottom [_ "Move bottom"] [mymethod move_group bottom] "" \
                ]
        set cmds6 [list \
                grouptolayer [_ "Groups from layers"]... [mymethod selected_layers_to_groups] "" \
                layertogroup [_ "Layers from groups"]... [mymethod groups_to_layers] "" \
                ]

        destroy $win.menu
        set m [menu $win.menu -tearoff 0]

        $self contextual_menu_add_cmds $m cmds
        cu::tooltip cancel $win
        tk_popup $m $x $y
    }
    method draw_selected_groups { w { ovname "" } { ov "" } } {
        
        set ::CUSTOM_LIB_USE_NATIVE_GROUPS_DRAW_NATIVE 0
        
        set ids [$tree selection get]
        if { [llength $ids] == 0 } {        
            GidUtils::SetWarnLine [_ "It is necessary to select one or more groups"] 
            return
        }
        if { $ov eq "" } {
            set menu $win.menu
            if {[winfo exists $menu]} {
                destroy $menu                
                return
            }
            menu $menu -tearoff 0

            switch [lindex [GiD_Info Project] 4] {
                GEOMETRYUSE {
                    set ents [list [_ "Points"] point point [_ "Lines"] line line \
                            [_ "Surfaces"] surface surface [_ "Volumes"] volume volume]
                }
                MESHUSE {
                    if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                        set ents [list [_ "Nodes"] node node [_ "Elements"] element element]
                    } else {
                        set ents [list [_ "Nodes"] point node [_ "L Elements"] line element \
                                [_ "S Elements"] surface element \
                                [_ "V Elements"] volume element]     
                    }
                }
            }
            foreach "name n img" $ents {
                $menu add command -label $name -command \
                    [mymethod draw_selected_groups_do $w $n] -image \
                    [gid_themes::GetImage $img small_icons] -compound left
            }
            set x [expr {[winfo rootx $w]}]
            set y [expr {[winfo rooty $w]+[winfo height $w]}]
            cu::tooltip cancel $w
            tk_popup $menu $x $y
        } else {
            $self draw_selected_groups_do $w $ov
        }
        set ::CUSTOM_LIB_USE_NATIVE_GROUPS_DRAW_NATIVE 1        
    }
    method draw_selected_groups_do { w ov } {
        set groups ""
        foreach i [$tree selection get] {
            lappend groups [list [$self give_full_group_name $i] $ov]
        }
        if { $groups eq "" } {
            set groups [list ALL $ov]
        }
        $self end_draw_selected_groups "" ""
        gid_groups_conds::drawall colors $groups
        GiD_Process 'Redraw
        # dirty trick to make delayedop work correctly
#         GidUtils::DisableWarnLine
#         GiD_Process Mescape 'SelectEntities
#         GidUtils::EnableWarnLine

        if { $options(-open_over) ne "" && [winfo class $options(-open_over)] eq "Button" } {
            set b0 $options(-open_over)
            set scr ""
            append scr "[list $b0 configure -image [$b0 cget -image]];"\
                "[list $b0 configure -command [$b0 cget -command]];"\
                "[list wm deiconify [winfo toplevel $win]];"\
                "grab $win"
            $b0 configure -image $gid_groups_conds::images(locationbar_erase-22) \
                -command \
                [mymethod end_draw_selected_groups $scr $b0]
            set b $b0

            wm withdraw [winfo toplevel $win]
            grab release $win
        } else {
            set t $win
            set f [ttk::frame $t.finish -style Mod.ForcedFrame]
            set scr "[list destroy $f]; raise $t"
            
            set len 0
            set txt1 [_ "End"]
            foreach i [list $txt1] {
                if { [string length $i] > $len } {
                    set len [string length $i]
                }
            }
            incr len 5
            set b [ttk::button $f.b -text [_ "End"] -image colorize_end \
                    -compound left -width $len -style Toolbutton -command \
                    [mymethod end_draw_selected_groups $scr $f.b]]
                 
            grid $b -padx 5 -pady 0 -sticky nw
                        
            grid columnconfigure $f 0 -weight 1
            grid rowconfigure $f 0 -weight 1
            cu::tooltip register $b [_ "Click on this button or Escape to finish the draw process"]
            
            place $f -in $t -x 0 -y 0 -anchor nw -width \
                [winfo width $t] -height [winfo height $t]
            #grab $b
            bind $b <Escape> "[list $b invoke]; break"
            bind $b <Return> [list $b invoke]
            focus $b
        }
#         delayedop changefunc [list $b invoke]
    }
    method end_draw_selected_groups { scr b } {
        if { $b ne "" } {
            set found [delayedop -cancel changefunc [list $b invoke]]
            if { $found } { GiD_Process escape escape }
        }
        eval $scr
        gid_groups_conds::end_drawall
    }
    method press1_gr { x y } {  
        set identify [$tree identify $x $y]
        set item [lindex $identify 1]
        if { $item eq "" && [$tree index active] != 0 } {           
            set id_active [$tree index active]
            $tree selection add $id_active
            $tree activate $id_active                        
        }  
        
        set col [lindex $identify 3]
        if { $col == 0 && [lindex $identify 5] eq "e_col_rect" } {
            $self change_color_group $item
            return 0
        } elseif { $col == 1 } {           
            $self set_to_onoff_gr $item             
            return 0
        } else {            
            return 1
        }
    }
      
    method double_click_table { x y } {
        set identify [$tree identify $x $y]       
        set col [lindex $identify 3]         
           
        if { $col == 0 } {
            $tree selection clear
            set item [lindex $identify 1]
            $tree selection add $item
            $self set_layers_to UNFREEZE
            $self set_layers_to TOUSE 
            
            set img switch_on
#             GidUtils::DisableGraphics
            $tree item element configure $item 1 e_image -image $img  
            $tree item element configure $item 3 e_image -image unlock           
            foreach id [$tree item range 0 end] {
                if { $id eq "0"} { continue }
                if { $id == $item } { continue }
                $tree item element configure $id 1 e_image -image blank-16            
            }               
#             GidUtils::EnableGraphics
#             GiD_Process 'Redraw            
            return 0        
        } else {
            return 1
        }
    }
    method press1_lay { x y } {                         
        set identify [$tree identify $x $y]         
        set item [lindex $identify 1]  
        if { $item eq "" && [$tree index active] != 0 } {           
            set id_active [$tree index active]
            $tree selection add $id_active
            $tree activate $id_active                        
        }        
        set col [lindex $identify 3] 
        set ::GidPriv(updatelayers) 0               
        if { $col == 0 && [lindex $identify 5] eq "e_col_rect" } {
            # Name
            $self change_color_layer $item 
            set ::GidPriv(updatelayers) 1       
            return 0
        } elseif { $col == 1 } {
            # Layer to use            
            $self set_to_lay $item  
            set ::GidPriv(updatelayers) 1       
            return 0
        } elseif { $col == 2 } {
            # Visible (On/Off)              
            $self set_to_visible_lay $item 
            set ::GidPriv(updatelayers) 1          
            return 0
        } elseif { $col == 3 } {
            # Freeze/Unfreeze
            $self set_to_f_u $item 
            set ::GidPriv(updatelayers) 1          
            return 0
        } elseif { $col == 4 } {
            # Transparent/Opaque
            $self set_to_t_o $item 
            set ::GidPriv(updatelayers) 1          
            return 0
        } elseif { $col == 5 } {
            # Back
            $self set_to_back $item 
            set ::GidPriv(updatelayers) 1           
            return 0
        } else {
            set ::GidPriv(updatelayers) 1
            return 1
        }                     
    }    
    method get_selected_layernames { } {
        if { ![winfo exists $tree] } { return }
        set LayersSelection ""
        set sel [$tree selection get]        
        foreach item $sel {
            lappend LayersSelection [$self DecodeName [$tree item tag names $item]]           
        }
        set LayersSelection [lsort -unique -dictionary $LayersSelection]
        return $LayersSelection
    }       
    method EncodeName { x } {
        return [binary encode base64 $x]
    }    
    method DecodeName { x } {
        return [binary decode base64 $x]
    }
    method set_layers_to { operation {newname ""} } {
              
        set LayersSelection [$self get_selected_layernames]  
        if { $LayersSelection == "" } {            
            GidUtils::SetWarnLine [_ "It is necessary to select one or more layers"]                      
            return
        }  
        set VarToSend 'Layers    
        foreach i $LayersSelection {           
            switch $operation {
                ON {                  
                    lappend VarToSend On $i
                }
                OFF {
                    lappend VarToSend Off $i
                }
                FREEZE {
                    lappend VarToSend Freeze $i
                }
                UNFREEZE {
                    lappend VarToSend UnFreeze $i
                }
                TRANSPARENT {
                    lappend VarToSend Transparent $i 127
                }
                OPAQUE {
                    lappend VarToSend Opaque $i
                }
                TOUSE {
                    lappend VarToSend ToUse $i
                }
                RENAME {
                    lappend VarToSend ChangeName $i $newname
                }
                BRINGTOFRONT {
                    lappend VarToSend BringToFront $i
                }
            }
        }
        lappend VarToSend escape   
                
        GiD_Process {*}$VarToSend
                
#        GiD_Layers can be used instead of GiD_Process.
#        Example: [GiD_Layers get opaque $i]
                
        #Layers::FillInfo ;#automatically called
        if { $type == "RENAME" } {
            #Trick to have selected the new name after Layers::FillInfo will be automatically called
            $self set_layer_as_selection $newname 
        }  
    }       
    method set_to_lay { id } {
        if { $id != "" } { 
            $tree selection clear
            $tree selection add $id
        }        
        set img [$tree item element cget active 1 e_image -image]               
        if { $img eq "blank-16" } {           
            set ison 1
        } else {
            set ison 0
        } 
        if { !$ison && [llength [GiD_Info Layers]] == 1 } {                       
            set LayersSelection [$self get_selected_layernames]
            GidUtils::SetWarnLine [_ "Can not unselect '%s' as layer to use. At least must \nexist one layer to use." $LayersSelection]                
            return            
        }                      
        
        if { $ison } {                             
            $self set_layers_to UNFREEZE         
            GiD_Process 'Redraw          
            
            $self set_layers_to TOUSE          
            foreach item [$tree item range 0 end] {
                if { $item eq "0"} { continue }
                if { $item in [$tree selection get] } { 
                    $tree item element configure $item 3 e_image -image unlock
                    $tree item element configure $item 1 e_image -image switch_on                                      
                } else {
                    $tree item element configure $item 1 e_image -image blank-16
                }
            }
        }                            
    } 
    method set_to_visible_lay { id } {        
        if { $id != "" } { 
            $self add_selection $id            
        }               
        set gList [$self give_full_layer_name_and_children [$tree selection get]]  
        set gList [lsort -index 1 $gList]
        
        set img [$tree item element cget active 2 e_image -image]                            
        if { $img eq "blank-16" } {           
            set img check-16           
            set ison 1
        } else {
            set img blank-16
            set ison 0            
        }       
        foreach i $gList {
            lassign $i it -
            $tree item element configure $it 2 e_image -image $img      
        }    
        if { $ison } {
            set VarToSend 'Layers
            foreach i $gList {
                lassign $i it iname                                   
                lappend VarToSend On $iname            
            }      
            lappend VarToSend escape
            GiD_Process {*}$VarToSend      
        } else {            
            set LayerToUse [GiD_Info Project LayerToUse]
            set VarToSend 'Layers
            foreach i $gList {
                lassign $i it iname                                   
                lappend VarToSend Off $iname            
            }      
            lappend VarToSend escape
            GiD_Process {*}$VarToSend           
            set LayersSelection [$self get_selected_layernames]
            if { $LayerToUse in $LayersSelection } {
                foreach ii [$tree item range 0 end] {
                    if { $ii eq "0" || $ii eq $id } { continue }
                    set img [$tree item element cget $ii 2 e_image -image]                                   
                    if { $img eq "check-16" } {
                        $self set_to_lay $ii
                        break
                    }
                }                             
            }            
        }  
        set win .gid.central.boundaryconds.gg
        gid_groups_conds::gridremove_open_all_window $win 1        
    }
    method set_to_f_u { id } {       
        if { $id != "" } { 
            $self add_selection $id             
        }        
        set gList [$self give_full_layer_name_and_children [$tree selection get]]  
        set gList [lsort -index 1 $gList]
        
        set img [$tree item element cget active 3 e_image -image]        
        if { $img eq "unlock" } {           
            set img lock           
            set ison 1
        } else {
            set img unlock
            set ison 0
        }       
        set LayersSelection [$self get_selected_layernames]           
    
        if { [llength [GiD_Info Layers]] == 1 && $ison } { 
            foreach id [$tree selection get] {
                set ilayer [$self DecodeName [$tree item tag names $id]]            
                GidUtils::SetWarnLine \
                    [_ "'%s' can not be frozen due to there is only one layer." $ilayer]             
            }
            return
        }                
        if { $ison } {
            set f_u Freeze
        } else {
            set f_u UnFreeze
        }
        
        set VarToSend 'Layers
        foreach i $gList {
            lassign $i it iname                                        
            lappend VarToSend $f_u $iname           
        }     
        lappend VarToSend escape
        GiD_Process {*}$VarToSend 
        
        set first_lay ""
        if { [llength [$tree item range 0 end]] eq [expr {[llength [$tree selection get]]+1}] && $ison } {
            # First layer is unfrozen 
            set first_lay 1
            set ilay [$self DecodeName [$tree item tag names 1]]
            set VarToSend 'Layers           
            lappend VarToSend UnFreeze $ilay escape                     
            GiD_Process {*}$VarToSend
            
            set VarToSend 'Layers                     
            lappend VarToSend ToUse $ilay escape
            GiD_Process {*}$VarToSend
            
            $tree item element configure 1 1 e_image -image switch_on
            $tree item element configure 1 3 e_image -image unlock  
            foreach id [$tree selection get] {
                if {$id eq "1"} { continue }
                $tree item element configure $id 1 e_image -image blank-16     
            }
        } else {
            set lay_to_use ""
            foreach ii [$tree item range 0 end] {
                if { $ii eq "0"} { continue }
                if {[$tree item element cget $ii 1 e_image -image] eq "switch_on"} { 
                    set lay_to_use [$self DecodeName [$tree item tag names $ii]]
                    set i_lay_to_use $ii
                    break 
                }                                                 
            }                 
            
            if { $lay_to_use in $LayersSelection && $ison } {
                foreach item [lsort -unique -integer [$tree item range 0 end]] {
                    if { $item eq "0" || $item in [$tree selection get] } { continue }
                    break
                }                               
                set ilay [$self DecodeName [$tree item tag names $item]]
                set VarToSend 'Layers           
                lappend VarToSend UnFreeze $ilay escape                     
                GiD_Process {*}$VarToSend
                
                set VarToSend 'Layers                     
                lappend VarToSend ToUse $ilay escape
                GiD_Process {*}$VarToSend
                
                $tree item element configure $item 1 e_image -image switch_on
                $tree item element configure $item 3 e_image -image unlock
                
                foreach jj [$tree item range 0 end] {
                    if { $jj eq "0" || $jj eq $item } { continue }               
                    $tree item element configure $jj 1 e_image -image blank-16
                }  
                set first_lay $item             
            }
        }    
        foreach i $gList {
            lassign $i it iname          
            if { $it eq $first_lay } { continue }        
            $tree item element configure $it 3 e_image -image $img              
        }
    }   
    method set_to_t_o { id } {
        if { $id != "" } { 
            $self add_selection $id            
        }
        set gList [$self give_full_layer_name_and_children [$tree selection get]]  
        set gList [lsort -index 1 $gList]
        
        set img [$tree item element cget active 4 e_image -image]      
        if { $img eq "opaque" } {           
            set img trans           
            set ison 1
        } else {
            set img opaque
            set ison 0
        }
        
        set VarToSend 'Layers
        foreach i $gList {
            lassign $i it iname                
            $tree item element configure $it 4 e_image -image $img                       
            if { $ison } {                
                lappend VarToSend Transparent $iname 127 
            } else {
                lappend VarToSend Opaque $iname
            }                                  
        }    
        lappend VarToSend escape        
        GiD_Process {*}$VarToSend        
    }   
    method set_to_back { id } { 
        if { $id != "" } { 
            $self add_selection $id            
        }
        set gList [$self give_full_layer_name_and_children [$tree selection get]]  
        set gList [lsort -index 1 $gList]
        
        set idbackList ""      
        set img [$tree item element cget active 5 e_image -image]        
        if { $img eq "back" } {           
            set img blank-16          
            set ison 1                  
        } else {
            set img back
            set ison 0
        }   
        
        set idbackList ""  
        foreach i $gList {
            lassign $i it iname
            set img_id [$tree item element cget $it 5 e_image -image]        
            if { $img_id eq "back" } {           
                lappend idbackList $it
            }  
        }
        set ids [$tree selection get]
        if { [llength $ids] == 0 || $idbackList == "" } {
            return
        }                                
        set VarToSend 'Layers
        foreach i $gList {
            lassign $i it iname                                               
            lappend VarToSend BringToFront $iname                     
        }             
        lappend VarToSend MEscape
        GiD_Process {*}$VarToSend
        
        foreach id $idbackList {
            $tree item element configure $id 5 e_image -image $img
        }                          
    }  
    method press2 { args } {
        # nothing by now      
    }
    method rename_start { - item col } {             
        if { $options(-layers) } {
            if { $col == 0 } {
                set values ""
                set LayerNames [GiD_Info layers]
                foreach ilay $LayerNames {
                    lappend values $ilay
                }                
                return [list combo 1 $values ""]
            } else {                
                return 0
            }
        } elseif {$options(-groups)} {
            if { $col == 0 } {
                set values ""
                set root [$options(-domdoc) documentElement]
                foreach node [$root selectNodes groups/group] {
                    lappend values [$node @n]
                }
                return [list combo 1 $values ""]
            } elseif { $col == 2 } {
                set group_editable 1
                if { $options(-domdoc) ne "" } {
                    set root [$options(-domdoc) documentElement]
                    set group_editable [$root selectNodes string(groups_types/@editable)]
                    if { $group_editable eq "" } { set group_editable 1 }
                }
                if { $group_editable } {
                    return [list combo 0 $group_types ""]
                }
            }            
            return 0
        }
    }
    method rename_end { - item col text } {
        if { $options(-layers) } { 
            if { $col == 0 } {
                $self rename_layer $item $text
            } 
        } elseif {$options(-groups)} {
            if { $col == 0 } {
                $self rename_group $item $text
            } elseif { $col == 2 } {
                $self change_group_type $item $text
            }
        }
    }
    method add { args } {                
        set optional {
            { -before item "" }                     
        }
        set compulsory "parent name onoff color gtype"
        parse_args $optional $compulsory $args
                
        if { $onoff } {
            set img [gid_themes::GetImage ok small_icons]
        } else { 
            set img blank-16 
        }
        
        set nameList [cu::splitx $name {//}]       
        
        if { [llength $nameList] > 1 } {    
            set name [lindex $nameList end]
            set before ""                        
        } else {            
            set found 0            
            foreach item [$tree item children $parent] {
                if { $name eq [$tree item text $item 0] } {                    
                    set found 1
                    break                    
                }
                if { $found } { return $item }
            }            
        }
        if { $before eq "" } {
            set treenode [$tree insert end [list $name $img $gtype] $parent]
        } else {
            set treenode [$tree insert prev [list $name $img $gtype] $before]
        }
        set colorState [list $color]   
        
        $tree item style set $treenode 0 s_color_text       
        $tree item element configure $treenode 0 e_col_rect -fill $colorState + \
            e_text_sel -text $name
        
        if { $parent != 0 } {
            $tree configure -indent 19 -showbutton 1 -showlines 0 \
                -folder_image ""
        }   
        return $treenode
    }
    method is_valid_layername { layername } {
        if { [string range $layername end-1 end] == "//" } {
            return 0
        } elseif { [string range $layername 0 1] == "//" } {
            return 0
        } elseif { $layername == "" } {
            return 0
        }
        return 1
    }
    # only changes the tree, not the GiD data
    
    method rename_gr { treenode name { onoff "" } } {               
        if { $onoff eq "" } {
            if { [$tree item element cget $treenode 1 e_image -image] eq \
                [gid_themes::GetImage ok small_icons] || \
                [$tree item element cget $treenode 1 e_image -image] eq "check-16" || \
                    [$tree item element cget $treenode 1 e_image -image] eq "check_star-16"} {
                set onoff 1
            } else {
                set onoff 0
            }
        }
      
        set err [catch { set colour [$tree item element cget active 0 e_col_rect -fill]} errstring]
        if { $err } {
            set color ""
        } else {
            set color [lindex $colour 0]   
        }                   
        set colorState [list $color]
        set type [$tree item element cget $treenode 2 e_text_sel -text]

        set nameList [cu::splitx $name {//}]
        if { [llength $nameList] > 1 } {           
            $self add -before $treenode [$tree item parent $treenode] $name $onoff \
                $color $type
            $tree item delete $treenode
            $tree activate_select_item first
        } else {           
            if { $onoff } {  
                set img [gid_themes::GetImage ok small_icons]
                # GiD change: only next line               
                set img [ groups_theme::GetAlternativeThemedImage $img check-16]                                                
            } else { 
                set img blank-16 
            }
            if {$color ne ""} {
                $tree item complex $treenode \
                    [list [list e_col_rect -fill $colorState] [list e_text_sel -text $name]] \
                    [list [list e_image -image $img]] \
                    [list [list e_text_sel -text $type]]
            } else {
                $tree item complex $treenode \
                    [list [list e_text_sel -text $name]] \
                    [list [list e_image -image $img]] \
                    [list [list e_text_sel -text $type]]
            }
        }            
    }
    method rename_la { treenode name { onoff "" } } {         
        set colorState [$tree item element cget $treenode 0 e_col_rect -fill]
        set color [lindex $colorState 0]                

        set nameList [cu::splitx $name {//}]
        if { [llength $nameList] > 1 } {
            $self add_lay -before $treenode [$tree item parent $treenode] $name $onoff \
                $color $type
            $tree item delete $treenode
            $tree activate_select_item first
        } else {            
            $tree item element configure $treenode 0 e_col_rect -fill $colorState + \
                e_text_sel -text $name                                           
        }
    }
    method filltablelayers { args } { 
        set optional {        
        }
        set compulsory "parent parentitem name actualize_lay icount"
        parse_args $optional $compulsory $args 
                
        if { $icount == "" || ($actualize_lay && $icount == "1") } {
            array unset stateitems
            array unset selectionitems
            #try to mantain the previous state: open, selected
            foreach item [$tree item range 0 end] {
                set fullnameencoded [$tree item tag names $item]
                set stateitems($fullnameencoded) [$tree item isopen $item]
                set selectionitems($fullnameencoded) [$tree selection includes $item]
            }
            $tree item delete all ;#could try not to delete and rebuild all...                                  
            
            $self filltableinfolaynew $parent $parentitem $actualize_lay $icount 
            
            if { [$tree column cget 0 -arrow] == "up" } {       
                $tree item sort 0 -dictionary -decreasing
            } else {
                $tree item sort 0 -dictionary -increasing
            }
            foreach item [$tree item range 0 end] {
                set fullnameencoded [$tree item tag names $item]
                if { [info exists stateitems($fullnameencoded)] &&
                    !$stateitems($fullnameencoded) } {
                    $tree item collapse $item
                }
                if { [info exists selectionitems($fullnameencoded)] &&
                    $selectionitems($fullnameencoded) } {
                    $tree selection add $item
                }
            }        
            set id [$tree item id "tag [$self EncodeName [GiD_Info Project LayerToUse]]"]
            if { $id != "" } {
                $tree activate $id
            }
            if { [$tree selection count] == 1 } {
                $tree see [$tree selection get]
            }
            set t [$tree givetreectrl]
            set ::TreeCtrl::Priv(DirCnt,$t) [$tree item count] ;#to use drag file-list bindings              
        }
    }
    
    method filltableinfolaynew { args } {        
        set optional {        
        }
        set compulsory "parent parentitem actualize_lay icount"
        parse_args $optional $compulsory $args      

        set LayerNames [$self get_ChildLayerNames $parent]        
        set LayerToUse [GiD_Info Project LayerToUse]        
        set BackLayerNames [GiD_Info back_layers]
        if { $BackLayerNames != "" } {
            regsub -all {_\*back\*} $BackLayerNames {} pp
            set BackLayerNames $pp
        }        
                
        foreach i $LayerNames {
            if { $parent != "" } {
                set fullname $parent//$i
            } else {
                set fullname $i
            }
            set textname $i
            set has_childs [$self has_childs $fullname]
            if { $has_childs } {
                set isbutton yes
            } else {
                set isbutton no
            }
                        
            set item [$tree item create -button $isbutton -tags [$self EncodeName $fullname]]
            $tree item lastchild $parentitem $item
            
            set LayersProp [lindex [GiD_Info layers $fullname] 0]
            set cur_col [format "#%02x%02x%02x" [lindex $LayersProp 2] \
                    [lindex $LayersProp 3] [lindex $LayersProp 4]]
            
            set colorState $cur_col  
       
            $tree item style set $item 0 s_color_text                   
            $tree item element configure $item 0 e_col_rect -fill $colorState + \
                e_text_sel -text $textname 
   
            for {set j 1} { $j <= 5 } { incr j } {
                $tree item style set $item $j image
            }
            
            if { $fullname == $LayerToUse } {
                $tree item element configure $item 1 e_image -image switch_on 
            } else {
                $tree item element configure $item 1 e_image -image blank-16  
            }   
            
            if { [lindex $LayersProp 0] } {
                $tree item element configure $item 2 e_image -image check               
            } else {
                $tree item element configure $item 2 e_image -image blank-16                
            }                        
            if { [lindex $LayersProp 1] } {
                $tree item element configure $item 3 e_image -image lock              
            } else {
                $tree item element configure $item 3 e_image -image unlock            
            }           
            if { [lindex $LayersProp 5] == 255 } {
                $tree item element configure $item 4 e_image -image opaque 
            } else {
                $tree item element configure $item 4 e_image -image trans 
            }                       
            if { [ lsearch $BackLayerNames $fullname] != -1 } {
                $tree item element configure $item 5 e_image -image back           
            } else {
                $tree item element configure $item 5 e_image -image blank-16                   
            }
            if { $parent != "" } {
                $tree configure -indent 19 -showbutton 1 -showlines 0 \
                    -folder_image ""
            }
            if { $has_childs } {
                $self filltableinfolaynew $fullname $item $actualize_lay $icount 
            }          
        }                         
    }
    method get_ChildLayerNames { {parent ""} } {
        global GidPriv
        set LayerNames0 [GiD_Info layers]
        set len [string length $parent]
        set LayerNames {}
        foreach layer $LayerNames0 {
            if { $len } {
                if { [string equal -length $len $parent $layer] } {
                    if { [string range $layer $len $len+1] == "//" } {
                        if { [string first // $layer $len+2] == -1 } {
                            lappend LayerNames [string range $layer $len+2 end]
                        } else {
                            #only append childs of one level
                        }
                    }
                }
            } else {
                if { [string first // $layer] == -1 } {
                    lappend LayerNames $layer
                } else {
                    #only append childs of one level
                }            
            }
        }
        if { [info exists GidPriv(LayersAlphabetic)] && $GidPriv(LayersAlphabetic) } {
            set LayerNames1 [lsort -unique -dictionary $LayerNames]
        } else {
            #must remove repetitions
            set LayerNames1 {}
            foreach layer $LayerNames {
                if { [info exists name($layer)] } continue
                lappend LayerNames1 $layer
                set name($layer) 1
            }
        }
        return $LayerNames1
    }    
    method has_childs { {parent ""} } {
        set LayerNames0 [GiD_Info layers]
        set len [string length $parent]
        foreach layer $LayerNames0 {
            if { $len } {
                if { [string equal -length $len $parent $layer] } {
                    if { [string range $layer $len $len+1] == "//" } {
                        return 1
                    } 
                }
            } else {
                return 1
            }
        }
        return 0
    }
    method add_lay { args } {                     
        set optional {  
            { -child "" 0 }  
            { -exist exist 0 } 
            { -actualize_lay actualize_lay 0 }    
            { -icount icount "" }      
        }
        set compulsory "name"
        parse_args $optional $compulsory $args   

        set or_name $name      
        
        $tree edit_item_cancel
        
        set id [_ "layer"]
        set msg [_ "Select one %s to be the parent of the new %s" $id $id]
                        
        set id [_ "Layer"]
        if { $child } {
            if { [llength [$tree selection get]] != 1 } {
                GidUtils::SetWarnLine $msg 
                return
            }
            set parent [lindex [$tree selection get] 0]
            $tree item expand $parent           
            set name0 [$self give_full_layer_name $parent]//$id        
        } else {                  
            set parent 0           
            set name0 $id
        }                      
      
        lassign [$self add_new_layer -exist $exist -permit_extended_name 1 \
                $name0] name color        
                           
        $self filltablelayers "" root $or_name $actualize_lay $icount
        
#         update ;#to fill in again the treectrl to create the item        
        set item [$tree item id "tag [$self EncodeName $name]"]
        if { $item != "" } {
            $self rename_gr_lay_start $item                      
        }               
    }                

    method add_group { args } {
                      
        set optional {
            { -child "" 0 }          
        }
        set compulsory ""
        parse_args $optional $compulsory $args
        
        $tree edit_item_cancel
              
        if {$options(-groups)} {
           set id [_ "group"]
        } elseif {$options(-layers)} {
           set id [_ "layer"]
        }
        set msg [_ "Select one %s to be the parent of the new %s" $id $id]
        
        if { $child } {
            if { [llength [$tree selection get]] != 1 } {
                GidUtils::SetWarnLine $msg 
                return
            }
            set parent [lindex [$tree selection get] 0]
            $tree item expand $parent
            set name0 [$self give_full_group_name $parent]//$id
        } else {
            if {$options(-groups)} {
                set parent 0              
                set name0 $id 
            } else {
                set parent 0
#                 set prefix ""
#                 set name0 [Layers::GetAutomaticLayerName $prefix]
                set id [_ "Layer"]
                set name0 $id                
            }
        }       
        
        if {$options(-groups)} {
             lassign [gid_groups_conds::add_new_group -noactualizegroups \
                    $name0] name color           
            
            set id [$self add $parent $name 1 $color [lindex $group_types 0]]
            $self rename_gr_lay_start $id
        } else {
            $self add_lay -exist 0 ""           
        }       
    }
    method copy_group {} {                
        set root [$options(-domdoc) documentElement]
        
        set gList [$self give_full_group_name_and_children [$tree selection get]]

        switch [llength $gList] {
            0 { 
                GidUtils::SetWarnLine [_ "Select at least one group to copy"] 
                return
            } 
            default {
                GidUtils::DisableWarnLine
                foreach i [lrange $gList 0 9] {                   
                    lassign $i item group    
                    set group_type [$tree item text $item end]               
                    lassign [gid_groups_conds::add_new_group -copy_group 1 -noactualizegroups $group] name color                    
                    set id [$self add 0 $name 1 $color [lindex $group_type 0]]                    
                    # Apply entities
                    if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                        foreach ovG [list point line surface volume] {
                            set over ${ovG}s
                            foreach jjnum [GiD_EntitiesGroups get $group ${ovG}s] {
                                GiD_EntitiesGroups assign $name $over $jjnum
                            }
                        }
                    } else {
                        foreach ovG [list point line surface volume] {
                            set over ${ovG}s
                            foreach jj [GiD_Info conditions ${ovG}_groups geometry] {
                                lassign $jj jjdummy jjnum jjdummy jjgroup
                                if { $jjgroup ne $group } { continue }
                                GiD_Process MEscape Data Conditions AssignCond ${ovG}_groups \
                                    NoRepeatField name Change $name $jjnum escape                               
                            }
                        }
                        foreach ovG [list point line surface volume] {
                            foreach jj [GiD_Info conditions ${ovG}_groups mesh] {
                                lassign $jj jjxx jjnum jjdummy jjgroup 
                                if { $jjxx eq "E" } {                   
                                    set over element                    
                                } elseif { $jjxx eq "N" } {
                                    set over node                              
                                } else {
                                    set over face
                                    set num [list $num $jjxx]
                                }    
                                GiD_Process MEscape Data Conditions AssignCond ${ovG}_groups \
                                    NoRepeatField name Change $name $jjnum escape                                                  
                            }
                        }
                    } 
                    if { [llength $gList] eq "1" } {
                        $self rename_gr_lay_start $id
                    }                    
                } 
                GidUtils::EnableWarnLine                
            }                     
        }       
    }
    method add_new_layer { args } {
        variable colorList
        
        set optional {            
            { -exactname "" 0 }
            { -exist exist 0 }
            { -ifnotexists "" 0 }
            { -noundo "" 0 }
            { -type typeName "" }
            { -permit_extended_name boolean 0 }
            { -color color "" }
        }
        set compulsory "prefix"
        parse_args $optional $compulsory $args                           
        
        if { $permit_extended_name } {
            set rex0 {^[-.\w\s/:*]+$}
            set rex1 {^[-.\w\s/#]+$}
            set rex2 {^[-.\w\s/]}
        } else {
            set rex0 {^[-.\w\s/]+$}
            set rex1 {^[-.\w\s/#]+$}
            set rex2 {^[-.\w\s/]}
        }
        if { ![regexp $rex0 $prefix] && ![regexp $rex1 $prefix] && ![regexp $rex2 $prefix]} {
            return -code error \
                [_ "error: A layer name can only contain letters, digits, space and _-./ (trying to create '%s')" $prefix]
        }
        set idx ""
        if { !$exist } {
            set idx 0
            foreach ilay [GiD_Info Layers] {
                if { [regexp "^$prefix\(\\d+\)\$" $ilay {} n] && $n > $idx } {
                    set idx $n
                }
            }    
            incr idx
        }          
        
        if { $exactname || $ifnotexists } {
            set name $prefix
            if { [$name in [GiD_Info Layers]] } {
                if { $ifnotexists } { return }
                return -code error [_ "error: Layer '%s' already exists" $name]
                
            }
        } else {
            set name $prefix$idx
        }
        if {$name in [GiD_Info Layers]} {
            # nothing
        } elseif {!$exist} {
            GiD_Process 'Layers New $name escape
        }
#         update
        
        if {$name in [GiD_Info Layers]} {          
            set LayersProp [lindex [GiD_Info layers $name] 0]
            set color [format "#%02x%02x%02x" [lindex $LayersProp 2] \
                    [lindex $LayersProp 3] [lindex $LayersProp 4]]
        }       
        
        return [list $name $color]
    }
    method delete_gr_lay {} {
        if {$options(-groups)} {
            $self delete_group
        } elseif {$options(-layers)} {
            $self delete_layer
        } 
    }
    method delete_group {} {

        set gList [$self give_full_group_name_and_children [$tree selection get]]
        
        set groups_pr ""
        switch [llength $gList] {
            0 { 
                GidUtils::SetWarnLine [_ "Select at least one group to delete"] 
                return
            }
            1 {
                set txt [_ "Are you sure to delete group '%s'?" [lindex $gList 0 1]]
            }
            default {
                foreach i [lrange $gList 0 9] {
                    lassign $i item group
                    if { $group ne "" } {
                        lappend groups_pr [lindex $i 1]
                    }
                }
                if { [llength $gList] > 10 } {
                    lappend groups_pr "..."
                }
                set txt [_ "Are you sure to delete groups '%s' (%d)?" $groups_pr [llength $gList]]
            }
        }
        set ret [snit_messageBox -type okcancel -default ok \
                -message $txt -parent $win]
        if { $ret eq "cancel" } { return }
        
        set isopenList ""
        foreach ido [$tree item range 0 end] {
            if { $ido eq "0"} { continue }
            set isopen [$tree item isopen $ido]
            if { $isopen } {
                lappend isopenList $ido
            }
        }
        
        foreach i $gList {
            lassign $i item group
            if { $group ne "" } {
                set ret [gid_groups_conds::delete_group -ifexists -noactualizegroups $group]
            } else {
                set ret ""
            }                       
#             if { $ret != "-cancel-" } {
                # catch is here because children items can already be deleted
#                 catch { $tree item delete $item }
#             }            
        }  
        if { !$::CUSTOM_LIB_USE_NATIVE_GROUPS } {
            gid_groups_conds::actualize_groups_window 
        } 
        foreach ido $isopenList {
            if { $ido in [$tree range 0 end] } {                
                $tree item expand $ido              
            }  
        }  
        $tree activate_select_item first
    }
    method delete_layer {} {       
        set tbl $tree
        set parent [winfo parent $tree]                
               
        set LayersSelection [$self get_selected_layernames]
        set gList [$self give_full_layer_name_and_children [$tree selection get]]

        set laytouse [GiD_Info Project LayerToUse]
        switch [llength $gList] {
            0 { 
                GidUtils::SetWarnLine [_ "Select at least one layer to delete"]                     
                return
            }
            1 {
                if { [lindex $gList 0 1] in $laytouse } { 
                    set txt [_ "'%s' cannot be deleted due to it is the layer to use" [lindex $gList 0 1]]
                    GidUtils::SetWarnLine $txt
                    return
                } else {                    
                    set txt [_ "Are you sure to delete layer '%s' and its entities?" [lindex $gList 0 1]]
                }
            }
            default {
                foreach i [lrange $gList 0 9] {
                    lassign $i item layer
                    if { $layer ne "" } {
                        lappend groups_pr [lindex $i 1]
                    }
                }
                if { [llength $gList] > 10 } {
                    append groups_pr "..."
                }
                set txt [_ "Are you sure to delete layers '%s' (%d) and its entities?" $groups_pr [llength $gList]]
            }
        }
        set ret [snit_messageBox -type okcancel -default ok \
                -message $txt -parent $win]
        if { $ret eq "cancel" } { return }

        ###
        set ask 1
        set ret 0
        set delete_only_empty_layers 1
        GiD_Process 'Layers  
        set LayersSelection [$self get_selected_layernames] 
      
        foreach i $LayersSelection {
            if { $i in $laytouse } { 
                set txt [_ "'%s' cannot be deleted due to it is the layer to use" [lindex $gList 0 1]]
                GidUtils::SetWarnLine $txt              
            }
            GiD_Process Delete $i
        }
        foreach i $gList {  
            lassign $i item lay
            if { $lay in $laytouse } { continue }
            if { $lay ne "" } {         
                # catch is here because children items can already be deleted
                catch { $tree item delete $item }            
            }
        }
        
        GiD_Process escape   
        $tree activate_select_item first
    }
    method select_group { group_name } {     
        foreach item [$tree item children 0] {
            if { [$tree item text $item 0] eq $group_name } {
                $tree selection clear all
                $tree selection add $item
                $tree activate $item
                $tree see $item
                focus [$tree givetreectrl]
                break
            }
        }
    }
    method change_group_type { id newtype } {
        set name [$self give_full_group_name $id]
        gid_groups_conds::change_group_att -noactualizegroups $name type $newtype
        $tree item element configure $id 2 text -text $newtype
    }
    method rename_group_start_name { group_name } {
        foreach item [$tree item children 0] {
            if { [$tree item text $item 0] eq $group_name } {
                $self rename_gr_lay_start $item
                break
            }
        }
    }
    method rename_gr_lay_start { { id "" } } {
        if { $id eq "" } { set id [$tree selection get] }
        if { [llength $id] != 1 } {
            if {$options(-groups)} {
                set id [_ "group"]
            } else {
                set id [_ "layer"]
            }        
            GidUtils::SetWarnLine [_ "It is necessary to select one %s for renaming" $id]                 
            return
        } 
        $tree selection clear all
        $tree selection add $id
        $tree activate $id
        $tree see $id
        focus [$tree givetreectrl]
        update
        event generate [$tree givetreectrl] <F2>        
    }
    method change_group_type_start  { { id "" } } {
        if { $id eq "" } { set id [$tree selection get] }
        if { [llength $id] != 1 } {
            GidUtils::SetWarnLine [_ "It is necessary to select one group for changing group type"] 
            return
        }      
        $tree selection clear all
        $tree selection add $id
        $tree activate $id
        $tree see $id
        focus [$tree givetreectrl]
        update
        $tree edit_item $id 2
    }
    method _rename_group_do { id oldnameF newnameF } {
        gid_groups_conds::rename_group -noactualizegroups $oldnameF $newnameF
        if { !$::CUSTOM_LIB_USE_NATIVE_GROUPS } {            
            foreach id_child [$tree item children $id] {
                set oldnameList_i [cu::splitx $oldnameF {//}]
                lappend oldnameList_i [$self give_short_gr_lay_name $id_child]
                set newnameList_i [cu::splitx $newnameF {//}]
                lappend newnameList_i [$self give_short_gr_lay_name $id_child]
                $self _rename_group_do $id_child [join $oldnameList_i //] [join $newnameList_i //]
            }
        } 
    }
    method rename_group { id newname } {       
        set oldname [$self give_full_group_name $id]
        set newnameList [lrange [cu::splitx $oldname {//}] 0 end-1]
        lappend newnameList $newname
        
        set err [catch { $self _rename_group_do $id [$self give_full_group_name $id] \
                    [join $newnameList //] } errstring]
        if { $err } {
            GidUtils::SetWarnLine [_ "Error renaming group '%s' (%s)" $oldname $errstring] 
            return
        }
        if {$options(-groups)} {
            $self rename_gr $id $newname
        } else {
            $self rename_la $id $newname
        }
    }
    method rename_layer { id newname } {
        
        set oldname [$self give_full_layer_name $id]
        set newnameList [lrange [cu::splitx $oldname {//}] 0 end-1]
        lappend newnameList $newname
        
        set err [catch { $self _rename_layer_do $id [$self give_full_layer_name $id] \
                    [join $newnameList //] } errstring]
        if { $err } {
            GidUtils::SetWarnLine [_ "Error renaming layer '%s' (%s)" $oldname $errstring] 
            return
        }
        if {$options(-groups)} {
            $self rename_gr $id $newname
        } else {
            $self rename_la $id $newname
        }
    }    
    method _rename_layer_do { id oldnameF newnameF } {
        $self rename_lay $oldnameF $newnameF
        if { !$::CUSTOM_LIB_USE_NATIVE_GROUPS } {
            foreach id_child [$tree item children $id] {
                set oldnameList_i [cu::splitx $oldnameF {//}]
                lappend oldnameList_i [$self give_short_gr_lay_name $id_child]
                set newnameList_i [cu::splitx $newnameF {//}]
                lappend newnameList_i [$self give_short_gr_lay_name $id_child]
                $self _rename_layer_do $id_child [join $oldnameList_i //] [join $newnameList_i //]
            }
        }
    }
    method move_menu { w } {
        
        set menu $win.menu
        if {[winfo exists $menu]} {
            destroy $menu            
        }
        menu $menu -tearoff 0
        
        set upbut "up"
        # GiD change
        # if another theme is loaded:
        set upbut [ groups_theme::GetAlternativeThemedImage $upbut upb]
        # additional alternative images
        foreach alt_img [list down top bottom] {
            groups_theme::GetAlternativeThemedImage $alt_img $alt_img
        }         
        # end GiD change    
        set items [list \
                $upbut [_ "Move up"] [mymethod move_group up] \
                down [_ "Move down"] [mymethod move_group down] \
                top [_ "Move top"] [mymethod move_group top] \
                bottom [_ "Move bottom"] [mymethod move_group bottom] \
                ]  
        
        foreach "img txt cmd" $items {
            # GiD change: only next line
            set img [ groups_theme::GetThemedImage $img]
            $menu add command -label $txt -image $img -compound left \
                -command $cmd
        }
        set x [expr {[winfo rootx $w]}]
        set y [expr {[winfo rooty $w]+[winfo height $w]}]
        cu::tooltip cancel $w
        tk_popup $menu $x $y
    }   
    method drag_and_drop { - receiving_item dragged_items relative_pos } {        
        $self move_group -itemList $dragged_items -relative_pos $relative_pos $receiving_item       
    }
    method _move_rename_group { id sibling_name relative_pos } {               
        set name [$self give_full_group_name $id]
        set childs_id [$tree item children $id]
        
        if { $relative_pos eq "center" } {
            set new_parent_nameList [cu::splitx $sibling_name {//}]            
        } else {
            set new_parent_nameList [lrange [cu::splitx $sibling_name {//}] 0 end-1]
        }
        set old_parent_nameList [lrange [cu::splitx $name {//}] 0 end-1]
        set newnameList $new_parent_nameList
        lappend newnameList [lindex [cu::splitx $name {//}] end]
        set newname [join $newnameList //]
        if { $new_parent_nameList ne $old_parent_nameList } {
            if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                gid_groups_conds::rename_group -noactualizegroups $name $newname
            } else {
                gid_groups_conds::rename_group -noactualizegroups $name $newname  
                switch $relative_pos {
                    prev { set before_after before }
                    next - center { set before_after after }
                }
#                 gid_groups_conds::move_group -noactualizegroups $sibling_name \
#                     $newname $before_after                
                foreach id_child $childs_id {
                    $self _move_rename_group $id_child $newname center                  
                }
            }
        }
    }        
    method _move_rename_layer { id sibling_name relative_pos } {      
        set name [$self give_full_layer_name $id]  

        if { $relative_pos eq "center" } {
            set new_parent_nameList [cu::splitx $sibling_name {//}]            
        } else {
            set new_parent_nameList [lrange [cu::splitx $sibling_name {//}] 0 end-1]
        }
        set old_parent_nameList [lrange [cu::splitx $name {//}] 0 end-1]
        set newnameList $new_parent_nameList
        if { $newnameList != "" } {
            lappend newnameList [lindex [cu::splitx $name {//}] end]
            set newname [join $newnameList //]
            if { $new_parent_nameList ne $old_parent_nameList } {
                $self rename_lay_mod $name $newname
            }  
            GiD_Process 'Layers ChangeParent $name $sibling_name escape
        } else {
            $self rename_lay_mod $name [lindex [cu::splitx $name {//}] end]
            GiD_Process 'Layers ChangeParent $name "" escape
        }      
    }    
    method rename_lay { args } {
        variable doc              
              
        set optional {        
            { -undo boolean 1 }
            { -merge_if_exists "" 0 }
        }
        set compulsory "oldname newname"
        parse_args $optional $compulsory $args
    
        if { $oldname eq $newname } { return }
        
        if { ![regexp {^[-.\w\s/]+$} $newname] && ![regexp {^[-.\w\s/#]+$} $newname] && \
            ![regexp {^[-.\w\s/]} $newname]} {
            return -code error [_ "error: A layer name can only contain letters, digits, space, and _-./"]
        }        
        
        if { $newname in [GiD_Info Layers] } {
            if { !$merge_if_exists } {
                return -code error [_ "error: layer '%s' already exists" $newname]
            } else {
                set exists_lay 1
            }
        } else {
            set exists_lay 0
        }
        
#         if { $newname == ""} return
#         if { $item == 0 } {
#             return -code error [_ "Root layer can't be edited"]      
#         }
        #     set fullname [DecodeName [$T item tag names $item]]
        #     set parts [GidUtils::Split $fullname //]
        #     lset parts end $newtext
        #     set newname [join $parts //]
        if { $oldname != $newname } {
            GiD_Process 'Layers ChangeName $oldname $newname escape
            GiD_Process 'Redraw
        }         
    }
    method rename_lay_mod { args } {
        variable doc              
              
        set optional {        
            { -undo boolean 1 }
            { -merge_if_exists "" 0 }
        }
        set compulsory "oldname newname"
        parse_args $optional $compulsory $args
    
        if { $oldname eq $newname } { return }
        
        if { ![regexp {^[-.\w\s/]+$} $newname] && ![regexp {^[-.\w\s/#]+$} $newname] && \
            ![regexp {^[-.\w\s/]} $newname]} {
            return -code error [_ "error: A layer name can only contain letters, digits, space, and _-./"]
        }        
        
        if { $newname in [GiD_Info Layers] } {
            if { !$merge_if_exists } {
                return -code error [_ "error: layer '%s' already exists" $newname]
            } else {
                set exists_lay 1
            }
        } else {
            set exists_lay 0
        }                       
    }
    method move_group { args } {
        
        set optional {
            { -itemList list "" }
            { -relative_pos prev|center|next|"" "" }
        }
        set compulsory "where"
        parse_args $optional $compulsory $args

        if { ![llength $itemList] } { set itemList [$tree selection get] }
        
        set ids ""
        foreach id $itemList {
            set found 0
            foreach idp [lrange [$tree item ancestors $id] 0 end-1] {
                if { $idp in $ids } {
                    set found 1
                    break
                }
            }
            if { !$found } { lappend ids $id }
        }
        set ids [lsort $ids]

        switch $where {
            up {
                set sibling [$tree item prevsibling [lindex $ids 0]]
                set relative_pos prev
            }
            top {
                set sibling [$tree item firstchild 0]
                set relative_pos prev
            }
            down {
                set sibling [$tree item nextsibling [lindex $ids end]]
                set relative_pos next
            }
            bottom {
                set sibling [$tree item lastchild 0]
                set relative_pos next
            }
            default {
                set sibling $where
                if { $relative_pos eq "" } {
                    set sibling_row [lindex [$tree item rnc $sibling] 0]
                    set ids0_row [lindex [$tree item rnc [lindex $ids 0]] 0]
                    set idsend_row [lindex [$tree item rnc [lindex $ids end]] 0]
                    if { $sibling_row < $ids0_row } {
                        set relative_pos prev
                    } elseif { $sibling_row > $idsend_row } {
                        set relative_pos next
                    } else {
                        return
                    }
                }
            }
        }
        if { $sibling eq "" || [lsearch -integer $ids $sibling] != -1 } { return }
        
        if { $relative_pos eq "next" } { set ids [lreverse $ids] }
        
        if {$options(-groups)} {
            set sibling_name [$self give_full_group_name $sibling]
        } elseif {$options(-layers)} {
            set sibling_name [$self give_full_layer_name $sibling]
        }        
        foreach id $ids {
            if {$options(-groups)} {                          
                set err [catch { $self _move_rename_group $id \
                                $sibling_name $relative_pos } errstring]               
            } elseif {$options(-layers)} {
                set err [catch { $self _move_rename_layer $id \
                            $sibling_name $relative_pos } errstring]
            }
            if { $err } {
                GidUtils::SetWarnLine [_ "%s" $errstring] 
                return
            }
            switch $relative_pos {
                prev {
                    $tree item prevsibling $sibling $id
                }
                next {
                    $tree item nextsibling $sibling $id 
                }
                center { 
                    if { !$::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                        $tree item lastchild $sibling $id        
                    }           
                }
            }          
        } 
        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
            if {$options(-groups)} {
                gid_groups_conds::actualize_groups_window      
            } elseif {$options(-layers)} {
                $self actualize_layers
            }
        }
    }
    method change_color_group { { id "" } } {
        if {$id != ""} { $self add_selection $id }        
        set color [lindex [$tree item element cget active 0 e_col_rect -fill] 0]
        set color [tk_chooseColor -initialcolor $color -parent $win \
                -title [_ "Choose color"]]
        if { $color eq "" } { return }
        
        set colorState [list $color]
        
        set gList [$self give_full_group_name_and_children [$tree selection get]]

        foreach i $gList {
            lassign $i item group
            if { $group ne "" } {
                gid_groups_conds::change_group_att -noactualizegroups \
                    $group color $color
            }
            $tree item element configure $item 0 e_col_rect -fill $colorState
        }
    }
    method change_color_layer { { id "" } } {
        if { $id != "" } { $self add_selection $id }
        set color [lindex [$tree item element cget active 0 e_col_rect -fill] 0]
        set retcol [tk_chooseColor -initialcolor $color -parent $win \
                -title [_ "Choose color"]]
        if { $retcol eq "" } { return }
               
        set colorState [list $retcol]
        
        set gList [$self give_full_layer_name_and_children [$tree selection get]]
        set LayersSelection [$self get_selected_layernames]
        foreach i $gList {
            lassign $i item layer
            if { $layer ne "" } {
                GiD_Process 'Layers
#                 set retcol [GIDChooseColor .gid.selectcolor -title [_ "Select color"] \
#                     -color $color]
                    
                set retcol [string range $retcol 1 end]
                if { [string length $retcol] == 8 } {
                    scan $retcol %2x%2x%2x%2x col1 col2 col3 alpha
                    set col [format %03i%03i%03i $col1 $col2 $col3]
                    foreach i $LayersSelection {
                        GiD_Process Color $i $col Transparent $i $alpha
                    }
                } else {
                    scan $retcol %2x%2x%2x col1 col2 col3
                    set col [format %03i%03i%03i $col1 $col2 $col3]
                    foreach i $LayersSelection {
                        GiD_Process Color $i $col
                    }
                }                           
                GiD_Process escape                                
            }
            $tree item element configure $item 0 e_col_rect -fill $colorState
        }
    }
    method add_selection { id } {
        if { $id == "" } { return }   
        if { [llength [$tree selection get]] == "1" } {
            $tree selection clear all
            $tree selection add $id                
        }            
        $tree activate $id       
    }  
    method set_to_onoff_gr { id } {
        if { $id != "" } { $self add_selection $id }

        set img [gid_themes::GetImage ok small_icons]
        # GiD change: only next line 
        set img [$tree item element cget $id 1 e_image -image]
        if { $img eq "blank-16"} {
            set img [ groups_theme::GetAlternativeThemedImage $img check-16]
            set ison 1
        } else {
            set img blank-16
            set ison 0
        }                 
        
        set gList [$self give_full_group_name_and_children [$tree selection get]]
        set gList [lsort -index 1 $gList]
        foreach i $gList {
            lassign $i item group                
            if { $group ne "" } {
                $tree item element configure $item 1 e_image -image $img
#                 gid_groups_conds::change_group_att -noactualizegroups $group onoff $ison
                gid_groups_conds::setAttributes [format_xpath {groups/group[@n=%s]} $group] \
                    [list onoff $ison]
            }                    
        }               
        set itemList [$tree range 0 end]
        set g_allList [$self give_full_group_name_and_children $itemList]
        set g_allList [lsort -index 1 $g_allList]
        
        set gid_version [GiD_Info GiDVersion]
        if { [regexp {(.*)d} $gid_version {} version_num] } {
            set version_num $version_num
        } else {
            set version_num [lindex [split $gid_version .] 0] 
        }
        if { $version_num >= "14.1.7" } {
            $self update_groups_visible $gList $g_allList $ison                
        } else {
            $self update_groups_visible_old $gList $g_allList 
        }
        set win .gid.central.boundaryconds.gg
        gid_groups_conds::gridremove_open_all_window $win 1
    }
    method set_to_onoff_gr_original { id } {  
        
        if { $id != "" } { $self add_selection $id }
                
        set img [$tree item element cget active 1 e_image -image]
        if { $img eq "blank-16" } {           
            set img [gid_themes::GetImage ok small_icons]
            # GiD change: only next line                 
            set img [groups_theme::GetAlternativeThemedImage $img check-16]
            set ison 1
        } else {
            set img blank-16
            set ison 0
        }       
        
        set gList [$self give_full_group_name_and_children [$tree selection get]]
        set gList [lsort -index 1 $gList]
        
        GidUtils::DisableGraphics
        foreach i $gList {
            lassign $i item group
            if { $group ne "" } {
                gid_groups_conds::change_group_att -noactualizegroups $group onoff $ison
            }
            if { $ison } { 
                set img [gid_themes::GetImage ok small_icons]
                # GiD change: only next line
                set img [ groups_theme::GetAlternativeThemedImage $img check-16]
            } else { 
                set img blank-16 
            }
            $tree item element configure $item 1 e_image -image $img
        }
        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
            #nothing to do
            GidUtils::EnableGraphics
            GiD_Process 'Redraw
        } else {
            if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                set geom_mesh mesh
            } else {
                set geom_mesh geometry
            }
            if { $ison } {
                set groups ""
                set root [$options(-domdoc) documentElement]
                foreach groupNode [$root selectNodes groups/group] {
                    if { [$groupNode @onoff] == 0 } {
                        lappend groups [$groupNode @n]
                    }
                }
                set groups [lsort $groups]
                GiD_Process 'Layers BringToFrontAll Mescape
                foreach i [list point line surface volume] {
                    set ents ""
                    foreach j [GiD_Info conditions ${i}_groups $geom_mesh] {
                        lassign $j - num - group
                        if { [lsearch -exact -sorted $groups $group] != -1 } {
                            lappend ents $num
                        }
                    }
                    set ents [lsort -integer -unique $ents]
                    if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                        switch $i {
                            point { set etype Nodes }
                            default { set etype Elements }
                        }
                    } else {
                        set etype $i
                    }
                    # WARNING: what happens with element's faces?               
                    GiD_Process Layers SendToBack ${etype}s {*}$ents escape
                }
            } else {
                foreach i [list point line surface volume] {
                    set ents ""
                    foreach j [GiD_Info conditions ${i}_groups $geom_mesh] {
                        lassign $j - num - group
                        if { [lsearch -exact -sorted -index 1 $gList $group] != -1 } {
                            lappend ents $num
                        }
                    }
                    set ents [lsort -integer -unique $ents]
                    if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                        switch $i {
                            point { set etype Nodes }
                            default { set etype Elements }
                        }
                    } else {
                        set etype $i
                    }
                    # WARNING: what happens with element's faces?              
                    GiD_Process Layers SendToBack ${etype}s {*}$ents escape
                }
            }
            GidUtils::EnableGraphics
            GiD_Process 'Redraw
        }
    }                                                                      
    method send_entities { what w { ovname "" } { ov "" } } { 

        set ids [$tree selection get]
        if {$options(-groups)} {
            set id [_ "group"]
            if { ([llength $ids] == 0 || [llength $ids] > 1) } {
                if { $what ne "out" || !$::CUSTOM_LIB_USE_NATIVE_GROUPS } {                     
                    snit_messageBox -parent $tree -message [_ "It is necessary to select one %s" $id] 
                    return
                } 
            }
        } elseif {$options(-layers)} {
            set id [_ "layer"]
            if { [llength $ids] == 0 || [llength $ids] > 1} {                     
                snit_messageBox -parent $tree -message [_ "It is necessary to select one %s" $id] 
                return
            }
        }                  
        if {$options(-groups)} {
            set gList [$self give_full_group_name_list $ids]
        } else {
            set gList [$self give_full_layer_name_list $ids]
        }        
        if { $ov eq "" } {
            set menu $win.menu
            if {[winfo exists $menu]} {
                destroy $menu                
            }
            menu $menu -tearoff 0
           
            set gid_version [GiD_Info GiDVersion]
            if { [regexp {(.*)d} $gid_version {} version_num] } {
                set version_num $version_num
            } else {
                set version_num [lindex [split $gid_version .] 0] 
            }
            set LayersAlsoLower 0
            if { $version_num >= "14.1.9" } {
                set ::GidPriv(LayersAlsoLower) [GiD_Set LayersAlsoLower]
            } 
            if { $options(-layers) } {
                $menu add check -label [_ "Also lower entities"] -variable ::GidPriv(LayersAlsoLower) \
                    -command {GiD_Set LayersAlsoLower $::GidPriv(LayersAlsoLower)}
                if {$what eq "out"} {
                    $menu add check -label [_ "Opposite"] -variable ::GidPriv(LayersBackOpposite)
                }   
                $menu add separator
            }
            switch [lindex [GiD_Info Project] 4] {
                GEOMETRYUSE {
                    set ents [list Points point point [_ Points] Lines line line [_ Lines] \
                    Surfaces surface surface [_ Surfaces] Volumes volume volume [_ Volumes]]
                }
                MESHUSE {
                    if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                        set ents [list Nodes node node [_ "Nodes"] \
                                "L Elements" element element [_ "L Elements"] \
                                "S Elements" element element [_ "S Elements"] \
                                "V Elements" element element [_ "V Elements"]]
                    } else { 
                        set ents [list Nodes point node [_ "Nodes"] \
                                "L Elements" line element [_ "L Elements"] \
                                "S Elements" surface element [_ "S Elements"] \
                                "V Elements" volume element [_ "V Elements"]]
                    }
                }
            }
            if { $options(-layers) } {
                set type "_layers"
            } else {
                set type "_groups"
            }
            foreach "name n img label" $ents {
                $menu add command -label $label -command \
                    [mymethod send_entities_do$type $what $name $n $gList] -image \
                    [gid_themes::GetImage $img small_icons] -compound left
            }
            
            if { $options(-layers) } {              
                $menu add separator
                if {$what eq "in"} {
                    set name [_ "Dimensions"] 
                    $menu add command -label $name -command \
                        [mymethod send_entities_do$type $what $name Dimensions $gList] -image \
                        selectent -compound left
                    set name [_ "All"] 
                    $menu add command -label $name -command \
                        [mymethod send_entities_do$type $what $name All $gList] -image \
                        selectent -compound left                                       
                } else {
                    set name [_ "All"]
                    $menu add command -label $name -command \
                        [mymethod send_entities_do$type $what $name All $gList] -image \
                        backlayer -compound left
                    $menu add separator
                    if { [GiD_Info layers -hasbacklayers] } {
                        $menu add command -label [_ "Bring to front"] \
                            -command [mymethod send_layers_to] \
                            -image backlayer -compound left
                        $menu add command -label [_ "Bring all to front"] \
                            -command [mymethod send_all_layers_to] \
                            -image backlayer -compound left
                    } else {
                        $menu add command -label [_ "Bring to front"] -state disabled \
                            -image backlayer -compound left
                        $menu add command -label [_ "Bring all to front"] -state disabled \
                            -image backlayer -compound left
                    }   
                }
            }            
            set x [expr {[winfo rootx $w]}]
            set y [expr {[winfo rooty $w]+[winfo height $w]}]
            cu::tooltip cancel $w
            tk_popup $menu $x $y
        } else {
            if {$options(-groups)} {
                $self send_entities_do_groups $what $ovname $ov $gList
            } elseif {$options(-layers)} {
                $self send_entities_do_layers $what $ovname $ov $gList
            }
        }
    }
    method set_layer_as_selection { layer } {      
        $tree selection clear
        set item [$tree item id "tag [$self EncodeName $layer]"]
        if { $item != "" } {
            $tree selection add $item
            $tree see $item
        }
    }
    method send_layers_to {} {
        $self set_layers_to BRINGTOFRONT 
    }
    method send_all_layers_to {} {
        GiD_Process 'Layers BringToFrontAll Mescape 
        Layers::FillInfo
    }    
    method send_entities_do_layers { what ovtype_name ovtype gList } {

        if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
            switch $ovtype {
                point { set ovseltype node }
                line - surface - volume { set ovseltype element }
                node { set ovseltype node }
                element { set ovseltype element }
            }
            set geom_mesh mesh
        } else {
            set ovseltype $ovtype
            set geom_mesh geometry
        }
        switch $what {
            in {
                if { [llength $gList] == 1 } {
                    set txt [_ "Select %s to add to layer '%s'" \
                            $ovtype_name [lindex $gList 0 1]]
                } else {
                    set groups_pr ""
                    foreach i [lrange $gList 0 6] {
                        lappend groups_pr [lindex $gList 1]
                    }
                    if { [llength $gList] > 7 } { append groups_pr ... }
                    set txt [_ "Select %s to add to layers '%s'" \
                            $ovtype_name $groups_pr]
                }
            }
            out {
                if { [llength $gList] == 1 } {
                    set txt [_ "Select %s to delete from layer '%s'" \
                            $ovtype_name [lindex $gList 0 1]]
                } else {
                    set groups_pr ""
                    foreach i [lrange $gList 0 6] {
                        lappend groups_pr [lindex $gList 1]
                    }
                    if { [llength $gList] > 7 } { append groups_pr ... }

                    set txt [_ "Select %s to delete from layers '%s'" \
                            $ovtype_name $groups_pr]
                }
            }
        }
        
        if { $ovtype eq "Dimensions" || $ovtype eq "All" } {
            set ovseltype $ovtype
        }
        GidUtils::SetWarnLine $txt
        GidUtils::DisableWarnLine              
        GiD_Process Mescape 'SelectEntities ${ovseltype}s                                       
        GidUtils::EnableWarnLine
        
        if { $options(-open_over) ne "" && [winfo class $options(-open_over)] eq "Button" } {
            set b0 $options(-open_over)
            set scr ""
            append scr "[list $b0 configure -image [$b0 cget -image]];"\
                "[list $b0 configure -command [$b0 cget -command]];"\
                "[list wm deiconify [winfo toplevel $win]];"\
                "grab $win"
            $b0 configure -image $gid_groups_conds::images(locationbar_erase-22) \
                -command \
                [mymethod end_send_entities_do $scr $b0]
            set b $b0
            
            wm withdraw [winfo toplevel $win]
            grab release $win
        } else {
            set t $win
            set len 0
            set txt1 [_ "End"]
            foreach i [list $txt1] {
                if { [string length $i] > $len } {
                    set len [string length $i]
                }
            }
            incr len 5
            set f [ttk::frame $t.finish -style Mod.ForcedFrame]
            set scr "[list destroy $f]; raise $t"
            set b [ttk::button $f.b -text [_ "End"] -image colorize_end \
                    -compound left -width $len -style Toolbutton \
                    -command [mymethod end_send_entities_do $scr $f.b]]           
            grid $b -padx 5 -pady 0 -sticky nw
            cu::tooltip register $b [_ "Click on this button or Escape to finish the draw process"]
            
            grid columnconfigure $f 0 -weight 1
            grid rowconfigure $f 0 -weight 1            
            
            place $f -in $t -x 0 -y -0 -anchor nw -width \
                [winfo width $t] -height [winfo height $t]
            #grab $b
            bind $b <Escape> "[list $b invoke]; break"
            bind $b <Return> [list $b invoke]
            focus $b            
        }
        delayedop changefunc [list $b invoke]
        vwait ::GidPriv(selection)
        set entities [lrange [lindex $::GidPriv(selection) 0] 1 end]
        
        switch $what {
            in {
                $self add_entities_into_layers $entities $gList $ovtype $ovtype_name              
            }
            out {
                $self add_entities_into_back $entities $gList $ovtype $ovtype_name        
            }
        }
    }  
    method add_entities_into_layers { entities gList ovtype ovtype_name } { 
        if { $entities == "" } { return }        
        foreach i $gList {
            lassign $i - layer
            if {$layer != ""} {  
                set gidversion [GiD_Info GiDVersion]
                if { [regexp {(.*)d} $gidversion {} version_num] } {
                    set version_num $version_num
                } else {
                    set version_num $gidversion
                }
                set LayersAlsoLower 0
                if { $version_num >= "14.1.9" } {
                    if { [GiD_Set LayersAlsoLower] } {
                        set LayersAlsoLower 1
                    } 
                } else {
                    if { $::GidPriv(LayersAlsoLower) } {
                        set LayersAlsoLower 1
                    } 
                }  
                if { $LayersAlsoLower } {
                    GiD_Process 'Layers Entities $layer LowerEntities ${ovtype}s {*}$entities Mescape
                } else {                    
                    GiD_Process 'Layers Entities $layer ${ovtype}s {*}$entities Mescape
                }
            }
        }       
    }   
    method add_entities_into_back { entities gList ovtype ovtype_name } {         
        if { $entities == "" } { return }
        set gid_version [GiD_Info GiDVersion]
        if { [regexp {(.*)d} $gid_version {} version_num] } {
            set version_num $version_num
        } else {
            set version_num [lindex [split $gid_version .] 0] 
        }
        if { $version_num >= "14.1.9" } {
            set LayersAlsoLower [GiD_Set LayersAlsoLower]
        } else {
            set LayersAlsoLower $::GidPriv(LayersAlsoLower) 
        }
        foreach i $gList {
            lassign $i item layer
            if {$layer != ""} {          
                if { $::GidPriv(LayersBackOpposite) } {
                    if { $LayersAlsoLower } {
                        GiD_Process 'Layers SendToBackOpposite LowerEntities ${ovtype}s {*}$entities Mescape
                    } else {
                        GiD_Process 'Layers SendToBackOpposite ${ovtype}s {*}$entities Mescape
                    }
                    set itemList [$tree range 0 end]
                    foreach item $itemList {
                        if {$item ne "0"} {
                            $tree item element configure $item 5 e_image -image back
                        }
                    }
                } else {
                    if { $LayersAlsoLower } {
                        GiD_Process 'Layers SendToBack LowerEntities ${ovtype}s {*}$entities Mescape
                    } else {
                        GiD_Process 'Layers SendToBack ${ovtype}s {*}$entities Mescape
                    }
                    $tree item element configure $item 5 e_image -image back
                }                                                                                          
            }
        }                    
    }
    method send_entities_do_groups { what ovtype_name ovtype gList } {

        if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
            switch $ovtype {
                point { set ovseltype node }
                line - surface - volume { set ovseltype element }
                node { set ovseltype node }
                element { set ovseltype element }
            }
            set geom_mesh mesh
        } else {
            set ovseltype $ovtype
            set geom_mesh geometry
        }
        switch $what {
            in {
                if { [llength $gList] == 1 } {
                    set txt [_ "Select %s to add to group '%s'" \
                            $ovtype_name [lindex $gList 0 1]]
                } else {
                    set groups_pr ""
                    foreach i [lrange $gList 0 6] {
                        lappend groups_pr [lindex $gList 1]
                    }
                    if { [llength $gList] > 7 } { append groups_pr ... }
                    set txt [_ "Select %s to add to groups '%s'" \
                            $ovtype_name $groups_pr]
                }
            }
            out {
                if { [llength $gList] == 1 } {
                    set txt [_ "Select %s to delete from group '%s'" \
                            $ovtype_name [lindex $gList 0 1]]
                } else {
                    set groups_pr ""
                    foreach i [lrange $gList 0 6] {
                        lappend groups_pr [lindex $gList 1]
                    }
                    if { [llength $gList] > 7 } { append groups_pr ... }

                    set txt [_ "Select %s to delete from groups '%s'" \
                            $ovtype_name $groups_pr]
                }
            }
        }
        GidUtils::SetWarnLine $txt
        GidUtils::DisableWarnLine              
        GiD_Process Mescape 'SelectEntities ${ovseltype}s
        GidUtils::EnableWarnLine
        
        if { $options(-open_over) ne "" && [winfo class $options(-open_over)] eq "Button" } {
            set b0 $options(-open_over)
            set scr ""
            append scr "[list $b0 configure -image [$b0 cget -image]];"\
                "[list $b0 configure -command [$b0 cget -command]];"\
                "[list wm deiconify [winfo toplevel $win]];"\
                "grab $win"
            $b0 configure -image $gid_groups_conds::images(locationbar_erase-22) \
                -command \
                [mymethod end_send_entities_do $scr $b0]
            set b $b0
            
            wm withdraw [winfo toplevel $win]
            grab release $win
        } else {
            set t $win
            set len 0
            set txt1 [_ "End"]
            foreach i [list $txt1] {
                if { [string length $i] > $len } {
                    set len [string length $i]
                }
            }
            incr len 5
            set f [ttk::frame $t.finish -style Mod.ForcedFrame]
            set scr "[list destroy $f]; raise $t"
            set b [ttk::button $f.b -text [_ "End"] -image colorize_end \
                    -compound left -width $len -style Toolbutton \
                    -command [mymethod end_send_entities_do $scr $f.b]]           
            grid $b -padx 5 -pady 0 -sticky nw
            cu::tooltip register $b [_ "Click on this button or Escape to finish the draw process"]
            
            grid columnconfigure $f 0 -weight 1
            grid rowconfigure $f 0 -weight 1            
            
            place $f -in $t -x 0 -y -0 -anchor nw -width \
                [winfo width $t] -height [winfo height $t]
            #grab $b
            bind $b <Escape> "[list $b invoke]; break"
            bind $b <Return> [list $b invoke]
            focus $b            
        }
        delayedop changefunc [list $b invoke]
        vwait ::GidPriv(selection)
        set entities [lrange [lindex $::GidPriv(selection) 0] 1 end]
        
        switch $what {
            in {
                gid_groups_conds::add_entities_to_groups $entities $gList $ovtype               
            }
            out {
                gid_groups_conds::delete_entities_from_groups $entities $gList $ovtype
            }
        }
    }        
    method end_send_entities_do { scr b } {

        set found [delayedop -cancel changefunc [list $b invoke]]
        if { $found } { GiD_Process escape }

        eval $scr
        # GiD_Process Redraw
    }
    method groups_info_check {} {
        if { [gid_groups_conds::is_popup_help_active $tree] } {
            $self groups_info
        }
    }
    method groups_info {} {
        set gList [$self give_full_group_name_and_children [$tree selection get]]
        if { ![llength $gList] } { return }
        
        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
            #count after
        } else {
            foreach geommesh [list geometry mesh] {
                foreach ovtype [list point line surface volume] {
                    foreach g $gList {
                        set group [lindex $g 1]
                        if { $group ne "" } {
                            set num_ents($geommesh,$ovtype,$group) 0
                        }
                    }
                    foreach j [GiD_Info conditions ${ovtype}_groups $geommesh] {
                        lassign $j - num - group
                        if { [lsearch -exact -index 1 $gList $group] == -1 } { continue }
                        incr num_ents($geommesh,$ovtype,$group)
                    }
                }
            }
        }
        set root [$options(-domdoc) documentElement]
        set text ""
        set line 0
        foreach g $gList {
            lassign $g - group
            if { $group eq "" } { continue }
            if { $text ne "" } { append text "\n\n" }
            append text "[_ "group"]: $group"
            set lineL 2
            
            if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                foreach {name ovtype} [list [_ "Points"] point [_ "Lines"] line [_ "Surfaces"] surface  [_ "Volumes"] volume \
                        [_ "Nodes"] node [_ "Elements"] element [_ "Faces"] face] {
                    set count [GiD_EntitiesGroups get $group ${ovtype}s -count]
                    if { $count } {
                        append text "\n\t# $name: $count" 
                        incr lineL
                    }
                }
            } else {                
                foreach {name ovtype} [list [_ "Points"] point [_ "Lines"] line [_ "Surfaces"] surface  [_ "Volumes"] volume] {
                    if { $num_ents(geometry,$ovtype,$group) > 0 } {
                        append text "\n\t# $name: $num_ents(geometry,$ovtype,$group)" 
                        incr lineL
                    }
                }
                if { $num_ents(mesh,point,$group) > 0 } {
                    append text "\n\t# [_ "Nodes"]: $num_ents(mesh,point,$group)" 
                    incr lineL
                }
                if { $num_ents(mesh,line,$group) > 0 || $num_ents(mesh,surface,$group) > 0 || \
                    $num_ents(mesh,volume,$group) > 0} {
                    set num [expr {$num_ents(mesh,line,$group)+$num_ents(mesh,surface,$group)+\
                            $num_ents(mesh,volume,$group)}]
                    append text "\n\t# [_ "Elements"]: $num" 
                    incr lineL
                }
            }
            if { $lineL == 2 } {
                append text [concat "\n\t" [_ "No entities"]]
                incr lineL
            }
            
            set cndnum 0
            set xp [format_xpath {//condition/group[@n=%s]} $group]
            foreach grpNode [$root selectNodes $xp] {
                if { [$grpNode selectNodes {ancestor::blockdata[@n="Internal data"]}] ne "" } { continue }
                if { !$cndnum } {
                    append text [concat "\n" [_ "Applied to conditions"]:]
                    incr lineL
                }
                append text "\n\t[[$grpNode parentNode] @pn]"
                incr lineL
                incr cndnum
            }
            incr line $lineL
            if { $line > 20 } {
                append text "\n\t..."
                break
            }
        }
        lassign [$tree item bbox [lindex $gList 0 0] 0 e_text_sel] x1 y1 x2 y2
        set x [expr {[winfo rootx $tree]+$x1+10}]
        set y [expr {[winfo rooty $tree]+$y2+4}]
        
        #         set x [expr {[winfo pointerx $tree]+15}]
        #         set y [expr {[winfo pointery $tree]+10}]
        
        gid_groups_conds::popup_help -minimum_time 2000 $tree $text $x $y
    }    
    method list_entities {} {
        
        set gList [$self give_full_group_name_and_children [$tree selection get]]
        if { ![llength $gList] } { return }

        #gid_groups_conds::list_entities $win $groups
        
        set gList [lsort -index 1 $gList]
        set domNodes ""
        foreach domNode [$options(-domdoc) selectNodes /*/groups/group] {
            if { [lsearch -sorted -index 1 $gList [$domNode @n]] != -1 } {
                lappend domNodes $domNode
            }
        }
        gid_groups_conds::list_entities_cnd $win $domNodes        
    }
    method update_groups_show_hide { show_hide } {           
        set gList [$self give_full_group_name_and_children [$tree range 0 end]]
        set gList [lsort -index 1 $gList]
        
        if { $show_hide eq "show" } {           
            set image "check-16"
            set ison 1 
        } else {           
            set image "blank-16"
            set ison 0
        }
        
        foreach i $gList {
            lassign $i item group
            if { $group ne "" } {
                $tree item element configure $item 1 e_image -image $image
#                 gid_groups_conds::change_group_att -noactualizegroups $group onoff $ison
                gid_groups_conds::setAttributes [format_xpath {groups/group[@n=%s]} $group] \
                    [list onoff $ison]
            }
        }
        
        GidUtils::DisableGraphics
        if { $show_hide eq "show" } { 
            array set entities_visible ""
            foreach i [list point line surface volume] {          
                set entities_visible($i,$i) ""
            }     
            if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                set geom_mesh mesh
            } else {
                set geom_mesh geometry
            }       
            foreach i $gList {
                lassign $i item group                
                if { $group ne "" } {                                                                       
                    foreach i [list point line surface volume] {
                        if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                            switch $i {
                                point { set etype node }
                                default { set etype element }
                            }
                        } else {                        
                            set etype $i
                        }
                        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {                    
                            set ent [GiD_EntitiesGroups get $group ${i}s]
                            if { $ent ne "" } {
                                lappend entities_visible($i,$etype) $ent                
                            }
                        } else {
                            set ent ""
                            foreach j [GiD_Info conditions ${i}_groups $geom_mesh] {
                                lassign $j num face - gr
                                if { $num in "E N" } {
                                    set num $face
                                } else {
                                    set num ""
                                }
                                if { $gr eq $group && $num ne "" } {
                                    lappend ent $num
                                }                           
                            }
                            if { $ent ne "" } {                                
                                lappend entities_visible($i,$etype) $ent                
                            }
                        }                                                  
                    }                                                
                } 
            }        
            foreach ient [array names entities_visible] {
                if {[regexp {(.*),(.*)} $ient {} i etype] } {  
                    if { $entities_visible($i,$etype) ne "" } {     
                        set entities_visible($i,$etype) [join $entities_visible($i,$etype) " "]                                      
                        set entities_visible($i,$etype) [lsort -integer -unique $entities_visible($i,$etype)]
                        GiD_EntitiesLayers assign_front_layer $geom_mesh ${etype}s $entities_visible($i,$etype)                        
                    }
                }
            }
        } else { 
            array set entities_not_visible ""
            foreach i [list point line surface volume] {          
                set entities_not_visible($i,$i) ""
            }     
            if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                set geom_mesh mesh
            } else {
                set geom_mesh geometry
            }       
            foreach i $gList {
                lassign $i item group                
                if { $group ne "" } {                                                                       
                    foreach i [list point line surface volume] {
                        if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                            switch $i {
                                point { set etype Nodes }
                                default { set etype Elements }
                            }
                        } else {                        
                            set etype $i
                        }
                        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {                    
                            set ent [GiD_EntitiesGroups get $group ${i}s]
                            if { $ent ne "" } {
                                lappend entities_not_visible($i,$etype) $ent                
                            }
                        } else {
                            set ent ""
                            foreach j [GiD_Info conditions ${i}_groups $geom_mesh] {
                                lassign $j num face - gr
                                if { $num in "E N" } {
                                    set num $face
                                } else {
                                    set num ""
                                }
                                if { $gr eq $group && $num ne "" } {
                                    lappend ent $num
                                }                           
                            }
                            if { $ent ne "" } {                                
                                lappend entities_not_visible($i,$etype) $ent                
                            }
                        }                                                  
                    }                                                
                } 
            }        
            foreach ient [array names entities_not_visible] {
                if {[regexp {(.*),(.*)} $ient {} i etype] } {  
                    if { $entities_not_visible($i,$etype) ne "" } {     
                        set entities_not_visible($i,$etype) [join $entities_not_visible($i,$etype) " "]                                      
                        set entities_not_visible($i,$etype) [lsort -integer -unique $entities_not_visible($i,$etype)]
                        GiD_Process 'Layers SendToBack ${etype}s {*}$entities_not_visible($i,$etype) Mescape
                    }
                }
            } 
        }  
        set backlayers_new [GiD_Info back_layers]   
        GiD_Redraw        
        GidUtils::EnableGraphics
        GiD_Process 'Redraw                
    }    
    method update_groups_visible_old { gList g_allList } {
        GidUtils::DisableGraphics              
        
        array set entities_visible ""
        array set entities_not_visible ""
        foreach i [list point line surface volume] {
            set entities_visible($i,$i) "" 
            set entities_visible($i,Node) ""
            set entities_visible($i,Element) ""
            set entities_not_visible($i,$i) ""
        }
        if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
            set geom_mesh mesh
        } else {
            set geom_mesh geometry
        }
        foreach igr $g_allList {
            lassign $igr item group   
            if { $item == "0" } { continue }
            set img [$tree item element cget $item 1 e_image -image]  
            if { $img eq "check-16" } {                                  
                foreach i [list point line surface volume] {           
                    if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                        switch $i {
                            point { set etype Nodes }
                            default { set etype Elements }
                        }
                    } else {                        
                        set etype $i
                    }    
                    if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {                    
                        set ent [GiD_EntitiesGroups get $group ${i}s]     
                        if { $ent ne "" } {
                            lappend entities_visible($i,$etype) $ent                
                        }
                    } else {
                        set ent ""
                        foreach j [GiD_Info conditions ${i}_groups $geom_mesh] {
                            lassign $j num face - gr
                            if { $num in "E N" } {
                                set num $face
                            } else {
                                set num ""
                            }
                            if { $gr eq $group && $num ne "" } {
                                lappend ent $num
                            }                           
                        }
                        if { $ent ne "" } {                                
                            lappend entities_visible($i,$etype) $ent                
                        }                                                
                    }                    
                }
            } 
        }
        GiD_Process 'Layers BringToFrontAll Mescape           
        foreach ient [array names entities_visible] {
            if {[regexp {(.*),(.*)} $ient {} i etype] } {  
                if { $entities_visible($i,$etype) ne "" } {     
                    set entities_visible($i,$etype) [join $entities_visible($i,$etype) " "]                                      
                    set entities_visible($i,$etype) [lsort -integer -unique $entities_visible($i,$etype)]
                    GiD_Process 'Layers SendToBackOpposite LowerEntities \
                        ${etype}s {*}$entities_visible($i,$etype) Mescape                                                
                }
            }            
        }
        GiD_Redraw
        GidUtils::EnableGraphics
        GiD_Process 'Redraw
    }
    method update_groups_visible { gList g_allList ison } {         

        GidUtils::DisableGraphics              
        
        array set entities_visible ""
        array set entities_not_visible ""
        foreach i [list point line surface volume] {
            set entities_visible($i,$i) "" 
            set entities_visible($i,Node) ""
            set entities_visible($i,Element) ""
            set entities_not_visible($i,$i) ""
        }
        if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
            set geom_mesh mesh
        } else {
            set geom_mesh geometry
        }
      
        if { !$ison } {
            foreach igr $gList {
                lassign $igr item group   
                if { $item == "0" } { continue }
                set img [$tree item element cget $item 1 e_image -image]  
                if { $img eq "blank-16" } {
                    foreach i [list point line surface volume] {                             
                        if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                            switch $i {
                                point { set etype node }
                                default { set etype element }
                            }
                        } else {                        
                            set etype $i
                        }    
                        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {                    
                            set ent [GiD_EntitiesGroups get $group ${i}s]                                 
                        } else {                        
                            set ent ""
                            foreach j [GiD_Info conditions ${i}_groups $geom_mesh] {
                                lassign $j num face - gr
                                if { $num in "E N" } {
                                    set num $face
                                } else {
                                    set num ""
                                }
                                if { $gr eq $group && $num ne "" } {
                                    lappend ent $num
                                }                           
                            }                                                                       
                        } 
                        if { $ent ne "" } {
                            lappend entities_not_visible($i,$i) "$ent"
                        } 
                    }                             
                }
            }
        } else {                     
            foreach igr $gList {
                lassign $igr item group   
                if { $item == "0" } { continue }
                set img [$tree item element cget $item 1 e_image -image]  
                if { $img eq "check-16" } {                                  
                    foreach i [list point line surface volume] {           
                        if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
                            switch $i {
                                point { set etype node }
                                default { set etype element }
                            }
                        } else {                        
                            set etype $i
                        }    
                        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {                    
                            set ent [GiD_EntitiesGroups get $group ${i}s]     
                            if { $ent ne "" } {
                                lappend entities_visible($i,$etype) $ent                
                            }
                        } else {
                            set ent ""
                            foreach j [GiD_Info conditions ${i}_groups $geom_mesh] {
                                lassign $j num face - gr
                                if { $num in "E N" } {
                                    set num $face
                                } else {
                                    set num ""
                                }
                                if { $gr eq $group && $num ne "" } {
                                    lappend ent $num
                                }                           
                            }
                            if { $ent ne "" } {                                
                                lappend entities_visible($i,$etype) "$ent"              
                            }                                                
                        }                    
                    }
                } 
            }
        }               
        foreach ient [array names entities_not_visible] {
            if {[regexp {(.*),(.*)} $ient {} i etype] } {  
                if { $entities_not_visible($i,$etype) ne "" } {     
                    set entities_not_visible($i,$etype) [join $entities_not_visible($i,$etype) " "]                                      
                    set entities_not_visible($i,$etype) [lsort -integer -unique $entities_not_visible($i,$etype)]
                    GiD_Process Layers SendToBack ${etype}s {*}$entities_not_visible($i,$etype) Mescape
                }
            }
        }                                           
        foreach ient [array names entities_visible] {
            if {[regexp {(.*),(.*)} $ient {} i etype] } {  
                if { $entities_visible($i,$etype) ne "" } {     
                    set entities_visible($i,$etype) [join $entities_visible($i,$etype) " "]                                      
                    set entities_visible($i,$etype) [lsort -integer -unique $entities_visible($i,$etype)]                              
                    GiD_EntitiesLayers assign_front_layer $geom_mesh ${etype}s $entities_visible($i,$etype)   
                }                        
            }
        } 
        set backlayers_new [GiD_Info back_layers]     
        
        GiD_Redraw                              
        GidUtils::EnableGraphics
        GiD_Process 'Redraw
    }
    method childs_group { parentname } {
        set childList ""
        foreach i [$tree item children root] {
            if {[$tree item text $i 0] eq $parentname } {
                set itemList [$self _give_tree_descendants $i]
                foreach item $itemList {
                    lappend childList [$tree item text $item 0]
                }
            }        
        }          
        return $childList
    }
    method select_all {} {
        $tree selection add all
    } 
    method layers_to_use { w } {
        set menu $w.menu
        if {[winfo exists $menu]} {
            destroy $menu            
        }
        menu $menu -tearoff 0
        
        switch [lindex [GiD_Info Project] 4] {
            GEOMETRYUSE {
                set ents [list [_ Points] point point Points [_ Lines] line line Lines \
                        [_ Surfaces] surface surface Surfaces [_ Volumes] volume volume Volumes]
            }
            MESHUSE {
                if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
                    set ents [list [_ Nodes] node node Nodes \
                            [_ "L Elements"] element element Elements \
                            [_ "S Elements"] element element Elements \
                            [_ "V Elements"] element element Elements]
                } else { 
                    set ents [list [_ Nodes] point node Nodes \
                            [_ "L Elements"] line element Elements \
                            [_ "S Elements"] surface element Elements \
                            [_ "V Elements"] volume element Elements]
                }
            }
        }   
        foreach "name n img what" $ents {
            $menu add command -label $name -command \
                [mymethod layers_to_use_do $what] -image \
                [gid_themes::GetImage $img small_icons] -compound left
        }
        set x [expr {[winfo rootx $w]}]
        set y [expr {[winfo rooty $w]+[winfo height $w]}]
        cu::tooltip cancel $w
        tk_popup $menu $x $y         
    }
    method layers_to_use_do { what } {
        GidUtils::SetWarnLine [_ "Pick one entity to set its layer as current layer to use"]
        GidUtils::DisableWarnLine
        set nnum [GidUtils::PickEntities $what single]
        GidUtils::EnableWarnLine
        if { $nnum != "" } {
            set retval [GiD_Info list_entities $what $nnum]
            set layername ""
            regexp {LAYER:([^\n\r]+)} $retval {} layername
            set layername [string trim $layername]
            if { $layername != "" } {
                GiD_Process 'Layers ToUse $layername
                Layers::FillInfo
                $self set_layer_as_selection $layername
            } else {
                WarnWin [_ "Selected entity does not belong to one layer"]
            }
        }    
    }  
    method selected_layers_to_groups { } {
        set tree_lay $win.ft.tl               
        if { [$tree_lay selection get] == "" } {                            
            set txt [_ "There are no selected layers"]
            set ret [snit_messageBox -type ok -default ok \
                    -message $txt -parent $win]
            return
        } else {
            set num [llength [$tree_lay selection get]]
            set txt [_ "Are you sure to create a group for every selected layer (%d layers) and add the layer geometrical/mesh entities into the group?" \
                $num]
            set ret [snit_messageBox -type okcancel -default ok \
                    -message $txt -parent $win]
            
            set list ""
            foreach id [$tree_lay selection get] {
                set layer_name [$self give_full_layer_name $id]
                lappend list $layer_name
            }          
            gid_groups_conds::layers_to_groups -layerList $list
        
#             $self actualize_groups   
        }
    }
    method layers_to_groups {} {
        
        set txt [_ "Are you sure to create a group for every layer and add the layer geometrical/mesh entities into the group?"]
        set ret [snit_messageBox -type okcancel -default ok \
                -message $txt -parent $win]
        if { $ret == "cancel" } { return }
        
        gid_groups_conds::layers_to_groups
        
#         $self actualize_groups
    }
    method groups_to_layers {} {
        if { [$tree selection get] eq "" } {
            set txt [_ "It is necessary to select one group"]
            snit_messageBox -type ok -parent $win -message $txt 
            return
        } else {
            set gList [$self give_full_group_name_and_children [$tree selection get]]
            set num [llength $gList]        
            set txt [_ "Are you sure to create a layer for every selected group (%d groups) and add the group geometrical/mesh entities into the layer?" \
                    $num]
        }
        set ret [snit_messageBox -type okcancel -default ok \
                -message $txt -parent $win]
        if { $ret eq "cancel" } { return }
        
        if { $::CUSTOM_LIB_USE_NATIVE_GROUPS } {
            set group_names [list]
            foreach igroup $gList {
                lassign $igroup treenode group_name
                if { $group_name eq "" } { continue } 
                lappend group_names $group_name   
            }
            Groups::GroupsToLayers $group_names
        }  else {
            GidUtils::DisableGraphics
            
            set frozen_layers [gid_groups_conds::_unfreeze_layers]
            set infoProj [GiD_Info Project ViewMode]
            
            foreach i $gList {
                lassign $i treenode group
                if { $group eq "" } { continue }
                if { $group ni [GiD_Layers list] } {
                    GiD_Process Layers New $group escape
                    set group_color [lindex [$tree item element cget $treenode 0 e_col_rect -fill] 0]
                    scan $group_color #%2x%2x%2x rrr ggg bbb
                    set layer_color [winfo rgb $tree $group_color]
                    set rrr [expr [lindex $layer_color 0]*255/65535]
                    set ggg [expr [lindex $layer_color 1]*255/65535]
                    set bbb [expr [lindex $layer_color 2]*255/65535]
                    set layer_color [format %03i%03i%03i $rrr $ggg $bbb]
                    GiD_Process 'Layers Color $group $layer_color escape
                }
                foreach geom_mesh [list geometry mesh] {
                    if { $infoProj eq "GEOMETRYUSE" && $geom_mesh eq "mesh" } {
                        MeshView 0
                    } elseif { $infoProj eq "MESHUSE" && $geom_mesh eq "geometry" } {
                        GeometryView
                    }
                    foreach i [list point line surface volume] {
                        set entities ""
                        foreach j [GiD_Info conditions ${i}_groups $geom_mesh] {
                            lassign $j num face - gr
                            if { $num in "E N" } {
                                set num $face
                            } else {
                                set num ""
                            }
                            if { $gr eq $group && $num ne "" } {
                                lappend entities $num
                            }
                        }
                        if { $entities != "" } {
                            GiD_Process 'Layers Entities $group ${i}s {*}$entities escape
                        }
                    }
                    if { $infoProj eq "GEOMETRYUSE" && $geom_mesh eq "mesh" } {
                        GeometryView
                    } elseif { $infoProj eq "MESHUSE" && $geom_mesh eq "geometry" } {
                        MeshView 0
                    }
                }
            }
            gid_groups_conds::_freeze_layers $frozen_layers
            GidUtils::EnableGraphics
            GidUtils::UpdateWindow LAYER    
        }
    }
}
































