
package require dialogwin
package require tooltip

proc gid_groups_conds::local_axes_menu { args } {
    variable is_draw_post
    
    set optional {
        { -ov_variable varname "" }
    }
    set compulsory "menu"
    parse_args $optional $compulsory $args
    
    
    #if { [$menu index end] ne "none" } { return }
    
    $menu delete 0 end
    
    if { !$is_draw_post } {
        set local_axes [GiD_Info localaxes]
    } else {
        set local_axes ""
    }
    $menu add command -label [_ Open]... -command gid_groups_conds::local_axes_window
    
    if { $ov_variable ne "" } {
        $menu add command -label [_ Draw]... -command \
            [list gid_groups_conds::_local_axes_draw_with_var $ov_variable]
    }
    
    $menu add separator
    
    foreach i [list Point Line Surface Volume] name [list [_ "Points"] [_ "Lines"] [_ "Surfaces"] [_ "Volumes"]] {
        $menu add cascade -label $name -menu $menu.m$i
        destroy $menu.m$i
        set m [menu $menu.m$i -tearoff 0]
        if { [lsearch "Line Surface Volume" $i] != -1 } {
            set needssep 0
            foreach local_axis_type {-Automatic- -Automatic_alt-} local_axis_type_label [list [_ "Automatic"] [_ "Automatic alt"] ] {
                if { [gid_groups_conds::condition_allow_local_axes ${i}_Local_axes $local_axis_type] } {
                    $m add command -label [_ "Assign %s" $local_axis_type_label] -command [list \
                            gid_groups_conds::_local_axes_assign ${i}_Local_axes $local_axis_type]
                    set needssep 1
                }
            }
            if { $needssep } {
                $m add separator
            }
        }
        set idx 0
        foreach j $local_axes {
            $m add command -label [_ "Assign '%s'" $j] -command [list \
                    gid_groups_conds::_local_axes_assign ${i}_Local_axes $j]
            incr idx
        }
        if { $idx } { $m add separator }
        $m add command -label [_ "Unassign"] -command [list \
                gid_groups_conds::_local_axes_unassign ${i}_Local_axes]
        
        $m add separator
        $m add command -label [_ "Draw"] -command [list GiD_Process \
                escape escape escape escape Data Conditions DrawCond -LocalAxes- \
                ${i}_Local_axes -draw-]
    }
    
    set ns [list [_ "Define#C#menu"] --- [_ "Draw#C#menu"] [_ "Draw all#C#menu"] \
            --- [_ "Delete#C#menu"] [_ "Delete all#C#menu"]]
    set cs {
        "Data LocalAxes DefineLocAxes"
        {} "Data LocalAxes DrawLocAxes"
        "Data LocalAxes DrawLocAxes -All-"
        {} "Data LocalAxes DeleteLA"
        "Data LocalAxes DeleteAllLA"
    }
    
    $menu add separator
    $menu add cascade -label [_ "Generic"] -menu $menu.mg
    destroy $menu.mg
    set m [menu $menu.mg -tearoff 0]
    
    foreach n $ns c $cs {
        if { $n eq "---" } {
            $m add separator
        } else {
            $m add command -label $n -command [concat "GiD_Process escape escape \
                    escape escape" $c]
        }
    }
}

proc gid_groups_conds::_local_axes_draw_with_var { varname } {
    upvar #0 $varname ov
    GiD_Process MEscape Data Conditions DrawCond -LocalAxes- \
        ${ov}_Local_axes -draw-
}

proc gid_groups_conds::_local_axes_assign { ov what } {
    
    GiD_Process MEscape Data Conditions AssignCond $ov change $what
    delayedop changefunc [list gid_groups_conds::_local_axes_assign_end]
}

proc gid_groups_conds::_local_axes_assign_end {} {
    set found [delayedop -cancel changefunc\
            [list gid_groups_conds::_local_axes_assign_end]]
    if { $found } { GiD_Process escape }
    
    gid_groups_conds::check_draw_nodes_state
}

proc gid_groups_conds::_local_axes_unassign { ov } {
    
    GiD_Process MEscape Data Conditions AssignCond $ov Unassign
    delayedop changefunc [list gid_groups_conds::_local_axes_unassign_end]
}

proc gid_groups_conds::_local_axes_unassign_end {} {
    set found [delayedop -cancel changefunc \
            [list gid_groups_conds::_local_axes_unassign_end]]
    if { $found } { GiD_Process escape }
    
    gid_groups_conds::check_draw_nodes_state
}

proc gid_groups_conds::_local_axes_unassign_window { w } {
    
    set etype [$w give_uservar_value entity_type]
    
    set txt [_ "Select entities to unassign local axes"]
    set cmd [list GiD_Process MEscape Data Conditions AssignCond ${etype}_Local_axes \
            Unassign]
    set end_cmd [list gid_groups_conds::check_draw_nodes_state]
    select_apply_entities -end_cmd $end_cmd $w $cmd $txt
}

proc gid_groups_conds::condition_allow_local_axes { condition_name local_axis_type } {
    
    set allow_local_axis_type 0
    if { [info command ::GiD_Info] eq "" } { return 0 }
    foreach {question value} [lrange [GiD_Info conditions $condition_name] 2 end] {
        if { [regexp -nocase {([^#]+)(?:#LA#(.*))?$} $question {} field type args] } {
            #type (-GLOBAL-,-Automatic-)
            set types [split [string range $type 1 end-1] ,]
            if { [lsearch $types $local_axis_type] != -1 } {
                set allow_local_axis_type 1
            }
            break
        }
    }
    return $allow_local_axis_type
}

proc gid_groups_conds::_local_axes_fill_menus { w a_menu ua_menu l_button d_button la_combo } {
    
    $a_menu delete 0 end
    
    set local_axes [GiD_Info localaxes]
    
    set end_cmd [list gid_groups_conds::check_draw_nodes_state]
    
    set etype [$w give_uservar_value entity_type]
    set needssep 0
    if { $etype eq "line" || $etype eq "surface" || $etype eq "volume" } {
        foreach local_axis_type {-Automatic- -Automatic_alt-} local_axis_type_label [list [_ "Automatic"] [_ "Automatic alt"] ] {
            if { [gid_groups_conds::condition_allow_local_axes ${etype}_Local_axes $local_axis_type] } {
                set txt [_ "Select entities to assign '%s' local axes" $local_axis_type_label]
                set cmd [list GiD_Process MEscape Data Conditions AssignCond ${etype}_Local_axes change $local_axis_type]
                $a_menu add command -label [_ "Assign %s" $local_axis_type_label] -command [namespace code \
                        [list select_apply_entities -end_cmd $end_cmd $w $cmd $txt]]
                set needssep 1
            }
        }                
    }
    set idx 0
    foreach j $local_axes {
        if { $needssep } {
            $a_menu add separator
            set needssep 0
        }
        set cmd [list GiD_Process MEscape Data Conditions AssignCond ${etype}_Local_axes change $j]
        set txt [_ "Select entities to assign '%s' local axes" $j]
        $a_menu add command -label [_ "Assign '%s'" $j] -command [namespace code \
                [list select_apply_entities -end_cmd $end_cmd $w $cmd $txt]]
        incr idx
    }
    
    if { $etype eq "point" && $idx == 0 } {
        $a_menu add command -label [_ "Define first a generic local axes system"] -state disabled
    }
    
    $ua_menu delete 0 end
    
    set cmd [list GiD_Process MEscape Data Conditions AssignCond ${etype}_Local_axes \
            Unassign]
    set txt [_ "Select entities to unassign local axes"]
    $ua_menu add command -label [_ "Unassign selection"] -command [namespace code \
            [list select_apply_entities -end_cmd $end_cmd $w $cmd $txt]]
    $ua_menu add command -label [_ "Unassign all"] -command [namespace code \
            [list _local_axes_unassign_all $w $etype]]    
    $l_button configure -command [list gid_groups_conds::_local_axes_list $w $etype]
    
    set cmd [list GiD_Process MEscape Data Conditions DrawCond -LocalAxes- \
            ${etype}_Local_axes -draw-]
    set txt [_ "Draw entities"]
    $d_button configure -command [namespace code \
            [list select_apply_entities $w $cmd $txt]]
    
    $la_combo configure -values $local_axes
    
    if { [lsearch -exact $local_axes [$la_combo get]] == -1 } {
        $la_combo delete 0 end
    }
}

proc gid_groups_conds::_local_axes_unassign_all { wp etype } {
    
    set text [_ "Are you sure to unassign all local axes?"]
    set retval [snit_messageBox -default ok -icon question -message $text \
            -type okcancel -parent $wp]
    if { $retval == "cancel" } { return }
    GiD_Process MEscape Data Conditions AssignCond ${etype}_Local_axes \
        DeleteAll Yes
    check_draw_nodes_state
}

proc gid_groups_conds::_local_axes_list_contextualmenu { tree items x y } {
    
    catch { destroy $tree._cmenu }
    set menu [menu $tree._cmenu -tearoff 0]
    
    $menu add command -label [_ "Select all"] \
        -command [list gid_groups_conds::_local_axes_list_contextualmenu_cmd \
            $tree selectall]
    $menu add separator
    $menu add command -label [_ "Copy"] \
        -command [list gid_groups_conds::_local_axes_list_contextualmenu_cmd \
            $tree copy]
    
    tk_popup $menu $x $y
}

proc gid_groups_conds::_local_axes_list_contextualmenu_cmd { tree what } {
    
    switch $what {
        selectall {
            $tree selection add all
        }
        copy {
            set data ""
            foreach i [$tree selection get] {
                lappend data [join [$tree item text $i] \t]
            }
            clipboard clear
            clipboard append [join $data \n]
        }
    }
}


proc gid_groups_conds::_local_axes_list { wp etype } {
    
    switch [lindex [GiD_Info Project] 4] {
        GEOMETRYUSE {
            set ents [dict create point [_ "Points"] line [_ "Lines"] surface [_ "Surfaces"] volume [_ "Volumes"]]
        }
        MESHUSE {
            set ents [dict create point [_ "Nodes"] line [_ "L Elements"] \
                    surface [_ "S Elements"] volume [_ "V Elements"]]
        }
    }
    set etype_txt [dict get $ents $etype]
    
    set w [dialogwin_snit $wp.%AUTO% -title [_ "List local axes for %s" $etype_txt] \
            -transient 1 -okname -]
    set f [$w giveframe]
    
    ttk::radiobutton $f.r1 -text [_ "Names"] -variable [$w give_uservar list_type] \
        -value name
    ttk::radiobutton $f.r2 -text [_ "Euler angles"] -variable [$w give_uservar list_type] \
        -value eulerangles
    ttk::radiobutton $f.r3 -text [_ "Transf. matrix"] -variable [$w give_uservar list_type] \
        -value matrix
    
    set toctree [fulltktree $f.toctree \
            -selecthandler2 "[list $w invokeok];#" \
            -contextualhandler [list gid_groups_conds::_local_axes_list_contextualmenu] \
            -expand 0 -width 500 -height 400 \
            -selectmode extended -showheader 1 -showlines 0  \
            -indent 0 -sensitive_cols all]
    
    grid $f.r1 $f.r2 $f.r3 -sticky w -padx 3 -pady 3
    grid $f.toctree - - -sticky nsew -padx 3 -pady 3
    grid columnconfigure $f 2 -weight 1
    grid rowconfigure $f 1 -weight 1
    
    tk::TabToWindow $f.toctree
    bind $w <Return> [list $w invokeok]
    
    $w add_trace_to_uservar list_type [namespace code [list _local_axes_list_fill \
                $w $toctree $etype]]
    $w set_uservar_value list_type name
    
    set action [$w createwindow]
    destroy $w
}

proc gid_groups_conds::_local_axes_list_fill { w toctree etype } {
    
    switch [$w give_uservar_value list_type] {
        name {
            set columns [list \
                    [list 8 [_ "Num"] left item 0] \
                    [list 25 [_ "Name"] left text 0] \
                    ]
        }
        eulerangles {
            set columns [list \
                    [list 8 [_ "Num"] left item 0] \
                    [list 15 [_ "Euler 1"] left text 0] \
                    [list 15 [_ "Euler 2"] left text 0] \
                    [list 15 [_ "Euler 3"] left text 0] \
                    ]
        }
        matrix {
            set columns [list \
                    [list 8 [_ "Num"] left item 0] \
                    ]
            for { set i 1 } { $i <= 9 } { incr i } {
                set row [expr {($i-1)/3+1}]
                set col [expr {$i%3}]
                lappend columns [list 10 "M$row,$col" right text 0]
            }
        }
    }
    $toctree item delete all
    $toctree configure -columns $columns
    
    if { [lindex [GiD_Info Project] 4] eq "MESHUSE" } {
        set geom_mesh mesh
    } else {
        set geom_mesh geometry
    }
    set realformat [GiD_Set RealFormat]
    
    switch [$w give_uservar_value list_type] {
        name {
            foreach j [GiD_Info conditions ${etype}_Local_axes $geom_mesh] {
                foreach "- num - name" $j break
                $toctree insert end [list $num $name]
            }
        }
        eulerangles {
            foreach j [GiD_Info conditions -localaxes ${etype}_Local_axes $geom_mesh] {
                foreach "- num - la" $j break
                foreach "l1 l2 l3" [eval [list format "$realformat $realformat $realformat"] $la] break
                $toctree insert end [list $num $l1 $l2 $l3]
            }            
        }
        matrix {
            set eps 1e-15
            foreach j [GiD_Info conditions -localaxesmat ${etype}_Local_axes $geom_mesh] {
                foreach "- num - M" $j break
                set elm [list $num]
                for { set i 0 } { $i < 9 } { incr i } {
                    set val [lindex $M $i]
                    if { abs($val) < $eps } { set val 0.0 }
                    lappend elm [format $realformat $val]
                }
                $toctree insert end $elm
            }
        }
    }
}

proc gid_groups_conds::_local_axes_generic { w what args } {
    
    set local_axes [GiD_Info localaxes]
    set la [string trim [$w give_uservar_value current_la]]
    
    switch $what {
        create {
            set ctype [lindex $args 0]
            set cmd [list GiD_Process MEscape Data LocalAxes DefineLocAxes $la]
            
            if { $la eq "" } {
                snit_messageBox -default ok -icon question -message \
                    [_ "Enter a name first for the new local axes"] \
                    -parent $w
                return
            }
            if { [lsearch -exact $local_axes $la] != -1 } {
                set retval [snit_messageBox -default ok -icon question -message \
                        [_ "Do you want to modify local axes '%s'?" $la] \
                        -type okcancel -parent $w]
                if { $retval == "cancel" } { return }
                lappend cmd "Yes"
            }
            lappend cmd $ctype
            select_apply_entities $w $cmd ""
        }
        delete {
            if { $la eq "" } {
                snit_messageBox -default ok -icon question -message \
                    [_ "Enter a name for the local axes to delete"] \
                    -parent $w
                return
            }
            if { [lsearch -exact $local_axes $la] == -1 } {
                snit_messageBox -default ok -icon question -message \
                    [_ "Local axes '%s' does not exist" $la] \
                    -parent $w
                return
            }
            set retval [snit_messageBox -default ok -icon question -message \
                    [_ "Are you sure to delete local axes '%s'?" $la] \
                    -type okcancel -parent $w]
            if { $retval == "cancel" } { return }
            GiD_Process MEscape Data LocalAxes DeleteLA $la
            $w set_uservar_value update 1 ;# to update the GUI
        }
        deleteall {
            if { [llength $local_axes] == 0 } {
                snit_messageBox -default ok -icon question -message \
                    [_ "There are no local axes to delete"] \
                    -parent $w
                return
            }
            GiD_Process MEscape Data LocalAxes DeleteAllLA
            delayedop changefunc [namespace code [list _local_axes_generic $w deleteall_end]]
        }
        deleteall_end {
            $w set_uservar_value update 1 ;# to update the GUI
        }
        draw {
            if { $la eq "" } {
                snit_messageBox -default ok -icon question -message \
                    [_ "Enter a name for the local axes to draw"] \
                    -parent $w
                return
            }
            if { [lsearch -exact $local_axes $la] == -1 } {
                snit_messageBox -default ok -icon question -message \
                    [_ "Local axes '%s' does not exist" $la] \
                    -parent $w
                return
            }
            set cmd [list GiD_Process MEscape Data LocalAxes DrawLocAxes $la]
            select_apply_entities $w $cmd [_ "Drawing local axes '%s'" $la]
        }
        drawall {
            if { [llength $local_axes] == 0 } {
                snit_messageBox -default ok -icon question -message \
                    [_ "There are no local axes to draw"] \
                    -parent $w
                return
            }
            set cmd [list GiD_Process MEscape Data LocalAxes DrawLocAxes -All-]
            select_apply_entities $w $cmd [_ "Drawing all local axes"]
        }
    }
}

proc gid_groups_conds::local_axes_window { { wp .gid } } {
    
    destroy $wp.la
    set w [dialogwin_snit $wp.la -title [_ "Local axes window"] -okname - \
            -grab 0 -callback [namespace code [list local_axes_window_do]] \
            -transient 1]
    set f [$w giveframe]
    
    set fp [ttk::notebook $f.fp]
    set f1 [ttk::frame $fp.f1]
    $fp add $f1 -text [_ "Entities"]
    
    switch [lindex [GiD_Info Project] 4] {
        GEOMETRYUSE {
            set ents [list [_ "Points"] point point [_ "Lines"] line line \
                    [_ "Surfaces"] surface surface [_ "Volumes"] volume volume]
        }
        MESHUSE {
            set ents [list [_ "Nodes"] point node [_ "L Elements"] line element \
                    [_ "S Elements"] surface element [_ "V Elements"] volume element]
        }
    }
    ttk::frame $f1.ents
    set idx 0
    foreach "name n img" $ents {
        ttk::radiobutton $f1.ents.r$idx \
            -text [string tolower $name] \
            -image [gid_themes::GetImage $img small_icons] -compound top \
            -variable [$w give_uservar entity_type] \
            -value $n
        tooltip::tooltip $f1.ents.r$idx \
            [_ "Apply local axes to %s" $name]
        grid $f1.ents.r$idx -sticky w -row 0 -column $idx
        incr idx
    }
    
    ttk::separator $f1.sep -orient horizontal
    
    ttk::menubutton $f1.a -text [_ "Assign"]... -image \
        [gid_themes::GetImage local_axis_assign small_icons] -compound left -menu $f1.a.m
    menu $f1.a.m -tearoff 0
    
    cu::menubutton_button $f1.ua -text [_ "Unssign"]... -image \
        [gid_themes::GetImage local_axis_unassign small_icons] -compound left -menu $f1.ua.m \
        -command [list gid_groups_conds::_local_axes_unassign_window $w]
    menu $f1.ua.m -tearoff 0
    
    ttk::button $f1.b1 -text [_ "List"] -compound left \
        -image [gid_themes::GetImage local_axis_list small_icons]
    ttk::button $f1.b2 -text [_ "Draw"] -compound left \
        -image [gid_themes::GetImage local_axis_draw small_icons]
    
    grid $f1.ents -    -sticky nw -pady "3"
    grid $f1.sep  -    -sticky ew -padx 6 -pady "3 20"
    grid $f1.a $f1.ua  -sticky w -padx 3 -pady 3
    grid $f1.b1 - -sticky w -padx 3 -pady 3
    grid $f1.b2 - -sticky w -padx 3 -pady 3
    grid columnconfigure $f1 1 -weight 1
    grid rowconfigure $f1 5 -weight 1
    
    set f2 [ttk::frame $fp.f2]
    $fp add $f2 -text [_ "Generic"]
    
    ttk::label $f2.l1 -text [_ "Local axes"]:
    ttk::combobox $f2.cb1 -textvariable [$w give_uservar current_la ""]
    
    ttk::menubutton $f2.a -text [_ "Create"]... -width 10 -image \
        [gid_themes::GetImage local_axis_new small_icons] -compound left -menu $f2.a.m
    menu $f2.a.m -tearoff 0
    
    $f2.a.m add command -label [_ "3 points XZ"] -command [namespace code [list \
                _local_axes_generic $w create 3PointsXZ]]
    $f2.a.m add command -label [_ "X and angle"] -command [namespace code [list \
                _local_axes_generic $w create XandAngle]]
    
    cu::menubutton_button $f2.a2 -text [_ "Delete"]... -width 10 -image \
        [gid_themes::GetImage delete small_icons] -compound left -menu $f2.a2.m \
        -command [namespace code [list _local_axes_generic $w delete]]
    menu $f2.a2.m -tearoff 0
    
    $f2.a2.m add command -label [_ "Delete"] -command [namespace code [list \
                _local_axes_generic $w delete]]
    $f2.a2.m add command -label [_ "Delete all"] -command [namespace code [list \
                _local_axes_generic $w deleteall]]
    
    cu::menubutton_button $f2.a3 -text [_ "Draw"]... -width 10 -image \
        [gid_themes::GetImage local_axis_draw small_icons] -compound left -menu $f2.a3.m \
        -command [namespace code [list _local_axes_generic $w draw]]
    menu $f2.a3.m -tearoff 0
    
    $f2.a3.m add command -label [_ "Draw"] -command [namespace code [list \
                _local_axes_generic $w draw]]
    $f2.a3.m add command -label [_ "Draw all"] -command [namespace code [list \
                _local_axes_generic $w drawall]]
    
    frame $f2.l2 -bg #ff00ff
    label $f2.l2.l -image [gid_themes::GetImage local_axes_def_img.png]
    grid $f2.l2.l -padx 2 -pady 2
    
    grid $f2.l1 $f2.cb1 - -sticky nw -padx 3 -pady 3
    grid $f2.a - $f2.l2 -sticky nw -padx 3 -pady 3
    grid $f2.a2 - ^ -sticky nw -padx 3 -pady 3
    grid $f2.a3 - ^ -sticky nw -padx 3 -pady 3
    
    grid configure $f2.cb1 -sticky ew
    grid configure $f2.l2 -padx 6
    grid columnconfigure $f2 2 -weight 1
    grid rowconfigure $f2 3 -weight 1
    
    grid $fp -sticky nsew
    grid columnconfigure $f 0 -weight 1
    grid rowconfigure $f "0" -weight 1
    
    set cmd [namespace code [list \
                _local_axes_fill_menus $w $f1.a.m $f1.ua.m $f1.b1 $f1.b2 $f2.cb1]]
    $w add_trace_to_uservar entity_type $cmd
    $w add_trace_to_uservar update $cmd
    
    $w set_uservar_value entity_type surface
    
    tk::TabToWindow $f1.a
    bind $w <Return> [list $w invokecancel]
    set action [$w createwindow]  
}

proc gid_groups_conds::local_axes_window_do { w } {
    destroy $w
}
