
################################################################################
#    Problemtype transform
################################################################################

set ::t_debug 0
if { [info exists ::transform_debug] && $::transform_debug == 1 } {
    set ::t_debug 1
}

namespace eval gid_groups_conds {
    variable doc_old
    variable transform_finished 0
}

#return true if text is enclosed in brackets [ ]
proc gid_groups_conds::string_is_proc { text } {
    set is_proc 0
    if { [string index $text 0] eq "\[" && [string index $text end] eq "\]" } {
        set is_proc 1
    }
    return $is_proc
}

#if there are more than one match try to find a single one, checking also parent
proc gid_groups_conds::try_to_select_a_single_match_valueNode { domNode_original domNode_select } {
    set value_n [$domNode_original @n]
    set domNode_original_parent [$domNode_original parentNode]
    set parent_n [$domNode_original_parent @n]
    set valueNodes_best [list]
    set xp [format_xpath {.//value[@n=%s and ancestor::*[@n=%s]]} $value_n $parent_n]
    set tmp_valueNodes [$domNode_select selectNodes $xp]
    set num_match [llength $tmp_valueNodes]
    if { $num_match == 1} {
        set valueNodes_best $tmp_valueNodes
    } elseif { $num_match > 1} {
        #try grandparent 
        set domNode_original_grandparent [$domNode_original_parent parentNode]
        set grandparent_n [$domNode_original_grandparent @n]
        set xp [format_xpath {.//value[@n=%s and parent::*[@n=%s] and ancestor::*[@n=%s]]} $value_n $parent_n $grandparent_n]
        set tmp_valueNodes [$domNode_select selectNodes $xp]
        set num_match [llength $tmp_valueNodes]
        if { $num_match == 1} {
            set valueNodes_best $tmp_valueNodes
        } else {          
            # can find if parent has attribute 'name' like kratos in old and new and try to find if there is only one with same name
            set parent_name [$domNode_original_parent @name ""]
            if { $parent_name != "" } {
                set xp [format_xpath {.//value[@n=%s and ancestor::*[@n=%s and @name=%s]]} $value_n $parent_n $parent_name]
                set tmp_valueNodes [$domNode_select selectNodes $xp]
                set num_match_name [llength $tmp_valueNodes]
                if { $num_match_name == 1} {
                    set valueNodes_best $tmp_valueNodes
                } else {
                    set valueNodes_best [list]
                }
            } else {
                set valueNodes_best [list]
            }
        }
    } 
    return $valueNodes_best
}

proc gid_groups_conds::_pt_copy_cond_values { baseNode_new domNode_new domNode_old } {
    set messages ""
    # baseNode_new will be the condition or a descendant container
    set numchanges 0
    set doc [$domNode_new ownerDocument]
    set valueNodes [$baseNode_new selectNodes value|container|edit_command]
    if { ![llength $valueNodes] && [$baseNode_new @isvalue 0] } {
        lappend valueNodes $baseNode_new
    }
    set new_condition_name [$baseNode_new @n]
    foreach valueNode $valueNodes {
        switch [$valueNode nodeName] {
            value - condition {
                set attDict [eval eval dict create [$valueNode selectNodes @*]]
                set values [split [get_domnode_attribute $valueNode values] ,]
                set values_old [split [get_domnode_attribute $domNode_old values] ,]
                set editable [get_domnode_attribute $valueNode editable 0]
                if { ![dict exists $attDict v] } {
                    dict set attDict v ""
                }
                set new_question [$valueNode @n]
                set old_question $new_question
                set xp [format_xpath {.//value[@n=%s]} $old_question]
                set valueNode_old [$domNode_old selectNodes $xp]
                if { [llength $valueNode_old]==0 } {
                    set old_question [gid_groups_conds::transform_map_inverse $new_condition_name $new_question "GID_TRANSFORM_MAP_CONDITION_QUESTION"]
                    if { $old_question != $new_question } {
                        #try old possible aliases
                        set xp [format_xpath {.//value[@n=%s]} $old_question]
                        set valueNode_old [$domNode_old selectNodes $xp]
                    }
                }
                if { [llength $valueNode_old]==0 } {
                    set combine_data [gid_groups_conds::transform_map_inverse $new_condition_name $new_question GID_TRANSFORM_MAP_CONDITION_COMBINE_QUESTIONS_PROC]
                    if { $combine_data != "" } {
                        lassign $combine_data data_old_questions data_proc_name_combine data_result_names
                        set old_questionNodes [list]
                        foreach old_question_i $data_old_questions {
                            set xp_i [format_xpath {.//value[@n=%s]} $old_question_i]
                            set valueNode_old_question_i [$domNode_old selectNodes $xp_i]
                            if { [llength $valueNode_old_question_i] == 1 } {
                                lappend old_questionNodes $valueNode_old_question_i
                            }
                        }
                        if { [llength $old_questionNodes] == 2 } {
                            set combined_value [$data_proc_name_combine {*}$old_questionNodes $data_result_names]
                            dict set attDict v $combined_value
                        }
                    }
                }
                if { [llength $valueNode_old]==1 && [$valueNode_old hasAttribute v] &&
                    ($values eq "" || $editable || [lsearch -exact $values [$valueNode_old @v]] != -1 || $values != $values_old) } {
                    set unit_magnitude [get_domnode_attribute $valueNode unit_magnitude]
                    if { [llength [split $unit_magnitude ,]] > 1 } {
                        set d [eval dict create [split $unit_magnitude ,]]
                        set ov [$valueNode_old selectNodes {string(ancestor::group/@ov)}]
                        if { [dict exists $d $ov] } {
                            set unit_magnitude [dict get $d $ov]
                        }
                    }
                    if { $unit_magnitude ne "" && [llength [split $unit_magnitude ,]] == 1 } {
                        set units [$valueNode_old @units ""]
                        if { $units eq "" || [is_unit_correct $unit_magnitude $units] } {
                            dict set attDict v [$valueNode_old @v]
                            dict set attDict units $units
                            dict set attDict unit_magnitude $unit_magnitude
                        } else {
                            #set in any case the old numeric value, without take into account its units
                            dict set attDict v [$valueNode_old @v]
                            append messages [_ "\n - Units not correct for values '%s' in '%s'" [$valueNode @n] [$baseNode_new @n]]
                            incr numchanges 1
                        }
                    } elseif { $unit_magnitude eq "" } {
                        dict set attDict v [$valueNode_old @v]
                    }
                    if { [$valueNode_old @state normal] in "normal hidden disabled" } {
                        if { $::t_debug != 1 } {
                            dict set attDict state [$valueNode_old @state normal]
                        }
                    }
                } else {
                    set numchanges 1
                }
                foreach i [list ov isvalue] {
                    if { [dict exists $attDict $i] } {
                        dict unset attDict $i
                    }
                }
                set xpn [format_xpath {value[@n=%s]} [$valueNode @n]]
                if { [$domNode_new selectNodes $xpn] == ""} {
                    set v [$domNode_new appendChild [$doc createElement value]]
                    eval [list $v setAttribute] $attDict
                    if { $valueNode_old ne "" } {
                        foreach n [$valueNode_old childNodes] {
                            if { [$n nodeName] in "dependencies edit_command #comment label" } { continue }
                            if { [$n nodeName] eq "function" && [$v selectNodes function] ne "" } {
                                set nnew [$v selectNodes function]
                                $nnew delete
                            }
                            $v appendChild [$n cloneNode -deep]
                        }
                    }
                } else {
                    set xpnc [format_xpath {.//value[@n=%s]} [$valueNode @n]]
                    foreach iNode [$domNode_new childNodes] {
                        if {[$iNode nodeName] in "#comment" } { continue }
                        if {[$iNode @n ""] ne [$domNode_old @n ""]} { continue }
                        if {[$iNode nodeName] ne [$domNode_old nodeName]} { continue }
                        set v [$iNode selectNodes $xpnc]
                        eval [list $v setAttribute] $attDict
                    }
                }
            }
            container {
                set attDict [eval eval dict create [$valueNode selectNodes @*]]
                set xpcont [format_xpath {container[@n=%s]} [$valueNode @n]]
                if { [$domNode_new selectNodes $xpcont] == ""} {
                    set c [$domNode_new appendChild [$doc createElement container]]
                    eval [list $c setAttribute] $attDict
                    lassign [_pt_copy_cond_values $valueNode $c $domNode_old] hc messages_local
                    if { $hc } {
                        set numchanges 1
                        append messages $messages_local
                    }
                }
            }
            edit_command {
                set xpn [format_xpath {.//edit_command[@n=%s]} [$valueNode @n]]
                if { [$domNode_new selectNodes $xpn] eq ""} {
                    set edit_commandNode [$domNode_old selectNodes $xpn]
                    if {$edit_commandNode ne ""} {
                        $domNode_new appendChild [$edit_commandNode cloneNode -deep]
                    }
                }
            }
        }
    }
    return [list $numchanges $messages]
}

proc gid_groups_conds::_pt_values { domNode domNode_old } {
    
    set numchanges 0
    set messages ""
    set doc [$domNode ownerDocument]
    
    foreach valueNode [$domNode selectNodes value|container] {
        switch [$valueNode nodeName] {
            value {
                set xp {self::*[@units_system_definition="1" or
                    @unit_definition or @unit_mesh_definition='1']}
                if { [$valueNode selectNodes $xp] ne "" } {
                    continue
                }
                set fail 0
                if { [$valueNode @name ""] ne "" } {
                    set xp [format_xpath {.//value[@n=%s and @name=%s]} [$valueNode @n] [$valueNode @name]]
                } else {
                    set xp [format_xpath {.//value[@n=%s]} [$valueNode @n]]
                }
                set v_old [$domNode_old selectNodes $xp]
                if { $v_old eq "" } {
                    set fail 1
                }
                if { !$fail && [llength $v_old] == 1 } {
                    set values [split [get_domnode_attribute $valueNode values] ,]
                    set editable [get_domnode_attribute $valueNode editable 0]
                    set value_old [$v_old @v ""]
                    set isproc [gid_groups_conds::string_is_proc [$valueNode @v ""]]
                    if { $values ne "" && !$editable && !$isproc && ([lsearch -exact $values $value_old] == -1 && $value_old != "") } {
                        set fail 1
                    }
                }
                if { !$fail && [llength $v_old] == 1 } {
                    set unit_magnitude [get_domnode_attribute $valueNode unit_magnitude]
                    if { $unit_magnitude ne "" } {
                        set units_old [get_domnode_attribute $v_old units]
                        if { [is_unit_correct $unit_magnitude $units_old] } {
                            if {!$isproc} {
                                $valueNode setAttribute v $value_old
                            }
                            $valueNode setAttribute units $units_old
                        } elseif { $units_old ne "" } {
                            set fail 1
                        }
                    } else {
                        if {!$isproc} {
                            $valueNode setAttribute v $value_old
                        }
                    }
                    set isproc_state [gid_groups_conds::string_is_proc [$valueNode @state normal]]
                    if { [$valueNode @state normal] in "hidden disabled readonly normal" && $::t_debug != 1 && !$isproc_state } {
                        set isp_state [gid_groups_conds::string_is_proc [$v_old @state normal]]
                        if { !$isp_state } {
                            $valueNode setAttribute state [get_domnode_attribute $v_old state normal]
                        }
                    }
                    if { [$v_old childNodes] != "" } {
                        set v_v_old [get_domnode_attribute $v_old v]
                        foreach n [$v_old childNodes] {
                            if { [$n nodeName] in "#comment label" } { continue }
                            if { [$n nodeName] in "dependencies" } {
                                if {[$n hasAttribute att]} {
                                    set iiList [list 1]
                                } else {
                                    set iiList [list 1 2]
                                }
                                if { [$n @value ""] eq $v_v_old } {
                                    foreach i $iiList {
                                        if { ![$n hasAttribute att$i] && ![$n hasAttribute att]} {
                                            continue
                                        }
                                        if { [$n hasAttribute att$i] } {
                                            set depattNode [$n @att$i]
                                        } elseif { [$n hasAttribute att] } {
                                            set depattNode [$n @att]
                                        }
                                        set tomodNodes [$valueNode selectNodes [$n @node ""]]
                                        foreach i_tomodNode $tomodNodes {
                                            switch -- $depattNode {
                                                "state" {
                                                    if { [$n hasAttribute att$i] } {
                                                        set state [get_domnode_attribute $n v$i]
                                                    } else {
                                                        set state [get_domnode_attribute $n v]
                                                    }  
                                                    set newstate [$i_tomodNode @state ""]
                                                    set is_state_proc 0
                                                    if { [string index $newstate 0] eq "\[" && [string index $newstate end] eq "\]" } {
                                                        set is_state_proc 1
                                                    }                
                                                    if { !$is_state_proc } {
                                                        $i_tomodNode setAttribute state $state                                               
                                                    }
                                                }                                       
                                                "v" {                                               
                                                    if { [$n hasAttribute att$i] } {
                                                        set v [get_domnode_attribute $n v$i]
                                                    } else {
                                                        set v [get_domnode_attribute $n v] 
                                                    }
                                                    $i_tomodNode setAttribute v $v
                                                }
                                            }
                                        }
                                    }
                                } elseif { [$n @condition ""] ne "" } {
                                    set cnd [$n @condition ""]
                                    if { $cnd ne "" && [$v_old selectNodes $cnd] } {
                                        foreach i $iiList {
                                            if { ![$n hasAttribute att$i] && ![$n hasAttribute att]} {
                                                continue
                                            }
                                            if { [$n hasAttribute att$i] } {
                                                set depattNode [$n @att$i]
                                            } elseif { [$n hasAttribute att] } {
                                                set depattNode [$n @att]
                                            }
                                            set tomodNodes [$valueNode selectNodes [$n @node ""]]
                                            foreach i_tomodNode $tomodNodes {
                                                switch -- $depattNode {
                                                "state" {
                                                        if { [$n hasAttribute att$i] } {
                                                            set state [get_domnode_attribute $n v$i]
                                                        } else {
                                                            set state [get_domnode_attribute $n v]
                                                        }  
                                                        set newstate [$i_tomodNode @state ""]
                                                        set is_state_proc 0
                                                        if { [string index $newstate 0] eq "\[" && [string index $newstate end] eq "\]" } {
                                                            set is_state_proc 1
                                                        }                
                                                        if { !$is_state_proc } {
                                                            $i_tomodNode setAttribute state $state                                               
                                                        }                                                                                                     
                                                    }                                       
                                                    "v" {                                               
                                                        if { [$n hasAttribute att$i] } {
                                                            set v [get_domnode_attribute $n v$i]
                                                        } else {
                                                            set v [get_domnode_attribute $n v] 
                                                        }
                                                        $i_tomodNode setAttribute v $v
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            
                            if { [$n nodeName] in "function" } {
                                if {[$valueNode selectNodes function] ne ""} {
                                    set funcNode [$valueNode selectNodes function]
                                    $funcNode delete
                                }
                                $valueNode appendChild [$n cloneNode -deep]
                            }
                        }
                    }
                }
                if { $fail } {
                    append messages  [_ "\n - Value '%s' has different fields" [$valueNode @n]]
                    incr numchanges 1
                }
            }
            container {
                lassign [_pt_blockdata_or_container_values $valueNode $domNode_old] hc messages_local
                if { $hc } {
                    incr numchanges $hc
                    append messages $messages_local
                }
            }
        }
    }
    return [list $numchanges $messages]
}


proc gid_groups_conds::_pt_blockdata_or_container_values { domNode domNode_old } {
    set numchanges 0
    set messages ""
    set doc [$domNode ownerDocument]
    set new_blockdata_name [$domNode @n]
    foreach valueNode [$domNode selectNodes value|container|condition|group|groups|blockdata] {
        switch [$valueNode nodeName] {
            value {
                set xp {self::*[@units_system_definition="1" or @unit_definition or @unit_mesh_definition='1']}
                if { [$valueNode selectNodes $xp] ne "" } {
                    continue
                }
                set fail 0
                set new_question [$valueNode @n]
                set old_question $new_question
                if { [$valueNode @name ""] ne "" } {
                    set xp [format_xpath {.//value[@n=%s and @name=%s]} $old_question [$valueNode @name]]
                } else {
                    set xp [format_xpath {.//value[@n=%s]} $old_question]
                }
                set v_old [$domNode_old selectNodes $xp]
                if { [llength $v_old] == 0 } {
                    set old_question [gid_groups_conds::transform_map_inverse $new_blockdata_name $new_question "GID_TRANSFORM_MAP_BLOCKDATA_QUESTION"]
                    if { $old_question != $new_question } {
                        #try old possible aliases
                        if { [$valueNode @name ""] ne "" } {
                            set xp [format_xpath {.//value[@n=%s and @name=%s]} $old_question [$valueNode @name]]
                        } else {
                            set xp [format_xpath {.//value[@n=%s]} $old_question]
                        }
                        set v_old [$domNode_old selectNodes $xp]
                    }
                }
                if { [llength $v_old] == 0 } {
                    set fail 1
                }
                if { [llength $v_old] > 1 } {
                    set valueNodeBest [gid_groups_conds::try_to_select_a_single_match_valueNode $valueNode $domNode_old]
                    if { [llength $valueNodeBest] == 1 } {
                        set v_old $valueNodeBest
                    }
                }
                if { !$fail && [llength $v_old] == 1 } {
                    set value_old [$v_old @v ""]
                    set values [split [get_domnode_attribute $valueNode values] ,]  
                    if { [$valueNode hasAttribute dict] } {
                        set values_from_dict [dict keys [split [get_domnode_attribute $valueNode dict] ,]]
                        if { $values != $values_from_dict } {
                            #incoherent
                            if { [llength $values] == 0 } {
                                set values $values_from_dict
                            } else {
                                set to_breakpoint 1
                            }
                        }
                    }
                    set editable [get_domnode_attribute $valueNode editable 0]                   
                    set isproc_v [gid_groups_conds::string_is_proc [$valueNode @v ""]]
                    set isproc_values [gid_groups_conds::string_is_proc [$valueNode @values ""]]
                    if { !$isproc_values } {
                        if { $values ne "" && $value_old != "" && !$editable && !$isproc_v && [lsearch -exact $values $value_old] == -1 } {
                            #set fail 1
                            #old value is not valid now, set arbitrarily the first allowed value
                            set value_old [lindex $values 0]
                        }
                    }
                    set unit_magnitude [get_domnode_attribute $valueNode unit_magnitude]
                    if { $unit_magnitude ne "" } {
                        set units_old [get_domnode_attribute $v_old units]
                        if { [is_unit_correct $unit_magnitude $units_old] } {
                            if {!$isproc_v} {
                                $valueNode setAttribute v $value_old
                            }
                            $valueNode setAttribute units $units_old
                        } elseif { $units_old ne "" } {
                            set fail 1
                        }
                    } else {
                        if { !$isproc_v } {
                            $valueNode setAttribute v $value_old
                        }
                    }
                    set isproc_state [gid_groups_conds::string_is_proc [$valueNode @state normal]]
                    if { [$v_old @state normal] in "hidden disabled readonly normal" && $::t_debug != 1 && !$isproc_state } {
                        set isp_state  [gid_groups_conds::string_is_proc [$v_old @state normal]]
                        if { !$isp_state } {
                            $valueNode setAttribute state [get_domnode_attribute $v_old state normal]
                        }
                    }
                    foreach n [$v_old childNodes] {
                        if { [$n nodeName] in "#comment dependencies label" } { continue }
                        if { [$n nodeName] in "function" } {
                            if {[$valueNode selectNodes function] ne ""} {
                                set funcNode [$valueNode selectNodes function]
                                $funcNode delete
                            }
                            $valueNode appendChild [$n cloneNode -deep]
                        }
                    }
                }
                if { $fail } {
                    set nodename [$domNode nodeName]
                    set nodename0 [string range $nodename 0 0]
                    set nodename0 [string toupper $nodename0]
                    set nodename1 [string range $nodename 1 end]
                    set nodename $nodename0$nodename1
                    append messages [_ "\n - %s '%s' has different field: '%s'" $nodename [$domNode @n] [$valueNode @n]]
                    incr numchanges 1
                }
            }
            blockdata {
                lassign [_pt_blockdata_or_container_values $valueNode $domNode_old] hc messages_local
                if { $hc } {
                    incr numchanges $hc
                    append messages $messages_local
                }
            }
            container {
                lassign [_pt_blockdata_or_container_values $valueNode $domNode_old] hc messages_local
                if { $hc } {
                    incr numchanges $hc
                    append messages $messages_local
                }
            }
            condition {
                set numchangesL 0
                if { [$valueNode @name ""] ne "" } {
                    set xp [format_xpath {.//condition[@n=%s and @name=%s]} [$valueNode @n] [$valueNode @name]]
                } else {
                    set xp [format_xpath {.//condition[@n=%s]} [$valueNode @n]]
                }
                set v_old [$domNode_old selectNodes $xp]
                if { [llength $v_old] == 1 } {
                    if {[[$valueNode parentNode] nodeName] == "container" && [[$valueNode parentNode] @n] != [[$v_old parentNode] @n] } {
                        # kratos: ignore old node if its parent is not container and with same
                        # e.g container @n=Structural//container @n="InitialConditions"
                        # do not copy condition @n="DISPLACEMENT" from container @n=Structural//container @n="Boundary Conditions"
                        set v_old [list]
                    }
                }
                if { [llength $v_old] == 1 } {
                    set cndNode_new $valueNode
                    foreach groupNode_old [$v_old selectNodes group|groupList] {
                        if { ![$groupNode_old hasAttribute n] } {
                            set name [$groupNode_old @name ""]
                        } else {
                            set name [$groupNode_old @n ""]
                        }
                        set isfound 0
                        foreach icnd [$cndNode_new selectNodes group|groupList] {
                            if { [$icnd @n ""] eq $name} { set isfound 1; break }
                        }
                        if {!$isfound} {
                            set groupNode_new [$cndNode_new appendChild [$groupNode_old cloneNode]]
                            if { [$groupNode_old nodeName] eq "groupList" } {
                                foreach grG_old [$groupNode_old selectNodes group] {
                                    $groupNode_new appendChild [$grG_old cloneNode]
                                }
                            }
                            lassign [_pt_copy_cond_values $cndNode_new $groupNode_new $groupNode_old] hc messages_local
                            if { $hc } {
                                incr numchangesL $hc
                                append messages $messages_local
                            }
                            if { !$numchangesL } {
                                set n1 [llength [$groupNode_new selectNodes .//value]]
                                set n2 [llength [$groupNode_old selectNodes .//value]]
                                if { $n1 != $n2 } {
                                    set numchangesL 1
                                }
                            }
                        }
                    }
                }
                if { $numchangesL } {
                    append messages [_ "\n - Condition '%s' has different fields" [$domNode @n]]
                    incr numchanges $numchangesL
                }
            }
            group {
                if { $gid_groups_conds::is_draw_post } {
                    return $numchanges
                }
                # only arrives here the: /groups/group
                if { ![$domNode_old hasAttribute n] } {
                    set name [$domNode_old @name]
                } else {
                    set name [$domNode_old @n]
                }
                set xp [format_xpath {groups/group[@n=%s]} $name]
                set group_nodes [$domNode selectNodes $xp]
                if { ![llength $group_nodes] } {
                    gid_groups_conds::add_new_group -noactualizegroups -exactname -noundo -permit_extended_name 1 -- $name
                    set group_nodes [$domNode_new selectNodes $xp]
                }
                foreach gr $group_nodes {
                    foreach i [$gr selectNodes @*] {
                        lassign $i name value
                        set value [$domNode_old @$name $value]
                        $gr setAttribute $name $value
                    }
                }
            }
            groups {
                foreach childNode_old [$domNode_old childNodes] {
                    lassign [_transform_problemtype_elm $childNode_old $domNode_new $domNode_old] hc messages_local
                    if { $hc } {
                        incr numchanges $hc
                        append messages $messages_local
                    }
                }
            }
        }
    }
    return [list $numchanges $messages]
}

# proc gid_groups_conds::_pt_blockdata_create_secuences { domNode tdomNode } {
# 
#     # tdomNode is the old tree
#     if { $domNode eq "" } { return "" }
# 
#     set s_t [$tdomNode selectNodes ..//blockdata\[@n='[$tdomNode @n]'\]]
#     set s [$domNode selectNodes "..//blockdata\[@n='[$tdomNode @n]' and @sequence='1'\]"]
# 
#     if { [llength $s] == 0 } {
#         set s [[lindex $s_t 0] cloneNode -deep]
#         $domNode appendChild $s
#     }
#     
#     if { [llength $s] == 1 } {
#         $s setAttribute state normal tree_state close \
#             pn [[lindex $s_t 0] @pn]
#     }
#     set s0 [lindex $s 0]
#     for { set i 1 } { $i <= [llength $s_t]-[llength $s] } { incr i } {
#         set newNode [$s0 cloneNode -deep]
#         [$s0 parentNode] appendChild $newNode
#         $newNode setAttribute state normal tree_state close \
#             pn [[lindex $s_t $i] @pn]
#     }
#     set s [$domNode selectNodes "..//blockdata\[@n='[$tdomNode @n]' and @sequence='1'\]"]
# 
#     set ipos [lsearch -exact $s_t $tdomNode]
#     return [lindex $s $ipos]
# }

proc gid_groups_conds::_pt_blockdata_select_or_create { blockdata_old blockdataList parentNode domNode_new } {
    set name [get_domnode_attribute $blockdata_old name]
    if { $name eq "" } {
        set name [get_domnode_attribute $blockdata_old pn]
    }
    set n_old [get_domnode_attribute $blockdata_old n]
    
    set containerList ""
    foreach i [$blockdata_old selectNodes container] {
        lappend containerList [$i @n ""]
    }
    
    set blockList ""
    foreach i [$blockdata_old selectNodes blockdata] {
        lappend blockList [$i @n ""]
    }
    set blocknamesList ""
    foreach i [$blockdata_old selectNodes blockdata] {
        lappend blocknamesList [$i @name ""]
    }
    set n1 [llength $blocknamesList]
    
    set xp {string(parent::container/@pn)}
    set container [$blockdata_old selectNodes $xp]
    
    set xp {string(parent::blockdata/@pn)}
    set blockdata [$blockdata_old selectNodes $xp]
    
    set similarB ""
    foreach b $blockdataList {
        set bL ""
        foreach i [$b selectNodes blockdata] {
            lappend bL [$i @n ""]
        }
        set bnameL ""
        foreach i [$b selectNodes blockdata] {
            set bname [$i @name ""]
            if {$bname eq ""} {
                set bname [$i @pn ""]
            }
            lappend bnameL $bname
        }
        set cL ""
        foreach i [$b selectNodes container] {
            lappend cL [$i @n ""]
        }
        set is_similar 1
        set is_active 1
        
        set is_compare_containers 0
        if { $containerList in $cL || $containerList eq $cL } {
            set is_compare_containers 1
        }
        
        if { (!$is_compare_containers && $n_old ne "material") || \
            ($blockList ne $bL && $blockList != "" && $blocknamesList != $bnameL) } {
            set is_similar 0
        }
        
        if { [get_domnode_attribute $b state normal] eq "disabled" } {
            set is_active 0
        }
        if { [$b @active 1] == 0 } { set is_active 0 }
        set n [get_domnode_attribute $b name]
        if { $n eq "" } {
            set n [get_domnode_attribute $b pn]
        }
        # && $bL == ""
        if { $n eq $name && $is_active && $bL == "" } {
            if { !$is_similar } { return "" }
            return $b
        }
        set xp {string(parent::container/@pn)}
        set c [$b selectNodes $xp]
        
        set xp {string(parent::blockdata/@pn)}
        set BB [$b selectNodes $xp]
        
        if {$blockList != ""} {
            if { $is_similar && ($similarB eq "" && $bnameL eq $blocknamesList) } {
                set similarB $b
            }
        }
    }
    if { $similarB eq "" } {
        if { $blockdataList eq "" } {
            set newB [$blockdata_old cloneNode -deep]
            if {$parentNode != ""} {
                foreach ipar [$parentNode selectNodes blockdata] {
                    set parname [get_domnode_attribute $ipar name]
                    if { $parname eq "" } {
                        set parname [get_domnode_attribute $ipar pn]
                    }
                    if { $parname eq $name } {
                        $ipar delete
                    }
                }
                $parentNode appendChild $newB
                return $newB
            }
        } else {
            if {$blockList != ""} {
                set similarB [lindex $blockdataList 0]
                set parentNode [$similarB parentNode]
            } else {
                array unset ncontainer
                foreach bdNode $blockdataList {
                    set bdname [get_domnode_attribute $bdNode name]
                    if { $bdname eq "" } {
                        set bdname [get_domnode_attribute $bdNode pn]
                    }
                    set ncontainer($bdname) ""
                    foreach icN [$bdNode selectNodes container] {
                        lappend ncontainer($bdname) [$icN @n ""]
                    }
                    foreach icN [$bdNode selectNodes container/container] {
                        lappend ncontainer($bdname) [$icN @n ""]
                    }
                }
                array unset ncontainerold
                set ncontainerold($name) ""
                foreach icN [$blockdata_old selectNodes container] {
                    lappend ncontainerold($name) [$icN @n ""]
                }
                foreach icN [$blockdata_old selectNodes container/container] {
                    lappend ncontainerold($name) [$icN @n ""]
                }
                foreach iname [array names ncontainer] {
                    if { $ncontainerold($name) in $ncontainer($iname) || \
                        $ncontainerold($name) eq $ncontainer($iname) } {
                        foreach bdNode $blockdataList {
                            set bdname [get_domnode_attribute $bdNode name]
                            if { $bdname eq "" } {
                                set bdname [get_domnode_attribute $bdNode pn]
                            }
                            if {$bdname eq $iname} {
                                set similarB $bdNode
                                break
                            }
                        }
                    } else {
                        continue
                    }
                }
            }
            if {$similarB eq ""} {
                set similarB [lindex $blockdataList 0]
            }
            set parentNode [$similarB parentNode]
        }
    } else {
        set parentNode [$similarB parentNode]
    }
    
    if { $similarB ne "" && $parentNode ne "" } {
        set newB [$similarB cloneNode -deep]
        $newB setAttribute state [$blockdata_old @state normal]
        $newB setAttribute name $name
        
        gid_groups_conds::_delete_groups $newB
        ###
        foreach ipar [$parentNode selectNodes blockdata] {
            set parname [get_domnode_attribute $ipar name]
            if { $parname eq "" } {
                set parname [get_domnode_attribute $ipar pn]
            }
            if { $parname eq $name } {
                $ipar delete
            }
        }
        $parentNode appendChild $newB
        return $newB
    }
    return ""
}

proc gid_groups_conds::_delete_groups { domNode } {
    foreach valueNode [$domNode selectNodes value|container|condition|group|groups] {
        switch [$valueNode nodeName] {
            value - container - condition - groups {
                gid_groups_conds::_delete_groups $valueNode
            }
            group {
                $valueNode delete
            }
        }
    }
}

proc gid_groups_conds::transform_map { old_name old_question transform_map_category } {
    set mapped_name ""
    if { $transform_map_category=="GID_TRANSFORM_MAP_CONDITION" } {
        if { [info exists ::TranformMapCondition($old_name)] } {
            set mapped_name $::TranformMapCondition($old_name)
        } else {
            set mapped_name $old_name
        }
    } elseif { $transform_map_category=="GID_TRANSFORM_MAP_CONDITION_QUESTION" } {
        set key [list $old_name $old_question]
        if { [info exists ::TranformMapConditionQuestion($key)] } {
            set mapped_name $::TranformMapConditionQuestion($key)
        } else {
            set mapped_name $old_question
        }
    } elseif { $transform_map_category=="GID_TRANSFORM_MAP_CONDITION_XPATH" } {
        if { [info exists ::TranformMapConditionXpath($old_name)] } {
            set mapped_name $::TranformMapConditionXpath($old_name)
        } else {
            set mapped_name [list]
        }
    } elseif { $transform_map_category=="GID_TRANSFORM_MAP_BLOCKDATA" } {
        if { [info exists ::TranformMapBlockdata($old_name)] } {
            set mapped_name $::TranformMapBlockdata($old_name)
        } else {
            set mapped_name $old_name
        }
    } elseif { $transform_map_category=="GID_TRANSFORM_MAP_BLOCKDATA_QUESTION" } {
        set key [list $old_name $old_question]
        if { [info exists ::TranformMapBlockdataQuestion($key)] } {
            set mapped_name $::TranformMapBlockdataQuestion($key)
        } else {
            set mapped_name $old_question
        }
    } else {
        set mapped_name $old_name
    }
    return $mapped_name
}

proc gid_groups_conds::transform_map_inverse { new_name new_question transform_map_category } {
    set mapped_name ""
    if { $transform_map_category=="GID_TRANSFORM_MAP_CONDITION" } {
        if { [info exists ::TranformMapConditionInverse($new_name)] } {
            set mapped_name $::TranformMapConditionInverse($new_name)
        } else {
            set mapped_name $new_name
        }
    } elseif { $transform_map_category=="GID_TRANSFORM_MAP_CONDITION_QUESTION" } {
        set key [list $new_name $new_question]
        if { [info exists ::TranformMapConditionQuestionInverse($key)] } {
            set mapped_name $::TranformMapConditionQuestionInverse($key)
        } else {
            set mapped_name $new_question
        }
    } elseif { $transform_map_category=="GID_TRANSFORM_MAP_CONDITION_COMBINE_QUESTIONS_PROC" } {
        set key [list $new_name $new_question]
        if { [info exists ::TranformMapConditionCombineQuestionsInverse($key)] } {
            set mapped_name $::TranformMapConditionCombineQuestionsInverse($key)
        } else {
            set mapped_name [list]
        }
    } elseif { $transform_map_category=="GID_TRANSFORM_MAP_BLOCKDATA" } {
        if { [info exists ::TranformMapBlockdataInverse($new_name)] } {
            set mapped_name $::TranformMapBlockdataInverse($new_name)
        } else {
            set mapped_name $new_name
        }
    } elseif { $transform_map_category=="GID_TRANSFORM_MAP_BLOCKDATA_QUESTION" } {
        set key [list $new_name $new_question]
        if { [info exists ::TranformMapBlockdataQuestionInverse($key)] } {
            set mapped_name $::TranformMapBlockdataQuestionInverse($key)
        } else {
            set mapped_name $new_question
        }
    } else {
        set mapped_name $new_name
    }
    return $mapped_name
}

proc gid_groups_conds::transform_combine_two_dom_boolean_values { a b {words {0 1 2 3}} } {
    set result ""
    if { [$a @v] } {
        if { [$b @v] } {
            set result 1
        } else {
            set result 3
        }
    } elseif { [$b @v] } {
        set result 2
    } else {
        set result 0
    }
    return [lindex $words $result]
}

proc gid_groups_conds::_transform_problemtype_elm { domNode_old domNode_new root_old } {
    set numchanges 0
    set messages ""
    switch [$domNode_old nodeName] {
        condition {
            set numchangesL 0
            set old_name [$domNode_old @n]
            set new_name $old_name
            set xp [format_xpath {.//condition[not(ancestor::blockdata[@n="Internal data"]) and @n=%s]} $new_name]
            set cndNode_new [$domNode_new selectNodes $xp]
            if { [llength $cndNode_new] == 0 } {
                set new_name [gid_groups_conds::transform_map $old_name "" "GID_TRANSFORM_MAP_CONDITION"]
                if { $new_name != $old_name } {
                    #try old possible aliases
                    set xp [format_xpath {.//condition[not(ancestor::blockdata[@n="Internal data"]) and @n=%s]} $new_name]
                    set cndNode_new [$domNode_new selectNodes $xp]
                }
            }
            if { [llength $cndNode_new] == 0 } {
                #try map xpath
                foreach map_xpaths [gid_groups_conds::transform_map $old_name "" "GID_TRANSFORM_MAP_CONDITION_XPATH"] {
                    lassign $map_xpaths old_xpath new_xpath
                    set domNodes_old_match [$domNode_old selectNodes $old_xpath]
                    if { [llength $domNodes_old_match] == 1 && [lindex $domNodes_old_match 0] == $domNode_old } {
                        #domNode_old match the old_xpath
                        set domNodes_new_match [$domNode_new selectNodes $new_xpath]
                        if { [llength $domNodes_new_match] == 1 } {
                            set cndNode_new [list [lindex $domNodes_new_match 0]]
                        }
                        break
                    }
                }
            }
            if { [llength $cndNode_new] == 0 } {
                append messages [_ "\n - Condition '%s' does not exists in new problemtype" $old_name]
                return 1
            } elseif { [llength $cndNode_new] > 1 } {
                append messages [_ "\n - Condition '%s' is repeated in new problemtype" $old_name]
                return 1
            }
            foreach groupNode_old [$domNode_old selectNodes group|groupList] {
                if { ![$groupNode_old hasAttribute n] } {
                    set name [$groupNode_old @name ""]
                } else {
                    set name [$groupNode_old @n ""]
                }
                set isfound 0
                foreach icnd [$cndNode_new selectNodes group|groupList] {
                    if { [$icnd @n ""] eq $name} {
                        set isfound 1
                        break
                    }
                }
                if { !$isfound } {
                    set groupNode_new [$cndNode_new appendChild [$groupNode_old cloneNode]]
                    if { [$groupNode_old nodeName] eq "groupList" } {
                        foreach grG_old [$groupNode_old selectNodes group] {
                            $groupNode_new appendChild [$grG_old cloneNode]
                        }
                    }
                } else {
                    set groupNode_new $cndNode_new
                }
                lassign [_pt_copy_cond_values $cndNode_new $groupNode_new $groupNode_old] hc messages_local
                if { $hc } {
                    incr numchangesL $hc
                }
                if { !$numchangesL } {
                    set n1 [llength [$groupNode_new selectNodes .//value]]
                    set n2 [llength [$groupNode_old selectNodes .//value]]
                    if { $n1 != $n2 } {
                        set numchangesL 1
                    }
                }
            }
            if { $numchangesL } {
                append messages [_ "\n - Condition '%s' has different fields" [$domNode_old @n]]
                incr numchanges $numchangesL
            }
        }
        group {
            if { $gid_groups_conds::is_draw_post } {
                return $numchanges
            }
            # only arrives here the: /groups/group
            if { ![$domNode_old hasAttribute n] } {
                set name [$domNode_old @name ""]
            } else {
                set name [$domNode_old @n ""]
            }
            set xp [format_xpath {groups/group[@n=%s]} $name]
            set group_nodes [$domNode_new selectNodes $xp]
            if { ![llength $group_nodes] } {
                gid_groups_conds::add_new_group -noactualizegroups -exactname -noundo -permit_extended_name 1 -- $name
                set group_nodes [$domNode_new selectNodes $xp]
            }
            foreach gr $group_nodes {
                foreach i [$gr selectNodes @*] {
                    foreach "name value" $i break
                    set value [$domNode_old @$name $value]
                    $gr setAttribute $name $value
                }
            }
        }
        groups - container {
            foreach childNode_old [$domNode_old childNodes] {
                if {[$childNode_old nodeName] in "#comment label"} { continue }
                lassign [_transform_problemtype_elm $childNode_old $domNode_new $root_old] hc messages_local
                if { $hc } {
                    incr numchanges $hc
                    append messages $messages_local
                }
            }
        }
        value {
            set xp [format_xpath {value[@n=%s]} [$domNode_old @n]]
            set valueNode [$domNode_new selectNodes $xp]
            if { $valueNode == "" } {
                set xp [format_xpath {.//value[@n=%s]} [$domNode_old @n]]
                set valueNode [$domNode_new selectNodes $xp]
            }
            if { $valueNode != "" && [$domNode_old @n] != "image"} {
                if {[llength $valueNode] > 1} {
                    set valueNodeBest [gid_groups_conds::try_to_select_a_single_match_valueNode $domNode_old $domNode_new]
                    if { [llength $valueNodeBest] == 1 } {
                        set valueNode $valueNodeBest
                    }
                }
                if {[llength $valueNode] > 1} {
                    append messages [_ "\n - value '%s' is not OK, it is repeated in new problemtype." [$domNode_old @n]]
                    return 1
                }
                lassign [_pt_values [$valueNode parentNode] [$domNode_old parentNode]] hc messages_local
                if { $hc } {
                    incr numchanges $hc
                    append messages $messages_local
                }
            }
        }
        blockdata {
            set name [get_domnode_attribute $domNode_old name]
            if { $name eq "" } {
                set name [get_domnode_attribute $domNode_old pn]
            }
            if {[get_domnode_attribute $domNode_old @state normal] eq "disabled"} { return 0 }
            if {[get_domnode_attribute $domNode_old @state normal] eq "hidden"} { return 0 }
            #             if { [$domNode_old @active 1] == 0 } { return 0 }
            
            set xp [format_xpath {.//blockdata[not(ancestor::blockdata[@n="Internal data"]) and @n=%s]} \
                    [$domNode_old @n]]
            set domNodes [$domNode_new selectNodes $xp]
            
            if { $domNodes eq "" && [$domNode_old @sequence 0] == 0 } {
                append messages [_ "\n - Blockdata '%s' does not exist in the new problemtype" [$domNode_old @n]]
                return 1
            }
            if { [$domNode_old @sequence 0] == 1 } {
                set container_old [$domNode_old parentNode]
                set xp [format_xpath {.//container[@n=%s]} [$container_old @n]]
                set parentNode [$domNode_new selectNodes $xp]
                if { [llength $parentNode] > 1 } {
                    set parentNode ""
                }
                
                set domNode [_pt_blockdata_select_or_create $domNode_old \
                        $domNodes $parentNode $domNode_new]
                if { $domNode eq "" } {
                    append messages [_ "\n - Blockdata '%s' cannot be created as a secuence" [$domNode_old @n]]
                    return 1
                }
                set xpb [format_xpath {.//blockdata[@n=%s]} [$domNode_old @n]]
                foreach d [$domNode_new selectNodes $xpb] {
                    set name_d [get_domnode_attribute $d name]
                    set isfound 0
                    foreach iparNode [$root_old selectNodes $xpb] {
                        set name_old [get_domnode_attribute $iparNode name]
                        if { $name_old eq $name_d} {
                            set isfound 1
                            break
                        }
                    }
                    if { !$isfound && [$domNode_old @sequence ""] == "1" && [$domNode_old  @n ""] ne "material"} {
                        $d delete
                    }
                }
                
            } elseif { [llength $domNodes] != 1 } {
                append messages [_ "\n - Blockdata '%s'is not OK in new problemtype" [$domNode_old @n]]
                return 1
            } else {
                set domNode [lindex $domNodes 0]
            }
            lassign [_pt_blockdata_or_container_values $domNode $domNode_old] numchangesL messages_local
            if { $numchangesL } {
                append messages $messages_local
            } else {
                set n1 [llength [$domNode selectNodes value|container/value]]
                set n2 [llength [$domNode_old selectNodes value|container/value]]
                if { $n1 != $n2 } {
                    set numchangesL 1
                }
            }
            if { $numchangesL } {
                append messages [_ "\n - Blockdata '%s' has different field" [$domNode @n]]
                incr numchanges $numchangesL
            }
            set xp {condition|blockdata|container[blockdata or condition or container]}
            foreach childNode_old [$domNode_old selectNodes $xp] {
                lassign [_transform_problemtype_elm $childNode_old $domNode $root_old] hc messages_local
                if { $hc } {
                    incr numchanges $hc
                    append messages $messages_local
                }
            }
        }
    }
    return [list $numchanges $messages]
}

proc gid_groups_conds::transform_problemtype_no_messages { spd_file } {
    variable transform_finished
    set transform_finished 0
    variable doc
    variable doc_old
    set numchanges 0
    
    if { ![info exists doc] } {
        return [list $numchanges ""]
    }
    
    TransformMapFill [GiD_Info project problemtype]
    TransformMapFillInverse
    
    set messages ""
    if { [catch { set doc_old [gid_groups_conds::open_XML_file_gzip $spd_file] } errstring] } {
        append messages [_ "ERROR: %s" $errstring]\n
    } else {
        set root [$doc documentElement]
        set root_old [$doc_old documentElement]
        if { [$root nodeName] ne [$root_old nodeName] } {
            append messages [_ "ERROR: expected xml root nodename '%s' instead '%s'\n" [$root_old nodeName] [$root nodeName]]\n
        }
        foreach domNode [$root_old selectNodes {//*[@di_ic!='0']}] {
            uncompress_subtree -mydoc $doc_old $domNode
        }
        foreach domNode [$root selectNodes {//*[@di_ic!='0']}] {
            uncompress_subtree $domNode
        }
        set domNode_from [$root_old selectNodes {/*/units}]
        if { $domNode_from != "" } {
            import_units_from -force $domNode_from
        }
        foreach itrans [$root selectNodes {/*/transforms/transform}] {
            set msg [gid_groups_conds::eval_transform $itrans]
            if {$msg != ""} {
                append messages $msg
                incr numchanges
            }
        }
        set xpList ""
        foreach i [list {blockdata[@n="Internal data"]} postprocess procs display_options \
                global_preferences local_preferences transforms] {
            lappend xpList self::$i
        }
        set xp [join $xpList |]
        foreach domNode_old [$root_old childNodes] {
            if { [$domNode_old nodeName] in "#comment label" } { continue }
            if { [$domNode_old selectNodes $xp] ne "" } { continue }
            lassign [_transform_problemtype_elm $domNode_old $root $root_old] hc messages_local
            if { $hc } {
                incr numchanges
                append messages $messages_local
            }
        }
        $doc_old delete
        foreach domNode [$root selectNodes {//*[@di_ic='0']}] {
            compress_subtree $domNode
        }
        actualize_groups_window
        actualize_conditions_window
        if { [catch { check_groups_entities raise_error } errstring] } {
            append messages $errstring
        }
        check_groups_auto_from_bc
        set transform_finished 1
    }
    return [list $numchanges $messages]
}

#for back compatibility, showing messages
proc gid_groups_conds::transform_problemtype { spd_file } {
    set messages [_ "Transforming model '%s'" [file root [file tail $spd_file]]]
    lassign [gid_groups_conds::transform_problemtype_no_messages $spd_file] numchanges messages_local
    if { $numchanges } {
        append messages $messages_local
        append messages \n[_ "End of transformation. %d changes" $numchanges]
    } else {
        append messages \n[_ "End of transformation. No changes"]
    }
    return [list $numchanges $messages]
}

proc gid_groups_conds::get_transform_finished { } {
    variable transform_finished
    return $transform_finished
}
