#package require tdom
#package require snit
package require customLib_utils
package require msgcat

set err [catch { package present Tk }]
if { !$err } {
    package require fulltktree
}

namespace eval gid_groups_conds {
    namespace export give_unit_all_magnitudes give_unit_magnitude_and_factor \
        give_unit_magnitudes_and_basic \
        give_nice_unit_all_units give_magnitude_pn give_unit_is_magnitude_basic \
        give_unit_factor give_units_list multiply_units \
        is_unit_correct is_unit_correct_for_units_system \
        convert_unit_value \
        convert_value_to_mesh_unit units_to_nice_units units_to_nice_unitsList \
        nice_units_to_units nice_number nice_number_print \
        give_active_default_unit \
        give_default_unit convert_v_to_default convert_value_to_default \
        give_active_unit set_active_unit give_value_in_active_unit convert_value_to_active \
        convert_value_to \
        give_mesh_unit set_mesh_unit \
        add_to_units_preferences get_units_preferences \
        give_units_system_list give_active_units_system set_active_units_system \
        give_printable_unit convert_value_to_printable_unit \
        units_get_global give_units_actualize_value
    
    variable unit_magnitude_and_factor_cache ""
}

# for tclIndex
proc gid_groups_conds::entry_units { args } {}

proc gid_groups_conds::give_unit_all_magnitudes {} {
    variable doc
    variable doc_units

    set ret ""
    set xp {/*/units/unit_magnitude}
    foreach n [$doc_units selectNodes $xp] {
        lappend ret [list [$n @n] [$n @pn]]
    }
    foreach n [$doc selectNodes $xp] {
        lappend ret [list [$n @n] [$n @pn]]
    }
    set xp {/*/units/magnitudes_alias/magnitude_alias}
    foreach n [$doc_units selectNodes $xp] {
        lappend ret [list [$n @n] [$n @pn]]
    }
    foreach n [$doc selectNodes $xp] {
        lappend ret [list [$n @n] [$n @pn]]
    }
    return [lsort -dictionary -index 0 -unique $ret]
}

proc gid_groups_conds::give_unit_magnitude_and_factor { unit } {
    variable unit_magnitude_and_factor_cache
    
    if { [dict exists $unit_magnitude_and_factor_cache $unit] } {
        return [dict get $unit_magnitude_and_factor_cache $unit]
    }
    set message [_ "error: unit '%s' not found for any magnitude" $unit] 
    
    lassign [_parse_unit -no_divisor $unit] unit_list
    
    set err [catch { _give_unit_magnitudes_from_units $unit_list } magnitude_list]
    if { $err } {
        error $message
    }
    set err [catch { _give_unit_factor_vec_list $magnitude_list $unit_list } ret]
    if { !$err } {
        set magnitude_list [_give_unit_magnitude_from_basic $magnitude_list]
        set magnitude [_units_list_to_unit $magnitude_list]
        set ret [list $magnitude {*}$ret]
        dict set unit_magnitude_and_factor_cache $unit $ret
        return $ret
    }
    error $message
}

proc gid_groups_conds::multiply_units { unit1 unit2 } {
    lassign [_parse_unit -no_divisor $unit1] unit_list1
    lassign [_parse_unit -no_divisor $unit2] unit_list2
    set unit_list [concat $unit_list1 $unit_list2]
    set d ""
    foreach i $unit_list {
        lassign $i name exp
        if { ![dict exists $d $name] } {
            dict set d $name $exp
        } else {
            dict set d $name [expr {[dict get $d $name]+$exp}]
        }
    }
    set unit_list ""
    dict for "name exp" $d {
        if { $exp == 0 } { continue }
        lappend unit_list [list $name $exp]
    }
    return [_units_list_to_unit $unit_list]
}

proc gid_groups_conds::give_unit_magnitudes_and_basic { magnitude } {
    variable doc
    variable doc_units

    set ret ""
    set magnitude [_units_resolve_links_magnitude $magnitude]
    set xp [format_xpath {/*/units/unit_magnitude[@n=%s]} $magnitude]
    foreach n [$doc_units selectNodes $xp] {
        lappend ret [list [$n @n] [$n @pn]]
    }
    foreach n [$doc selectNodes $xp] {
        lappend ret [list [$n @n] [$n @pn]]
    }
    if { [llength $ret] == 0 } {
        set xp [format_xpath {/*/units/unit_magnitude[@basic=%s]} $magnitude]
        foreach n [$doc_units selectNodes $xp] {
            lappend ret [list [$n @n] [$n @pn]]
        }
        foreach n [$doc selectNodes $xp] {
            lappend ret [list [$n @n] [$n @pn]]
        }
    }
    if { [llength $ret] == 0 } {
        lappend ret [list $magnitude [units_to_nice_units $magnitude]]
    }

    lassign [_parse_unit -as_magnitude -no_divisor $magnitude] magnitude_list
    
    if { [llength $magnitude_list] > 1 || [lindex $magnitude_list 0 1] != 1 } {
        foreach i $magnitude_list {
            lassign $i m exp
            lappend ret {*}[give_unit_magnitudes_and_basic $m]
        }
    }
    set new_magnitude_list [_give_unit_magnitude_to_basic $magnitude_list one]
    if { [llength $new_magnitude_list] != [llength $magnitude_list] } {
        foreach i $new_magnitude_list {
            lassign $i m exp
            lappend ret {*}[give_unit_magnitudes_and_basic $m]
        }
    }
    return [lsort -dictionary -index 0 -unique $ret]
}

proc gid_groups_conds::give_unit_is_magnitude_basic { magnitude } {
    lassign [_parse_unit -as_magnitude -no_divisor $magnitude] magnitude_list
    if { [llength $magnitude_list] > 1 } { return 0 }

    set new_magnitude_list [_give_unit_magnitude_to_basic $magnitude_list one]
    if { [llength $new_magnitude_list] != [llength $magnitude_list] } {
        return 0
    } else {
        return 1
    }
}

proc gid_groups_conds::give_nice_unit_all_units { magnitude } {
    variable doc
    variable doc_units

    set ret ""
    set magnitude [_units_resolve_links_magnitude $magnitude]
    set xp [format_xpath {/*/units/unit_magnitude[@n=%s]/unit} $magnitude]
    
    set nodes [$doc_units selectNodes $xp]
    lappend nodes {*}[$doc selectNodes $xp]
    
    set dict [dict create "" ""]
    foreach i [give_units_system_list] {
        dict set dict {*}$i
    }
    
    foreach node $nodes {
        set n [$node @n]
        set pn [$node @pn $n]
        set units_system [dict get $dict [$node @units_system ""]]
        set factor [$node @factor ""]
        regsub {e([+-])0(\d\d)$} $factor {e\1\2} factor
        lappend ret [list [units_to_nice_units $n] $pn $factor $units_system [$node @p 3]]
    }
    set xp [format_xpath {string(/*/units/unit_magnitude[@n=%s]/@SI_base)} $magnitude]
    set base [$doc selectNodes $xp]
    if { $base eq "" } {
        set base [$doc_units selectNodes $xp]
    }
    set xp [format_xpath {/*/units/unit_magnitude[@n=%s]/unit[@n=%s and @factor]} $magnitude $base]
    set base_n [$doc selectNodes $xp]
    if { $base_n eq "" } {
        set base_n [$doc_units selectNodes $xp]
    }
    if { $base_n ne "" && ![regexp {^[[:alpha:]]+} [$base_n @n]] } {
        set base_n "" 
    }
    if { $base_n ne "" } {
        if { ![regexp {^[[:alpha:]]+\^(\d+)} [$base_n @n] {} unit_exp] } {
            set unit_exp 1
        }
        set xp [format_xpath {/*/units/SI_prefixes/SI_prefix}]
        foreach node [$doc_units selectNodes $xp] {
            if { [regexp {(.*)\^(.*)} [$node @factor] {} b exp] } {
                set fac [format %.3g [expr {[$base_n @factor]*pow($b,$exp*$unit_exp)}]]
            } else {
                set fac [format %.3g [expr {[$base_n @factor]*pow([$node @factor],$unit_exp)}]]
            }
            regsub {e([+-])0(\d\d)$} $fac {e\1\2} fac
            set n [$base_n @n]
            set base_pn [$base_n @pn $n]
            set units_system [dict get $dict [$base_n @units_system ""]]
            set nice_n [units_to_nice_units [$node @n]$n]
            lappend ret [list $nice_n [$node @pn]$base_pn $fac $units_system 3]
        }
    }
    # remember that lsort is stable and do not change ordering of equal items
    set ret [lsort -index 4 -integer -decreasing $ret]
    set ret [lsort -dictionary -index 0 $ret]
    
    for { set i 0 } { $i < [llength $ret] } { incr i } {
        for { set j [expr {$i+1}] } { $j < [llength $ret] } { incr j } {
            if { [lindex $ret $j 0] ne [lindex $ret $i 0] } { break }
            for { set k 1 } { $k <= 3 } { incr k } {
                if { [lindex $ret $j $k] eq "" } {
                    lset ret $j $k [lindex $ret $i $k]
                }
            }
        }
    }
    set ret [lsort -dictionary -index 0 -unique $ret]
    set ret [lsort -index 4 -integer -increasing $ret]
    
    return $ret
}


proc gid_groups_conds::give_magnitude_pn { magnitude } {
    variable doc
    variable doc_units
    
    set magnitude [_units_resolve_links_magnitude $magnitude]
    set xp [format_xpath {/*/units/unit_magnitude[@n=%s]} $magnitude]
    set node [$doc selectNodes $xp]
    if { $node eq "" } {
        set node [$doc_units selectNodes $xp] 
    }
    if { $node eq "" } {
        error "error: magnitude '$magnitude' does not exist"
    }
    return [$node @pn]
}

proc gid_groups_conds::give_unit_factor { magnitude unit } {
    
    set message [_ "error: unit '%s' not found for magnitude '%s'" $unit $magnitude] 
    
    lassign [_parse_unit -as_magnitude -no_divisor $magnitude] magnitude_list
    lassign [_parse_unit -no_divisor $unit] unit_list
    
    set err [catch { _give_unit_factor_vec_list $magnitude_list $unit_list } ret]
    if { !$err } { return $ret }
    
    set err [catch { _give_unit_magnitudes_from_units $unit_list } new_magnitude_list]
    if { $err } {
        error $message
    }
    set b1 [_give_unit_magnitude_to_basic $magnitude_list all]
    set b2 [_give_unit_magnitude_to_basic $new_magnitude_list all]
    if { [lsort $b1] ne [lsort $b2] } {
        error $message
    }
    set err [catch { _give_unit_factor_vec_list $new_magnitude_list $unit_list } ret]
    if { !$err } { return $ret }

    error $message
}

proc gid_groups_conds::_give_unit_magnitudes_from_units { unit_list } {
    variable doc
    variable doc_units
   
    set magnitude_list ""
    foreach ni $unit_list {
        lassign $ni unit_i exp
        set xp [format_xpath {/*/units/unit_magnitude[unit[@n=%s]]} $unit_i]
        if { [info exists doc] } {
            set n [$doc selectNodes $xp]
        } else {
            set n ""
        }
        if { $n eq "" } {
            set n [$doc_units selectNodes $xp]
        }
        if { $n eq "" } {
            set xp {/*/units/SI_prefixes/SI_prefix}
            foreach pNode [$doc_units selectNodes $xp] {
                if { [string match [$pNode @n]* $unit_i] } {
                    set unit_base [string range $unit_i [string length [$pNode @n]] end]
                    set xp [format_xpath {/*/units/unit_magnitude[@SI_base=%s]} $unit_base]
                    if { [info exists doc] } {
                        set n [$doc selectNodes $xp]
                    } else {
                        set n ""
                    }
                    if { $n eq "" } {
                        set n [$doc_units selectNodes $xp]
                    }
                }
            }
        }
        if { $n eq "" } {
            error "error in gid_groups_conds::_give_unit_magnitudes_from_units"
        }
        lappend magnitude_list [list [$n @n] $exp]
    }
    return $magnitude_list
}

proc gid_groups_conds::_give_unit_magnitude_from_basic { magnitude_list } {
    variable doc
    variable doc_units

    set magnitude_list [lsort $magnitude_list]
    set xp {/*/units/unit_magnitude[@basic]}
    set nodes [$doc_units selectNodes $xp]
    if { [info exists doc] } {
        lappend nodes {*}[$doc selectNodes $xp]
    }
    foreach n $nodes {
        lassign [_parse_unit -as_magnitude -no_divisor [$n @basic]] magnitude_i_list
        if { [lsort $magnitude_i_list] eq $magnitude_list } {
            return [list [list [$n @n] 1]]
        }
    }
    return $magnitude_list
}

proc gid_groups_conds::_give_unit_magnitude_to_basic { magnitude_list one_or_all } {
    variable doc
    variable doc_units

    set new_magnitude_list ""
    
    foreach ni_m $magnitude_list {
        lassign $ni_m magnitude_i exp
        set xp [format_xpath {/*/units/unit_magnitude[@n=%s and @basic]} $magnitude_i]
        if { [info exists doc] } {
            set n [$doc selectNodes $xp]
        } else {
            set n ""
        }
        if { $n eq "" } {
            set n [$doc_units selectNodes $xp]
        }
        if { $n eq "" } {
            lappend new_magnitude_list $ni_m
        } else {
            lassign [_parse_unit -as_magnitude -no_divisor [$n @basic]] magnitude_i_list
            if { $one_or_all eq "all" } {
                set magnitude_i_list [_give_unit_magnitude_to_basic $magnitude_i_list all]
            }
            set ipos_in 0
            foreach i $magnitude_i_list {
                set exp_in [lindex $magnitude_i_list $ipos_in 1]
                lset magnitude_i_list $ipos_in 1 [expr {$exp_in*$exp}]
                incr ipos_in
            }
            lappend new_magnitude_list {*}$magnitude_i_list
        }
    }
    return $new_magnitude_list
}

proc gid_groups_conds::_give_unit_factor_vec_list { magnitude_list unit_list } {

    set fac 1.0
    set addend ""
    foreach ni_m $magnitude_list ni $unit_list {
        lassign $ni_m magnitude_i exp
        lassign $ni unit_i exp_u
        if { $exp != $exp_u } {
            error "error in _give_unit_factor_vec_list. exp different"
        }
        set err [catch { _give_unit_simple $magnitude_i $unit_i } ret]
        if { $err } {
            return [_give_unit_factor_vec_list_basic $magnitude_list $unit_list]
        } else {
            lassign $ret uNode prefix_exp
        }
        set fac [expr {$fac*pow([$uNode @factor]*$prefix_exp,$exp)}]
        if { [$uNode hasAttribute addend] && [llength $magnitude_list] == 1 && $exp == 1.0 } {
            set addend [$uNode @addend]
        }
    }
    return [list $fac $addend]
}

proc gid_groups_conds::_give_unit_factor_vec_list_basic { magnitude_list unit_list } {
    variable doc
    variable doc_units

    set ipos 0
    foreach ni_m $magnitude_list {
        lassign $ni_m magnitude_i exp
        set xp [format_xpath {/*/units/unit_magnitude[@n=%s and @basic]} $magnitude_i]
        set n [$doc selectNodes $xp]
        if { $n eq "" } {
            set n [$doc_units selectNodes $xp]
        }
        if { $n ne "" } {
            lassign [_parse_unit -as_magnitude -no_divisor [$n @basic]] \
                magnitude_i_list
            set ipos_in 0
            foreach i $magnitude_i_list {
                set exp_in [lindex $magnitude_i_list $ipos_in 1]
                lset magnitude_i_list $ipos_in 1 [expr {$exp_in*$exp}]
                incr ipos_in
            }
            set magnitude_list [lreplace $magnitude_list $ipos $ipos {*}$magnitude_i_list]
            return [_give_unit_factor_vec_list $magnitude_list $unit_list]
        }
        incr ipos
    }
    error "error in gid_groups_conds::_give_unit_factor_vec_list_basic"
}

# unit must be a simple unit name (without ^ or / or ())
proc gid_groups_conds::_give_unit_simple { magnitude unit } {
    variable doc
    variable doc_units

    set magnitude [_units_resolve_links_magnitude $magnitude]
    set xp0 {(/*/units/unit_magnitude[@n=%s]/unit[@n=%s and @factor])[1]}
    set xp [format_xpath $xp0 $magnitude $unit]
    if { [info exists doc] } {
        if { [set n [$doc selectNodes $xp]] ne "" } { return [list $n 1.0] }
    }
    if { [set n [$doc_units selectNodes $xp]] ne "" } { return [list $n 1.0] }
    
    set xp0 {string(/*/units/unit_magnitude[@n=%s]/@SI_base)}
    set xp [format_xpath $xp0 $magnitude]
    set base [$doc_units selectNodes $xp]
    set xp0 {(/*/units/unit_magnitude[@n=%s]/unit[@n=%s])[1]}
    set xp [format_xpath $xp0 $magnitude $base]
    set n [$doc_units selectNodes $xp]

    set xp {/*/units/SI_prefixes/SI_prefix}
    foreach pNode [$doc_units selectNodes $xp] {
        if { $unit eq "[$pNode @n]$base" } {
            if { [regexp {(.*)\^(.*)} [$pNode @factor] {} base exp] } {
                set fac [expr {pow($base,$exp)}]
            } else {
                set fac [$pNode @factor]
            }
            return [list $n $fac]
        }
    }
    error [_ "error: unit '%s' not found for magnitude '%s'" $unit $magnitude]
}

proc gid_groups_conds::_units_resolve_links_magnitude { magnitude } {
    variable doc
    variable doc_units

    set xp0 {/*/units/magnitudes_alias/magnitude_alias[@n=%s]}
    
    if { [info exists doc] } {
        while { [set mNode [$doc selectNodes [format_xpath $xp0 $magnitude]]] ne "" } {
            set magnitude [$mNode @target]
        }
    }
    while { [set mNode [$doc_units selectNodes [format_xpath $xp0 $magnitude]]] ne "" } {
        set magnitude [$mNode @target]
    }
    return $magnitude
}

proc gid_groups_conds::_units_resolve_links { vec } {
    variable doc
    variable doc_units

    set ipos 0
    foreach i $vec {
        set magnitude [_units_resolve_links_magnitude [lindex $i 0]]
        lset vec $ipos 0 $magnitude
        incr ipos
    }
    return $vec
}

proc gid_groups_conds::_parse_unit { args } {
    
    set optional {
        { -as_magnitude "" 0 }
        { -no_divisor "" 0 }
    }
    set compulsory "unit"
    parse_args $optional $compulsory $args
    
    set unit [nice_units_to_units $unit]

    lassign "" numerator divisor
    set middot \u00b7
    set rex_id "(\[^*/${middot}()\]+)"
    set num_div_stack num
    while { $unit ne "" } {
        if { [regexp -indices "^\\s*$rex_id" $unit {} idxs] } {
            set id [string range $unit {*}$idxs]
            set num_div num
            foreach i $num_div_stack {
                if { $i eq "div" } {
                    if { $num_div eq "num" } {
                        set num_div "div"
                    } else { set num_div "par" }
                }
            }
            if { ![regexp {(.*)\^(.*)} $id {} u exponent] } {
                lassign [list $id 1] u exponent
            }
            if { $num_div eq "num" } {
                lappend numerator [list $u $exponent]
            } else {
                lappend divisor [list $u $exponent]
            }
            lset num_div_stack end par
        } elseif { [regexp -indices "^\\s*(\[*${middot}\])" $unit {} idxs] } {
            # nothing
        } elseif { [regexp -indices {^\s*([/])} $unit {} idxs] } {
            lset num_div_stack end div
        } elseif { [regexp -indices {^\s*([(])} $unit {} idxs] } {
            lappend num_div_stack par
        } elseif { [regexp -indices {^\s*([)])} $unit {} idxs] } {
            set num_div_stack [lrange $num_div_stack 0 end-1]
            lset num_div_stack end par
        } elseif { [regexp -indices {^(\s+)} $unit {} idxs] } {
            #nothing
        } else {
            error "error in give_units"
        }
        set unit [string range $unit [expr {[lindex $idxs 1]+1}] end]
    }
    if { $as_magnitude } {
        set numerator [_units_resolve_links $numerator]
        set divisor [_units_resolve_links $divisor]
    }
    if { $no_divisor } {
        if { [llength $numerator] == 1 && [lindex $numerator 0 0] == 1 } {
            set numerator ""
        }
        foreach di $divisor {
            lassign $di unit exp
            lappend numerator [list $unit [expr {-1*$exp}]]    
        }
        set divisor ""
    }
    return [list $numerator $divisor]
}

proc gid_groups_conds::give_units_list { args } {
    variable doc
    variable doc_units

    set optional {
        { -include_this unit "" }
        { -p auto|all|1|2|3 auto }
        { -for_units_system units_system "-" }
    }
    set compulsory "magnitude"
    parse_args $optional $compulsory $args

    if { $magnitude eq "" } {
        error "error in gid_groups_conds::give_units_list. magnitude is void"
    }
    set units_list ""
    
    lassign [_parse_unit -as_magnitude -no_divisor $magnitude] magnitude_list

    lappend units_list {*}[_give_units_list_do -for_units_system $for_units_system \
            $magnitude_list 1]
    if { $p in "auto all 2" } {
        lappend units_list {*}[_give_units_list_do -for_units_system $for_units_system \
                $magnitude_list 2]
    }
    if { $p eq "auto" && [llength $units_list] < 2 } {
        lappend units_list {*}[_give_units_list_do -for_units_system $for_units_system \
                -only_for_units_system 1 $magnitude_list 3]
    } elseif { $p in "all 3" && [llength $units_list] < 200 } {
        lappend units_list {*}[_give_units_list_do -for_units_system $for_units_system \
                $magnitude_list 3]
    }
    
    set new_magnitude_list [_give_unit_magnitude_to_basic $magnitude_list one]
    
    if { [llength $new_magnitude_list] != [llength $magnitude_list] } {
        lappend units_list {*}[_give_units_list_do -for_units_system $for_units_system \
                $new_magnitude_list 1]
        if { $p in "auto all 2" } {
            lappend units_list {*}[_give_units_list_do -for_units_system $for_units_system \
                    $new_magnitude_list 2]
        }
        if { $p eq "auto" && [llength $units_list] < 5 } {
            lappend units_list {*}[_give_units_list_do -for_units_system $for_units_system \
                    -only_for_units_system 1 $new_magnitude_list 3]
        } elseif { $p in "all 3" && [llength $units_list] < 200 } {
            lappend units_list {*}[_give_units_list_do -for_units_system $for_units_system \
                    $new_magnitude_list 3]
        }
    }

    set active_unit [give_active_unit $magnitude]
    
    if { $for_units_system ne "-" } {
        set active_unit [give_active_default_unit -for_units_system $for_units_system \
                -check_this_unit $active_unit $magnitude]
    }
    
    set uList ""
    lappend uList {*}[get_units_preferences $magnitude]
    if { $include_this ne "" } { lappend uList $include_this }
    if { $active_unit ne "" } { lappend uList $active_unit }

    foreach unit $uList {
        if { ![is_unit_correct $magnitude $unit] } { continue }
        set ipos [lsearch -exact $units_list $unit]
        if { $ipos != -1 } { set units_list [lreplace $units_list $ipos $ipos] }
        set units_list [linsert $units_list 0 $unit]
    }
    set unique_units_list ""
    foreach i $units_list {
        if { $i ni $unique_units_list } {
            lappend unique_units_list $i
        }
    }
    return $unique_units_list
}

proc gid_groups_conds::give_units_system_list {} {
    variable doc
    variable doc_units

    # [= Any] for translations
    #set ret_units_system_list [list [list "" Any]]
    set ret_units_system_list ""
    
    set xp {/*/units/units_system}
    foreach node [$doc selectNodes $xp] {
        set n [$node @n]
        if { [$node hasAttribute pn] } {
            set pn [$node @pn]
        } elseif { [$node hasAttribute dict] } {
            set d [split [$node @dict] ","]
            set pn [dict get $d $n]
        } else {
            set pn $n
        }
        lappend ret_units_system_list [list $n $pn]
    }
    foreach node [$doc_units selectNodes $xp] {
        set n [$node @n]
        if { [$node hasAttribute pn] } {
            set pn [$node @pn]
        } elseif { [$node hasAttribute dict] } {
            set d [split [$node @dict] ","]
            set pn [dict get $d $n]
        } else {
            set pn $n
        }
        lappend ret_units_system_list [list $n $pn]
    }
    return [lsort -dictionary -unique -index 0 $ret_units_system_list]
}

proc gid_groups_conds::give_active_units_system {} {
    variable doc
    variable doc_units
    
    set xp {/*/units/units_system[@active="1"]}
    if { [info exists doc] } {
        set node [$doc selectNodes $xp]
    } else {
        set node ""
    }
    if { $node eq "" } {
        set node [$doc_units selectNodes $xp]
    }
    # [= Any] for translations
    if { $node eq "" } { return [list "" Any] }
    set n [$node @n]
    if { [$node hasAttribute pn] } {
        set pn [$node @pn]
    } elseif { [$node hasAttribute dict] } {
        set d [split [$node @dict] ","]
        set pn [dict get $d $n]
    } else {
        set pn $n
    }
    return [list $n $pn]
}

proc gid_groups_conds::set_active_units_system { units_system } {
    variable doc
    variable doc_units

    set found 0
    set xp {/*/units/units_system}
    foreach node [$doc selectNodes $xp] {
        if { [$node @n] eq $units_system } {
            set found 1
            if { [$node @active ""] != 1 } { setAttributesN $node [list active 1] }
        } else {
            if { [$node @active ""] != 0 } { setAttributesN $node [list active 0] }
        }
    }
    if { !$found && $units_system ne "" } {
        set xp [format_xpath {/*/units/units_system[@n=%s]} $units_system]
        set node [$doc_units selectNodes $xp]
        set parent [$doc selectNodes {/*/units}]
        set node [addN $parent units_system [concat {*}[$node selectNodes @*]]]
        if { [$node @active ""] != 1 } { setAttributesN $node [list active 1] }
    }
    set parent [get_preference_node -local_global global units]
    set found 0
    foreach node [$parent selectNodes units_system] {
        if { [$node @n] eq $units_system } {
            set found 1
            $node setAttribute active 1
        } else {
            $node delete
        }
    }
    if { !$found && $units_system ne "" } {
        set xp [format_xpath {/*/units/units_system[@n=%s]} $units_system]
        set node [$doc_units selectNodes $xp]
        set node [$parent appendChild [$node cloneNode]]
        $node setAttribute active 1
    }
}

proc gid_groups_conds::give_units_actualize_value { args } {

    set optional {
        { -for_units_system units_system "-" }
        { -check_this_unit unit "" }
        { -perform_the_change boolean 0 }
    }
    set compulsory "node"
    parse_args $optional $compulsory $args
    
    if { $check_this_unit ne "" } {
        set unit $check_this_unit
    } else {
        set unit ""
    }
    if { [$node @unit_definition ""] ne "" } {
        set magnitude [_units_resolve_links_magnitude [$node @unit_definition]]
        if { $unit eq "" } { set unit [give_active_unit $magnitude] }
    } elseif { [$node @unit_mesh_definition ""] == 1 }  {
        set magnitude L
        if { $unit eq "" } { set unit [give_mesh_unit] }
    } else {
        return ""
    }
    if { $for_units_system eq "-" } {
        lassign [give_active_units_system] for_units_system
    }
    set newunit [give_active_default_unit -for_units_system $for_units_system -check_this_unit \
            $unit $magnitude]
    if { $unit eq $newunit } {
        set haschanges 0
    } else {
        set haschanges 1
        if { $perform_the_change } {
            if { [$node @unit_definition ""] ne "" } {
                gid_groups_conds::set_active_unit $magnitude $newunit
            } else {
                gid_groups_conds::set_mesh_unit $newunit
            }
        }
    }
    return [list $newunit $haschanges]
}

proc gid_groups_conds::_give_units_list_do { args } {
    variable doc
    variable doc_units
    
    set optional {
        { -for_units_system units_system "-" }
        { -only_for_units_system boolean 0 }
    }
    set compulsory "magnitude_list p"
    parse_args $optional $compulsory $args

    set ret_units_list ""
    
    if { $for_units_system ne "-" } {
        set units_system $for_units_system
    } else {
        lassign [give_active_units_system] units_system
    }
    set lists ""
    foreach ni $magnitude_list {
        lassign $ni magnitude_i exp
        set list ""
        foreach i [_give_units_list1 $magnitude_i $units_system $p $only_for_units_system] {
            lappend list [list $i $exp]
        }
        lappend lists $list
    }
    foreach units_list [_give_units_combine $lists] {
        set ipos 0
        foreach units_i $units_list {
            lassign $units_i u_i exp
            lassign [_parse_unit -no_divisor $u_i] u_i_list
            if { [llength $u_i_list] > 1 } {
                set ipos_in 0
                foreach ui $u_i_list {
                    lassign $ui u exp_in
                    lset u_i_list $ipos_in 1 [expr {$exp_in*$exp}]
                    incr ipos_in
                }
                set units_list [lreplace $units_list $ipos $ipos {*}$u_i_list]
            }
            incr ipos
        }
        lappend ret_units_list [_units_list_to_unit $units_list]
    }
    return $ret_units_list
}

proc gid_groups_conds::_units_list_to_unit { units_list } {

    lassign [list ""] nList dList
    foreach units_i $units_list {
        lassign $units_i u_i exp
        if { $exp > 0 } {
            if { $exp == 1 } {
                lappend nList $u_i
            } else {
                lappend nList $u_i^$exp 
            }
        } else {
            if { $exp == -1 } {
                lappend dList $u_i
            } else {
                lappend dList $u_i^[expr {-1*$exp}]
            } 
        }
    }
    if { [llength $nList] } {
        set unit [join $nList *]
    } else {
        set unit 1
    }
    if { [llength $dList] } {
        append unit "/"
        if { [llength $dList] > 1 } { append unit "(" }
        append unit [join $dList *]
        if { [llength $dList] > 1 } { append unit ")" }
    }
    return $unit
}

proc gid_groups_conds::_give_units_combine { lists } {
     
    set ret ""
    if { [llength $lists] == 1 } {
        foreach j [lindex $lists 0] {
            lappend ret [list $j]
        }
    } else {
        foreach i [_give_units_combine [lrange $lists 1 end]] {
            foreach j [lindex $lists 0] {
                lappend ret [concat [list $j] $i]
            }
        }
    }
    return $ret
}

proc gid_groups_conds::_give_units_list1 { magnitude1 units_system p only_for_units_system } {
    variable doc
    variable doc_units
    
    set xp0 {/*/units/unit_magnitude[@n=%s]/unit[}
    if { $units_system ne "" } {
        if { $only_for_units_system } {
            append xp0 {@units_system=%s and }
        } else {
            append xp0 {(@units_system=%s or not(@units_system)) and }
        }
    }
    if { $p == 1 } {
        append xp0 {(@p=%s or not(@p))}
    } else {
        append xp0 {@p=%s}
    }
    append xp0 {]}
    if { $units_system ne "" } {
        set xp [format_xpath $xp0 $magnitude1 $units_system $p]
    } else {
        set xp [format_xpath $xp0 $magnitude1 $p]
    }
    set units_list ""
    if { [info exists doc] } {
        foreach n [$doc selectNodes $xp] {
            lappend units_list [$n @n]
        }
    }
    foreach n [$doc_units selectNodes $xp] {
        lappend units_list [$n @n]
    }
    return [lsort -dictionary -unique $units_list]
}

proc gid_groups_conds::is_unit_correct { magnitude unit } {
    
    set err [catch { give_unit_factor $magnitude $unit }]
    if { $err } {
        return 0
    } else {
        return 1
    }
}

proc gid_groups_conds::is_unit_correct_for_units_system { units_system magnitude unit } {
    variable doc
    variable doc_units
    
    if { ![is_unit_correct $magnitude $unit] } { return 0 }
    if { $units_system eq "" } { return 1 }
    
    set magnitude [_units_resolve_links_magnitude $magnitude]
    
    set xp [format_xpath {/*/units/unit_magnitude[@n=%s]/unit[@n=%s and @units_system=%s]} \
            $magnitude $unit $units_system]
    set node [$doc selectNodes $xp]
    if { $node eq "" } {
        set node [$doc_units selectNodes $xp]
    }
    if { $node ne "" } { return 1 }

    lassign [_parse_unit -as_magnitude -no_divisor $magnitude] magnitude_list
    lassign [_parse_unit -no_divisor $unit] units_list
    
    set err [catch { _give_unit_magnitudes_from_units $units_list } new_magnitude_list]
    if { $err } { return 0 }

    
    if { [llength $new_magnitude_list] > 1 || [lindex $new_magnitude_list 0 1] != 1 } {
        foreach i $new_magnitude_list j $units_list {
            lassign $i m exp1
            lassign $j u exp2
            if { $exp1 != $exp2 } { return 0 }
            if { ![is_unit_correct_for_units_system $units_system $m $u] } {
                return 0
            }
        }
        return 1
    }
    return 0
}

proc gid_groups_conds::convert_unit_value { magnitude value unit_from unit_to} {

    if { $unit_from eq "" } { return $value }
    if { $value eq "" } { return "" }
    if { ![string is double $value] } { return "" }

    lassign [give_unit_factor $magnitude $unit_from] fac_from addend_from
    lassign [give_unit_factor $magnitude $unit_to] fac_to addend_to
  
    if { $addend_from ne "" } {
        set value [expr {($value+$addend_from)*$fac_from}]
    } else {
        set value [expr {$value*$fac_from}]
    }
    if { $addend_to ne "" } {
        set value [expr {$value/double($fac_to)-$addend_to}]
    } else {
        set value [expr {$value/double($fac_to)}]
    }
    return $value
    #return [nice_number $value]
}

proc gid_groups_conds::give_default_unit { magnitude } {
    variable doc
    variable doc_units

    set magnitude [_units_resolve_links_magnitude $magnitude]
    set xp [format_xpath {string(/*/units/unit_magnitude[@n=%s]/@default)} \
            $magnitude]
    if { [info exists doc] } {
        set unit [$doc selectNodes $xp]
    } else {
        set unit ""
    }
    if { $unit eq "" } {
        set unit [$doc_units selectNodes $xp]
    }
    if { $unit ne "" } { return $unit }
    
    lassign [_parse_unit -as_magnitude -no_divisor $magnitude] magnitude_list
    
    if { [llength $magnitude_list] > 1 || [lindex $magnitude_list 0 1] != 1 } {
        set units_list ""
        foreach i $magnitude_list {
            lassign $i m exp
            lassign [_parse_unit -no_divisor [give_default_unit $m]] units_i_list
            foreach i_in $units_i_list {
                lassign $i_in u exp_in
                lappend units_list [list $u [expr {$exp*$exp_in}]]
            }
        }
        return [_units_list_to_unit $units_list]
    } else {
        set xp [format_xpath {/*/units/unit_magnitude[@n=%s and @basic]} $magnitude]
        if { [info exists doc] } {
            set node [$doc selectNodes $xp]
        } else {
            set node ""
        }
        if { $node eq "" } {
            set node [$doc_units selectNodes $xp]
        }
        if { $node eq "" } {
            error "error in gid_groups_conds::give_default_unit"
        }
        return [give_default_unit [$node @basic]]
    }
}

proc gid_groups_conds::give_active_default_unit { args } {
    variable doc
    variable doc_units

    set optional {
        { -for_units_system units_system "-" }
        { -check_this_unit unit "" }
    }
    set compulsory "magnitude"
    parse_args $optional $compulsory $args
    
    set magnitude [_units_resolve_links_magnitude $magnitude]
    
    if { $for_units_system ne "-" } {
        set units_system $for_units_system
    } else {
        lassign [give_active_units_system] units_system
    }
    
    if { $check_this_unit ne "" } {
        set ret [is_unit_correct_for_units_system $units_system $magnitude $check_this_unit]
        if { $ret } { return $check_this_unit }
    }
    set unit [give_active_unit $magnitude]
    set ret [is_unit_correct_for_units_system $units_system $magnitude $unit]
    if { $ret } { return $unit }
   
    set unit [give_default_unit $magnitude]
    set ret [is_unit_correct_for_units_system $units_system $magnitude $unit]
    if { $ret } { return $unit }

    if { $units_system ne "" } {
        set xp [format_xpath {(/*/units/unit_magnitude[@n=%s]/unit[@units_system=%s])[1]} \
                $magnitude $units_system]
    } else {
        set xp [format_xpath {(/*/units/unit_magnitude[@n=%s]/unit)[1]} \
                $magnitude]
    }
    set node [$doc selectNodes $xp]
    if { $node eq "" } {
        set node [$doc_units selectNodes $xp]
    }
    if { $node ne "" } { return [$node @n] }
    
    lassign [_parse_unit -as_magnitude -no_divisor $magnitude] magnitude_list
    
    if { [llength $magnitude_list] > 1 || [lindex $magnitude_list 0 1] != 1 } {
        set units_list ""
        foreach i $magnitude_list {
            lassign $i m exp
            set u [give_active_default_unit -for_units_system $units_system $m]
            lassign [_parse_unit -no_divisor $u] units_i_list
            foreach i_in $units_i_list {
                lassign $i_in u exp_in
                lappend units_list [list $u [expr {$exp*$exp_in}]]
            }
        }
        return [_units_list_to_unit $units_list]
    } else {
        set xp [format_xpath {/*/units/unit_magnitude[@n=%s and @basic]} $magnitude]
        set node [$doc selectNodes $xp]
        if { $node eq "" } {
            set node [$doc_units selectNodes $xp]
        }
        if { $node eq "" } {
            return [give_default_unit $magnitude]
        }
        return [give_active_default_unit -for_units_system $units_system [$node @basic]]
    }
}

proc gid_groups_conds::convert_v_to_default { v magnitude unit_from } {

    set unit_to [give_default_unit $magnitude]
    return [convert_unit_value $magnitude $v $unit_from $unit_to]
}

proc gid_groups_conds::convert_value_to_default { valueNode } {

    set v [get_domnode_attribute $valueNode v]
    set magnitude [$valueNode @unit_magnitude ""]
    if { $magnitude eq "" } { return $v }

    set unit_to [give_default_unit $magnitude]  
    return [convert_unit_value $magnitude $v [$valueNode @units] $unit_to]
}

proc gid_groups_conds::convert_value_to_active { valueNode } {

    set v [get_domnode_attribute $valueNode v]
    if { [$valueNode @unit_magnitude ""] eq "" } {
        return $v
    }
    set unit_to [give_active_unit [$valueNode @unit_magnitude]]

    return [convert_unit_value [$valueNode @unit_magnitude] $v [$valueNode @units ""] $unit_to]
}

proc gid_groups_conds::convert_value_to { valueNode unit_to } {

    set v [get_domnode_attribute $valueNode v]
    if { [$valueNode @unit_magnitude ""] eq "" } {
        return $v
    }
    return [convert_unit_value [$valueNode @unit_magnitude] $v [$valueNode @units] $unit_to]
}

proc gid_groups_conds::convert_value_to_printable_unit { valueNode } {

    return [give_printable_unit [$valueNode @units ""]]
}
proc gid_groups_conds::give_mesh_unit {} {
    variable doc
    variable doc_units

    set xp {string(/*/units/unit_mesh/@n)}
    set unit [$doc selectNodes $xp]
    if { $unit eq "" } {
        set unit [$doc_units selectNodes $xp]
    }
    if { $unit eq "" } {
        set unit [give_active_unit L]
    }
    return $unit
}

proc gid_groups_conds::set_mesh_unit { unit } {
    variable doc
    variable doc_units
    
    set parent [$doc selectNodes {/*/units}]
    set node [addN -ifnotexists $parent unit_mesh ""]

    if { [$node @n ""] ne $unit } {
        setAttributesN $node [list n $unit]
    }
    set parent [get_preference_node -local_global global units]
    set xp2 {unit_mesh}
    set node [$parent selectNodes $xp2]
    if { $node eq "" } {
        set node [$parent appendChildTag unit_mesh]
    }
    $node setAttribute n $unit
}

proc gid_groups_conds::give_active_unit { magnitude } {
    variable doc
    variable doc_units

    set magnitude [_units_resolve_links_magnitude $magnitude]
    set xp [format_xpath {string(/*/units/unit_magnitude[@n=%s]/@active)} \
            $magnitude]
    if { [info exists doc] } {
        set unit [$doc selectNodes $xp]
    } else {
        set unit ""
    }
    if { $unit eq "" } {
        set unit [$doc_units selectNodes $xp]
    }
    if { $unit ne "" } { return $unit }
    
    set xp [format_xpath {/*/units/unit_magnitude[@n=%s]} $magnitude]
    if { [info exists doc] } {
        set node1 [$doc selectNodes $xp]
    } else {
        set node1 ""
    }
    set node2 [$doc_units selectNodes $xp]
#     set has_basic 0
#     if { $node1 ne "" && [$node1 hasAttribute basic] } { set has_basic 1 }
#     if { $node2 ne "" && [$node2 hasAttribute basic] } { set has_basic 1 }
#     if { !$has_basic } {
#         if { $node1 ne "" && [$node1 hasAttribute default] } { return [$node1 @default] }
#         if { $node2 ne "" && [$node2 hasAttribute default] } { return [$node2 @default] }
#     }
    if { $node1 ne "" && [$node1 hasAttribute default] } { return [$node1 @default] }
    if { $node2 ne "" && [$node2 hasAttribute default] } { return [$node2 @default] }


    lassign [_parse_unit -as_magnitude -no_divisor $magnitude] magnitude_list
    
    if { [llength $magnitude_list] > 1 || [lindex $magnitude_list 0 1] != 1 } {
        set units_list ""
        foreach i $magnitude_list {
            lassign $i m exp
            lassign [_parse_unit -no_divisor [give_active_unit $m]] units_i_list
            foreach i_in $units_i_list {
                lassign $i_in u exp_in
                lappend units_list [list $u [expr {$exp*$exp_in}]]
            }
        }
        return [_units_list_to_unit $units_list]
    } else {
        set xp [format_xpath {/*/units/unit_magnitude[@n=%s and @basic]} $magnitude]
        if { [info exists doc] } {
            set node [$doc selectNodes $xp]
        } else {
            set node ""
        }
        if { $node eq "" } {
            set node [$doc_units selectNodes $xp]
        }
        if { $node eq "" } {
            return [give_default_unit $magnitude]
        }
        return [give_active_unit [$node @basic]]
    }
}

proc gid_groups_conds::set_active_unit { magnitude unit } {
    variable doc
    variable doc_units

    set haschanges 0
    set magnitude [_units_resolve_links_magnitude $magnitude]
    set xp [format_xpath {/*/units/unit_magnitude[@n=%s]} $magnitude]
    set node [$doc selectNodes $xp]
    if { $node eq "" } {
        set node [$doc_units selectNodes $xp]
        set parent [$doc selectNodes {/*/units}]
        set node [addN $parent unit_magnitude [concat {*}[$node selectNodes @*]]]
        set haschanges 1
    }
    if { [$node @active ""] ne $unit } {
        setAttributesN $node [list active $unit]
        set haschanges 1
    }
    set parent [get_preference_node -local_global global units]
    set xp2 [format_xpath {unit_magnitude[@n=%s]} $magnitude]
    set node [$parent selectNodes $xp2]
    if { $node eq "" } {
        set node [$doc_units selectNodes $xp]
        set node [$parent appendChild [$node cloneNode]]
    }
    $node setAttribute active $unit
    
    return $haschanges
}

proc gid_groups_conds::add_to_units_preferences { magnitude unit } {

    set parent [get_preference_node -local_global global units]
    set xp [format_xpath {unit_magnitude_user_sel[@n=%s]} $magnitude]
    set node [$parent selectNodes $xp]
    if { $node eq "" } {
        set node [$parent appendChildTag unit_magnitude_user_sel]
        $node setAttribute n $magnitude
    }
    set values [split [$node @values ""] ","]
    set ipos [lsearch -exact $values $unit]
    if { $ipos != -1 } { set values [lreplace $values $ipos $ipos] }
    set values [linsert [lrange $values 0 0] 0 $unit]
    $node setAttribute values [join $values ","]
}

proc gid_groups_conds::get_units_preferences { magnitude } {

    set parent [get_preference_node -local_global global units]
    set xp2 [format_xpath {unit_magnitude_user_sel[@n=%s]} $magnitude]
    set node [$parent selectNodes $xp2]
    if { $node eq "" } { return "" }
    return [split [$node @values ""] ","]
}

proc gid_groups_conds::give_printable_unit { unit } {
    variable doc
    variable doc_units

    set xp [format_xpath {/*/units/unit_magnitude/unit[@n=%s]} $unit]
    set node [$doc selectNodes $xp]
    if { $node eq "" } {
        set node [$doc_units selectNodes $xp]
    }
    if { $node ne "" } {
        return [$node @printable_name [$node @n]]
    }
    
    lassign [_parse_unit -no_divisor $unit] unit_list
    
    if { [llength $unit_list] > 1 || [lindex $unit_list 0 1] != 1 } {
        set unit_list_new ""
        foreach i $unit_list {
            lassign $i u exp
            lappend unit_list_new [list [give_printable_unit $u] $exp]
        }
        set unit [_units_list_to_unit $unit_list_new]
    }
    return $unit
}

proc gid_groups_conds::units_get_global {} {
    
    set domNode_from [get_preference_node -local_global global units]
    import_units_from $domNode_from
}

proc gid_groups_conds::import_units_from { args } {
    variable doc
    variable doc_units
    
    set optional {
        { -force "" 0 }
    }
    set compulsory "domNode_from"
    parse_args $optional $compulsory $args

    foreach node [$domNode_from selectNodes unit_magnitude] {
        set magnitude [$node @n]
        set active [$node @active 1]
        if { ![is_unit_correct $magnitude $active] } { continue }
        
        set xp [format_xpath {/*/units/unit_magnitude[@n=%s]} $magnitude]
        if { [info exists doc_units] } {
            set node1 [$doc_units selectNodes $xp]
        } else {
            set node1 ""
        }
        set node2 [$doc selectNodes $xp]
        if { $node1 eq "" } { continue }
        if { $node2 ne "" && !$force } { continue }
        if { $node2 eq "" } { 
            set parent [$doc selectNodes {/*/units}]
            set node2 [$parent appendChild [$node1 cloneNode]]
        }
        $node2 setAttribute active $active
    }
    set xp {unit_mesh[1]}
    set node [$domNode_from selectNodes $xp]
    if { $node ne "" } {
        set unit [$node @n]
        set xp {/*/units/unit_mesh}
        set node2 [$doc selectNodes $xp]
        set do_change 1
        if { $node2 ne "" && !$force } { set do_change 0 }
        if { ![is_unit_correct L $unit] } { set do_change 0 }
        if { $do_change } {
            if { $node2 eq "" } { 
                set parent [$doc selectNodes {/*/units}]
                set node2 [$parent appendChildTag unit_mesh]
            }
            $node2 setAttribute n $unit
        }
    }
    set xp {units_system[@active="1"]}
    set node [$domNode_from selectNodes $xp]
    if { $node ne "" } {
        set units_system [$node @n]
        set xp [format_xpath {/*/units/units_system[@n=%s]} $units_system]
        if { [info exists doc_units] } {
            set node1 [$doc_units selectNodes $xp]
        } else {
            set node1 ""
        }
        set node2 [$doc selectNodes $xp]
        if { $node1 ne "" || $node2 ne "" } {
            set found 0
            foreach node_in [$doc selectNodes {/*/units/units_system}] {
                if { [$node_in @n] eq $units_system } {
                    set found 1
                    if { [$node_in @active ""] != 1 } { $node_in setAttribute active 1 }
                } else {
                    if { [$node_in @active ""] != 0 } { $node_in setAttribute active 0 }
                }
            }
            if { !$found } {
                set parent [$doc selectNodes {/*/units}]
                set node [$parent appendChild [$node1 cloneNode]]
                if { [$node @active ""] != 1 } { $node setAttribute active 1 }
            }
        }
    }
}

proc gid_groups_conds::give_value_in_active_unit { args } {
    variable doc
    variable doc_units
    
    set optional {
        { -ov point|line|surface|volume "" }
    }
    set compulsory "domNode"
    parse_args $optional $compulsory $args

    set value [get_domnode_attribute $domNode v]
    set unit_magnitude [get_domnode_attribute $domNode unit_magnitude]

    if { $ov ne "" } {
        set unit_magnitude [give_magnitude $unit_magnitude $ov]
    }

    set active_unit [give_active_unit $unit_magnitude]
    set v [convert_unit_value $unit_magnitude $value [$domNode @units ""] $active_unit]
    return [list $v $active_unit]
}


proc gid_groups_conds::convert_value_to_mesh_unit { value unit } {
    variable doc

    set units_mesh [$doc selectNodes "string(/*/units/unit_mesh/@n)"]
    return [convert_unit_value L $value $unit $units_mesh]
}

proc gid_groups_conds::units_to_nice_units { units } {
    set map [list ^2 \u0b2 ^3 \u0b3 ^4 \u2074 * \u00b7]
    return [string map $map $units]
}

proc gid_groups_conds::units_to_nice_unitsList { unitsList } {
    set map [list ^2 \u0b2 ^3 \u0b3 ^4 \u2074 * \u00b7]

    set ret ""
    foreach i $unitsList {
        lappend ret [string map $map $i]
    }
    return $ret
}

proc gid_groups_conds::nice_units_to_units { units } {
    set map [list \u0b2 ^2 \u0b3 ^3 \u2074 ^4 \u00b7 *]
    return [string map $map $units]
}

proc gid_groups_conds::give_magnitude { magnitude ov } {
    
    set magnitude [_units_resolve_links_magnitude $magnitude]
    if { [llength [split $magnitude ,]] > 1 } {
        return [dict get [split $magnitude ,] $ov]
    } else {
        return $magnitude
    }
}

################################################################################
#    entry_units
################################################################################

if { [info command ::snit::widgetadaptor] ne "" } {
snit::widgetadaptor gid_groups_conds::entry_units {
    option -unit_magnitude_variable ;# this takes precedence over -unit_magnitude
    option -unit_magnitude
    option -ov_variable
    option -units_width 0
    option -edit_mode 0
    option -units_system "-"
    option -create_tree init
    option -toctree ""
    option -state_combobox_tree "normal"
    option -state_entry "normal"

    option -value_variable ""
    option -units_variable
    
    option -value_node ""    

    delegate method * to hull
    delegate option * to hull

    variable units_old ""
    variable units_list ""
    variable entry
    variable combo
    variable transform_unit_value 1
    
    variable internal_units_variable
    
    constructor args {
        installhull using ttk::panedwindow -orient horizontal

        $self configurelist $args
        
        if { $options(-units_variable) eq "" } {
            $self configure -units_variable [myvar internal_units_variable]
        }

        if { [info command ::gid_groups_conds::entry_units::1rightarrow-16] eq "" } {
            image create photo ::gid_groups_conds::entry_units::1rightarrow-16 -data {
                R0lGODlhEAAQAIQAAPwCBDySRBR6JFy2ZLzixHTKfCyKPEyqXPT69IzelDyGRPz+/JzypHzShJTm
                nJzupDSORFyiZCSGNESmVGS+bEyWVDyaRAxuJCRyLARqFARaFAAAAAAAAAAAAAAAAAAAACH5BAEA
                AAAALAAAAAAQABAAAAU7ICCOZGmeKBCkpjCwpEAUsGgcSFIrQ7EwsMCg4Vg8WJBewhFhSSaUQgUm
                sEwUtYsEUwNkNF0AOEyu+UMAIf5oQ3JlYXRlZCBieSBCTVBUb0dJRiBQcm8gdmVyc2lvbiAyLjUN
                CqkgRGV2ZWxDb3IgMTk5NywxOTk4LiBBbGwgcmlnaHRzIHJlc2VydmVkLg0KaHR0cDovL3d3dy5k
                ZXZlbGNvci5jb20AOw==
            }
        }
        if { $options(-value_variable) ne "" || $options(-value_node) ne "" } {
            set has_entry 1
        } else {
            set has_entry 0
        }
        if { $options(-value_node) ne "" } {
            set unit_definition [get_domnode_attribute $options(-value_node) unit_definition]
            set unit_mesh_definition [get_domnode_attribute $options(-value_node) unit_mesh_definition]
            if { $unit_definition ne "" || $unit_mesh_definition == 1 } {
                set has_entry 0
            }
        }
        if { $has_entry } {
            if { [info command ::GiD_Process] eq "" } {
                set entry [ttk::entry $win.e -textvariable $options(-value_variable) -width 10]
            } else {
                set entry [ttk::entry $win.e -textvariable $options(-value_variable) -width 10 \
                        -style Padding0.TEntry]
            }           
            bindtags $entry [linsert [bindtags $entry] end $win]
        }
        if {[winfo exists $win.e]} {
            $win.e configure -state $options(-state_entry)
        }
               
        set combo [cu::combobox_tree $win.c -textvariable $options(-units_variable) \
                -width 6 -popup_width 170 -create_tree $options(-create_tree) -state $options(-state_combobox_tree)]        
        
        bindtags $win.c [linsert [bindtags $win.c] end $win]
        if {$options(-create_tree) eq "init"} {
            [$combo give_tree] state define on
        }
                
        if { $options(-units_width) } {
            $combo configure -width $options(-units_width)
        }
        if { [info exists entry] } {
            $win add $entry -weight 3
        }
        $win add $win.c -weight 1
        
        set units ""
        if { $options(-value_node) ne "" } {          
            if { $unit_definition ne "" } {
                set options(-unit_magnitude) [_units_resolve_links_magnitude $unit_definition]
                set units [gid_groups_conds::give_active_unit $options(-unit_magnitude)]
            } elseif { $unit_mesh_definition == 1 } {
                set options(-unit_magnitude) L
                set units [gid_groups_conds::give_mesh_unit]
            } else {
                if { $options(-value_variable) ne "" } {
                    upvar #0 $options(-value_variable) vv
                    set vv [get_domnode_attribute $options(-value_node) v]
                } else {
                    $entry delete 0 end
                    $entry insert end [get_domnode_attribute $options(-value_node) v]
                }
                set options(-unit_magnitude) [gid_groups_conds::_units_resolve_links_magnitude \
                        [get_domnode_attribute $options(-value_node) unit_magnitude]]
                
                set units [get_domnode_attribute $options(-value_node) units]
            }
        }
        if { $units eq "" && [$self give_magnitude] ne "" } {
            set units [gid_groups_conds::give_active_unit [$self give_magnitude]]
        }
        $combo delete 0 end
        $combo insert end [gid_groups_conds::units_to_nice_units $units]

            
        if { $options(-value_node) ne "" && [$options(-value_node) @units_state ""] eq "disabled" } {
            $combo state disabled
        } else {
            $combo state readonly
        }
        if {$options(-create_tree) eq "init"} {
            $self _setup_units_combo
        }

        bind $win <FocusIn> [mymethod check_focus FocusIn]
        if {$options(-create_tree) eq "aposteriori"} {
            bind $combo <ButtonPress-1> [mymethod call_cu_fulltktree]
            
            bind $combo <Shift-ButtonPress-1> [mymethod call_cu_fulltktree]
            bind $combo <Double-ButtonPress-1> [mymethod call_cu_fulltktree]
            bind $combo <Triple-ButtonPress-1> [mymethod call_cu_fulltktree]
        }
#         bind $win <<TraverseIn>> [mymethod check_focus TraverseIn]
#         bind $win <<TraverseOut>> [mymethod check_focus TraverseOut]
    }
    destructor {
        if { $options(-units_variable) ne "" } {
            upvar #0 $options(-units_variable) v2
            set cmd "[mymethod _changed_units_combo] ;#"
            trace remove variable v2 write $cmd
        }
        if { $options(-ov_variable) ne "" } {
            upvar #0 $options(-ov_variable) v2
            set cmd "[mymethod _setup_units_combo] ;#"
            trace remove variable v2 write $cmd
        }
        if { $options(-unit_magnitude_variable) ne "" } {
            upvar #0 $options(-unit_magnitude_variable) v2
            set cmd "[mymethod _setup_units_combo] ;#"
            trace remove variable v2 write $cmd
        }
    }
    onconfigure -units_width {value} {
        set options(-units_width) $value
        
        if { [info exists combo] } {
            $combo configure -width $value
        }
    }
    onconfigure -units_variable {value} {
        if { $options(-units_variable) ne "" } {
            upvar #0 $options(-units_variable) v2
            set cmd "[mymethod _changed_units_combo] ;#"
            trace remove variable v2 write $cmd
        }
        set options(-units_variable) $value
        
        if { [info exists combo] } {
            $combo configure -textvariable $options(-units_variable)
        }
        upvar #0 $options(-units_variable) v2
        set cmd "[mymethod _changed_units_combo] ;#"
        trace add variable v2 write $cmd
    }
    onconfigure -ov_variable {value} {
        if { $options(-ov_variable) ne "" } {
            upvar #0 $options(-ov_variable) v2
            set cmd "[mymethod _setup_units_combo] ;#"
            trace remove variable v2 write $cmd
        }
        set options(-ov_variable) $value
        
        if { $options(-ov_variable) ne "" } {
            upvar #0 $options(-ov_variable) v2
            set cmd "[mymethod _setup_units_combo] ;#"
            trace add variable v2 write $cmd
        }
    }
    onconfigure -unit_magnitude_variable {value} {
        if { $options(-unit_magnitude_variable) ne "" } {
            upvar #0 $options(-unit_magnitude_variable) v2
            set cmd "[mymethod _setup_units_combo] ;#"
            trace remove variable v2 write $cmd
        }
        set options(-unit_magnitude_variable) $value
        
        if { $options(-unit_magnitude_variable) ne "" } {
            upvar #0 $options(-unit_magnitude_variable) v2
            set cmd "[mymethod _setup_units_combo] ;#"
            trace add variable v2 write $cmd
        }
    }
    onconfigure -edit_mode {value} {
        set options(-edit_mode) $value
        if { $options(-edit_mode) } {
            $combo state !readonly
            focus $combo
        } else {
            $combo state readonly
        }
        $self update_menu
    }
    onconfigure -units_system {value} {
        set options(-units_system) $value
        if { $value eq "-" } { return }
        if { ![info exists combo] } { return }
                
        $self _setup_units_combo
    }
    onconfigure -unit_magnitude {value} {
        set options(-unit_magnitude) $value
        if { ![info exists combo] } { return }
                
        $self _setup_units_combo
    }
    method give_entry {} { return $entry }
    
    method give_combo {} { return $combo }
    
    method call_cu_fulltktree {} {       
        if {[winfo exists $combo.popdown.f]} {
            set popdown $combo.popdown.f
        } else {
            set popdown $combo.popdown
        }                
        set columns [list [list 10 "" left imagetext 1]]
        
        destroy $popdown.l
        set toctree [cu::_fulltktree $popdown.l \
                -selecthandler [list $combo press] \
                -columns $columns -expand 1 \
                -selectmode browse -showheader 0 -showlines 0  \
                -showbutton 0 -indent 0 -bd 0 \
                -have_vscrollbar 1 \
                -have_search_button automatic]                
        grid $toctree -sticky nsew 
        #bind $popdown.l.t <ButtonPress-1>    [list $combo button1 %W %x %y]
        #         bind $combo.popdown <ButtonPress-1>    [list $combo button1 %W %x %y]
        bind $combo.popdown <ButtonPress-1> [list $combo button1 %W %x %y]
        
        bind $popdown.l.t <KeyPress-Return>  [list $combo select_entry]
        bind $popdown.l.t <KeyPress-Escape>  [list $combo cancel]
        #         bind $popdown.l.t <KeyPress-Tab>     { ttk::combobox::LBTab [winfo parent %W] next }
        #         bind $popdown.l.t <<PrevWindow>>     { ttk::combobox::LBTab [winfo parent %W] prev }
        bind $popdown.l.t <Destroy>          { catch { ttk::combobox::LBCleanup [winfo parent %W] }}
        
        switch -- [tk windowingsystem] {
            win32  {
                bind $combo.popdown <FocusOut>  {
                    if { "%W" eq [winfo toplevel %W] } {
                        ttk::combobox::LBCancel [winfo parent %W]
                    }
                }
                #bind $popdown.l.t <FocusOut>  { ttk::combobox::LBCancel [winfo parent %W] }
            }
            aqua {
                bind $combo.popdown <Deactivate> { ttk::combobox::LBCancel [winfo parent %W] }
            }
        }
        bind $combo.popdown <Map> [list $combo begin_post %W]
        bind $combo.popdown <Unmap> [list $combo end_post %W]
        $toctree state define on
        
        set options(-toctree) $toctree
        $self _setup_units_combo         
    }
    
    method disable {} {
        if { $options(-value_variable) ne "" || $options(-value_node) ne "" } {
            $entry configure -state disable
        }
        $combo configure -state disable
    }
        
    method disable_units {} {
        $combo configure -state disable
    }
    
    method enable {} {
        if { $options(-value_variable) ne "" || $options(-value_node) ne "" } {
            $entry configure -state normal
        }
        $combo configure -state normal
    }
        
    method enable_units {} {
        $combo configure -state normal
    }
    
    method remove { option } {
        if {$option==1} {
            $entry delete 0 end
        } elseif {$option==2} {
            $combo set ""
        } else {
            $entry delete 0 end
            $combo set ""
        }
    }
    
    method check_focus { what } {
        if { [info exists entry] && [focus] ne $combo && [focus] ne $entry } {
            switch $what {
                FocusIn {
                    focus $entry
                    event generate $entry <<TraverseIn>>
                }
                default {
                    event generate $entry <<$what>>
                }
            }
        }
    }
    method traverse_in { what } {
        switch $what {
            entry {
                focus $entry
                event generate $entry <<TraverseIn>>
            }
            units {
                focus $combo
                event generate $combo <<TraverseIn>>
            }
        }
    }
    method get_text {} {
        if { [info exists entry] } {
            return [list [$entry get] [$combo get]]
        } else {
            return [$combo get]
        }
    }
    method _changed_units_combo {} {
        
        if { $options(-value_variable) ne "" } {
            upvar #0 $options(-value_variable) v1
        } elseif { [info exists entry] } {
            set v1 [$entry get]
        }
        upvar #0 $options(-units_variable) v2
        
        if { $v2 eq $units_old } { return }
        if { $units_old eq "" } {
            set units_old $v2
            return
        }
        set unit_from [gid_groups_conds::nice_units_to_units $units_old]
        set unit_to [gid_groups_conds::nice_units_to_units $v2]
        
        set magnitude [$self give_magnitude]
        if { ![gid_groups_conds::is_unit_correct $magnitude $unit_from] } { return }
        if { ![gid_groups_conds::is_unit_correct $magnitude $unit_to] } { return }
        
        if { $transform_unit_value } {
            set err [catch {
                    set v1 [gid_groups_conds::convert_unit_value \
                            $magnitude $v1 $unit_from $unit_to]
                }]
        }
        
        set ipos [lsearch -exact $units_list $unit_to]
        if { $ipos != 0 } {
            if { $ipos != -1 } {
                set units_list [lreplace $units_list $ipos $ipos]
            }
            set units_list [linsert $units_list 0 $unit_to]
            $self update_menu
        }
        if { $options(-value_variable) eq "" && [info exists entry] } {
            set state [$entry state !readonly]
            $entry delete 0 end
            $entry insert end $v1
            $entry state $state
        }
        set units_old $v2
    }
    method give_magnitude {} {
        if { $options(-unit_magnitude_variable) ne "" } {
            upvar #0  $options(-unit_magnitude_variable) umv
            if { [info exists umv] } {
                set options(-unit_magnitude) $umv
            }
        }
        if { [llength [split $options(-unit_magnitude) ,]] > 1 } {
            upvar #0 $options(-ov_variable) ov_variable
            if { ![info exists ov_variable] } { return "" }
            set ov $ov_variable
            return [dict get [split $options(-unit_magnitude) ,] \
                    $ov_variable]
        } else {
            return $options(-unit_magnitude)
        }
    }
    method _setup_units_combo {} {

        set unit_magnitude [$self give_magnitude]
        # if { $unit_magnitude eq "" } { return }
        
        set unit [gid_groups_conds::nice_units_to_units [$combo get]]
        
        set change_unit_to_active 1
        if { $options(-value_node) ne "" } {
            set n [$options(-value_node) selectNodes function]
            if { $n ne "" } {
                set change_unit_to_active 0
            }
            set unit_mesh_definition [get_domnode_attribute $options(-value_node) unit_mesh_definition]
            if { $unit_mesh_definition == 1 } {
                set change_unit_to_active 0
            }
        }
        
        if { $options(-value_node) ne "" } {
            set units_default [$options(-value_node) @units_default ""]
            if { $units_default ne "" && $units_default eq $unit } {
                set change_unit_to_active 0        
            }
        }
        
        if { $unit ne "" && $change_unit_to_active } {
            set units_system $options(-units_system)
#             if { [info exists entry] } {
#                 set value [string trim [$entry get]]
#                 if { [string is double -strict $value] && $value != 0.0 } {
#                     set units_system ""
#                 }
#             }
            set unit [gid_groups_conds::give_active_default_unit -for_units_system \
                    $units_system -check_this_unit $unit $unit_magnitude]
            set state [$combo state !readonly]
            $combo delete 0 end
            $combo insert end [gid_groups_conds::units_to_nice_units $unit]
            $combo state $state
        }
        if { $unit_magnitude ne "" } {
            set units_list [gid_groups_conds::give_units_list -for_units_system \
                    $options(-units_system) $unit_magnitude]
        } else {
            set units_list ""
        }
        $self update_menu
    }
    method check {} {
        upvar #0 $options(-units_variable) v2
        set unit [gid_groups_conds::nice_units_to_units $v2]
        set unit_magnitude [$self give_magnitude]
        if { ![gid_groups_conds::is_unit_correct $unit_magnitude $unit] } {
            set color red
            bell
        } else {
            set color green   
        }
        set fg [$combo cget -foreground]
        
        for { set i 0 } { $i < 3 } { incr i  } {
            $combo configure -foreground $color
            update
            after 300
            $combo configure -foreground $fg
            update
            after 100
        }
        
    }
    method toogle_transform_unit_value { id } {
        if { $transform_unit_value } {
            set transform_unit_value 0
            $combo tree_item state set $id !on
        } else {
            set transform_unit_value 1
            $combo tree_item state set $id on
        }
    }
    method insert { txt } {
        $combo insert insert $txt
    }
    
    method _write_value_unit { value unit } {
        $combo set $unit
        $entry delete 0 end
        $entry insert 0 $value
    }
    method insert_unit { unit } {
        if { [$combo index end] == 0 || [$combo index insert] < [$combo index end] } {
            $combo insert insert $unit
            return
        }
        set txt [string trim [$combo get]]
        set middot \u00b7
        if { [string index $txt end] ni [list $middot * /] } {
            append txt $middot
        }
        append txt $unit
        $combo delete 0 end
        $combo insert end $txt
        $combo icursor end
    }
    method update_menu { { toctree "" }} {       
        if { [$combo give_tree] eq "" && $options(-toctree) eq ""} { return }        
        if { $options(-toctree) ne "" } {
            $combo def_tree $options(-toctree) 
        }
        $combo clear
        
        if { [$self give_magnitude] eq "" } {
            $combo tree_insert -active 1 -image ::gid_groups_conds::entry_units::1rightarrow-16 end \
                [_ "No units"]  "" 0    
            return       
        }
        foreach u [gid_groups_conds::units_to_nice_unitsList $units_list] {
            $combo tree_insert -image ::gid_groups_conds::entry_units::1rightarrow-16 end $u $u 0
        }
        set cmd "[mymethod insert_unit_window];#"
        $combo tree_insert -image ::gid_groups_conds::entry_units::1rightarrow-16 \
            -command $cmd end [_ "More units"]... mu 0
        set id [$combo tree_insert -active 0 end [_ "Edit mode"] edm 0]
        $combo tree_item state set $id emphasis
        
        if { !$options(-edit_mode) } {
            $combo tree_insert -image ::gid_groups_conds::entry_units::1rightarrow-16 \
                -command "[mymethod configure -edit_mode 1];#" end [_ "Edit field"] ef $id
        } else {
            $combo tree_insert -image ::gid_groups_conds::entry_units::1rightarrow-16 \
                -command "[mymethod check];#" end [_ "Check"] che $id
            set id_i [$combo tree_insert -active 0 end [_ "Insert"] ins $id]
            $combo tree_item state set $id_i emphasis
            foreach i [list \u0b2 \u0b3 \u2074 \u00b7] {
                $combo tree_insert -image ::gid_groups_conds::entry_units::1rightarrow-16 \
                    -command "[mymethod insert $i];#" end "  $i  " ef $id_i
            }
        }
        $combo tree_item collapse $id
        
        if { [info exists entry] } {
            set id [$combo tree_insert -active 0 end [= Options] opts 0]
            $combo tree_item state set $id emphasis
            set cmd [mymethod toogle_transform_unit_value]
            set id_i [$combo tree_insert -image [gid_themes::GetImage cancel small_icons] -command $cmd end [_ "Transform unit value"] tuv $id]
            $combo tree_item state set $id_i on
            $combo tree_item collapse $id
        }
    }
    method insert_unit_window {} {
        after idle [mymethod insert_unit_window_do]
    }
    method insert_unit_window_do {} {
        set w $win.iuw
        destroy $w
        dialogwin_snit $w -title [_ "Select unit to insert"]
        set f [$w giveframe]
        
        lassign "" values dict
        foreach i [gid_groups_conds::give_unit_magnitudes_and_basic [$self give_magnitude]] {
            lappend values [lindex $i 0]
            dict set dict {*}$i
        }
        
        set f1 [ttk::labelframe $f.f1 -text [_ "insert units"]]
        
        ttk::label $f1.l1 -text [_ "Unit magnitude"]:
        cu::combobox $f1.cb1 -textvariable [$w give_uservar magnitude] \
            -values $values -dict $dict -state readonly
        
        $w add_traceN_to_uservar magnitude [mymethod insert_unit_change_magnitude $w]
                
        set columns [list \
                [list 12 [_ "Unit"] left text 1] \
                [list 20 [_ "Name"] left text 1] \
                [list 10 [_ "Factor"] left text 1] \
                [list 14 [_ "System"] left text 1] \
                ]

        package require fulltktree
        fulltktree $f1.tree -selecthandler2 \
            "[list $w invokeok];#" \
            -columns $columns -expand 1 \
            -selectmode browse -showlines 0 -indent 0 -width 355
        $w set_uservar_value tree $f1.tree
        
        ttk::entry $f1.e1 -textvariable [$w give_uservar search] -width 12
        
        $w add_traceN_to_uservar search [mymethod insert_unit_from_list_search $w]
        
        ttk::button $f1.b1 -image [gid_themes::GetImage cancel small_icons] -style Toolbutton \
            -command "[list $w set_uservar_value search ""] ; [list focus $f1.e1]"
        
        tooltip::tooltip $f1.e1 [_ "Search in list"]
        tooltip::tooltip $f1.b1 [_ "Clear search entry"]
        
        grid $f1.l1 $f1.cb1    -     -   -sticky ew -padx 2 -pady "2 4"
        grid $f1.tree  -      -     -   -sticky nsew -padx 2 
        grid   x     x    $f1.e1 $f1.b1 -sticky e -padx 2 -pady "1 1"

        grid columnconfigure $f1 1 -weight 1
        grid rowconfigure $f1 1 -weight 1
        
        grid $f1 -sticky nsew
        grid columnconfigure $f 0 -weight 1
        grid rowconfigure $f 0 -weight 1

        $w set_uservar_value magnitude [$self give_magnitude]
        tk::TabToWindow $f1.tree
        bind [winfo toplevel $f1] <Return> [list $w invokeok]
        set action [$w createwindow]
        
        while 1 {
            if { $action <= 0 } { 
                destroy $w
                return
            }
            set item [$f1.tree selection get]
            if { [llength $item] == 1 } {
                set unit [$f1.tree item text $item 0]
                
                if { [$w give_uservar_value magnitude] ne [$self give_magnitude] } {
                    if { !$options(-edit_mode) } {
                        $self configure -edit_mode 1
                    }
                    $self insert_unit $unit
                } else {
                    set state [$combo state !readonly]
                    $combo delete 0 end
                    $combo insert end $unit
                    $combo state $state
                }
                break
            }
            set action [$w waitforwindow]
        }
        destroy $w
    }
    method insert_unit_change_magnitude { w magnitude } {
        set tree [$w give_uservar_value tree]
        
        $tree item delete all
        set num 0
        if { [gid_groups_conds::give_unit_is_magnitude_basic $magnitude] } {
            foreach i [gid_groups_conds::give_nice_unit_all_units $magnitude] {
                $tree insert end [lrange $i 0 3]
                incr num
            }
        } else {
            set basic [gid_groups_conds::give_nice_unit_all_units $magnitude]
            foreach i $basic {
                $tree insert end [lrange $i 0 3]
                incr num
            }
            set basic [lsort -index 0 $basic]
            set ul [gid_groups_conds::give_units_list -p all $magnitude]
            foreach u [gid_groups_conds::units_to_nice_unitsList $ul] {
                if { [lsearch -sorted $basic $u] != -1 } { continue }
                $tree insert end [list $u]
                incr num
            }
        }
        if { $num } {
            $tree selection add 1
            $tree see 1
        }
        $w set_uservar_value search ""
        tk::TabToWindow $tree
    }
    variable insert_unit_from_list_search_id ""
    method insert_unit_from_list_search { w value } {
        after cancel $insert_unit_from_list_search_id
        set insert_unit_from_list_search_id [after 500 \
                [mymethod insert_unit_from_list_search_do $w $value]]
    }
    method insert_unit_from_list_search_do { w value } {
        set select_language_from_list_search_id ""
        
        if { ![winfo exists $w] } { return }

        set tree [$w give_uservar_value tree]
        set value [string trim [string tolower $value]]
        if { $value eq "" } {
            foreach item [$tree item children root] {
                $tree item configure $item -visible 1
            }
        } else {
            set values [regexp -inline -all {\S+} $value]
            foreach item [$tree item children root] {
                foreach value $values {
                    set found 0
                    foreach t [$tree item text $item] {
                        if { [string match *$value* [string tolower $t]] } {
                            set found 1
                            break
                        }
                    }
                    if { !$found } { break }
                }
                if { $found } {
                    $tree item configure $item -visible 1
                    if { $value eq [string tolower $t] } {
                        $tree selection clear all
                        $tree selection add $item
                        $tree see $item
                    }
                } else {
                    $tree item configure $item -visible 0
                }
            }
        }
    }
}

if 0 {
    interp alias "" = "" msgcat::mc
    source [file join [file dirname [info script]] gid_groups_conds.tcl]
    
    set file [file join [file dirname [info script]] units.xml]
    set gid_groups_conds::doc_units [p::xml parse -file $file]
#     set data [tDOM::xmlReadFile [file join [file dirname [info script]] units.xml]]
#     set gid_groups_conds::doc_units [dom parse $data]
    set gid_groups_conds::doc $gid_groups_conds::doc_units

    #console show
    
    set values [list \
            Temp 32 oF oC \
            Temp 100 oC oF \
            P 1.0 N/mm^2 MPa \
            F/L^2 1.0 N/mm^2 MPa \
            L 1.0 mm km \
            L 1.0 km mm \
            ]
    
    foreach "m v u1 u2" $values {
        puts "convert_unit_value $m $v $u1 $u2=[gid_groups_conds::convert_unit_value $m $v $u1 $u2]"
    }
    
    set values [list L P]
    foreach m $values {
        puts "give_units_list $m=[gid_groups_conds::give_units_list $m]"
    }
    
    mylog::init -view_binding <Control-l> debug
    pack [gid_groups_conds::entry_units .e -unit_magnitude P -value_variable value \
            -units_variable units]
    lassign [list 23.0 Pa] value units
    
    pack [ttk::entry .e2]
    pack [ttk::combobox .e3]
    #exit
}
}





