
package require math::linearalgebra
package require math::constants

package provide customLib_utils::math 1.6

namespace eval ::math::linearalgebra { namespace export transpose }
namespace eval m {
    namespace import -force ::math::linearalgebra::*
    ::math::constants::constants pi degtorad radtodeg onethird
    
    proc matrix_to_row_major_order { mat } {
	set ret ""
	foreach i $mat {
	    lappend ret {*}$i
	}
	return $ret
    }
    proc vec_angle { vect1 vect2 } {
	set dp [dotproduct $vect1 $vect2]
	set n1 [norm $vect1]
	set n2 [norm $vect2]
	
	if { $n1 == 0.0 || $n2 == 0.0 } {
	    return -code error "Angle not defined for null vector"
	}
	set dp1 [expr {$dp/$n1/$n2}]
	if { $dp1 > 1.0 } { set dp1 1.0 }
	if { $dp1 < -1.0 } { set dp1 -1.0 }
	return [expr {acos($dp1)}]
    }
    # between -PI and PI. Negative if (v1 x v2) has opposite sense to normal
    proc vec_angle12 { vect1 vect2 normal } {
	set dp [dotproduct $vect1 $vect2]
	set n1 [norm $vect1]
	set n2 [norm $vect2]
	
	if { $n1 == 0.0 || $n2 == 0.0 } {
	    return -code error "Angle not defined for null vector"
	}
	set vect1 [m::scale [expr {1.0/$n1}] $vect1]
	set vect2 [m::scale [expr {1.0/$n2}] $vect2]
	set dp1 [expr {$dp/$n1/$n2}]
	if { $dp1 > 1.0 } { set dp1 1.0 }
	if { $dp1 < -1.0 } { set dp1 -1.0 }
	set angle [expr {acos($dp1)}]
	set normalV [vectorprod3 $vect1 $vect2]
	if { [dotproduct $normal $normalV] < 0.0 } {
	    set angle [expr {-1.0*$angle}]
	}
	return $angle
    }
    # between 0 and 2*PI
    proc vec_angle12_2PI { vect1 vect2 normal } {
	variable pi
	
	set dp [dotproduct $vect1 $vect2]
	set n1 [norm $vect1]
	set n2 [norm $vect2]
	
	if { $n1 == 0.0 || $n2 == 0.0 } {
	    return -code error "Angle not defined for null vector"
	}
	set dp1 [expr {$dp/$n1/$n2}]
	if { $dp1 > 1.0 } { set dp1 1.0 }
	if { $dp1 < -1.0 } { set dp1 -1.0 }
	set angle [expr {acos($dp1)}]
	set normalV [vectorprod3 $vect1 $vect2]
	if { [dotproduct $normal $normalV] < 0.0 } {
	    set angle [expr {2.0*$pi-$angle}]
	}
	return $angle
    }
    proc vectorprod3 { vec1 vec2 } {
	return [list [expr {[lindex $vec1 1]*[lindex $vec2 2]-[lindex $vec2 1]*[lindex $vec1 2]}] \
		[expr {[lindex $vec1 2]*[lindex $vec2 0]-[lindex $vec2 2]*[lindex $vec1 0]}] \
		[expr {[lindex $vec1 0]*[lindex $vec2 1]-[lindex $vec2 0]*[lindex $vec1 1]}]]
    }
    proc nice3 { v } {
	return [nice $v 3]
    }
    proc nice { v precision } {
	set ret ""
	foreach i $v {
	    if { $precision == 3 && abs($i) >= 1000 && abs($i) < 10000 } {
		lappend ret [format %.4g $i]
	    } else {
		lappend ret [format %.${precision}g $i]
	    }
	}
	return $ret
    }
    proc vector_normal_triangle { p1 p2 p3 } {
	
	set d1 [m::sub $p2 $p1]
	set d2 [m::sub $p3 $p2]
	return [unitLengthVector [vectorprod3 $d1 $d2]]
    }
    proc triangle_quad_center { args } {
	if { [llength $args] ni "2 3 4" } {
	    error "error: triangle_quad_center"
	}
	set sum [list 0.0 0.0 0.0]
	foreach i $args {
	    set sum [add $sum $i]
	}
	return [scale [expr {1.0/[llength $args]}] $sum]
    }
}
