
package require snit

namespace eval cu {
    variable topdir [file normalize [file dirname [info script]]]
}
namespace eval cu::file {}

# for tclIndex to work 
proc cu::combobox_tree { args } {}
proc cu::menubutton_button { args } {}
proc cu::notebook { args } {}
proc cu::nicelabel { args } {}
proc cu::check_listbox { args } {}

interp alias "" nicelabel "" cu::nicelabel

snit::widgetadaptor cu::menubutton_button {
    option -command ""
    option -image ""
    option -text ""
    option -image_delta ""
    option -display_active 0
    
    delegate method * to hull
    delegate option * to hull
    delegate option -_image to hull as -image
    delegate option -_text to hull as -text

    variable xmin
    variable is_button_active 1 ; # 0, 1, only_button
    variable press_after ""
    variable new_image ""
    
    constructor args {
        installhull using ttk::menubutton -style Toolbutton
        bind $win <ButtonPress-1> [mymethod BP1 %x %y]
        bind $win <ButtonRelease-1> [mymethod BR1 %x %y]
        bind $win <Down> [list ttk::menubutton::Popdown %W]
        bind $win <Motion> [mymethod check_cursor %x %y]
        bind $win <Configure> [mymethod  _calc_xmin]
        
        foreach i [list <ButtonPress-1> <ButtonRelease-1> <B1-Leave>] {
            regsub {1} $i {3} i3
            bind $win $i3 [bind TMenubutton $i]
        }
        $self configurelist $args
    }
    destructor {
#         if { $new_image ne "" } {
#             catch { image delete $new_image }
#         }
    }
    onconfigure -display_active { value } {
        set options(-display_active) $value
        if { 0&& $options(-display_active) } {
            if { $::tcl_platform(platform) eq "windows" } {    
                $win state active
                bind $win <Leave> "break"
            }
        } else {
            if { $::tcl_platform(platform) eq "windows" } {    
                $win state !active
                bind $win <Leave> ""
            }
        }
    }
    onconfigure -image {img} {
        set options(-image) $img

#         if { $new_image ne "" } {
#             catch { image delete $new_image }
#             set new_image ""
#         }
        if { $options(-text) ne "" } {
            $self configure -_image $img
            return
        }
        
        set new_image [cu::add_down_arrow_to_image -delta $options(-image_delta) $img]
        $self configure -_image $new_image
    }
    onconfigure -image_delta {value} {
        set options(-image_delta) $value

#         if { $new_image ne "" } {
#             image delete $new_image
#             set new_image ""
#         }
        if { $options(-image) ne "" } {
            set new_image [cu::add_down_arrow_to_image -delta $options(-image_delta) $options(-image)]
            $self configure -_image $new_image
        }
    }
    onconfigure -text {value} {
        set options(-text) $value

        if { $options(-text) ne "" } {
            $self configure -style ""
            if { $options(-image) ne "" } {
                $self configure -_image $options(-image)
            }
        }
        $self configure -_text $value
    }
    method _calc_xmin {} {
        if { [winfo width $win] > 1 } {
            set xmin  [expr {[winfo width $win]-12}]
        } else {
            set xmin  [expr {[winfo reqwidth $win]-12}]
        }
    }
    method give_is_button_active_var {} {
        return [myvar is_button_active]
    }
    method set_is_button_active { value } {
        set is_button_active $value
    }
    method BP1 { x y } {
        if { $is_button_active == 0 } { return }
        
        if { ($x < $xmin || $is_button_active eq "only_button") && $options(-command) ne "" } {
            $win instate !disabled {
                catch { tile::clickToFocus $win }
                catch { ttk::clickToFocus $win }
                $win state pressed
            }
            set press_after [after 700 [mymethod BP1_after]]
            return -code break
        }
    }
    method BP1_after {} {
        set press_after ""
        $win instate {pressed !disabled} {
            $win state !pressed
            ttk::menubutton::Pulldown $self
        }
    }
    method BR1 { x y } {       
        if { $is_button_active == 0 } { return }
        
        cu::tooltip cancel $win
        
        if { $press_after ne "" } {
            after cancel $press_after
        }
        if { $press_after ne "" && ($x < $xmin || $is_button_active eq "only_button") && 
            $options(-command) ne "" } {
            set press_after ""
            $win instate {pressed !disabled} {
                $win state !pressed
                uplevel #0 $options(-command)
            }            
            return -code break
        }       
        set press_after ""        
    }
    method check_cursor { x y } {
        if { $x < $xmin || $is_button_active eq "only_button" } {
            $win configure -cursor ""
        } else {
            $win configure -cursor bottom_side
        }
    }
}

snit::widgetadaptor cu::combobox {
    option -valuesvariable ""
    option -textvariable ""
    option -statevariable ""
    option -values ""
    option -dict ""
    option -dictvariable ""

    variable _translated_textvariable ""

    delegate method * to hull
    delegate option * to hull
    delegate option -_values to hull as -values
    delegate option -_textvariable to hull as -textvariable

    constructor args {
        
        if { [info command ::GiD_Process] eq "" } {
            installhull using ttk::combobox 
        } else {
            installhull using ttk::combobox -style Padding0.TCombobox
        }       

        cu::add_contextual_menu_to_entry $win init
        bind $win <<ComboboxSelected>> [mymethod combobox_selected]
        $self configurelist $args
    }
    destructor {
        catch {
            if { $options(-valuesvariable) ne "" } {
                upvar #0 $options(-valuesvariable) v
                trace remove variable v write "[mymethod _changed_values_var];#"
            }
            if { $options(-dictvariable) ne "" } {
                upvar #0 $options(-dictvariable) v
                trace remove variable v write "[mymethod _changed_values_var];#"
            }
            if { $options(-textvariable) ne "" } {
                upvar #0 $options(-textvariable) v
                trace remove variable v write "[mymethod _written_textvariable];#"
            }
            if { $options(-statevariable) ne "" } {
                upvar #0 $options(-statevariable) v
                trace remove variable v write "[mymethod _written_statevariable];#"
                trace remove variable v read "[mymethod _read_statevariable];#"
            }
        }
    }
    onconfigure -textvariable {value} {
        set options(-textvariable) $value
        $self configure -_textvariable [myvar _translated_textvariable]

        upvar #0 $options(-textvariable) v
        trace add variable v write "[mymethod _written_textvariable];#"
        trace add variable [myvar _translated_textvariable] write \
            "[mymethod _read_textvariable];#"
        if { [info exists v] } {
            $self _written_textvariable
        }
    }
    onconfigure -dictvariable {value} {
        set options(-dictvariable) $value
        $self _changed_values_var
        upvar #0 $options(-dictvariable) v
        trace add variable v write "[mymethod _changed_values_var];#"
    }
    onconfigure -statevariable {value} {
        set options(-statevariable) $value

        upvar #0 $options(-statevariable) v
        trace add variable v write "[mymethod _written_statevariable];#"
        trace add variable v read "[mymethod _read_statevariable];#"
        if { [info exists v] } {
            set v $v
        }
    }
    onconfigure -valuesvariable {value} {
        set options(-valuesvariable) $value

        upvar #0 $options(-valuesvariable) v

        if { $options(-dictvariable) ne "" } {
            upvar #0 $options(-dictvariable) vd
            if { [info exists vd] } {
                set dict $vd
            } else {
                set dict ""
            }
        } else {
            set dict $options(-dict)
        }
        if { ![info exists v] } {
            set v ""
            foreach value [$self cget -_values] {
                catch { 
                    set value [dict get [dict_inverse $dict] $value]
                }
                lappend v $value
            }
        } else {
            set vtrans ""
            foreach value $v {
                catch { set value [dict get $dict $value] }
                lappend vtrans $value
            }
            $self configure -_values $vtrans
        }
        trace add variable v write "[mymethod _changed_values_var];#"
    }
    onconfigure -dict {value} {
        set options(-dict) $value
        $self _changed_values_var
    }
    onconfigure -values {values} {
        if { $options(-valuesvariable) ne "" } {
            upvar #0 $options(-valuesvariable) v
            set v $values
        } else {
            if { $options(-dictvariable) ne "" } {
                upvar #0 $options(-dictvariable) vd
                if { [info exists vd] } {
                    set dict $vd
                } else {
                    set dict ""
                }
            } else {
                set dict $options(-dict)
            }
            set vtrans ""
            foreach value $values {
                catch { set value [dict get $dict $value] }
                lappend vtrans $value
            }
            $self configure -_values $vtrans
        }
    }
    oncget -values {
        set v ""
        foreach value [$self cget -_values] {
#             catch {
#                 set value [dict get [dict_inverse $options(-dict)] $value]
#             }
            lappend v $value
        }
        return $v
    }
    method _changed_values_var {} {
        if { $options(-valuesvariable) ne "" } {
            upvar #0 $options(-valuesvariable) v
        } else {
            set v [$self cget -values]
        }
        if { $options(-dictvariable) ne "" } {
            upvar #0 $options(-dictvariable) vd
            if { [info exists vd] } {
                set dict $vd
            } else {
                set dict ""
            }
        } else {
            set dict $options(-dict)
        }
        set vtrans ""
        foreach value $v {
            catch { set value [dict get $dict $value] }
            lappend vtrans $value
        }
        $self configure -_values $vtrans
        $self _written_textvariable
    }
    method _written_textvariable { args } {

        set optional {
            { -force_dict "" 0 }
        }
        set compulsory ""
        parse_args $optional $compulsory $args

        upvar #0 $options(-textvariable) v
        if { ![info exists v] } { return }
        set value $v
        if { $options(-dictvariable) ne "" } {
            upvar #0 $options(-dictvariable) vd
            if { [info exists vd] } {
                set dict $vd
            } else {
                set dict ""
            }
        } else {
            set dict $options(-dict)
        }
        if { $force_dict || [$self instate readonly] } {
            catch { set value [dict get $dict $value] }
        }
        if { $_translated_textvariable ne $value } {
            set _translated_textvariable $value
        }
    }
    method _read_textvariable {} {
        upvar #0 $options(-textvariable) v
        set value $_translated_textvariable
        if { $options(-dictvariable) ne "" } {
            upvar #0 $options(-dictvariable) vd
            if { [info exists vd] } {
                set dict $vd
            } else {
                set dict ""
            }
        } else {
            set dict $options(-dict)
        }
        catch {
            set value [dict get [dict_inverse $dict] $value]
        }
        if { ![info exists v] || $v ne $value } {
            set v $value
        }
    }
    method _written_statevariable {} {
        upvar #0 $options(-statevariable) v
        $self state $v
    }
    method _read_statevariable {} {
        upvar #0 $options(-statevariable) v
        set v [$self state]
    }
    method combobox_selected {} {
        if { ![$self instate readonly] } {
            $self _written_textvariable -force_dict
        }
    }
}

################################################################################
#    check_listbox
################################################################################

snit::widgetadaptor cu::check_listbox {
    option -values
    option -add_new_button 0
    option -permit_rename 0
    option -permit_delete 0

    delegate method * to hull
    delegate option * to hull

    variable tree
    variable is_built
    variable itemDict ""
    variable exclusiveDict ""
    variable prev_selection ""
    variable add_item ""
    
    constructor args {
        
        package require fulltktree
        
        set is_built 0
        
        set columns [list \
                [list 25 Groups left imagetext 1] \
                ]
        
        installhull using ::fulltktree -width 130 -height 150 \
            -columns $columns -expand 1 \
            -selectmode extended -showlines 1 -showrootlines 0 -indent 1 -showbutton 1 \
            -showheader 0 -sensitive_cols all -buttonpress_open_close 0 \
            -have_vscrollbar 1 \
            -contextualhandler_menu [mymethod contextual_menu]  \
            -editbeginhandler [mymethod edit_name_begin] \
            -editaccepthandler [mymethod edit_name_accept] \
            -selecthandler [mymethod _selection]
        set tree $self
        
        $tree state define check
        $tree state define on
        
        set is_built 1
        $self configurelist $args
    }
    onconfigure -values { values } {
        set options(-values) $values
        
        if { !$is_built } { return }
        $self _update_from_values
    }
    onconfigure -add_new_button { value } {
        set options(-add_new_button) $value
        
        if { !$is_built } { return }
        $self _update_from_values
    }
    oncget -values {
        foreach i [range 0 [llength $options(-values)]] {
            set elm [lindex $options(-values) $i]
            foreach j [range [llength $elm] 4] { lappend elm "" }
            lassign $elm name state parentName open_close
            set item [dict get $itemDict values_idx $i]

            if { [$tree item state get $item on] } {
                lset elm 1 on
            } else {
                lset elm 1 off
            }
            if { $parentName ne "" } {
                if { [$tree item state get $item open] } {
                    lset elm 3 open
                } else {
                    lset elm 3 close
                }
            }
            lset options(-values) $i $elm
        }
        return $options(-values)
    }
    method _update_from_values {} {
        
        $tree item delete all
        set itemDict ""
        set exclusiveDict ""
        set values_idx 0
        foreach value $options(-values)  {
            $self _insert end $value $values_idx
            incr values_idx
        }
        if { $options(-add_new_button) } {
            $self _create_add_button
        }
    }
    method contextual_menu { - menu item selection } {
        
        $menu add command -label [_ "Activate"] -command [mymethod set_state $selection on]
        $menu add command -label [_ "Dectivate"] -command [mymethod set_state $selection !on]
        
        set children [$tree item children [$tree item parent $item]]
        set ipos [lsearch -exact $children $add_item]
        if { $ipos != -1 } {
            set children [lreplace $children $ipos $ipos]
        }
        if { [lsearch -exact $children $item] > 0 } {
            $menu add command -label [_ "Move up"] -command [mymethod move_item $item up]
        }
        if { [lsearch -exact $children $item] < [llength $children]-1 } {
            $menu add command -label [_ "Move down"] -command [mymethod move_item $item down]
        }
        $menu add separator
        
        if { $options(-permit_rename) } {
            $menu add command -label [_ "Rename"] -command [mymethod rename [lindex $selection 0]]
        }
        if { $options(-add_new_button) } {
            $menu add command -label [_ "Add"] -command [mymethod add_new_item]
        }
        if { $options(-permit_delete) } {
            $menu add separator
            $menu add command -label [_ "Delete"] -command [mymethod delete_items $selection]
        }
    }
    method rename { { item "" } } {
        if { $item ne "" } {
            $tree selection clear
            $tree selection add $item
            $tree activate $item
            $tree see $item
        }
        focus [$tree givetreectrl]
        update
        event generate $tree <F2>
    }
    method _create_add_button {} {
        set b [$tree givetreectrl].message
        destroy $b
        button $b -text [_ "Add"] \
            -foreground blue -background white -bd 0 -cursor hand2 \
            -command [mymethod add_new_item]
        set font [concat [font actual [$b cget -font]] -underline 1]
        $b configure -font $font
        
        set add_item [$tree insert end ""]
        $tree item style set $add_item 0 window
        $tree item element configure $add_item 0 e_window -destroy 1 \
            -window $b
    }
    method add_new_item {} {
        
        foreach item [$tree item children 0] {
            if { [$tree item style set $item 0] eq "window" } {
                $tree item delete $item
                break
            }
        }
        set idx 1
        while { [lsearch -index 0 $options(-values) [_ "Unnamed%d" $idx]] != -1 } {
            incr idx
        }
        set value [list [_ "Unnamed%d" $idx] on "" open]
        set values_idx [llength $options(-values)]
        lappend options(-values) $value
        set item [$self _insert end $value $values_idx]
        if { $options(-add_new_button) } {
            $self _create_add_button
        }
        $tree selection clear
        $tree selection add $item
        $tree activate $item
        $tree see $item
        #focus $tree
        focus [$tree givetreectrl]
        update
        event generate $tree <F2>
    }
    method edit_name_begin { args } {
        if { $options(-permit_rename) == 0 } {
            return 0
        }
        return 1
    }
    method edit_name_accept { args } {

        lassign $args - item col text
        
        if { ![regexp {[-\w.]{2,}} $text] } {
            tk_messageBox -message [_ "Name can only contain letters, digits, -_.'"] -parent $win
            return
        }
        $tree item text $item 0 $text
        set values_idx [dict get $itemDict item $item]
        lset options(-values) $values_idx 0 $text
    }
    method move_item { item up_down } {
        
        set values_idx [dict get $itemDict item $item]
        
        switch $up_down {
            up {
                set sibling [$tree item prevsibling $item]
                set values_idx_sibling [dict get $itemDict item $sibling]

                set v [lindex $options(-values) $values_idx]
                lset options(-values) $values_idx [lindex $options(-values) $values_idx_sibling]
                lset options(-values) $values_idx_sibling $v
                $tree item prevsibling $sibling $item
            }
            down {
                set sibling [$tree item nextsibling $item]
                set values_idx_sibling [dict get $itemDict item $sibling]

                set v [lindex $options(-values) $values_idx]
                lset options(-values) $values_idx [lindex $options(-values) $values_idx_sibling]
                lset options(-values) $values_idx_sibling $v
                $tree item nextsibling $sibling $item
            }
        }
    }
    method delete_items { itemList } {
        
        for { set il 0 } { $il < [llength $itemList] } { incr il } {
            set item [lindex $itemList $il]
            if { [$tree item id $item] eq "" } { continue }
            
            set values_idx [dict get $itemDict item $item]
            set options(-values) [lreplace $options(-values) $values_idx $values_idx]
            while 1 {
                set values_idx_next [expr {$values_idx+1}]
                if { ![dict exists $itemDict values_idx $values_idx_next] } {
                    dict unset itemDict values_idx $values_idx_next
                    break
                }
                set item [dict get $itemDict values_idx $values_idx_next]
                dict set itemDict values_idx $values_idx $item
                dict set itemDict item $item $values_idx
                set values_idx $values_idx_next
            }
            lappend itemList {*}[$tree item children $item]
            $tree item delete $item
        }
    }
    method get_selected_comma_list { args } {
        set optional {
            { -varname name "" }
        }
        set compulsory ""
        parse_args $optional $compulsory $args

        set ret ""
        set map [list "," "&#44;"]
        foreach elm [$self cget -values] {
            if { [lindex $elm 1] eq "on" } {
                lappend ret [string map $map [lindex $elm 0]]
            }
        }
        if { $varname ne "" } {
            uplevel 1 [list set $varname [join $ret ","]]
        } else {
            return [join $ret ","]
        }
    }
    method set_selected_comma_list { args } {
        set optional {
            { -varname name "" }
            { -insert "" 0 }
        }
        set compulsory "data"
        set args [parse_args -raise_compulsory_error 0 \
                $optional $compulsory $args]
        
        if { $varname ne "" } {
            set data [uplevel 1 [list set $varname]]
        }
        set map [list "&#44;" "," "&comma;" ","]
        set list ""
        foreach i [split $data ","] {
            lappend list [string map $map $i]
        }
        set list [lsort $list]
        
        foreach i [range 0 [llength $options(-values)]] {
            set elm [lindex $options(-values) $i]
            lassign $elm name state parentName open_close
            
            set item [dict get $itemDict values_idx $i]

            if { [llength [lindex $options(-values) $i]] < 2 } {
                set v [list [lindex $options(-values) $i 0] on]
                lset options(-values) $i $v
            }
            if { [lsearch -sorted $list $name] != -1 } {
                $tree item state set $item on
                lset options(-values) $i 1 on
            } else {
                $tree item state set $item !on
                lset options(-values) $i 1 off
            }
        }
        if { $insert } {
            set changes 0
            set values $options(-values)
            foreach i $list {
                if { [lsearch -index 0 $values $i] == -1 } {
                    lappend values [list $i on "" open]
                    set changes 1
                }
            }
            if { $changes } {
                $self configure -values $values
            }
        }
    }
    method _insert { index list values_idx } {

        set imgState [list \
                [cu::get_image_selected internet-check-on] {selected check on} \
                [cu::get_image_selected internet-check-off] {selected check} \
                [cu::get_image internet-check-on] {check on} \
                [cu::get_image internet-check-off] {check} \
                ]
        
        lassign $list name state parentName open_close
        
        if { $parentName eq "" } {
            set parent 0
        } else {
            set parent [dict get $itemDict name $parentName]
        }
        set newlist [list [list ::cu::images::internet-check-off $name]]
        set item [$tree insert $index $newlist $parent]
        
        dict set itemDict name $name $item
        dict set itemDict item $item $values_idx
        dict set itemDict values_idx $values_idx $item

        $tree item state set $item check
        if { $state eq "on" } {
            $tree item state set $item on
        }
        switch $open_close {
            open { $tree item expand $item }
            close { $tree item collapse $item }
            disabled { $tree item enabled $item 0 }
        }
        $tree item element configure $item 0 e_image -image $imgState
        return $item
    }
    method _recursive_change_state { item state } {
        $tree item state set $item $state
        foreach child [$tree item children $item] {
            $self _recursive_change_state $child $state
        }
    }
    method add_exclusive_item { name } {
        
        set item [dict get $itemDict name $name]
        dict set exclusiveDict $item [$tree item state get $item on]
    }
    method set_state { itemList state } {
        set sel [lsort -integer $itemList]
        foreach item $sel {
            $self _recursive_change_state $item $state
        }
    }
    method _selection { _tree ids } {
        
        set sel [lsort -integer $ids]
        
        foreach item [dict keys $exclusiveDict] {
            dict set exclusiveDict $item [$tree item state get $item on]
        }
        set num_activated 0
    
        if { $sel eq $prev_selection } {
            set item active
            if { [$tree item state get $item on] } {
                $self _recursive_change_state $item !on
            } else {
                $self _recursive_change_state $item on
                incr num_activated
            }
        } else {
            foreach item $sel {
                if { [lsearch -integer -sorted $prev_selection $item] == -1 } {
                    if { [$tree item state get $item on] } {
                        $self _recursive_change_state $item !on
                    } else {
                        $self _recursive_change_state $item on
                        incr num_activated
                    }
                }
            }
        }
        set prev_selection $sel
        
        foreach item [dict keys $exclusiveDict] {
            set new_state [$tree item state get $item on]
            if { $new_state && ![dict get $exclusiveDict $item] } {
                foreach child [$tree item children 0] {
                    $self _recursive_change_state $child !on
                }
                foreach item_in [dict keys $exclusiveDict] {
                    dict set exclusiveDict $item_in 0
                }
                $tree item state set $item on
                dict set exclusiveDict $item 1
                return
            }
        }
        if { $num_activated } {
            foreach item [dict keys $exclusiveDict] {
                $tree item state set $item !on
                dict set exclusiveDict $item 0
            }
        }
    }
}

################################################################################
# cu::multiline_entry
################################################################################

snit::widget cu::multiline_entry {
    option -textvariable ""
    option -takefocus 0 ;# option used by the tab standard bindings
    option -values ""
    option -valuesvariable ""
    option -state ""
    option -justify ""
    option -height ""

    hulltype frame

    variable text
    variable updating 0
    
    variable toctree
    variable fnames ""
    variable no_active_items ""
    variable no_active_values ""
    variable cmd_items ""
    
    delegate method * to text
    delegate method _insert to text as insert
    delegate option * to text
    delegate option -_state to text as -state
    delegate option -_height to text as -height
    
    delegate method tree_item to toctree as item
    delegate option -columns_list to toctree
    
    constructor args {

        $hull configure -background #a4b97f -bd 0
        install text using text $win.t -wrap word -width 40 -height 3 -borderwidth 0 -highlightthickness 0
        
        cu::add_contextual_menu_to_entry $text init

        grid $text -padx 1 -pady 1 -sticky nsew
        grid columnconfigure $win 0 -weight 1
        grid rowconfigure $win 0 -weight 1

        bind $self <Configure> [mymethod _check_configure]
        bind $text <Tab> "[bind all <Tab>] ; break"
        bind $text <<PrevWindow>> "[bind all <<PrevWindow>>] ; break"
        bind $text <KeyPress> [mymethod keypress]
        bindtags $text [list $win $text [winfo class $win] [winfo class $text] [winfo toplevel $text] all]
        bind $win <FocusIn> [list focus $text]
        $self configurelist $args
    }
    destructor {
        $self _clean_traces
    }
    onconfigure -state {value} {
        set options(-state) $value
        $self _update_state
    }
    onconfigure -height {value} {
        set options(-height) $value
        
        $self configure -_height [lindex $value 0]
    }
    onconfigure -textvariable {value} {
        $self _clean_traces
        set options(-textvariable) $value

        set cmd "[mymethod _check_textvariable_read] ;#"
        trace add variable $options(-textvariable) read $cmd
        set cmd "[mymethod _check_textvariable_write] ;#"
        trace add variable $options(-textvariable) write $cmd
        $self _check_textvariable_write
    }
    onconfigure -values {value} {
        set options(-values) $value
        
        if { $options(-values) ne "" || $options(-valuesvariable) ne "" } {
            $self activate_menubutton

            $self tree_item delete all
            $self tree_insert end [_ "(Clear)"] "" 0
            foreach i $value {
                $self tree_insert end $i $i 0
            }
        } elseif { ![winfo exists $win.b] } {
            destroy $win.b
        }
    }
    onconfigure -valuesvariable {value} {
        set options(-valuesvariable) $value

        
        if { $options(-values) ne "" || $options(-valuesvariable) ne "" } {
            $self activate_menubutton
        }
        upvar #0 $options(-valuesvariable) v

        if { [info exists v] } {
            $self _changed_values_var
            trace add variable v write "[mymethod _changed_values_var];#"
        } elseif { ![winfo exists $win.b] } {
            destroy $win.b
        }
    }
    method activate_menubutton {} {

        if { [winfo exists $win.b] } { return }

        if { [info command ::cu::multiline_entry::nav1downarrow16] eq "" } {
            image create photo ::cu::multiline_entry::nav1downarrow16 -data {
                R0lGODlhEAAQAIAAAPwCBAQCBCH5BAEAAAAALAAAAAAQABAAAAIYhI+py+0PUZi0zmTtypflV0Vd
                RJbm6fgFACH+aENyZWF0ZWQgYnkgQk1QVG9HSUYgUHJvIHZlcnNpb24gMi41DQqpIERldmVsQ29y
                IDE5OTcsMTk5OC4gQWxsIHJpZ2h0cyByZXNlcnZlZC4NCmh0dHA6Ly93d3cuZGV2ZWxjb3IuY29t
                ADs=
            }
        }
        ttk::menubutton $win.b -image ::cu::multiline_entry::nav1downarrow16 -style Toolbutton
        
        bind $win.b <ButtonPress-1> [mymethod BP1 %x %y]
        bind $win.b <ButtonRelease-1> [mymethod BR1 %x %y]
        
        set toctree $win.b.m
        cu::_menubutton_tree_helper $toctree -parent $win \
            -button1handler [mymethod press] -returnhandler [mymethod press_return]
        wm withdraw $toctree
        
        bind  $win.b.m <space> [mymethod post]
        bind  $win.b.m <Return> [mymethod post]
        
        bind $toctree <<ComboboxSelected>> [mymethod endpost]
        
        grid $win.b -row 0 -column 1 -padx "0 1" -pady 1 -sticky wns
    }
    method give_win {} {
        return $text
    }
    method give_tree {} {
        return $toctree
    }
    method set_text { txt } {
        focus $text
        update idletasks
        $text delete 1.0 end
        $text insert end $txt
        $text tag add sel 1.0 end-1c
        $self _check_configure
        $self _update_state
    }
    method insert { args } {
        $self _insert {*}$args
        $self _check_configure
        $self _update_state
    }
    method keypress {} {
        after idle [mymethod _check_configure]
        after idle [mymethod _update_state]
    }
    method _update_state {} {
        
        switch $options(-state) {
            disabled {
                $self configure -_state disabled -foreground grey -highlightthickness 0
                grid configure $text -padx 1 -pady 1
            }
            readonly {
                $self configure -_state disabled -foreground black -highlightthickness 0
                grid  configure $text -padx 0 -pady 0
            }
            default {
                $self configure -_state normal -foreground black -highlightthickness 1
                grid  configure $text -padx 1 -pady 1
            }
        }
        if { $options(-justify) ne "" } {
            $self tag configure justify -justify $options(-justify)
            $self tag add justify 1.0 end
        }
    }
    method _clean_traces {} {
        if { $options(-textvariable) ne "" } {
            set cmd "[mymethod _check_textvariable_read] ;#"
            trace remove variable $options(-textvariable) read $cmd
            set cmd "[mymethod _check_textvariable_write] ;#"
            trace remove variable $options(-textvariable) write $cmd
        }
        if { $options(-valuesvariable) ne "" } {
            upvar #0 $options(-valuesvariable) v
            trace remove variable v write "[mymethod _changed_values_var];#"
        }
    }
    method BP1 { x y } {
       $self post
    }
    method BR1 { x y } {
        $win.b instate {pressed !disabled} {
            $win.b state !pressed
        }
        return -code break
    }
    method press_return { t ids } {
        $self press $t $ids [list item [lindex $ids 0] column 0 elem e_text_sel] "" ""
    }
    method press { t ids identify x y } {
        set id [lindex $ids 0]
        
        if { ![regexp {item \S+ column \S+ elem (\S+)} $identify {} elem] } {
            return
        }
        if { $elem eq "e_image_r" } {
            return [$self clear_tree_entry $id]
        }
        if { [dict exists $cmd_items $id] } {
            if { ![dict exists $no_active_items $id] } { 
                $self action LBCancel
            }
            uplevel #0 [dict get $cmd_items $id] $id
            return
        }
        if { [dict exists $no_active_items $id] } { return }
        
        $self set_text [dict get $fnames $id]
        #uplevel #0 $options(-command) [list [dict get $fnames $id]]
        $toctree unpost
    }
    method clear_tree_entry { id } {
        set txt [dict get $fnames $id]
        if { $options(-textvariable) ne "" } {
            upvar #0 $options(-valuesvariable) v
            set ipos [lsearch -exact $v $txt]
            set v [lreplace $v $ipos $ipos]
        } else {
            error "error in clear_tree_entry. not implemented"
        }
    }
    method post {} {
        $win.b instate !disabled {
            catch { tile::clickToFocus $win.b }
            catch { ttk::clickToFocus $win.b }
            $win.b state pressed
            set x [winfo rootx $win]
            set y [expr {[winfo rooty $win]+[winfo height $win]}]
            $toctree deiconify $x $y [expr {[winfo width $win]-0}]
        }
    }
    method endpost {} {
        $win.b instate {pressed !disabled} {
            $win.b state !pressed
        }
        event generate $win.b <<ComboboxSelected>>
    }
    method _check_textvariable_read {} {
        if { $updating } { return }
        upvar #0 $options(-textvariable) v
        set v [$text get 1.0 end-1c]
    }
    method _check_textvariable_write {} {
        upvar #0 $options(-textvariable) v
        $text delete 1.0 end
        set updating 1
        if { [info exists v] } {
            $text insert end $v
        }
        set updating 0
        $self _check_configure
        $self _update_state
    }
    method _changed_values_var {} {
        if { $options(-valuesvariable) ne "" } {
            upvar #0 $options(-valuesvariable) v
            
            $self tree_item delete all
            $self tree_insert end [_ "(Clear)"] "" 0
            foreach i $v {
                $self tree_insert end $i $i 0
            }
        }
    }
    method _check_configure {} {
        
        if { [winfo width $win] <= 1 } { return }
        if { [llength $options(-height)] < 2 } { return }
        set ds [$win count -displaylines 1.0 end]
        lassign $options(-height) min max
        if { $ds < $min } { set ds $min }
        if { $ds > $max } { set ds $max }
        if { $ds != [$win cget -_height] } {
            $win configure -_height $ds
        }
    }
    method tree_insert { args } {
        set optional {
            { -image image "" }
            { -collapse boolean 0 }
            { -active boolean 1 }
            { -command cmd "" }            
        }
        set compulsory "idx name fullname parent"
        parse_args $optional $compulsory $args  
        
        if { $image eq "" } {
            set image appbook16
        } elseif { $image eq "-" } {
            set image ""
        }
        if { [$self cget -columns_list] eq "" } {
            set data [list [list $image $name]]
        } else {
            set data [list [list $image [lindex $name 0]]]
            lappend data {*}[lrange $name 1 end]
        }
        set id [$toctree insert $idx $data $parent]
        if { $collapse } {
            $toctree item collapse $id
        }
        if { !$active } {
            dict set no_active_items $id ""
            dict set no_active_values $fullname ""
        }
        if { ![info exists cmd_items] } {
            set cmd_items ""
        }
        if { $command ne "" } {
            dict set cmd_items $id $command
        }
        dict set fnames $id $fullname
        
        $toctree item style set $id 0 imagetextimage
        $toctree item element configure $id 0 e_text_sel -text $name
        $toctree item element configure $id 0 e_image -image $image
        $toctree item element configure $id 0 e_image_r -image [cu::get_image actitemdelete16]
        
        return $id
    }
}

################################################################################
#    menubutton_tree
################################################################################

snit::widget cu::_menubutton_tree_helper {
    option -parent ""
    option -columns_list ""
    hulltype toplevel
    
    delegate method * to tree
    delegate option * to tree

    variable tree
    variable marker_resize_xy0

    constructor args {
        wm overrideredirect $win 1

        package require fulltktree
        set columns [list [list 20 "" left imagetext 1]]
        set tree [fulltktree $win.tree -height 50 -has_sizegrip 1 \
                -columns $columns -expand 1]
        
        grid $tree -sticky nsew
        grid columnconfigure $win 0 -weight 1
        grid rowconfigure $win 0 -weight 1
        
        $self configure -bd 1 -relief solid -background white
        $self configurelist $args

        grid configure $tree.t -rowspan 2

        bind $win <ButtonPress-1> [mymethod check_unpost %x %y]
        bind $win <Escape> "[mymethod unpost] ; break"
    }
    onconfigure -columns_list { value } {
        if { $value eq $options(-columns_list) } { return }
        set options(-columns_list) $value

        set columns ""
        set idx 0
        foreach i $options(-columns_list) {
            lassign $i name dict
            if { $idx == 0 } {
                set type imagetext
            } else {
                set type text
            }
            foreach "opt default" [list len 10 justify left is_editable 1 expand ""] {
                set $opt [dict_getd $dict $opt $default]
            }
            lappend columns [list $len $name $justify $type $is_editable $expand]
            incr idx
        }
        $tree configure -expand 0 -columns $columns
    }
    method deiconify { x y min_width } {
                
        set n [$tree index last]
        if { $n < 7 } { set n 7 }
        if { $n > 15 } { set n 15 }
        if { [$tree cget -itemheight] != 0 } {
            set h [expr {[$tree cget -itemheight]*$n}]
        } else {
            set h [expr {[$tree cget -minitemheight]*$n}]
        }
        $tree configure -height $h
        set wi [winfo width $win]
        if { $wi < $min_width } { set wi $min_width }
        if { $wi+$x > [winfo screenwidth $win] } {
            set wi [expr {[winfo screenwidth $win]-$x}]
        }
        if { $y+$h+10 > [winfo screenheight $win] } {
            set h [expr {[winfo screenheight $win]-$y-10}]
        }
        wm geometry $win ${wi}x$h+$x+$y
        update
        wm deiconify $win
        focus $tree
        grab -global $win
    }
    method check_unpost { x y } {
        if { $x < 0 || $x > [winfo width $win] || 
            $y < 0 || $y > [winfo height $win] } {
            $self unpost
        }
    }
    method unpost {} {
        
        $tree close_search_label

        grab release $win
        wm withdraw $win
        event generate $win <<ComboboxSelected>> 
    }
}

################################################################################
# cu::combobox_tree
################################################################################

snit::widgetadaptor cu::_fulltktree {
    option -height 10

    delegate method * to hull
    delegate option * to hull
    delegate option -_height to hull as -height
 
    delegate method _activate to hull as activate
    delegate method _selection to hull as selection
    delegate method _see to hull as see

    #variable marker_resize_xy0

    constructor args {
        package require fulltktree
        installhull using ::fulltktree -height 10 -has_sizegrip 1
        $self configurelist $args
    }
    onconfigure -height {value} {
        set options(-height) $value
        if { $value ne "" } {
            set n [$self index last]
            if { $n < 2 } { set n 2 }
            if { $n > 10 } { set n 10 }
            set h1 [$self cget -itemheight]
            if { $h1 == 0 } {
                set h1 [$self cget -minitemheight]
            }
            set h [expr {$h1*$n}]
            $self configure -_height $h
        }
    }
#     method marker_resize { what x y } {
# 
#         switch $what {
#             BP1 {
#                 set marker_resize_xy0 [list $x $y]
#             }
#             BM1 {
#                 foreach "x0 y0" $marker_resize_xy0 break
#                 set t [winfo toplevel $win]
#                 set width [expr {[winfo width $t]+$x-$x0}]
#                 set height [expr {[winfo height $t]+$y-$y0}]
#                 if { $width < 0 } { set width 0 }
#                 if { $height < 0 } { set height 0 }
#                 wm geometry $t ${width}x$height
#                 set marker_resize_xy0 [list $x $y]
#             }
#         }
#     }
    method curselection {} {
        set ret ""
        foreach i [$self selection get] {
            lappend ret [expr {$i-1}]
        }
        return $ret
    }
    method activate { args } {
        catch { $self _activate {*}$args }
    }
    method see { args } {
        catch { $self _see {*}$args }
    }
    method selection { what args } {
        if { $what eq "set" } {
            lassign $args index
            # changed because of templates menu in export lognoter
            set item [expr {$index+1}]
#             set item [$self item id "rnc $index 0"]
            set err [catch { $self _selection add $item } ret]
            return $ret
        }
        return [$self _selection $what {*}$args]
    }
}

snit::widgetadaptor cu::combobox_tree {
    option -textvariable ""
    option -popup_width 0
    option -semi_readonly 0
    option -nice_print_separator ""    
    option -create_tree init

    delegate method * to hull
    delegate option * to hull
    delegate method tree_item to toctree as item
    delegate method _insert to hull as insert
    delegate method _delete to hull as delete
    delegate option -_textvariable to hull as -textvariable

    variable popdown
    variable toctree
    variable no_active_items ""
    variable no_active_values ""
    variable cmd_items ""
    variable internal_textvariable
    variable prev_textvariable
    variable batch_insert_handler
    variable save_grab ""
    variable changing_vars 0
    
    constructor args {
        
        package require fulltktree

        if { [info command ::GiD_Process] eq "" } {
            installhull using ttk::combobox 
        } else {
            installhull using ttk::combobox -style Padding0.TCombobox
        }
        
        cu::add_contextual_menu_to_entry $win init

        set popdown [toplevel $win.popdown -relief solid -bd 1]
        wm withdraw $popdown
        wm overrideredirect $popdown 1
        update idletasks
        wm transient $popdown [winfo toplevel $win]

        namespace eval ::ttk::combobox [list set Values($win) ""]
        
        if { [package vcompare [package present Tcl] 8.5.9] >= 0 } {
            set popdown [ttk::frame $popdown.f -style ComboboxPopdownFrame]
            grid $win.popdown.f -sticky nsew
            grid columnconfigure $win.popdown 0 -weight 1
            grid rowconfigure $win.popdown 0 -weight 1
        }
        # tk8.5a6
        ttk::scrollbar $popdown.sb -orient vertical
        bind $win.popdown <Configure> [list grid remove $popdown.sb]
        
        set columns [list [list 10 "" left imagetext 1]]
        
        $self configurelist $args
                
        if {$options(-create_tree) eq "init"} {
            set toctree [cu::_fulltktree $popdown.l \
                    -selecthandler [mymethod press] \
                    -columns $columns -expand 1 \
                    -selectmode browse -showheader 0 -showlines 0  \
                    -showbutton 0 -indent 0 -bd 0 \
                    -have_vscrollbar 1 \
                    -have_search_button automatic]                             
            grid $toctree -sticky nsew
        }
        grid columnconfigure $popdown 0 -weight 1
        grid rowconfigure $popdown 0 -weight 1          
        
        if {$options(-create_tree) eq "init"} {              
            #bind $popdown.l.t <ButtonPress-1>    [mymethod button1 %W %x %y]
            bind $win.popdown <ButtonPress-1>    [mymethod button1 %W %x %y]
            bind $popdown.l.t <KeyPress-Return>  [mymethod select_entry]
            bind $popdown.l.t <KeyPress-Escape>  [mymethod cancel]
            #         bind $popdown.l.t <KeyPress-Tab>     { ttk::combobox::LBTab [winfo parent %W] next }
            #         bind $popdown.l.t <<PrevWindow>>     { ttk::combobox::LBTab [winfo parent %W] prev }
            bind $popdown.l.t <Destroy>          { catch { ttk::combobox::LBCleanup [winfo parent %W] }}
            
            switch -- [tk windowingsystem] {
                win32  {
                    bind $win.popdown <FocusOut>  {
                        if { "%W" eq [winfo toplevel %W] } {
                            ttk::combobox::LBCancel [winfo parent %W]
                        }
                    }
                    #bind $popdown.l.t <FocusOut>  { ttk::combobox::LBCancel [winfo parent %W] }
                }
                aqua {
                    bind $popdown.l.t <Deactivate> { ttk::combobox::LBCancel [winfo parent %W] }
                }
            }
            bind $win.popdown <Map> [mymethod begin_post %W]
            bind $win.popdown <Unmap> [mymethod end_post %W]
        }
        
        set internal_textvariable ""
        $self configure -_textvariable [myvar internal_textvariable]
        #trace add variable [myvar internal_textvariable] write [mymethod changed_textvariable_int]
       
        $self configurelist $args
        
        $self _check_insert_new      
        
        return $self
    }
    destructor {
        #trace remove variable [myvar internal_textvariable] write [mymethod changed_textvariable_int]

        if { $options(-textvariable) ne "" } {
            trace remove variable $options(-textvariable) write [mymethod changed_textvariable_ext]
        } 
    }
    onconfigure -textvariable {value} {
        if { $options(-textvariable) ne "" } {
            trace remove variable $options(-textvariable) write [mymethod changed_textvariable_ext]
        }
        set options(-textvariable) $value
        trace add variable $options(-textvariable) write [mymethod changed_textvariable_ext]
        if { [info exists $value] } {
            $self changed_textvariable_ext
        }
    }
    onconfigure -semi_readonly {value} {
        set options(-semi_readonly) $value
        
        if { $options(-semi_readonly) } {
            $self state readonly
        }
    }      
    method insert { args } {
        $self _insert {*}$args
        $self changed_textvariable_int
    }
    method delete { args } {
        $self _delete {*}$args
        $self changed_textvariable_int
    }
    method make_editable { item } {
        $self state !readonly
        $self delete 0 end
        focus $win
    }

    method give_tree {} {
        return $toctree
    }
    method button1 { w x y } {     
        set top [winfo toplevel $w]
        if { $x < -10 || $x > [winfo width $top]+10 || $y < 0 || $y > [winfo height $top]+10 } {
            $self action LBCancel
            return
        }
        return
        focus $popdown.l.t
        return -code continue
    }
    method select_entry {} {   
        set id [$toctree index active]
        if { [dict exists $no_active_items $id] && ![dict exists $cmd_items $id] } {
            return
        }
        ttk::combobox::LBSelected $popdown.l
        set item [lindex [$popdown.l selection get] 0]
        if { $item eq "" } {
            set item [$popdown.l index active]
        }
        if { $item eq "" } { return }
                       
        if { [dict exists $cmd_items $item] } {
            uplevel #0 [dict get $cmd_items $item] $item
            if { [dict exists $no_active_items $item] } { return }
            return
        }
        #set internal_textvariable [$popdown.l item text $item 0]
        set internal_textvariable [lindex [$self cget -values] $item-1]
        
        update idletasks
        $self icursor end
        $self xview [$self index end]
        $self changed_textvariable_int
    }
    method cancel {} {
        ttk::combobox::LBCancel $popdown.l
        $self icursor end
        $self xview [$self index end] 
    }
    method press { t id } {     
        if { [dict exists $cmd_items $id] } {
            if { ![dict exists $no_active_items $id] } { 
                $self action LBCancel
            }
            uplevel #0 [dict get $cmd_items $id] $id
            return
        }
        if { [dict exists $no_active_items $id] } { 
            $self expand_collapse
            return 
        }
        wm withdraw [winfo toplevel $popdown]
        $self select_entry
    }
    method expand_collapse {} {   
        set id [$toctree index active]
        set isopen [$toctree item state get $id open]
        if {!$isopen} {
            $toctree item expand $id
        } else {
            $toctree item collapse $id
        }
    }
    method action { action } {
        set id [$toctree index active]
        if { $action eq "LBSelected" && [dict exists $no_active_items $id] } { return }
        
        if { $action eq "LBSelected" } {
            if { [dict exists $no_active_items $id] } { return }
            $self select_entry
        } elseif { $action eq "LBCancel" } {
            $self cancel
        } else {
            ttk::combobox::$action $popdown.l
            $self icursor end
            $self xview [$self index end]
        }
    }
    method _check_insert_new {} {       
        if { $options(-semi_readonly) && [$toctree item count] == 1 && [$self instate readonly] } {
            $self tree_insert -image [cu::get_iconF filenew-16] -active 1 -command \
                [mymethod make_editable] \
                -check_insert_new 0 end [_ "New#C#Masculino"] [_ "New#C#Masculino"] 0
        }
    }
    method change_item_active { item active_inactive } {
        if { $active_inactive eq "active" } {
            set fullname [lindex  [$self cget -values] $item-1]
            dict unset no_active_items $item
            dict unset no_active_values $fullname
        } else {
            dict set no_active_items $item ""
            dict set no_active_values $fullname ""
        }
    }    
    method def_tree { tree } {
        set toctree $tree
    }
    method tree_insert { args } {
        set optional {
            { -image image "" }
            { -collapse boolean 0 }
            { -active boolean 1 }
            { -command cmd "" }
            { -check_insert_new boolean 1 }           
        }
        set compulsory "idx name fullname parent"
        parse_args $optional $compulsory $args               
        
        if { $check_insert_new } {
            $self _check_insert_new
        }
        if { $image eq "" } {
            set image appbook16
        } elseif { $image eq "-" } {
            set image ""
        }       
        if { [$toctree item style set $parent] eq "imagetext" } {
            set pimg [$toctree item image $parent 0]
            if { [lindex $pimg end-1] eq "appbook16" } {
                $toctree item image $parent 0 [list mac-expand open mac-collapse {}]
            }
        }
        set id [$toctree insert $idx [list [list $image $name]] $parent]
        if { $collapse } {
            $toctree item collapse $id
        }        
        if { !$active } {
            dict set no_active_items $id ""
            dict set no_active_values $fullname ""
        }
        if { $command ne "" } {
            dict set cmd_items $id $command
        }
        set fullnameT [$self convert_to_textvar_value $fullname]
        if { $fullnameT eq [$self get] } {
            $toctree selection clear
            $toctree selection add $id
            $toctree activate $id
            foreach i [lrange [$toctree item ancestors $id] 0 end-1] {
                $toctree item expand $i
            }
            $toctree see $id
        }
        set values [$self cget -values]
        lappend values $fullname
        $self configure -values $values

        return $id
    }
    method tree_insert_batch { values_tree { last_item "-1 root" } } {

        set delta 100
        if { [info exists batch_insert_handler] } {
            after cancel $batch_insert_handler
            unset batch_insert_handler
        }
        foreach i [lrange $values_tree 0 $delta-1] {
            lassign $i level name fname image selectable dict
            if { $selectable eq "" } { set selectable 1 }
            if { $fname eq "" } { set fname $name }
            set parent [dict get $last_item [expr {$level-1}]]
            if { $image eq "" } { set image - }
            set command [dict_getd $dict command ""]
                        
            set item [$self tree_insert -image $image \
                    -active $selectable -command $command end \
                    $name $fname $parent]
            
            if { [dict_getd $dict emphasis 0] } {
                $self tree_item state set $item emphasis
            }
            if { [dict_getd $dict expand 1] == 0 } {
                $self tree_item collapse $item
            }
            dict set last_item $level $item
        }
        if { [llength $values_tree] > $delta } {
            set batch_insert_handler [after 100 [mymethod tree_insert_batch \
                        [lrange $values_tree $delta end] $last_item]]
        }
    }    
    method clear {} {       
        set no_active_items ""
        set no_active_values ""
        set cmd_items ""     
        if {$toctree != ""} { 
            $toctree item delete all
        }
        $self configure -values ""      
        $self _check_insert_new
        wm geometry $win.popdown ""        
    }
    method give_first_active {} {
        foreach i [$self cget -values] {
            if { ![dict exists $no_active_values $i] } {
                return $i
            }
        }
#         set idx 0
#         foreach id [$toctree item range 1 end] {
#             if { ![dict exists $no_active_items $id] } {
#                 return [lindex [$self cget -values] $idx]
#             }
#             incr idx
#         }
        return ""
    }
    variable _begin_post_handler
    method begin_post { w } {
        if { $w ne [winfo toplevel $popdown] && [winfo height $popdown] >= 50 } { return }
        if { [info exists _begin_post_handler] } { return }
        $self _begin_post_after
        set _begin_post_handler [after 400 [mymethod begin_post_unset]]
    }
    method begin_post_unset {} {
        unset -nocomplain _begin_post_handler
    }
    method _begin_post_after {} {
        
        set pattern [string trimright [$self get] *]*
        foreach id [rangeF 1 [$toctree index last]] {
            if { [string match -nocase $pattern [$toctree item text $id 0]] } {
                $toctree selection clear
                $toctree selection add $id
                $toctree activate $id
                break
            }
        }
        set y [winfo rooty $popdown]
        set height [winfo height $popdown]
        if { $height < 50 } {
            set height 50
        }
        
        if { $options(-popup_width) == 0 } {
            set delta 40
        } else {
            set delta [expr {$options(-popup_width)-[winfo width $win]}]
        }
        set x [expr {[winfo rootx $win]-$delta}]
        if { $x < 0 } { set x 0 }
        set width [expr {[winfo width $win]+$delta}]

        wm geometry [winfo toplevel $popdown] ${width}x$height+$x+$y
        
        wm attributes [winfo toplevel $popdown] -topmost 1
        raise [winfo toplevel $popdown]
        after 100 [list catch [list focus -force [$toctree givetreectrl]]]
        set save_grab [grab current $win]
        catch { grab -global [winfo toplevel $popdown] }
    }
    method end_post { w } {
        if { $w ne [winfo toplevel $popdown] } { return }
        
        $toctree close_search_label

        grab release $toctree
        if { $save_grab ne "" } {
            catch { grab $save_grab }
        }
        focus $self
        
#         if { $options(-textvariable) ne "" } {
#             trace remove variable $options(-textvariable) write [mymethod changed_textvariable]
#         }
        if { [info exists batch_insert_handler] } {
            after cancel $batch_insert_handler
            unset batch_insert_handler
        }
    }
    method changed_textvariable_int { args } {
        
        if { $changing_vars } { return }
#         if { $internal_textvariable eq "" } {
#             set prev_textvariable $internal_textvariable
#             return
#         }
#         if { $internal_textvariable eq "" || ![dict exists $no_active_values $internal_textvariable] } {
#             set prev_textvariable $internal_textvariable
#             return
#         }
        set changing_vars 1
        set deltaList [list 1 -1]
        set deltaPos 0
        set vLocal $internal_textvariable
        set idx [lsearch -exact [$self cget -values] $vLocal]
        set idxLocal $idx
        if { [info exists prev_textvariable] } {
            set idx_prev [lsearch -exact [$self cget -values] $prev_textvariable]
            if { $idx_prev > $idx } {
                set deltaList [list -1 1]
            }
        }
        while { [dict exists $no_active_values $vLocal] } {
            incr idxLocal [lindex $deltaList $deltaPos]
            set vLocal [lindex [$self cget -values] $idxLocal]
            if { $vLocal eq "" } {
                incr deltaPos
                if { $deltaPos >= [llength $deltaList] } { break }
                set idxLocal $idx
                set vLocal [lindex [$self cget -values] $idxLocal]
            }
        }

        #  && [$win instate readonly]
        if { $options(-nice_print_separator) ne "" } {
            set l ""
            foreach i $vLocal {
                if { $i ne "" } { lappend l $i }
            }
            set internal_textvariable [join $l $options(-nice_print_separator)]
        } else {
            set err [catch { llength $vLocal } len]
            if { !$err && $len > 1 } {
                set space_at_end ""
                regexp {(.*\S)(\s+)$} $vLocal {} vLocal space_at_end
                set internal_textvariable [list_beautify $vLocal]$space_at_end
            }
        }
        if { $options(-textvariable) ne "" } {
            upvar #0 $options(-textvariable) v
            set v $vLocal
        }
        set prev_textvariable $vLocal
        
        set val $internal_textvariable
        if { [string match "-*" $val] } {
            set val " $val" 
        }
        tooltip::tooltip $self $val

        set changing_vars 0
    }
    method convert_to_textvar_value { v } {
        if { $options(-nice_print_separator) ne "" } {
            return [join $v $options(-nice_print_separator)]
        } else {
            return [list_beautify $v]
        }
    }
    method changed_textvariable_ext { args } {
        
        if { $changing_vars } { return }
        set changing_vars 1
        upvar #0 $options(-textvariable) v
        set internal_textvariable [$self convert_to_textvar_value $v]
        tooltip::tooltip $self $internal_textvariable
        set changing_vars 0
    }
    method keypress { key char } {
        if { ![$win instate readonly] } { return }
        if { ![string is print -strict $char] } { return }
        uplevel #0 [string map [list %W [list $win]] [bind TCombobox <KeyPress-Down>]]
        if { ![string is space $char] } {
            $toctree keypress $key $char
        }
    }
}

################################################################################
#    cu::notebook
################################################################################

snit::widgetadaptor cu::notebook {
    option -menu_callback ""
    option -last_menu_callback ""
    
    delegate method * to hull
    delegate option * to hull
    
    delegate method _tab to hull as tab
    delegate method _tabs to hull as tabs
    delegate method _add to hull as add
    delegate method _insert to hull as insert

    constructor args {
        installhull using ttk::notebook
        
        if { [info command img::cu::notebook::plus] eq "" } {
            image create photo img::cu::notebook::plus -data {
                iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAFo9M/3AAAABGdBTUEAAYagMeiWXwAAAQRJ
                REFUOI2lUjuuhDAMHK+34BYchopr0CBZ4jpINNBSpqShoUSRcgZqTkAR5RX7AoSP2Kc3pTOe
                zNgmZgYAgJmhlAL5CnVdhyiKPk/MDGMMmBkvHLD2eLwAQETcWhARl6YptW27/VcUhfPigYZv
                LcuSfO191MyyjABsRC/lpZVSQe1k8zHHEUFyEXF7PyfCr8EAxxR0jEvMjHEcMc8zrLVYlgXW
                WgzD4E4xjTHBFC/X8+eYj4Qn3FoQEae1htYax9l9JfAt1ggi4vI8J2stiAjW2oDo3GaiaRrn
                T+69J1VVFVhNkoQAoO/72wirwP6IvaM4jjFN0+X7SeAKdV3f/uzx7zX+ANg6cEaakf1xAAAA
                AElFTkSuQmCC
            }
            image create photo img::cu::notebook::plus_active -data {
                iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAIAAAHnlligAAAABGdBTUEAAYagMeiWXwAAAiVJ
                REFUKJF9kb1v00AYxh+7dmLnQ2HInIhISBldYZAKqod2AKRKSAhWtgwta6ZKniL1D4Ch/0AG
                F6RK6RKoooJaGCLaOFI6JAxpPmrkcCEYtY2NE8cMJ4IHyjucfnruveeee2/BsixJkhYkSWrz
                i4ymaQC474nbkTAPTdNM0yzXvpqmyQLwPI+uzN7e3ng8BpDNZmGappwrmqbpOM7Ch5/S88d3
                D7+Mb934BcMw5FzRMIzBYIBSqeS6LiFEzhXlXJEQQgixLIuzbfvei9cAABTWVx5tvgPw8dVT
                DsCbzfsAnm190s8uKBNCGF3X0+m0KIoI1NHREVOv15vNZlCNRCKKojCWZU0mEwDUvbz1IJFI
                lMtlVhAEz/M8zyusr9DHjUYj27Y5AGtqhUaac36VYzqdzumQ73wbTz3/wp65U7/8/iS/yjH9
                fn86nQJo/gir2wc0brVa5UKhEMuy80ihUIgCs7+/v7y8HIxr23a322V0XR8Oh7Isx2IxXF+X
                l5fHx8fJZJJttVqKoriuO/pTSxs7lQapNMjSxs5cdF1XUZRWq8UB8H3f9/1/Ggd1yiylNbXS
                JGgM/NOBTwdDJ1QzZifns9r5bE2t0AMc/SsA+ZeVudmTh3cA7L79HLyNtnEAeJ7fVZW/3YXD
                xZtx/ewCQFDneR4AJ4oiz/PxePzq6mq+p24fUAiHwxSi0ajjOKIoMpZltdvtXq/nOM5/xioI
                QiqVymQyvwG9NCv9DrlLmwAAAABJRU5ErkJggg==
            }
        }
        $self configurelist $args
    }
    onconfigure -menu_callback { value } {
        set options(-menu_callback) $value
        $self _update_icons
    }
    onconfigure -last_menu_callback { value } {
        set options(-last_menu_callback) $value
        $self _update_icons
    }
    method add { args } {
        if { $options(-last_menu_callback) ne "" } {
            set ret [$self _insert [expr {[llength [$self _tabs]]-1}] {*}$args]
        } else {
            set ret [$self _add {*}$args]
        }
        $self _update_icons
        return $ret
    }
    method insert { args } {
        set ret [$self _insert {*}$args]
        $self _update_icons
        return $ret
    }
    method tab { args } {
        set ret [$self _tab {*}$args]
        $self _update_icons
        return $ret
    }
    method tabs {} {
        if { $options(-last_menu_callback) ne "" } {
            return [lrange [$self _tabs] 0 end-1]
        } else {
            return [$self _tabs]
        }
    }
    method _update_icons {} {
        if { $options(-last_menu_callback) ne "" } {
            if { ![winfo exists $win.__last_] } {
                ttk::frame $win.__last_
                $self _add $win.__last_
            }
            set idx [$self index $win.__last_]
            if { $idx > 0 && $idx == [$self index current] } {
                $self select [expr {$idx-1}]
            }
            bind $win <<NotebookTabChanged>> [mymethod notebook_changed]
        } else {
            if { [winfo exists $win.__last_] } {
                destroy $win.__last_
            }
            bind $win <<NotebookTabChanged>> ""
        }
        if { $options(-menu_callback) ne "" } {
            foreach t [$self _tabs] {
                $self _tab $t -image img::cu::notebook::plus -compound right
            }
            bind $win <ButtonPress-1> [mymethod BP1 %x %y]
            bind $win <Motion> [mymethod motion %x %y]
            bind $win <Leave> [mymethod leave]
            bind $win <ButtonRelease-1> [mymethod BR1 %x %y]
        } else {
            foreach t [$self _tabs] {
                if { [$self _tab $t -image] eq "img::cu::notebook::plus" } {
                    $self _tab $t -image ""
                }
            }
            bind $win <ButtonPress-1> ""
            bind $win <Motion> ""
            bind $win <Leave> ""
            bind $win <ButtonRelease-1> ""
        }
    }
    method notebook_changed {} {
        set idx [$self index current]
        if { $idx > 0 && $options(-last_menu_callback) ne "" && $idx == [llength [$self _tabs]]-1 } {
            $self select 0
        }
    }
    method leave {} {
        foreach t [$self _tabs] {
            $self _tab $t -image img::cu::notebook::plus
        }
    }
    method motion { x y } {
        set idx [$self index @$x,$y]
        if { $idx eq "" } { return }
        
        foreach t [$self _tabs] {
            set i [$self index $t]
            if { $idx == $i && [$self index @[expr {$x+18}],$y] != $idx } {
                $self _tab $i -image img::cu::notebook::plus_active
            } else {
                $self _tab $i -image img::cu::notebook::plus
            }
        }
    }
    method BP1 { x y } {
        if { $options(-last_menu_callback) eq "" } { return }
        if { [$self index @$x,$y] == [llength [$self _tabs]]-1 } {
            return -code break
        }
    }
    method BR1 { x y } {
        if { [$self index @[expr {$x+18}],$y] != [$self index @$x,$y] } {
            set idx [$self index @$x,$y]
            set x [expr {$x+[winfo rootx $win]}]
            set y [expr {$y+[winfo rooty $win]}]
            destroy $win.menu
            menu $win.menu -tearoff 0
            if { $options(-last_menu_callback) ne "" && $idx == [llength [$self _tabs]]-1 } {
                uplevel #0 $options(-last_menu_callback) $win.menu
            } else {
                uplevel #0 $options(-menu_callback) $win.menu
            }
            tk_popup $win.menu $x $y
        }
    }
}

snit::widgetadaptor cu::nicelabel {
    option -link_callback ""
    option -link_color ""

    delegate method _insert to hull as insert
    delegate method _delete to hull as delete
    delegate method * to hull
    delegate option * to hull

    variable underlinefont
    variable underlinefontsmall
    variable underlinefontbold
    
    constructor args {
        if { [lsearch -exact [font names] TkDefaultFont] != -1 } {
            set font TkDefaultFont
        } else {
            set font "-size 10"
        }
        text $self -width 5 -height 1 -bd 0 -insertwidth 0 -spacing3 3 -font $font \
            -takefocus 0 -wrap word -highlightthickness 0
                
        set bg [cu::give_widget_background [winfo parent $self]]
        $self configure -background $bg
        $self configure -cursor ""
        installhull $self
        
        $self configurelist $args
        
        set font [font actual [$self cget -font]]
        set size [expr {[font actual $font -size]+1}]
        set font [font actual $font]
        set ipos [expr {[lsearch $font -size]+1}]
        set font [lreplace $font $ipos $ipos $size]
        $self configure -font $font

        set font [font actual [$self cget -font]]
        set size_small [expr {[font actual $font -size]-3}]
        set font [font actual $font]
        set ipos [expr {[lsearch $font -size]+1}]
        set font [lreplace $font $ipos $ipos $size_small]

        set boldfont [font actual [$self cget -font]]
        set ipos [expr {[lsearch $boldfont -weight]+1}]
        set boldfont [lreplace $boldfont $ipos $ipos bold]
        
        set underlinefont [list {*}[font actual [$self cget -font]] -underline 1]
        set underlinefontsmall [list {*}[font actual [$self cget -font]] -underline 1 -size 6]
        set underlinefontbold [list {*}[font actual [$self cget -font]] -underline 1 -weight bold]

        $self tag configure subscript -font $font -offset -3
        $self tag configure superscript -font $font -offset 3
        $self tag configure bold -font $boldfont
        $self tag configure red -foreground red
        $self tag configure grey -foreground grey
        
        bind $self <1> break
        bind $self <Tab> "[bind all <Tab>] ; break"
        bind $self <<PrevWindow>> "[bind all <<PrevWindow>>] ; break"
        bind $self <Configure> [mymethod _check_configure %w]
        $self configure -state disabled
        $self configurelist $args
    }
    method insert { index txt { tagList "" } } {
        if { $index ne "end" } { set index 1.$index }
        $self configure -state normal
        $self _insert $index $txt $tagList
        $self configure -state disabled
    }
    method insert_link { args } {
        
        set optional {
            { -tags tagList "" }
        }
        set compulsory "index txt link title"
        parse_args $optional $compulsory $args

        if { [string is integer $index] } {
            set index 1.$index
        } elseif { $index eq "end" } {
            set index end-1c
        }
        set index [$self index $index]
        set tag link$index
        lappend tags $tag
        if { $options(-link_color) ne "" } {
            set color $options(-link_color)
        } else {
            set color blue
        }
        if { "small" in $tags } {
            $self tag configure $tag -foreground $color -font $underlinefontsmall
        } elseif { "bold" in $tags } {
            $self tag configure $tag -foreground $color -font $underlinefontbold
        } else {
            $self tag configure $tag -foreground $color -font $underlinefont
        }
        if { $title ne "" } {
            tooltip::tooltip $self -tag $tag $title
        }
        $self configure -state normal
        $self _insert $index $txt $tags
        $self configure -state disabled
        
        $self tag bind $tag <1> [mymethod _eval_link $link]
        $self tag bind $tag <Enter> +[list $self configure -cursor hand2]
        $self tag bind $tag <Leave> +[list $self configure -cursor ""]
    }
    method delete { index1 index2 } {
        $self configure -state normal
        $self _delete $index1 $index2
        $self configure -state disabled
    }
    method _check_configure { width } {
        if { [winfo width $win] <= 1 } { return }
        set ds [$win count -displaylines 1.0 end]
        set max [expr {([$win cget -height]>4)?[$win cget -height]:4}]
        if { $ds > $max } { set ds $max }
        if { $ds != [$win cget -height] } {
            $win configure -height $ds
        }
    }
    method _eval_link { cmd } {
        uplevel #0 $options(-link_callback) $cmd
    }
}
