if { ![catch { package present snit }] } {
  package require snit
}
catch {
    package require textutil
}
package require msgcat

package provide customLib_utils 1.20

msgcat::mcload [file join [file dirname [info script]] msgs]

################################################################################
#   Small utilities
################################################################################

proc puts_OFF { args } {}

proc xml_map_inv { args } {
    set map [list "&gt;" > "&lt;" < "&amp;" & "&quot;" \" "&apos;" ']
    return [string map $map [join $args ""]]
}

proc xml_map1 { args } {
    global _xml_map

    set map [list > "&gt;" < "&lt;" & "&amp;"]

    _xml_map_init    
    set ret [string map $_xml_map [join $args ""]]
    return [string map $map $ret]
}

proc xml_map1_inv { args } {
    set map [list "&gt;" > "&lt;" < "&amp;" &]
    return [string map $map [join $args ""]]
}

proc js_map { args } {
    set map [list \" \\\" ' \\' & \\&]
    return [string map $map [join $args ""]]
}

namespace eval ::math::constants {}
proc ::math::constants::give_constant { name } {
    ::math::constants::constants $name
    return [set $name]
}

namespace eval cu {
    variable topdir [file dirname [info script]]
}
namespace eval cu::file {}

proc cu::init {} {
    variable topdir
    
    uplevel #0 [list msgcat::mcload [file join $topdir msgs]]
}

if { [info commands ::_] eq "" } {
    proc ::_ { args } {
        set ret [uplevel 1 ::msgcat::mc $args]
        regexp {(.*)#C#(.*)} $ret {} ret
        return $ret
    }
}

if { [info commands ::=] eq "" } {
    proc ::= { args } { return [_ {*}$args] }
}

proc range {from to {step 1}} {

    if { ![string is double -strict $to] } {
        error "to='$to' is not a number"
    }
    set res ""
    if { $step > 0 } {
        for { set i $from } { $i < $to } { set i [expr {$i+$step}] } {
            lappend res $i
        }
    } else {
        for { set i $from } { $i > $to } { set i [expr {$i+$step}] } {
            lappend res $i
        }
    }
    return $res
}

proc rangeF {from to {step 1}} {

    if { ![string is double -strict $to] } {
        error "to='$to' is not a number"
    }
    set res ""
    if { $step > 0 } {
        for { set i $from } { $i <= $to } { set i [expr {$i+$step}] } {
            lappend res $i
        }
    } else {
        for { set i $from } { $i >= $to } { set i [expr {$i+$step}] } {
            lappend res $i
        }
    }
    return $res
}

if { [info command lreverse] eq "" } {
    proc lreverse { L } {
        set res {}
        set i [llength $L]
        #while {[incr i -1]>=0} {lappend res [lindex $L $i]}
        while {$i} {lappend res [lindex $L [incr i -1]]} ;# rmax
        return $res
    }
}

proc cu::info_level_stack {} {
    
    set sList ""
    set num [info level]
    for {  set i 1} { $i < $num } { incr i } {
        lappend sList [info level -$i]
    }
    return [join $sList " --- "]
}

proc cu::commalist_to_list { data } {
    set map [list "&#44;" "," "&amp;" "&"]
    set list ""
    foreach i [split $data ","] {
        lappend list [string map $map $i]
    }
    return $list
}

proc cu::list_to_commalist { list } {
    set ret ""
    set map [list "," "&#44;" "&" "&amp;"]
    foreach elm $list {
        lappend ret [string map $map $elm]
    }
    return [join $ret ","]
}

proc cu::package_forget { package } {

    set err [catch { package present $package } ret]
    if { $err } { return }

    set args ""
    set script [package ifneeded $package [package require $package]]
    foreach line [cu::splitx $script {[;\n]}] {
        switch -- [lindex $line 0] {
            "load" {
                set args [lrange $line 1 end]
            }
            "source" {
                #nothing
            }
            default {
                interp create cup_for
                cup_for eval [info_fullproc [lindex $line 0] [lindex $line 0]]
                cup_for alias load lappend args
                set err [catch { cup_for eval $line } errstring]
                interp delete cup_for
            }
        }
        if { $args ne "" } {
            break
        }
    }
    if { $args eq "" } {
        error "error in cu::package_forget. package ifneeded is not in the accepted syntax"
    }
    unload {*}$args
    package forget $package
}

# Maps accented vowels (also capitals), and substitutes "bad" charachters
proc cu::unicode_to_ascii { name } {
    set groups [list a\ue0\ue1\ue2\ue3\ue4\ue5 e\ue8\ue9\uea\ueb \
            i\uec\ued\uee\uef o\uf2\uf3\uf4\uf5\uf6 u\uf9\ufa\ufb\ufc \
            A\uc0\uc1\uc2\uc3\uc4\uc5 E\uc8\uc9\uca\ucb \
            I\ucc\ucd\uce\ucf O\ud2\ud3\ud4\ud5\ud6 U\ud9\uda\udb\udc ]        
    set maps ""
    foreach group $groups {
        set rest [lassign [split $group ""] base]
        foreach i $rest {
            lappend maps $i $base
        }
    }
    set name [string map $maps $name]
    regsub -all {[^-a-zA-Z\d_]} $name "_" name   
    return $name
}

proc list_unique { list } {
    return [lsort -dictionary -unique $list]
}

proc list_union { list1 list2 } {
    return [lsort -dictionary -unique [concat $list1 $list2]]
}

proc list_intersection { list1 list2 } {
    set nlist ""
    foreach i $list2 {
        if { [lsearch -exact $list1 $i] != -1 } {
            lappend nlist $i
        }
    }
    return $nlist
}

proc list_difference { list1 list2 } {
    set nlist ""
    foreach i $list1 {
        if { [lsearch -exact $list2 $i] == -1 } {
            lappend nlist $i
        }
    }
    return $nlist
}

proc list_expr { list } {
    set ret ""
    foreach i $list {
        lappend ret [uplevel 1 expr $i]
    }
    return $ret
}

proc dict_inverse { dict } {
    set ret ""
    dict for "n v" $dict {
        dict set ret $v $n
    }
    return $ret
}

proc linsert0 { args } {
    set optional {
        { -max_len len "" }
        { -remove_prefixes "" 0 }
    }
    set compulsory "list element"
    parse_args $optional $compulsory $args

    set ipos [lsearch -exact $list $element]
    if { $ipos != -1 } {
        set list [lreplace $list $ipos $ipos]
    }
    if { $remove_prefixes } {
        for { set i 0 } { $i < [llength $list] } { incr i } {
            if { [string match "[lindex $list $i]*" $element] } {
                set list [lreplace $list $i $i]
                incr i -1
            }
        }
    }
    set list [linsert $list 0 $element]
    if { $max_len ne "" } {
        set list [lrange $list 0 $max_len-1]
    }
    return $list
}

proc list_beautify { args } {
 
    set optional {
        { -level level 1 }
        { -clean_empties boolean 0 }
    }
    set compulsory "list"
    parse_args $optional $compulsory $args

    set ret ""
    foreach i $list {
        if { $level > 1 } {
            set i [list_beautify -level [expr {$level-1}] -clean_empties $clean_empties $i]
        }
        if { $clean_empties && $i eq "" } {
            continue
        }
        if { $ret ne "" } { append ret " " }
        if { [regexp {^[{].*[}]$} [list $i]] } {
            set l "\"$i\""
            if { ![catch { lindex $l 0 }] && [lindex $l 0] eq $i } {
                append ret $l
            } else {
                append ret [list $i]
            }
        } else {
            append ret [list $i]
        }
    }
    return $ret
}

# regmap re string var script
# substitute all the occurrences of 're' in 'string' with
# the result of the evaluation of 'script'. Script is called
# for every match with 'var' variable set to the matching string.
#
# Example: regmap {[0-9]+} "1 2 hello 3 4 world" x {expr {$x+1}}
# Result:  "2 3 hello 4 5 world"
proc regmap {re string var script} {
    set submatches [lindex [regexp -about $re] 0]
    lappend varlist idx
    while {[incr submatches -1] >= 0} {
        lappend varlist _
    }
    set res $string
    set indices [regexp -all -inline -indices $re $string]
    set delta 0
    foreach $varlist $indices {
        foreach {start end} $idx break
        set substr [string range $string $start $end]
        uplevel 1 [list set $var $substr]
        set subresult [uplevel 1 $script]
        incr start $delta
        incr end $delta
        if { $end < $start } {
            set res1 ""
            append res1 [string range $res 0 [expr {$start-1}]] $subresult \
                [string range $res $start end]
            set res $res1
        } else {
            set res [string replace $res $start $end $subresult]
        }
        incr delta [expr {[string length $subresult]-[string length $substr]}]
    }
    return $res
}

proc regexpfile { args } {

    set idx 0
    foreach i $args {
        if { ![string match -* $i] } { break }
        if { $i eq "--" } { incr idx ; break }
        incr idx
    }
    incr idx
    set file [lindex $args $idx]
    
    set fin [open $file r]
    set data [read $fin]
    close $fin
    lset args $idx $data
    return [eval regexp $args]

}

proc regsubfile { args } {

    set opts ""
    set noopts ""
    set isopt 0
    foreach i $args {
        if { $isopt || ![string match -* $i] } {
            lappend noopts $i
            set isopt 1
        } else {
            lappend opts $i
            if { $i == "--" } { set isopt 1 }
        }
    }
    if { [llength $noopts] == 4 } {
        foreach "exp file subSpec varName" $noopts break
    } elseif { [llength $noopts] == 3 } {
        foreach "exp file subSpec" $noopts break
    } else { error "error. usage: regsubfile ?switches? exp file subSpec ?varName?" }

    set fin [open $file r]
    
    set fconf [fconfigure $fin]
    foreach opt [list -length -allocated] {
        dict unset fconf $opt
    }
    fconfigure $fin -translation binary
    set header [read $fin 1024]
    seek $fin 0
    fconfigure $fin {*}$fconf
    set file_endline auto
    set len [string length $header]
    if { $len > 0 } {
        if { [regexp -all {\r\n} $header]*1.0/$len >= 0.005 } {
            set file_endline crlf
        } elseif { [regexp -all {\n} $header]*1.0/$len >= 0.005 } {
            set file_endline lf
        }
    }
    set data [read $fin]
    close $fin
    set retval [regsub {*}$opts $exp $data $subSpec data_out]

    if { [info exists varName] } {
        upvar $varName x
        set x $data_out
    }

    #file copy -force $file $file~
    if { $data ne $data_out } {
        if { $::tcl_platform(platform) ne "windows" } {
            file attributes $file -permissions u+w
        }
        set fout [open $file w]
        fconfigure $fout -translation $file_endline
        puts -nonewline $fout $data_out
        close $fout
    } elseif { $retval } { set retval "(change already applied)" }
    return $retval
}


proc dict_getd { args } {
    
    set dictionaryValue [lindex $args 0]
    set keys [lrange $args 1 end-1]
    set default [lindex $args end]
    if { [dict exists $dictionaryValue {*}$keys] } {
        return [dict get $dictionaryValue {*}$keys]
    }
    return $default
}

proc dict_lappend_r { dictionaryVariable args } {
    upvar 1 $dictionaryVariable dict
    
    if { ![info exists dict] } { set dict "" }
    set list [dict_getd $dict {*}[lrange $args 0 end-1] ""]
    lappend list [lindex $args end]
    dict set dict {*}[lrange $args 0 end-1] $list
}

proc dict_incr_r { dictionaryVariable args } {
    upvar 1 $dictionaryVariable dict
    set value [dict_getd $dict {*}[lrange $args 0 end-1] ""]
    if { $value eq "" } {
        set value [lindex $args end]
    } else {
        incr value [lindex $args end]
    }
    dict set dict {*}[lrange $args 0 end-1] $value
}

# prints a tree defined as struct::tree
proc print_tree { tree { node root } } {
    set _ ""
    $tree walk $node i {
        append _ "$i [$tree parent $i] [$tree getall $i]\n"
    }
    return $_
}

proc print_tree_xml { tree { node root } } {
    package require tdom
    set _ ""
    $tree walk $node -order both "action i" {
        if { $action eq "enter" } {
            append _ "<$i "
            foreach "n v" [$tree getall $i] {
                if { [string is integer $n] } {
                    set n integer_$n
                }
                if { [string is double -strict $v] } {
                    set v [format %.3g $v]
                }
                append _ "$n='[xml_map $v]' "
            }
            append _ ">"
        } else {
            append _ "</$i>"
        }
    }
    dom parse $_ doc
    
    package require RamDebugger
    interp eval ramdebugger [list RamDebugger::OpenFileSaveHandler \
            "*formula (xml)*" [$doc asXML] ""]
    
}

proc incr! { varName { increment 1 } } {

    upvar $varName v
    if { ![info exists v] } { set v 0 }
    return [incr v $increment]
}

proc set! { varName args } {
    
    upvar 1 $varName v
    if { [llength $args] == 0 && ![info exists v] } {
        return ""
    }
    return [uplevel 1 [list set $varName {*}$args]]
}

proc info_fullargs { procname } {
    set ret ""
    foreach arg [uplevel 1 [list info args $procname]] {
        if { [uplevel 1 [list info default $procname $arg value]] } {
            upvar 1 value value
            lappend ret [list $arg $value]
        } else {
            lappend ret $arg
        }
    }
    return $ret
}

################################################################################
#    store preferences
################################################################################

proc cu::store_program_preferences { args } {

    set optional {
        { -valueName name ""  }
        { -debug boolean   0  }
        { -prefs_dir "dir" "" }
    }
    set compulsory "program_name data"

    parse_args $optional $compulsory $args

    if { $valueName eq "" } {
        set valueNameF IniData
    } else {
        set valueNameF IniData_$valueName
    }

    if { $::tcl_platform(platform) eq "windows" && $::tcl_platform(os) ne "Windows CE" } {
        set key "HKEY_CURRENT_USER\\Software\\Compass\\$program_name"
        package require registry
        registry set $key $valueNameF $data
    } else {
        set err_pack [catch { package present compass_utils::c }]
        if { $err_pack } {
            package require tdom
        } else {
            package require compass_utils::c
        }
        if { $prefs_dir ne "" } {
            set file [file normalize [file join $prefs_dir .compass_${program_name}_prefs]]
        } elseif { $::tcl_platform(os) eq "Windows CE" } {
            set dir [file join / "Application Data" Compass $program_name]
            file mkdir $dir
            set file [file join $dir prefs]
        } elseif { [info exists ::env(HOME)] } {
            set file [file normalize ~/.compass_${program_name}_prefs]
        } else {
            set file [file normalize [file join /var/tmp .compass_${program_name}_prefs]]
        }
        if { $debug } {
            puts "cu::store_program_preferences file=$file"
        }
        if { $err_pack } {
            set err [catch { tDOM::xmlReadFile $file } xml]
            if { $err } { set xml "<preferences/>" }
            set err [catch { dom parse $xml } doc]
            if { $err } {
                set doc [dom parse "<preferences/>"]            
            } 
        } else {
            set err [catch { p::xml parse -file $file } doc opts]   
            if { $err } {
                set doc [p::xml parse "<preferences/>"]
            }              
        }        
        set root [$doc documentElement]
        set domNode [$root selectNodes "pref\[@n=[xpath_str $valueNameF]\]"]
        if { $domNode ne "" } { $domNode delete }
        set p [$root appendChildTag pref]
        $p setAttribute n $valueNameF
        $p appendChildText $data
        
        set fout [open $file w]
        fconfigure $fout -encoding utf-8
        puts $fout [$doc asXML]
        close $fout
    }
}

proc cu::get_program_preferences { args } {

    set optional {
        { -valueName name "" }
        { -default default_value "" }
        { -debug boolean 0 }
        { -prefs_dir "dir" "" }
    }
    set compulsory "program_name"

    parse_args $optional $compulsory $args

    if { $valueName eq "" } {
        set valueNameF IniData
    } else {
        set valueNameF IniData_$valueName
    }

    set data $default
    if { $::tcl_platform(platform) eq "windows" && $::tcl_platform(os) ne "Windows CE" } {
        set key "HKEY_CURRENT_USER\\Software\\Compass\\$program_name"
        package require registry
        set err [catch { registry get $key $valueNameF } data]
        if { $err } {
            set data $default
        }
    } else {
        set err_pack [catch { package present compass_utils::c }]
        if { $err_pack } {
            package require tdom
        } else {
            package require compass_utils::c
        }     
        if { $prefs_dir ne "" } {
          set file [file normalize [file join $prefs_dir .compass_${program_name}_prefs]] 
        } elseif { $::tcl_platform(os) eq "Windows CE" } {
            set dir [file join / "Application Data" Compass $program_name]
            file mkdir $dir
            set file [file join $dir prefs]
        } elseif { [info exists ::env(HOME)] } {
            set file [file normalize ~/.compass_${program_name}_prefs]
        } else {
            set file [file normalize [file join /var/tmp .compass_${program_name}_prefs]]
        }
        if { $debug } {
            puts "cu::get_program_preferences file=$file"
        }
        if { $err_pack } {
            set err [catch { dom parse $xml } doc opts]
        } else {
            set err [catch { p::xml parse -file $file } doc opts] 
        }     
        if { !$err } {
            set root [$doc documentElement]
            set domNode [$root selectNodes "pref\[@n=[xpath_str $valueNameF]\]"]
            if { $domNode ne "" } {
                set data [$domNode text]
            }
        }
        if { $err && $debug } {
            puts [dict get $opts -errorinfo]
        }    
    }
    return $data
}

################################################################################
#    XML & xpath utilities
################################################################################

proc xpath_str { str } {
    
    foreach "strList type pos" [list "" "" 0] break
    while 1 {
        switch $type {
            "" {
                set ret [regexp -start $pos -indices {['"]} $str idxs]
                if { !$ret } {
                    lappend strList "\"[string range $str $pos end]\""
                    break
                }
                set idx [lindex $idxs 0]
                switch -- [string index $str $idx] {
                    ' { set type apostrophe }
                    \" { set type quote }
                }
            }
            apostrophe {
                set ret [regexp -start $pos -indices {["]} $str idxs]
                if { !$ret } {
                    lappend strList "\"[string range $str $pos end]\""
                    break
                }
                set idx [lindex $idxs 0]
                lappend strList "\"[string range $str $pos [expr {$idx-1}]]\""
                set type quote
                set pos $idx
            }
            quote {
                set ret [regexp -start $pos -indices {[']} $str idxs]
                if { !$ret } {
                    lappend strList "'[string range $str $pos end]'"
                    break
                }
                set idx [lindex $idxs 0]
                lappend strList "'[string range $str $pos [expr {$idx-1}]]'"
                set type apostrophe
                set pos $idx
            }
        }
    }
    if { [llength $strList] > 1 } {
        return "concat([join $strList ,])"
    } else {
        return [lindex $strList 0]
    }
}

proc format_xpath { string args } {
    set cmd [list format $string]
    foreach i $args {
        lappend cmd [xpath_str $i]
    }
    return [eval $cmd]
}

namespace eval ::dom::domNode {}

# args can be one or more tags
proc ::dom::domNode::appendChildTag { node args } {
    if { [::llength $args] == 0 } {
        error "error in appendChildTag. At list one tag"
    }
    ::set doc [$node ownerDocument]
    foreach tag $args {
        if { [string match "text() *" $tag] } {
            foreach child [$node selectNodes text()] { $child delete }
            ::set newnode [$doc createTextNode [lindex $tag 1]]
            $node appendChild $newnode
            ::set node $newnode
        } elseif { [string match "attributes() *" $tag] } {
            foreach "n v" [lrange $tag 1 end] {
                $node setAttribute $n $v
            }
        } else {
            ::set newnode [$doc createElement $tag]
            $node appendChild $newnode
            ::set node $newnode
        }
    }
    return $node
}

proc ::dom::domNode::appendChildText { node text } {
    ::set doc [$node ownerDocument]
    foreach child [$node selectNodes text()] { $child delete }
    ::set newnode [$doc createTextNode $text]
    $node appendChild $newnode
    return $newnode
}

################################################################################
#     cu::file::execute, cu::kill and cu::ps
################################################################################

proc cu::kill { pid } {

    if { $::tcl_platform(platform) eq "windows" } {
        package require customLib_utils::c
        return [cu::_kill_win $pid]
    } else {
        exec kill $pid 
    }
}

proc cu::ps { args } {

    if { $::tcl_platform(platform) eq "windows" } {
        package require customLib_utils::c
        set ps_args ""
        foreach i $args {
            if { $i eq "" } { continue }
            if { ![regexp {^\*} $i] } {
                set i "*$i"
            }
            if { ![regexp {\*$} $i] } {
                set i "$i*"
            }
            lappend ps_args $i
        }
        set ret [cu::_ps_win {*}$ps_args]
        catch { package require twapi }
        set retret ""
        foreach i $ret {
            lassign $i cmd pid
            if { [info command ::twapi::get_process_info] ne "" } {
                set d [twapi::get_process_info $pid -createtime -privilegedtime -workingset]
                set stime [clock format [twapi::large_system_time_to_secs [dict get $d -createtime]] \
                        -format "%H:%M:%S"]
                set cputime [clock format [twapi::large_system_time_to_secs \
                            [dict get $d -privilegedtime]] -format "%H:%M:%S" -timezone :UTC]
                set size [expr {[dict get $d -workingset]/1024}]
                set i [list $cmd $pid $stime $cputime $size]
            }
            lappend retret $i
        }
        return $retret
    } else {
        # does not do exactly the same than in Windows
        #set err [catch { exec pgrep -l -f [lindex $args 0] } ret]
        #set retList  [split $ret \n]
        lassign $args pattern
        if { $pattern eq "" } {
            set err [catch { exec ps -u $::env(USER) --no-headers -o pid,stime,time,pcpu,size,cmd } ret]
        } elseif { [string is integer -strict $pattern] } {
            set err [catch { exec ps --pid $pattern --no-headers -o pid,stime,time,pcpu,size,cmd } ret]
        } else {
            set err [catch { exec ps -u $::env(USER) --no-headers -o pid,stime,time,pcpu,size,cmd | grep -i $pattern } ret]
        }        
        if { $err } {
            return ""
        } else {
            set retList ""
            foreach line [split $ret \n] {
                regexp {(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)} $line {} pid stime cputime \
                    pcpu size cmd
                set pcpu [format "%02.0f%%" $pcpu]
                if { $pattern ne "" && $cmd eq "grep -i $pattern" } { continue }
                lappend retList [list $cmd $pid $stime "$cputime ($pcpu)" $size]
            }
            return $retList
        }
    }
}

proc cu::file::correct_name { file } {
    if { $::tcl_platform(platform) eq "windows" } {
        regsub -all {[:*?""<>|]} $file {_} file
    }
    return [string trim $file]
}

proc cu::file::execute { args } {
    
    set optional {
        { -workdir directory "" }
        { -wait boolean 0 }
        { -hide_window boolean 0 }
    }
    set compulsory "what file"

    set args [parse_args -raise_compulsory_error 0 $optional $compulsory $args]

    switch -- $what {
        gid {
            set exe [get_executable_path gid]
            if { $exe eq "" } { return }
            if { $wait || $hide_window } {
                set err [catch { package require twapi }]
                if { $err } { set has_twapi 0 } else { set has_twapi 1 }
            }
            if { !$wait || $has_twapi } { lappend args & }
            set pid [exec $exe $file {*}$args]
           
            if { !$wait && !$hide_window } { return }
            if { !$has_twapi } { return }

            if { $hide_window } {
                foreach hwin [twapi::find_windows -pids $pid -visible true] {
                    twapi::hide_window $hwin
                }
            }
            if { $wait } {
                while { [twapi::process_exists $pid] } {
                    after 200
                }
            }
        }
        emacs {
            exec runemacs -g 100x72 &
        }
        wish {
            set pwd [pwd]
            cd [file dirname $file]
            eval exec wish [list [file normalize $file]] $args &
            cd $pwd
        }
        tkdiff {
            set pwd [pwd]
            cd [file dirname $file]
            exec wish ~/myTclTk/tkcvs/bin/tkdiff.tcl -r [file tail $file] &
            cd $pwd
        }
        start {
            if { $::tcl_platform(platform) eq "unix" } {
                set programs [list xdg-open gnome-open]
                if { $::tcl_platform(os) eq "Darwin" } {
                    set programs [linsert $programs 0 open]
                }
                foreach i $programs {
                    if { [auto_execok $i] ne "" } {
                        exec $i $file &
                        return
                    }
                }
                error "could not open file '$file'"
            } elseif { [regexp {[&]} $file] } {
                set bat [file join [file dirname $file] a.bat]
                set fout [open $bat w]
                puts $fout "start \"\" \"$file\""
                close $fout
                exec $bat 
                file delete $bat
            } else {
                eval exec [auto_execok start] \"\" [list $file] {*}$args &
            }
        }
        url {
            if { [regexp {^[-\w.]+$} $file] } {
                set file http://$file
            }
            if { ![regexp {(?i)^\w+://} $file] && ![regexp {(?i)^mailto:} $file] } {
                set txt [_ "url does not begin with a known handler like: %s. Proceed?" \
                        "http:// ftp:// mailto:"]
                set retval [tk_messageBox -default ok -icon question -message $txt \
                        -type okcancel]
                if { $retval == "cancel" } { return }
            }
            if { $::tcl_platform(platform) eq "windows" } {
                exec rundll32 url.dll,FileProtocolHandler $file &
            } else {
                set programs [list xdg-open gnome-open]
                if { $::tcl_platform(os) eq "Darwin" } {
                    set programs [linsert $programs 0 open]
                }
                foreach i $programs {
                    if { [auto_execok $i] ne "" } {
                        exec $i $file &
                        return
                    }
                }
                set cmdList ""
                foreach i [list firefox konqueror mozilla opera netscape] {
                    lappend cmdList "$i \"$file\""
                }
                exec sh -c [join $cmdList "||"] & 
            }
        }
        exec {
            if { $workdir ne "" } {
                set pwd [pwd]
                cd $workdir
            }
            set err [catch { exec $file {*}$args } errstring opts]
            
            if { $err && $::tcl_platform(platform) eq "windows" } {
                package require registry
                set key0 {HKEY_CLASSES_ROOT\Applications\%s\shell\open\command}
                set file [file root [file tail $file]].exe
                set key [format $key0 $file]
                set err [catch { registry get $key "" } value]
                if { !$err } {
                    set cmd [string map [list %1 [lindex $args 0]] $value]
                    regsub -all {\\} $cmd / cmd
                    set err [catch { exec {*}$cmd } errstring opts]
                }
            }
            
            if { $workdir ne "" } { cd $pwd }
            if { $err } {
                error $errstring [dict get $opts -errorinfo]
            }
        }
        execList {
            foreach i $file {
                if { [auto_execok [lindex $i 0]] ne "" } {
                    exec {*}$i &
                    return
                }
            }
          error "Could not execute files"
        }
        default {
            if { $workdir ne "" } {
                set pwd [pwd]
                cd $workdir
            }
            set err [catch { exec $file {*}$args & } errstring]
            if { $workdir ne "" } { cd $pwd }
            if { $err } {
                error $errstring $::errorInfo
            }
        }
    }  
}

################################################################################
#    zip and unzip files
################################################################################

proc cu::is_zipfile { zipfile } {
    set err [catch { cu::zip open $zipfile r } h]
    if { $err } {
        return 0
    } else {
        cu::zip close $h
        return 1
    }
}

proc cu::zipfile { args } {
    
    # this is for backwards compatibility
    set nargs $args
    set args ""
    foreach i $nargs {
        switch -regexp -- $i {
            {^-r$} { lappend args -recursive }
            {^-([0-9])$} {
                regsub -- {^-([0-9])$} $i {-compression \1} i
                lappend args {*}$i
            }
            default { lappend args $i }
        }
    }

    set optional {
        { -recursive "" 0 }
        { -compression 0|...|9 9 }
        { -data "" 0 }
        { -encoding name "" }
    }
    set compulsory "zipfile"
    set files [parse_args -raise_compulsory_error 0 \
            $optional $compulsory $args]

    if { $data && $recursive } {
        error "error: options -data and -recursive cannot be used together in cu::zipfile"
    }

    if { $recursive } {
        for { set i 0 } { $i < [llength $files] } { incr i } {
            if { [file isdirectory [lindex $files $i]] } {
                lappend files {*}[glob -nocomplain -dir [lindex $files $i] *]
            }
        }
    }
    
    set h [cu::zip open $zipfile w]
    if { !$data } {
        foreach i $files {
            if { [file isdirectory $i] } { continue }
            cu::zip set $h $i -level $compression -time [file mtime $i]
            set fin [open $i r]
            fconfigure $fin -translation binary
            if { $encoding ni [list "" "binary"] } {
                cu::zip write $h [encoding convertto $encoding [read $fin]]
            } else {
                cu::zip write $h [read $fin]
            }
            close $fin
        }
    } else {
        set now [clock seconds]
        foreach i $files {
            lassign $i name dataL comment
            cu::zip set $h $name -level $compression -time $now -comment $comment
            if { $encoding ni [list "" "binary"] } {
                set dataL [encoding convertto $encoding $dataL]
            }
            cu::zip write $h $dataL
        }
    }
    cu::zip close $h
}

proc cu::unzipfile { args } {
    
    # this is for backwards compatibility
    set nargs $args
    set args ""
    foreach i $nargs {
        if { $i eq "-o" } {
            lappend args -overwrite always
        } else {
            lappend args $i
        }
    }

    set optional {
        { -overwrite ask|always|never ask }
        { -data "" 0 }
        { -list "" 0 }
        { -encoding name "" }
    }
    set compulsory "zipfile"
    parse_args $optional $compulsory $args
    
    if { $data && $list } {
        error "error: options -data and -list cannot be used together in cu::unzipfile"
    }
    
    set retval ""
    set h [cu::zip open $zipfile r]
    if { $list } {
        foreach file [cu::zip files $h] {
            lassign [cu::zip info $h $file] mtime csize size att comment
            lappend retval [list $file $mtime]
        }
    } elseif { !$data } {
        foreach file [cu::zip files $h] {
            lassign [cu::zip info $h $file] mtime csize size att comment
            if { ![file exists [file dirname $file]] } { file mkdir [file dirname $file] }
            if { [file exists $file] && $overwrite ne "always" } {
                if { $overwrite eq "ask" } {
                    package require Tk
                    package require dialogwin
                    set w [dialogwin_snit .ask -okname [_ Yes] -morebuttons \
                            [list [_ "Yes to all"] No [_ "No to all"]] -entrytext \
                            [_ "Are you sure to overwrite file '%s'?" $file]]
                    set action [$w createwindow]
                    destroy $w
                    switch -- $action {
                        -1 - 0 {
                            cu::zip close $h
                            return
                        }
                        1 { #nothing }
                        2 { set overwrite always }
                        3 { continue }
                        4 { set overwrite never ; continue }
                    }
                } elseif { $overwrite eq "never" } { continue }
            }
            cu::zip set $h $file
            set fout [open $file w]
            fconfigure $fout -translation binary
            if { $encoding ni [list "" "binary"] } {
                set data [encoding convertfrom $encoding [cu::zip read $h]]
            } else {
                set data [cu::zip read $h]
            }
            puts -nonewline $fout $data
            close $fout
            file mtime $file $mtime
            lappend retval [list $file $mtime]
        }
    } else {
        foreach file [cu::zip files $h] {
            lassign [cu::zip info $h $file] mtime csize size att comment
            cu::zip set $h $file
            if { $encoding ni [list "" "binary"] } {
                set data [encoding convertfrom $encoding [cu::zip read $h]]
            } else {
                set data [cu::zip read $h]
            }
            lappend retval [list $file $mtime $data $comment]
        }
    }
    cu::zip close $h
    return $retval
}


################################################################################
#    parse args
#
# example:
#  proc myproc { args } {
#     set optional {
#         { -view_binding binding "" }
#         { -file file "" }
#         { -restart_file boolean 0 }
#         { -flag1 "" 0 }
#     }
#     set compulsory "levels"
#     parse_args $optional $compulsory $args
#
#     if { $view_binding ne "" } { puts hohoho }
#     if { $flag1 } { puts "activated flag" }
#  }
# 
################################################################################

proc ::parse_args { args } {

    set optional {
        { -raise_compulsory_error boolean 1 }
        { -compulsory_min min_number "" }
    }
    set compulsory "optional compulsory arguments"

    if { [info level] == 1 } {
        set cmdname [list [info nameofexecutable]]
        if { ![info exists ::starkit::topdir] } {
            lappend cmdname $::argv0
        }
    } else {
        set cmdname [lindex [info level [expr {[info level]-1}]] 0]
    }
    if { [llength $args] > [llength $compulsory] && [string match -* [lindex $args 0]] } {
        parse_args $optional $compulsory $args
    } else {
        set raise_compulsory_error 1
        set compulsory_min ""
        if { [llength $args] != [llength $compulsory] } {
            uplevel 1 [list error [_parse_args_string $cmdname $optional \
                        $compulsory $arguments]]
            return ""
        }
        lassign $args {*}$compulsory
    }

    foreach i $optional {
        lassign $i name namevalue default
        set opts_value($name) $namevalue
        if { [llength $i] > 2 } {
            set opts($name) $default
        }
    }
    set arguments0 $arguments
    while { (!$raise_compulsory_error || [llength $arguments] > [llength $compulsory]) &&
        [string match -* [lindex $arguments 0]] } {
        if { [lindex $arguments 0] eq "--" } {
            set arguments [lrange $arguments 1 end]
            break
        }
        lassign [lrange $arguments 0 1] name value
        if { [regexp {(.*)=(.*)} $name {} name value] } {
            set has_att_value 1
        } else {
            set has_att_value 0
        }
        if { [info exists opts_value($name)] } {
            if { $has_att_value } {
                set opts($name) $value
                set arguments [lrange $arguments 1 end]
            } elseif { $opts_value($name) eq "" } {
                set opts($name) 1
                set arguments [lrange $arguments 1 end]
            } else {
                if { [llength $arguments] == 1 } {
                    uplevel 1 [list error [_parse_args_string $cmdname $optional \
                                $compulsory $arguments0]]
                    return ""
                }
                set opts($name) $value
                set arguments [lrange $arguments 2 end]
            }
        } else {
            uplevel 1 [list error [_parse_args_string $cmdname $optional \
                        $compulsory $arguments0]]
            return ""
        }
    }
    if { $raise_compulsory_error } {
        if { $compulsory_min ne "" } {
            if { [llength $arguments] < $compulsory_min || \
                [llength $arguments] > [llength $compulsory] } {
                uplevel 1 [list error [_parse_args_string $cmdname $optional $compulsory $arguments]]
                return ""
            }
        } elseif { [llength $arguments] != [llength $compulsory] } {
            uplevel 1 [list error [_parse_args_string $cmdname $optional \
                        $compulsory $arguments]]
            return ""
        }

    }
    foreach name [array names opts] {
        uplevel 1 [list set [string trimleft $name -] $opts($name)]
    }
    set inum 0
    foreach i $compulsory {
        uplevel 1 [list set $i [lindex $arguments $inum]]
        incr inum
    }
    return [lrange $arguments $inum end]
}

proc ::_parse_args_string { cmd optional compulsory arguments } {

    set str "error. usage: $cmd "
    foreach i $optional {
        lassign $i name namevalue default
        if { $namevalue ne "" } {
            append str "?$name $namevalue? "
        } else {
            append str "?$name? "
        }
    }
    append str $compulsory
    append str "\n\targs: '$cmd $arguments'"
    return $str
}

proc ::parse_args_create_cmd { optional compulsory } {

    set frame [info frame -1]
    
    if { [regexp {RDC::infoproc} $frame] } {
        # this is to make the proc work inside ramdebugger
        set frame [info frame -3]
    }
    
    if { ![dict exists $frame proc] } { return "" }
    set cmdname [dict get $frame proc]
    set cmdname2 [dict get $frame cmd]
    set cmd [list $cmdname]
    
    foreach i $optional {
        lassign $i name namevalue default
        set n [string trimleft $name "-"]
        set v [uplevel 1 set $n]
        if { $namevalue eq "" } {
            if { $v } {
                lappend cmd $name
            }
        } elseif { $v ne $default } {
            lappend cmd $name $v
        }
    }
    foreach i $compulsory {
        set v [string trimleft $i "-"]
        lappend cmd [uplevel 1 set $v]
    }
    return $cmd
}

################################################################################
#    XML & xpath utilities & tdom
#
################################################################################

namespace eval cu::dom {}

proc cu::dom::parse { args } {
    
    set optional {
        { -keepEmpties "" 0 }
    }
    set compulsory "xml"
    parse_args $optional $compulsory $args

    set opts ""
    if { $keepEmpties } { lappend opts -keepEmpties }
    
    set err [catch { dom parse {*}$opts $xml } doc]
    if { $err } {
        set xml [cu::xml_correct_incorrect_characters $xml]
        set doc [dom parse {*}$opts $xml]
    }
    return $doc
}

################################################################################
#    Date utilities 
#
################################################################################

proc date_sql2tcl { date } {
    if { [regexp {^[-: 0.]*$} $date] } { return "" }
    return [clock scan $date]
}
proc date_tcl2sql { date } {
    if { $date == "" } { return "" }
    return [clock format $date -format "%Y-%m-%d %H:%M:%S"]
}

proc cu::xml_correct_incorrect_characters { xml } {
    global _xml_map
    
    _xml_map_init
    return [string map $_xml_map $xml]

#    set chars "\u1\u2\u3\u4\u5\u6\u7\u8\ub\uc\ue\uf\u10\u11\u12\u13\u14\u15\u16\u17\u18\u19\u1a\u1b\u1c\u1d\u1e\u1f"
# 
#     set map ""
#     foreach i [split $chars ""]  {
#         lappend map $i ""
#     }
#     return [string map $map $xml]
}

#return 0 if is not a link,  1 if the file is a link or  2 if it is Windows shell link 
proc cu::islink { lnk } {
    set islink 0
    if { ![file exists $lnk] } {
        return 0
    }
    if { [string match "windows" $::tcl_platform(platform)] } {
        set type [file type $lnk]
        if { $type == "file" && [string tolower [file extension $lnk]] == ".lnk" } {
            set fp [open $lnk]
            fconfigure $fp -encoding binary -translation binary -eofchar {}
            binary scan [read $fp 4] i tmp
            if { $tmp == "76" } { 
                binary scan [read $fp 16] h32 tmp
                if { $tmp == "10412000000000000c00000000000064" } { 
                    set islink 2 ;#windows shell link
                }
            }
            close $fp
        } 
        if { !$islink } {
            if { ![catch  {set res [file readlink $lnk]}] } {
                set islink 1
            }
        }
    } else {
        if { ![catch  {set res [file readlink $lnk]}] } {
            set islink 1
        }
    }
    return $islink
}

proc _xml_map_init {} {
    global _xml_map
    
    if { [info exists _xml_map] } { return }
    
    set _xml_map ""
    for { set i 0 } { $i < 32 } { incr i } {
        if { $i in [list 9 10 13] } { continue }
        lappend _xml_map [subst [format "\\u%04x" $i]] ""
    }
}

proc xml_map { args } {
    global _xml_map

    set map [list > "&gt;" < "&lt;" & "&amp;" \" "&quot;" ' "&apos;"]

    _xml_map_init    
    set ret [string map $_xml_map [join $args ""]]
    return [string map $map $ret]
}

proc cu::splitx {str {regexp {[\t \r\n]+}}} {
    if {[string length $str] == 0} {
        return {}
    }
    if {[string length $regexp] == 0} {
        return [::split $str ""]
    }
    set list  {}
    set start 0
    while {[regexp -start $start -indices -- $regexp $str match submatch]} {
        foreach {subStart subEnd} $submatch break
        foreach {matchStart matchEnd} $match break
        incr matchStart -1
        incr matchEnd
        lappend list [string range $str $start $matchStart]
        if {$subStart >= $start} {
            lappend list [string range $str $subStart $subEnd]
        }
        set start $matchEnd
    }
    lappend list [string range $str $start end]
    return $list
}

################################################################################
#    Give native file
################################################################################

proc cu::file::give_native_file { program_name file } {
    
    if { [lindex [file system $file] 0] ne "native" } {
        set nfile [file join [cu::file::appdatadir $program_name] \
                [file tail $file]]
        if { ![file exists $nfile] || [file mtime $nfile] < [file mtime $file] } {
            file copy -force $file $nfile
        }
        set file $nfile
    }
    return $file
}

################################################################################
#    write excel 2007 (.xlsx) files
################################################################################

namespace eval cu::write_xlsx {
    set template [file normalize [file join [file dirname [info script]] excel_templates/excel_template1.xlsx]]
}

proc cu::write_xlsx::excel_cell { row col } {
    set letters "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    
    set l ""
    incr row
    incr col
    while { $col > 0 } {
        set l "[string index $letters [expr {($col-1)%26}]]$l"
        set col [expr {($col-1)/26}]
    }
    return "$l$row"
}

proc cu::write_xlsx::excel_row_col { cell } {
    set letters "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    
    regexp {([a-zA-Z]+)(\d+)} $cell {} lts row1
    set row [expr {$row1-1}]
    
    set fac 1
    set col 0
    set num 1
    foreach i [split $lts ""] {
        set fac [expr {26**([string length $lts]-$num)}]
        incr col [expr {$fac*([string first $i $letters]+1)}]
        incr num
    }
    incr col -1
    return [list $row $col]
}

proc cu::write_xlsx::excel_cellR { row col cell } {

    regexp {R([-+0-9]+)C([-+0-9]+)} $cell {} r c
    set row [expr $row+$r]
    set col [expr $col+$c]
    return [excel_cell $row $col]
}

proc cu::write_xlsx::update_worksheet { data values } {
    
    dom parse $data doc
    
    set ns "http://schemas.openxmlformats.org/spreadsheetml/2006/main" 
    
    set sNode [$doc selectNodes -namespaces [list w $ns] w:worksheet/w:sheetData]
    
    foreach domNode [$sNode childNodes] {
        $domNode delete 
    }
    set irow 0
    foreach row $values {
        set xml {<row xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">}
        set icol 0
        foreach col $row {
            if { [dict_getd $col t ""] eq "date" } {
                set s 2
            } elseif { [dict_getd $col t ""] eq "datetime" } {
                set s 3
            } elseif { [dict_getd $col s ""] eq "bold" } {
                set s 1
            } else {
                set s 0
            }
            append xml [format "<c r='%s' s='%d' " [excel_cell $irow $icol] $s]
            switch [dict_getd $col t ""] {
                "value" - "date" - "datetime" {
                    append xml "t='n'><v>[xml_map [dict_getd $col v ""]]</v>"
                }
                "formula" {
                    set f [dict_getd $col v ""]
                    set f [regmap {R[-0-9]+C[-0-9]+} $f x "excel_cellR $irow $icol \$x"]
                    append xml "t='n'><f>[xml_map $f]</f>"
                }
                default {
                    append xml "t='inlineStr'><is><t>[xml_map [dict_getd $col v ""]]</t></is>"
                }
            }
            append xml "</c>"
            incr icol
        }
        append xml "</row>"
        $sNode appendXML $xml
        incr irow
    }
    return [$doc asXML]
}


proc cu::write_xlsx::read_worksheet { data textDict } {
    
    dom parse $data doc
    
    set ns "http://schemas.openxmlformats.org/spreadsheetml/2006/main" 

    set sNode [$doc selectNodes -namespaces [list w $ns] w:worksheet/w:sheetData]
    set valuesList ""
    set values ""
    foreach rowNode [$sNode selectNodes -namespaces [list w $ns] w:row] {
        set row ""
        foreach colNode [$rowNode selectNodes -namespaces [list w $ns] w:c] {
            set col ""
            set s [$colNode @s ""]
            switch $s {
                0 - 1 - 2 - 3 { dict set col s normal }
                1 { dict set col s bold }
            }
            set r [$colNode @r ""]
            if { $r ne "" } {
                dict set col r $r
                lassign [cu::write_xlsx::excel_row_col $r] r c
                dict set col row $r
                dict set col col $c
            }
            switch [$colNode @t] {
                "s" {
                    set v [$colNode selectNodes -namespaces [list w $ns] {string(w:v)}]
                    dict set col t string
                    dict set col v [dict_getd $textDict $v ""]
                }
                "inlineStr" {
                    dict set col t string
                    dict set col v [$colNode selectNodes -namespaces [list w $ns] {string(w:is/w:t)}]
                }
                "n" {
                    set v [$colNode selectNodes -namespaces [list w $ns] {string(w:v)}]
                    set f [$colNode selectNodes -namespaces [list w $ns] {string(w:f)}]
                    if { $v ne "" } {
                        if { $s == 2 } {
                            dict set col t date
                        } elseif { $s == 3 } {
                            dict set col t datetime
                        } else {
                            dict set col t value
                        }
                        dict set col v $v
                    } elseif { $f ne "" } {
                        dict set col t formula
                        dict set col v $f
                    }
                }
            }
            lappend row $col
        }
        lappend values $row
    }
    lappend valuesList $values
    
    #puts [$sNode asXML]

    return $valuesList
}

proc cu::write_xlsx::convert_docbook_to_values { data } {
    
    package require tdom
    dom parse $data doc
    return [convert_docbook_to_valuesDoc $doc]
}

proc cu::write_xlsx::convert_docbook_to_valuesDoc { doc } {
    return [convert_docbook_to_valuesNode [$doc selectNodes {/*}]]
}

proc cu::write_xlsx::date_to_excel_date { date } {
    return [expr {([clock scan $date -timezone :UTC]-[clock scan "1899-12-30" -timezone :UTC])/3600.0/24.0}]
}

proc cu::write_xlsx::date_to_excel_dateF { format date } {
    return [expr {([clock scan $date -format $format -timezone :UTC]-
            [clock scan "1899-12-30" -timezone :UTC])/3600.0/24.0}]
}

proc cu::write_xlsx::convert_docbook_to_valuesNode { domNode } {

    set values ""
    foreach node [$domNode selectNodes {*}] {
        switch [$node nodeName] {
            section {
                set valuesL [convert_docbook_to_valuesNode $node]
                lappend values {*}$valuesL
            }
            para {
                if { [$node selectNodes {table}] ne "" } {
                    set valuesL [convert_docbook_to_valuesNode $node]
                    lappend values {*}$valuesL
                } else {
                    if { [$node selectNodes {emphasis[@role="strong"]}] ne "" } {
                        set s bold
                    } else {
                        set s normal
                    }
                    set v [$node selectNodes {string(.)}]
                    lappend values [list [list s $s t string v $v]]
                }
            }
            table {
                set dates [dict create \
                    "%Y-%m-%d" date \
                    "%Y-%m-%d %H:%M" datetime \
                    "%Y-%m-%d %H:%M:%S" datetime \
                    ]
                
                set row ""
                foreach rowNode [$node selectNodes {row|tgroup/theader/row}] {
                    set row ""
                    foreach entryNode [$rowNode selectNodes entry] {
                        if { [$entryNode selectNodes {.//emphasis[@role="strong"]}] ne "" } {
                            set s bold
                        } else {
                            set s normal
                        }
                        set v [string trim [$entryNode selectNodes {string(.)}]]
                        
                        set t ""
                        foreach "format type" $dates {
                            set err [catch { date_to_excel_dateF $format $v } ret]
                            if { !$err } {
                                set v $ret
                                set t $type
                                break
                            }
                        }
                        if { $t eq "" } {
                            if { [string is double -strict $v] } {
                                set t value
                            } else {
                                set t string
                            }
                        }
                        lappend row [list s $s t $t v $v]
                    }
                    lappend values $row
                }
            }
        }
    }
    return $values        
}

proc cu::write_xlsx::write { file values } {
    variable template
    
    package require tdom
    package require customLib_utils::c

    set template [cu::file::give_native_file customLib_utils $template]
    set files [cu::unzipfile -data -encoding utf-8 $template]
    set files_new ""
    foreach i $files {
        lassign $i f mtime data comment
        if { $f eq "xl/worksheets/sheet1.xml" } {
            set data [update_worksheet $data $values]
        }
        lappend files_new [list $f $data $comment]
    }
    cu::zipfile -compression 9 -data -encoding utf-8 $file {*}$files_new
}

proc cu::write_xlsx::read { file } {

    set valuesList ""    
    set files [cu::unzipfile -data -encoding utf-8 $file]
    
    set found 0
    foreach i $files {
        lassign $i f mtime data comment
        if { $f eq "xl/sharedStrings.xml" } {
            set found 1
            break
        }
    }
    if { $found } {
        dom parse $data doc
        set ns "http://schemas.openxmlformats.org/spreadsheetml/2006/main" 
        set num 0
        set textDict ""
        foreach domNode [$doc selectNodes -namespaces [list w $ns] {/*/w:si/w:t}] {
            dict set textDict $num [$domNode selectNodes {string(.)}]
            incr num
        }
    }
    
    foreach i $files {
        lassign $i f mtime data comment
        if { $f eq "xl/worksheets/sheet1.xml" } {
            lappend valuesList {*}[read_worksheet $data $textDict]
        }
    }
    return $valuesList
}

proc cu::write_xlsx::check_modify_callback { file mtime modify_callback } {
 
    if { ![file exists $file] } {
        return
    }
    if { [file mtime $file] > $mtime } {
        set err [catch { uplevel #0 $modify_callback [list [cu::write_xlsx::read $file]] } ret]
        if { $err } {
            return
        }
        set mtime [file mtime $file]
    }
    after 1000 [list cu::write_xlsx::check_modify_callback $file $mtime $modify_callback]
}

proc cu::write_xlsx::write_open_temp { args } {
    
    set optional {
        { -modify_callback cmdList "" }
    }
    set compulsory "values"
    parse_args $optional $compulsory $args

    set file [cu::file::tempfile spreadsheet .xlsx]
    write $file $values
    cu::file::execute start $file
    
    if { $modify_callback ne "" } {
        set mtime [file mtime $file]
        check_modify_callback $file $mtime $modify_callback
    }
    return $file
}

if 0 {
    set values [list "" ""]
    lappend values [list "" "s bold t string v {List of values}"]
    lappend values [list "" "s normal t string v {value1}" "t value v 23.2"]
    lappend values [list "" "s normal t string v {value2}" "t value v 42.1"]
    lappend values [list "" "s bold t string v {TOTAL}" "s bold t formula v {R-2C0+R-1C0}"]
    lappend values [list "" "s bold t string v {TOTAL PROD}" "s bold t formula v {C4*C5}"]
    set file [cu::write_xlsx::write_open_temp $values]
    puts [cu::write_xlsx::read $file]
}

################################################################################
#    cu::string
################################################################################

namespace eval cu::string {
    variable unique_id 0
}

proc cu::string::give_unique_id { { prefix uid } } {
    variable unique_id
    return $prefix[incr unique_id]
}

proc cu::string::maintain_case { word case_word } {
    if { $word ne [string tolower $word] } { return $word }
    if { $case_word eq [string toupper $case_word] } { return [string toupper $word] }
    set cw0 [string index $case_word 0]
    if { $cw0 eq [string toupper $cw0] } { return [string totitle $word] }
    return $word
}

proc cu::string::csv_join {values {sepChar ,} {delChar \"}} {
    set out ""
    set sep {}
    foreach val $values {
        if {[string match "*\[${delChar}$sepChar\]*" $val]} {
            append out $sep${delChar}[string map [list $delChar ${delChar}${delChar}] $val]${delChar}
        } else {
            append out $sep${val}
        }
        set sep $sepChar
    }
    return $out
}

proc cu::string::compare_list { list } {
    set ret ""
    foreach i $list {
        lappend ret [compare_string $i]
    }
    return $ret
}

proc cu::string::compare_string { str } {
    
    set groups [list a\ue1\ue2\ue3\ue4\ue5 e\ue8\ue9\uea\ueb \
            i\uec\ued\uee\uef o\uf2\uf3\uf4\uf5\uf6 u\ufa\ufb\ufc n\uf1]
    
    set maps ""
    foreach group $groups {
        set rest [lassign [split $group ""] base]
        foreach i $rest {
            lappend maps $i $base
        }
    }
    return [string map $maps [string tolower $str]]
}
