

package provide customLib_utils::tk 1.20

namespace eval cu {      
    variable topdir [file normalize [file dirname [info script]]]
    variable init_tile_styles_done
    variable add_down_arrow_to_image_delta 7
    variable inside_gid 0       
    variable icons_lib_db
    variable get_images_dirs   
}

source [file join $cu::topdir megawidgets.tcl]

################################################################################
#    cu::adapt_text_length
################################################################################

# remember to grid the label to fill all space. For example with -sticky ew
proc cu::adapt_text_length { args } {
    foreach w $args {
	bind $w <Configure> [list cu::_adapt_text_length_do $w]
    }
}

proc cu::_adapt_text_length_do { w } {
    if { [winfo width $w] > 1 } {
	$w configure -wraplength [winfo width $w] -justify left
    }
}

################################################################################
#    add_contextual_menu_to_entry
################################################################################

proc cu::add_contextual_menu_to_entry { w what args } {
    switch $what {
	init {
	    bind $w <ButtonRelease-3> [list cu::add_contextual_menu_to_entry $w post %X %Y]
	}
	post {
	    lassign $args x y
	    set menu $w.menu
	    catch { destroy $menu }
	    menu $menu -tearoff 0
	    foreach i [list cut copy paste --- select_all --- clear] \
		txt [list [_ "Cut"] [_ "Copy"] [_ "Paste"] --- [_ "Select all"] --- [_ "Clear"]] {
		if { $i eq "---" } {
		    $menu add separator
		} else {
		    $menu add command -label $txt -command [list cu::add_contextual_menu_to_entry $w $i]
		}
	    }
	    tk_popup $menu $x $y
	}
	clear {
	    if { [winfo class $w] eq "Text" } {
		$w delete 1.0 end
	    } else {
		$w delete 0 end
	    }
	}
	cut {
	    event generate $w <<Cut>>
	}
	copy {
	    event generate $w <<Copy>>
	}
	paste {
	    event generate $w <<Paste>>
	}
	select_all {
	    if { [winfo class $w] eq "Text" } {
		$w tag add sel 1.0 end-1c
	    } else {
		$w selection range 0 end
	    }
	}
    }
}

################################################################################
#     cu::give_manager_parent
################################################################################

proc cu::give_manager_parent { w } {
    switch [winfo manager $w] {
	"grid" {
	    set w [dict get [grid info $w] -in]
	}
	"pack" {
	    set w [dict get [pack info $w] -in]
	}
	"canvas" - "treectrl" {
	    switch [winfo manager $w] {
		"canvas" { set class "Canvas" }
		"treectrl" { set class "TreeCtrl" }
	    }
	    if { [winfo class [winfo parent $w]] eq $class } {
		set w [winfo parent $w]
	    } else {
		set found 0
		foreach i [winfo children [winfo parent $w]] {
		    if { $i eq $w } { continue }
		    if { [winfo class $i] eq $class } {
		        set w $i
		        set found 1
		        break
		    }
		}
		if { !$found } {
		    set w [winfo parent $w]
		}
	    }
	}
	default {
	    set w [winfo parent $w]
	}
    }
    return $w
}

################################################################################
#     cu::text_entry_bindings
################################################################################

proc cu::text_entry_bindings { w } {

    if { ![info exists ::control] } {
	if { $::tcl_platform(platform) eq "windows" } {
	    set ::control Control
	} elseif { [tk windowingsystem] eq "aqua" } {
	    set ::control Command
	} else {
	    set ::control Control
	}
    }
    # "backslash" and "c" are here to help with a problem in Android VNC
    bind $w <$::control-backslash> "[list cu::text_entry_insert $w];break"
    bind $w <$::control-less> "[list cu::text_entry_insert $w];break"
    foreach "acc1 acc2 c" [list plus "" {[]} c "" {{}} ccedilla "" {{}} 1 "" || 1 1 \\ 3 "" {#}] {
	set cmd "[list cu::text_entry_insert $w $c];break"
	if { $acc2 eq "" } {
	    set k2 ""
	} else {
	    set k2 <KeyPress-$acc2>
	}
	bind $w <$::control-less><KeyPress-$acc1>$k2 $cmd
	bind $w <$::control-backslash><KeyPress-$acc1>$k2 $cmd
    }
    
    if { $::tcl_platform(platform) ne "windows" } {
	foreach "ev k" [list braceleft \{ braceright \} bracketleft \[ bracketright \] backslash \\ \
		bar | at @ numbersign # asciitilde ~] {
	    # EuroSign €
	    # they are class bindings so as search in text widgets can continue working
	    bind Text <$ev> "[list tk::TextInsert %W $k]; break"
	    bind TEntry <$ev> "[list ttk::entry::Insert %W $k]; break"
	    bind TCombobox <$ev> "[list ttk::entry::Insert %W $k]; break"
	    bind Entry <$ev> "[list tk::EntryInsert %W $k]; break"
	}
    }
}

proc cu::text_entry_insert { w { what "" } } {
    variable last_text_enty_bindings
    
    if { ![info exists last_text_enty_bindings] } {
	set last_text_enty_bindings ""
    }
    set list [list "{}" "\[\]" "||" "\\" "#"]
    set t [clock milliseconds]
    lassign [dict_getd $last_text_enty_bindings $w ""] time d
    
    if { $d eq "" } { set d "{}" }
    if { $time ne "" && $t < $time+3000 } {
	if { [winfo class $w] eq "Text" } {
	    set idx [$w search $d insert-1c]
	    if { [$w compare $idx == insert-1c] } {
		if { [string length $d] == 1 } {
		    $w delete insert-1c
		} else {
		    $w delete insert-1c insert+1c
		}
	    }
	} else {
	    set idx [$w index insert]
	    if { $idx > 0 } {
		set idx1 [expr {$idx-1}]
		set idx2 [expr {$idx-1+[string length $d]}]
		set txt [string range [$w get] $idx1 $idx2]
		if { [string equal $d $txt] } {
		    $w delete $idx1 $idx2
		}
	    }
	}
	if { $what eq "" } {
	    set ipos [lsearch -exact $list $d]
	    incr ipos
	    if { $ipos >= [llength $list] } {
		set ipos 0
	    }
	    set d [lindex $list $ipos]
	}
    }
    if { $what ne "" } {
	set d $what
    }
    if { [winfo class $w] eq "Text" } {
	set idx [$w index insert]
	$w insert insert $d
	$w mark set insert "$idx+1c"
    } else {
	set idx [$w index insert]
	$w insert insert $d
	$w icursor [expr {$idx+1}]
    }
    dict set last_text_enty_bindings $w [list $t $d]
}

################################################################################
#     cu::text operations on text widget
################################################################################

namespace eval cu::text {}

proc cu::text::get_selection_or_word { args } {
    
    set optional {
	{ -return_range boolean 0 }
    }
    set compulsory "text idx"
    parse_args $optional $compulsory $args
    
    set range [$text tag ranges sel]
    if { $range != "" && [$text compare [lindex $range 0] <= $idx] && \
	[$text compare [lindex $range 1] >= $idx] } {
	if { $return_range } {
	    return $range
	} else {
	    return [$text get {*}$range]
	}
    } else {
	if { $idx != "" } {
	    set var ""
	    set idx0 $idx
	    set char [$text get $idx0]
	    if { [regexp {[\s,;]} $char] } {
		set c [$text get "$idx0-1c"]
		if { [string is wordchar $c] } {
		    set idx [$text index "$idx0-1c"]
		    set idx0 $idx
		    set char [$text get $idx0]
		}
	    }
	    while { [string is wordchar $char] } {
		#  || $char == "(" || $char == ")"
		set var $char$var
		set idx0 [$text index $idx0-1c]
		if { [$text compare $idx0 <= 1.0] } { break }
		set char [$text get $idx0]
	    }
	    set idx1 [$text index $idx+1c]
	    set char [$text get $idx1]
	    while { [string is wordchar $char] } {
		#  || $char == "(" || $char == ")"
		append var $char
		set idx1 [$text index $idx1+1c]
		if { [$text compare $idx1 >= end-1c] } { break }
		set char [$text get $idx1]
	    }
	    if { ![regexp {[^()]*\([^\)]+\)} $var] } {
		set var [string trimright $var "()"]
	    }
	} else { set var "" }
    }
    if { $return_range } {
	return [list [$text index "$idx0+1c"] [$text index "$idx1"]]
    } else {
	return $var
    }
}

################################################################################
#    cu::set_window_geometry u::give_window_geometry
################################################################################

proc cu::give_window_geometry { w } {

    regexp {(\d+)x(\d+)([-+])([-\d]\d*)([-+])([-\d]+)} [wm geometry $w] {} width height m1 x m2 y
    if { $::tcl_platform(platform) eq "unix" } {
	# note: this work in ubuntu 9.04
	incr x -4
	incr y -24
    }
    return ${width}x$height$m1$x$m2$y
}

proc cu::set_window_geometry { w geometry } {

    if { ![regexp {(\d+)x(\d+)([-+])([-\d]\d*)([-+])([-\d]+)} $geometry {} width height m1 x m2 y] } {
	regexp {(\d+)x(\d+)} $geometry {} width height
	lassign [list 0 0 + +] x y m1 m2
    }
    if { $x < 0 } { set x 0 }
    if { $y < 0 } { set y 0 }
    if { $x > [winfo screenwidth $w]-100 } { set x [expr {[winfo screenwidth $w]-100}] }
    if { $y > [winfo screenheight $w]-100 } { set y [expr {[winfo screenheight $w]-100}] }
    
    if { $m2 eq "+" && $y+$height > [winfo screenheight $w] } {
	if { $y > 0.5*[winfo screenheight $w] } {
	    set y [expr {round(0.5*[winfo screenheight $w])}]
	}
	set height [expr {[winfo screenheight $w]-$y}]
    }
    wm geometry $w ${width}x$height$m1$x$m2$y
}

proc cu::create_tooltip_toplevel { args } {

    set optional {
	{ -withdraw "" 0 }
    }
    set compulsory "b"
    parse_args $optional $compulsory $args

    toplevel $b -class Tooltip
    if { $withdraw } {
	wm withdraw $b
    }
    if {[tk windowingsystem] eq "aqua"} {
	::tk::unsupported::MacWindowStyle style $b help none
    } else {
	wm overrideredirect $b 1
    }
    catch {wm attributes $b -topmost 1}
    # avoid the blink issue with 1 to <1 alpha on Windows
    catch {wm attributes $b -alpha 0.99}
    wm positionfrom $b program
    if { [tk windowingsystem]  eq "x11" } {
	set focus [focus]
	focus -force $b
	raise $b
	if { $focus ne "" } {
	    after 100 [list catch [list focus -force $focus]]
	}
    }
    return $b
}

proc cu::give_widget_background { w } {
 
    set err [catch { $w cget -background } bgcolor]
    if { $err } {
	set err [catch {
		set style [$w cget -style]
		if { $style eq "" } {
		    set style [winfo class $w]
		}
		set bgcolor [ttk::style lookup $style -background]
	    }]
	if { $err } {
	    if { $::tcl_platform(platform) eq "windows" } {
		set bgcolor SystemButtonFace
	    } else {
		set bgcolor grey
	    }
	}
    }
   return $bgcolor
}

proc cu::_do_scroll { w d } {
    if { [tk windowingsystem] eq "aqua" } {
	$w yview scroll [expr {-1*$d}] units
    } elseif { ![catch { package vcompare [package provide Tk] 8.5 } ret] && $ret < 0} {
	$w yview scroll [expr {- ($d / 120) * 4}] units
    } elseif { [winfo class $w] in "Canvas TreeCtrl" } {
	$w yview scroll [expr {- ($d / 120) * 4}] units
    } else {
	$w yview scroll [expr {-$d/5}] pixels
    }
}

proc cu::give_manager_parent { w } {
    switch [winfo manager $w] {
	"grid" {
	    set w [dict get [grid info $w] -in]
	}
	"pack" {
	    set w [dict get [pack info $w] -in]
	}
	"canvas" - "treectrl" {
	    switch [winfo manager $w] {
		"canvas" { set class "Canvas" }
		"treectrl" { set class "TreeCtrl" }
	    }
	    if { [winfo class [winfo parent $w]] eq $class } {
		set w [winfo parent $w]
	    } else {
		set found 0
		foreach i [winfo children [winfo parent $w]] {
		    if { $i eq $w } { continue }
		    if { [winfo class $i] eq $class } {
		        set w $i
		        set found 1
		        break
		    }
		}
		if { !$found } {
		    set w [winfo parent $w]
		}
	    }
	}
	default {
	    set w [winfo parent $w]
	}
    }
    return $w
}

proc cu::normalize_color { color } {
    lassign [winfo rgb . $color] r g b
    return [format "#%02x%02x%02x" [expr {$r/256}] [expr {$g/256}] [expr {$b/256}]]
}

################################################################################
#    add_down_arrow_to_image
################################################################################

proc cu::add_down_arrow_to_image { args } {
    variable add_down_arrow_to_image_delta
    
    set optional {
	{ -color color black }
	{ -w widget "" }
	{ -delta value "" }
    }
    set compulsory "img"
    parse_args $optional $compulsory $args

    set color [cu::normalize_color $color]
    
    if {![info exists add_down_arrow_to_image_delta] } {
	set add_down_arrow_to_image_delta 7
    }
    if { $delta eq "" } {
	set delta $add_down_arrow_to_image_delta
    }
    if { $img ne "" } {
	set width [image width $img]
	set height [image height $img]
    } else {
	lassign [list 0 16] width height
    }
    set new_img ${img}::down_arrow[expr {$width+$delta}]
    if { [info command $new_img] ne "" } {
	return $new_img
    }
    image create photo $new_img -width [expr {$width+$delta}] -height $height
    if { $img ne "" } { $new_img copy $img -to 0 0 }
    set coords {
	-3 -1
	-4 -2 -3 -2 -2 -2
	-5 -3 -4 -3 -3 -3 -2 -3 -1 -3
    }
    foreach "x y" $coords {
	$new_img put $color -to [expr {$width+$delta+$x}] [expr {$height+$y}]
    }
#     if { $w ne "" } {
#         $w configure -image $new_img
#         bind $w <Destroy> +[list image delete $new_img]
#     }
    return $new_img
}

################################################################################
#    clipboard
################################################################################

namespace eval cu::img {
    variable registered_clipboard_types ""
}

proc cu::img::clipboard_unix { what args } {
    variable registered_clipboard_types
    
    switch -- $what {
	clear  {
	    ::clipboard clear
	}
	append {
	    set optional {
		{ -type type "" }
		{ -format format "" }
		{ -displayof window . }
	    }
	    set compulsory "data"
	    parse_args $optional $compulsory $args

	    switch $type {
		tclimage {
		    set data [binary decode base64 [$data data -format png]]
		    ::clipboard append  -displayof $displayof -type image/png $data
		}
		default {
		    ::clipboard append {*}$args
		}
	    }
	}
	get {
	    set optional {
		{ -type type "" }
		{ -displayof window . }
	    }
	    set compulsory ""
	    parse_args $optional $compulsory $args
	    
	    switch -- $type {
		"HTML Format" {
		    if { "text/html" ni [::clipboard get -type TARGETS] } {
		        error "error in clipboard. Type text/html not available"
		    }
		    set txt [binary format c* [::clipboard get -displayof $displayof -type text/html]]
		    #return [encoding convertfrom unicode $txt]
		    #return $txt
		    return [encoding convertfrom utf-8 $txt]
		}
		tclimage {
		    if { "image/png" ni [::clipboard get -type TARGETS] } {
		        error "error in clipboard. Type image/png not available"
		    }
		    package require img::png
		    set data [::clipboard get -displayof $displayof -type image/png]
		    set err [catch { image create photo -data $data } img]
		    if { $err } {
		        set err [catch { image create photo -data [binary format c* $data] } img]
		    }
		    if { $err } {
		        error "error in clipboard. Contents is not a valid image"
		    }
		    return $img
		}
		tcltext - UTF8_STRING - STRING - "" {
		    if { $type eq "tcltext" } {
		        set type ""
		    }
		    # note: without the type, the string contained utf-8 bytes incorrectly in ubuntu 10.10
		    return [::clipboard get {*}$args -type UTF8_STRING]
		}
		default {
		    if { $type ni [::clipboard get -type TARGETS] } {
		        error "error in clipboard. Type $type not available"
		    }
		    if { [dict exists $registered_clipboard_types $type] } {
		        return [::clipboard get {*}$args]
		    } else {
		        return [binary format c* [::clipboard get {*}$args]]
		    }
		}
	    }
	}
	register {
	    set optional {
		{ -encoding encoding "" }
	    }
	    set compulsory "name"
	    parse_args $optional $compulsory $args
	    dict set registered_clipboard_types $name $encoding
	}
	isimage {
	    set err [catch { ::clipboard get -type TARGETS } targets]
	    if { $err } { return 0 }
	    return [expr {  "image/png" in $targets }]
	}
	types - alltypes {
	    set types [::clipboard get -type TARGETS]
	    if { "STRING" in $types } {
		set types [linsert $types 0 tcltext]
	    }
	    if { "image/png" in $types } {
		set types [linsert $types 0 tclimage]
	    }
	    return $types
	}
	default {
	    error "error. argument should be clear, append, get, register, isimage, types, alltypes"
	}
    }
}

if { $::tcl_platform(platform) eq "unix" } {
    interp alias "" cu::img::clipboard "" cu::img::clipboard_unix
}

################################################################################
#    init_tile_styles
################################################################################

proc cu::init_tile_styles { args } {
    variable topdir
    variable init_tile_styles_done
    variable inside_gid 
    if { [ info exists ::GIDDEFAULT]} {
	set inside_gid 1
    }
    
    set optional {
	{ -theme blue|none "none" }
    }
    set compulsory ""
    parse_args $optional $compulsory $args

    package require Tk
    if { [info exists init_tile_styles_done] } { return }
    set init_tile_styles_done 1
    
    if { [info exists ::ispocket] } {
	set ispocket $::ispocket
    } elseif { $::tcl_platform(os) eq "Windows CE" } {
	set ispocket 1
    } else {
	set ispocket 0
    }
    if { !$inside_gid } {
	if { [ tk windowingsystem] eq "x11" || $ispocket } {
	    set err [catch {
		ttk::style theme use clam
		#                 package require ttk::theme::tilegtk
		#                 ttk::style theme use tilegtk
	    }]
	    if { $err } {
		ttk::style theme use clam
	    }
	}
	if { "clam" in [ttk::style theme names] } {
	    ttk::style theme settings clam {
		ttk::style configure TButton -padding 1
		ttk::style configure TMenubutton -padding 1
		ttk::style map Toolbutton -background "focus grey [ttk::style map Toolbutton -background]"
		ttk::style map TCombobox -foreground "disabled #999999 [ttk::style map TCombobox -foreground]"
		ttk::style map TCombobox -fieldbackground "disabled #d9d9d9 [ttk::style map TCombobox -fieldbackground]"
	    }
	}
	if { "xpnative" in [ttk::style theme names] } {
	    ttk::style theme settings xpnative {
		ttk::style map TMenubutton -background "focus grey [ttk::style map Toolbutton -background]"
	    }
	}
	
	if { $theme eq "blue" } {
	    # this is the part that keeps blue color for forms
	    
	    set color #e4e8ed            
	    set color_dark #e4e8ed
	    set color_darker #818486
	    
	    foreach i [list TLabel TCheckbutton TRadiobutton \
		           TCombobox TFrame TLabelframe TNotebook \
		           TLabelframe.Label TSizegrip Toolbutton] {
		ttk::style configure $i -background $color
		ttk::style map $i -background [list disabled $color]
	    }
	    foreach i [list TButton TMenubutton TCombobox TScrollbar] {
		ttk::style configure $i -background $color_dark
		ttk::style map $i -background [list disabled $color_dark]
	    }
	    ttk::style map TCombobox -fieldbackground [list readonly $color_dark]
	
	    foreach i [list TLabelframe Frame Label Checkbutton Panedwindow Menu] {
		option add *$i.Background $color
	    }
	    option add *Menu.Background $color
	    option add *Menu.activeForeground $color_darker
	    option add *Menu.selectColor $color_darker
	}
    } ;# !inside_gid
    
    image create photo wntoolbar_toolbar_bg -file [file join $topdir images wntoolbar_toolbar_bg.gif]
    image create photo wntoolbar_button_bg -file [file join $topdir images wntoolbar_button_bg.gif]
    image create photo wntoolbar_button_bg_pressed -file [file join $topdir images wntoolbar_button_bg_pressed.gif]

    image create photo wntoolbar_toolbar_bgH -file [file join $topdir images wntoolbar_toolbar_bgH.gif]
    image create photo wntoolbar_button_bgH -file [file join $topdir images wntoolbar_button_bgH.gif]
    
    image create photo wntoolbar_toolbar_bgG -file [file join $topdir images wntoolbar_toolbar_bgG.gif]
    image create photo wntoolbar_button_bgG -file [file join $topdir images wntoolbar_button_bgG.gif]
    image create photo wntoolbar_button_bgG_pressed -file [file join $topdir images wntoolbar_button_bgG_pressed.gif]

    set err [catch {
	    ttk::style element create WNtoolbar.background image wntoolbar_toolbar_bg \
		-border "2 30 2 0" -sticky nsew
    } errstring]
    if { $err } {
	return -code error -errorinfo $::errorInfo $errstring
    }
    ttk::style element create WNtoolbarH.background image wntoolbar_toolbar_bgH \
		-border "30 2 0 2" -sticky nsew
    
    ttk::style layout WNtoolbar { WNtoolbar.background }
    ttk::style layout WNtoolbarH { WNtoolbarH.background }
    
    #catch { ttk::style default WNtoolbar -padding "14 1 1 1" }
    catch { ttk::style configure WNtoolbar -padding "14 1 1 1" }
    catch { ttk::style configure WNtoolbarH -padding "1 14 1 1" }
    
    ttk::style element create WNbutton.background image \
	[list wntoolbar_button_bg active wntoolbar_button_bg_pressed pressed wntoolbar_button_bg_pressed \
	    selected wntoolbar_button_bg_pressed] \
	-sticky nsew -width 0 -height 0 -border "2 0 2 0"
    
    ttk::style element create WNbuttonG.background image \
	[list wntoolbar_button_bgG active wntoolbar_button_bgG_pressed pressed wntoolbar_button_bgG_pressed \
	    selected wntoolbar_button_bgG_pressed] \
	-sticky nsew -width 0 -height 0 -border "2 0 2 0"
    
    ttk::style element create WNbuttonH.background image \
	[list wntoolbar_button_bgH active wntoolbar_button_bg_pressed pressed wntoolbar_button_bg_pressed \
	    selected wntoolbar_button_bg_pressed] \
	-sticky nsew -width 0 -height 0 -border "0 2 0 2"
    
    # if { [ tk windowingsystem] eq "x11" } {
    #         ttk::style layout WNbutton [ttk::style layout TButton]
    #         ttk::style configure WNbutton -background #bfe4df
    # } else {
    #         ttk::style layout WNbutton {
    #             WNbutton.background -sticky nsew
    #             Toolbutton.border -sticky nswe -children {
    #                 Toolbutton.padding -sticky nswe -children {
    #                     Toolbutton.label -sticky nswe
    #                 }
    #             }
    #         }
    # }
    
    
    ttk::style layout WNbutton {
	WNbutton.background -sticky nsew
	Toolbutton.border -sticky nswe -children {
	    Toolbutton.padding -sticky nswe -children {
		Toolbutton.label -sticky nswe
	    }
	}
    }
    ttk::style layout WNbuttonH {
	WNbuttonH.background -sticky nsew
	Toolbutton.border -sticky nswe -children {
	    Toolbutton.padding -sticky nswe -children {
		Toolbutton.label -sticky nswe
	    }
	}
    }
    ttk::style layout WNlabel {
	WNbutton.background -sticky nsew
	Label.border -sticky nswe -border 1 -children {
	    Label.padding -sticky nswe -border 1 -children {Label.label -sticky nswe}
	}
    }
    
    ttk::style layout TEntry.warning [ttk::style layout TEntry]
    ttk::style configure TEntry.warning -foreground red
    
    image create photo wnsizegrip_bg -width 1 -height 16
    wnsizegrip_bg copy wntoolbar_toolbar_bg -from 15 10
    ttk::style element create WNSizegrip.background image wnsizegrip_bg \
		-sticky nsew
    
    ttk::style layout WNSizegrip {
	WNSizegrip.background -sticky nsew
	Sizegrip.sizegrip -side bottom -sticky se
    }
    #option add *TSizegrip*Style WNSizegrip
    
    set l "WNbutton.background -sticky nsew\n[ttk::style layout TMenubutton]"
    ttk::style layout WNmenubutton $l
    
    set file [file join $topdir images search.gif]
    image create photo search1 -file $file -format "gif -index 0"
    image create photo search2 -file $file -format "gif -index 1"
    
    ttk::style element create WNsearch.field image \
	[list search1 focus search2] \
	-border {22 7 14} -sticky ew -padding "22 0 0 0"
    
    ttk::style layout WNsearch.entry {
	WNbutton.background -sticky nsew
	WNsearch.field -sticky nswe -border 1 -children {
	    Entry.padding -sticky nswe -children {
		Entry.textarea -sticky nswe
	    }
	}
    }
   
    ttk::style layout TSizegripWN [ttk::style layout TSizegrip]
    ttk::style configure TSizegripWN -background #bfe4df

    ttk::style layout SmallToolbutton [ttk::style layout Toolbutton]
    ttk::style configure SmallToolbutton {*}[ttk::style configure Toolbutton] -padding 0

    ttk::style layout SmallToolbuttonGreen [ttk::style layout Toolbutton]
    ttk::style configure SmallToolbuttonGreen {*}[ttk::style configure Toolbutton] -padding 0 -background green
  
    ttk::style layout TCheckbutton.small [ttk::style layout TCheckbutton]
    ttk::style configure TCheckbutton.small {*}[ttk::style configure Toolbutton] -padding 0 \
	-font TkSmallCaptionFont
    
    if { !$inside_gid } {
	if { "aqua" in [ttk::style theme names] } {
	    ttk::style theme settings aqua {    
		ttk::style configure Toolbutton -padding 0
	    }
	}
	
	option add *Text*Background white
    } ;# !inside_gid
    option add *Multiline_entry*Background white
    option add *Wordwidget_and_toolbox*Background white

    setup_generic_bindings
}

proc cu::get_images_add_dir { dir } {
    variable get_images_dirs
    
    if { ![info exists get_images_dirs] } {
	set get_images_dirs ""
    }
    
    set dir [file normalize $dir]
    if { [lsearch -exact $get_images_dirs $dir] == -1 } {
	lappend get_images_dirs $dir
    }
    return $get_images_dirs
}

################################################################################
#    generic bindings
################################################################################

proc cu::setup_generic_bindings {} {
    if { [tk windowingsystem] eq "x11" } {
	bind Text <4> ""
	bind Text <5> ""
	bind Canvas <4> ""
	bind Canvas <5> ""

	bind all <4> {
	    set w %W
	    for { set i 0 } { $i < 10 } { incr i } {
		if { [winfo class $w] eq "Toplevel" || $w eq "." } {
		    break
		}
		if { [winfo class $w] in "Text Canvas TreeCtrl" && [$w yview] ne "0.0 1.0" } {
		    $w yview scroll -1 units
		    break
		}
		set w [cu::give_manager_parent $w]
	    }
	}
	bind all <5> {
	    set w %W
	    for { set i 0 } { $i < 10 } { incr i } {
		if { [winfo class $w] eq "Toplevel" || $w eq "." } {
		    break
		}
		if { [winfo class $w] in "Text Canvas TreeCtrl" && [$w yview] ne "0.0 1.0" } {
		    $w yview scroll 1 units
		    break 
		}
		set w [cu::give_manager_parent $w]
	    }
	}
    }
    
    bind Text <MouseWheel> "[list cu::_do_scroll %W %D]; break"
    bind Canvas <MouseWheel> "[list cu::_do_scroll %W %D]; break"
    bind TCombobox <MouseWheel> ""
    
    bind all <MouseWheel> {
	set w [winfo containing %X %Y]
	for { set i 0 } { $i < 10 } { incr i } {
	    if { $w eq "" } {
		return
	    }
	    if { [winfo class $w] eq "Toplevel" || $w eq "." } {
		break
	    }
	    if { [winfo class $w] in "Text Canvas TreeCtrl"  && [$w yview] ne "0.0 1.0" } {
		cu::_do_scroll $w %D
		break
	    }
	    set w [cu::give_manager_parent $w]
	}
    }
    
    if { $::tcl_platform(platform) eq "windows" } {
	event add <<ContextualPress>> <ButtonPress-3>
	event add <<Contextual>> <ButtonRelease-3>
	event add <<Contextual>> <App>
	set ::control Control
	set ::control_txt Ctrl
    } elseif { [tk windowingsystem] eq "aqua" } {
	event add <<ContextualPress>> <ButtonPress-2>
	event add <<Contextual>> <ButtonRelease-2>
	set ::control Command
	set ::control_txt Command
	
	foreach ev [bind Text] {
	    if { [regsub {Control} $ev {Command} evC] } {
		bind Text $evC [bind Text $ev]
	    }
	}    
    } else {
	event add <<ContextualPress>> <ButtonPress-3>
	event add <<Contextual>> <ButtonRelease-3>
	set ::control Control
	set ::control_txt Ctrl
    }
}

# GiD change
proc cu::scale_image_2_hi_res { image_name} {
    variable inside_gid 
    if { $inside_gid} {
	set image_name [ gid_themes::HighResolutionImage $image_name]
    }
    return $image_name
}
# end GiD change

proc cu::get_image { args } {
    variable get_images_dirs
    
    set optional {
	{ -contents boolean 0 }
	{ -fileName boolean 0 }
	{ -width width "" }
	{ -height height "" }
    }
    set compulsory "name"
    parse_args $optional $compulsory $args
    
    if { $name eq "" } {
	error "image with void name not valid"
    }
    if { $width ne "" && [info command ::cu::img::zoom] ne "" } {
	# GiD change
	set image_name ::cu::images::$name-$width
	if { [info command $image_name] eq "" } {
	    package require customLib_utils::img
	    set img [get_image $name]
	    if { $height eq "" } {
		set height [expr {$width*[image height $img]/[image width $img]}]
	    }
	    image create photo $image_name -width $width -height $height
	    cu::img::zoom $image_name $img Lanczos3
	}
	return [ cu::scale_image_2_hi_res $image_name]
	# end GiD change
    }
    if { $fileName } {
	# nothing
    } elseif { !$contents } {
	if { [info command ::cu::images::$name] ne "" } {
	    # GiD change: only next line
	    return [ cu::scale_image_2_hi_res ::cu::images::$name]
	}
    } else {
	if { [info exists ::cu::images_contents::$name] } {
	    # GiD change: only next line
	    return [ cu::scale_image_2_hi_res [ set ::cu::images_contents::$name]]
	}
    }
    if { ![info exists get_images_dirs] } {
	set get_images_dirs ""
    }
    if { [file pathtype $name] eq "absolute" } {
	set file $name
    } else {
	set found 0
	foreach dir $get_images_dirs {
	    set ret [glob -nocomplain -dir $dir $name]
	    if { [llength $ret] > 0 } {
		set found 1
		break
	    }
	    set ret [glob -nocomplain -dir $dir $name.*]
	    if { [llength $ret] > 0 } {
		set found 1
		break
	    }
	}
	if { !$found } {
	    error "image '$name' not found in '$get_images_dirs'"
	}
	set file [lindex $ret 0]
    }
    if { $fileName } {
	return $file
    } elseif { !$contents } {
	image create photo ::cu::images::$name -file $file
	# GiD change: only next line
	return [ cu::scale_image_2_hi_res ::cu::images::$name]
    } else {
	set fin [open $file rb]
	set ::cu::images_contents::$name [read $fin]
	close $fin
	# GiD change: only next line
	return [ cu::scale_image_2_hi_res [ set ::cu::images_contents::$name]]
    }
}

################################################################################
#    icons
################################################################################

# program_name is to store a copy of the database (if necessary) in a program
# dependent dir

namespace eval cu::icons {}
namespace eval cu::icons_contents {}

proc cu::init_icons_lib { dbfile program_name } {
    variable icons_lib_db

    set dbfile [file normalize $dbfile]
    
    package require sqlite3
    # temporally disconnected: it crashed sometimes in Windows
    if {0 && [info command ::memvfs::create] ne "" } {
	memvfs::create [file tail $dbfile] [cu::file::get $dbfile]
	set dbfile "file:[file tail $dbfile]?vfs=memvfs"
    } else {
	if { [lindex [file system $dbfile] 0] ne "native" } {
	    set nfile [file join [cu::file::appdatadir $program_name] \
		    [file tail $dbfile]]
	    if { ![file exists $nfile] || [file mtime $nfile] < [file mtime $dbfile] } {
		file copy -force $dbfile $nfile
	    }
	    set dbfile $nfile
	}
    }
    if { ![info exists icons_lib_db] } {
	set idx 1
	while { [info command ::cu::icons_lib_db$idx] ne "" } { incr idx }
	set icons_lib_db ::cu::icons_lib_db$idx
	sqlite3 $icons_lib_db ":memory:"
    }
    foreach "num name file" [$icons_lib_db eval { pragma database_list }] {
	if { $name in "main temp" } { continue }
	if { $name eq $dbfile } {
	    return
	}
    }
    $icons_lib_db eval { attach database $dbfile as $dbfile }
    #puts [$icons_lib_db eval { pragma database_list }]
}

proc cu::is_icons_lib_init {} {
    variable icons_lib_db
    return [info exists icons_lib_db]
}

proc cu::get_icon { args } {
    variable icons_lib_db
    
    set optional {
	{ -contents boolean 0 }
    }
    set compulsory "icon_name"
    parse_args $optional $compulsory $args
    
    if { ![info exists icons_lib_db] } {
	return "icons lib library has not been initialized"
    }

    if { $icon_name eq "" } {
	error "icon with void name not valid"
    }
    if { !$contents } {
	if { [info command ::cu::icons::$icon_name] ne "" } {
	    return ::cu::icons::$icon_name
	}
    } else {
	if { [info exists ::cu::icons_contents::$icon_name] } {
	    return [set ::cu::icons_contents::$icon_name]
	}
    }
    set cmdList ""
    foreach "num name file" [$icons_lib_db eval { pragma database_list }] {
	if { $name in "main temp" } { continue }
	lappend cmdList "select data from \"$name\".icons where name=\$icon_name"
    }
    set cmd [join $cmdList { union }]
    set data [$icons_lib_db onecolumn $cmd]
    if { $data eq "" } {
	error "error in cu::get_icon. $icon_name not found"
    }
    if { !$contents } {
	image create photo ::cu::icons::$icon_name -data $data
	return ::cu::icons::$icon_name
    } else {
	set ::cu::icons_contents::$icon_name $data
	return [set ::cu::icons_contents::$icon_name]
    }
}

proc cu::get_all_icon_names {} {
    variable icons_lib_db
    
    return [$icons_lib_db eval { select name from icons order by name }]
}

namespace eval cu::images {}
namespace eval cu::images_contents {}

proc cu::get_image_name_from_fullpath { file } {
    variable get_images_dirs
    
    set fdir [file normalize [file dirname $file]]
    foreach dir $get_images_dirs {
	if { [file normalize $dir] ne $fdir } { continue }
	if { [file exists $file] } {
	    return [file tail $file]
	}
    }
    return ""
}

interp alias "" cu::get_iconF "" cu::get_image_or_icon

proc cu::get_image_or_icon { args } {
    
    set optional {
	{ -contents boolean 0 }
    }
    set compulsory "name"
    parse_args $optional $compulsory $args
    
    set err [catch { get_image -contents $contents $name } img]
    if { !$err } { return $img }
    return [get_icon -contents $contents $name]
}

proc cu::get_image_selected { name } {

    package require fulltktree
    
    get_image $name
    set name_sel $name-selected
    
    if { [info command ::cu::images::$name_sel] ne "" } {
	return ::cu::images::$name_sel
    }
    image create photo ::cu::images::$name_sel
    ::cu::images::$name_sel copy ::cu::images::$name
    imagetint ::cu::images::$name_sel $fulltktree::SystemHighlight 128
    return ::cu::images::$name_sel
}

proc cu::get_image_color { color } {
    
    if { $color ne "" } {
	set color [cu::normalize_color $color]
    }
    set name color_$color
    if { [info command ::cu::images::$name] ne "" } {
	return ::cu::images::$name
    }
    set img [image create photo ::cu::images::$name -width 16 -height 8]
    $img put $color -to 0 0 15 7
    $img put black -to 0 0 15 1
    $img put black -to 0 6 15 7
    $img put black -to 0 0 1 7
    $img put black -to 14 0 15 7

    return ::cu::images::$name
}

proc cu::reset_images_cache {} {
    
    foreach i [info vars ::cu::images::*] {
	catch { image delete $i }
	unset -nocomplain $i
    }
}

proc cu::get_cursors_init { program_name } {
    variable get_cursors_dirs
    variable get_cursors_appname_dir
    
    if { ![info exists get_cursors_dirs] } {
	set get_cursors_dirs ""
    }
    set get_cursors_appname_dir [file normalize [file join \
	    [cu::file::appdatadir $program_name] cursors]]
    set get_cursors_dirs [linsert0 $get_cursors_dirs $get_cursors_appname_dir]
    return $get_cursors_appname_dir

}

proc cu::get_cursors_add_dir { dir } {
    variable get_cursors_dirs
    
    if { ![info exists get_cursors_dirs] } {
	set get_cursors_dirs ""
    }
    
    set dir [file normalize $dir]
    if { [lsearch -exact $get_cursors_dirs $dir] == -1 } {
	lappend get_cursors_dirs $dir
    }
    return $get_cursors_dirs
}

namespace eval cu::cursors {}

# cursors in xwindows can be created with gimp export x11 bitmap
proc cu::get_cursor { name } {
    variable get_cursors_dirs
    variable get_cursors_appname_dir
    
    if { $name eq "" } {
	error "cursor with void name not valid"
    }
    if { [info exists ::cu::cursors::$name] } {
	return [set ::cu::cursors::$name]
    }

    if  { [tk windowingsystem] eq "aqua" } {
	set ::cu::cursors::$name ""
	return [set ::cu::cursors::$name]
    }

    if { ![info exists get_cursors_dirs] } {
	set get_cursors_dirs ""
    }
    if { [file pathtype $name] eq "absolute" } {
	if { $::tcl_platform(platform) eq "windows" } {
	    set ::cu::cursors::$name "[list @$name.cur]"
	} else {
	    set ::cu::cursors::$name "[list @$name.x11cursor black]"
	}
	return [set ::cu::cursors::$name]
    }
    set found 0
    foreach dir $get_cursors_dirs {
	if { $::tcl_platform(platform) eq "windows" } {
	    set file [file join $dir $name.cur]
	} else {
	    set file [file join $dir $name.x11cursor]
	}
	if { [file exists $file] } {
	    if { [lindex [file system $file] 0] ne "native" } {
		file mkdir $get_cursors_appname_dir
		set nfile [file join $get_cursors_appname_dir [file tail $file]]
		if { ![file exists $nfile] || [file mtime $nfile] < [file mtime $file] } {
		    file copy -force $file $nfile
		}
		set file $nfile
	    }
	    set found 1
	    break
	}
    }
    if { !$found } {
	error "cursor '$name' not found in '$get_cursors_dirs'"
    }
    if { $::tcl_platform(platform) eq "windows" } {
	set ::cu::cursors::$name "[list @$file]"
    } else {
	set ::cu::cursors::$name "[list @$file black]"
    }
    return [set ::cu::cursors::$name]
}

################################################################################
#     cu::tooltip
################################################################################

proc cu::tooltip { what w args } {
    global register_popup_idle
    global register_popup_txt
    
    switch $what {
	register {
	    set optional {
		{ -width auto|size auto }
		{ -max_width auto|free|size auto }
	    }
	    set compulsory "txt"
	    set args [parse_args $optional $compulsory $args]
	    foreach w_i $w {
		cu::add_bindtag_recursive $w_i cu::tooltip
		dict set register_popup_txt $w_i txt $txt
		dict set register_popup_txt $w_i width $width
		dict set register_popup_txt $w_i max_width $max_width
	    }
	    bind cu::tooltip <1> [list cu::tooltip B1 %W]
	    bind cu::tooltip <Enter> [list cu::tooltip enter %W]
	    bind cu::tooltip <Motion> [list cu::tooltip motion %W]
	    bind cu::tooltip <Leave>  [list cu::tooltip leave %W]
	}
	cancel {
	    destroy $w.t
	    catch { grab release $w }
	    if { ![info exists register_popup_idle] } { set register_popup_idle "" }
	    if { [dict exists $register_popup_idle $w after] } {
		after cancel [dict get $register_popup_idle $w after] 
	    }
	}
	B1 {
	    destroy $w.t
	    catch { grab release $w }
	    update
	}
	enable_motion {
	    dict set register_popup_txt enable_motion 1
	}
	disable_motion {
	    dict set register_popup_txt enable_motion 0
	}
	motion {
	    if { ![dict_getd $register_popup_txt enable_motion 1] } {
		return
	    }
	    cu::tooltip enter $w
	}
	enter {
	    if { ![dict_getd $register_popup_txt enable_motion 1] } {
		return
	    }
	    destroy $w.t
	    catch { grab release $w }
	    if { ![info exists register_popup_idle] } { set register_popup_idle "" }
	    if { [dict exists $register_popup_idle $w after] } {
		after cancel [dict get $register_popup_idle $w after] 
	    }
	    set txt [dict_getd $register_popup_txt $w txt ""]
	    if { $txt ne "" } {
		set xy [winfo pointerxy $w]
		dict set register_popup_idle $w coords $xy
		dict set register_popup_idle $w after [after 500 [list cu::tooltip do $w]]
	    }
	}
	leave {
	    if { [dict exists $register_popup_idle $w after] } {
		after cancel [dict get $register_popup_idle $w after] 
	    }
	    dict unset register_popup_idle $w
	}
	do {
	    set optional {
		{ -width auto|size auto }
		{ -max_width auto|free|size auto }
		{ -text text "" }
	    }
	    set compulsory ""
	    parse_args $optional $compulsory $args
	    
	    if { ![winfo exists $w] } {
		dict unset register_popup_idle $w
		return
	    }
	    set xy [winfo pointerxy $w]
	    
	    if { $text eq "" } {
		if { $xy ne [dict get $register_popup_idle $w coords] } {
		    dict unset register_popup_idle $w
		    return
		}
		set text [dict_getd $register_popup_txt $w txt ""]
	    }
	    if { $text eq "" } {
		return
	    }
	    if { ![winfo exists $w.t] } {
		cu::create_tooltip_toplevel -withdraw $w.t
		update
	    }
	    if { ![winfo exists $w.t] } { return }
	    
	    if { $max_width eq "" } {
		if { ![info exists register_popup_txt] } {
		    set register_popup_txt ""
		}
		set max_width [dict_getd $register_popup_txt $w max_width auto]
		set width [dict_getd $register_popup_txt $w width auto]
	    }
	    if { $max_width eq "auto" } {
		set max_width 260
	    } elseif { $max_width eq "free" } {
		set max_width [winfo screenwidth $w]
	    }
	    lassign [list white #baba45] background highlightcolor

	    set nlines [expr {[regexp -all {\n} $text]+1}]
	    if { ![winfo exists $w.t.l] } {
		text $w.t.l -background $background -highlightthickness 1 -highlightcolor $highlightcolor -height \
		    $nlines -width 80 -bd 0 -font TkTooltipFont -tabs "3mm" -wrap word -padx 2 -pady 2
		grid $w.t.l -sticky nsew -padx 0 -pady 0
		grid columnconfigure $w.t 0 -weight 1
		grid rowconfigure $w.t 0 -weight 1
	    } else {
		$w.t.l configure -height $nlines
		$w.t.l delete 1.0 end
	    }
	    update idletasks
	    if { ![winfo exists $w.t.l] } { return }
	    $w.t.l insert end $text
	    set boldfont [list {*}[font actual TkTooltipFont] -weight bold]
	    $w.t.l tag configure bold -font $boldfont
	    $w.t.l tag configure item -lmargin1 15 -lmargin2 24
	    while { [set idx [$w.t.l search -count len -nolinestop -regexp {<ul>(.*?)</ul>} 1.0]] ne "" } {
		set h [$w.t.l get "$idx+4c" "$idx+[expr {$len-5}]c"]
		$w.t.l delete $idx "$idx+$len c"
		$w.t.l insert $idx $h
	    }
	    while { [set idx [$w.t.l search -count len -nolinestop -regexp {\s*?<li>(.*?)</li>\s*?} 1.0]] ne "" } {
		set h [$w.t.l get $idx "$idx+$len c"]
		regexp {\s*?<li>(.*?)</li>\s??} $h {} text
		$w.t.l delete $idx "$idx+$len c"
		$w.t.l insert $idx "\n\u2022 $text" item
	    }
	    while { [set idx [$w.t.l search -count len -nolinestop -regexp {<b>(.*?)</b>} 1.0]] ne "" } {
		set h [$w.t.l get "$idx+3c" "$idx+[expr {$len-4}]c"]
		$w.t.l delete $idx "$idx+$len c"
		$w.t.l insert $idx $h bold
	    }
	    while { [set idx [$w.t.l search -count len -nolinestop -regexp {<img src=['"](.*?)['"]/>} 1.0]] ne "" } {
		set h [$w.t.l get "$idx+10c" "$idx+[expr {$len-3}]c"]
		$w.t.l delete $idx "$idx+$len c"
		$w.t.l image create $idx -image $h -align center -padx 5
	    }
	    while { [set idx [$w.t.l search -count len -nolinestop -regexp {\s*?<p>(.*?)</p>\s*?} 1.0]] ne "" } {
		set h [$w.t.l get $idx "$idx+$len c"]
		regexp {\s*?<p>(.*?)</p>\s??} $h {} text
		$w.t.l delete $idx "$idx+$len c"
		$w.t.l insert $idx "\n $text" item
	    }
	    $w.t.l configure -wrap none
	    update
	    if { ![winfo exists $w.t.l] } { return }

	    set width_line 0
	    set nlines [llength [split $text \n]]
	    for { set line 1 } { $line <=  $nlines } { incr line } {
		set xi [$w.t.l count -update -xpixels "$line.0" "$line.0 lineend"]
		if { $xi > $width_line } { set width_line $xi }
	    }
	    $w.t.l configure -wrap word
	    set width_line [expr {$width_line+10}]
	    if { $width ne "auto" } {
		set width_line $width
	    }
	    if { $width_line > $max_width } { set width_line $max_width }
	    if { [wm state $w.t] eq "withdrawn" } {
		wm geometry $w.t ${width_line}x5000
	    } else {
		wm geometry $w.t ${width_line}x[winfo height $w.t]
	    }
	    update
	    if { ![winfo exists $w.t.l] } { return }
	    
	    set height [$w.t.l count -update -ypixels 1.0 end]
	    set height [expr {$height+5}]
	    if { $height < 8 } {
		set nlines [scan [$w.t.l index end-1c] "%d"]
		set n 0
		for { set i 1 } { $i <= $nlines } { incr i } {
		    set t [$w.t.l get "$i.0" "$i.0 lineend"]
		    incr n [expr {int([font measure [$w.t.l cget -font] $t]/$width_line)+1}]
		}
		set height [expr {$n*[font metrics [$w.t.l cget -font] -linespace]}]
		set height [expr {$height+$n*4}]
	    }
	    set geometry ${width_line}x$height
	    wm geometry $w.t $geometry
	    update
	    if { ![winfo exists $w.t] } { return }

	    set x [expr {[winfo rootx $w]+[winfo width $w]}]
	    if { $x+[winfo width $w.t] > [winfo screenwidth $w]-10 } {
		set x [expr {[winfo screenwidth $w]-$width_line-10}]
	    }
	    if { $x < 0 } { set x 0 }
	    set y [expr {[winfo rooty $w]+[winfo height $w]}]
	    if { $y+[winfo height $w.t] > [winfo screenheight $w]-10 } {
		set y [expr {[winfo screenheight $w]-[winfo height $w.t]-10}]
	    }
	    if { $y < 0 } { set y 0 }
	    wm geometry $w.t $geometry+$x+$y
	    wm deiconify $w.t

	    catch {
		tkwait visibility $w.t
		grab $w
	    }
	}
	default {
	    error "cu::tooltip unknown action $what"
	}
    }
}

namespace eval cu {      
    get_images_add_dir [file join $topdir images]   
}
