
set ::debug 0

namespace eval test_CTB {}

proc test_CTB::load_gid_groups_conds {} {
    if { $::debug != 0 } {  
	
	lappend ::auto_path {C:\TclTk} {C:\Tcl32} \
	    {C:\Tcl\lib\teapot\package\win32-ix86\lib} \
	    {C:\TclTk\ActiveTcl8.5\lib\teapot\package\win32-ix86\lib} \
	    {/usr/local/ActiveTcl-8.5/lib/teapot/package/linux-glibc2.3-ix86/lib} \
	    ~/mytcltk ~/mytcltk/compass ~/mytcltk/customLib_dir \
	    {C:/Users/peyrau/Documents/problemtypes/customLib/customLib} 
	
    }         
    set packs [list customLib customLib_utils customLib_utils::tk]
    foreach pack $packs {
	package forget $pack
    }
    foreach pack $packs {
	package require $pack
    } 
    if { $::debug != 0 } {  
	mylog::init -view_binding <Control-L> debug    
    }
}


proc InitGIDProject { dir } {
    
    GidChangeDataLabel "Data units" ""
    GidChangeDataLabel "Interval" ""
    GidChangeDataLabel "Conditions" ""
    GidChangeDataLabel "Materials" ""
    GidChangeDataLabel "Interval Data" ""
    GidChangeDataLabel "Problem Data" ""
    GidChangeDataLabel "Local axes" ""

    GidAddUserDataOptions "---" "" 3
    GidAddUserDataOptions [_ "Groups"] "gid_groups_conds::open_groups .gid window" 5
    GidAddUserDataOptions [_ "Data"] \
	"gid_groups_conds::open_conditions window" 6

    GidAddUserDataOptions [_ "Data (internal)"] \
	"gid_groups_conds::open_conditions menu" 7

    GidAddUserDataOptions "---" "" 8
    GidAddUserDataOptionsMenu [_ "Local axes"] \
	"gid_groups_conds::local_axes_menu %W" 9

    set err [catch { test_CTB::load_gid_groups_conds } ret]
    if { $err } {
        tk_messageBox "error loading problemtype: $ret" {*}[ GidUtils::GiDDefaultParentIfPresent]
        return
    }
    gid_groups_conds::SetProgramName "test CTB"
    gid_groups_conds::begin_problemtype [file join $dir test_CTB_default.spd] ""     
    
    if { ![llength [GiD_Geometry list point 1:]] } {
	set mff [GiD_ModifiedFileFlag get]
	GiD_Process MEscape Utilities Variables QuadraticType 1 Mescape
	if { !$mff } { GiD_ModifiedFileFlag set 0 }
    }
	if { [info command ::draw_post::add_post_menu] ne "" } {
        draw_post::add_post_menu -refresh_menu 1
	}
}

proc EndGIDProject {} {
    gid_groups_conds::end_problemtype ""          
}

proc init_problem_GP { dir } {
    # nothing by now
}

proc end_problem_GP {} {
    # nothing by now
}

# Load GiD project files (initialise XML Tdom structure)
proc LoadGIDProject { filespd } {
    if { ![file exists $filespd] } { return }
    set versionPT [gid_groups_conds::give_data_version]

    gid_groups_conds::open_spd_file $filespd

    set versionData [gid_groups_conds::give_data_version]

    if { [package vcompare $versionPT $versionData] == 1 } {
	after idle test_CTB::upgrade_problemtype
    }
}

# Save GiD project files (save XML Tdom structure to spd file)
proc SaveGIDProject { filespd } {
    gid_groups_conds::save_spd_file $filespd
}

proc AfterTransformProblemType { filename oldproblemtype newproblemtype messages } {
    set spd_file [file join $filename.gid [file tail $filename].spd]
	lassign [gid_groups_conds::transform_problemtype_no_messages $spd_file] numchanges messages_local
	if { $numchanges } {
		append messages $messages_local
	}
    return [list $numchanges $messages]
}

proc AfterWriteCalcFileGIDProject { filename errorflag } {

    if { ![info exists gid_groups_conds::doc] } {
	WarnWin [_ "Error: data not OK"]
	return
    }

    set err [catch { test_CTB::writeCalcFile $gid_groups_conds::doc \
		$filename } ret]
    if { $err } {
	if { $::debug } {
	    append ret "\n$::errorInfo"
	}
	snit_messageBox -parent .gid -message \
	    [_ "Error when preparing data for analysis (%s)" $ret]
	return "-cancel-"
    }
    return $ret
}

proc BeforeMeshGeneration { elementsize } {
    set doc $gid_groups_conds::doc
    set root [$doc documentElement]     

    gid_groups_conds::change_default_meshing_criteria_all init
    
    set groupsDictStructured ""          
    set xp {condition[@n="piles"]/group}  
    foreach gNode [$root selectNodes $xp] {
	dict set groupsDictStructured [$gNode @n] 1
    }
    set err [catch { change_line_structured_divisions init $groupsDictStructured } errstring]          
    if { $err } {
	snit_messageBox -message $errstring
    }

}

proc AfterMeshGeneration { fail } {
    set err [catch { AfterMeshGeneration_do $fail } ret]
    if { $err } {
        tk_messageBox -message $ret {*}[ GidUtils::GiDDefaultParentIfPresent]
    }
    return $ret
}

proc AfterMeshGeneration_do { fail } {
    
    set err [catch {
	    gid_groups_conds::change_default_meshing_criteria_all clear
	    change_line_structured_divisions clear
	} errstring]
    if { $err } {
	snit_messageBox -message $errstring
    }    
    set root [$gid_groups_conds::doc documentElement]
    set groupsDict ""
    set xp {container/condition[@n="interface_elements"]/group}
    foreach gNode [$root selectNodes $xp] {
	set group [$gNode @n]
	set name [$gNode selectNodes {string(value[@n="name"]/@v)}]
	set c_less_flag [$gNode selectNodes {string(value[@n="c_less_flag"]/@v)}]
	set c_more_flag [$gNode selectNodes {string(value[@n="c_more_flag"]/@v)}]
	dict set groupsDict $group [list $name $c_less_flag $c_more_flag]
    }
    set groupsDictN ""
    set xp {container/condition[@n="interface_elements_collapse"]/group}
    foreach gNode [$root selectNodes $xp] {
	set group [$gNode @n]
	dict set groupsDictN $group ""
    }

    if { [dict size $groupsDict] } {
	set err [catch { create_contacts_from_groups -collapse_nodes $groupsDictN \
		    $groupsDict } errstring]
	if { $err } {
	    tk_messageBox -message $errstring {*}[ GidUtils::GiDDefaultParentIfPresent]
	}
    }
    
    gid_groups_conds::delete_special_groups
    set dict ""          
    set xp {condition[@n="piles"]/group}  
    foreach gNode [$root selectNodes $xp] {
	dict set dict [$gNode @n] ""
    }
    if { [dict size $dict] } {
	set last_number_before_piles [create_line_elems_intersect_volumes $dict]
	gid_groups_conds::set_preference -local_global local last_number_before_piles \
	    $last_number_before_piles
    }
    
    set xp {condition[@n="local_axes_directions"]/groupList}
    foreach glistNode [$root selectNodes $xp] {
	set surfacesGroup [$glistNode selectNodes {string(group[1]/@n)}]
	set linesGroup [$glistNode selectNodes {string(group[2]/@n)}]
	correct_local_axes_with_lines elements [list $surfacesGroup] [list $linesGroup]
    }
}

proc BeforeInitGIDPostProcess { } {

    set err [catch { draw_post::can_goto_traditional_postprocess } ret]
    if { $err } {
        tk_messageBox -message $ret {*}[ GidUtils::GiDDefaultParentIfPresent]
    } elseif { $ret == 0 } {
        return "-cancel-"
    }
}

proc test_CTB::upgrade_problemtype {} {

    set w [dialogwin_snit .gid._ask -title [_ "Action"] -entrytext \
	    [_ "The model needs to be upgraded. Do you want to upgrade to new version?"]]
    set action [$w createwindow]
    destroy $w
    if { $action < 1 } { return }
    set project [lindex [GiD_Info Project] 0]
    
    #GiD_Process escape escape escape escape Data Defaults TransfProblem $project
	GiD_Project transform_problemtype $project
}


proc test_CTB::writeCalcFile { doc file } {
    
    set root [$doc documentElement]
    
    write_calc_data init $file
	
    write_calc_data puts "coordinates"
    write_calc_data coordinates "%d %g %g %g\n"
    write_calc_data puts "end coordinates"

#     write_calc_data puts "connectivities"
#     write_calc_data all_connectivities "%d %d %d %d\n"
#     write_calc_data puts "end connectivities"
# 
#     write_calc_data puts "connectivities"
#     write_calc_data all_connectivities "%d %d %d %d\n"
#     write_calc_data puts "end connectivities"
# 
#     write_calc_data puts "connectivities (alternative style)"
#     set str [write_calc_data all_connectivities -return "%.0s%d\n%d\n%d\n"]
#     write_calc_data puts -nonewline $str
#     write_calc_data puts "end connectivities"

    write_calc_data puts "coordinates (alternative style)"
    set str [write_calc_data coordinates -return "%.0s%g\n%g\n%g\n"]
    write_calc_data puts -nonewline $str
    write_calc_data puts "end coordinates"
    
    set nmat 1
    set formats ""
    set formats2 ""
    set mat_groups ""
    set mat_props ""
    set xp {condition[@n="Material"]/group}
    foreach gNode [$root selectNodes $xp] {
	set n [$gNode @n]
	dict set mat_groups $n $nmat
	lappend mat_props [$gNode selectNodes {string(value[@n="Material"]/@v)}]
	dict set formats [list $n ""] "%d $nmat\n"
	dict set formats2 [list $n ""] "%.0s $nmat\n"
	incr nmat
    }
    
    write_calc_data puts "materials props"
    set nmat 1
    foreach i $mat_props {
	write_calc_data puts "$nmat $i"
	incr nmat
    }
    write_calc_data puts "end materials props"
    
    set mat_groups ""
    set xp {condition[@n="Material"]/group}
    foreach gNode [$root selectNodes $xp] {
	set n [$gNode @n]
	dict set mat_groups [$gNode @n] "%d %d %d %d %d %d %d %d %d %d\n"
    }
    write_calc_data puts "connectivities for sel materials"
    write_calc_data connectivities -sorted $mat_groups
    write_calc_data puts "end connectivities for sel materials"

    dict set formats "" "%d 0\n"
    dict set formats2 "" "%.0s 0\n"

    set xp {condition[@n="Active_material"]/group}
    foreach gNode [$root selectNodes $xp] {
	set active_flag [$gNode selectNodes {string(value[@n="active_flag"]/@v)}]
	dict for "mat_names nmat" $mat_groups {
	    if { $active_flag } {
		dict set formats [list $mat_names [$gNode @n]] "%d $nmat\n"
		dict set formats2 [list $mat_names [$gNode @n]] "%.0s $nmat\n"
	    } else {
		dict set formats [list $mat_names [$gNode @n]] "%d -$nmat\n"
		dict set formats2 [list $mat_names [$gNode @n]] "%.0s -$nmat\n"
	    }
	}
    }
    
    write_calc_data puts "materials list"
    write_calc_data elements -unique -multiple -all_entities $formats
    write_calc_data puts "end materials list"
    
    set num [write_calc_data elements -unique -multiple -all_entities -count $formats2]
    write_calc_data puts "materials list num=$num"
    set str [write_calc_data elements -unique -multiple -all_entities -return $formats2]
    set idx 0
    set str [regmap {(?n)^} [string range $str 0 end-1] x {set ret "[incr idx] "}]
    write_calc_data puts $str
    write_calc_data puts "end materials list"
    
    
    set numcoord [expr [GiD_Info Mesh NumNodes]*3]            
    write_calc_data puts "Constraints $numcoord values"

    set xp {condition[@n="Fixed_Fixities"]/group}
    foreach gNode [$root selectNodes $xp] {
	set list ""
	foreach i [list X Y Z] {
	    set xp ".//value\[@n='${i}_Fixities'\]"
	    set valueNode [$gNode selectNodes $xp]
	    if { [$valueNode @v] } {
		lappend list 0
	    } else {
		lappend list 1e5
	    }
	}
	dict set formats [$gNode @n] "%d [join $list " "]\n"
    }
    dict set formats "" "%d 1e5 1e5 1e5\n"
    write_calc_data nodes -all_entities -unique $formats    
    
    set isquadratic [lindex [GiD_Info Project] 5]
    switch $isquadratic {
	0 { set fc "%.0s%d %d\n" }
	1 - 2 { set fc "%.0s%d %d %d\n" }
    } 
    set format_conecs ""
    set format_values ""
    set xp {condition[@n="face_load"]/group}
    foreach gNode [$root selectNodes $xp] {
	set n [$gNode @n]
	dict set format_conecs $n $fc
	set f "%.0s%.0s"
	foreach i [list x y] {
	    set v [$gNode selectNodes [format_xpath {string(value[@n=%s]/@v)} load_$i]]
	    append f " [format %.4g $v]"
	}
	append f "\n"
	dict set format_values $n $f
    }
    write_calc_data puts "face loads connectivities"
    write_calc_data elements -elements_faces faces -sorted -print_faces_conecs $format_conecs
    write_calc_data puts "end face loads connectivities"

    write_calc_data puts "face loads values"
    write_calc_data elements -elements_faces faces -sorted $format_values
    write_calc_data puts "end face loads values"

    set isquadratic [lindex [GiD_Info Project] 5]
    switch $isquadratic {
	0 { set fc "%.0s%d %d %d\n" }
	1 - 2 { set fc "%.0s%d %d %d %d %d %d\n" }
    }
    set format_conecs ""
    set format_values ""
    set xp {condition[@n="3D_face_load"]/group}
    foreach gNode [$root selectNodes $xp] {
	set n [$gNode @n]
	dict set format_conecs $n $fc
	set f "%.0s%.0s"
	foreach i [list x y z] {
	    set v [$gNode selectNodes [format_xpath {string(value[@n=%s]/@v)} load_$i]]
	    append f " [format %.4g $v]"
	}
	append f "\n"
	dict set format_values $n $f
    }
    
    set num [write_calc_data elements -elements_faces faces -sorted -count \
	    -print_faces_conecs $format_conecs]
    write_calc_data puts "3D face loads connectivities num=$num"
    write_calc_data elements -elements_faces faces -sorted -print_faces_conecs $format_conecs
    write_calc_data puts "end 3D face loads connectivities"

    write_calc_data puts "3D face loads connectivities (corner_face_corner_face) num=$num"
    write_calc_data elements -elements_faces faces -sorted -print_faces_conecs \
	-connec_ordering corner_face_corner_face $format_conecs
    write_calc_data puts "end 3D face loads connectivities (corner_face_corner_face)"

    write_calc_data puts "3D face loads values"
    write_calc_data elements -elements_faces faces -sorted $format_values
    write_calc_data puts "end 3D face loads values"
    
    set isquadratic [lindex [GiD_Info Project] 5]     
    switch $isquadratic {
	0 { 
	    set fc "%d%.0s\n%.0s\n%.0s\n%.0s\n"
	    set nodeselem 4
	}
	1 - 2 {        
	    set fc "%d%.0s\n%.0s\n%.0s\n%.0s\n%.0s\n%.0s\n%.0s\n%.0s\n%.0s\n%.0s\n"
	    set nodeselem 10
	}
    } 
    set formats ""
    set f "Element %d"
    for { set i 1 } { $i <= 10 } { incr i } {
	append f " Node $i 0.0\n"
    }
    dict set formats "" $f
    
    set xp {condition[@n="internal_water_pressure"]/group}
    foreach gNode [$root selectNodes $xp] {
	foreach i [list yposition0 pressure0 delta_pressure] {
	    set xp ".//value\[@n='$i'\]"
	    set valueNode [$gNode selectNodes $xp]
	    set $i [gid_groups_conds::convert_value_to_default $valueNode]
	}
#         set pt1 "0.0 $yposition0 0.0"
#         set pt2 "0.0 [expr {$yposition0-1.0}] 0.0"
#         set pressure1 [expr {$pressure0+1.0*$delta_pressure}]
#         set formula [list linear_interpolate $pt1 $pt2 $pressure0 $pressure1]
	
	set pt1 "0.0 $yposition0 0.0"
	set pt2 "0.0 [expr {$yposition0-1.0}] 0.0"
	set pt3 "0.0 [expr {$yposition0+1.0}] 0.0"         
	set pressure1 [expr {$pressure0+1.0*$delta_pressure}]
	set formula [list linear_interpolateN [list $pt3 $pt1 $pt2] [list $pressure0 $pressure0 $pressure1]]
	
	set f "Element %d"
	for { set i 1 } { $i <= $nodeselem } { incr i } {
	    set pt "0.0 \[elem_coord $i 2\] 0.0"
	    append f " Node $i \[$formula \"$pt\"\]\n"
	}
	dict set formats [$gNode @n] $f
    }
    write_calc_data puts "internal_water_pressure"
    write_calc_data elements -all_entities -unique -do_subst $formats
    write_calc_data puts "end internal_water_pressure"
    
    set isquadratic [lindex [GiD_Info Project] 5]
    switch $isquadratic {
	0 { set fc "%.0s%d %d\n" }
	1 - 2 { set fc "%.0s%d %d %d\n" }
    }
    set format_conecs ""
    set format_LA ""
    set xp {condition[@n="face_load"]/group}
    foreach gNode [$root selectNodes $xp] {
	set n [$gNode @n]
	dict set format_conecs $n $fc
	set f {%.0s[euler_angles matrix {%g %g %g %g %g %g %g %g %g}]}
	append f "\n"
	dict set format_LA $n $f
    }
    write_calc_data puts "face_load as connectivities"
    write_calc_data connectivities -connec_ordering corner_face_corner_face \
	$format_conecs
    write_calc_data puts "end face_load as connectivities"

    write_calc_data puts "face_load as connectivities"
    write_calc_data connectivities -connec_ordering corner_face_corner_face \
	$format_conecs
    write_calc_data puts "end face_load as connectivities"

    write_calc_data puts "face_load as Local axes matrix"
    write_calc_data elements -do_subst -localaxes $format_LA ""
    write_calc_data puts "end face_load as Local axes matrix"
  
    set isquadratic [lindex [GiD_Info Project] 5]
    switch $isquadratic {
	0 { set fc "%.0s%d %d %d %d %d %d %d %d %d\n" }
	1 - 2 { set fc "%.0s%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n" }
    }   
    set format_conecs ""
    set xp {container[@n="interface_elements"]/condition[@n="interface_elements"]/group}
    foreach gNode [$root selectNodes $xp] {  
	set n [$gNode @n]
	set c_less_flagNode [$gNode selectNodes {.//value[@n="c_less_flag"]}]
	set c_less_flag [$c_less_flagNode @v]
	
	set c_more_flagNode [$gNode selectNodes {.//value[@n="c_more_flag"]}]
	set c_more_flag [$c_more_flagNode @v]                
	if {$c_less_flag} {     
	    foreach igroupNode [$root selectNodes groups/group] {
		if {[$igroupNode @n] == "$n B"} { 
		    dict set format_conecs [$igroupNode @n] $fc 
		    break 
		}
	    }             
	} 
	if {$c_more_flag} {         
	    foreach igroupNode [$root selectNodes groups/group] {
		if {[$igroupNode @n] == "$n A"} { 
		    dict set format_conecs [$igroupNode @n] $fc  
		    break 
		}
	    }              
	}
    }
    write_calc_data puts "writing interface_elements"
    if { [dict size $format_conecs] } {         
	set num [write_calc_data connectivities -sorted -count $format_conecs]
	
	# atencio: el 15 depèn de isquadratic
	write_calc_data puts "[expr {$num*15}] values" 
	write_calc_data connectivities -sorted -connec_ordering corner_face_corner_face \
	    $format_conecs
	
	write_calc_data puts "writing alternative numeration"
	
	set format_conecs_alt ""
	set fc "%.0s%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n"
	set fc {%1$.0s%2$d %5$d %3$d %6$d %4$d %7$d }
	append fc {%8$d %14$d %9$d %15$d %10$d %16$d}
	append fc {%11$.0s%12$.0s%13$.0s} "\n"
	dict for "n v" $format_conecs {
	    dict set format_conecs_alt $n $fc
	}
	write_calc_data connectivities -sorted -connec_ordering corner_face_corner_face \
	    $format_conecs_alt
    } else {
	write_calc_data puts "0 values"    
    }
    write_calc_data puts "end writing interface_elements"

    ###############
    
    set isquadratic [lindex [GiD_Info Project] 5]
    switch $isquadratic {
	0 { 
	    set fc "%.0s%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n"          
	}
	1 - 2 { 
	    set fc "%.0s%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n%d\n"            
	}
    }   
    
    set format_LA ""  
    set xp {container[@n="interface_elements"]/condition[@n="interface_elements"]/group}
    foreach gNode [$root selectNodes $xp] {  
	set nint [$gNode @n]                                  
	set c_less_flagNode [$gNode selectNodes {.//value[@n="c_less_flag"]}]
	set c_less_flag [$c_less_flagNode @v]        
	
	set c_more_flagNode [$gNode selectNodes {.//value[@n="c_more_flag"]}]
	set c_more_flag [$c_more_flagNode @v]                   
	
	if {$c_less_flag} {  
	    set f {%.0s[EAmat 1 1]\n[EAmat 2 1]\n[EAmat 3 1]\n[expr -[EAmat 1 2]]\n[expr -[EAmat 2 2]]\n[expr -[EAmat 3 2]]\n[expr -[EAmat 1 3]]\n[expr -[EAmat 2 3]]\n[expr -[EAmat 3 3]]\n}            
	    set f [subst -novariables -nocommands $f] 
	    foreach igroupNode [$root selectNodes groups/group] {
		if {[$igroupNode @n] == "$nint A"} { 
		    dict set format_LA "$nint A" $f
		    break 
		}
		if {[$igroupNode @n] == "$nint AB"} {
		    dict set format_LA "$nint AB" $f
		    break 
		}
	    }             
	} 
	if {$c_more_flag} {  
	    set f {%.0s[EAmat 1 1]\n[EAmat 2 1]\n[EAmat 3 1]\n[EAmat 1 2]\n[EAmat 2 2]\n[EAmat 3 2]\n[EAmat 1 3]\n[EAmat 2 3]\n[EAmat 3 3]\n}        
	    set f [subst -novariables -nocommands $f] 
	    
	    foreach igroupNode [$root selectNodes groups/group] {
		if {[$igroupNode @n] == "$nint B"} {
		    dict set format_LA "$nint B" $f
		    break 
		}
		if {[$igroupNode @n] == "$nint AB"} { 
		    dict set format_LA "$nint AB" $f                    
		    break 
		}
	    }              
	}                                                      
    }        
    
    if { [dict size $format_LA] } {  
	write_calc_data puts "writing interface local axes"
	set err [catch {write_calc_data elements -do_subst -count -localaxes $format_LA ""} numelems]  
	if { $err } {          
	    snit_messageBox -parent .gid -message "Error when define the local coordinates ($numelems)" 
	    return
	} else {
	    write_calc_data puts "[expr $numelems*9] values" 
	    write_calc_data elements -do_subst -localaxes $format_LA "" 
	}  
	write_calc_data puts "end writing interface local axes"   
    }

    set patternList ""          
    set xp {condition[@n="piles"]/group}  
    foreach gNode [$root selectNodes $xp] {
	lappend patternList [$gNode @n]:*
    }
    set groups [gid_groups_conds::give_groupsP $patternList]
    
    if { [llength $groups] } {
	set lnb [gid_groups_conds::get_preference last_number_before_piles 0]
	write_calc_data puts "First node of piles: [expr {$lnb+1}]"
	write_calc_data puts "Number of piles: [llength $groups]"
	foreach group $groups {
	    regexp {\d+$} $group pile_number
	    write_calc_data puts "pile number: $pile_number"
	    set f {%.0s %d %d %d tetra: [element_pair]}
	    append f "\n"
	    set dict ""
	    dict set dict $group $f
	    set num [write_calc_data connectivities -count $dict]
	    write_calc_data puts "Number of elements in pile: $num"
	    write_calc_data connectivities -do_subst $dict
	}
    }
    
    # Writing the coordinates of the nodes per element     
    set numnodes 3
    set formats ""
    set xp {condition[@n="centers"]/group}      
    foreach gNode [$root selectNodes $xp] {               
	set f "Element %d\n"
	for { set i 1 } { $i <= $numnodes } { incr i } {               
	    set pt "\[elem_coord $i 1\] \[elem_coord $i 2\] \[elem_coord $i 3\]"
#             set pt "\[expr \[elem_coord $i 1\]+\[elem_coord $i 2\]+\[elem_coord $i 3\]\]"   
	    append f " Node $i \"$pt\"\n"           
	}
	dict set formats [$gNode @n] $f
    }
    write_calc_data puts "coordinates of the nodes per element"
#     write_calc_data elements -do_subst $formats
    write_calc_data connectivities -sorted -connec_ordering corner_face_corner_face -do_subst $formats
    write_calc_data puts "end coordinates of the nodes per element"
    
    # Writing the coordinates for the centers of the triangles 
    # belonging to a selected surface.          
  
    set formats ""
    set xp {condition[@n="centers"]/group}      
    foreach gNode [$root selectNodes $xp] {               
	set f "Element %d "
	for { set j 1 } { $j <= 3 } { incr j } {     
	    set ptj 0.0            
	    for { set i 1 } { $i <= $numnodes } { incr i } {                           
		set ptj "\[expr \"$ptj\" + \[elem_coord $i $j\]/$numnodes\]"             
	    }
#                      
	    append f " \"$ptj\" " 
	}        
	append f "\n"
	dict set formats [$gNode @n] $f
    }
    write_calc_data puts "centers"
#     write_calc_data elements -do_subst $formats
    write_calc_data connectivities -sorted -connec_ordering corner_face_corner_face -do_subst $formats
    write_calc_data puts "end centers"
    
    write_calc_data end
}



















