package require customLib
package provide customlib_extras 0.4

namespace eval customlib {
    variable mat_dict
    set mat_dict [dict create]
}

# can not be deleted due to it is called by kratos
proc customlib::GetBaseRoot {} {
    return [$::gid_groups_conds::doc documentElement]
}

# can not be deleted due to it is called by kratos
proc customlib::UpdateDocument { } {
    variable mat_dict
    if { ![info exists ::gid_groups_conds::doc] } {
        W "customlib::Init. variable gid_groups_conds::doc doesn't exists"
    }
    set mat_dict [dict create]
}

# customlib::ProcessIncludes can not be deleted due to it is called by existing problem types
proc customlib::ProcessIncludes { dir } {
    return [gid_groups_conds::ProcessIncludes $dir ]
}

# customlib::ProcessIncludesRecurse can not be deleted due to it is called by existing problem types
proc customlib::ProcessIncludesRecurse { filename basedir } {
    return [gid_groups_conds::ProcessIncludesRecurse $filename $basedir ]
}

# can not be deleted due to it is called by kratos
proc customlib::AddConditionGroupOnXPath {xpath groupid} {
    return [gid_groups_conds::AddConditionGroupOnXPath $xpath $groupid]
}

# can not be deleted due to it is called by kratos
proc customlib::AddConditionGroupOnNode {basenode groupid} {
    return [gid_groups_conds::AddConditionGroupOnNode $basenode $groupid]
}



proc customlib::GetNumberOfMaterials { {state all }} {
    return [llength [dict keys [customlib::GetMaterials $state]]]
}

proc customlib::GetMaterials { {state all }} {
    variable mat_dict
    if {$mat_dict eq ""} {
        customlib::InitMaterials
    }
    if {$state eq "used"} {
        set usedMaterials [dict create]
        foreach material [dict keys $mat_dict] {
            if {[dict exists $mat_dict $material MID]} {
                dict set usedMaterials $material [dict get $mat_dict $material]
            }
        }
        return $usedMaterials
    } else {
        return $mat_dict
    }
}

# condition_n is a condition name
# parameters is
# unit_mode can be default or active or none
proc customlib::GetElementsFormats {condition_n parameters {unit_mode "default"}} {
    set mat_dict ""
    set formats [dict create]
    set rootNode [customlib::GetBaseRoot]
    set xp [format_xpath {//condition[@n=%s]} $condition_n]
    foreach condNode [$rootNode selectNodes $xp] {
        foreach gNode [$condNode selectNodes {.//group}] {
            set group [$gNode @n]
            set ov ""
            catch {set ov [$gNode @ov]}
            if {$ov eq ""} {
                set ov [[$gNode parent] @ov]
            }
            set ovm [[$gNode parent] @ovm]
            if { $ovm == "node" } {
                #there are not expected elements
                set nnodes 0
                set etype ""
            } else {
                lassign [customlib::GetElementTypeAndAmountNodes $group $ov] etype nnodes
            }
            if {$nnodes ne ""} {
                set my_mat ""
                set valueNode [lindex [$gNode selectNodes {.//value[@n="material"]}] 0]
                if {$valueNode ne ""} {
                    if {$mat_dict eq ""} {
                        set mat_dict [customlib::GetMaterials used]
                    }
                    set material_name [get_domnode_attribute $valueNode v]
                    if {![dict exists $mat_dict $material_name MID]} {
                        error [_ "It's mandatory to call customlib::InitMaterials with the used conditions"]
                    } else {
                        set my_mat [dict get $mat_dict $material_name]
                    }
                }
                set val [customlib::GetFormatDict $gNode $my_mat $parameters $unit_mode]
                dict set formats $group "$val\n"
            }
        }
    }
    return $formats
}

# unit_mode can be default or active or none
proc customlib::GetFormatDict {gNode material parameters {unit_mode "default"}} {
    set f ""
    set unit_conversion_system convert_value_to_default
    if {$unit_mode ne "default"} {set unit_conversion_system convert_value_to_active}
    if {$gNode ne ""} {
        set group [$gNode @n]
        set ov ""
        catch {set ov [$gNode @ov]}
        if {$ov eq ""} {
            set ov [[$gNode parent] @ov]
        }
        set ovm [[$gNode parent] @ovm]
        if { $ovm == "node" } {
            #there are not expected elements
            set nnodes 0
            set etype ""
        } else {                       
            lassign [customlib::GetElementTypeAndAmountNodes $group $ov] etype nnodes
        }
    }
    foreach item $parameters {
        lassign $item fmt what attr
        switch $what {
            "element" {
                if {$attr eq "id"} {
                    append f "$fmt "
                } elseif {$attr eq "connectivities"} {
                    append f "[string repeat "$fmt " $nnodes]"
                }
            }
            "node" {
                if {$attr eq "id"} {
                    append f "$fmt "
                }
            }
            "material" {
                set attr [dict get $material $attr]
                append f "[format $fmt $attr]"
            }
            "property" {
                set xp2 {.//value[@n=$attr]}
                set valueNode [lindex [$gNode selectNodes $xp2] 0]
                if {$unit_mode ne "none"} {
                    set value [gid_groups_conds::$unit_conversion_system $valueNode]
                } else {
                    set value [get_domnode_attribute $valueNode v]
                }
                append f "[format $fmt $value]"
            }
            "string" {
                append f "[format $fmt $attr]"
            }
            default {
                error [_ "Error defining parameters for %s - %s - %s" $group $attr $what]
            }
        }
    }
    return $f
}

proc customlib::GetElementTypeAndAmountNodes {group ov} {
    set isquadratic [customlib::IsQuadratic]
    set ret [list "" ""]
    if {$ov eq "point"} {
        set ret [list Point 1]
    }
    if {$ov eq "line"} {
        switch $isquadratic {
            0 { set ret [list Linear 2] }
            default { set ret [list Linear 3] }
        }
    }
    if {$ov eq "surface"} {
        if {[GiD_EntitiesGroups get $group elements -count -element_type Triangle]} {
            switch $isquadratic {
                0 { set ret [list Triangle 3]  }
                default { set ret [list Triangle 6]  }
            }
        }
        if {[GiD_EntitiesGroups get $group faces -count -element_type Tetrahedra]} {
            switch $isquadratic {
                0 { set ret [list Triangle 3]  }
                default { set ret [list Triangle 6]  }
            }
        }
        if {[GiD_EntitiesGroups get $group elements -count -element_type Quadrilateral]} {
            switch $isquadratic {
                0 { set ret [list Quadrilateral 4]  }
                1 { set ret [list Quadrilateral 8]  }
                2 { set ret [list Quadrilateral 9]  }
            }
        }
        if {[GiD_EntitiesGroups get $group faces -count -element_type Hexahedra]} {
            switch $isquadratic {
                0 { set ret [list Quadrilateral 4]  }
                1 { set ret [list Quadrilateral 8]  }
                2 { set ret [list Quadrilateral 9]  }
            }
        }
    }
    if {$ov eq "volume"} {
        if {[GiD_EntitiesGroups get $group elements -count -element_type Tetrahedra]} {
            switch $isquadratic {
                0 { set ret [list Tetrahedra 4]  }
                1 { set ret [list Tetrahedra 10] }
                2 { set ret [list Tetrahedra 10] }
            }
        }
        if {[GiD_EntitiesGroups get $group elements -count -element_type Hexahedra]} {
            switch $isquadratic {
                0 { set ret [list Hexahedra 8]  }
                1 { set ret [list Hexahedra 20]  }
                2 { set ret [list Hexahedra 27]  }
            }
        }
    }
    return $ret
}

proc customlib::IsQuadratic {} {
    return [GiD_Set Model(QuadraticType)]
}

proc customlib::InitWriteFile {filename} {
    GiD_WriteCalculationFile init $filename ;#initialize writting
}

proc customlib::EndWriteFile { } {
    GiD_WriteCalculationFile end
}

proc customlib::WriteString { str } {
    GiD_WriteCalculationFile puts $str
}

# elements_conditions is a list of every <condition> where we use a material
# unit_mode can be default or active or none
proc customlib::InitMaterials {elements_conditions {unit_mode "default"} } {
    variable mat_dict
    set unit_conversion_system convert_value_to_default
    if {$unit_mode ne "default"} {set unit_conversion_system convert_value_to_active}
    set root [customlib::GetBaseRoot]
    set xp2 {.//container[@n="materials"]/blockdata[@n="material"]}
    set matnodes [$root selectNodes $xp2]
    if {$matnodes eq ""} {error [_ "No materials block found"]}
    foreach matnode $matnodes {
        set mat_name [$matnode @name]
        #if {[dict exists $mat_dict $mat_name] } {set props_dict [dict get $mat_dict $mat_name]} {set props_dict [dict create] }
        set props_dict [dict create]
        foreach prop_node [$matnode selectNodes value] {
            set value [get_domnode_attribute $prop_node v]
            if {$unit_mode ne "none"} {set value [gid_groups_conds::$unit_conversion_system $prop_node]}
            dict set props_dict [$prop_node @n] $value
        }
        dict set mat_dict $mat_name $props_dict
    }
    set material_number 0
    set condNodes [[customlib::GetBaseRoot] getElementsByTagName condition]
    foreach condNode $condNodes {
        if {[$condNode @n] in $elements_conditions} {
            set groups [$condNode getElementsByTagName group]
            set xp2 {.//value[@n="material"]}
            foreach gNode $groups {
                set valueNode [lindex [$gNode selectNodes $xp2] 0]
                set material_name [get_domnode_attribute $valueNode v]
                if {![dict exists $mat_dict $material_name MID]} {
                    incr material_number
                    dict set mat_dict $material_name MID $material_number
                }
            }
        }
    }
}

# unit_mode can be default or active or none
proc customlib::WriteConnectivities {list_condition_n parameters {unit_mode "default"}} {
    foreach condition_n $list_condition_n {
        set formats [customlib::GetElementsFormats $condition_n $parameters $unit_mode]
        GiD_WriteCalculationFile connectivities -elements_faces all $formats
    }
}

# list_condition_n is a list of condition names,
# parameters is a list of items with "fmt what attr"
# unit_mode can be default or active or none
proc customlib::WriteNodes {list_condition_n parameters {flags ""} {unit_mode "default"}} {
    set formats [dict create]
    foreach cond $list_condition_n {
        foreach {k v} [customlib::GetElementsFormats $cond $parameters $unit_mode] {
            dict set formats $k $v
        }
    }
    if {$flags eq ""} {
        set result [GiD_WriteCalculationFile nodes $formats]
    } else {
        set result [GiD_WriteCalculationFile nodes $flags $formats]
    }
    return $result
}

#allow repeat group name for a condition
proc customlib::WriteNodesByGroup {list_condition_n parameters {flags ""} {unit_mode "default"}} {
    if { $flags == "-count" } {
        set result 0
    } else {
        set result ""
    }
    set rootNode [$::gid_groups_conds::doc documentElement]
    foreach cond $list_condition_n {
        foreach gNode [$rootNode selectNodes [format_xpath {//condition[@n=%s]//group} $cond]] {
            set val [customlib::GetFormatDict $gNode "" $parameters $unit_mode]
            set dict_format_group [dict create [$gNode @n] "$val\n"]
            if {$flags eq ""} {
                set local_result [GiD_WriteCalculationFile nodes $dict_format_group]
            } else {
                set local_result [GiD_WriteCalculationFile nodes $flags $dict_format_group]
            }
            if { $flags == "-count" } {
                incr result $local_result
            } else {
                append result $local_result
            }
        }
    }
    return $result
}

proc customlib::GetNumberOfNodesByGroup {list_condition_n {flags ""} } {
    return [customlib::WriteNodesByGroup $list_condition_n "" [concat $flags -count]]
}

proc customlib::GetNumberOfNodes {list_condition_n {flags ""} } {
    #return [GiD_WriteCalculationFile nodes -count [customlib::GetElementsFormats $list_condition_n ""]]
    return [customlib::WriteNodes $list_condition_n "" [concat $flags -count]]
}

# unit_mode can be default or active or none
proc customlib::GetNodes {list_condition_n parameters {flags ""} {unit_mode "default"}} {
    return [customlib::WriteNodes $list_condition_n $parameters [concat $flags -return] $unit_mode]
}

proc customlib::GetCoordinates {format} {
    return [customlib::WriteCoordinates $format -return]
}

proc customlib::GetNumberCoordinates {format} {
    return [customlib::WriteCoordinates $format -count]
}

#format must be a "C-like" format for an integer (node id) and three doubles (x y z). e.g. "%d %g %g %g"
#flags could be: -count -return
proc customlib::WriteCoordinates {format {flags ""}} {
    # Geometry factor (here the geometry unit declared by the user is converted to 'm')
    set mesh_unit [gid_groups_conds::give_mesh_unit]
    set mesh_factor [lindex [gid_groups_conds::give_unit_factor L $mesh_unit] 0]
    # efficient CustomLib specialized procedure to print everything related with nodes or elements
    if {$flags eq ""} {
        set result [GiD_WriteCalculationFile coordinates -factor $mesh_factor $format]
    } else {
        set result [GiD_WriteCalculationFile coordinates $flags -factor $mesh_factor $format]
    }
    return $result
}

# user_mode must be default or active or none
proc customlib::WriteMaterials { parameters {state "used"} {unit_mode "default"}} {
    set mat_dict [GetMaterials $state]
    foreach material [dict values $mat_dict] {
        set str [customlib::GetFormatDict "" $material $parameters $unit_mode]
        GiD_WriteCalculationFile puts $str
        #GiD_WriteCalculationFile puts "[format "%4d %13.5e" [dict get $mat_dict $material MID] [format "%13.5e" [dict get $mat_dict $material Density]]]\n"
    }
}

proc customlib::GetNodeValue { xpath } {
    set value ""
    set document [$::gid_groups_conds::doc documentElement]
    set xml_nodes [$document selectNodes $xpath]
    if { [llength $xml_nodes] == 1 } {
        set value [get_domnode_attribute [lindex $xml_nodes 0] v]
    }
    return $value
}
