#common procedures, with body valid for all platforms

namespace eval gid_cross_platform {
    variable _same_scaling_factor_on_all_monitors    -1   ;# cache variable of probable costly gid_cross_platform::get_display_monitors may be costly
    variable _number_of_monitors_connected           -1   ;# same as above
}

########## PROCESS #####################

#e.g. gid_cross_platform::track_process to follow an asyncronous process and raise callback events
# status will be= ok|user_stop|timeout
# proc my_finish_proc { pid status args } { ... }
# returned user_stop must be 0 or 1 (1 to kill the process)
# proc my_progress_proc { args }  { ... return $user_stop }
# set timeout 20000 ;#seconds, 0 to no limit
# set maxmemory 0 ;#bytes 0 to no limit
# set pid [eval exec $cmd &]
# gid_cross_platform::track_process $pid 1000 [clock seconds] $timeout $maxmemory [list my_finish_proc $args] [list my_progress_proc a b]
proc gid_cross_platform::track_process { pid ms t0 timeout maxmemory finish_procedure progress_procedure } {
    if { [gid_cross_platform::process_exists $pid] } {
        set memory_used [gid_cross_platform::get_process_info_memory $pid]
        if { ![string is integer $memory_used] } {
            #probably the process has finished
            set memory_used 0
        }
        if { $timeout > 0 && [clock seconds]-$t0 > $timeout } {
            if { [catch {gid_cross_platform::end_process_and_childs $pid} err] } {
                W "error '$err'"
            }
            [lindex $finish_procedure 0] $pid timeout {*}[lrange $finish_procedure 1 end]
        } elseif { $maxmemory > 0 && $memory_used > $maxmemory } {
            if { [catch {gid_cross_platform::end_process_and_childs $pid} err] } {
                W "error '$err'"
            }
            [lindex $finish_procedure 0] $pid maxmemory {*}[lrange $finish_procedure 1 end]
        } else {
            if { $progress_procedure != "" } {
                set user_stop [eval $progress_procedure]
            } else {
                set user_stop 0
            }
            if { $user_stop } {
                if { [catch {gid_cross_platform::end_process_and_childs $pid} err] } {
                    W "error '$err'"
                }
                [lindex $finish_procedure 0] $pid user_stop {*}[lrange $finish_procedure 1 end]
            } else {
                after $ms [list gid_cross_platform::track_process $pid $ms $t0 $timeout $maxmemory $finish_procedure $progress_procedure]
                update ;#to allow after to be processed, else the finish_procedure could be invoked later than the true end
            }
        }
    } else {
        [lindex $finish_procedure 0] $pid ok {*}[lrange $finish_procedure 1 end]
    }
}

proc gid_cross_platform::cancel_track_process { pid } {
    foreach id [after info] {
        lassign [after info $id] script kind_event
        if { $kind_event == "timer" } {
            if { [lindex $script 0] == "gid_cross_platform::track_process" && [lindex $script 1] == $pid } {
                after cancel $id
                break
            }
        }
    }
}

#kill also its recursive child process
proc gid_cross_platform::end_process_and_childs { pid } {
    set fail 0
    set child_pids [gid_cross_platform::get_process_childs $pid 0]
    foreach child_pid $child_pids {
        if { [gid_cross_platform::process_exists $child_pid] } {
            gid_cross_platform::end_process $child_pid
        }
    }
    if { [gid_cross_platform::process_exists $pid] } {
        #check because maybe the process has finished killing its childs
        gid_cross_platform::end_process $pid
        if { [gid_cross_platform::process_exists $pid] } {
            set fail 1
        }
    }
    return $fail
}


########## GUI #####################

#create a raster image from a widget c with Img (image create photo -format window -data $c)
#limitation: to capture the image the whole window must be visible in screen
proc gid_cross_platform::capture_window { win } {
    # package require Img
    package require img::window
    # Make the base image based on the window
    set image [image create photo -format window -data $win]
    foreach child [winfo children $win] {
        gid_cross_platform::_capture_sub_window $child $image 0 0
    }
    #save image to disk
    #$image write $filename -format GIF
    return $image
}

#auxiliary procedure, used by gid_cross_platform::capture_window
proc gid_cross_platform::_capture_sub_window { win image px py } {
    if {![winfo ismapped $win]} {
        return
    }
    regexp {([0-9]*)x([0-9]*)\+([0-9]*)\+([0-9]*)} [winfo geometry $win] - w h x y
    incr px $x
    incr py $y
    # Make an image from this widget
    set tempImage [image create photo -format window -data $win]
    # Copy this image into place on the main image
    $image copy $tempImage -to $px $py
    image delete $tempImage
    foreach child [winfo children $win] {
        gid_cross_platform::_capture_sub_window $child $image $px $py
    }
}

###     -np- W [ font metrics TkDefaultFont]
### -ascent 14 -descent 3 -linespace 17 -fixed 0
### -ascent 11 -descent 3 -linespace 14 -fixed 0
### size specified = 12
###     -np- W [ font metrics BigFont]
### -ascent 14 -descent 4 -linespace 18 -fixed 0
### -ascent 18 -descent 4 -linespace 22 -fixed 0 ( display +125%)
### 
### -np- W [ font configure FixedFont -size] = 9
###     -np- W [ font metrics FixedFont]  (preferences size = 9)
### -ascent 10 -descent 3 -linespace 13 -fixed 1
### -np- W [ font measure FixedFont GiD] = 21
### -np- W [ font measure FixedFont GiD-TheUniversalPreAdnPostprocessor] = 245
### -ascent 12 -descent 3 -linespace 15 -fixed 1 ( display +125%)
### -np- W [ font measure FixedFont GiD] = 27
### -np- W [ font measure FixedFont GiD-TheUniversalPreAdnPostprocessor] = 315
### -np- W [ font configure FixedFont -size] = 11
### -np- W [ font measure FixedFont GiD-TheUniversalPreAdnPostprocessor] = 280 ( 100%)
### -np- W [ font measure FixedFont GiD-TheUniversalPreAdnPostprocessor] = 385 ( 125%)

### -np- W [ font configure FixedFont -size] = 9
### -np- W [ font measure FixedFont GiD-TheUniversalPreAdnPostprocessor] = 490 ( display +150%)
### -np- W [ font configure FixedFont -size] = 11
### -np- W [ font measure FixedFont GiD-TheUniversalPreAdnPostprocessor] = 630 ( display +150%)

# needed because
# GID-2764 two monitors with different scaling on windows: topmost and transient are the flags that makes contextual menu kapput
proc gid_cross_platform::all_monitors_use_same_scaling_factor {} {
    variable _same_scaling_factor_on_all_monitors

    # caching this variable avoids automatic detection when monitor is being plugged in and out during a gid session
    #    but eventually if automatic detection, windows should be reopened when new monitor is attached.
    if { $_same_scaling_factor_on_all_monitors == -1} {
        # assuming defaul same scaling, missing detection in linux
        set same_scaling 1
        if { $::tcl_platform(platform) == "windows" } {
            # implemented in gid_cross_platform_windows.tcl
            set dict_monitors [ gid_cross_platform::get_display_monitors]
            set cmp_dpi_value -1
            foreach mon_name [ dict keys $dict_monitors] {
                set mon_info [ dict get $dict_monitors $mon_name]
                if { [ dict exist $mon_info -dpi-value]} {
                    set mon_dpi_value [ dict get $mon_info -dpi-value]
                    # Windows uses DpiValue = 0, 1, 2, 3 for scaling = 100%, 125%, 150%, 175%
                    if { $cmp_dpi_value == -1} {
                        set cmp_dpi_value $mon_dpi_value
                    } else {
                        if { $cmp_dpi_value != $mon_dpi_value} {
                            set same_scaling 0
                            break
                        }
                    }
                }
            }
        }
        
        if {  $::tcl_platform(os) == "Darwin"} {
            # implemented in gid_cross_platform_unix.tcl
            set dict_monitors [ gid_cross_platform::get_display_monitors]
            set cmp_scale_factor -1
            foreach mon_name [ dict keys $dict_monitors] {
                set mon_info [ dict get $dict_monitors $mon_name]
                if { [ dict exist $mon_info -scale-factor]} {
                    set mon_scale_factor [ dict get $mon_info -scale-factor]
                    if { $cmp_scale_factor == -1} {
                        set cmp_scale_factor $mon_scale_factor
                    } else {
                        if { $cmp_scale_factor != $mon_scale_factor} {
                            set same_scaling 0
                            break
                        }
                    }
                }
            }
        }
        set _same_scaling_factor_on_all_monitors $same_scaling
    }
    return $_same_scaling_factor_on_all_monitors
}

proc gid_cross_platform::get_number_of_monitors_connected {} {
    variable _number_of_monitors_connected

    if { $_number_of_monitors_connected == -1} {
        # assuming 1 by default, missing detection in linux
        set num_monitors 1
        # in fact we should check if 'mirror' is being used ...
        if { $::tcl_platform(platform) == "windows" } {
            set num_monitors [ dict size [ gid_cross_platform::get_display_monitors]]
            # TODO check if monitors are mirroed
        }
        if {  $::tcl_platform(os) == "Darwin"} {
            set dict_monitors [ gid_cross_platform::get_display_monitors]
            set num_monitors [ dict size $dict_monitors]
            # check if monitor is mirroing
            foreach mon_name [ dict keys $dict_monitors] {
                set mon_info [ dict get $dict_monitors $mon_name]
                if { [ dict exist $mon_info spdisplays-mirror]} {
                    set mirror_status [ dict get $mon_info spdisplays-mirror]
                    # can be "spdisplays_off" or "spdisplays_on"
                    # eventually check "spdisplays-mirror-status"
                    #  to get "spdisplays_master_mirror" or "spdisplays_hardware_mirror"
                    if { $mirror_status == "spdisplays_on"} {
                        set num_monitors 1
                        break
                    }
                }
            }
        }
        set _number_of_monitors_connected $num_monitors
    }
    return $_number_of_monitors_connected
}


########## FILE SYSTEM #####################

proc gid_cross_platform::path_append { item {set_first 0}} {
    if { [info exists ::env(PATH)] } {
        set separator [gid_cross_platform::get_path_separator]
        if { [lsearch [split $::env(PATH) $separator] $item] == -1 } {
            if { $set_first } {
                set ::env(PATH) $item$separator$::env(PATH)
            } else {
                append ::env(PATH) $separator$item
            }
        }
    } else {
        set ::env(PATH) $item
    }
}

#returns a non existent filename inside a directory
proc gid_cross_platform::get_unused_filename_in_folder { path {basename gid} {extension .tmp} } {
    set tmpname ""
    set chars "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
    set nrand_chars 10
    set maxtries 1000
    for { set i 0 } { $i < $maxtries } {incr i} {
        set random_part ""
        for {set j 0} {$j < $nrand_chars} {incr j} {
            append random_part [string index $chars [expr {int(rand()*62)}]]
        }
        set tmpname [file join $path ${basename}_${random_part}]$extension
        if { ![file exists $tmpname] } break
    }
    if { $i== $maxtries } {
        error "cannot find an unused filename in the folder $path"
    }
    return $tmpname
}

#returns a non existent filename inside the temporary directory
proc gid_cross_platform::get_unused_tmp_filename { {basename gid} {extension .tmp} } {
    return [gid_cross_platform::get_unused_filename_in_folder [gid_cross_platform::get_tmp] $basename $extension]
}

#returns a new folder inside the temporary directory
proc gid_cross_platform::create_folder_tmp { {extension ""}  } {
    set tmpdir [gid_cross_platform::get_tmp]
    set tmpname ""
    set chars "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"
    set nrand_chars 10
    set maxtries 1000
    for { set i 0 } { $i < $maxtries } {incr i} {
        set random_part ""
        for {set j 0} {$j < $nrand_chars} {incr j} {
            append random_part [string index $chars [expr {int(rand()*62)}]]
        }
        set tmpname [file join $tmpdir gidtmp_${random_part}${extension}]
        if { ![file exists $tmpname] } {
            file mkdir $tmpname
            break
        }
    }
    if { $i== $maxtries} {
        error "cannot find an unused folder name in the folder $tmpdir"
    }
    return $tmpname
}

# filename the full filename to be tracked for access or deletion
# ms milliseconds between tracks
# proc callback_procedure { filename status args } { ... }
# status will be modified or deleted
# e.g.
# proc my_callback { filename status } { W "$filename $status" }
# gid_cross_platform::track_file R:/temp/my_file.log 5000 my_callback
# gid_cross_platform::track_file_cancel R:/temp/my_file.log
proc gid_cross_platform::track_file { filename ms callback_procedure } {
    variable cancel_track
    variable mtime_prev
    variable after_id
    if { [info exists cancel_track($filename)] } {
        if { [info exists after_id($filename)] } {
            after cancel $after_id($filename)
            unset after_id($filename)
        }
        unset cancel_track($filename)
        unset -nocomplain mtime_prev($filename)
    } else {
        set mtime 0
        if { [file exists $filename] } {
            set mtime [file mtime $filename]
        }
        if { ![info exists mtime_prev($filename)] } {
            set mtime_prev($filename) $mtime
        }
        if { $mtime != $mtime_prev($filename) } {
            set mtime_prev($filename) $mtime
            if { $mtime } {
                set status modified
            } else {
                set status deleted
            }
            set proc_name [lindex $callback_procedure 0]
            #cannot check it with info procs or info commands, it doesn't work in internal interpreters because of protections
            $proc_name $filename $status {*}[lrange $callback_procedure 1 end]
        }
        set after_id($filename) [after $ms [list gid_cross_platform::track_file $filename $ms $callback_procedure]]
        update ;#to allow after to be processed, else the callback_procedure could be invoked later than the true end
    }
}

proc gid_cross_platform::track_file_cancel { filename } {
    variable cancel_track
    set cancel_track($filename) 1
}

proc gid_cross_platform::file_writable { path} {
    # like file writable
    # file writable return 0 for Cloud Paths in Windows
    # file writable return 1 for Cloud Paths in Linux
    if { [ GidDataManager::isCloudPath $path]} {
        # check if it is shared read only...
        # as file stat $path info ; $info(mode) will always return writable despite being SharedReadOnly        
        #
        #trim left part of path e.g. Windows Z:/hello.gid/hello.geo -> hello.gid/hello.geo
        set left_part [GidDataManager::TrimLeftCloudPath $path]
        if { [llength $left_part] } {
            set err [ catch { set isSharedReadOnly [ GidDataManager::isSharedReadOnly $left_part] } err_txt]
            if { $err} {
                # path not present
                # W "error in isSharedReadOnly = $err_txt"
                set is_writable 0
            } else {
                set is_writable [ expr !$isSharedReadOnly]
            }
        } else {
            # e.g. gid_cross_platform::file_writable Z:
            # I cannot ask if is shared read only, I don't have any existent filename!!
            # assume 1 to try to write, if fail is better that not to try
            set is_writable 1
        }
        # May be the mount point has been mounted read-only
        if { $is_writable} {
            if { $::tcl_platform(platform) != "windows"} {
                # on Windows 'file writable' on cloud folder always returns '0'
                set is_writable [ file writable $path]
            }
        }
    } else {
        set is_writable [ file writable $path]
    }
    return $is_writable
}

########## COMPRESSION #####################

# this proc do not require an unzip.exe external file
proc gid_cross_platform::gid_zip { zipfile {file_to_add ""} {run_as_administrator 0} } {
    if { $run_as_administrator } {
        set exe [GidUtils::GetFullPathGiDExe]
        set filename_run_as_administrator_procs [file join [gid_filesystem::get_folder_standard scripts] run_as_administrator_procs.tcl]
        gid_cross_platform::run_as_administrator $exe -tclsh $filename_run_as_administrator_procs ::gid_zip $zipfile $file_to_add    
    } else {
        ::gid_zip $zipfile $file_to_add
    }
}

# this proc do not require an unzip.exe external file
proc gid_cross_platform::gid_unzip { zipfile {extraction_dir ""} {run_as_administrator 0} } {
    if { $run_as_administrator } {
        set exe [GidUtils::GetFullPathGiDExe]
        set filename_run_as_administrator_procs [file join [gid_filesystem::get_folder_standard scripts] run_as_administrator_procs.tcl]
        gid_cross_platform::run_as_administrator $exe -tclsh $filename_run_as_administrator_procs ::gid_unzip $zipfile $extraction_dir    
    } else {
        ::gid_unzip $zipfile $extraction_dir
    }
}

#compress a .gz
proc gid_cross_platform::gzip { filename {filename_gz ""} } {
    if { [file exists $filename] } {
        set f [open $filename "rb"]
        set data [read $f]
        close $f
        if { $filename_gz == "" } {
            set filename_gz ${filename}.gz
        }
        set f_out [open $filename_gz wb]
        zlib push gzip $f_out
        puts -nonewline $f_out $data
        close $f_out
    }
}

#uncompress a .gz
proc gid_cross_platform::gunzip { filename_gz {filename ""}} {
    if { [file exists $filename_gz] } {
        set f [open $filename_gz "rb"]
        zlib push gunzip $f
        set data [read $f]
        close $f
        if { $filename == "" } {
            if { [file extension $filename_gz] == ".gz" } {
                set filename [file rootname $filename_gz]
            } else {
                error "an outputfilename must be provided or the input file must be like the input witouut .gz extension"
            }
        }
        set f_out [open $filename wb]
        puts -nonewline $f_out $data
        close $f_out
    }
}

########## OTHER #####################
