package provide gid_json 1.1

namespace eval gid_json {
    variable _debug        0
    variable _debug_indent 0
}

proc gid_json::SetDebug { value} {
    variable _debug
    set _debug $value
}

proc gid_json::printDebug { txt} {
    variable _debug
    variable _debug_indent
    if { $_debug} {
        W "gid_json debug : [ string repeat { } $_debug_indent]$txt"
    }
}

proc gid_json::increaseDebugIndent {} {
    variable _debug
    variable _debug_indent
    if { $_debug} {
        incr _debug_indent 4
    }
}
proc gid_json::decreaseDebugIndent {} {
    variable _debug
    variable _debug_indent
    if { $_debug} {
        incr _debug_indent -4
        # string repeat { } of negative values = 0
        # if { $_debug_indent < 0} {
        #     set _debug_indent 0
        # }
    }
}

# as in $gid/scripts/dev_kitl.tcl  GidUtils::GetInternalRepresentation
proc gid_json::GetInternalRepresentation { x } {
    set type ""
    # if $x is a command like 'text', 'button' or ... 
    # ::tcl::unsupported::representation returns a wrong type like
    # list, localVarName, parsedVarName, ...
    set type string
    if { [ info commands $x] == ""} {
        regexp {^value is a (.*?) with a refcount} [::tcl::unsupported::representation $x] -> type
    }
    return $type
}

proc gid_json::stringlist2json { value} {
    set lst [ list]
    foreach item $value {
        lappend lst [ gid_json::string2json $item]
    }
    return [ gid_json::list2json $lst]
}

proc gid_json::numericArray2json { obj} {
    set num_items [ objarray length $obj]
    set ll [ list]
    for { set idx_item 0} { $idx_item < $num_items} { incr idx_item} {
        lappend ll [ expr [ objarray get $obj $idx_item]]
    }
    return [ gid_json::list2json $ll]
}

proc gid_json::charArray2json { obj} {
    set num_items [ objarray length $obj]
    set ll [ list]
    for { set idx_item 0} { $idx_item < $num_items} { incr idx_item} {
        lappend ll "[ objarray get $obj $idx_item]"
    }
    return [ gid_json::list2json $ll]
}

proc gid_json::object2json { value} {
    gid_json::increaseDebugIndent

    set val_type [ gid_json::GetInternalRepresentation $value]
    set json_value $value
    gid_json::printDebug "obj2json val_type = $val_type"
    switch $val_type {
        "dict" {
            set json_value [ gid_json::dict2json $value]
        }
        "list" {
            set json_value [ gid_json::list2json $value]
        }
        "int" {
            set json_value $value
        }
        "double" {
            set json_value $value
        }
        "chararray" -
        "shortarray" -
        "intarray" -
        "longarray" -
        "longlongarray" -
        "floatarray" -
        "doublearray" -
        "intarray" {
            # all of these are objarrays
            set json_value [ gid_json::numericArray2json $value]
        }
        "bytearray" {
            # this is Tcl's binary array ( binary encode/decode)
            # already [ binary encode base64 $value] outside here, or at least should be
            set json_value \"$value\"
        }
        default {
            # may be some special cases can be handles like
            # bytearray
            # set objarray_types [ list chararray shortarray intarray longarray longlongarray floatarray doublearray]
            # set str_copy $value
            # if { ( $val_type != "string") && ( $val_type != "pure string")} {
            #     W "    gid_json::object2json type '$val_type' handled as value2json.... [ string range $str_copy 0 100]"
            # }
            set json_value [ gid_json::value2json $value]
        }
    }
    gid_json::printDebug "obj2json json_value = $json_value"
    gid_json::decreaseDebugIndent
    return $json_value
}

proc gid_json::list2json { value} {
    # ::json::list2json does not add \"\" to strings.
    # return [ ::json::list2json $value]
    set listVal [ lmap item $value { gid_json::object2json $item}]
    return "\[[join $listVal ", "]\]"
}

proc gid_json::string2json { value} {
    # regsub -all {\n} $value {\\n} escape_new_lines
    # return \"$escape_new_lines\"
    set str_utf8 [ encoding convertto utf-8 $value]
    set str_escaped [ string map {
        \b "\\\\b"    \f "\\\\f"    \n "\\\\n"    \r "\\\\r"    \t "\\\\t"    \v "\\\\v"
    } $str_utf8 ]
    return \"$str_escaped\"
}

proc gid_json::json2string_plain { value} {
    # regsub -all {\n} $value {\\n} escape_new_lines
    # return \"$escape_new_lines\"
    set str_local [ encoding convertfrom utf-8 $value]
    set str_unescaped [ string map {
        "\\b" \b    "\\f" \f    "\\n" \n    "\\r" \r    "\\t" \t    "\\v" \v
    } $str_local ]
    return \"$str_unescaped\"
}

proc gid_json::json2string { value} {
    return \"[ gid_json::json2string_plain $value ]\"
}

proc gid_json::value2json { value } {
    # val may be a number, string or bare word (true|false|null)
    # undefined is not a valid json-value
    set check_special_keyworkds 1
    # checking here will turn value into a string....
    if { $check_special_keyworkds && ( [ regexp {^(?:true|false|null)$} $value] == 1) } {
        # leave val as it is
        return $value
    }
    
    # check if it's already an already json formated list [...]
    set is_json_array [ regexp {^\[.*\]$} $value]
    set json_value $value
    if { $is_json_array != 1} {
        # it is not: if it is a list, do list2json, otherwise string2json
        set is_list  [ regexp {^\{.*\}$} $value]
        if { $is_list} {
            set json_value [ gid_json::list2json $value]
        } else {
            # check if it already has \"\"
            set is_json_string [ regexp {^\".*\"$} $value]
            if { $is_json_string != 1} {
                set json_value [ gid_json::string2json $value]
            }
        }
    }
    return $json_value
}

# will decorate with indent and \n if 'enable_indent > 0'
proc gid_json::dict2json { dictVal { enable_indent 0 }} {
    # XXX: Currently this API isn't symmetrical, as to create proper
    # XXX: JSON text requires type knowledge of the input data
    set json ""
    set prefix ""
    set margin ""

    # empty dictionary, return "" ?or may be return {} ?
    if { [ dict size $dictVal] == 0} {
        return "\"\""
    }

    if { $enable_indent} {
        set margin [ string repeat "    " $enable_indent]
    }

    gid_json::increaseDebugIndent
    gid_json::printDebug "dict2json dict for ...."
    dict for {key val} $dictVal {
        # key must always be a string, 
        set val_type [ gid_json::GetInternalRepresentation $val]
        gid_json::printDebug "dict2json    key = $key    val_type = $val_type"
        if { $val_type == "dict"} {
            set new_indent 0
            if { $enable_indent} {
                set new_indent [ expr $enable_indent + 1]
            }
            set val [ gid_json::dict2json $val $new_indent]
        } else {
            set val [ gid_json::object2json $val]
        }
        append json $prefix \n $margin "\"$key\": $val"
        set prefix ,
    }
    gid_json::decreaseDebugIndent

    if { $enable_indent > 0} {
        incr enable_indent -1
        set margin [ string repeat "    " $enable_indent]
        set return_json "\{${json}\n${margin}\}"
    } else {
        set return_json "\{${json}\}"
        set return_json [ regsub -all {\n} $return_json {}]
    }
    return $return_json
}

proc gid_json::json2dict { json_object} {
    # at the moment we are using tcllib/json's json2dict
    package require json
    return [ gid_json::json2string_plain [ json::json2dict $json_object]]
}
