package netcdf 1.0
====

cdftcl is an interface to the NetCDF library for tcl by Dan Schmitt

Implement commands
=====
`ncdf`
`ncdfdim`
`ncdfvar`
`ncdfatt`

`ncdf`
=====
All the ncdf file operations are implemented as a single command with options.

### Syntax
`ncdf` \<option\> ?args?  
\<option\<: create, open, close, sync, abort, setfill, inquire, enddef, redef

`ncdf` create \<file\> ?clobber?  
`ncdf` open \<file\> ?readonly?  
`ncdf` close \<CID\>  
`ncdf` inquire \<CID\> dims|atts|vars|recs  
`ncdf` sync \<CID\>  
`ncdf` setfill \<CID\<|nofill  

 
### Examples

set cid  [`ncdf` create foo.ncd]      # create file foo.ncd  
set cid2 [`ncdf` open foo2.ncd]       # open existing foo2.ncd  
`ncdf` close $cid2                    # close foo2.ncd  
`ncdf` sync $cid                      # commit changes to foo.ncd  
`ncdf` setfill $cid -1                # set a fill value  
`ncdf` enddef $cid                    # leave file definition mode  
`ncdf` redef $cid                     # enter file definition mode  
set vars [`ncdf` inquire $cid vars]   # get list of variables in foo.ncd  
set dims [`ncdf` inquire $cid dims]   # get list of dimensions in foo.ncd  
set atts [`ncdf` inquire $cid atts]   # get global attributes from foo.ncd  
set recv [`ncdf` inquire $cid recs]    # get the unlimited (record) dimension   
`ncdf` abort $cid                     # close foo.ncd (no changes)  


`ncdfdim`
=====
All the ncdf dimension operations are implemented as a single command with options.

### Syntax
`ncdfdim` \<CID\> \<option\> ?args?  
\<option\<: create, inquire, rename
  
`ncdfdim` \<CID\> create \<dimension_name\> \<length\>  
`ncdfdim` \<CID\> inquire \<dimension_name\>  
`ncdfdim` \<CID\> rename \<dimension_name\> \<dimension_new_name\>  

### Examples
`ncdfdim` $Cid create "lat" 20             # create a 20 length Latitude dim  
`ncdfdim` $Cid create "Longitute" 10       # create a 10 length Longitude dim  
`ncdfdim` $Cid create "Time" -1            # create an unlimited time dim  
`ncdfdim` $Cid inquire "Depth"             # get the size of a "Depth" dim  
`ncdfdim` $Cid rename "Longitude" "lon"    # rename the longitude dimension to lon  

`ncdfatt`
=====
All the ncdf attribute operations are implemented as a single command with options.


### Syntax
`ncdfatt` \<CID\> \<var\> \<option\> ?args?  
\<option\<: put, get, copy, rename, delete, inquire

`ncdfatt` \<CID\> \<var\> put \<name\> byte|char|short|long|float|double \<list of values\>  
`ncdfatt` \<CID\> \<var\> get \<name\>  
`ncdfatt` \<CID\> \<var\> copy \<name\> \<oCid\> \<oVar\>  
`ncdfatt` \<CID\> \<var\> rename \<name\> \<newname\>  
`ncdfatt` \<CID\> \<var\> delete \<name\>  
`ncdfatt` \<CID\> \<var\> inquire \<name\> type|length  

### Examples

`ncdfatt` $cid Radiation put Units char {L a n g l e y s}     # adds the char array Langleys as the attribute Units for the variable Radiation  
`ncdfatt` $cid Radiation get Units                            # Get the value of the Units attribute for Radiation  
`ncdfatt` $cid Radiation rename Units units                   # Rename the attribute Units to units  
`ncdfatt` $cid Radiation inquire units type                   # get the type of the units attribute  
`ncdfatt` $cid Radiation inquire units length                 # get the number of values the units attribute has  
`ncdfatt` $cid Radiation copy units $cid2 Radiation           # copy the units attribute to the Radiation variable in the file referenced by $cid2  
`ncdfatt` $cid Radiation delete units                         # delete the units attribute from Radiation  


`ncdfvar`
=====
All the ncdf dimension operations are implemented as a single command with options.

### Syntax
`ncdfvar` \<CID\> \<option\> ?args?  
\<option\<: create, inquire, put, get

`ncdfvar` \<CID\> create \<name\> \<type\> \<list of dims\>  
`ncdfvar` \<CID\> inquire \<varname\> type|dims|atts  
`ncdfvar` \<CID\> put \<var\> \<list of dim lists\> \<list of values\>  
`ncdfvar` \<CID\> get \<var\> \<list of dim lists\>  

### Examples

`ncdfvar` $cid create Radiation double Time Lon Lat                 # create a double variable Radation with Time,Lon and Lat dimensions (3d array)  
`ncdfvar` $cid put Rad {{0 0 0} {1 0 0} {2 0 0}} {23.1 88.9 27.5}   # put 23.1 in Rad ( Time 0, Lon 0, Lat 0 )  
                                                                  # put 88.9 in Rad ( Time 1, Lon 0, Lat 0 )  
                                                                  # put 27.5 in Rad ( Time 2, Lon 0, Lat 0 )  
`ncdfvar` $cid get Rad {{1 0 0} {2 0 0}}                            # get Rad values for Time 1 and Time 2 for Lon 0 Lat 0  
`ncdfvar` $cid inquire Rad atts                                     # get attribute name list for variable Rad  
`ncdfvar` $cid inquire Rad type                                     # get type for variable Rad  
`ncdfvar` $cid inquire Rad dims                                     # get dimension list for variable Rad  
