
proc test_objarray { } {
    set fail 0
    package require objarray
    set obj [objarray new intarray 5 1]    
    if { [objarray sum $obj] != 5 } {
        incr fail
    }
    set obj [objarray new intarray -values {6 3 8}]    
    if { [objarray sum $obj] != 17 } {
        incr fail
    }
    set obj2 [objarray clone $obj]   
    if { [objarray sum $obj2] != 17 } {
        incr fail
    }
    set obj [objarray new_from_to shortarray 20000 20125]
    set length [objarray length $obj]
    if { $length != 126 } {
        incr fail
    }   
    if { [objarray sum $obj] != 2527875 } {
        incr fail
    }
    set obj [objarray new intarray 3 0]
    objarray set $obj {6 3 8}
    if { [objarray sum $obj] != 17 } {
        incr fail
    }
    objarray set $obj 2 25
    set value [objarray get $obj 2]
    if { $value != 25 } {
        incr fail
    }
    set obj3 [objarray concat $obj $obj2]
    if { $obj3 != "6 3 25 6 3 8" } {
        incr fail
    }
    set obj [objarray new_from_to intarray 2 5]
    set obj2 [objarray new_from_to intarray 4 6]
    set obj3 [objarray insert $obj end-1 $obj2]
    if { $obj3 != "2 3 4 5 6 4 5" } {
        incr fail
    }
    set obj_to_insert [objarray new_from_to intarray 1 5]
    set obj4 [objarray replace $obj3 2 4 $obj_to_insert]
    if { $obj4 != "2 3 1 2 3 4 5 4 5" } {
        incr fail
    }
    set obj [objarray new intarray 5 1]
    set obj2 [objarray new intarray 5 2]
    set obj_deinterleaved [objarray concat $obj $obj2]
    if { $obj_deinterleaved != "1 1 1 1 1 2 2 2 2 2" } {
        incr fail
    }
    set obj_interleaved [objarray interleave $obj_deinterleaved 2]
    if { $obj_interleaved != "1 2 1 2 1 2 1 2 1 2" } {
        incr fail
    }
    set obj [objarray new intarray -values {10 15 3 15 20 3}]
    set old_ids [objarray new intarray -values {3 10 15 20}]
    set obj_renumbered [objarray renumber $obj $old_ids]
    if { $obj_renumbered != "1 2 0 2 3 0" } {
        incr fail
    }
    set obj [objarray new_from_to intarray 1 5]
    set obj2 [objarray range $obj 1 end-2]
    if { $obj2 != "2 3" } {
        incr fail
    }
    set obj [objarray new_from_to intarray 2 5]
    set pos [objarray search -sorted $obj 4]
    if { $pos != 2 } {
        incr fail
    }
    set obj [objarray new intarray -values {6 3 5 5 8}]    
    objarray sort $obj ;#obj is modified
    if { $obj != "3 5 5 6 8" } {
        incr fail
    }
    objarray sort -unique $obj ;#obj is modified
    if { $obj != "3 5 6 8" } {
        incr fail
    }
    objarray incr $obj 2 ;#obj is modified
    if { $obj != "5 7 8 10" } {
        incr fail
    }
    objarray incr $obj 2 1 ;#obj is modified
    if { $obj != "5 9 8 10" } {
        incr fail
    }
    objarray scale $obj 3 ;#obj is modified
    if { $obj != "15 27 24 30" } {
        incr fail
    }
    set sum [objarray sum $obj]
    if { $sum != 96 } {
        incr fail
    }
    set min [objarray minimum $obj]
    if { $min != 15 } {
        incr fail
    }
    set max [objarray maximum $obj]
    if { $max != 30 } {
        incr fail
    }
    set obj1 [objarray new_from_to intarray 2 5]
    set obj2 [objarray new_from_to intarray 3 8]
    #objects that are in obj1 and obj2
    set obj3 [objarray intersection -sorted $obj1 $obj2]
    if { $obj3 != "3 4 5" } {
        incr fail
    }
    #objects that are in obj1 or obj2
    set obj3 [objarray union -sorted $obj1 $obj2]
    if { $obj3 != "2 3 4 5 6 7 8" } {
        incr fail
    }
    objarray reverse $obj3
    if { $obj3 != "8 7 6 5 4 3 2" } {
        incr fail
    }
    objarray reverse $obj3
    if { $obj3 != "2 3 4 5 6 7 8" } {
        incr fail
    }
    #objects that are in obj1 but not in obj2
    set obj3 [objarray substraction -sorted $obj1 $obj2]
    if { $obj3 != "2" } {
        incr fail
    }

    set obj_a [ objarray new intarray -values { 1 2 3 4}]
    # W [ set obj_a_b64 [ binary encode base64 [ objarray get_binary $obj_a]]]
    set bin [ objarray get_binary $obj_a]
    set len [ string length $bin]
    binary scan $bin i4 lst_ints
    # check if binary we have 1 2 3 4:
    for { set idx 0} { $idx < [ expr $len / 4] } { incr idx} {
        set cur_int [ lindex $lst_ints $idx]
        # W "a cur_int = $cur_int"
        if { [ expr $idx + 1] != $cur_int} {
            incr fail
        }
    }
    set obj_b [ objarray new intarray -binary $bin]
    # W [ set obj_b_b64 [ binary encode base64 [ objarray get_binary $obj_b]]]
    set bin [ objarray get_binary $obj_b]
    set len [ string length $bin]
    binary scan $bin i4 lst_ints
    # check if binary we have 1 2 3 4:
    for { set idx 0} { $idx < [ expr $len / 4] } { incr idx} {
        set cur_int [ lindex $lst_ints $idx]
        # W "b cur_int = $cur_int"
        if { [ expr $idx + 1] != $cur_int} {
            incr fail
        }
    }

    set obj_c [ objarray new intarray -binary [ binary decode base64 AQAAAAIAAAADAAAABAAAAA==]]
    # W [ set obj_c_b64 [ binary encode base64 [ objarray get_binary $obj_c]]]
    set bin [ objarray get_binary $obj_c]
    set len [ string length $bin]
    binary scan $bin i4 lst_ints
    # check if binary we have 1 2 3 4:
    for { set idx 0} { $idx < [ expr $len / 4] } { incr idx} {
        set cur_int [ lindex $lst_ints $idx]
        # W "c cur_int = $cur_int"
        if { [ expr $idx + 1] != $cur_int} {
            incr fail
        }
    }


    set obj_1 [objarray new doublearray -values {3.5 2.1}]
    set obj_2 [objarray new doublearray -values {0.3 3.5}]
    set obj_3 [objarray new doublearray -values {2.6 0.4}]
    set obj_sum [objarray vector_sum $obj_1 $obj_2 $obj_3]
    if { $obj_sum != "6.4 6.0" } {
        incr fail
    }

    #without -type default is doublearray (but if first argument is an objarray its type is used instead)
    set obj_sum [objarray vector_sum {1.5 2.3} {0.5 0.7}]
    if { [objarray type $obj_sum] != "doublearray"} {
        incr fail  
    }
    if { $obj_sum != "2.0 3.0" } {
          incr fail
    }

    #arguments converted to explicit type
    set obj_sum [objarray vector_sum -type floatarray {1.5 2.3} {0.5 0.7}]
    if { [objarray type $obj_sum] != "floatarray"} {
        incr fail  
    }
    if { $obj_sum != "2.0 3.0" } {
        incr fail
    }
    
    #convert mix: a doublearray argument to floatarray and a string to floatarray
    set obj_sum [objarray vector_sum -type floatarray [objarray new doublearray -values {1.5 2.3}] {0.5 0.7}]
    if { [objarray type $obj_sum] != "floatarray"} {
        incr fail  
    }
    if { $obj_sum != "2.0 3.0" } {
        incr fail
    }
    
    #without -type but if first argument is an objarray its type is used instead
    set obj_sum [objarray vector_sum [objarray new floatarray -values {1.5 2.3}] {0.5 0.7}]
    if { [objarray type $obj_sum] != "floatarray"} {
        incr fail  
    }
    if { $obj_sum != "2.0 3.0" } {
        incr fail
    }
    
    
    return $fail
}
