
package require treectrl

namespace eval posttree {
    # variables
    variable posttree_wid
    variable _images

    #from demo.tcl
    # http://wiki.tcl.tk/950
    proc sbset {sb first last} {
	# Get infinite loop on X11
	if { $::tcl_platform(platform) ne "unix"} {
	    if {$first <= 0 && $last >= 1} {
		grid remove $sb
	    } else {
		grid $sb
	    }
	}
	$sb set $first $last
	return
    }

    proc ShouldShowLines {T} {
	# if {![$T cget -usetheme]} {
     	return 1
	# }
	# switch -- [$T theme platform] {
	# 	aqua -
	# 	gtk {
	# 	    return 0
	# 	}
	# }
	# return 1
    }

    proc CreateTreeAndScrollbarsInAFrame {f h v} {
	frame $f -borderwidth 0 -height 400
	treectrl $f.t -highlightthickness 0 -borderwidth 0
	if { $::tcl_platform(platform) == "unix"} {
	    # $f.t configure -headerfont [$f.t cget -font]
	}
	$f.t configure -xscrollincrement 20 -xscrollsmoothing 1
	$f.t debug configure -enable no -display yes -erasecolor pink \
	    -drawcolor orange -displaydelay 30 -textlayout 0 -data 0 -span 0
	if {$h} {
	    ::ttk::scrollbar $f.sh -orient horizontal -command "$f.t xview"
	    $f.t notify bind $f.sh <Scroll-x> { posttree::sbset %W %l %u }
	    bind $f.sh <ButtonPress-1> "focus $f.t"
	}
	if {$v} {
	    ::ttk::scrollbar $f.sv -orient vertical -command "$f.t yview"
	    $f.t notify bind $f.sv <Scroll-y> { posttree::sbset %W %l %u }
	    bind $f.sv <ButtonPress-1> "focus $f.t"
	}
	grid columnconfigure $f 0 -weight 1
	grid rowconfigure $f 0 -weight 1
	grid configure $f.t -row 0 -column 0 -sticky news
	if {$h} {
	    grid configure $f.sh -row 1 -column 0 -sticky we
	}
	if {$v} {
	    grid configure $f.sv -row 0 -column 1 -sticky ns
	}

	bind $f.t <Control-Shift-ButtonPress-1> {
	    ::TreeCtrl::MarqueeBegin %W %x %y
	    set DebugExpose(x1) %x
	    set DebugExpose(y1) %y
	    break
	}
	bind $f.t <Control-Shift-Button1-Motion> {
	    ::TreeCtrl::MarqueeUpdate %W %x %y
	    set DebugExpose(x2) %x
	    set DebugExpose(y2) %y
	    break
	}
	bind $f.t <Control-Shift-ButtonRelease-1> {
	    ::TreeCtrl::MarqueeEnd %W %x %y
	    %W debug expose $DebugExpose(x1) $DebugExpose(y1) $DebugExpose(x2) $DebugExpose(y2)
	    break
	}
	return
    }

    proc ReadImages {} {
	variable _images

	# set act_theme [ gid_themes::GetCurrentTheme]
	# if { $act_theme == ""} {
	#     set act_theme GiD_classic
	# }
	# set folder [ file join [ gid_themes::GetThemesBaseFolder] $act_theme images PostTree]
	set folder [ file join  $::GIDDEFAULTTCL posttree images]

	# TODO: get images according to current theme	

	foreach img [ list post-on post-off post-action-open post-action-closed post-info \
			  post-blank-color-button-14] {
	    if { ![ info exists _images($img)]} {
		set _images($img) [ image create photo -file [ file join $folder ${img}.gif]]
	    }
	}

	foreach img [ list bodylines transparent interior cullfrontfaces send_to_back] {
	    if { ![ info exists _images(post-style-$img)]} {
		set _images(post-style-$img) [ image create photo -file \
						   [ file join $folder post-style-${img}.gif]]
	    }
	}
    }

    proc GetImage { name} {
	variable _images
	return $_images($name)
    }

    proc CreateStyles { T} {
	variable _images

	# # this does not work:
	# set w [ winfo toplevel $T]
	# set ::SystemButtonFace [$w cget -highlightbackground]
	# set ::SystemHighlight [$w cget -selectbackground]
	# set ::SystemHighlightText [$w cget -selectforeground]
	# set ::SystemHighlight  [ CCColorActivo orange 129]
	set ::SystemHighlight  [ CCColorActivo orange 65]
	set ::SystemHighlightText black

	# read images
	ReadImages
	set height [ font metrics [$T cget -font] -linespace]
	if {$height < 18} {
	    set height 18
	}

	# create basic elements
	$T element create elemTxtName text -wrap none \
	    -fill [list $::SystemHighlightText {selected focus}]
	$T element create elemFrame window
	$T element create elemImgOnOff image \
	    -image [ list $_images(post-on) {} $_images(post-off) {}]
	$T element create elemImgAction image \
	    -image [ list $_images(post-action-open) {open} $_images(post-action-closed) {}]
	$T element create elemImgClosedAction image -image $_images(post-action-closed)
	$T element create elemImgInfo image -image $_images(post-info)
	$T element create elemImgAny image
	$T element create elemTxtAny text

	#create styles
	# styFolder - just the folder name
	set S [ $T style create styFolder]
	$T style elements $S {elemTxtName}
	$T style layout $S elemTxtName -minwidth 12 -padx {0 4} -expand ns -squeeze x -height $height

	# styActionFolder - open/closed action img + folder name
	set S [$T style create styActionFolder]
	$T style elements $S {elemImgAction elemTxtName}
	$T style layout $S elemImgAction -padx {0 4} -expand ns
	$T style layout $S elemTxtName -minwidth 12 -padx {0 4} -expand ns -squeeze x -height $height
	
	# styOnOffColor - mesh entry: on/off image + color button + mesh name
	set S [$T style create styOnOffColor]
	$T style elements $S {elemImgOnOff elemFrame elemTxtName}
	$T style layout $S elemImgOnOff -padx {4 0} -expand ns
	$T style layout $S elemFrame -padx {2 2} -expand ns -height $height
	$T style layout $S elemTxtName -minwidth 12 -padx {0 4} -expand ns -squeeze x -height $height

	# styClosedActionFrame - leaf with frame:  with buttons with mesh options OR button to open layer window
	set S [$T style create styClosedActionFrame]
	$T style elements $S {elemImgClosedAction elemFrame}
	$T style layout $S elemImgClosedAction -padx {0 4} -expand ns
	$T style layout $S elemFrame -padx {0 4} -expand ns 

	#styInfo - almost leaf with : info image + text
	set S [$T style create styInfo]
	$T style elements $S {elemImgInfo elemTxtName}
	$T style layout $S elemImgInfo -padx {0 4} -expand ns
	$T style layout $S elemTxtName -minwidth 12 -padx {0 4} -expand ns -squeeze x -height $height

	# styAny - leaf with text and posibly an image
	set S [$T style create styAny]
	$T style elements $S {elemTxtAny elemImgAny}
	$T style layout $S elemTxtAny -padx 6 -expand ns -height $height
	$T style layout $S elemImgAny -padx {0 4} -expand ns
    }

    proc MyToogleCollapse { T item} {
	set is_open [ $T item isopen $item]
	set new_state expand
	if { $is_open} {
	    set new_state collapse
	}
	$T item $new_state $item
    }

    proc MyToogleOnOff { T item} {
	variable _images
	set act [ $T item element cget $item postItem elemImgOnOff -image]
	set new $_images(post-on)
	if { $act == $_images(post-on)} {
	    set new $_images(post-off)
	}
	$T item element configure $item postItem elemImgOnOff -image $new
    }

    proc MyDoubleButton1 {T x y} {
	variable ::TreeCtrl::Priv
	focus $T
	set id [$T identify $x $y]
	set Priv(buttonMode) ""
	
	if {$id eq ""} {
	    # $T selection clear
	    ::TreeCtrl::DoubleButton1 $T $x $y
	    
	    # Click in header
	} elseif {[lindex $id 0] eq "header"} {
	    # ::TreeCtrl::ButtonPress1 $T $x $y
	    ::TreeCtrl::DoubleButton1 $T $x $y
	    
	    # Click in item
	} else {
	    lassign $id where item arg1 arg2 arg3 arg4
	    switch $arg1 {
		button {
		    ::TreeCtrl::DoubleButton1 $T $x $y
		}
		line {
		    ::TreeCtrl::DoubleButton1 $T $x $y
		}
		column {
		    # elemImgOnOff is used to turn mesh on or off
		    # so do not confuse the user by open folder by double-click
		    if { $arg4 != "elemImgOnOff"} {
			if { $arg2 == "0"} {
			    # column with name ...
			    MyToogleCollapse $T $item
			} else {
			    ::TreeCtrl::DoubleButton1 $T $x $y
			}
		    }
		}
	    }
	}
	# ::TreeCtrl::DoubleButton1 $T $x $y
    }
    
    proc MyButton1 {T x y} {
	variable ::TreeCtrl::Priv
	focus $T
	set id [$T identify $x $y]
	set Priv(buttonMode) ""
	
	# Click outside any item
	if {$id eq ""} {
	    $T selection clear
	    
	    # Click in header
	} elseif {[lindex $id 0] eq "header"} {
	    ::TreeCtrl::ButtonPress1 $T $x $y
	    
	    # Click in item
	} else {
	    #           item   37 column 0   elem elemImgOnOff
	    lassign $id where item arg1 arg2 arg3 arg4
	    switch $arg1 {
		button {
		    ::TreeCtrl::ButtonPress1 $T $x $y
		}
		line {
		    ::TreeCtrl::ButtonPress1 $T $x $y
		}
		column {
		    # # may confuse the user because it disables the selection
		    
		    if { $arg4 == "elemImgOnOff"} {
			MyToogleOnOff $T $item
		    } else {
			::TreeCtrl::ButtonPress1 $T $x $y
		    }
		}
	    }
	}
	return
    }
    
    proc CreateRawTree { w} {
	variable posttree_wid
	CreateTreeAndScrollbarsInAFrame $w 1 1
	set T $w.t
	
	# set posttree_wid $w
	set posttree_wid $T

	# configure tree
	$T configure  -selectmode extended \
	    -showroot yes -showrootbutton no -showbuttons yes \
	    -showlines [ShouldShowLines $T] -linestyle solid \
	    -scrollmargin 16 -xscrolldelay "500 50" -yscrolldelay "500 50"
	
	# create columns: just on
	$T column create -expand yes -weight 4 -text "Mesh info" -tags postItem \
	    
	# -itembackground {#e0e8f0 {}}
	
	$T configure -treecolumn postItem
	
	gid_themes::ConfigureHeaders $T    
	
	CreateStyles $T
	
	gid_themes::ApplyHeadersConfig $T    
	
	bind PostTree <Double-ButtonPress-1> {
	    posttree::MyDoubleButton1 %W %x %y
	    # ::TreeCtrl::DoubleButton1 %W %x %y
	    break
	}
	bind PostTree <ButtonPress-1> {
	    set ::TreeCtrl::Priv(selectMode) set
	    posttree::MyButton1 %W %x %y
	    break
	}

	bindtags $T [list $T PostTree TreeCtrl [winfo toplevel $T] all]

	return $T
    }

    #procedures
    proc create { w} {
	variable posttree_wid
	set posttree_wid $w
	
	return [ CreateRawTree $w]
    }
}

package provide posttree 1.0

# main
