package require objarray

proc NewRtree { dimension } {
    set rtree [rtree_new_simple $dimension]
    return $rtree
}

proc DeleteRtree { rtree } {
    rtree_free $rtree
}

proc NewRectangle {x0 y0 x1 y1} {
    set rect [new_doubleArray 4]
    SetRectangle $rect $x0 $y0 $x1 $y1
    return $rect
}

proc SetRectangle { rect x0 y0 x1 y1} {
    doubleArray_setitem $rect 0 $x0
    doubleArray_setitem $rect 1 $y0
    doubleArray_setitem $rect 2 $x1
    doubleArray_setitem $rect 3 $y1
}

proc DeleteRectangle { rect } {
    delete_doubleArray $rect
}

proc NewBox {x0 y0 z0 x1 y1 z1} {
    set rect [new_doubleArray 6]
    SetBox $rect $x0 $y0 $z0 $x1 $y1 $z1
    return $rect
}

proc SetBox { rect x0 y0 z0 x1 y1 z1} {
    doubleArray_setitem $rect 0 $x0
    doubleArray_setitem $rect 1 $y0
    doubleArray_setitem $rect 2 $z0
    doubleArray_setitem $rect 3 $x1
    doubleArray_setitem $rect 4 $y1
    doubleArray_setitem $rect 5 $z1
}

proc RtreeInsertPoints2D { rtree points ids } {
    set rect [NewRectangle 0 0 0 0]
    foreach {x y} $points id $ids {
        SetRectangle $rect $x $y $x $y
        rtree_insert_simple $rtree $rect $id
    }
    DeleteRectangle $rect
    return 0
}

proc RtreeInsertPoints3D { rtree points ids } {
    set rect [NewBox 0 0 0 0 0 0]
    foreach {x y z}  $points id $ids {
        SetBox $rect $x $y $z $x $y $z
        rtree_insert_simple $rtree $rect $id
    }
    DeleteRectangle $rect
    return 0
}

proc RtreeInsertPoint { rtree coordinates id } {
    if { [llength $coordinates] == 2 } {
        lassign $coordinates x y
        set rect [NewRectangle $x $y $x $y]
    } else {
        lassign $coordinates x y z
        set rect [NewBox $x $y $z $x $y $z]
    }
    set result [rtree_insert_simple $rtree $rect $id]
    DeleteRectangle $rect
    return $result
}

proc RtreeSearch { rtree rectangle_corners } {
    if { [llength $rectangle_corners] == 4 } {
        lassign $rectangle_corners x0 y0 x1 y1
        set rect [NewRectangle $x0 $y0 $x1 $y1]
    } else {
        lassign $rectangle_corners x0 y0 z0 x1 y1 z1
        set rect [NewBox $x0 $y0 $z0 $x1 $y1 $z1]
    }
    set result_tmp [rtree_search_simple $rtree $rect]
    DeleteRectangle $rect
    set n [result_get_num_items $result_tmp]
    set result_objarray [objarray new intarray $n]
    for {set i 0} {$i<$n} {incr i} {
        objarray set $result_objarray $i [result_get_item $result_tmp $i]
    }
    delete_result $result_tmp
    return $result_objarray
}

proc Rtree_Test2D { } {    
    set rtree [NewRtree 2]
    RtreeInsertPoint $rtree {1.0 1.0} 1
    RtreeInsertPoint $rtree {1.5 2.0} 2
    RtreeInsertPoint $rtree {1.1 0.9} 3
    set result [RtreeSearch $rtree {0.5 0.5 1.5 1.5}]
    DeleteRtree $rtree
    #puts $result
    return $result
}

proc Rtree_Test3D { } {
    set rtree [NewRtree 3]
    set points {1.0 1.0 0.8 1.5 2.0 0.8 1.1 0.9 0.8}
    set ids {1 2 3}
    RtreeInsertPoints3D $rtree $points $ids
    set result [RtreeSearch $rtree {0.5 0.5 0.5 1.5 1.5 1.5}]
    DeleteRtree $rtree
    #puts $result
    return $result
}

#Test3D

