#plugin to Generate a 2D triangle mesh from the current boundary line mesh, using Triangle
package provide triangle 1.0

namespace eval Triangle {
    variable options
    set options(constrained) 0    
    set options(quality) 5.0 ;#minimum angle in degrees (0 to not constraint)
    set options(quadratic) 0
    set options(maxarea) 0.0 ;#0 to not constraint
}

proc Triangle::SetPath { path } {   
    variable options
    set options(path) $path
}

proc Triangle::SetMeshOption { key value } {
    variable options
    if { [info exists options($key)] } {
        set options($key) $value
    } else {
        error "Triangle::SetMeshOption. $key doesn't exists. valid ones [lsort -dictionary [array names options]]"                
    }
}

proc Triangle::GetMeshOption { key } {
    variable options
    return $options($key)
}

proc Triangle::OptionsWin { } {    
    variable options    

    set w .gid.triangle
    InitWindow2 $w -title [_ "Triangle"] -geometryvariable PreTriangleMesherWindowGeom
    set def_back [$w cget -background]
    set f [ttk::frame $w.f]
    
    ttk::checkbutton $f.constrained -text [_ "Constrained"] -variable Triangle::options(constrained)   
    GidHelp $f.constrained [_ "To force the initial lines to be the triangle boundary"]    
                             
    ttk::label $f.lquality -text [_ "Quality"]
    ttk::entry $f.quality -textvariable Triangle::options(quality)
    GidHelp $f.quality [_ "Quality mesh generation. A minimum radius-edge ratio may be specifyed (default 2.0)"]    

    ttk::label $f.lmaxarea -text [_ "Max area"]
    ttk::entry $f.maxarea -textvariable Triangle::options(maxarea)
    GidHelp $f.maxarea [_ "No trianglee is generated whose area is larger than that number"]
    
    ttk::checkbutton $f.quadratic -text [_ "Quadratic"] -variable Triangle::options(quadratic)   
    GidHelp $f.quadratic [_ "Generates second-order elements"]

    #lower buttons
    frame $w.frmButtons -bg [CCColorActivo $def_back]
    
    ttk::button $w.frmButtons.btngenerate -text [_ "Generate"] -command Triangle::GenerateFromCurrentMesh -underline 0
    ttk::button $w.frmButtons.btnclose -text [_ "Close"] -command [list destroy $w] -underline 0
    #SetWidgetsWidthFromText $w.frmButtons.btngenerate $w.frmButtons.btnclose
    
    grid $f.constrained -sticky w -columnspan 2
    grid $f.lquality $f.quality -sticky w
    grid $f.lmaxarea $f.maxarea -sticky w
    grid $f.quadratic -sticky w -columnspan 2

    grid $f -sticky nsew -padx 3 -pady 3
    grid columnconfigure $f 1 -weight 1
    grid rowconfigure $f 10 -weight 1

    grid $w.frmButtons -sticky ews -columnspan 7
    grid $w.frmButtons.btngenerate $w.frmButtons.btnclose -padx 3 -pady 3
    grid columnconfigure $w 0 -weight 1
    grid rowconfigure $w 0 -weight 1
    if { $::tcl_version >= 8.5 } { grid anchor $w.frmButtons center }
    
    focus $w.frmButtons.btnclose
    bind $w <Alt-c> "$w.frmButtons.btnclose invoke"
    bind $w <Escape> "$w.frmButtons.btnclose nvoke"    
} 

proc Triangle::DeleteGiDElements { element_ids } {  
    GiD_Mesh delete element $element_ids
}

proc Triangle::CreateGiDTriangles { created_elements } {  
    foreach element $created_elements {            
        GiD_Mesh create element append triangle 3 $element            
    }    
    return 0
}

proc Triangle::CreateGiDNodesAndTriangles { boundary_nodes created_elements boundary_edges_out } {
    set fail 0
    #nodes are renumbered
    foreach node $boundary_nodes {
        lassign $node i_node x y
        set new_node_num($i_node) [GiD_Mesh create node append [list $x $y]]
    }
    if { 0 } {
        foreach edge $boundary_edges_out {
            lassign $edge n0 n1
            GiD_Mesh create element append linear 2 [list $new_node_num($n0) $new_node_num($n1)]    
        }
    } else {
        foreach element $created_elements {
            lassign $element n0 n1 n2
            GiD_Mesh create element append triangle 3 [list $new_node_num($n0) $new_node_num($n1) $new_node_num($n2)]
        }       
    }
    return $fail
}


proc Triangle::GenerateFromCurrentMesh { } {    
    set boundary_edges [list]
    set boundary_element_ids [list]
    foreach {element_id node_0 node_1 mat} [GiD_Info mesh elements linear] {
        lappend boundary_edges [list $node_0 $node_1]       
        lappend boundary_element_ids $element_id 
    }
    set boundary_nodes [GiD_Info mesh nodes -sublist]
    set extra_edges [list]
    set extra_nodes [list]
    set result [Triangle::GenerateTriangles $boundary_edges $boundary_nodes $extra_edges $extra_nodes]
    lassign $result elements_out nodes_out boundary_edges_out
    if { ![llength $elements_out] } {
        WarnWinText [_ "The mesh can't be generated"]
    } else {    
        Triangle::DeleteGiDElements $boundary_element_ids
        Triangle::CreateGiDTriangles $elements_out
        MeshView 0
        #GiD_Redraw
    }    
    return 0
}

#generate a 2D constraint mesh of triangles with 1 division by line and any interior node
proc Triangle::GenerateMeshFromSurface { surface_id size mesh_variables } {
    set fail 0        
    Triangle::SetMeshOption maxarea [expr ($size*$size)*0.5]
    foreach {key value} $mesh_variables {
        Triangle::SetMeshOption $key $value
    }
    set surface_data [GiD_Geometry -v2 get surface $surface_id]
    set boundary_edges_initial [list]
    set boundary_nodes [list]
    set surface_points [list]
    foreach line_id [lindex $surface_data 3] verso [lindex $surface_data 4] {
        set line_data [GiD_Geometry -v2 get line $line_id]
        lassign [lrange $line_data 2 3] p0 p1
        if { $verso } {
            lappend boundary_edges_initial [list $p1 $p0]
        } else {
            lappend boundary_edges_initial [list $p0 $p1]
        }
        lappend surface_points $p0 $p1
    }
    #renumber from 1
    set i 0
    foreach point_id [lsort -unique -integer $surface_points] {
        incr i
        set point_data [GiD_Geometry -v2 get point $point_id]
        lassign [lrange $point_data 1 2] x y
        lappend boundary_nodes [list $i $x $y]
        set new_node_num($point_id) $i
    }
    set boundary_edges [list]
    foreach edge $boundary_edges_initial {
        lassign $edge n0 n1
        lappend boundary_edges [list $new_node_num($n0) $new_node_num($n1)]
    }    
    unset boundary_edges_initial
    set extra_edges [list]
    set extra_nodes [list]
    set result [Triangle::GenerateTriangles $boundary_edges $boundary_nodes $extra_edges $extra_nodes]
    lassign $result elements_out nodes_out boundary_edges_out
    if { ![llength $elements_out] } {
        set fail 1        
    } else {
        set fail [Triangle::CreateGiDNodesAndTriangles $nodes_out $elements_out $boundary_edges_out]
        MeshView 0
        #GiD_Redraw
    }        
    return $fail
}

proc Triangle::GetExe { } {   
    variable options    
    if { $::tcl_platform(pointerSize) == 8 } {
        set bits 64
    } else {
        set bits 32
    }
    if { $::tcl_platform(platform) == "windows" } { 
        set exefile [file join $options(path) bin win-x${bits} triangle.exe]
    } elseif { $::tcl_platform(os) == "Darwin" } { 
        #Mac OS X
        set exefile [file join $options(path) bin mac-x${bits} triangle.exe]        
    } else {
        #Linux/Unix        
        set exefile [file join $options(path) bin linux-x${bits} triangle.exe]
    }
    return $exefile
}

proc Triangle::GenerateTriangles { boundary_edges boundary_nodes extra_edges extra_nodes } {
    variable options    
    set nodes_out [list]
    set boundary_edges_out [list]
    set elements_out [list]
    set exe [Triangle::GetExe]    
    if { ![file exists $exe] } {
        WarnWinText [_ "File '%s' not found" $exe]
        return ""
    }
    set dir [gid_cross_platform::create_folder_tmp]

    set filename [file join $dir boundary.poly]
    set fp [open $filename w]
    if { $fp != "" } {
        set num_nodes [expr {[llength $boundary_nodes]+[llength $extra_nodes]}]
        set num_edges [expr {[llength $boundary_edges]+[llength $extra_edges]}]
        set num_attributes 0
        set num_boundary_markers_nodes 0
        puts $fp "$num_nodes 2 $num_attributes $num_boundary_markers_nodes"
        set i 0 ;#must number noder from 1 to n without jumps
        foreach node $boundary_nodes {
            incr i
            lassign $node i_node x y
            set new_node_num($i_node) $i
            set old_node_num($i) $i_node
            puts $fp "$i $x $y"
            set xy_in($i_node) [list $x $y]
        }
        foreach node $extra_nodes {
            incr i
            lassign $node i_node x y
            set new_node_num($i_node) $i
            set old_node_num($i) $i_node
            puts $fp "$i $x $y"
        }
        set num_boundary_markers_edges 1
        puts $fp "$num_edges $num_boundary_markers_edges"
        set i 0
        foreach edge $boundary_edges {
            incr i
            lassign $edge n0 n1
            puts $fp "$i $new_node_num($n0) $new_node_num($n1) $i"            
            set direction_original_edge($i) [MathUtils::VectorDiff $xy_in($n0) $xy_in($n1)]
        }
        foreach edge $extra_edges {
            incr i
            lassign $edge n0 n1
            puts $fp "$i $new_node_num($n0) $new_node_num($n1) $i"
        }
        #puts $fp "1";#num_holes
        #puts $fp "1 0.0274512 -1.09149" ;#a point inside the hole
        puts $fp "0" ;#num_holes

        puts $fp "0";#num_regional_attributes
        close $fp
        set flags -p
        #append flags B ;#to supress output boundary markers
        if { $options(quality)> 0.0 } {
            append flags q$options(quality)
        }
        if { $options(constrained) } {
            #N to not write nodes file (unneded if won't add more nodes)         
            #p tp not write poly modified (unneded if won't add more nodes)
            append flags PNYY
        } else {
            #allow write nodes and new poly (forced edges modified adding nodes)
            
        }        
        if { $options(quadratic) } {
            append flags o2
        }
        if { $options(maxarea) > 0.0 } {
            append flags a$options(maxarea)
        }        
        
        #flag g for .off output
        catch { exec "$exe" $flags "$filename" } err
        
        if { $options(constrained) } {
            set nodes_out $boundary_nodes            
            set boundary_edges_out $boundary_edges
        } else {
            set outfilename_nodes [file rootname $filename].1.node
            if { [file exists $outfilename_nodes] } {
                set fp [open $outfilename_nodes r]
                if { $fp != "" } {
                    gets $fp line
                    lassign $line num_nodes num_dimension num_attributes num_boundary_markers_nodes
                    for { set i 0 } {$i<$num_nodes} {incr i } {
                        gets $fp line
                        lassign $line node_id x y
                        lappend nodes_out [list $node_id $x $y]
                        set xy_out($node_id) [list $x $y]        
                    }            
                    close $fp
                    file delete $outfilename_nodes
                }
            }
            
            set outfilename_poly [file rootname $filename].1.poly
            if { [file exists $outfilename_poly] } {
                set fp [open $outfilename_poly r]
                if { $fp != "" } {
                    gets $fp line
                    lassign $line num_nodes num_dimension num_attributes num_boundary_markers_nodes
                    #expected num_nodes==0
                    for { set i 0 } {$i<$num_nodes} {incr i } {
                        gets $fp line
                    }
                    gets $fp line
                    lassign $line num_edges num_boundary_markers_edges
                    for { set i 0 } {$i<$num_edges} {incr i } {
                        gets $fp line
                        lassign $line edge_id n0 n1 marker                        
                        set direction_out [MathUtils::VectorDiff $xy_out($n0) $xy_out($n1)]
                        if { [MathUtils::VectorDotProd $direction_original_edge($marker) $direction_out] < 0.0 } {
                            #edge splitted returned with opposite oriention
                            lappend boundary_edges_out [list $n1 $n0]
                        } else {
                            lappend boundary_edges_out [list $n0 $n1]
                        }                        
                    }    
                    gets $fp line
                    lassign $line num_holes
                    #expected num_holes==0
                    close $fp
                    file delete $outfilename_poly
                }
            }
        }
        
        set outfilename_elements [file rootname $filename].1.ele
        if { [file exists $outfilename_elements] } {
            set fp [open $outfilename_elements r]
            if { $fp != "" } {
                gets $fp line
                lassign $line num_elements n_node n_markers
                if { $options(constrained) } {                                        
                    for { set i 0 } {$i<$num_elements} {incr i } {
                        gets $fp line
                        lassign $line element_id n0 n1 n2
                        if { [info exists old_node_num($n0)] && [info exists old_node_num($n1)] && [info exists old_node_num($n2)] } {
                            lappend elements_out [list $old_node_num($n0) $old_node_num($n1) $old_node_num($n2)]
                        } else {
                            set elements_out [list]                    
                            break
                        }                        
                    }
                } else {
                    for { set i 0 } {$i<$num_elements} {incr i } {
                        gets $fp line
                        lassign $line element_id n0 n1 n2
                        lappend elements_out [list $n0 $n1 $n2]                        
                    }
                }
                close $fp
                file delete $outfilename_elements
                #doesn't use the triangle inner mechanism to remove triangles of holes because is hard to provide an inner coordinate by hole
                #use my own procedure, based on the edges orientation
                set elements_out [Triangle::RemoveTrianglesHoles $elements_out $boundary_edges_out]                
            }
        }
        file delete $filename   
    }
    file delete $dir  
    return [list $elements_out $nodes_out $boundary_edges_out]
}

proc Triangle::RemoveTrianglesHoles { created_elements boundary_edges } {   
    set remaining_elements [list]    
    foreach edge $boundary_edges {
        lassign $edge n0 n1
        set key $n1:$n0
        set boundary_edge_opposite($key) 1
    }    
    while { 1 } {
        foreach element $created_elements {
            set to_delete 0
            lassign $element n0 n1 n2
            foreach key [list $n0:$n1 $n1:$n2 $n2:$n0] {
                if { [info existe boundary_edge_opposite($key)] } {
                    set to_delete 1
                    break
                }
            }
            if { $to_delete } {        
                #to remove also its neighbor triangles of the hole in next steps
                foreach key [list $n0:$n1 $n1:$n2 $n2:$n0] {
                    if { ![info existe boundary_edge_opposite($key)] } {
                        lassign [split $key :] n1 n0
                        set key_opposite $n0:$n1
                        set boundary_edge_opposite($key_opposite) 1
                    }
                }            
            } else {
                lappend remaining_elements $element
            }
        }

        if { [llength $created_elements] == [llength $remaining_elements] } {
            break
        } else {
            #invoke repeatly until no changes            
            set created_elements $remaining_elements            
            set remaining_elements [list]
        }
    }
    return $remaining_elements
}
