
package provide vfs::webdav 0.3

package require tdom
package require vfs 1.0
package require http 2.6
# part of tcllib
package require base64

# This works for very basic operations.
# It has been put together, so far, largely by trial and error!
# What it really needs is to be filled in with proper xml support,
# using the tclxml package.

namespace eval vfs::webdav {}

proc vfs::webdav::Mount {dirurl local} {
    ::vfs::log "webdav::Mount http-vfs: attempt to mount $dirurl at $local"
    if {[string index $dirurl end] != "/"} {
        append dirurl "/"
    }
    
    set use_ssl 0

    if {[string range $dirurl 0 6] == "http://"} {
        set rest [string range $dirurl 7 end]
    } elseif {[string range $dirurl 0 7] == "https://"} {
        set rest [string range $dirurl 8 end]
        set use_ssl 1
    } else {
        set rest $dirurl
        set dirurl "http://${dirurl}"
    }
    
    if {![regexp {(([^:]*)(:([^@]*))?@)?([^/]*)(/(.*/)?([^/]*))?$} $rest \
              junk1 junk2 user junk3 pass host junk4 path file]} {
        return -code error "Sorry I didn't understand\
          the url address \"$dirurl\""
    }
    
    # WV [ list junk1 junk2 user junk3 pass host junk4 path file]

    if {[string length $file]} {
        return -code error "Can only mount directories, not\
          files (perhaps you need a trailing '/' - I understood\
          a path '$path' and file '$file')"
    }
    
    if {![string length $user]} {
        set user anonymous
    }
    
    set dirurl "http://$host/$path"
    if { $use_ssl} {
        set dirurl "https://$host/$path"        
    }
    
    set extraHeadersList [list Authorization \
                              [list Basic [base64::encode ${user}:${pass}]]]
    ::vfs::log "geturl $dirurl"
    if { $use_ssl} {
        ::vfs::log "enabling https"
        package require tls
        ::http::register https 443 [list ::tls::socket -autoservername 1]
    }
    set token [::http::geturl $dirurl -headers $extraHeadersList -validate 1]
    ::vfs::log "token = token"
    http::cleanup $token
    
    if {![catch {vfs::filesystem info $dirurl}]} {
        # unmount old mount
        ::vfs::log "ftp-vfs: unmounted old mount point at $dirurl"
        vfs::unmount $dirurl
    }
    ::vfs::log "http $host, $path mounted at $local"
    vfs::filesystem mount $local [list vfs::webdav::handler \
                                      $dirurl $extraHeadersList $path]
    # Register command to unmount
    vfs::RegisterMount $local [list ::vfs::webdav::Unmount $dirurl]
    return $dirurl
}

proc vfs::webdav::Unmount {dirurl local} {
    vfs::filesystem unmount $local
}

proc vfs::webdav::handler {dirurl extraHeadersList path cmd root relative actualpath args} {
    ::vfs::log "webdav::handler $dirurl $path $cmd"
    ::vfs::log "    root = $root"
    ::vfs::log "    relative = $relative"
    ::vfs::log "    actualpath = $actualpath"
    ::vfs::log "    args = $args"
    if {$cmd == "matchindirectory" || $cmd == "open"} {
        eval [list $cmd $dirurl $extraHeadersList $relative $actualpath] $args
    } else {
        ::vfs::log "[list $cmd $dirurl $extraHeadersList $relative] $args"
        eval [list $cmd $dirurl $extraHeadersList $relative] $args
    }
}

# If we implement the commands below, we will have a perfect
# virtual file system for remote http sites.

proc vfs::webdav::stat {dirurl extraHeadersList name} {
    ::vfs::log "webdav::stat $name"
    ::vfs::log "geturl $dirurl$name PROPFIND"
    set token [::http::geturl $dirurl$name -method PROPFIND \
                   -headers [concat $extraHeadersList [list Depth 0]] -protocol 1.1]
    upvar #0 $token state
    set httpcode [lindex [split $state(http) " "] 1]
    ::vfs::log "    state(http) = $state(http)"
    ::vfs::log "    httpcode = $httpcode"
    if {$httpcode != 200 && $httpcode != 207} {
        ::vfs::log "    No good: $state(http)"
        #parray state
        ::http::cleanup $token
        return [vfs::filesystem posixerror [::vfs::posixError ENOENT]]
    }
    set data [::http::data $token]
    ::http::cleanup $token
    ::vfs::log $data
    set xmldoc [ dom parse $data]
    #TODO other stat info
    # set resourcetype [::dom::selectNode $xmldoc {/d:multistatus/d:response/d:propstat/d:prop/d:resourcetype/d:collection} -namespaces {d DAV:}]
    set resourcetype [ $xmldoc selectNodes -namespaces {d DAV:} {/d:multistatus/d:response/d:propstat/d:prop/d:resourcetype/d:collection}]
    if {$resourcetype != ""} {
        set type "directory"
    } else {
        set type "file" 
    }
    ::vfs::log "    resourcetype = $resourcetype"
    ::vfs::log "    type = $type"
    # set filesize [::dom::selectNode $xmldoc {/d:multistatus/d:response/d:propstat/d:getcontentlength} -namespaces {d DAV:}]
    set filesize [ $xmldoc selectNodes -namespaces {d DAV:} {/d:multistatus/d:response/d:propstat/d:getcontentlength/text()}]
    if {$filesize != ""} {
        # set filesize [$filesize stringValue]
        set filesize [$filesize nodeValue]
    } else {
        set filesize 0
    }
    ::vfs::log "    filesize = $filesize"
    return [list dev -1 uid -1 gid -1 nlink 1 depth 0 size $filesize atime 0 mtime 0 ctime 0 mode 777 type $type]
}

proc vfs::webdav::access {dirurl extraHeadersList name mode} {
    ::vfs::log "webdav::access $name $mode"
    if {$name == ""} { return 1 }
    set token [::http::geturl $dirurl$name -headers $extraHeadersList]
    upvar #0 $token state
    set httpcode [lindex [split $state(http) " "] 1]
    if {$httpcode != 200 && $httpcode != 207} {
        ::vfs::log "No good: $state(http)"
        #parray state
        ::http::cleanup $token
        return 0
    } else {
        ::http::cleanup $token
        return 1
    }
}

# We've chosen to implement these channels by using a memchan.
# The alternative would be to use temporary files.
proc vfs::webdav::open {dirurl extraHeadersList name actualpath mode permissions} {
    ::vfs::log "webdav::open $name $actualpath $mode $permissions"
    # return a list of two elements:
    # 1. first element is the Tcl channel name which has been opened
    # 2. second element (optional) is a command to evaluate when
    #    the channel is closed.
    set resultchannel [vfs::memchan]
    fconfigure $resultchannel -encoding binary -translation binary
    if {$mode == ""} { set mode "r" }
    if {[file isdirectory $actualpath]} {
        ::vfs::log "can not read/write to directory."
        return [vfs::filesystem posixerror [::vfs::posixError EISDIR]]
    }
    # There is a tricky part. we need to download the file from server in all cases where it must exists
    # r, r+ and a absolutly need the file
    # w, w+ do not need the file at all
    # a+ works with file and without
    if {[string match "\[ar\]*" $mode ]} {
        # we should at least try to download the file.
        set token [::http::geturl $dirurl$name -headers $extraHeadersList]
        upvar #0 $token state
        set httpcode [lindex [split $state(http) " "] 1]
        if {$httpcode != 200} {
            # TODO give better error message.
            if {[string match "r*" $mode] || [string equal $mode "a"]} {
                ::vfs::log "File not found"
                ::http::cleanup $token
                close $resultchannel
                return [vfs::filesystem posixerror [::vfs::posixError ENOENT]]
            }
        } else {
            # load file into buffer
            puts -nonewline $resultchannel [::http::data $token]
            ::http::cleanup $token
        }
    }
    # we need to handle uploading of the file back if writing is permitted.
    # r+, w, w+, a, a+
    # The pointer is at the beginning for: r+, w, w+
    switch -glob -- $mode {
        "" -
        "r" {
            seek $resultchannel 0
            return [list $resultchannel]
        }
        "r+" -
        "w" -
        "w+" {
            seek $resultchannel 0
        }
        "a" -
        "a+" {
            # reading and writing file created if it does not exist. position at end of file
            # seeking does not work but we should already be at the end.
        }
        default {
            close $resultchannel
            return -code error "illegal access mode \"$mode\""
        }
    }
    ::vfs::log "[list $resultchannel [list ::vfs::webdav::_closing $dirurl$name $resultchannel $extraHeadersList]]"
    return [list $resultchannel [list ::vfs::webdav::_closing $dirurl$name $resultchannel $extraHeadersList]]
}

proc vfs::webdav::_closing {url channel extraHeadersList} {
    ::vfs::log "webdav::_closing $url $channel $extraHeadersList"
    seek $channel 0
    set token [::http::geturl $url -headers $extraHeadersList -method PUT -querychannel $channel]
    upvar #0 $token state
    set httpcode [lindex [split $state(http) " "] 1]
    if {$httpcode > 204} {
        # TODO give better error message.
        ::vfs::log "Error Writing: $httpcode"
        ::http::cleanup $token
        return [vfs::filesystem posixerror [::vfs::posixError EIO]]
    }
    ::vfs::log "Upload successfull"
    ::http::cleanup $token
}

proc vfs::webdav::matchindirectory {dirurl extraHeadersList path actualpath pattern type} {
    ::vfs::log "webdav::matchindirectory $dirurl $path $actualpath $pattern $type"
    set res [list]
    if {[string length $pattern]} {
        ::vfs::log "    matchindirectory several files"
        # need to match all files in a given remote http site.
        set token [::http::geturl $dirurl$path -method PROPFIND \
                       -headers [concat $extraHeadersList [list Depth 1]]]
        upvar #0 $token state
        set httpcode [lindex [split $state(http) " "] 1]
        if {$httpcode != 200 && $httpcode != 207} {
            ::vfs::log "No good: $state(http)"
            ::http::cleanup $token
            return [vfs::filesystem posixerror [::vfs::posixError ENOENT]]
        }
        set body [::http::data $token]
        ::http::cleanup $token
        ::vfs::log "body = $body"
        set xmldoc [ dom parse $body]
        # set data [::dom::selectNode $xmldoc {/d:multistatus/d:response/d:href} -namespaces {d DAV:}]
        set data [ $xmldoc selectNodes -namespaces {d DAV:} {/d:multistatus/d:response/d:href/text()}]
        # ::vfs::log  "data = $data"
        set currentdir [lindex $data 0]
        set content [lrange $data 1 end]
        foreach node $content {
            # strip path
            # set itemname [string map [list [$currentdir stringValue] ""] [$node stringValue]]
            # ::vfs::log "$currentdir nodeValue = [ $currentdir nodeValue]"
            # ::vfs::log "$node nodeValue = [ $node nodeValue]"
            set itemname [string map [list [$currentdir nodeValue] ""] [$node nodeValue]]
            if {[string index $itemname end] == "/"} {
                # Directories should not be show with slash at the end but without.
                set itemname [string range $itemname 0 end-1]
            }
            if {[string match $pattern $itemname]} {
                if {$type == 0} {
                    lappend res [file join $actualpath $itemname]
                } else {
                    # W "doing _matchtypes of [ [ $node parent] toXPath]"
                    # $node is /d:multistatus/d:response/d:href/text()
                    # so parent ir /d:multistatus/d:response/d:href
                    # but we need /d:multistatus/d:response
                    # as _matchtypes checks for (/d:multistatus/d:response/)d:propstat/d:prop/d:resourcetype/d:collection
                    set node_parent [ [ $node parent] parent]
                    eval lappend res [ _matchtypes $node_parent [file join $actualpath $itemname] $type]        
                }
                vfs::log "match: $itemname"
            }
        }
    } else {
        # single file
        ::vfs::log "    matchindirectory single files"
        set token [::http::geturl $dirurl$path -method PROPFIND \
                       -headers [concat $extraHeadersList [list Depth 0]]]
        
        upvar #0 $token state
        set httpcode [lindex [split $state(http) " "] 1]
        if {$httpcode != 200 && $httpcode != 207} {
            ::vfs::log "No good: $state(http)"s
            ::http::cleanup $token
            return [vfs::filesystem posixerror [::vfs::posixError ENOENT]]
        }
        set body [::http::data $token]
        ::http::cleanup $token
        set xmldoc [ dom parse $body]
        # set response [::dom::selectNode $xmldoc {/d:multistatus/d:response} -namespaces {d DAV:}]
        set response [ $xmldoc selectNodes -namespaces {d DAV:} {/d:multistatus/d:response}]
        #::vfs::log $body
        eval lappend res [ _matchtypes $response $actualpath $type]
    } 
    return $res
}

# Helper function
proc vfs::webdav::_matchtypes {item actualpath type} {
    ::vfs::log "webdav::_matchtypes [list $item $actualpath $type]"
    ::vfs::log "    check collection = [$item selectNodes -namespaces {d DAV:} {d:propstat/d:prop/d:resourcetype/d:collection}]"
    ::vfs::log "    toXPath = [ $item toXPath]"

    if {[$item selectNodes -namespaces {d DAV:} {d:propstat/d:prop/d:resourcetype/d:collection}] != ""} {
        ::vfs::log " matchDirectories $type = [::vfs::matchDirectories $type]"
        if {![::vfs::matchDirectories $type]} {
            return ""
        }
    } else {
        ::vfs::log " matchFiles $type = [::vfs::matchFiles $type]"
        if {![::vfs::matchFiles $type]} {
            return ""
        }
    }
    return [list $actualpath]
}

proc vfs::webdav::createdirectory {dirurl extraHeadersList name} {
    ::vfs::log "webdav::createdirectory $dirurl $extraHeadersList $name"
    set token [::http::geturl $dirurl$name -method MKCOL \
                   -headers [concat $extraHeadersList [list Depth 0]]]
    upvar #0 $token state
    set httpcode [lindex [split $state(http) " "] 1]
    ::http::cleanup $token
    if {$httpcode == 201} {
        return 1
    }
    ::vfs::log "No good: $state(http)"
    switch -- $httpcode {
        403 {
            return [vfs::filesystem posixerror [::vfs::posixError EACCES]]
        }
        507 {
            return [vfs::filesystem posixerror [::vfs::posixError ENOSPC]]
        }
        409 {
            return [vfs::filesystem posixerror [::vfs::posixError ENOENT]]
        }
        405 {
            return [vfs::filesystem posixerror [::vfs::posixError EPERM]]
        }
    }
    return [vfs::filesystem posixerror [::vfs::posixError ENODEV]]
}

proc vfs::webdav::removedirectory {dirurl extraHeadersList name recursive} {
    ::vfs::log "webdav::removedirectory $dirurl $name $recursive"
    # deletion is always recursive.
    
    set token [::http::geturl $dirurl$name -method DELETE -headers $extraHeadersList]
    upvar #0 $token state
    set httpcode [lindex [split $state(http) " "] 1]
    set body [::http::data $token]
    ::vfs::log "$state(http)"
    ::http::cleanup $token
    switch -- $httpcode {
        404 {
            return [vfs::filesystem posixerror [::vfs::posixError ENOENT]]
        }
        204 -
        200 {
            return 1
        }
        - {
            return [vfs::filesystem posixerror [::vfs::posixError ENOTEMPTY]]
        }
    }
}

proc vfs::webdav::deletefile {dirurl extraHeadersList name} {
    ::vfs::log "deletefile $name"
    removedirectory $dirurl $extraHeadersList $name 0
}

proc vfs::webdav::_getfiletype {dirurl extraHeadersList path } {
    ::vfs::log "webdav::_getfiletype $dirurl $extraHeadersList"
    set token [::http::geturl $dirurl$path -method PROPFIND \
        -headers [concat $extraHeadersList [list Depth 0]] -protocol 1.1]
    upvar #0 $token state
    set httpcode [lindex [split $state(http) " "] 1]
    ::vfs::log "    state(http) = $state(http)"
    ::vfs::log "    httpcode = $httpcode"
    if {$httpcode != 200 && $httpcode != 207} {
        ::vfs::log "    No good: $state(http)"
        #parray state
        ::http::cleanup $token
        return [vfs::filesystem posixerror [::vfs::posixError ENOENT]]
    }
    set data [::http::data $token]
    ::http::cleanup $token
    ::vfs::log $data
    set xmldoc [ dom parse $data]
    #TODO other stat info
    # set resourcetype [::dom::selectNode $xmldoc {/d:multistatus/d:response/d:propstat/d:prop/d:resourcetype/d:collection} -namespaces {d DAV:}]
    set resourcetype [ $xmldoc selectNodes -namespaces {d DAV:} {/d:multistatus/d:response/d:propstat/d:prop/d:resourcetype/d:collection}]
    set is_archive 0
    if {$resourcetype != ""} {
        set type "directory"
    } else {
        set type "file"
    }
    ::vfs::log "    resourcetype = $resourcetype"
    ::vfs::log "    type = $type"
    return $type
}

proc vfs::webdav::fileattributes {dirurl extraHeadersList path args} {
    ::vfs::log "webdav::fileattributes $dirurl $extraHeadersList"
    ::vfs::log "    path = $path"
    ::vfs::log "    args = $args"

    if { $path == ""} {
        # it means current path
        set path "."
    }

    switch -- [llength $args] {
        0 {
            # just return name of attributes
            set all_attributes [ list]
            if { $::tcl_platform(platform) == "windows" } {
                # 1st keyword is type of file
                set all_attributes [ list -archive -hidden -longname -readonly -shortname -system]
            } else {
                # 1st keyword is permissions which needs to know type of file
                set all_attributes [ list -permissions -group -owner ]
            }
            return $all_attributes
        }
        1 {
            # get value
            set index [lindex $args 0]
            set is_archive 0
            if { $index == 0} {
                # get value of -archive in Windows or -permissions in Linux.
                # both need to know the right type of file
                set type [ vfs::webdav::_getfiletype $dirurl $extraHeadersList $path]
                if { $type == "file"} {
                    set is_archive 1
                }
            }

            # build the whole attribute list and then get the asked one
            set all_attributes [ list]
            if { $::tcl_platform(platform) == "windows" } {
                # set all_attributes [ list -archive $is_archive -hidden 0 -longname $path -readonly 0 -shortname $path -system 0]
                set all_attributes [ list  $is_archive  0  $path  0  $path  0]
            } else {
                set permissions "040777"
                if { $is_archive} {
                    set permissions "00777"
                }
                # set all_attributes [ list  -permissions $permissions -group -1 -owner -1]
                set all_attributes [ list  $permissions -1  -1 ]
            }

            return [ lindex $all_attributes $index]
        }
        2 {
            # set value
            # atm not usable
            set index [lindex $args 0]
            set val [lindex $args 1]
            return [vfs::filesystem posixerror [::vfs::posixError ENOSYS]]
        }
    }

    ::vfs::log "    returnDEF [ list]"
    return [ list]
}

proc vfs::webdav::utime {dirurl extraHeadersList path actime mtime} {
    return [vfs::filesystem posixerror [::vfs::posixError ENOSYS]]
}

