# Visualization Toolkit (VTK) Tcl package configuration.

# macOS is still using vtk 5.8
# with vtk 8.0.1 GiD crashes when importing vtk files in post
# it's very hard to track down, so reverting to old version which works.
if { $::tcl_platform(os) == "Darwin" } {
    source [ file join $dir pkgIndex_macos.tcl]
    return
}

#miguel: 
#set ::vtk_lst_pkgs [ list CommonCore CommonMath CommonMisc CommonSystem CommonTransforms CommonDataModel CommonColor CommonComputationalGeometry CommonExecutionModel FiltersCore FiltersGeneral ImagingCore ImagingFourier FiltersStatistics FiltersExtraction FiltersGeometry ImagingSources FiltersSources RenderingCore FiltersHybrid FiltersModeling IOCore IOEnSight IOXMLParser IOExodus IOImage RenderingFreeType RenderingContextIID RenderingOpenGLII RenderingGLtoPSOpenGLII IOExport IOExportOpenGLII IOLegacy IOGeometry IOImport IOXML InfovisCore IOInfovis IOPLY IOTecplotTable IOXdmfII ImagingColor ImagingGeneral ImagingHybrid ImagingMath InteractionStyle RenderingAnnotation RenderingVolume InteractionWidgets InteractionImage RenderingContextOpenGLII RenderingVolumeOpenGLII ]
#kike :without some of them
set ::vtk_lst_pkgs [ list CommonCore CommonMath CommonMisc CommonSystem CommonTransforms CommonDataModel CommonColor CommonComputationalGeometry CommonExecutionModel FiltersCore FiltersGeneral ImagingCore ImagingFourier FiltersStatistics FiltersExtraction FiltersGeometry ImagingSources FiltersSources RenderingCore FiltersHybrid FiltersModeling IOCore IOImage IOLegacy IOXMLParser IOXML ImagingColor ImagingGeneral ImagingHybrid ImagingMath InteractionStyle RenderingFreeType RenderingAnnotation RenderingVolume InteractionWidgets InteractionImage RenderingOpenGLII RenderingVolumeOpenGLII]
#set ::vtk_lst_pkgs [ list CommonCore CommonSystem CommonDataModel IOCore IOXMLParser IOExport IOXML ]

package ifneeded vtkinit {8.0} {
    namespace eval ::vtk::init {
	proc load_library_package {libName libPath} {
	    set libPrefix {}
	    if { $::tcl_platform(platform) != "windows" } {
		set libPrefix {lib}
	    }     
	    #first try to load a static package, then try the shared package.
	    # W "load {} $libName"
	    if {[catch "load {} $libName" err_txt]} {
		# WV err_txt
		if { $::tcl_platform(os) == "Linux" } {
		    set libExt [info sharedlibextension].1
		} else {
		    set libExt [info sharedlibextension]
		}
		set currentDirectory [pwd]
		set libFile [file join $libPath "$libPrefix$libName-8.0$libExt"]
		# W " doing cd {$libPath}; load {$libFile}"
		if {[catch "cd {$libPath}; load {$libFile}" errorMessage]} {
		    # WV errorMessage
		    puts $errorMessage
		}
		cd $currentDirectory
	    }
	}
	proc require_package {name {version {8.0}}} {
	    if {[catch "package require -exact $name $version" errorMessage]} {
		puts $errorMessage
		return 0
	    } else {
		return 1
	    }
	}
	set version {8.0}
	set kits {}
	foreach kit [ list base {*}$::vtk_lst_pkgs] {
	    lappend kits [string tolower "${kit}"]
	}
    }
    package provide vtkinit {8.0}
}

if { $::tcl_platform(pointerSize) == 8} {
    set bits 64
} else {
    set bits 32
}
if { $::tcl_platform(platform) == "windows" } {
    set sub_folder [file join $dir lib x${bits}]
} else {
    set sub_folder $dir
}

foreach kit $::vtk_lst_pkgs {
    package ifneeded "vtk${kit}TCL" {8.0} "
    package require -exact vtkinit {8.0}
    # ::vtk::init::load_library_package {vtk${kit}TCL} {[file join [file dirname [file dirname [file dirname [file dirname [info script]]]]] bin]}
    ::vtk::init::load_library_package {vtk${kit}TCL} {$sub_folder}
  "
    package ifneeded "vtk[string tolower ${kit}]" {8.0} "
    package require -exact vtkinit {8.0}
    if {\[catch {source \[file join {[file dirname [info script]]} {vtk[string tolower ${kit}]} {vtk[string tolower ${kit}].tcl}\]} errorMessage\]} {
      puts \$errorMessage
    }
  "
}

set ::vtk_toplevel_pkgs [ list vtk vtkbase vtkinteraction vtktesting]
#set ::vtk_toplevel_pkgs [ list vtk vtkbase]

foreach src $::vtk_toplevel_pkgs {
    package ifneeded ${src} {8.0} "
    package require -exact vtkinit {8.0}
    if {\[catch {source \[file join {[file dirname [info script]]} {$src} {$src.tcl}\]} errorMessage\]} {
      puts \$errorMessage
    }
  "
}
