# Visualization Toolkit (VTK) Tcl package configuration.

proc VtkWriteDebug { txt} {
    catch {
	set debug_file_name /tmp/vtk_dbg.txt
	if { $::tcl_platform(os) == "Darwin"} {
	    set debug_file_name /var/tmp/vtk_dbg.txt
	}
	if { ![ info exists VtkWriteDebugFirstTime]} {
	    set VtkWriteDebugFirstTime 1
	    #delete previous contents
	    set fo [ open $debug_file_name a]
	} else {
	    set fo [ open $debug_file_name a]
	}
	puts $fo $txt
	close $fo
    }
}

proc VtkPrintError { txt} {
    if { [ catch "puts $txt"]} {
	catch {
	    # tk_messageBox -icon error -message $txt
	    WarnWinText "VTK error: $txt"
	}
    }
}

package ifneeded vtkinit {5.8} {
    namespace eval ::vtk::init {
	proc load_library_package {libName libPath {libPrefix {lib}} {version {5.8}}} {
	    set currentDirectory [pwd]
	    if { $::tcl_platform(platform) == "windows" } {
		set libExt [info sharedlibextension]
		set libPrefix ""
	    } elseif { $::tcl_platform(os) == "Darwin" } { 
		set libExt .$version[info sharedlibextension]
	    } else {
		set libExt [info sharedlibextension].$version
	    }
	    set libFile [file join $libPath "$libPrefix$libName$libExt"]

	    # if { $::tcl_platform(os) == "Darwin"} {
	    # 	WarnWinText "VTK import not available for Mac OS X at the moment: $libFile"
	    # 	return
	    # }
	    
	    # ::VtkWriteDebug "currentDirectory = $currentDirectory\nlibPath = $libPath\nloading $libFile"
	    # ::VtkPrintError "load_library_package $libName"
	    if {[catch "cd {$libPath}; load {$libFile}" errorMessage]} {
		::VtkPrintError "load_library_package $libName: $errorMessage"
	    }
	    cd $currentDirectory
	}
	proc require_package {name {version {5.8}}} {

	    # if { $::tcl_platform(os) == "Darwin"} {
	    # 	WarnWinText "VTK import not available for Mac OS X at the moment: $name $version"
	    # 	return 0
	    # }
	    if {[catch "package require -exact $name $version" errorMessage]} {
		# puts $errorMessage
		::VtkPrintError "require_package $name $version: $errorMessage"
		return 0
	    } else {
		return 1
	    }
	}
	set version {5.8}
	set kits {}
	foreach kit { base Common Filtering IO Imaging Graphics
	    
	    
	    
	} {
	    lappend kits [string tolower "${kit}"]
	}
    }
    package provide vtkinit {5.8}
}

if { $::tcl_platform(pointerSize) == 8} {
    set bits 64
} else {
    set bits 32
}
if { $::tcl_platform(platform) == "windows" } {
    set sub_folder [file join lib x${bits}]
} else {
    set sub_folder ""
}

set ____vtk_version 5.8

foreach kit { Common Filtering IO Imaging Graphics } {
    package ifneeded "vtk${kit}TCL" {5.8} "
    package require -exact vtkinit {5.8}
    ::vtk::init::load_library_package {vtk${kit}TCL} {[file join [file dirname [info script]] $sub_folder]}
  "

    package ifneeded "vtk[string tolower ${kit}]" {5.8} "
    package require -exact vtkinit {5.8}
    if {\[catch {source \[file join {[file dirname [info script]]/tcl-${____vtk_version}} {vtk[string tolower ${kit}]} {vtk[string tolower ${kit}].tcl}\]} errorMessage\]} {
      ::VtkPrintError \$errorMessage
    }
  "
}

foreach src {vtk vtkbase vtkinteraction vtktesting} {
    package ifneeded ${src} {5.8} "
    package require -exact vtkinit {5.8}
    if {\[catch {source \[file join {[file dirname [info script]]/tcl-${____vtk_version}} {$src} {$src.tcl}\]} errorMessage\]} {
      ::VtkPrintError \$errorMessage
    }
  "
}
